
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package dialogs;

import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;

import spinworks.SW_main.modes;
import spinworks.SW_main.modes_3D;
import spinworks.SW_main.units;
import spinworks.PeakPick;
import spinworks.SW_main;
import spinworks.Util;
import spinworks.SW_main.data_types;
import org.eclipse.wb.swt.SWTResourceManager;

import simulation.SS_util;
import simulation.SimPar;

public class CalibrateDialog extends Dialog 
{

	protected Object result;
	protected Shell shell;
	private Text textF2Value;
	private Text textF1Value;
	
	Font currentFont;

	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public CalibrateDialog(Shell parent, int style) 
	{
		super(parent, style);
		setText("Calibrate Spectrum");
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() 
	{
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) 
		{
			if (!display.readAndDispatch()) 
			{
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() 
	{
		shell = new Shell(getParent(), SWT.DIALOG_TRIM);
		shell.setSize(333, 380);
		shell.setText(getText());
		
		textF2Value = new Text(shell, SWT.BORDER);
		textF2Value.setBounds(127, 12, 98, 21);
		
		Button zeroF2Button = new Button(shell, SWT.NONE);	
		zeroF2Button.setBounds(242, 10, 75, 25);
		zeroF2Button.setText("Zero");
		
		Label lblFppm = new Label(shell, SWT.NONE);
		lblFppm.setAlignment(SWT.RIGHT);
		lblFppm.setBounds(10, 15, 98, 18);
		
		Label lblFppm_1 = new Label(shell, SWT.NONE);
		lblFppm_1.setAlignment(SWT.RIGHT);
		lblFppm_1.setBounds(10, 51, 98, 18);
		
		/*if(SW_main.gp.axis_unit_indirect == units.PPM)
			lblFppm_1.setText("F1 (PPM):");
		else
			lblFppm_1.setText("F1 (Hz):");*/
			
		textF1Value = new Text(shell, SWT.BORDER);
		textF1Value.setBounds(127, 48, 98, 21);		
			
		// load values into text boxes
		if (SW_main.gp.axis_unit == units.PPM)
		{
			lblFppm.setText("X (direct, PPM):");
			textF2Value.setText(String.format("%.6f", SW_main.gp.new_freq1 / SW_main.q.SF));  
		}
		else
		{
			textF2Value.setText(String.format("%.3f", SW_main.gp.new_freq1));
			lblFppm.setText("X (direct, Hz):");
		}
		
		if (SW_main.gp.axis_unit_indirect == units.PPM)
		{
			if (SW_main.q.data_mode == modes.MODE_2D ||
	                (SW_main.q.data_mode == modes.MODE_3D && SW_main.gp.mode_3D == modes_3D.XZ))
	            {
	            	textF1Value.setText(String.format("%.6f",
	                		SW_main.gp.new_freq1_indirect / SW_main.q_f1.SF));
	            	if(SW_main.q.data_mode == modes.MODE_3D)
	            		lblFppm_1.setText("Z (F1, PPM):");
	            	else
	            		lblFppm_1.setText("Y (F1, PPM):");
	            }
	            else if (SW_main.q.data_mode == modes.MODE_3D && SW_main.gp.mode_3D == modes_3D.XY)
	            {
	            	textF1Value.setText(String.format("%.6f",
	                		SW_main.gp.new_freq1_indirect / SW_main.q_f2.SF));
	            	lblFppm_1.setText("Y (F2, PPM):");
	            }			
		}
		else
		{
			textF1Value.setText(String.format("%.3f", SW_main.gp.new_freq1_indirect));
			lblFppm_1.setText("indirect (Hz)");
		}
		
		Button zeroF1Button = new Button(shell, SWT.NONE);		
		zeroF1Button.setText("Zero");
		zeroF1Button.setBounds(242, 46, 75, 25);
		
		Group grpRefFTo = new Group(shell, SWT.NONE);
		grpRefFTo.setText("ref. F2 to Solvent");
		grpRefFTo.setBounds(10, 92, 304, 206);
		grpRefFTo.setBackground(new Color(Display.getCurrent(), 200, 200, 200));	// something nicer
		
		final Combo comboSolvent1H = new Combo(grpRefFTo, SWT.NONE);
		comboSolvent1H.setItems(new String[] {"CDCl3", "D2O", "C6D6", "CD3OD", "Acetone-d6", "DMSO-d6", "CD3CN"});
		comboSolvent1H.setText("sel. solvent");
		comboSolvent1H.setBounds(22, 52, 91, 23);
		
		final Combo comboSolvent13C = new Combo(grpRefFTo, SWT.NONE);
		comboSolvent13C.setItems(new String[] {"CDCl3", "------", "C6D6", "CD3OD", "Acetone-d6", "DMSO-d6", "CD3CN"});
		comboSolvent13C.setText("sel. solvent");
		comboSolvent13C.setBounds(151, 52, 91, 23);
		
		Label lblh = new Label(grpRefFTo, SWT.NONE);
		lblh.setBounds(22, 21, 55, 15);
		lblh.setText("1H");
		
		Label lblc = new Label(grpRefFTo, SWT.NONE);
		lblc.setBounds(151, 21, 55, 15);
		lblc.setText("13C");
		
		Button buttonCancel = new Button(shell, SWT.NONE);	
		currentFont = buttonCancel.getFont();
		buttonCancel.setFont(SWTResourceManager.getBoldFont(currentFont));
		buttonCancel.setBackground(SWTResourceManager.getColor(SWT.COLOR_RED));
		buttonCancel.setBounds(237, 317, 80, 27);
		buttonCancel.setText("Cancel");
		
		Button buttonOK = new Button(shell, SWT.NONE);
		currentFont = buttonOK.getFont();
		buttonOK.setFont(SWTResourceManager.getBoldFont(currentFont));
		buttonOK.setBackground(SWTResourceManager.getColor(SWT.COLOR_GREEN));
		buttonOK.setBounds(145, 317, 80, 27);
		buttonOK.setText("OK");

//-------------------------------------------------------------------------------------------------
// Button Listeners		
	
// solvent drop-boxes
		comboSolvent1H.addSelectionListener(new SelectionAdapter() 		// 1H
		{
			@Override
			public void widgetSelected(SelectionEvent e) 	
			{
				int index = comboSolvent1H.getSelectionIndex();
				
				if(index == 0)
					textF2Value.setText("7.26");
				else if(index == 1)
					textF2Value.setText("4.75");
				else if(index == 2)
					textF2Value.setText("7.16");
				else if(index == 3)
					textF2Value.setText("3.31");
				else if(index == 4)
					textF2Value.setText("2.05");
				else if(index == 5)
					textF2Value.setText("2.50");
				else if(index == 6)
					textF2Value.setText("1.94");
			}
		});
		
		comboSolvent13C.addSelectionListener(new SelectionAdapter() 	// 13C
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				int index = comboSolvent13C.getSelectionIndex();
				System.out.println(index);
				
				if(index == 0)
					textF2Value.setText("77.02");
				else if(index == 1)
					textF2Value.setText("0.00");
				else if(index == 2)
					textF2Value.setText("128.39");
				else if(index == 3)
					textF2Value.setText("49.00");
				else if(index == 4)
					textF2Value.setText("29.84");
				else if(index == 5)
					textF2Value.setText("39.52");
				else if(index == 6)
					textF2Value.setText("1.32");
			}
		});
		
//	OK button selected, apply new referencing
		buttonOK.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				double tmp;
	            try { tmp = Double.parseDouble(textF2Value.getText()); }	// get value from dialog
	            catch (Exception ex)
	            {
	                Util.show_and_log("Bad F2 format", ex.getMessage());
	                return;
	            }
					
				if(SW_main.gp.axis_unit == SW_main.units.PPM) tmp = tmp * SW_main.q.SF;    // convert to Hz. if necessary 
		     	 
				if (SW_main.gp.interp_on == false || SW_main.gp.display_mode == SW_main.modes.MODE_2D) 
					SW_main.q.offset += tmp - SW_main.gp.new_freq1;
				else 
					SW_main.q.offset += tmp 
					                  - PeakPick.findPeak(SW_main.gp.new_freq1, 4.0, 
					                		  SW_main.gp, SW_main.q, SW_main.sp);

				SW_main.p.OFFSET = SW_main.q.offset/SW_main.p.SF;	
				SW_main.q.OFFSET = SW_main.p.OFFSET;

				SW_main.q.SF = SW_main.q.SFO1 - (SW_main.q.offset - (SW_main.q.SF * SW_main.q.SW / 2.0))/1000000.0;	
				SW_main.p.SF = SW_main.q.SF;

//		regenerate simulated spectrum, if simulating 
				if (SW_main.gp.calc_array != null && SW_main.gp.display_mode == SW_main.modes.MODE_1D)
				{
	                if (SW_main.sp.calc_mode == SimPar.HIRES) 
	                	SS_util.genspec(SW_main.gp, SW_main.sp, SW_main.q);
	                SW_main.gp.calc_max = Math.abs(Util.dscan(SW_main.gp.calc_array, 
	                		SW_main.q.SI, 0, SW_main.q.SI));
					if(SW_main.gp.calc_max < 1) SW_main.gp.calc_max = 10000000.0;
					SW_main.gp.calc_scale = (double)SW_main.gp.dysize * 0.80 / (double)SW_main.gp.calc_max;
				}
								
				if(SW_main.gp.display_mode == modes.MODE_2D &&
						(SW_main.q.data_mode == modes.MODE_2D ||
						(SW_main.q.data_mode == modes.MODE_3D && SW_main.gp.mode_3D == modes_3D.XZ)))	// now do F1 if in 2D mode
				{
					double old_offset_f1 = SW_main.q_f1.offset;
					double tmp_f1;
					try { tmp_f1 = Double.parseDouble(textF1Value.getText()); } // get value from dialog
					catch (Exception ex)
					{
						Util.show_and_log("Bad indirect format" + ex.getMessage());
						return;
					}

					if(SW_main.gp.axis_unit_indirect == units.PPM) tmp_f1 = tmp_f1 * SW_main.q_f1.SF; // convert to Hz.
					SW_main.q_f1.offset += tmp_f1 - SW_main.gp.new_freq1_indirect;

					SW_main.p_f1.OFFSET = (float)(SW_main.q_f1.offset/SW_main.q_f1.SF);
					SW_main.q_f1.OFFSET = SW_main.p_f1.OFFSET;

					if(SW_main.q.data_type != data_types.VNMR)
					{
						SW_main.q_f1.SF = SW_main.q_f1.SFO1 
								- (SW_main.q_f1.offset - (SW_main.q_f1.SF * SW_main.q_f1.SW / 2.0))/1000000.0;
						SW_main.p_f1.SF = SW_main.q_f1.SF;
					}
					else	// Varian data must be handled slightly differently, because of the silly
					{		// way they do referencing
						SW_main.p_f1.SF = SW_main.p_f1.SF - (SW_main.q_f1.offset - old_offset_f1)/1000000.0;
						SW_main.q_f1.SF = SW_main.p_f1.SF;
					}

					// calibration has put frequency out side of limits!
					if (SW_main.gp.freq1 > SW_main.q.offset 
							|| SW_main.gp.freq2 < (SW_main.q.offset - SW_main.q.SW * SW_main.q.SFO1)
							|| SW_main.gp.freq1_indirect > SW_main.q_f1.offset 
							|| SW_main.gp.freq2_indirect < (SW_main.q_f1.offset - SW_main.q_f1.SW * SW_main.q_f1.SFO1))
					{
						SW_main.gp.freq1 = SW_main.q.offset;
						SW_main.gp.freq2 = SW_main.q.offset - SW_main.q.SW * SW_main.q.SFO1;                     
						SW_main.gp.freq1_indirect = SW_main.q_f1.offset;
						SW_main.gp.freq2_indirect = SW_main.q_f1.offset - SW_main.q_f1.SW * SW_main.q_f1.SFO1;                       
					}

					// now do F2 (Y) if in 2D mode
					if (SW_main.gp.display_mode == modes.MODE_2D &&
							(SW_main.q.data_mode == modes.MODE_3D && SW_main.gp.mode_3D == modes_3D.XY))	
					{

						double old_offset_f2 = SW_main.q_f2.offset;
						double tmp_f2;
						try { tmp_f2 = Double.parseDouble(textF1Value.getText()); }   // get value from dialog
						catch (Exception ex)
						{                    
							Util.show_and_log("Bad indirect format" + ex.getMessage());
							return;
						}

						if (SW_main.gp.axis_unit_indirect == units.PPM) tmp_f2 = tmp_f2 * SW_main.q_f2.SF; // convert to Hz.
						SW_main.q_f2.offset += tmp_f2 - SW_main.gp.new_freq1_indirect;

						SW_main.p_f2.OFFSET = (float)(SW_main.q_f2.offset / SW_main.q_f2.SF);
						SW_main.q_f2.OFFSET = SW_main.p_f2.OFFSET;

						if (SW_main.q.data_type != data_types.VNMR)
						{
							SW_main.q_f2.SF = SW_main.q_f2.SFO1 
									- (SW_main.q_f2.offset - (SW_main.q_f2.SF * SW_main.q_f2.SW / 2.0)) / 1000000.0;
							SW_main.p_f2.SF = SW_main.q_f2.SF;
						}
						else	// Varian data must be handled slightly differently, because of the silly
						{		// way they do referencing
							SW_main.p_f2.SF = SW_main.p_f2.SF - (SW_main.q_f2.offset - old_offset_f2) / 1000000.0;
							SW_main.q_f2.SF = SW_main.p_f2.SF;
						}

						// calibration has put frequency out side of limits!
						if (SW_main.gp.freq1 > SW_main.q.offset 
								|| SW_main.gp.freq2 < (SW_main.q.offset - SW_main.q.SW * SW_main.q.SFO1)
								|| SW_main.gp.freq1_indirect > SW_main.q_f2.offset 
								|| SW_main.gp.freq2_indirect < (SW_main.q_f2.offset - SW_main.q_f2.SW * SW_main.q_f2.SFO1))
						{
							SW_main.gp.freq1 = SW_main.q.offset;
							SW_main.gp.freq2 = SW_main.q.offset - SW_main.q.SW * SW_main.q.SFO1;                 
							SW_main.gp.freq1_indirect = SW_main.q_f2.offset;
							SW_main.gp.freq2_indirect = SW_main.q_f2.offset - SW_main.q_f2.SW * SW_main.q_f2.SFO1;                
						}
					}				
				}
				result = SWT.OK;
				shell.dispose();
			}
		});
				

//	reference to zero buttons
		zeroF2Button.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				textF2Value.setText("0.0");
			}
		});
		
		zeroF1Button.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				textF1Value.setText("0.0");
			}
		});

// cancel button
		buttonCancel.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				result = SWT.CANCEL;
				shell.dispose();
			}
		});
		
		shell.setDefaultButton(buttonOK);
	}
//-------------------------------------------------------------------------------------------------
}
