package dialogs;

import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Button;

import java.awt.Desktop;
import java.io.File;
import java.io.PrintWriter;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import decon.DeconUtil;
import spinworks.PeakInfo;
import spinworks.SW_main;
import spinworks.Trace;
import spinworks.Util;

import org.eclipse.swt.widgets.Label;

public class DeconDialog extends Dialog 
{
	protected Object result;
	protected Shell shell;
	private Text textFrequency;
	private Text textAmplitude;
	private Text textWidth;
	private Text textGauss;
	private Text textRMS;
	private Spinner spinnerPeak;

	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public DeconDialog(Shell parent, int style) 
	{
		super(parent, style);
		setText("Deconvolution (Band Fitting)");
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() 
	{
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) 
		{
			if (!display.readAndDispatch()) 
			{
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() 
	{
		shell = new Shell(getParent(), getStyle());
		shell.setSize(508, 268);
		shell.setText(getText());
		
		Button buttonDump = new Button(shell, SWT.NONE);	
		buttonDump.setBounds(10, 202, 85, 25);
		buttonDump.setText("Dump");
		
		Button buttonList = new Button(shell, SWT.NONE);		
		buttonList.setBounds(91, 202, 85, 25);
		buttonList.setText("List");
		
		Button buttonStack = new Button(shell, SWT.NONE);
		buttonStack.setBounds(172, 202, 85, 25);
		buttonStack.setText("Stack");
		
		Button buttonOptimize = new Button(shell, SWT.NONE);
		buttonOptimize.setBounds(253, 202, 85, 25);
		buttonOptimize.setText("Optimize");
		
		Button buttonApply = new Button(shell, SWT.NONE);
		buttonApply.setBounds(334, 202, 85, 25);
		buttonApply.setText("Apply");
		
		Button buttonClose = new Button(shell, SWT.NONE);
		buttonClose.setBounds(415, 202, 85, 25);
		buttonClose.setText("Close");
		
		spinnerPeak = new Spinner(shell, SWT.BORDER);
		spinnerPeak.setBounds(133, 10, 47, 22);		
		if(SW_main.gp.n_decon_peaks != 0)
		{
			spinnerPeak.setMinimum(1);
			spinnerPeak.setMaximum(SW_main.gp.n_decon_peaks);
		}
		spinnerPeak.setSelection(1);
		
		textFrequency = new Text(shell, SWT.BORDER);
		textFrequency.setBounds(133, 39, 97, 21);
		textFrequency.setText(String.format("%10.4f", SW_main.gp.decon_peaks[0].freq / SW_main.q.SF));
		
		textAmplitude = new Text(shell, SWT.BORDER);
		textAmplitude.setBounds(133, 66, 97, 21);
		textAmplitude.setText(String.format("%.2f", SW_main.gp.decon_peaks[0].intensity));
		
		textWidth = new Text(shell, SWT.BORDER);
		textWidth.setBounds(133, 93, 97, 21);
		textWidth.setText(String.format("%10.4f", SW_main.gp.decon_peaks[0].width));
		
		textGauss = new Text(shell, SWT.BORDER);
		textGauss.setBounds(133, 120, 97, 21);
		textGauss.setText(String.format("%10.2f", SW_main.gp.decon_peaks[0].gauss));
		
		textRMS = new Text(shell, SWT.BORDER);
		textRMS.setBounds(133, 147, 97, 21);
		textRMS.setText(String.format("%10.5f", 
                Util.rms_dev(SW_main.gp.freq1, SW_main.gp.freq2, SW_main.gp, SW_main.q, false)));
		
		Button buttonFreqMinus = new Button(shell, SWT.NONE);
		buttonFreqMinus.setBounds(236, 37, 35, 25);
		buttonFreqMinus.setText("-");
		
		Button buttonAmpMinus = new Button(shell, SWT.NONE);
		buttonAmpMinus.setBounds(236, 64, 35, 25);
		buttonAmpMinus.setText("-");
		
		Button buttonWidthMinus = new Button(shell, SWT.NONE);
		buttonWidthMinus.setBounds(236, 91, 35, 25);
		buttonWidthMinus.setText("-");
		
		Button buttonGaussMinus = new Button(shell, SWT.NONE);
		buttonGaussMinus.setBounds(236, 118, 35, 25);
		buttonGaussMinus.setText("-");
		
		Button buttonFreqPlus = new Button(shell, SWT.NONE);
		buttonFreqPlus.setText("+");
		buttonFreqPlus.setBounds(277, 37, 35, 25);
		
		Button buttonAmpPlus = new Button(shell, SWT.NONE);
		buttonAmpPlus.setText("+");
		buttonAmpPlus.setBounds(277, 64, 35, 25);
		
		Button buttonWidthPlus = new Button(shell, SWT.NONE);
		buttonWidthPlus.setText("+");
		buttonWidthPlus.setBounds(277, 91, 35, 25);
		
		Button buttonGaussPlus = new Button(shell, SWT.NONE);
		buttonGaussPlus.setText("+");
		buttonGaussPlus.setBounds(277, 118, 35, 25);
		
		Button buttonFreqOptimize = new Button(shell, SWT.NONE);
		buttonFreqOptimize.setBounds(318, 37, 85, 25);
		buttonFreqOptimize.setText("Optimize");
		
		Button buttonAmpOptimize = new Button(shell, SWT.NONE);
		buttonAmpOptimize.setBounds(318, 64, 85, 25);
		buttonAmpOptimize.setText("Optimize");
		
		Button buttonWidthOptimize = new Button(shell, SWT.NONE);
		buttonWidthOptimize.setBounds(318, 91, 85, 25);
		buttonWidthOptimize.setText("Optimize");
		
		Button buttonGaussOptimize = new Button(shell, SWT.NONE);
		buttonGaussOptimize.setBounds(318, 118, 85, 25);
		buttonGaussOptimize.setText("Optimize");
		
		Button buttonRescale = new Button(shell, SWT.NONE);
		buttonRescale.setBounds(236, 147, 157, 25);
		buttonRescale.setText("Rescale Display");
		
		Label lblNewLabel = new Label(shell, SWT.NONE);
		lblNewLabel.setAlignment(SWT.RIGHT);
		lblNewLabel.setBounds(28, 45, 88, 15);
		lblNewLabel.setText("Freq. (ppm):");
		
		Label lblAmplitude = new Label(shell, SWT.NONE);
		lblAmplitude.setText("Amplitude:");
		lblAmplitude.setAlignment(SWT.RIGHT);
		lblAmplitude.setBounds(41, 72, 75, 15);
		
		Label lblWidthhz = new Label(shell, SWT.NONE);
		lblWidthhz.setText("Width (Hz.):");
		lblWidthhz.setAlignment(SWT.RIGHT);
		lblWidthhz.setBounds(28, 99, 88, 15);
		
		Label lblGaussian = new Label(shell, SWT.NONE);
		lblGaussian.setAlignment(SWT.RIGHT);
		lblGaussian.setText("% Gaussian:");
		lblGaussian.setBounds(0, 126, 116, 15);
		
		Label lblRms = new Label(shell, SWT.NONE);
		lblRms.setText("RMS:");
		lblRms.setAlignment(SWT.RIGHT);
		lblRms.setBounds(61, 153, 55, 15);
		
		Label lblPeak = new Label(shell, SWT.NONE);
		lblPeak.setText("Peak:");
		lblPeak.setAlignment(SWT.RIGHT);
		lblPeak.setBounds(61, 17, 55, 15);
					
//	listeners
		
		spinnerPeak.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				//System.out.println(String.format("selection = %d", spinnerPeak.getSelection()));
				updateAll();
			}
		});
		
		buttonFreqMinus.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].freq 
					-= (SW_main.gp.freq1 - SW_main.gp.freq2) / 400.0;
				
				DeconUtil.genspec(SW_main.gp, SW_main.q);
				updateAll();
			}
		});
		
		buttonFreqPlus.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].freq 
					+= (SW_main.gp.freq1 - SW_main.gp.freq2) / 400.0;
				DeconUtil.genspec(SW_main.gp, SW_main.q);
				updateAll();
			}
		});
		
		buttonFreqOptimize.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DeconUtil.optimize_freq(spinnerPeak.getSelection() - 1, 20,
						     SW_main.gp.freq1, SW_main.gp.freq2, 
			                (SW_main.gp.freq1 - SW_main.gp.freq2) / 200.0, SW_main.gp, SW_main.q);			
				DeconUtil.genspec(SW_main.gp, SW_main.q);
				updateAll();
			}
		});
		
		buttonAmpMinus.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].intensity /= 1.02;				
				DeconUtil.genspec(SW_main.gp, SW_main.q);
				updateAll();
			}
		});
		
		buttonAmpPlus.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].intensity *= 1.02;
				DeconUtil.genspec(SW_main.gp, SW_main.q);
				updateAll();
			}
		});
		
		buttonAmpOptimize.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{	
				DeconUtil.optimize_amp(spinnerPeak.getSelection() - 1, 20, 
						SW_main.gp.freq1, SW_main.gp.freq2, 
						SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].intensity / 10.0,
						SW_main.gp, SW_main.q);
				DeconUtil.genspec(SW_main.gp, SW_main.q);
				updateAll();
			}
		});
		
		buttonWidthMinus.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].width /= 1.05;
				DeconUtil.genspec(SW_main.gp, SW_main.q);
				updateAll();
			}
		});
		
		buttonWidthPlus.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].width *= 1.05;
				DeconUtil.genspec(SW_main.gp, SW_main.q);
				updateAll();
			}
		});
		
		buttonWidthOptimize.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DeconUtil.optimize_width(spinnerPeak.getSelection() - 1, 20, 
						SW_main.gp.freq1, SW_main.gp.freq2,
						SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].width / 10.0,
						SW_main.gp, SW_main.q);
				DeconUtil.genspec(SW_main.gp, SW_main.q);
				updateAll();
			}
		});
		
		buttonGaussMinus.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].gauss > 0.0)
					SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].gauss -= 5.0 ;
				DeconUtil.genspec(SW_main.gp, SW_main.q);
				updateAll();
			}
		});
		
		buttonGaussPlus.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].gauss < 100.0)
					SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].gauss += 5.0 ;
				DeconUtil.genspec(SW_main.gp, SW_main.q);
				updateAll();
			}
		});
		
		buttonGaussOptimize.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DeconUtil.optimize_gauss(spinnerPeak.getSelection() - 1, 20, 
						SW_main.gp.freq1, SW_main.gp.freq2,
						10.0, SW_main.gp, SW_main.q);
				DeconUtil.genspec(SW_main.gp, SW_main.q);
				updateAll();
			}
		});
		
//------------------	
		
		buttonRescale.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.calc_scale = SW_main.gp.expt_scale;
				SW_main.update_spect = true;
				SW_main.canvas.redraw();
			}
		});
		
		buttonDump.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				Util.check_order(SW_main.gp);

	            int point1 = (int)((SW_main.q.offset - SW_main.gp.freq1) / SW_main.q.hzperpt);
	            int point2 = (int)((SW_main.q.offset - SW_main.gp.freq2) / SW_main.q.hzperpt);
	            if (point1 < 0) point1 = 0;
	            if (point2 >= SW_main.q.SI) point2 = SW_main.q.SI - 1;

	            PrintWriter pw;

	            try
				{
					pw = new PrintWriter(SW_main.q.data_folder + "decon_peaks.txt");
				}
				catch (Exception ex)
				{
		            Util.show_and_log("decon dump: " + SW_main.q.data_folder + "decon_peaks.txt", ex.getMessage());
					return;
				}
	            
	            
	            pw.println("ti: " + SW_main.gp.plot_title);
	            pw.println(String.format("##frequency %10.5f", SW_main.q.SFO1));

	            try
	            {
	                for (int i = point1; i <= point2; i++)
	                {
	                    if (SW_main.gp.axis_unit == SW_main.units.PPM)
	                    	pw.println(String.format("%10.5f  %10.3f", (SW_main.q.offset - (double)i * SW_main.q.hzperpt) / SW_main.q.SF,
	                        		SW_main.gp.calc_array[i] / 1000.0));
	                    else
	                    	pw.println(String.format("%10.3f  %10.3f", (SW_main.q.offset - (double)i * SW_main.q.hzperpt),
	                        		SW_main.gp.calc_array[i] / 1000.0));
	                }
	            }
	            catch (Exception ex)
	            {
	                Util.show_and_log("error writing decon_peaks.txt ", ex.getMessage());
	                return;
	            }

	            pw.close();

	            for (int j = 0; j < SW_main.gp.n_traces; j++)
	            {
	                try
	                {
	                    pw = new PrintWriter(String.format("%sdecon_peak_%d.txt", SW_main.q.data_folder, j+1));
	                }
	                catch (Exception ex)
	                {                 
	                    Util.show_and_log("error opening  decon_peak_#.txt ", ex.getMessage());
	                    return;
	                }

	                try
	                {
	                    for (int i = point1; i <= point2; i++)
	                    {
	                        if (SW_main.gp.axis_unit == SW_main.units.PPM)
	                            pw.println(String.format("%10.5f  %10.3f", (SW_main.q.offset - (double)i * SW_main.q.hzperpt) / SW_main.q.SF,
	                            		SW_main.gp.stack_trace[j].data[i] / 1000.0));
	                        else
	                            pw.println(String.format("%10.3f  %10.3f", (SW_main.q.offset - (double)i * SW_main.q.hzperpt),
	                            		SW_main.gp.stack_trace[j].data[i] / 1000.0));
	                    }
	                }
	                catch (Exception ex)
	                {
	                    Util.show_and_log("error writing to:  decon_peak_#.txt ", ex.getMessage());
	                    return;
	                }

	                if (SW_main.gp.log_on) Util.log_error("Decon peak X,Y file(s) written");
	                pw.close();
	            }  
			}
		});
		
		buttonList.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				PrintWriter pw;
				String peaksFileName = SW_main.q.data_folder + "decon_peaks.txt";
				
				if (SW_main.gp.log_on) Util.log_error("Decon. listing to: " + peaksFileName);

				try
				{
					pw = new PrintWriter(peaksFileName);
				}
				catch (Exception ex)
				{
		            Util.show_and_log("integ_list_event(): " + peaksFileName, ex.getMessage());
					return;
				}

				double max_int = 0.0; // scale to total intensity in spectrum
				for (int i = 0; i < SW_main.gp.n_decon_peaks; i++)
					max_int = max_int + SW_main.gp.decon_peaks[i].intensity;

				pw.println("SpinWorks peak peak deconvolution list for: " + SW_main.q.data_folder);
				pw.println();
				pw.println(" #      freq.(Hz)   freq.(ppm)    int.(rel)     int.(abs)    width(Hz)   %Gauss  area(rel)  label");
				pw.println();

				for (int i = 0; i < SW_main.gp.n_decon_peaks; i++)
				{
					if (SW_main.gp.decon_peaks[i].freq != 0.0 && SW_main.gp.decon_peaks[i].freq > -1.0e09)
					{
						pw.println(String.format(
								/*"{0:00}   {1:######.00}   {2:#####.0000}    {3:F3}     {4:E3}   {5:F2}    {6:F0}    {7:F3}",*/
								"%3d   %10.2f   %10.4f   %10.3f   %12.2f  %10.3f  %8.1f  %9.3f   %s",
								i + 1, SW_main.gp.decon_peaks[i].freq, SW_main.gp.decon_peaks[i].freq / SW_main.q.SF,
								SW_main.gp.decon_peaks[i].intensity / max_int,
								SW_main.gp.decon_peaks[i].intensity,
								SW_main.gp.decon_peaks[i].width, SW_main.gp.decon_peaks[i].gauss,
								(SW_main.gp.decon_peaks[i].intensity / max_int) * SW_main.gp.decon_peaks[i].width,
								SW_main.gp.decon_peaks[i].label));
					}
				}

				pw.close();

				//	List with whatever editor is associated with the ".txt" file type
				if (Desktop.isDesktopSupported()) // new way!  portable??
				{
				    try 
				    {
				        Desktop.getDesktop().open(new File(peaksFileName));
				    } 
				    catch (Exception ex) 
				    {
				        Util.show_and_log("no application for this file type or file not found " +
				             peaksFileName, ex.getMessage());
				    }
				}
			}
		});
		
		buttonStack.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				SW_main.gp.n_traces = 0;

				if (SW_main.gp.log_on) Util.log_error("Decon. generating stacked traces");

				for (int i = 0; i < SW_main.gp.n_decon_peaks; i++)
				{
					//	create a stacked trace object
					SW_main.gp.stack_trace[i] = new Trace(SW_main.gp, SW_main.q);

					DeconUtil.genspec(i, SW_main.gp, SW_main.q);                // generate individual spectra
					SW_main.gp.stack_trace[i].yscale = SW_main.gp.calc_scale;   // use common scaling
					SW_main.gp.n_traces++;		                        		// bump trace number
				}

				DeconUtil.genspec(SW_main.gp, SW_main.q);
				SW_main.gp.calc_max = Math.abs(Util.dscan(SW_main.gp.calc_array, SW_main.q.SI, 0, SW_main.q.SI));
				updateAll();
			}
		});
		
		buttonOptimize.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				  Cursor currentCursor = shell.getCursor();				  
				  shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));

		            if (SW_main.gp.log_on) Util.log_error("Decon.  optimizing all peaks");

		            for (int j = 1; j <= 5; j++)
		            	for (int i = 0; i < SW_main.gp.n_decon_peaks; i++)
		            	{
		            		if (SW_main.gp.decon_peaks[i].freq_iterate)
		            			DeconUtil.optimize_freq(i, 20, SW_main.gp.freq1, SW_main.gp.freq2,
		            					(SW_main.gp.freq1 - SW_main.gp.freq2) / (j * 200.0), SW_main.gp, SW_main.q);

		            		if (SW_main.gp.decon_peaks[i].intensity_iterate)
		            			DeconUtil.optimize_amp(i, 20, SW_main.gp.freq1, SW_main.gp.freq2,
		            					SW_main.gp.decon_peaks[i].intensity / (j * 10.0), SW_main.gp, SW_main.q);

		            		if (SW_main.gp.decon_peaks[i].width_iterate)
		            			DeconUtil.optimize_width(i, 20, SW_main.gp.freq1, SW_main.gp.freq2,
		            					SW_main.gp.decon_peaks[i].width / (j * 10.0), SW_main.gp, SW_main.q);

		            		if (SW_main.gp.log_on)
		            			Util.log_error(String.format("   peak %d  rms = %f", i + 1, 
		            					Util.rms_dev(SW_main.gp.freq1, SW_main.gp.freq2, SW_main.gp, SW_main.q, false)));
		            		
		            		DeconUtil.genspec(SW_main.gp, SW_main.q);
		            		updateAll();
		            	}		            

		            DeconUtil.genspec(SW_main.gp, SW_main.q);	            
		            updateAll();
		            
		            shell.setCursor(currentCursor);
			}
		});
		
		//TODO find cleaner way to do this without "Apply" button
		buttonApply.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].freq = 
						Double.parseDouble(textFrequency.getText()) * SW_main.q.SF;
				
				SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].intensity = 
						Double.parseDouble(textAmplitude.getText());
				
				SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].width = 
						Double.parseDouble(textWidth.getText());
				
				SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].gauss = 
						Double.parseDouble(textGauss.getText());
				
				DeconUtil.genspec(SW_main.gp, SW_main.q);
				updateAll();
			}
		});
//Cursor
		buttonClose.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				shell.dispose();
			}
		});

		shell.setDefaultButton(buttonApply);
//-------------------------------------------------------------------------------------------------
	}	// end createContents()
	
	private void updateAll()
	{
		 textRMS.setText(String.format("%10.5f", 
	                Util.rms_dev(SW_main.gp.freq1, SW_main.gp.freq2, SW_main.gp, SW_main.q, false)));
		 
		 textFrequency.setText(String.format("%10.4f", 
					SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].freq / SW_main.q.SF));
		 textAmplitude.setText(String.format(
					"%.2f", SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].intensity));
		 textWidth.setText(String.format(
					"%10.4f", SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].width));
		 textGauss.setText(String.format(
					"%10.4f", SW_main.gp.decon_peaks[spinnerPeak.getSelection() - 1].gauss));
		 
		 for (int i = 0; i < SW_main.gp.n_peaks; i++)
         {
			 SW_main.gp.peaks[i].freq = SW_main.gp.decon_peaks[i].freq;
         }
		 
		 SW_main.update_spect = true;
		 SW_main.canvas.redraw();
	}
}
