
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package dialogs;

import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.wb.swt.SWTResourceManager;

import spinworks.PreferenceUtil;
import spinworks.SW_main;

import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.RGB;

// Sorry for the Brit/Canadian spelling: "colour"
public class DisplayColourDialog extends Dialog 
{
	protected Object result;
	protected Shell shell;
	
	RGB rgbBackground, 
		rgbSpect, 
		rgbSim, 
		rgbAxis,
		rgbInteg, 
		rgbCursor;
	Font currentFont;

	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public DisplayColourDialog(Shell parent, int style) 
	{
		super(parent, style);
		setText("Edit Display Colours");
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() 
	{	
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) 
		{
			if (!display.readAndDispatch()) 
			{
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() 
	{
		rgbBackground = SW_main.gp.back_colour.getRGB();
		rgbSpect = SW_main.gp.spect_colour.getRGB(); 
		rgbSim = SW_main.gp.sim_colour.getRGB();
		rgbAxis = SW_main.gp.axis_colour.getRGB();
		rgbInteg = SW_main.gp.integ_colour.getRGB(); 
		rgbCursor = SW_main.gp.cursor_colour.getRGB();
		
		shell = new Shell(getParent(), SWT.DIALOG_TRIM);
		shell.setSize(285, 276);
		shell.setText("Edit Display Colours");
		
		Button buttonOK = new Button(shell, SWT.NONE);		
		currentFont = buttonOK.getFont();
		buttonOK.setFont(SWTResourceManager.getBoldFont(currentFont));
		buttonOK.setBounds(103, 212, 80, 27);
		buttonOK.setText("OK");	
		buttonOK.setBackground(SWTResourceManager.getColor(SWT.COLOR_GREEN));
		
		Button buttonCancel = new Button(shell, SWT.NONE);	
		currentFont = buttonCancel.getFont();
		buttonCancel.setFont(SWTResourceManager.getBoldFont(currentFont));
		buttonCancel.setBounds(189, 212, 80, 27);
		buttonCancel.setText("Cancel");
		buttonCancel.setBackground(SWTResourceManager.getColor(SWT.COLOR_RED));
		
		Button buttonBackground = new Button(shell, SWT.NONE);		
		buttonBackground.setBounds(10, 10, 90, 25);
		buttonBackground.setText("Background");
		buttonBackground.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ColorDialog dlg = new ColorDialog(shell);
				rgbBackground = dlg.open();
			}
		});
		
		Button buttonSpect = new Button(shell, SWT.NONE);		
		buttonSpect.setBounds(10, 41, 90, 25);
		buttonSpect.setText("Spectrum");
		buttonSpect.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ColorDialog dlg = new ColorDialog(shell);
				rgbSpect = dlg.open();
			}
		});
		
		Button buttonSim = new Button(shell, SWT.NONE);		
		buttonSim.setBounds(10, 72, 90, 25);
		buttonSim.setText("Simulated");
		buttonSim.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ColorDialog dlg = new ColorDialog(shell);
				rgbSim = dlg.open();
			}
		});
		
		Button buttonAxis = new Button(shell, SWT.NONE);		
		buttonAxis.setBounds(10, 103, 90, 25);
		buttonAxis.setText("Axis");
		buttonAxis.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ColorDialog dlg = new ColorDialog(shell);
				rgbAxis = dlg.open();			
			}
		});
		
		Button buttonInteg = new Button(shell, SWT.NONE);		
		buttonInteg.setBounds(10, 134, 90, 25);
		buttonInteg.setText("Integrals");
		buttonInteg.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ColorDialog dlg = new ColorDialog(shell);
				rgbInteg = dlg.open();
			}
		});
		
		Button buttonCursor = new Button(shell, SWT.NONE);		
		buttonCursor.setBounds(10, 165, 90, 25);
		buttonCursor.setText("Cursor");
		buttonCursor.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ColorDialog dlg = new ColorDialog(shell);
				rgbCursor = dlg.open();
			}
		});
		
		Button buttonRestoreDefaults = new Button(shell, SWT.NONE);		
		buttonRestoreDefaults.setBounds(106, 10, 163, 25);
		buttonRestoreDefaults.setText("Restore Defaults");
		buttonRestoreDefaults.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.back_colour = new Color(Display.getCurrent(), 0, 0, 0);     
				SW_main.gp.spect_colour = new Color(Display.getCurrent(), 255, 255, 0);
				SW_main.gp.sim_colour = new Color(Display.getCurrent(), 80, 80, 255);
				SW_main.gp.integ_colour = new Color(Display.getCurrent(), 255, 255, 255);
				SW_main.gp.axis_colour = new Color(Display.getCurrent(), 0, 255, 255);
				SW_main.gp.cursor_colour = new Color(Display.getCurrent(), 0, 255, 0);
				PreferenceUtil.setColours();
				shell.dispose();
			}
		});
		
		Button buttonSetPreferences = new Button(shell, SWT.NONE);		
		buttonSetPreferences.setBounds(106, 41, 163, 25);
		buttonSetPreferences.setText("Set as Preferences");
		buttonSetPreferences.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(rgbBackground != null)
					SW_main.gp.back_colour = new Color(Display.getCurrent(), rgbBackground);
				if(rgbSpect != null)
					SW_main.gp.spect_colour = new Color(Display.getCurrent(), rgbSpect);
				if(rgbSim != null)
					SW_main.gp.sim_colour = new Color(Display.getCurrent(), rgbSim);
				if(rgbAxis != null)
					SW_main.gp.axis_colour = new Color(Display.getCurrent(), rgbAxis);
				if(rgbInteg != null)
					SW_main.gp.integ_colour = new Color(Display.getCurrent(), rgbInteg);
				if(rgbCursor != null)
					SW_main.gp.cursor_colour = new Color(Display.getCurrent(), rgbCursor);
				
				PreferenceUtil.setColours();
			}
		});
					
		// OK button
		buttonOK.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(rgbBackground != null)
					SW_main.gp.back_colour = new Color(Display.getCurrent(), rgbBackground);
				if(rgbSpect != null)
					SW_main.gp.spect_colour = new Color(Display.getCurrent(), rgbSpect);
				if(rgbSim != null)
					SW_main.gp.sim_colour = new Color(Display.getCurrent(), rgbSim);
				if(rgbAxis != null)
					SW_main.gp.axis_colour = new Color(Display.getCurrent(), rgbAxis);
				if(rgbInteg != null)
					SW_main.gp.integ_colour = new Color(Display.getCurrent(), rgbInteg);
				if(rgbCursor != null)
					SW_main.gp.cursor_colour = new Color(Display.getCurrent(), rgbCursor);
				
				shell.dispose();
			}
		});
		
		// Cancel button
		buttonCancel.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				shell.dispose();
			}
		});	
		
		shell.setDefaultButton(buttonOK);
	}
}
