package dialogs;

import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Button;

import java.awt.Desktop;
import java.io.File;
import java.io.PrintWriter;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.wb.swt.SWTResourceManager;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Label;

import spinworks.FileUtil;
import spinworks.SW_main;
import spinworks.Util;

public class IntegDialog extends Dialog 
{
	protected Object result;
	protected Shell shlIntegrationDialog;
	private Text textCalValue;
	private double bias_sensitivity = 10.0;
	private double slope_sensitivity = 1.0;
	
	private Font currentFont;

	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public IntegDialog(Shell parent, int style) 
	{
		super(parent, style);
		setText("Integration Dialog");
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() 
	{
		createContents();
		shlIntegrationDialog.open();
		shlIntegrationDialog.layout();
		Display display = getParent().getDisplay();
		while (!shlIntegrationDialog.isDisposed()) 
		{
			if (!display.readAndDispatch()) 
			{
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	public void createContents()
	{
		shlIntegrationDialog = new Shell(getParent(), SWT.BORDER | SWT.TITLE);
		shlIntegrationDialog.setSize(282, 361);
		shlIntegrationDialog.setText("Integration Dialog");
		
		// calibrate button
		Button btnCalibrate = new Button(shlIntegrationDialog, SWT.NONE);
		btnCalibrate.setBounds(10, 10, 80, 25);
		btnCalibrate.setText("Calibrate");
		
		btnCalibrate.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{	
				try{ SW_main.gp.int_cal_value = Double.parseDouble(textCalValue.getText()); }
				catch (Exception ex)
				{ Util.show_and_log("integ_calibrate_event: number in wrong format", ex.getMessage()); }

				if(SW_main.gp.display_mode == SW_main.modes.MODE_1D)
				{
	                    if(SW_main.gp.current_integral >= 0 && SW_main.gp.current_integral < SW_main.gp.n_integrals)
						{
	                    	SW_main.gp.integ_cal = SW_main.gp.integrals[SW_main.gp.current_integral].value 
	                    			                / SW_main.gp.int_cal_value;      
	                    	SW_main.gp.cursor_flag = 0;
	                    	SW_main.gp.integ_calibrated = true;
						}

						else
							Util.show_and_log("You must select an integral \n with 1 cursor first");
					}
					else if(SW_main.gp.display_mode == SW_main.modes.MODE_2D)
					{
						// TODO 2D integration
					}

				SW_main.update_spect = true;
				SW_main.canvas.redraw();
			}
		});
		
		// calibrate text box
		textCalValue = new Text(shlIntegrationDialog, SWT.BORDER);
		textCalValue.setText(String.format("%8.3f", SW_main.gp.int_cal_value));
		textCalValue.setBounds(97, 12, 70, 21);
		
		// scale up button
		Button btnScaleUp = new Button(shlIntegrationDialog, SWT.NONE);
		btnScaleUp.setBounds(173, 10, 45, 25);
		btnScaleUp.setText("*2");
		
		btnScaleUp.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.integ_scale = SW_main.gp.integ_scale * 2.0;
				SW_main.update_spect = true;
				SW_main.canvas.redraw();
			}
		});		
		
		// scale down button
		Button btnScaleDown = new Button(shlIntegrationDialog, SWT.NONE);
		btnScaleDown.setText("/2");
		btnScaleDown.setBounds(224, 10, 45, 25);
		
		btnScaleDown.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.integ_scale = SW_main.gp.integ_scale / 2.0;
				SW_main.update_spect = true;
				SW_main.canvas.redraw();
			}
		});
						
		Group grpDelete = new Group(shlIntegrationDialog, SWT.NONE);
		//grpDelete.setBackground(new Color(Display.getCurrent(), 0, 255, 0));
		grpDelete.setText("Delete");
		grpDelete.setBounds(10, 61, 100, 95);
		
		// delete current button
		Button btnDeleteCurrent = new Button(grpDelete, SWT.NONE);
		btnDeleteCurrent.setBounds(10, 20, 80, 25);
		btnDeleteCurrent.setText("Current");
		
		btnDeleteCurrent.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(SW_main.gp.current_integral >= 0 && SW_main.gp.current_integral < SW_main.gp.n_integrals)
				{
					SW_main.gp.integrals[SW_main.gp.current_integral].start = -1.0e40;
					SW_main.gp.cursor_flag = 0;
					SW_main.gp.current_integral = -1;
				}
				SW_main.update_spect = true;
				SW_main.canvas.redraw();
			}
		});
						
		// delete all button
		Button btnDeleteAll = new Button(grpDelete, SWT.NONE);
		btnDeleteAll.setText("All");
		btnDeleteAll.setBounds(10, 51, 80, 25);
		
		btnDeleteAll.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.n_integrals = 0;
				SW_main.gp.current_integral = -1;
	            SW_main.update_spect = true;
				SW_main.canvas.redraw();
			}
		});
			
		// list button
		Button btnList = new Button(shlIntegrationDialog, SWT.NONE);
		btnList.setText("List");
		btnList.setBounds(116, 81, 80, 25);
		
		btnList.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				listIntegrals();
			}
		});
						
		// read button
		Button btnRead = new Button(shlIntegrationDialog, SWT.NONE);
		btnRead.setText("Read");
		btnRead.setBounds(116, 112, 80, 25);
		
		btnRead.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				FileUtil.readInteg1D();
				SW_main.update_spect = true;
	    		SW_main.canvas.redraw();
			}
		});	
		
		// slope slider
		final Scale scaleSlope = new Scale(shlIntegrationDialog, SWT.NONE);
		scaleSlope.setMaximum(100);
		scaleSlope.setMinimum(0);
		scaleSlope.setSelection(50);
		scaleSlope.setBounds(10, 241, 170, 42);
		
		// slope slider listener
		scaleSlope.addSelectionListener(new SelectionAdapter() 
	    {
	    	public void widgetSelected(SelectionEvent e) 
	    	{
	    		if (SW_main.gp.current_integral < 0 || SW_main.gp.current_integral >= SW_main.gp.n_integrals)
	    	    return;
		          
	    		SW_main.gp.integrals[SW_main.gp.current_integral].slope =  
	    				(scaleSlope.getSelection() - 50) * slope_sensitivity 
	                 +  SW_main.gp.current_slope;

	    		SW_main.update_spect = true;
	    		SW_main.canvas.redraw();
	    	}
	    });
		
		// bias slider
		final Scale scaleBias = new Scale(shlIntegrationDialog, SWT.NONE);
		scaleBias.setMaximum(100);
		scaleBias.setMinimum(0);
		scaleBias.setSelection(50);
		scaleBias.setBounds(10, 183, 170, 42);
		
		// bias slider listener
		scaleBias.addSelectionListener(new SelectionAdapter() 
	    {
	    	public void widgetSelected(SelectionEvent e) 
	    	{
	    		if (SW_main.gp.current_integral < 0 || SW_main.gp.current_integral >= SW_main.gp.n_integrals)
	                return;
	          
	    		SW_main.gp.integrals[SW_main.gp.current_integral].bias =  
	    				(scaleBias.getSelection() - 50) * bias_sensitivity 
	                 +  SW_main.gp.current_bias;

	    		SW_main.update_spect = true;
	    		SW_main.canvas.redraw();
	    	}
	    });
		
		Group grpSens = new Group(shlIntegrationDialog, SWT.NONE);
		grpSens.setText("Sens.");
		grpSens.setBounds(184, 187, 65, 96);
		
		// sens up button
		Button btn_SensUp = new Button(grpSens, SWT.NONE);
		btn_SensUp.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		btn_SensUp.setText("+");
		btn_SensUp.setBounds(15, 22, 36, 25);
		
		btn_SensUp.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				slope_sensitivity = slope_sensitivity * 2;
				bias_sensitivity = bias_sensitivity * 2;
			}
		});		
		
		// sens minus button
		Button btnSensMinus = new Button(grpSens, SWT.NONE);
		btnSensMinus.setFont(SWTResourceManager.getFont("Segoe UI", 9, SWT.BOLD));
		btnSensMinus.setText("-");
		btnSensMinus.setBounds(15, 53, 36, 25);
		
		btnSensMinus.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				slope_sensitivity = slope_sensitivity / 2;
				bias_sensitivity = bias_sensitivity / 2;
			}
		});
				
		// close button
		Button btnClose = new Button(shlIntegrationDialog, SWT.NONE);
		currentFont = btnClose.getFont();
		btnClose.setFont(SWTResourceManager.getBoldFont(currentFont));
		btnClose.setBackground(SWTResourceManager.getColor(SWT.COLOR_GREEN));
		btnClose.setBounds(194, 289, 75, 25);
		btnClose.setText("Close");
		
		btnClose.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.current_integral = -1;
				SW_main.update_spect = true;
	    		SW_main.canvas.redraw();
				SW_main.gp.integ_mode = false;
				FileUtil.saveInteg1D();
				shlIntegrationDialog.dispose();
			}
		});
						
		Label lblBias = new Label(shlIntegrationDialog, SWT.NONE);
		lblBias.setBounds(10, 162, 55, 15);
		lblBias.setText("Bias");
		
		Label lblSlope = new Label(shlIntegrationDialog, SWT.NONE);
		lblSlope.setBounds(10, 229, 55, 15);
		lblSlope.setText("Slope");
	}	// end createContents()

//-------------------------------------------------------------------------------------------------
	private void listIntegrals()
	{
		int i;

		String integFileName;
		PrintWriter pw;

        integFileName = SW_main.q.data_folder + "integ.txt";

		try
		{
			pw = new PrintWriter(integFileName);
		}
		catch (Exception ex)
		{
            Util.show_and_log("integ_list_event(): " + integFileName, ex.getMessage());
			return;
		}

		pw.println("SpinWorks integral list for:   " + SW_main.q.data_folder);
		pw.println("");

		if(SW_main.gp.display_mode == SW_main.modes.MODE_2D)
			pw.println("peak #   start(F2 ppm)   end(F2 ppm)  start(F1 ppm)   end(F1 ppm)  intensity  intensity(Absolute)  Label");
		else
			pw.println("peak #   start (ppm)   end (ppm)  intensity  intensity(Absolute)");

		pw.println("");

		for (i = 0; i < SW_main.gp.n_integrals; i++)
		{
			if (SW_main.gp.display_mode == SW_main.modes.MODE_2D)
			{
				if (SW_main.gp.integrals[i].start > -1.0e40)
				{
					pw.println(String.format(
						"%3d    %10.3f  %10.3f    %10.3f  %10.4f  %10.4f   %10.0f  %s", 
                        i + 1,
                        SW_main.gp.integrals[i].start / SW_main.q.SF, 
                        SW_main.gp.integrals[i].end / SW_main.q.SF,
                        SW_main.gp.integrals[i].start_f1 / SW_main.q_f1.SF, SW_main.gp.integrals[i].end_f1 / SW_main.q_f1.SF,
                        SW_main.gp.integrals[i].scaled, ((double)SW_main.gp.integrals[i].value / 1000.0),
                        SW_main.gp.integrals[i].label));
				}
			}
			else		// 1D
			{
				if (SW_main.gp.integrals[i].start > -1.0e40)
				{
					pw.println(String.format("%3d    %10.3f  %10.3f  %10.4f  %10.0f",
                        i + 1,
                        SW_main.gp.integrals[i].start / SW_main.q.SF, SW_main.gp.integrals[i].end / SW_main.q.SF,
                        SW_main.gp.integrals[i].scaled, ((double)SW_main.gp.integrals[i].value / 1000.0)));
				}
			}
		}

		pw.close();
		
		if (Desktop.isDesktopSupported()) // new way!  portable??
		{
		    try 
		    {
		        Desktop.getDesktop().open(new File(integFileName));
		    } 
		    catch (Exception ex) 
		    {
		        Util.show_and_log("no application for this file type or file not found " +
		             integFileName, ex.getMessage());
		    }
		}
	}

//-------------------------------------------------------------------------------------------------
}
