/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>
 
package dialogs;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

import spinworks.SW_main;
import spinworks.Util;

/**
 * @author marat
 *
 */
public class ParF2Advanced extends Dialog 
{

	protected Object result;
	protected Shell shell;
	private Text textFilterWidth;
	private Text textThreshold;
	private Text textLambda;
	private Text textRefFreq;
	private Text textSpectralWidth;

	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public ParF2Advanced(Shell parent, int style) 
	{
		super(parent, style);
		setText("Advanced Processing Parameters (F2)");
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() 
	{
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) 
		{
			if (!display.readAndDispatch()) 
			{
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() 
	{
		shell = new Shell(getParent(), getStyle());
		shell.setSize(339, 420);
		shell.setText("Advanced Processing Parameters (F2)");
		
		Group groupBaseline = new Group(shell, SWT.NONE);
		groupBaseline.setText("Baseline");
		groupBaseline.setBounds(10, 22, 265, 153);
		
		Group groupSpecialCases = new Group(shell, SWT.NONE);
		groupSpecialCases.setText("Special Cases");
		groupSpecialCases.setBounds(10, 196, 313, 125);
		
		Label lblBaselinePolyDegree = new Label(groupBaseline, SWT.NONE);
		lblBaselinePolyDegree.setAlignment(SWT.RIGHT);
		lblBaselinePolyDegree.setBounds(11, 30, 128, 15);
		lblBaselinePolyDegree.setText("Baseline Poly. Degree:");
		
		Label lblFilterWidth = new Label(groupBaseline, SWT.NONE);
		lblFilterWidth.setAlignment(SWT.RIGHT);
		lblFilterWidth.setBounds(49, 57, 90, 15);
		lblFilterWidth.setText("Filter Width:");
		
		Label lblTreshold = new Label(groupBaseline, SWT.NONE);
		lblTreshold.setAlignment(SWT.RIGHT);
		lblTreshold.setBounds(40, 84, 99, 15);
		lblTreshold.setText("Treshold:");
		
		Label lblLambda = new Label(groupBaseline, SWT.NONE);
		lblLambda.setAlignment(SWT.RIGHT);
		lblLambda.setBounds(63, 111, 76, 15);
		lblLambda.setText("Lambda:");
		
		Label labelRefFreq = new Label(groupSpecialCases, SWT.NONE);
		labelRefFreq.setText("Reference Frequency:");
		labelRefFreq.setAlignment(SWT.RIGHT);
		labelRefFreq.setBounds(10, 24, 131, 15);
		
		Label labelSW = new Label(groupSpecialCases, SWT.NONE);
		labelSW.setText("Specctral Width:");
		labelSW.setAlignment(SWT.RIGHT);
		labelSW.setBounds(10, 51, 131, 15);
			
		Label labelMHz = new Label(groupSpecialCases, SWT.NONE);
		labelMHz.setText("MHz.");
		labelMHz.setBounds(256, 24, 55, 15);
		
		Label labelHz = new Label(groupSpecialCases, SWT.NONE);
		labelHz.setText("Hz");
		labelHz.setBounds(256, 51, 55, 15);
		
		textFilterWidth = new Text(groupBaseline, SWT.BORDER);
		textFilterWidth.setBounds(160, 54, 90, 21);
		textFilterWidth.setText(String.format("%d", SW_main.p_f2.baseline_filter));
		
		textThreshold = new Text(groupBaseline, SWT.BORDER);
		textThreshold.setBounds(160, 81, 90, 21);
		textThreshold.setText(String.format("%.3f", SW_main.p_f2.baseline_thresh));
		
		textLambda = new Text(groupBaseline, SWT.BORDER);
		textLambda.setBounds(160, 108, 90, 21);
		textLambda.setText(String.format("%.0f", SW_main.p_f2.baseline_lambda));
			
		Font currentFont;
		Button buttonOK = new Button(shell, SWT.NONE);	
		buttonOK.setBounds(157, 356, 80, 25);
		buttonOK.setText("OK");		
		currentFont = buttonOK.getFont();
		buttonOK.setFont(SWTResourceManager.getBoldFont(currentFont));
		buttonOK.setBackground(SWTResourceManager.getColor(SWT.COLOR_GREEN));
		shell.setDefaultButton(buttonOK);	// Default is OK button	
		
		Button buttonCancel = new Button(shell, SWT.NONE);
		
		buttonCancel.setBounds(243, 356, 80, 25);
		currentFont = buttonCancel.getFont();
		buttonCancel.setFont(SWTResourceManager.getBoldFont(currentFont));
		buttonCancel.setBackground(SWTResourceManager.getColor(SWT.COLOR_RED));
		buttonCancel.setText("Cancel");
						
		textRefFreq = new Text(groupSpecialCases, SWT.BORDER);
		textRefFreq.setBounds(160, 21, 90, 21);
		textRefFreq.setText(String.format("%.8f", SW_main.q_f2.SFO1));
		
		textSpectralWidth = new Text(groupSpecialCases, SWT.BORDER);
		textSpectralWidth.setBounds(160, 48, 90, 21);
		textSpectralWidth.setText(String.format("%.4f", SW_main.q_f2.SFO1 * SW_main.q_f2.SW));
					
		Button buttonObserve = new Button(groupSpecialCases, SWT.NONE);	
		buttonObserve.setText("Observe");
		buttonObserve.setBounds(14, 85, 85, 25);
		
		Button buttonDec = new Button(groupSpecialCases, SWT.NONE);	
		buttonDec.setText("Dec. ");
		buttonDec.setBounds(113, 85, 85, 25);
		
		Button buttonDec2 = new Button(groupSpecialCases, SWT.NONE);		
		buttonDec2.setText("Dec. 2");
		buttonDec2.setBounds(212, 85, 85, 25);
			
//-------------------------------------------------------------------------------------------------
//	Listeners
		
		buttonObserve.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				//SW_main.q_f2.SFO1 = SW_main.q.SFO1;
				textRefFreq.setText(String.format("%.8f", SW_main.q.SFO1));
			}
		});
		
		buttonDec.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				//SW_main.q_f2.SFO1 = SW_main.q.SFO2;
				textRefFreq.setText(String.format("%.8f", SW_main.q.SFO2));
			}
		});
		
		buttonDec2.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				//SW_main.q_f2.SFO1 = SW_main.q.SFO3;
				textRefFreq.setText(String.format("%.8f", SW_main.q.SFO3));
			}
		});
		
		buttonOK.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				try
				{
					SW_main.p_f2.baseline_filter = Integer.parseInt(textFilterWidth.getText());
					SW_main.p_f2.baseline_lambda = Double.parseDouble(textLambda.getText());
					SW_main.p_f2.baseline_thresh = Double.parseDouble(textThreshold.getText());
					SW_main.q_f2.SFO1 = Double.parseDouble(textRefFreq.getText());
								
					double new_SW = Double.parseDouble(textSpectralWidth.getText());
					
					if(Math.abs(SW_main.q_f2.SFO1 * SW_main.q_f2.SW - new_SW) > 0.001 && new_SW!= 0.0 && SW_main.q_f2.SFO1 > 0.001 )
					{
						SW_main.q_f2.SW = new_SW / SW_main.q_f2.SFO1;
						SW_main.p_f2.SW = SW_main.q_f2.SW;
						SW_main.q_f2.SF = SW_main.q_f2.SFO1; SW_main.p_f2.SF = SW_main.q_f2.SF;
						SW_main.q_f2.offset = new_SW / 2.0;
						SW_main.gp.freq1_indirect = SW_main.q_f2.offset;
						SW_main.gp.freq2_indirect = -SW_main.q_f2.offset;
						SW_main.q_f2.hzperpt = new_SW / (double)(SW_main.p_f2.SI);
						Util.show_and_log("SW in F2 manually changed,\nreference frequency set to transmitter");
					}       	
				}
				catch(Exception ex)
				{
					ex.printStackTrace(); 	// TODO something more later???
					Util.show_and_log(ex.getMessage());
				}
				
				shell.dispose();
			}
		});
		
		buttonCancel.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				shell.dispose();
			}
		});
			
	}	// end createContents()
//-------------------------------------------------------------------------------------------------
}
