/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package dialogs;

import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import spinworks.DX_Util;
import spinworks.Fft;
import spinworks.FileUtil;
import spinworks.Proc_2D;
import spinworks.SW_main;
import spinworks.Util;
import spinworks.Util_2D;
import spinworks.SW_main.modes;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Button;

/**
 * Interactive phasing dialog, 1D and 2D
 * @author Kirk Marat, 
 * University of Manitoba
 */
public class PhaseDialog extends Dialog
{

	protected Object result;
	protected Shell shell;
	
	double phase_0 = 0.0, phase_1 = 0.0, ph1_offset = 0.0;
	double ph0_coarse_scale = 180.0, ph0_fine_scale = 50.0, 
			      ph1_coarse_scale = 180.0, ph1_fine_scale = 50.0;
	
	static Label ph0_Value, ph1_Value;

	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public PhaseDialog(Shell parent, int style) 
	{
		super(parent, style);
		setText("Interactive Phasing");
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() 
	{
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) 
			{
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() 
	{
		shell = new Shell(getParent(), SWT.BORDER | SWT.TITLE);
		shell.setSize(482, 300);
		shell.setText(getText());
		
		final Scale ph0_scale = new Scale(shell, SWT.NONE);
		ph0_scale.setBounds(61, 10, 341, 42);
		ph0_scale.setMinimum(0);
		ph0_scale.setMaximum(360);
		ph0_scale.setSelection(180);
		
		final Scale ph0_scale_fine = new Scale(shell, SWT.NONE);
		ph0_scale_fine.setBounds(61, 56, 341, 42);
		ph0_scale_fine.setMinimum(0);
		ph0_scale_fine.setMaximum(100);
		ph0_scale_fine.setSelection(50);
		
		final Scale ph1_scale = new Scale(shell, SWT.NONE);
		ph1_scale.setBounds(61, 116, 341, 42);
		ph1_scale.setMinimum(0);
		ph1_scale.setMaximum(360);
		ph1_scale.setSelection(180);
		
		final Scale ph1_scale_fine = new Scale(shell, SWT.NONE);
		ph1_scale_fine.setBounds(61, 164, 341, 42);
		ph1_scale_fine.setMinimum(0);
		ph1_scale_fine.setMaximum(100);
		ph1_scale_fine.setSelection(50);
		
		Label label = new Label(shell, SWT.NONE);
		label.setBounds(10, 25, 55, 15);
		
		Label lblNewLabel = new Label(shell, SWT.NONE);
		lblNewLabel.setBounds(10, 37, 55, 15);
		lblNewLabel.setText("coarse");
		
		Label lblNewLabel_1 = new Label(shell, SWT.NONE);
		lblNewLabel_1.setBounds(10, 68, 55, 15);
		lblNewLabel_1.setText("fine");
		
		Label lblNewLabel_2 = new Label(shell, SWT.NONE);
		lblNewLabel_2.setBounds(10, 126, 55, 15);
		lblNewLabel_2.setText("coarse");
		
		Label lblNewLabel_3 = new Label(shell, SWT.NONE);
		lblNewLabel_3.setBounds(10, 170, 55, 15);
		lblNewLabel_3.setText("fine");
		
		ph0_Value = new Label(shell, SWT.BORDER);
		ph0_Value.setBounds(408, 68, 55, 25);
		ph0_Value.setText("0.0");
		
		ph1_Value = new Label(shell, SWT.BORDER);
		ph1_Value.setBounds(408, 180, 55, 25);
		ph1_Value.setText("0.0");
		
		Button buttonCancel = new Button(shell, SWT.NONE);
		buttonCancel.setBounds(391, 237, 75, 25);
		buttonCancel.setText("Cancel");
			
		Button buttonApplyExit = new Button(shell, SWT.NONE);	
		buttonApplyExit.setBounds(258, 237, 127, 25);
		buttonApplyExit.setText("Apply and Exit");
		
		Label lblPhase = new Label(shell, SWT.NONE);
		lblPhase.setBounds(408, 37, 55, 15);
		lblPhase.setText("phase 0");
		
		Label lblPhase_1 = new Label(shell, SWT.NONE);
		lblPhase_1.setText("phase 1");
		lblPhase_1.setBounds(408, 152, 55, 15);
		
		Button ph1Minus180_Button = new Button(shell, SWT.NONE);		
		ph1Minus180_Button.setBounds(10, 237, 101, 25);
		ph1Minus180_Button.setText("ph1 -180");
		
		Button ph1Plus180_Button = new Button(shell, SWT.NONE);	
		ph1Plus180_Button.setBounds(128, 237, 101, 25);
		ph1Plus180_Button.setText("ph1 +180");
		
		buttonApplyExit.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
				if(SW_main.gp.display_mode == SW_main.modes.MODE_1D)
				{				
					SW_main.p.PHC0 = phase_0;  
					SW_main.p.PHC1 = phase_1;
					
					Fft.do_phase(SW_main.gp.data_array, SW_main.gp.i_data_array, 
							SW_main.q.SI, SW_main.p.PHC0, SW_main.p.PHC1);
					
					SW_main.q.PHC0 = SW_main.p.PHC0 + SW_main.q.PHC0;        // update status parameters 	
					SW_main.q.PHC1 = SW_main.p.PHC1 + SW_main.q.PHC1;
					SW_main.p.PHC0 = SW_main.q.PHC0;
					SW_main.p.PHC1 = SW_main.q.PHC1;
					
					Util.rescale(SW_main.gp, SW_main.q);					 // generate scaling paramaters

					if (SW_main.gp.auto_save)
					{
	                    DX_Util.write_jdx(SW_main.q.data_folder + "spectrum.dx" , SW_main.gp, SW_main.q);
						FileUtil.saveProc1D(SW_main.gp, SW_main.q);
					}
				}
				else	// 2D
				{
					if(SW_main.gp.row_col == SW_main.trace_modes.ROWS)	
			        {
						SW_main.p.PHC0 = phase_0;
						SW_main.p.PHC1 = phase_1;

	                    // apply any requested phasing 
	                    if (SW_main.q.data_mode == SW_main.modes.MODE_3D && SW_main.gp.mode_3D == SW_main.modes_3D.XY)
	                        Proc_2D.phase_rows(SW_main.gp, SW_main.p, SW_main.q, SW_main.q_f2);	    // 3D XY		
	                    else
	                        Proc_2D.phase_rows(SW_main.gp, SW_main.p, SW_main.q, SW_main.q_f1);     // 2D or 3D XZ

	                    SW_main.q.PHC0 = SW_main.p.PHC0 + SW_main.q.PHC0;
	                    SW_main.q.PHC1 = SW_main.p.PHC1 + SW_main.q.PHC1;

	                    SW_main.p.PHC0 = SW_main.q.PHC0;
	                    SW_main.p.PHC1 = SW_main.q.PHC1;
			        }
			        else if(SW_main.gp.row_col == SW_main.trace_modes.COLUMNS)
			        {
				        
	                    if (SW_main.q.data_mode == SW_main.modes.MODE_3D && SW_main.gp.mode_3D == SW_main.modes_3D.XY)
	                    {                   	
	                    	SW_main.p_f2.PHC0 = phase_0;
	                    	SW_main.p_f2.PHC1 = phase_1;	                    	
	                    }
	                    else
	                    {
	                    	SW_main.p_f1.PHC0 = phase_0;
	                    	SW_main.p_f1.PHC1 = phase_1;
	                    }

	                    // apply any requested phasing 
	                    if (SW_main.q.data_mode == SW_main.modes.MODE_3D && SW_main.gp.mode_3D == SW_main.modes_3D.XY)
	                        Proc_2D.phase_columns(SW_main.gp, SW_main.q, SW_main.p_f2, SW_main.q_f2);	// 3D XY	
	                    else
	                        Proc_2D.phase_columns(SW_main.gp, SW_main.q, SW_main.p_f1, SW_main.q_f1);	// 2D or 3D XZ

	                    if (SW_main.q.data_mode == SW_main.modes.MODE_3D && SW_main.gp.mode_3D == SW_main.modes_3D.XY)
	                    {   // 3D XY
	                    	SW_main.q_f2.PHC0 = SW_main.p_f2.PHC0 + SW_main.q_f2.PHC0;
	                    	SW_main.q_f2.PHC1 = SW_main.p_f2.PHC1 + SW_main.q_f2.PHC1;
	                    	SW_main.p_f2.PHC0 = SW_main.q_f2.PHC0;
	                    	SW_main.p_f2.PHC1 = SW_main.q_f2.PHC1;
	                    }
	                    else
	                    {   // 2D or 3D XZ
	                    	SW_main.q_f1.PHC0 = SW_main.p_f1.PHC0 + SW_main.q_f1.PHC0;
	                    	SW_main.q_f1.PHC1 = SW_main.p_f1.PHC1 + SW_main.q_f1.PHC1;
	                    	SW_main.p_f1.PHC0 = SW_main.q_f1.PHC0;
	                    	SW_main.p_f1.PHC1 = SW_main.q_f1.PHC1;
	                    }
			        }

	                if (SW_main.q.data_mode == SW_main.modes.MODE_3D && SW_main.gp.mode_3D == SW_main.modes_3D.XY)
	                {   // 3D XY
	                	SW_main.gp.data_max_2D = Util_2D.max_2D(SW_main.gp.rows_real, 
	                			SW_main.q.SI, SW_main.q_f2.SI);	// find global maximum
	                	SW_main.gp.data_noise = Util_2D.noise_2D(SW_main.gp, SW_main.q, SW_main.q_f2);	// and noise level
	                }
	                else  // must be 2D or 3D XZ
	                {
	                	SW_main.gp.data_max_2D = Util_2D.max_2D(SW_main.gp.rows_real, 
	                			SW_main.q.SI, SW_main.q_f1.SI);	// find global maximum
	                	SW_main.gp.data_noise = Util_2D.noise_2D(SW_main.gp, SW_main.q, SW_main.q_f1);	// and noise level
	                }

	                if(SW_main.q.data_mode == modes.MODE_2D)
	                	FileUtil.saveProc2D(false, SW_main.gp, SW_main.q, SW_main.q_f1);
	                SW_main.gp.new_2D_view = true;							// 2D has changed, so must re-contour
				}
				
				SW_main.gp.phase_mode = false;
				SW_main.update_spect = true;
	    		SW_main.canvas.redraw();
	    		shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
				shell.dispose();
			}
		});
		
		buttonCancel.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.gp.phase_mode = false;
				SW_main.update_spect = true;
	    		SW_main.canvas.redraw();
				shell.dispose();
			}
		});
			
		ph0_scale.addSelectionListener(new SelectionAdapter() 
	    {
	    	public void widgetSelected(SelectionEvent e) 
	    	{
	    		//System.out.println(new Integer(ph0_scale.getSelection()).toString());
	    		ph0_coarse_scale = (double)ph0_scale.getSelection();
	    		calc_phases();
	    		SW_main.update_spect = true;
	    		SW_main.canvas.redraw();
	    	}
	    });
		
		ph0_scale_fine.addSelectionListener(new SelectionAdapter() 
	    {
	    	public void widgetSelected(SelectionEvent e) 
	    	{
	    		//System.out.println(new Integer(ph0_scale_fine.getSelection()).toString());
	    		ph0_fine_scale = (double)ph0_scale_fine.getSelection();
	    		calc_phases();
	    		SW_main.update_spect = true;
	    		SW_main.canvas.redraw();
	    	}
	    });
		
		ph1_scale.addSelectionListener(new SelectionAdapter() 
	    {
	    	public void widgetSelected(SelectionEvent e) 
	    	{
	    		//System.out.println(new Integer(ph1_scale.getSelection()).toString());
	    		ph1_coarse_scale = (double)ph1_scale.getSelection();
	    		calc_phases();
	    		SW_main.update_spect = true;
	    		SW_main.canvas.redraw();
	    	}
	    });		
				
		ph1_scale_fine.addSelectionListener(new SelectionAdapter() 
	    {
	    	public void widgetSelected(SelectionEvent e) 
	    	{
	    		//System.out.println(new Integer(ph1_scale_fine.getSelection()).toString());
	    		ph1_fine_scale = (double)ph1_scale_fine.getSelection();
	    		calc_phases();
	    		SW_main.update_spect = true;
	    		SW_main.canvas.redraw();
	    	}
	    });
		
		ph1Minus180_Button.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ph1_offset -= 180.0;
				calc_phases();
				
				ph1_Value.setText(String.format("%4.1f", phase_1));
	            ph0_Value.setText(String.format("%4.1f", phase_0));
	             
				SW_main.update_spect = true;
	    		SW_main.canvas.redraw();
			}
		});
		
		ph1Plus180_Button.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ph1_offset += 180.0;
				calc_phases();
				
				ph1_Value.setText(String.format("%4.1f", phase_1));
	            ph0_Value.setText(String.format("%4.1f", phase_0));
	            
				SW_main.update_spect = true;
	    		SW_main.canvas.redraw();
			}
		});
		
	}	// createContents
	
	private void calc_phases()
    {		
		phase_1 = ph1_coarse_scale - 180.0
                + (ph1_fine_scale - 50.0) / 10.0
                + ph1_offset; 
		
		if(SW_main.gp.row_col == SW_main.trace_modes.ROWS)
		{
			phase_0 = ph0_coarse_scale - 180.0
					+ (ph0_fine_scale - 50.0) / 10.0
					- phase_1 * ((double)SW_main.gp.pivot / (double)SW_main.q.SI);
		}
		
		else if(SW_main.gp.row_col == SW_main.trace_modes.COLUMNS)
        {
			if(SW_main.q.data_mode == modes.MODE_3D && SW_main.gp.mode_3D == SW_main.modes_3D.XY)
			{
				phase_0 = ph0_coarse_scale - 180.0
						+ (ph0_fine_scale - 50.0) / 10.0
						- phase_1 * ((double)SW_main.gp.pivot_ind / (double)SW_main.q_f2.SI);
			}
			else	// 2D or 3D  XZ
			{
				phase_0 = ph0_coarse_scale - 180.0
						+ (ph0_fine_scale - 50.0) / 10.0
						- phase_1 * ((double)SW_main.gp.pivot_ind / (double)SW_main.q_f1.SI);
			}
        }
		        
        ph0_Value.setText(String.format("%4.1f", phase_0));
        ph1_Value.setText(String.format("%4.1f", phase_1));
        SW_main.gp.ph0 = phase_0; 	SW_main.gp.ph1 = phase_1;
    }
//-------------------------------------------------------------------------------------------------
}	// class
