
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package dialogs;

import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.wb.swt.SWTResourceManager;

import spinworks.SW_main;
import spinworks.Util;

public class PlotParEditor extends Dialog 
{
	protected Object result;
	protected Shell shell;
	private Text textPlotWidth;
	private Text textPlotHeight;
	private Text textAxisFontHeight;
	private Text textIntegralFontHeight;
	private Text textParameterFontHeight;
	private Text textPeaklabelFontHeight;
	private Text textTitleFontHeight;
	private Text textSpectrumPenWidth;
	private Text textTraceOffset2D;
	
	Font currentFont;

	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public PlotParEditor(Shell parent, int style) 
	{
		super(parent, style);
		setText("Edit Plot Objects and Parameters");
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() 
	{
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) 
		{
			if (!display.readAndDispatch()) 
			{
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() 
	{
		shell = new Shell(getParent(), SWT.DIALOG_TRIM);
		shell.setSize(508, 575);
		shell.setText(getText());
		
		Group grpPlotObjects = new Group(shell, SWT.NONE);
		grpPlotObjects.setText("Plot Objects");
		grpPlotObjects.setBounds(20, 29, 162, 329);
		
		final Button buttonAxis = new Button(grpPlotObjects, SWT.CHECK);
		buttonAxis.setBounds(10, 27, 93, 16);
		buttonAxis.setText("Axis");
		if(SW_main.gp.plot_axis)
			buttonAxis.setSelection(true);
		else 
			buttonAxis.setSelection(false);
		
		final Button buttonFrame = new Button(grpPlotObjects, SWT.CHECK);
		buttonFrame.setBounds(10, 49, 93, 16);
		buttonFrame.setText("Frame");
		if(SW_main.gp.plot_frame)
			buttonFrame.setSelection(true);
		else
			buttonFrame.setSelection(false);
		
		final Button buttonGrid = new Button(grpPlotObjects, SWT.CHECK);
		buttonGrid.setBounds(10, 71, 93, 16);
		buttonGrid.setText("Grid");
		if(SW_main.gp.plot_grid)
			buttonGrid.setSelection(true);
		else
			buttonGrid.setSelection(false);
		
		final Button buttonIntegrals = new Button(grpPlotObjects, SWT.CHECK);
		buttonIntegrals.setBounds(10, 93, 93, 16);
		buttonIntegrals.setText("Integrals");
		if(SW_main.gp.plot_integrals)
			buttonIntegrals.setSelection(true);
		else
			buttonIntegrals.setSelection(false);
		
		final Button buttonIntegralValues = new Button(grpPlotObjects, SWT.CHECK);
		buttonIntegralValues.setBounds(10, 115, 93, 16);
		buttonIntegralValues.setText("Integral Values");
		if(SW_main.gp.plot_integral_values)
			buttonIntegralValues.setSelection(true);
		else
			buttonIntegralValues.setSelection(false);
		
		final Button buttonParameters = new Button(grpPlotObjects, SWT.CHECK);
		buttonParameters.setBounds(10, 137, 93, 16);
		buttonParameters.setText("Parameters");
		if(SW_main.gp.plot_parameters)
			buttonParameters.setSelection(true);
		else
			buttonParameters.setSelection(false);
		
		final Button buttonPeakLabels = new Button(grpPlotObjects, SWT.CHECK);
		buttonPeakLabels.setBounds(10, 159, 93, 16);
		buttonPeakLabels.setText("Peak Labels");
		if(SW_main.gp.plot_peak_labels)
			buttonPeakLabels.setSelection(true);
		else
			buttonPeakLabels.setSelection(false);
		
		final Button buttonSimulated = new Button(grpPlotObjects, SWT.CHECK);
		buttonSimulated.setBounds(10, 181, 93, 16);
		buttonSimulated.setText("Simulated");
		if(SW_main.gp.plot_simulated)
			buttonSimulated.setSelection(true);
		else
			buttonSimulated.setSelection(false);
		
		final Button buttonSpectrum = new Button(grpPlotObjects, SWT.CHECK);
		buttonSpectrum.setBounds(10, 203, 93, 16);
		buttonSpectrum.setText("Spectrum");
		if(SW_main.gp.plot_spectrum)
			buttonSpectrum.setSelection(true);
		else
			buttonSpectrum.setSelection(false);
		
		final Button buttonTitle = new Button(grpPlotObjects, SWT.CHECK);
		buttonTitle.setBounds(10, 225, 93, 16);
		buttonTitle.setText("Title");
		if(SW_main.gp.plot_title_flag)
			buttonTitle.setSelection(true);
		else
			buttonTitle.setSelection(false);
		
		Button buttonZeroLine = new Button(grpPlotObjects, SWT.CHECK);
		buttonZeroLine.setBounds(10, 247, 93, 16);
		buttonZeroLine.setText("Zero Line");
		
		final Button buttonIntegralsOnInsets = new Button(grpPlotObjects, SWT.CHECK);
		buttonIntegralsOnInsets.setBounds(10, 291, 142, 16);
		buttonIntegralsOnInsets.setText("Integrals on Insets");
		if(SW_main.gp.plot_integrals_inset)
			buttonIntegralsOnInsets.setSelection(true);
		else
			buttonIntegralsOnInsets.setSelection(false);
		
		final Button button2DParametersRight = new Button(shell, SWT.CHECK);
		button2DParametersRight.setBounds(30, 405, 162, 16);
		button2DParametersRight.setText("2D Parameters on Right");
		if(SW_main.gp.parameters_right)
			button2DParametersRight.setSelection(true);
		else
			button2DParametersRight.setSelection(false);
		
		final Button button2DContoursColour = new Button(shell, SWT.CHECK);
		button2DContoursColour.setBounds(30, 436, 172, 16);
		button2DContoursColour.setText("2D Contours in Colour");
		if(SW_main.gp.colour_contour)
			button2DContoursColour.setSelection(true);
		else
			button2DContoursColour.setSelection(false);
		
		Button buttonOK = new Button(shell, SWT.NONE);			// OK button
		currentFont = buttonOK.getFont();
		buttonOK.setFont(SWTResourceManager.getBoldFont(currentFont));
		buttonOK.setBounds(311, 510, 80, 27);
		buttonOK.setText("OK");
		buttonOK.setBackground(SWTResourceManager.getColor(SWT.COLOR_GREEN));
		
		Button buttonCancel = new Button(shell, SWT.NONE);		// Cancel button
		currentFont = buttonCancel.getFont();
		buttonCancel.setFont(SWTResourceManager.getBoldFont(currentFont));
		buttonCancel.setBounds(407, 510, 80, 27);
		buttonCancel.setText("Cancel");
		buttonCancel.setBackground(SWTResourceManager.getColor(SWT.COLOR_RED));
	
		
		textPlotWidth = new Text(shell, SWT.BORDER);
		textPlotWidth.setText(String.format("%.1f", SW_main.gp.plot_xsize / 10.0));
		textPlotWidth.setBounds(411, 50, 76, 21);
		
		textPlotHeight = new Text(shell, SWT.BORDER);
		textPlotHeight.setText(String.format("%.1f", SW_main.gp.plot_ysize / 10.0));
		textPlotHeight.setBounds(411, 77, 76, 21);
		
		textAxisFontHeight = new Text(shell, SWT.BORDER);
		textAxisFontHeight.setText(String.format("%.1f", SW_main.gp.axis_font_height));
		textAxisFontHeight.setBounds(411, 116, 76, 21);
		
		textIntegralFontHeight = new Text(shell, SWT.BORDER);
		textIntegralFontHeight.setText(String.format("%.1f", SW_main.gp.integ_font_height));
		textIntegralFontHeight.setBounds(411, 143, 76, 21);
		
		textParameterFontHeight = new Text(shell, SWT.BORDER);
		textParameterFontHeight.setText(String.format("%.1f", SW_main.gp.par_font_height));
		textParameterFontHeight.setBounds(411, 170, 76, 21);
		
		textPeaklabelFontHeight = new Text(shell, SWT.BORDER);
		textPeaklabelFontHeight.setText(String.format("%.1f", SW_main.gp.peak_font_height));
		textPeaklabelFontHeight.setBounds(411, 197, 76, 21);
		
		textTitleFontHeight = new Text(shell, SWT.BORDER);
		textTitleFontHeight.setText(String.format("%.1f", SW_main.gp.title_font_height));
		textTitleFontHeight.setBounds(411, 224, 76, 21);
		
		textSpectrumPenWidth = new Text(shell, SWT.BORDER);
		textSpectrumPenWidth.setText(String.format("%.1f", SW_main.gp.spect_pen_thickness));
		textSpectrumPenWidth.setBounds(411, 251, 76, 21);
			
		textTraceOffset2D = new Text(shell, SWT.BORDER);
		textTraceOffset2D.setText(String.format("%.2f", SW_main.gp.trace_offset));
		textTraceOffset2D.setBounds(411, 278, 76, 21);
		
		
		Button buttonSpectrumColour = new Button(shell, SWT.NONE);	
		buttonSpectrumColour.setBounds(411, 333, 75, 25);
		buttonSpectrumColour.setText("Select");
		buttonSpectrumColour.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ColorDialog dlg = new ColorDialog(shell);
				RGB rgb = dlg.open();
				if(rgb != null)
					SW_main.gp.print_spect_colour = new Color(Display.getCurrent(), rgb);
				//TODO how to get printer here???
			}
		});
		
		Button buttonSimulatedColour = new Button(shell, SWT.NONE);	
		buttonSimulatedColour.setBounds(411, 364, 75, 25);
		buttonSimulatedColour.setText("Select");
		buttonSimulatedColour.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ColorDialog dlg = new ColorDialog(shell);
				RGB rgb = dlg.open();
				if(rgb != null)
					SW_main.gp.print_sim_colour = new Color(Display.getCurrent(), rgb);
			}
		});
		
		Button buttonAxisColour = new Button(shell, SWT.NONE);		
		buttonAxisColour.setBounds(412, 395, 75, 25);
		buttonAxisColour.setText("Select");
		buttonAxisColour.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ColorDialog dlg = new ColorDialog(shell);
				RGB rgb = dlg.open();
				if(rgb != null)
					SW_main.gp.print_axis_colour = new Color(Display.getCurrent(), rgb);
			}
		});
		
		Button buttonIntegralColour = new Button(shell, SWT.NONE);		
		buttonIntegralColour.setBounds(412, 426, 75, 25);
		buttonIntegralColour.setText("Select");	
		buttonIntegralColour.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ColorDialog dlg = new ColorDialog(shell);
				RGB rgb = dlg.open();
				if(rgb != null)
					SW_main.gp.print_integ_colour = new Color(Display.getCurrent(), rgb);
			}
		});
		
		
		Label lblPlotWidthcm = new Label(shell, SWT.NONE);
		lblPlotWidthcm.setAlignment(SWT.RIGHT);
		lblPlotWidthcm.setBounds(255, 53, 136, 15);
		lblPlotWidthcm.setText("Plot Width (cm):");
		
		Label lblPlotHeightcm = new Label(shell, SWT.NONE);
		lblPlotHeightcm.setAlignment(SWT.RIGHT);
		lblPlotHeightcm.setBounds(255, 80, 136, 15);
		lblPlotHeightcm.setText("Plot Height (cm):");
		
		Label lblAxisFontHeight = new Label(shell, SWT.NONE);
		lblAxisFontHeight.setAlignment(SWT.RIGHT);
		lblAxisFontHeight.setBounds(229, 119, 162, 15);
		lblAxisFontHeight.setText("Axis Font Height (mm):");
		
		Label lblIntegralFontHeight = new Label(shell, SWT.NONE);
		lblIntegralFontHeight.setAlignment(SWT.RIGHT);
		lblIntegralFontHeight.setBounds(212, 146, 179, 15);
		lblIntegralFontHeight.setText("Integral Font Height (mm):");
		
		Label lblParameterFontHeight = new Label(shell, SWT.NONE);
		lblParameterFontHeight.setAlignment(SWT.RIGHT);
		lblParameterFontHeight.setBounds(200, 173, 191, 15);
		lblParameterFontHeight.setText("Parameter Font Height (mm):");
		
		Label lblPeakLabelFont = new Label(shell, SWT.NONE);
		lblPeakLabelFont.setAlignment(SWT.RIGHT);
		lblPeakLabelFont.setBounds(189, 200, 202, 15);
		lblPeakLabelFont.setText("Peak Label Font Height (mm):");
		
		Label lblTitleFontHeight = new Label(shell, SWT.NONE);
		lblTitleFontHeight.setAlignment(SWT.RIGHT);
		lblTitleFontHeight.setBounds(229, 227, 162, 15);
		lblTitleFontHeight.setText("Title Font Height (mm):");
		
		Label lblSpectrumPenWidth = new Label(shell, SWT.NONE);
		lblSpectrumPenWidth.setAlignment(SWT.RIGHT);
		lblSpectrumPenWidth.setBounds(216, 254, 175, 15);
		lblSpectrumPenWidth.setText("Spectrum Pen Width (mm):");
		
		Label lbl1DTraceOffset = new Label(shell, SWT.NONE);
		lbl1DTraceOffset.setAlignment(SWT.RIGHT);
		lbl1DTraceOffset.setBounds(200, 281, 191, 15);
		lbl1DTraceOffset.setText("1D Trace Offset in 2D (mm):");
		
		Label lblSpectrumColour = new Label(shell, SWT.NONE);
		lblSpectrumColour.setAlignment(SWT.RIGHT);
		lblSpectrumColour.setBounds(273, 338, 118, 15);
		lblSpectrumColour.setText("Spectrum Colour:");
		
		Label lblSimulatedColour = new Label(shell, SWT.NONE);
		lblSimulatedColour.setAlignment(SWT.RIGHT);
		lblSimulatedColour.setBounds(281, 369, 110, 15);
		lblSimulatedColour.setText("Simulated Colour:");
		
		Label lblAxisColour = new Label(shell, SWT.NONE);
		lblAxisColour.setAlignment(SWT.RIGHT);
		lblAxisColour.setBounds(282, 400, 109, 15);
		lblAxisColour.setText("Axis Colour:");
		
		Label lblIntegralColour = new Label(shell, SWT.NONE);
		lblIntegralColour.setAlignment(SWT.RIGHT);
		lblIntegralColour.setBounds(294, 431, 97, 15);
		lblIntegralColour.setText("Integral Colour:");
		
		buttonOK.addSelectionListener(new SelectionAdapter() 		// OK Button listener
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				try
				{
					SW_main.gp.plot_xsize = Double.parseDouble(textPlotWidth.getText()) * 10.0;
					SW_main.gp.plot_ysize = Double.parseDouble(textPlotHeight.getText()) * 10.0;
					SW_main.gp.axis_font_height = Float.parseFloat(textAxisFontHeight.getText());
					SW_main.gp.integ_font_height = Float.parseFloat(textIntegralFontHeight.getText());
					SW_main.gp.par_font_height = Float.parseFloat(textParameterFontHeight.getText());
					SW_main.gp.peak_font_height = Float.parseFloat(textPeaklabelFontHeight.getText());
					SW_main.gp.title_font_height = Float.parseFloat(textTitleFontHeight.getText());
					SW_main.gp.spect_pen_thickness = Float.parseFloat(textSpectrumPenWidth.getText());
					SW_main.gp.trace_offset = Double.parseDouble(textTraceOffset2D.getText());
					SW_main.gp.plot_axis = buttonAxis.getSelection();
					SW_main.gp.plot_frame = buttonFrame.getSelection();
					SW_main.gp.plot_grid = buttonGrid.getSelection();
					SW_main.gp.plot_integrals = buttonIntegrals.getSelection();
					SW_main.gp.plot_integral_values = buttonIntegralValues.getSelection();
					SW_main.gp.plot_integrals_inset = buttonIntegralsOnInsets.getSelection();
					SW_main.gp.plot_parameters = buttonParameters.getSelection();
					SW_main.gp.plot_peak_labels = buttonPeakLabels.getSelection();
					SW_main.gp.plot_simulated = buttonSimulated.getSelection();
					SW_main.gp.plot_spectrum = buttonSpectrum.getSelection();
					SW_main.gp.plot_title_flag = buttonTitle.getSelection();
					SW_main.gp.parameters_right = button2DParametersRight.getSelection();
					SW_main.gp.colour_contour = button2DContoursColour.getSelection();
				}
				catch(Exception ex)
				{
					Util.show_and_log("Error parsing plot parameters, check uour input",
							ex.getMessage());
				}
				shell.dispose();
			}
		});
		
		buttonCancel.addSelectionListener(new SelectionAdapter() 	// Cancel Button listener
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				shell.dispose();
			}
		});
		
		shell.setDefaultButton(buttonOK);				
	}
}
