package dialogs;

// could we just use the PNG utilities and a PNG viewer for print preview???

import org.eclipse.swt.widgets.Dialog;

import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import spinworks.PrintHandler;
import spinworks.PrintUtil;
import spinworks.PrintUtil2D;
import spinworks.SW_main;
import spinworks.data_par;
import spinworks.SW_main.modes;
import spinworks.SW_main.modes_3D;

import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;


// TODO  The way scaling is handled is very crude.  This needs to be cleaned-up

/**
 * This class creates a print preview dialog
 * @author marat
 */
public class PrintPreviewDialog extends Dialog 
{
	protected Object result;
	protected Shell shell;
	
	private int xsize, ysize;
	private Image image;
	private static GC gcImage;
	private static Color black;
	
	static double  printScaleFactorX;
	static double  printScaleFactorY;
	static double  scale_factor_y, scale_factor_x;
	final public static double mmToPoint = 2.835;  // converts mm to points

	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public PrintPreviewDialog(Shell parent, int style) 
	{
		super(parent, style);
		setText("Print Preview");
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() 
	{		
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) 
		{
			if (!display.readAndDispatch()) 
			{
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() 
	{
		shell = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.RESIZE);
		shell.setSize(1034, 850);
		shell.setText("Print Preview");

		Rectangle rect = shell.getClientArea();
		final Canvas canvas = new Canvas(shell, SWT.NONE);
		canvas.setBounds(0, 0, rect.width, rect.height - 50);
		
		Button buttonClose = new Button(shell, SWT.NONE);		
		buttonClose.setBounds(rect.width - 90, rect.height - 34,  80, 27);
		buttonClose.setText("Close");
		
		Button buttonPrint = new Button(shell, SWT.NONE);		
		buttonPrint.setText("Print");
		buttonPrint.setBounds(rect.width - 180, rect.height - 34, 80, 27);

		final int paperWidth = rect.width; final int paperHeight = rect.height - 50;
		xsize = paperWidth - 50;  ysize = paperHeight - 185;
		
		// scale factors, print sizes to preview sizes
		printScaleFactorX = (double)xsize / 250.0;  // scaled to "standard" 25 x 15 cm plots
		printScaleFactorY = (double)ysize / 150.0;
				
		// TODO find a better way to make sure scale factors in X and X are the same!
	
		// scale factor, display to print
		scale_factor_y = SW_main.gp.plot_ysize/((double)SW_main.gp.dysize);
		scale_factor_x = SW_main.gp.plot_xsize / ((double)SW_main.gp.dxsize);
		
		// paint listener
        canvas.addPaintListener(new PaintListener()
        { 
            public void paintControl(PaintEvent e)
            {
            	if(image != null) image.dispose();
        		if(gcImage != null)gcImage.dispose();
        		image = new Image(shell.getDisplay(), canvas.getBounds());
        		gcImage = new GC(image);        		
        		black = new Color(gcImage.getDevice(), 0, 0, 0);	// default colour
        		
        		if(SW_main.gp.print_spect_colour == null)
        			SW_main.gp.print_spect_colour = black;
        		
        		if(SW_main.gp.print_integ_colour == null)
        			SW_main.gp.print_integ_colour = black;
        		
        		if(SW_main.gp.print_axis_colour == null)
        			SW_main.gp.print_axis_colour = black;
        		
        		// Outline paper limits
        		gcImage.setForeground(black);
        		gcImage.drawRectangle(1, 1, paperWidth - 2, paperHeight - 2);       		
        		       		
        		if(SW_main.gp.display_mode == modes.MODE_1D)
        			doPreview_1D();
        		else if(SW_main.gp.display_mode == modes.MODE_2D)
        			doPreview_2D();
        		
        		e.gc.drawImage(image, 0, 0);				// map bitmap to screen             
            }     
        }); 
               
		buttonClose.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				shell.dispose();
			}
		});

		buttonPrint.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				PrintHandler.doPrint();
				shell.dispose();
			}
		});
		
		shell.setDefaultButton(buttonClose);				
	}
	
//-------------------------------------------------------------------------------------------------
	private static void doPreview_1D()
	{	
		int points = 
				Math.abs((int)((SW_main.q.offset - SW_main.gp.freq1) / SW_main.q.hzperpt) 
				       - (int)((SW_main.q.offset - SW_main.gp.freq2) / SW_main.q.hzperpt));
		
		double thresh;			// threshold for print speed-up
		if (points > 40000)
			thresh = 0.1;
		else
			thresh = 0.02;
		
		double int_off;			// added space for integrals

        if (SW_main.gp.plot_integral_values && SW_main.gp.n_integrals != 0)	// add space for integral
        	int_off = 10.0;								    // values if necessary
        else
        	int_off = 0.0;	
		 
//    	print any stacked traces
//		Do this first so that any whitewashing won't erase other objects, if present
        for (int i = SW_main.gp.n_traces - 1; i >= 0; i--)
        {
        	int j; 

        	if (SW_main.gp.plot_spectrum == false)
        		j = i;
        	else
        		j = i + 1;

        	points =
        			Math.abs((int)((SW_main.gp.stack_trace[i].offset - SW_main.gp.freq1) / SW_main.gp.stack_trace[i].hzperpt)
        					- (int)((SW_main.gp.stack_trace[i].offset - SW_main.gp.freq2) / SW_main.gp.stack_trace[i].hzperpt));

        	if (points > 40000)
        		thresh = 0.1;
        	else
        		thresh = 0.02;

        	double xsize;

        	if (SW_main.gp.stack_x_size_auto)
        		xsize = SW_main.gp.plot_xsize;
        	else
        		xsize = SW_main.gp.stack_x_size * scale_factor_x;

        	PrintUtil.print_spect(gcImage, SW_main.gp.stack_trace[i].print_colour,
        			SW_main.gp.freq1, SW_main.gp.freq2,
        			SW_main.gp.stack_trace[i].yscale * scale_factor_y,
        			(SW_main.gp.expt_y_offset + SW_main.gp.stack_y_offset * j - SW_main.gp.axis_offset) * scale_factor_y + int_off,
        			SW_main.gp.stack_trace[i].data, thresh,
        			xsize, SW_main.gp.plot_ysize,
        			SW_main.gp.plot_xoffset + (SW_main.gp.stack_x_offset * j) * scale_factor_x, 
        			SW_main.gp.plot_yoffset, 
        			printScaleFactorX, SW_main.gp, SW_main.q);
        	
        	double off = SW_main.gp.stack_y_offset * j * scale_factor_y;
        	
        	String label;

        	if (SW_main.gp.stack_label_type == 0)
        		label = "";
        	else if (SW_main.gp.stack_label_type == 1)
        		label = String.format("%d", i + 1);
        	else if (SW_main.gp.stack_label_type == 2)
        		label = SW_main.gp.stack_trace[i].title;
        	else if (SW_main.gp.stack_label_type == 3)
        		label = SW_main.gp.stack_trace[i].folder;
        	else
        		label = "";
        				
        	gcImage.drawText(label, 
        			(int)((SW_main.gp.plot_xoffset + (SW_main.gp.stack_x_offset * j * scale_factor_x)) 
        					* printScaleFactorX),
        			(int)((SW_main.gp.plot_ysize /*- SW_main.gp.plot_yoffset*/ - off) * printScaleFactorY));      			
        }
        
		if(SW_main.gp.plot_spectrum)		// plot the spectrum
			PrintUtil.print_spect(gcImage, SW_main.gp.print_spect_colour,
					SW_main.gp.freq1, SW_main.gp.freq2, 
					SW_main.gp.expt_scale * scale_factor_y,
					(SW_main.gp.expt_y_offset - SW_main.gp.axis_offset) * scale_factor_y + int_off,
					SW_main.gp.data_array, thresh,
					SW_main.gp.plot_xsize, SW_main.gp.plot_ysize, 
					SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset,
					printScaleFactorX, SW_main.gp, SW_main.q);
		
		if(SW_main.gp.plot_simulated)
			PrintUtil.print_spect(gcImage, SW_main.gp.print_sim_colour,
					SW_main.gp.freq1, SW_main.gp.freq2, 
					SW_main.gp.calc_scale * scale_factor_y,
					(SW_main.gp.calc_y_offset - SW_main.gp.axis_offset) * scale_factor_y + int_off,
					SW_main.gp.calc_array, thresh,
					SW_main.gp.plot_xsize, SW_main.gp.plot_ysize, 
					SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset,
					printScaleFactorX, SW_main.gp, SW_main.q);
        
		if(SW_main.gp.plot_axis)			// Plot the axis
		{
			Font axisFont = new Font(gcImage.getDevice(), "Arial", 
					(int)(SW_main.gp.axis_font_height * mmToPoint), 
					SWT.NORMAL);  // font sizes are in points	

			PrintUtil.print_axis(gcImage, axisFont, SW_main.gp.print_axis_colour, 
					SW_main.gp.freq1, SW_main.gp.freq2, 
					SW_main.gp.plot_xsize, SW_main.gp.plot_ysize,
					SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset,
					printScaleFactorX, SW_main.gp, SW_main.q, SW_main.q_f1);	// TODO change to q_ind
			axisFont.dispose();
		}
		
		if(SW_main.gp.plot_integrals)
        {
        	Font integFont = new Font(gcImage.getDevice(), "Arial", (int)(SW_main.gp.integ_font_height * mmToPoint), 
        			SWT.NORMAL);  // font sizes are in points
        	PrintUtil.print_integ(gcImage, integFont, SW_main.gp.print_integ_colour,
        			SW_main.gp.freq1, SW_main.gp.freq2, 
        			SW_main.gp.integ_scale * scale_factor_y, SW_main.gp.expt_y_offset * scale_factor_y, 
        			SW_main.gp.plot_xsize, SW_main.gp.plot_ysize, 
        			SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset, printScaleFactorX, 
        			SW_main.gp, SW_main.q);
        	integFont.dispose();
        }
        
        if(SW_main.gp.plot_peak_labels)
        {
        	Font peakFont = new Font(gcImage.getDevice(), "Arial", (int)(SW_main.gp.peak_font_height * mmToPoint), 
        			SWT.NORMAL);  // font sizes are in points       	
        	PrintUtil.print_peaks(gcImage, peakFont, SW_main.gp.print_integ_colour, // peaks same colour as integrals for now
        			SW_main.gp.freq1, SW_main.gp.freq2, 
        			SW_main.gp.plot_xsize, SW_main.gp.plot_ysize, 
        			SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset, 
        			printScaleFactorX, 
        			SW_main.gp.expt_scale * scale_factor_y, 
        			(SW_main.gp.expt_y_offset - SW_main.gp.axis_offset) * scale_factor_y + int_off,
        			SW_main.gp, SW_main.q);
        	peakFont.dispose();
        }
		
		if(SW_main.gp.plot_parameters)	// plot the parameters, if desired
		{
			Font parFont = new Font(SW_main.gcImage.getDevice(), "Arial", (int)(SW_main.gp.par_font_height * mmToPoint), 
					SWT.NORMAL);  // font sizes are in points
			PrintUtil.print_parameters(gcImage, parFont, black,
					SW_main.gp.plot_xsize, SW_main.gp.plot_ysize - 5.0, 
					SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset, printScaleFactorX,
					SW_main.gp, SW_main.q);
			parFont.dispose();
		}
		
		if(SW_main.gp.plot_title_flag)	// print the plot title
        {
        	Font titleFont = new Font(gcImage.getDevice(), "Arial", (int)(SW_main.gp.title_font_height * mmToPoint), 
        			SWT.NORMAL);  // font sizes are in points
        	gcImage.setFont(titleFont);
        	gcImage.setForeground(black);
	        gcImage.drawText(String.format("SpinWorks Java: %s", SW_main.gp.plot_title),  
		                   (int)((SW_main.gp.plot_xoffset + 1.0) * printScaleFactorX), 
		                   (int)((SW_main.gp.plot_yoffset - SW_main.gp.title_font_height - 1.0) * printScaleFactorY));
	        titleFont.dispose();
        }
		
		if(SW_main.gp.plot_frame)			// draw a nice frame around it, if desired
	        gcImage.drawRectangle((int)(SW_main.gp.plot_xoffset * printScaleFactorX), 
	        		         (int)(SW_main.gp.plot_yoffset * printScaleFactorY), 
	                         (int)(SW_main.gp.plot_xsize * printScaleFactorX), 
	                         (int)(SW_main.gp.plot_ysize * printScaleFactorY));
		
//    	print any inset windows (boxes)including integrals within the inset box
		for (int i = 0; i < SW_main.gp.n_boxes; i++)
		{
			if (SW_main.gp.printInsetBox[i].active)
			{
				Font axisFont = new Font(gcImage.getDevice(), "Arial", 
						(int)(SW_main.gp.axis_font_height * mmToPoint), 
						 SWT.NORMAL);  // font sizes are in points					 
				 PrintUtil.print_axis(gcImage, axisFont, SW_main.gp.print_axis_colour, 
						 SW_main.gp.printInsetBox[i].freq1, SW_main.gp.printInsetBox[i].freq2, 
						 SW_main.gp.printInsetBox[i].page_width, SW_main.gp.printInsetBox[i].page_height,
						 SW_main.gp.plot_xoffset + SW_main.gp.printInsetBox[i].page_x, 
						 SW_main.gp.plot_yoffset + SW_main.gp.printInsetBox[i].page_y,
						 printScaleFactorX, SW_main.gp, SW_main.q, SW_main.q_f1);	// TODO change to q_ind
				 axisFont.dispose();
				 
				gcImage.setLineWidth((int)(SW_main.gp.spect_pen_thickness * printScaleFactorX));
	        	PrintUtil.print_spect(gcImage, SW_main.gp.print_spect_colour,
	        			SW_main.gp.printInsetBox[i].freq1, SW_main.gp.printInsetBox[i].freq2, 
	        			SW_main.gp.printInsetBox[i].yscale * scale_factor_y,
	        			(SW_main.gp.expt_y_offset - SW_main.gp.axis_offset) * scale_factor_y + int_off,
	        			SW_main.gp.data_array, thresh,
	        			SW_main.gp.printInsetBox[i].page_width, 
	        			SW_main.gp.printInsetBox[i].page_height, 
	        			SW_main.gp.plot_xoffset + SW_main.gp.printInsetBox[i].page_x, 
	        			SW_main.gp.plot_yoffset + SW_main.gp.printInsetBox[i].page_y,
	        			printScaleFactorX, SW_main.gp, SW_main.q);	
	        	
	        	if(SW_main.gp.plot_integrals && SW_main.gp.plot_integrals_inset)
	            {
	            	Font integFont = new Font(gcImage.getDevice(), "Arial", 
	            			(int)(SW_main.gp.integ_font_height * mmToPoint), 
	            			SWT.NORMAL);  // font sizes are in points
	            	PrintUtil.print_integ(gcImage, integFont, SW_main.gp.print_integ_colour,
	            			SW_main.gp.printInsetBox[i].freq1, SW_main.gp.printInsetBox[i].freq2, 
	            			SW_main.gp.integ_scale * scale_factor_y, 
	            			SW_main.gp.expt_y_offset * scale_factor_y, 
	            			SW_main.gp.printInsetBox[i].page_width, 
	            			SW_main.gp.printInsetBox[i].page_height, 
	            			SW_main.gp.plot_xoffset + SW_main.gp.printInsetBox[i].page_x, 
	            			SW_main.gp.plot_yoffset + SW_main.gp.printInsetBox[i].page_y, 
	            			printScaleFactorX, SW_main.gp, SW_main.q);
	            	integFont.dispose();
	            }
	              	
				/*gcImage.drawRectangle((int)(SW_main.gp.plot_xoffset * printScaleFactorX 
						+ SW_main.gp.printInsetBox[i].page_x * printScaleFactorX), 
						(int)(SW_main.gp.plot_yoffset * printScaleFactorY
								+ SW_main.gp.printInsetBox[i].page_y * printScaleFactorX),
						(int)(SW_main.gp.printInsetBox[i].page_width * printScaleFactorX), 
						(int)(SW_main.gp.printInsetBox[i].page_height * printScaleFactorY));*/
			}                   
		}  			
	}

//------------------------------------------------------------------------------------------------
	private static void doPreview_2D()
	{
		double top_space, left_space;

		if (!SW_main.gp.rotate_2D)			// normal (F2 horizontal) orientation
		{
			if (SW_main.gp.f2_trace_on)
				top_space = 40.0;
			else
				top_space = 0.0;

			if (SW_main.gp.f1_trace_on)
				left_space = 40.0;
			else
				left_space = 0.0;
		}
		else								// rotated (F1 horizontal) orientation
		{		
			if (SW_main.gp.f2_trace_on)
				left_space = 40.0;
			else
				left_space = 0.0;

			if (SW_main.gp.f1_trace_on)
				top_space = 40.0;
			else
				top_space = 0.0;
		}

		//adjust sizes to allow for "projections"
		double plot_xsize_2D = SW_main.gp.plot_xsize  - left_space;
		double plot_ysize_2D = SW_main.gp.plot_ysize  - top_space;

		// select correct parameters for indirect dimension
		data_par q_ind = new data_par();
		if (SW_main.q.data_mode == modes.MODE_3D && SW_main.gp.mode_3D == modes_3D.XY)
			q_ind = (data_par)SW_main.q_f2;
		else
			q_ind = (data_par)SW_main.q_f1;

		// print the 2D contour map
		if(!SW_main.gp.rotate_2D)	// normal orientatikon
			PrintUtil2D.print_2D_contour(gcImage, gcImage.getDevice(),							// graphics object and pens
					plot_xsize_2D, plot_ysize_2D,												// sizes in mm
					SW_main.gp.plot_xoffset + left_space, SW_main.gp.plot_yoffset + top_space,	// offsets from UL in mm
					printScaleFactorX, SW_main.gp, SW_main.q, q_ind);							// parameter structures

		else				// rotated (indrect horizonral) orientation
			PrintUtil2D.print_2D_contour_rotated(gcImage, gcImage.getDevice(),					// graphics object and pens
					plot_xsize_2D, plot_ysize_2D,								// sizes in mm
					SW_main.gp.plot_xoffset + left_space, SW_main.gp.plot_yoffset + top_space,	// offsets from UL in mm
					printScaleFactorX, SW_main.gp, SW_main.q, q_ind);							// parameter structures

		if(SW_main.gp.plot_axis)			// Plot the axis
		{
			Font axisFont = new Font(gcImage.getDevice(), "Arial", 
					(int)(SW_main.gp.axis_font_height * mmToPoint), SWT.NORMAL);  // font sizes are in points					 
			gcImage.setLineWidth((int)(SW_main.gp.spect_pen_thickness * printScaleFactorX));
			gcImage.setForeground(new Color(gcImage.getDevice(), 0, 0, 0)); // TODO proper colour handling

			//  print axes
			if (!SW_main.gp.rotate_2D)	// normal orientation
			{
				//		print the x axis
				PrintUtil.print_axis(gcImage, axisFont, SW_main.gp.print_axis_colour,
						SW_main.gp.freq1, SW_main.gp.freq2,
						plot_xsize_2D, plot_ysize_2D,
						SW_main.gp.plot_xoffset + left_space, SW_main.gp.plot_yoffset + top_space,
						printScaleFactorX, SW_main.gp, SW_main.q, q_ind);

				//		print the y axis
				PrintUtil2D.print_y_axis(gcImage, axisFont, SW_main.gp.print_axis_colour,
						SW_main.gp.freq1_indirect, SW_main.gp.freq2_indirect,
						plot_xsize_2D, plot_ysize_2D,
						SW_main.gp.plot_xoffset + left_space, SW_main.gp.plot_yoffset + top_space,
						printScaleFactorX, SW_main.gp, SW_main.q, q_ind);
			}
			else				// rotated orientation
			{
				//		print the x axis
				PrintUtil.print_axis(gcImage, axisFont, SW_main.gp.print_axis_colour,
						SW_main.gp.freq1_indirect, SW_main.gp.freq2_indirect,
						plot_xsize_2D, plot_ysize_2D,
						SW_main.gp.plot_xoffset + left_space, SW_main.gp.plot_yoffset + top_space,
						printScaleFactorX, SW_main.gp, SW_main.q, q_ind);

				//		print the y axis
				PrintUtil2D.print_y_axis(gcImage, axisFont, SW_main.gp.print_axis_colour,
						SW_main.gp.freq1, SW_main.gp.freq2,
						plot_xsize_2D, plot_ysize_2D,
						SW_main.gp.plot_xoffset + left_space, SW_main.gp.plot_yoffset + top_space,
						printScaleFactorX, SW_main.gp, SW_main.q, q_ind);
			}
			axisFont.dispose();
		}

		//  print the F2 trace
		if (SW_main.gp.f2_trace_on)
		{
			if (!SW_main.gp.rotate_2D)			// normal orientation
				PrintUtil2D.print_spect_f2(gcImage, SW_main.gp.print_spect_colour,				// print objects
						SW_main.gp.freq1, SW_main.gp.freq2, SW_main.gp.f2_trace_scale * 4.2,	// freq and scaling
						SW_main.gp.f2_trace, 0.1,												// array and threshold
						plot_xsize_2D, top_space,												// sizes
						SW_main.gp.plot_xoffset + left_space,                           		// offsets
						SW_main.gp.plot_yoffset - SW_main.gp.trace_offset,						
						printScaleFactorX, 														// printer scale factor
						SW_main.gp);															// global parameters
			else						// rotated orientation
			{
				PrintUtil2D.print_spect_f2_rotated(gcImage, SW_main.gp.print_spect_colour,
						SW_main.gp.freq1, SW_main.gp.freq2, SW_main.gp.f2_trace_scale * 4.2,
						SW_main.gp.f2_trace, 0.1,
						plot_ysize_2D, left_space,
						SW_main.gp.plot_yoffset + top_space,
						SW_main.gp.plot_xoffset - SW_main.gp.trace_offset,
						printScaleFactorX, SW_main.gp);
			}
		}

		//  print F1 trace
		if(SW_main.gp.f1_trace_on)
		{
			if (!SW_main.gp.rotate_2D)			// normal orientation
				PrintUtil2D.print_spect_f1(gcImage, SW_main.gp.print_spect_colour,
						SW_main.gp.freq1_indirect, SW_main.gp.freq2_indirect, SW_main.gp.f1_trace_scale * 4.2,
						SW_main.gp.f1_trace, 0.1,
						plot_ysize_2D, left_space,
						top_space + SW_main.gp.plot_yoffset,
						SW_main.gp.plot_xoffset - SW_main.gp.trace_offset,
						printScaleFactorX, SW_main.gp);
			else						// rotated orientation
			{
				PrintUtil2D.print_spect_f1_rotated(gcImage, SW_main.gp.print_spect_colour,
						SW_main.gp.freq1_indirect, SW_main.gp.freq2_indirect, SW_main.gp.f1_trace_scale * 4.2,
						SW_main.gp.f1_trace, 0.1,
						plot_xsize_2D, top_space,
						left_space + SW_main.gp.plot_xoffset,
						SW_main.gp.plot_yoffset - SW_main.gp.trace_offset,
						printScaleFactorX, SW_main.gp);
			}
		}

		//	draw a nice frame around it, if desired
		if(SW_main.gp.plot_frame)
		{
			gcImage.setForeground(black); // TODO proper colour handling
			gcImage.drawRectangle((int)((SW_main.gp.plot_xoffset + left_space) * printScaleFactorX), 
					(int)((SW_main.gp.plot_yoffset + top_space) * printScaleFactorX), 
					(int)(plot_xsize_2D * printScaleFactorX), (int)(plot_ysize_2D * printScaleFactorX));
		}

		//	print the plot title
		if(SW_main.gp.plot_title_flag)
		{
			gcImage.setFont(new Font(gcImage.getDevice(), "Arial", 
					(int)(SW_main.gp.title_font_height * mmToPoint), SWT.NORMAL));  // font sizes are in points
			gcImage.setForeground(black); 
			gcImage.drawText("SpinWorks Java: " + SW_main.gp.plot_title, 
					(int)((SW_main.gp.plot_xoffset + 1.0) * printScaleFactorX), 
					(int)((SW_main.gp.plot_yoffset - SW_main.gp.title_font_height - 1.0) * printScaleFactorX));
		}

		//	plot the parameters, if desired
		if(SW_main.gp.plot_parameters)	
		{
			Font parFont = new Font(gcImage.getDevice(), "Arial", (int)(SW_main.gp.par_font_height * mmToPoint), 
					SWT.NORMAL);  // font sizes are in points
			gcImage.setFont(parFont);
			PrintUtil2D.print_parameters_2D(gcImage, parFont, black,
					SW_main.gp.plot_xsize, SW_main.gp.plot_ysize, 
					SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset, printScaleFactorX,
					SW_main.gp, SW_main.q, q_ind);
			parFont.dispose();
		}
	}
//-------------------------------------------------------------------------------------------------
}
