
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package dialogs;

import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.wb.swt.SWTResourceManager;

import spinworks.SW_main;
import spinworks.Util;

import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Combo;

public class ScaleDialog extends Dialog 
{
	protected Object result;
	protected Shell shell;
	Font currentFont;
	private Text text;

	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public ScaleDialog(Shell parent, int style) 
	{
		super(parent, style);
		setText("Set Fixed Scale Dialog");
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() 
	{
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) 
		{
			if (!display.readAndDispatch()) 
			{
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() 
	{
		shell = new Shell(getParent(), SWT.DIALOG_TRIM);
		shell.setSize(319, 107);
		shell.setText("Set Fixed Scale");
		
		Button buttonOK = new Button(shell, SWT.NONE);	
		if(SW_main.currentOS != SW_main.osTypes.MACOS)
		{
			currentFont = buttonOK.getFont();
			buttonOK.setFont(SWTResourceManager.getBoldFont(currentFont));
			buttonOK.setBackground(SWTResourceManager.getColor(SWT.COLOR_GREEN));
		}
		buttonOK.setBounds(136, 43, 80, 27);
		buttonOK.setText("OK");
		shell.setDefaultButton(buttonOK);	// Default is OK button		
		
		Button buttonCancel = new Button(shell, SWT.NONE);	
		if(SW_main.currentOS != SW_main.osTypes.MACOS)
		{
			currentFont = buttonCancel.getFont();
			buttonCancel.setFont(SWTResourceManager.getBoldFont(currentFont));
			buttonCancel.setBackground(SWTResourceManager.getColor(SWT.COLOR_RED));
		}
		buttonCancel.setBounds(222, 44, 75, 25);
		buttonCancel.setText("Cancel");
		
		Label lblPlotScale = new Label(shell, SWT.NONE);
		lblPlotScale.setBounds(10, 10, 55, 15);
		lblPlotScale.setText("Plot Scale:");
		
		text = new Text(shell, SWT.BORDER);
		text.setBounds(84, 7, 125, 21);
		text.setText(String.format("%.4f", 
				(SW_main.gp.freq1 - SW_main.gp.freq2) / (SW_main.gp.plot_xsize / 10.0)));
		
		final Combo comboUnit = new Combo(shell, SWT.NONE);
		comboUnit.setItems(new String[] {"Hz/cm", "ppm/cm"});
		comboUnit.setBounds(222, 5, 75, 23);
		comboUnit.select(0);
		
//	Listeners
		buttonOK.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(comboUnit.getSelectionIndex() == 0)			// Hz/cm
				{
					try
					{
						double hzcm = Double.parseDouble(text.getText());
						SW_main.gp.freq2 = SW_main.gp.freq1 
								- (hzcm * SW_main.gp.plot_xsize / 10.0);
					}
					catch(Exception ex)
					{
						Util.show_and_log("Error converting hz/cm", ex.getMessage());
					}
				}
				else if (comboUnit.getSelectionIndex() == 1)	// ppm/cm
				{
					try
					{
					double ppmcm = Double.parseDouble(text.getText());
					SW_main.gp.freq2 = SW_main.gp.freq1 
							- (ppmcm * SW_main.q.SF * SW_main.gp.plot_xsize / 10.0);
					}
					catch(Exception ex)
					{
						Util.show_and_log("Error converting ppm/cm", ex.getMessage());
					}
				}
				
				shell.dispose();
			}
		});
		
		buttonCancel.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				shell.dispose();
			}
		});
		
		comboUnit.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(comboUnit.getSelectionIndex() == 0)			// Hz/cm
				{
					text.setText(String.format("%.4f", 
							(SW_main.gp.freq1 - SW_main.gp.freq2) / (SW_main.gp.plot_xsize / 10.0)));
				}
				else if (comboUnit.getSelectionIndex() == 1)	// ppm/cm
				{
					text.setText(String.format("%.3f", 
							((SW_main.gp.freq1 - SW_main.gp.freq2) / SW_main.q.SF) 
							    / (SW_main.gp.plot_xsize / 10.0)));
				}
			}
		});					
	}	// end createContents()
//-------------------------------------------------------------------------------------------------
}
