package lp;

/**
 * Complex number class
 * @author marat
 */
public class Complex 
{
//	definition of the Complex type
	public double r;		// real part
	public double i;		// imaginary (complex) part

	public Complex(double _r, double _i)							// constructor
	{ this.r = _r; this.i = _i; }

//------------------------------------------------------------------------------------------------
//	basic operators (no operator overloads in Java  :-(
	
	public static Complex add(Complex lhs, Complex rhs)
	{
		return new Complex(lhs.r + rhs.r, lhs.i + rhs.i);
	}
	
	public static Complex sub(Complex lhs, Complex rhs)
	{
		return new Complex(lhs.r - rhs.r, lhs.i - rhs.i);
	}
	
	public static Complex  mult(Complex lhs, Complex rhs)		// cmplx * cmplx multiply
	{
		return new Complex(lhs.r * rhs.r - lhs.i * rhs.i,
							lhs.r * rhs.i + lhs.i * rhs.r);
	}
	
	public static Complex mult(double lhs, Complex rhs)		   	// real * cmplx  multiply
	{
		return new Complex(lhs * rhs.r, lhs * rhs.i);
	}
	
	public static Complex div(Complex lhs, Complex rhs)			// cmplx / cmplx  divide
	{
		double denom = rhs.r * rhs.r + rhs.i * rhs.i;
		double a = (lhs.r * rhs.r + lhs.i * rhs.i) / denom;
		double b = (lhs.i * rhs.r - lhs.r * rhs.i) / denom;

		return new Complex(a, b);
	}
	
	public static Complex div(double lhs, Complex rhs)			// real / cmplx divide
	{
		double denom = rhs.r * rhs.r + rhs.i * rhs.i;
		double a = (lhs * rhs.r) / denom;
		double b = -(lhs * rhs.i) / denom;

		return new Complex(a, b);
	}
	
	public static Complex div(Complex lhs, double rhs)			// complex / real divide
	{
		return new Complex(lhs.r / rhs, lhs.i / rhs);
	}
	
//-------------------------------------------------------------------------------------------------
//  Other Complex utility functions
	
	public void minus() { r = -r; i = -i; }							// unary minus
	
	public static Complex minus(Complex x)							// unary minus as function
	{
		return new Complex(-x.r, -x.i);
	}
	
	public void conj() { i = -i; } 									// complex conjugate
		
	public static Complex conjf(Complex x)							// complex conjugate 
	{																// as function
		return new Complex(x.r, -x.i);
	}
	
	public void zero() { r = 0.0; i = 0.0; }						// set to zero
	public void set(double a, double b) { r = a; i = b; }			// set to value
	
	public static Complex cmplx(double a, double b)					// returns a Complex Object
	{ 																// duplicates constructor?
		return new Complex(a, b);
	}

//-------------------------------------------------------------------------------------------------
	/**
	 * @return a String representing the Complex number
	 */
	public String ToString()
	{
		return String.format("%f  +  %f  i", this.r, this.i);
	}
	
//-------------------------------------------------------------------------------------------------
	/**
	 * complex sqrt (modern version)
	 * @param x a comples number
	 * @return the comples square root of x
	 */
	public static Complex Sqrt(Complex x)
	{
		double a = x.r, b = x.i;
		double c = Math.sqrt(((a + Math.sqrt((a * a + b * b))) / 2));
		double d = Math.sqrt((c * c - a));

		return new Complex(c, d);
	}
		
//-------------------------------------------------------------------------------------------------
	/**
	 * complex absolute value
	 * @param x a complex number
	 * @return the absolute value (real) of x
	 */
	public static double Abs(Complex x)
	{
		double a = Math.sqrt((x.r * x.r + x.i * x.i));
		return a;
	}

//-------------------------------------------------------------------------------------------------
	/**
	 * Copies a complex vector from x2 to x1
	 * @param x1 destination
	 * @param x2 origin
	 * @param n number of points
	 */
	public static void complex_copy(Complex[] x1, Complex[] x2, int n)
	{ for (int j = 0; j < n; j++) { x1[j].r = x2[j].r; x1[j].i = x2[j].i; } }

//-------------------------------------------------------------------------------------------------
	/**
	 * scale a complex vector
	 * @param x the complex vector to scale
	 * @param s the scaling factor
	 * @param n the number of points
	 */
	public static void complex_scale_vect(Complex[] x, double s, int n)
	{ for (int j = 0; j < n; j++) { x[j].r = x[j].r * s; x[j].i = x[j].i * s; } }

//-------------------------------------------------------------------------------------------------
	/**
	 * add two complex vectors
	 * @param x1 the result
	 * @param x2 the first vector
	 * @param x3 the second vector
	 * @param n number of complex points
	 */
	public static void complex_add_vect(Complex[] x1, Complex[] x2, Complex[] x3, int n)
	{ 
		for (int j = 0; j < n; j++) 
		{
			x1[j].r = x2[j].r + x3[j].r; 
			x1[j].i = x2[j].i + x3[j].i; 
		}
	}

//------------------------------------------------------------------------------------------------
	/**
	 * zero a complex vector (or portion)
	 * @param v the complex vector
	 * @param start starting index
	 * @param n the ending index
	 */
	public static void zero_vector(Complex[] v, int start, int n)
	{ for (int i = start; i < n; i++) { v[i].r = 0.0; v[i].i = 0.0; } }

//-------------------------------------------------------------------------------------------------
	/**
	 * returns true if two complex values are equal
	 * @param x1 the frist complex vector
	 * @param x2 the second complex vector
	 * @return
	 */
	public static boolean complex_equal(Complex x1, Complex x2)
	{ return ((x1.r == x2.r) && (x1.i == x2.i)); }
	
//-------------------------------------------------------------------------------------------------
}	// end class
 