/**
 * 
 */
package simulation;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

import spinworks.SW_main;
import spinworks.Util;

/**
 * @author marat
 *
 */
public class DCouplingEditor extends Dialog 
{
	protected Object result;
	protected Shell shell;
	
	private ScrolledComposite scrolledComposite;
	
	private static int n_couplings;
	private static int[] i_index = new int[SimPar.MAX_COUPLINGS];
	private static int[] j_index = new int[SimPar.MAX_COUPLINGS];
	
	static Label[] labelCoupling = new Label[SimPar.MAX_COUPLINGS];
	static Text[] textCoupling = new Text[SimPar.MAX_COUPLINGS];
	static Button[] buttonIterate = new Button[SimPar.MAX_COUPLINGS];
	

	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public DCouplingEditor(Shell parent, int style) 
	{
		super(parent, style);
		setText("Edit Dipolar (D) Couplings");
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() 
	{
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) 
		{
			if (!display.readAndDispatch()) 
			{
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() 
	{
		shell = new Shell(getParent(), getStyle());
		shell.setSize(565, 605);
		shell.setText(getText());
		
		Button buttonOK = new Button(shell, SWT.NONE);	
		buttonOK.setBounds(383, 531, 75, 25);
		buttonOK.setFont(SWTResourceManager.getBoldFont(buttonOK.getFont()));
		buttonOK.setBackground(shell.getDisplay().getSystemColor(SWT.COLOR_GREEN));
		buttonOK.setText("OK");
		
		Button buttonCancel = new Button(shell, SWT.NONE);
		buttonCancel.setBounds(464, 531, 75, 25);
		buttonCancel.setFont(SWTResourceManager.getBoldFont(buttonCancel.getFont()));
		buttonCancel.setBackground(shell.getDisplay().getSystemColor(SWT.COLOR_RED));
		buttonCancel.setText("Cancel");
		
		scrolledComposite = new ScrolledComposite(shell, 
				SWT.BORDER | SWT.V_SCROLL /*| SWT.H_SCROLL*/);
		scrolledComposite.setAlwaysShowScrollBars(true);
		scrolledComposite.setBounds(10, 10, 337, 546);
		
		Composite parent = createForm();
		scrolledComposite.setContent(parent);
			
		//	listeners
		
		buttonOK.addSelectionListener(new SelectionAdapter() // OK button
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				int k;

				for (k = 0; k < n_couplings; k++)
				{
					try
					{
						SW_main.sp.d_coupling.value[i_index[k]][j_index[k]] =
								Double.parseDouble(textCoupling[k].getText());
					}
					catch (Exception ex)
					{
						Util.show_and_log("Error converting: " + textCoupling[k].getText() + " to Double", 
								ex.getMessage());
					}
					SW_main.sp.d_coupling.iterate[i_index[k]][j_index[k]] = buttonIterate[k].getSelection();
				}

				String target;
				if (SW_main.gp.data_array == null || SW_main.q.data_folder.length() < 2)
					target = SW_main.gp.scratch_folder + "/spin_system";
				else
					target = SW_main.q.data_folder + "spin_system";

				if (SW_main.gp.log_on) Util.log_error("Edit D couplings: writing spin system to: " + target);	            
				SS_util.write_spins(target, SW_main.gp, SW_main.sp, SW_main.q);

				shell.dispose();
			}
		});
				
		buttonCancel.addSelectionListener(new SelectionAdapter() // Cancel button
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				shell.dispose();
			}
		});
		
	}	// end createContents()
	
//-------------------------------------------------------------------------------------------------	
	private Composite createForm()
	{
		Composite result = new Composite(scrolledComposite, SWT.NONE);
		result.setBounds(10, 10, 337, (SimPar.MAX_COUPLINGS + 1) * 25);

		for(int i = 0; i < SimPar.MAX_COUPLINGS; i++)
		{
			labelCoupling[i] = new Label(result, SWT.NONE);
			labelCoupling[i].setBounds(10, 20 + i * 25, 125, 15);

			textCoupling[i] = new Text(result, SWT.BORDER);
			textCoupling[i].setBounds(161, 14 + i * 25, 76, 21);

			buttonIterate[i] = new Button(result, SWT.CHECK);
			buttonIterate[i].setBounds(257, 21 + i * 25, 54, 16);
			buttonIterate[i].setText("iterate");
		}	

		int k = 0;

		for (int i = 0; i < SW_main.sp.n_shifts; i++)
		{
			for (int j = i; j < SW_main.sp.n_shifts; j++)
			{
				if ((j != i) || ((j == i) &&
					(SW_main.sp.shifts[i].type.equals("2") ||
					 SW_main.sp.shifts[i].type.equals("3"))))
				{
					labelCoupling[k].setText 
						(String.format("D(%d,%d) = D(%s,%s) = ", i + 1, j + 1,
								SW_main.sp.shifts[i].label, SW_main.sp.shifts[j].label));
						
                    textCoupling[k].setText(String.format("%.4f", 
                        SW_main.sp.d_coupling.value[i][j]));
                    
					buttonIterate[k].setSelection(SW_main.sp.d_coupling.iterate[i][j]);
					
					i_index[k] = i; j_index[k] = j;
					k++;
				}
			}	// for (int(int j...
		}	// for (int i...

		n_couplings = k;
			
		return result;
	}	// end createForm 
		
//-------------------------------------------------------------------------------------------------
}	// end class
