
package simulation;

/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.wb.swt.SWTResourceManager;

import spinworks.SW_main;
import spinworks.Util;

import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.custom.ScrolledComposite;

/**
 * @author marat
 *
 */
public class JCouplingEditor extends Dialog 
{	
	protected Object result;
	protected Shell shell;
		
	private ScrolledComposite scrolledComposite;
	
	private static int n_couplings;
	private static int[] i_index = new int[SimPar.MAX_COUPLINGS];
	private static int[] j_index = new int[SimPar.MAX_COUPLINGS];
	
	static Label[] labelCoupling = new Label[SimPar.MAX_COUPLINGS];
	static Text[] textCoupling = new Text[SimPar.MAX_COUPLINGS];
	static Button[] buttonIterate = new Button[SimPar.MAX_COUPLINGS];

	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public JCouplingEditor(Shell parent, int style) 
	{
		super(parent, style);
		setText("Edit Scalar (J) Couplings");
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() 
	{
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) 
		{
			if (!display.readAndDispatch()) 
			{
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() 
	{
		shell = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.RESIZE);
		shell.setSize(565, 605);
		shell.setText(getText());
		
		Button buttonOK = new Button(shell, SWT.NONE);	
		buttonOK.setBounds(383, 531, 75, 25);
		buttonOK.setFont(SWTResourceManager.getBoldFont(buttonOK.getFont()));
		buttonOK.setBackground(shell.getDisplay().getSystemColor(SWT.COLOR_GREEN));
		buttonOK.setText("OK");
		
		Button buttonCancel = new Button(shell, SWT.NONE);
		buttonCancel.setBounds(464, 531, 75, 25);
		buttonCancel.setFont(SWTResourceManager.getBoldFont(buttonCancel.getFont()));
		buttonCancel.setBackground(shell.getDisplay().getSystemColor(SWT.COLOR_RED));
		buttonCancel.setText("Cancel");
		
		scrolledComposite = new ScrolledComposite(shell, SWT.BORDER | SWT.V_SCROLL);
		scrolledComposite.setAlwaysShowScrollBars(true);
		scrolledComposite.setBounds(10, 10, 337, 546);
		
		Composite parent = createForm();
		scrolledComposite.setContent(parent);
				
		// listeners
		buttonOK.addSelectionListener(new SelectionAdapter() // OK button
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				int k;
							
				for (k = 0; k < n_couplings; k++)
				{
					try
					{
						SW_main.sp.j_coupling.value[i_index[k]][j_index[k]] =
	                        Double.parseDouble(textCoupling[k].getText());
					}
					catch (Exception ex)
					{
						Util.show_and_log("Error converting: " + textCoupling[k].getText() + " to Double", 
	                        ex.getMessage());
					}
					SW_main.sp.j_coupling.iterate[i_index[k]][j_index[k]] = buttonIterate[k].getSelection();
				}
				
				String target;
				if (SW_main.gp.data_array == null || SW_main.q.data_folder.length() < 2)
					target = SW_main.gp.scratch_folder + "/spin_system";
				else
					target = SW_main.q.data_folder + "spin_system";

	            if (SW_main.gp.log_on) Util.log_error("Edit J couplings: writing spin system to: " + target);	            
	            SS_util.write_spins(target, SW_main.gp, SW_main.sp, SW_main.q);
	            
				shell.dispose();
			}
		});
		
		buttonCancel.addSelectionListener(new SelectionAdapter() // Cancel button
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				shell.dispose();
			}
		});
	}	// end createContents(...

//-------------------------------------------------------------------------------------------------	
	private Composite createForm()
	{
		Composite result = new Composite(scrolledComposite, SWT.NONE);
		result.setBounds(10, 10, 337, (SimPar.MAX_COUPLINGS + 1) * 25);
		
		for(int i = 0; i < SimPar.MAX_COUPLINGS; i++)
		{
			labelCoupling[i] = new Label(result, SWT.NONE);
			labelCoupling[i].setBounds(10, 20 + i * 25, 125, 15);

			textCoupling[i] = new Text(result, SWT.BORDER);
			textCoupling[i].setBounds(161, 14 + i * 25, 76, 21);

			buttonIterate[i] = new Button(result, SWT.CHECK);
			buttonIterate[i].setBounds(257, 21 + i * 25, 54, 16);
			buttonIterate[i].setText("iterate");
		}	
				
		boolean dupflag;
		int k = 0;

		for (int i = 0; i < SW_main.sp.n_shifts; i++)
		{
			dupflag = true;
			if ((SW_main.sp.shifts[i].type.equals("2*1")
				|| SW_main.sp.shifts[i].type.equals("2*2")
				|| SW_main.sp.shifts[i].type.equals("2*3"))
				&& dupflag == true)
			{

				labelCoupling[k].setText( 
					String.format("J(%d,%d) = J(%s,%s) = ", i + 1, i + 1,
							SW_main.sp.shifts[i].label, SW_main.sp.shifts[i].label));
					
                textCoupling[k].setText(String.format("%.5f", 
                		SW_main.sp.j_coupling.value[i][i]));
                
				buttonIterate[k].setSelection(SW_main.sp.j_coupling.iterate[i][i]);

				SW_main.sp.j_coupling.index[i][i] = k;
				i_index[k] = i; j_index[k] = i;
				k++;
				dupflag = false;
			}

			for( int j = i + 1; j < SW_main.sp.n_shifts; j++)
			{
				if (SW_main.sp.calc_mode == SimPar.HIRES || (SW_main.sp.calc_mode == SimPar.DNMR &&
					SW_main.sp.shifts[i].species.equals(SW_main.sp.shifts[j].species)))
				{
					//	in DNMR mode only allow couplings within the species, as species really
					//	means chemical configuration in this case 

					labelCoupling[k].setText( 
						String.format("J(%d,%d) = J(%s,%s) = ", i + 1, j + 1,
						SW_main.sp.shifts[i].label, SW_main.sp.shifts[j].label));
					
                    textCoupling[k].setText(String.format("%5f",
                        SW_main.sp.j_coupling.value[i][j]));
                    
					buttonIterate[k].setSelection(SW_main.sp.j_coupling.iterate[i][j]);

					SW_main.sp.j_coupling.index[i][j] = k;
					i_index[k] = i; j_index[k] = j;
					k++;

					if((SW_main.sp.shifts[i].type.equals("2*1")
						|| SW_main.sp.shifts[i].type.equals("2*2")
						|| SW_main.sp.shifts[i].type.equals("2*3"))
						&&(SW_main.sp.shifts[j].type.equals("2*1")
						|| SW_main.sp.shifts[j].type.equals("2*2")
						|| SW_main.sp.shifts[j].type.equals("2*3")))						
					{

						labelCoupling[k].setText( 
							String.format("J(%d,%d)= J(%s,%s) = ", j + 1, i + 1,
									SW_main.sp.shifts[j].label, SW_main.sp.shifts[i].label));
						
                        textCoupling[k].setText(String.format("%.5f", 
                        		SW_main.sp.j_coupling.value[j][i]));
                        
						buttonIterate[k].setSelection(SW_main.sp.j_coupling.iterate[j][i]);

						SW_main.sp.j_coupling.index[j][i] = k;
						i_index[k] = j; j_index[k] = i;
						k++;
						dupflag = false;
					}
				}
			}	// end for (int j...
			n_couplings = k;
		}	// end for (int i...		
		return result;
	}	// end createForm 
//-------------------------------------------------------------------------------------------------	
}	// end class
