/**
 * 
 */
package simulation;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import spinworks.data_par;
import spinworks.global_par;
import spinworks.Util;
import spinworks.SW_main;

/**
 * @author marat
 *
 */
public class ReadSpins 
{
     private static int OPTIONS = 0;  // TODO change to enum
     private static int TITLE = 1;
     private static int SHIFTS = 2;
     private static int COUPLINGS = 3;
     private static int PARAMETERS = 4;
     
     
     
     public static int read(String fileName,
 			global_par gp, SimPar sp, data_par q)
 		{

	    	if(gp.log_on)
	    		 Util.log_error("Opening spin_system file:  " + fileName);
	    	
 			String instring;						// general temporary input string 
 			String tempstring = ""; ;

 			String delim = "[=\t ]+";
 			String delim2 = "[,]+";
 			

 			String species = "";
 			String index1_label;
 			String title;

 			int index1, index2;
 			int i, j, species_spin = 1;
 			
 			int status_flag;						// current read status
 			
 			 // try to open the parameter file 
 	        BufferedReader br;
 	        try { br = new BufferedReader(new FileReader(fileName)); }
 	        catch (Exception ex)
 	        {
 	            Util.show_and_log("ReadSpins.read(), cannot open: " + fileName, ex.getMessage());
 	            return -1;
 	        }
 	
 			// clear existing settings 	 is this necessary???
 			for(i=0; i < sp.n_shifts; i++)
 			{
 				
 				sp.shifts[i].type = "";
 				sp.shifts[i].freq = 0.0;
 				sp.shifts[i].spin = 1;		
 				sp.shifts[i].species = "";			
 				sp.shifts[i].label = "";
 				sp.shifts[i].iterate = false;

 				for(j = 0; j < sp.n_shifts; j++)
 				{
 					sp.j_coupling.value[i][j] = 0.0;
 					sp.j_coupling.iterate[i][j] = false;
 				}
 			}
 			
 			i = 0;
 			status_flag = OPTIONS;
 			sp.n_shifts = 0;

 			SS_util.init_sim(SimPar.MAX_SHIFTS, sp);	// should this be done once at prog start???

 			for (;;)
 			{
 				String[] token;
 				 try {instring = br.readLine();} 
 				 catch (IOException ex) 
 				 {
 					 ex.printStackTrace();
 					 Util.show_and_log("ReadSpins.read(), read error: " + fileName, ex.getMessage());
 					 try { br.close(); } 
 					 catch (IOException e) { e.printStackTrace(); }
 					 return -2;
 	     		 }
 	        	 
 	        	if(instring == null) break;
 	        	//System.out.println("instring(1) = " + instring);	///////////////////
 	        	//System.out.println(String.format("length = %d" , instring.length()));

 				//token = instring.Split(delim, StringSplitOptions.RemoveEmptyEntries);
 	        	token = instring.split(delim);
 	        	
 	        	/*System.out.println(String.format(" token length = %d", token.length));
 	        	System.out.print("   tokens = ");
 	        	for(int k = 0; k < token.length; k++)
 	        	{
 	        		System.out.print(token[k] + "  ");
 	        	}
 	        	System.out.println("");*/

 				// a single blank line separates various sections 
 				if (instring.length() == 0 && status_flag != SHIFTS)
 				{
 					//System.out.println("   increasing status flag");
 					status_flag++; // move to next section 
 					i = 0;
 				}

 				// two blank lines between shifts and couplings 
 				if (instring.length() == 0 && tempstring.length() == 0
 						&& status_flag == SHIFTS)
 				{
 					status_flag = COUPLINGS;
 					i = 0;
 				}

 				tempstring = instring;

 				if (status_flag == OPTIONS)							// simulation options 
 				{	
 					if(gp.log_on)
 			    		 Util.log_error("   Reading Option: " + instring);
 					
 					instring = instring.replace('#', ' ');
 					instring = instring.trim();
 					
 					token = instring.split(delim);
 					

 					if (token[0].length() != 0 && token.length == 2)
 					{
 						try
 						{
 							if (token[0].equals("optimize"))
 							{

 								if (token[1].equals("yes"))
 									sp.optmiz = 1;
 								else
 									sp.optmiz = 0;
 							}

 							else if (token[0].equals("autoassign"))
 							{
 								if (token[1].equals("yes"))
 									sp.autasn = 1;
 								else
 									sp.autasn = 0;
 							}

 							else if (token[0].equals("autoignore"))
 							{
 								if (token[1].equals("yes"))
 									sp.autdel = 1;
 								else
 									sp.autdel = 0;
 							}

 							else if (token[0].equals("linewidth"))
 							{
 								sp.group_linewidth = Double.parseDouble(token[1]);
 							}

 							else if (token[0].equals("iterations"))
 							{
 								sp.itrlim = Integer.parseInt(token[1]);
 							}

 							else if (token[0].equals("rmslimit"))
 							{
 								sp.rmslim = Double.parseDouble(token[1]);	
 							}

 							else if (token[0].equals("assignrms"))
 							{
 								sp.asnrms = Double.parseDouble(token[1]);
 							}

 							else if (token[0].equals("assignlimit"))
 							{
 								sp.asnlim = Double.parseDouble(token[1]);
 							}

 							else if (token[0].equals("ignorelimit"))
 							{
 								sp.ignlim = Double.parseDouble(token[1]);
 							}

 							else if (token[0].equals("rangefraction"))
 							{
 								sp.rngfra = Double.parseDouble(token[1]);
 							}

 							else if (token[0].equals("assignmod"))
 							{
 								sp.asnmod = Integer.parseInt(token[1]);
 							}
 						}
 						catch (Exception ex)
 						{
 							Util.show_and_log("conversion error spin system file", ex.getMessage());
 							try {br.close(); } 
 				 			catch (IOException e2) {	e2.printStackTrace();	}
 							return -1;
 						}
 					}
 				}

 				if (status_flag == TITLE)							// case title 
 				{	
 					if(gp.log_on)
			    		 Util.log_error("   Reading Title " + instring);
 					title = instring;

 					if (gp.data_array == null)
 					{
 						gp.plot_title = title;
 					}
 				}

 				if (status_flag == SHIFTS && token.length > 0 && instring.length() != 0)		// species and shifts
 				{
 					if(gp.log_on)
			    		 Util.log_error("   Reading Shifts: " + instring);
 							
 					char[] ch = token[0].toCharArray();	// TODO  check  only first two characters!!!!!

 					if (token[0].equals("species"))					// new species
 					{	
 						//System.out.println("   new species");
 						if (token.length > 1) species = token[1];

 						if (token.length > 2)
 						{
 							if (token[2].equals("spin"))
 							{
 								try { species_spin = Integer.parseInt(token[3]); }
 								catch (Exception ex)
 								{
 									Util.show_and_log("conversion error: spin system file:" 
 								          + token[3] , ex.getMessage());
 									try {br.close(); } 
 		 				 			catch (IOException e2) {	e2.printStackTrace();	}
 									return -1;
 								}
 							}
 							else
 								species_spin = 1;							// default to spin 1/2 
 						}
 					}

 					else if (is_digit(ch[0]) && tempstring.length() != 0)	// same species, 
 					{                                                   	// new shift
 						sp.shifts[i].type = token[0];
 						try
 						{
 							if (sp.shift_unit == SW_main.units.PPM)
 								sp.shifts[i].freq = Double.parseDouble(token[1]) / q.SFO1;
 							else
 								sp.shifts[i].freq = Double.parseDouble(token[1]);
 						}
 						catch (Exception ex)
 						{
 							Util.log_error("conversion error: spin system file: " + fileName,  
 									ex.getMessage());
 							try {br.close(); } 
 				 			catch (IOException e2) {	e2.printStackTrace();	}
 							return -1;
 						}

 						sp.shifts[i].species = species;
 						sp.shifts[i].spin = species_spin;
 						if (token[2].equals("name"))
 							sp.shifts[i].label = token[3];

 						i++;
 						sp.n_shifts++;
 					}
 				}

 				if (status_flag == COUPLINGS && tempstring.length() != 0)		// coupling  constants 
 				{
 					if(gp.log_on)
			    		 Util.log_error("   Reading Couplings: " + instring);
 					char[] ch = token[0].toCharArray();

 					if (is_digit(ch[0]))										// ## form UNTESTED
 					{
 						try
 						{
 							index1 = Integer.parseInt(String.valueOf(ch[0]));
 							index2 = Integer.parseInt(String.valueOf(ch[1]));

 							sp.j_coupling.value[index1 - 1][index2 - 1]
 									= Double.parseDouble(token[1]);
 						}
 						catch (Exception ex)
 						{
 							Util.show_and_log("conversion error: spin system file: " + fileName, 
 									ex.getMessage());
 							try {br.close(); } 
 				 			catch (IOException e2) {	e2.printStackTrace();	}
 							return -1;
 						}
 					}

 					//else if(token1[0] == 'j' && isdigit(token1[1])) /* j## form */
 					//{
 					//    // can't do this yet
 					//}

 					else if (ch[0] == 'j' && ch[1] == '(')			// j(?,?) form
 					{
 						ch[0] = ' '; ch[1] = ' ';	// necessary???????
 						ch[token[0].length() - 1] = ' ';

 						/*char[] trim_char = new char[] { 'j', '(' };
 						String temp_str = token[0].TrimStart(trim_char);*/

 						/*trim_char = new char[] { ')' };
 						temp_str = temp_str.TrimEnd(trim_char);*/
 						
 						String temp_str = token[0].replace('j', ' ');
 						temp_str = temp_str.replace('(', ' ');
 						temp_str = temp_str.replace(')', ' ');
 						temp_str = temp_str.trim();

 						String[] j_tokens = temp_str.split(delim2);

 						index1 = label_to_index(j_tokens[0], sp);
 						index2 = label_to_index(j_tokens[1], sp);

 						try
 						{
 							sp.j_coupling.value[index1][index2]
 									= Double.parseDouble(token[1]);
 						}
 						catch (Exception ex)
 						{
 							Util.show_and_log("conversion error: spin system file: " + fileName, ex.getMessage());
 							try {br.close(); } 
 				 			catch (IOException e2) {	e2.printStackTrace();	}
 							return -1;
 						}
 					}

 					else if (ch[0] == 'd' && ch[1] == '(')			// d(?,?) form
 					{
 						int temp;

 						ch[0] = ' '; ch[1] = ' ';	// necessary??
 						ch[token[0].length() - 1] = ' ';

 						/*char[] trim_char = new char[] { 'd', '(' };
 						String temp_str = token[0].TrimStart(trim_char);

 						trim_char = new char[] { ')' };
 						temp_str = temp_str.TrimEnd(trim_char);*/
 						
 						String temp_str = token[0].replace('d', ' ');
 						temp_str = temp_str.replace('(', ' ');
 						temp_str = temp_str.replace(')', ' ');
 						temp_str = temp_str.trim();

 						String[] d_tokens = temp_str.split(delim2);

 						index1 = label_to_index(d_tokens[0], sp);
 						index2 = label_to_index(d_tokens[1], sp);

 						/* swap indices if reversed  */
 						if (index1 > index2)
 						{
 							temp = index1;
 							index1 = index2;
 							index2 = temp;
 						}

 						try
 						{
 							sp.d_coupling.value[index1][index2]
 									= Double.parseDouble(token[1]);
 						}
 						catch (Exception ex)
 						{
 							Util.show_and_log( "conversion error: spin system file: " + fileName, ex.getMessage());
 							try {br.close(); } 
 				 			catch (IOException e2) {	e2.printStackTrace();	}
 							return -1;
 						}
 					}
 				}

 				if (status_flag == PARAMETERS && tempstring.length() != 0) // parameters to vary 
 				{
 					if(gp.log_on)
			    		 Util.log_error("   Reading Parameters: " + instring);

 					char[] ch = token[0].toCharArray();

 					if (is_digit(ch[0]) && token[0].length() == 1)		//  shift, # form
 					{
 						try { index1 = Integer.parseInt(token[0]); }
 						catch (Exception ex)
 						{
 							Util.show_and_log(
 									"conversion error: spin system file: " + fileName, ex.getMessage());
 							try {br.close(); } 
 				 			catch (IOException e2) {	e2.printStackTrace();	}
 							return -1;
 						}
 						sp.shifts[index1 - 1].iterate = true;
 					}

 					else if (is_digit(ch[0]) && token[0].length() == 2)	// coupling ## form 
 					{
 						try
 						{
 							index1 = Integer.parseInt(String.valueOf(ch[0]));
 							index2 = Integer.parseInt(String.valueOf(ch[1]));
 						}
 						catch (Exception ex)
 						{
 							Util.show_and_log("conversion error: spin system file: " + fileName, ex.getMessage());
 							try {br.close(); } 
 				 			catch (IOException e2) {	e2.printStackTrace();	}
 							return -1;
 						}
 						sp.j_coupling.iterate[index1 - 1][index2 - 1] = true;
 					}

 					else if (ch[0] == 'v' && ch[1] == '(')	// shift v(?) form 								
 					{
 						ch[0] = ' '; ch[1] = ' ';		// necessary???
 						ch[token[0].length() - 1] = ' ';

 						/*char[] trim_char = new char[] { 'v', '(' };
 						String temp_str = token[0].TrimStart(trim_char);

 						trim_char = new char[] { ')' };
 						temp_str = temp_str.TrimEnd(trim_char);*/
 						
 						String temp_str = token[0].replace('v', ' ');
 						temp_str = temp_str.replace('(', ' ');
 						temp_str = temp_str.replace(')', ' ');
 						temp_str = temp_str.trim();

 						index1_label = temp_str;

 						index1 = label_to_index(index1_label, sp);
 						sp.shifts[index1].iterate = true;
 					}

 					else if (ch[0] == 'j' && ch[1] == '(')	// coupling j(?,?) form 		        			
 					{
 						ch[0] = ' '; ch[1] = ' ';		// necessary???
 						ch[token[0].length() - 1] = ' ';

 						/*char[] trim_char = new char[] { 'j', '(' };
 						String temp_str = token[0].TrimStart(trim_char);

 						trim_char = new char[] { ')' };
 						temp_str = temp_str.TrimEnd(trim_char);*/
 						
 						String temp_str = token[0].replace('j', ' ');
 						temp_str = temp_str.replace('(', ' ');
 						temp_str = temp_str.replace(')', ' ');
 						temp_str = temp_str.trim();

 						String[] j_tokens = temp_str.split(delim2);

 						index1 = label_to_index(j_tokens[0], sp);
 						index2 = label_to_index(j_tokens[1], sp);

 						sp.j_coupling.iterate[index1][index2] = true;
 					}

 					else if (ch[0] == 'd' && ch[1] == '(')	// coupling d(?,?) form 													
 					{
 						int temp;

 						ch[0] = ' '; ch[1] = ' ';			// necessary????
 						ch[token[0].length() - 1] = ' ';

 						/*char[] trim_char = new char[] { 'd', '(' };
 						String temp_str = token[0].TrimStart(trim_char);

 						trim_char = new char[] { ')' };
 						temp_str = temp_str.TrimEnd(trim_char);*/
 						
 						String temp_str = token[0].replace('d', ' ');
 						temp_str = temp_str.replace('(', ' ');
 						temp_str = temp_str.replace(')', ' ');
 						temp_str = temp_str.trim();

 						String[] d_tokens = temp_str.split(delim2);

 						index1 = label_to_index(d_tokens[0], sp);
 						index2 = label_to_index(d_tokens[1], sp);

 						/* swap indices if reversed  */
 						if (index1 > index2)
 						{
 							temp = index1;
 							index1 = index2;
 							index2 = temp;
 						}

 						sp.d_coupling.iterate[index1][index2] = true;
 					}
 				}
 			}	// end for(;;)...

 			try {br.close(); } 
 			catch (IOException ex) {	ex.printStackTrace();	}

 			if(sp.calc_mode == SimPar.DNMR 
 				|| sp.calc_mode == SimPar.HIRES)	// check for any DNMR parameters appended to
 			{										// end of spin system file (JCAMP DX Format)				
 				try { br = new BufferedReader(new FileReader(fileName)); }
 	 	        catch (Exception ex)
 	 	        {
 	 	            Util.show_and_log("ReadSpins.read(), cannot open: " + fileName, ex.getMessage());
 	 	            return -1;
 	 	        }

 				for(;;)
 				{
 					String[] token;
 					try {instring = br.readLine();} 
 														
 	 				 catch (IOException ex) 
 	 				 {
 	 					 ex.printStackTrace();
 	 					 Util.show_and_log("ReadSpins.read(), read error: " + fileName, ex.getMessage());
 	 					 try { br.close(); } 
 	 					 catch (IOException e) { e.printStackTrace(); }
 	 					 return -2;
 	 	     		 }
 	 	        	 
 					if(gp.log_on)
			    		 Util.log_error("   Reading DNMR parameters: " + instring);
 					
 	 	        	if(instring == null) break;
 	 	        	
 					if (instring.length() > 3)
 					{
 						try
 						{
 							token = instring.split(delim);

 							if (token[0].equals("##$RC[1][2]"))
 								sp.rc[1][2] = Double.parseDouble(token[1]);

 							if (token[0].equals("##$RC[1][3]"))
 								sp.rc[1][3] = Double.parseDouble(token[1]);

 							if (token[0].equals("##$RC[2][3]"))
 								sp.rc[2][3] = Double.parseDouble(token[1]);

 							if (token[0].equals("##$T2"))
 								sp.t2 = Double.parseDouble(token[1]);

 							if (token[0].equals("##$MU"))
 							{
 								int k = Integer.parseInt(token[1]);
 								if (k == 1) sp.mu = true; else sp.mu = false;
 							}

 							if (token[0].equals("##$NCONFIG"))
 								sp.nconfig = Integer.parseInt(token[1]);

 							if (token[0].equals("##$PVEC2"))
 							{
 								//char[] ch = token[1].toCharArray();
 								sp.ie[2][1] = Integer.parseInt(token[1]);
 								sp.ie[2][2] = Integer.parseInt(token[2]);
 								sp.ie[2][3] = Integer.parseInt(token[3]);
 								sp.ie[2][4] = Integer.parseInt(token[4]);
 								sp.ie[2][5] = Integer.parseInt(token[5]);
 							}

 							if (token[0].equals("##$PVEC3"))
 							{
 								//char[] ch = token[1].toCharArray();
 								sp.ie[3][1] = Integer.parseInt(token[1]);
 								sp.ie[3][2] = Integer.parseInt(token[2]);
 								sp.ie[3][3] = Integer.parseInt(token[3]);
 								sp.ie[3][4] = Integer.parseInt(token[4]);
 								sp.ie[3][5] = Integer.parseInt(token[5]);
 							}

 							if (token[0].equals("##$POP[1]"))
 								sp.pop[1] = Double.parseDouble(token[1]);

 							if (token[0].equals("##$POP[2]"))
 								sp.pop[2] = Double.parseDouble(token[1]);

 							if (token[0].equals("##$POP[3]"))
 								sp.pop[3] = Double.parseDouble(token[1]);

                             if (token[0].equals("##$LWIDTH"))
 								sp.linewidth = Double.parseDouble(token[1]);
 						}
 						catch (Exception ex)
 						{
                             Util.show_and_log("Exception parsing DNMR parameters, instring = " + instring, ex.getMessage());
 						}
 					}
 				}
 				try {br.close(); } 
 	 			catch (IOException ex) {	ex.printStackTrace();	}
 			}

 			return 1;
 		}
     
     
     
     
//-------------------------------------------------------------------------------------------------
// 	this function turns takes a label string and returns the corresponding index number 

     private static int label_to_index(String label_string, SimPar sp)
     {
    	 int i, index = 0;

    	 for (i = 0; i < sp.n_shifts; i++)
    	 {
    		 if(label_string.equals(sp.shifts[i].label))
    		 {
    			 index = i;
    			 break;
    		 }
    	 }
    	 return index;
     }
//-------------------------------------------------------------------------------------------------
// 	Determine whether a string is a valid floating point number

    @SuppressWarnings("unused")
	private static boolean is_float(String s)
     {
		 double f;

    	 try { f = Double.parseDouble(s); }
    	 catch (Exception ex) { return false; }

    	 return true;
     }	
 
//-------------------------------------------------------------------------------------------------
// 	determine whether a character is a digit

     private static boolean is_digit(char c)
     {
    	 @SuppressWarnings("unused")
		 int i;

    	 try { i = Integer.parseInt(Character.toString(c)); }
    	 catch (Exception ex) { return false; }

    	 return true;
     }
   
//-------------------------------------------------------------------------------------------------
}//	end class
