/**
 * 
 */
package simulation;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;

import spinworks.SW_main;
import spinworks.SW_main.units;
import spinworks.data_par;
import spinworks.global_par;
import spinworks.Util;

/**
 * @author marat
 *
 */
public class SS_util 
{
	public final static double M_LN2 = 0.693147181;		// ln(2.0)

//	write a copy of the spin system information to disk.  It is rather
//	complicated in order to write it in proper NUMMRIT format 2016 meets 1970...

public static int write_spins(String fileName, global_par gp, SimPar sp, data_par q)
{
	int i, j;

	PrintWriter pw;

	//	open file
	try { pw = new PrintWriter(fileName); }
	catch (Exception ex)
	{
		Util.show_and_log("write_spins(): " + fileName, ex.getMessage());
		return -1;
	}


	pw.print("# ");							// initial record

	if (sp.optmiz == 1)
		pw.println("optimize=yes");
	else
		pw.println("optimize=no");

	if (sp.autasn == 1)
		pw.println("autoassign=yes");
	else
		pw.println("autoassign=no");

	if (sp.autdel == 1)
		pw.println("autoignore=yes");
	else
		pw.println("autoignore=no");

	pw.println(String.format("iterations=%d", sp.itrlim));
	pw.println(String.format("rmslimit=%.4f", sp.rmslim));
	pw.println(String.format("assignrms=%.4f", sp.asnrms));
	pw.println(String.format("assignlimit=%.4f", sp.asnlim));
	pw.println(String.format("ignorelimit=%.4f", sp.ignlim));
	pw.println(String.format("rangefraction=%.4f", sp.rngfra));
	pw.println(String.format("assignmod=%d", sp.asnmod));


	pw.println("");						// blank line 

	if (gp.plot_title.length() < 1)
		pw.println("NUMMRIT spin system file generated by SpinWorks Java");
	else
	{          
		String temp = gp.plot_title.replace('\n',' ');
		temp = temp.replace('\r', ' ');

		/* if(temp.length() >= 79) 
                temp = temp.e(79); */        // TODO keep string to a reasonable length

		pw.println(temp);
	}

	pw.println("");						 // blank line 

	for(i=0; i < sp.n_shifts; i++)	     // output the shifts 
	{
		if(i == 0)
		{
			if(sp.shifts[i].spin == 1)
				pw.println(String.format("species=%s", sp.shifts[i].species));
			else
				pw.println(String.format("species=%s   spin=%d", sp.shifts[i].species, 
						sp.shifts[i].spin));
		}
		else if(i != 0)
		{
			if(!sp.shifts[i].species.equals(sp.shifts[i-1].species) 
					&& sp.shifts[i].type.length() > 0)
			{
				if(sp.shifts[i].spin == 1)
				{
					pw.println("");
					pw.println(String.format("species=%s", sp.shifts[i].species));
				}
				else
				{
					pw.println("");
					pw.println(String.format("species=%s  spin=%d", sp.shifts[i].species, 
							sp.shifts[i].spin));
				}
			}
		}

		if(sp.shifts[i].type.length() > 0)
		{
			if(sp.shift_unit == units.PPM)
				pw.println(String.format("%s  %.4f  name=%s", 
						sp.shifts[i].type, sp.shifts[i].freq * q.SFO1, sp.shifts[i].label));
			else    // must be Hz
				pw.println(String.format("%s  %.4f  name=%s",
						sp.shifts[i].type, sp.shifts[i].freq, sp.shifts[i].label));
		}
	}		// end for i = 0...

	pw.println("");							// 2 blank lines -after last species 
	pw.println("");

	for(i=0; i < sp.n_shifts; i++)		    // scalar couplings
	{
		for(j=0; j < sp.n_shifts; j++)
		{
			if((sp.j_coupling.value[i][j] > 0.00001) 
					|| (sp.j_coupling.value[i][j] < -0.00001))
			{
				if(sp.shifts[i].label.length() == 0 || sp.shifts[j].label.length() == 0)
					pw.println(String.format("j%d%d  %.5f", 
							i + 1, j + 1, sp.j_coupling.value[i][j]));
				else
					pw.println(String.format( 
							"j(%s,%s)  %.5f", sp.shifts[i].label, 
							sp.shifts[j].label, sp.j_coupling.value[i][j]));
			}
		}
	}

	for(i=0; i < sp.n_shifts; i++)		     // dipolar couplings
	{
		for(j=i; j < sp.n_shifts; j++)
		{
			if((sp.d_coupling.value[i][j] > 0.00001) || (sp.d_coupling.value[i][j] < -0.00001))
			{
				if(sp.shifts[i].label.length() == 0 || sp.shifts[j].label.length() == 0)
					pw.println(String.format( 
							"d%d%d  %.5f", i+1, j+1, sp.d_coupling.value[i][j]));
				else
					pw.println(String.format("d(%s,%s)  %.5f", 
							sp.shifts[i].label, sp.shifts[j].label,
							sp.d_coupling.value[i][j]));
			}
		}
	}

	pw.println("");								// blank line -terminates couplings 

	for(i=0; i < sp.n_shifts; i++)			    // now do parameters to be varied 
	{
		if(sp.shifts[i].iterate == true && sp.shifts[i].type.length() != 0)
		{
			pw.println(String.format("v(%s)", sp.shifts[i].label));
		}
	}

	for(i=0; i < sp.n_shifts; i++)			   // scalar couplings
	{ 
		for(j=0; j < sp.n_shifts; j++)
		{
			if(sp.j_coupling.iterate[i][j] == true) 
			{
				if(sp.shifts[i].label.length() == 0 || sp.shifts[j].label.length() == 0)
					pw.println(String.format("j%d%d", i+1, j+1));
				else
					pw.println(String.format("j(%s,%s)", sp.shifts[i].label, sp.shifts[j].label));
			}
		}
	} 

	for(i=0; i < sp.n_shifts; i++)			    // dipolar couplings
	{
		for(j=i; j < sp.n_shifts; j++)
		{
			if (sp.d_coupling.iterate[i][j] == true)
			{
				if (sp.shifts[i].label.length() == 0 || sp.shifts[j].label.length() == 0)
					pw.println(String.format("d%d%d", i + 1, j + 1));
				else
					pw.println(String.format("d(%s,%s)", sp.shifts[i].label, sp.shifts[j].label));
			}
		}
	}

	// add additional DNMR/MEXICO parameters in JCAMP DX format
	if(sp.calc_mode == SimPar.DNMR ||sp.calc_mode == SimPar.HIRES)	
	{			
		pw.println("");						// blank line separator
		pw.println(String.format("##$T2= %.5f", sp.t2));
		pw.println(String.format("##$RC[1][2]= %.4f", sp.rc[1][2]));
		pw.println(String.format("##$RC[1][3]= %.4f", sp.rc[1][3]));
		pw.println(String.format("##$RC[2][3]= %.4f", sp.rc[2][3]));

		if (sp.mu)
			pw.println("##$MU= 1");
		else
			pw.println("##$MU = 0");

		pw.println(String.format("##$NCONFIG= %d", sp.nconfig));
		pw.println(String.format("##$PVEC2= %d %d %d %d %d",
				sp.ie[2][1], sp.ie[2][2], sp.ie[2][3], sp.ie[2][4], sp.ie[2][5]));

		if(sp.nconfig == 3)					// second permutation vector if 3 config
			pw.println(String.format("##$PVEC3= %d %d %d %d %d",
					sp.ie[3][1], sp.ie[3][2], sp.ie[3][3], sp.ie[3][4], sp.ie[3][5]));

		if(sp.mu == false)					// populations for non-mutual exchange
		{
			pw.println(String.format("##$POP[1]= %.4f", sp.pop[1]));
			pw.println(String.format("##$POP[2]= %.4f", sp.pop[2]));
			pw.println(String.format("##$POP[3]= %.4f", sp.pop[3]));
		}

		pw.println(String.format("##$LWIDTH= %.4f", 
				sp.linewidth));   // simulation linewidth

		//	add others as necessary
	}

	pw.println("");						// blank line -terminates file
	pw.close();

	//	TODO if we are in DNMR mode, then write the appropiate files for MEXICO or DNMR3

	/*if(sp.calc_mode == SimPar.DNMR)
		{
			String mex_file;
			String dnmr_file;

			if (gp.data_array == null)
			{
				mex_file = String.Format("{0}{1}mexico.mch", 
                    gp.scratch_folder, SW_main.sep);	        // for MEXICO
                dnmr_file = String.Format("{0}{1}dnmr.mch", 
                    gp.scratch_folder, SW_main.sep);	        // for DNMR3
			}
			else
			{
                mex_file = String.Format("{0}{1}mexico.mch", 
                    q.data_folder, SW_main.sep);	            // for MEXICO
                dnmr_file = String.Format("{0}{1}dnmr.mch", 
                    q.data_folder, SW_main.sep);	            // for DNMR3
			}

			extract_dnmr_pars(gp, sp);
			mex_out(mex_file, gp, sp, q);
			dnmr_out(dnmr_file, gp, sp, q);
		}*/

	return 1;
}


//-------------------------------------------------------------------------------------------------
// get transitions (NUMMRIT output)

	public static int getTrans(String fileName, global_par gp, SimPar sp, data_par p, data_par q)
	{
		int i, j;
	
		double	temp_calc_freq, temp_obs_freq, temp_intensity,
		temp_rel_width;
		int temp_trans_no;
	
		String instring;
		//char[] delim = new char[] { ' ', '=' };
		String delim = "[=\t ]+";
	
		// allocate some memory for calculated spectrum 
		Util.alloc_calc(gp, p, q);
	
		sp.n_trans = 0;
	
		BufferedReader br;
		try { br = new BufferedReader(new FileReader(fileName)); }
		catch (Exception ex)
		{
			Util.show_and_log("getTrans(), cannot open: " + fileName, ex.getMessage());
			return -1;
		}
	
		i=0;
		for(;;)
		{
			String[] token;
			 try {instring = br.readLine();} 
			 catch (IOException ex) 
			 {
				 ex.printStackTrace();
				 Util.show_and_log("getTrans.read(), read error: " + fileName, ex.getMessage());
				 try { br.close(); } 
				 catch (IOException e) { e.printStackTrace(); }
				 return -2;
     		 }
        	 
        	if(instring == null) break;
        	instring = instring.trim();
			token = instring.split(delim);	
			if(token[0].equals("$$end")) break;
			gp.calc_peaks[i] = new Transition();
	
			try { gp.calc_peaks[i].calc_freq = Double.parseDouble(token[0]); }
			catch (Exception ex)
			{   
				Util.show_and_log("get_trans() bad conversion (freq)", ex.getMessage()); 
				try { br.close(); } 
				catch (IOException e) { e.printStackTrace(); }
				return -1;    
			}
	
			try { gp.calc_peaks[i].intensity = Double.parseDouble(token[1]); }
			catch (Exception ex)
			{ 
				Util.show_and_log("get_trans() bad conversion (intensity)", ex.getMessage()); 
				try { br.close(); } 
				catch (IOException e) { e.printStackTrace(); }
				return -1; 
			}
	
			try { gp.calc_peaks[i].rel_width = Double.parseDouble(token[2]); }
			catch (Exception ex)
			{ 
				Util.show_and_log("get_trans() bad conversion (width)", ex.getMessage()); 
				try { br.close(); } 
				catch (IOException e) { e.printStackTrace(); }
				return -1; 
			}
	
			try { gp.calc_peaks[i].trans_no = Integer.parseInt(token[3]); }
			catch (Exception ex)
			{ 
				Util.show_and_log("get_trans() bad conversion (transition #)", ex.getMessage()); 
				try { br.close(); } 
				catch (IOException e) { e.printStackTrace(); }
				return -1; 
			}
	
			i++;
	
		}  // end while 
	
		sp.n_trans = i; // record the number of found transitions 
	
		try { br.close(); } 
		catch (IOException e) { e.printStackTrace(); }
	
		// now sort transitions using simple bubble sort 
		for(i=0; i < sp.n_trans; i++)
		{
			for(j=i; j < sp.n_trans; j++)
			{
				if(gp.calc_peaks[i].calc_freq > gp.calc_peaks[j].calc_freq)
				{
					temp_calc_freq = gp.calc_peaks[i].calc_freq;
					temp_obs_freq = gp.calc_peaks[i].obs_freq;
					temp_intensity = gp.calc_peaks[i].intensity;
					temp_rel_width = gp.calc_peaks[i].rel_width;
					temp_trans_no = gp.calc_peaks[i].trans_no;
	
					gp.calc_peaks[i].calc_freq = gp.calc_peaks[j].calc_freq;
					gp.calc_peaks[i].obs_freq = gp.calc_peaks[j].obs_freq;
					gp.calc_peaks[i].intensity = gp.calc_peaks[j].intensity;
					gp.calc_peaks[i].rel_width = gp.calc_peaks[j].rel_width;
					gp.calc_peaks[i].trans_no = gp.calc_peaks[j].trans_no;
	
					gp.calc_peaks[j].calc_freq = temp_calc_freq;
					gp.calc_peaks[j].obs_freq = temp_obs_freq;
					gp.calc_peaks[j].intensity = temp_intensity;
					gp.calc_peaks[j].rel_width = temp_rel_width;
					gp.calc_peaks[j].trans_no = temp_trans_no;
				}
			}
		}
	
		// now position the transition cursor within the spectral width 
	
		for (gp.trans_index = 0;
				((gp.calc_peaks[gp.trans_index].calc_freq >= q.offset)
						|| (gp.calc_peaks[gp.trans_index].calc_freq <= q.offset - p.SW * p.SF))
				&& gp.trans_index < sp.n_trans-1;
				gp.trans_index++); 
	
		if(sp.n_trans > 0)	 // update the status shifts 
		{
			for(i=0; i < SimPar.MAX_SHIFTS; i++)
			{
				sp.status_shifts[i].freq = sp.shifts[i].freq;
				sp.status_shifts[i].label = sp.shifts[i].label;
			}
	
			return sp.n_trans;
		}
		else
		{
			//util.free_calc(gp);
			return -1;
		}
	}

//-------------------------------------------------------------------------------------------------
//	Extract shifts and couplings from NUMMRIT spin system for DNMR3

//	Note on array indices:  Since DNMR and NUMMRIT were originally 
//	FORTRAN (barf) programs.  Some of the arrays in SpinWorks run
//	from 1 to n rather than in the C fashion of 0 to n-1.  This made
//	for MUCH easier translation.  Many of the larger arrays are dynamically
//	dimensioned as 1 to n in the fashion suggested "Numerical Recipies in C".
//	Some of the smaller ones are, for now, dimensioned 1 larger than needed
//	with the 0th element(s) unused.

//	The absolutely awful variable names are those used in the original DNMR3
//	program.

		/*public static void extract_dnmr_pars(global_par gp, SimPar sp)
		{
			int i,j,k;						// handy counters 

			sp.nspins = 0;
			sp.nmes = 0;

			for(i=0; i < sp.n_shifts; i++)		// extract number of spins and spins per group 
			{
				try { sp.nnmes[i + 1] = Convert.ToInt32(sp.shifts[i].type); }
				catch (Exception ex)
				{
                    util.show_and_log("extract_dnmr_pars(): conversion error! check your parameters ",
                        ex.Message);			
				}

				sp.nspins = sp.nspins + sp.nnmes[i + 1];
				sp.nmes++;

				if(!Equals(sp.shifts[i].species, sp.shifts[i + 1].species)) break;
			}

			i = 0; j = 0;  

			for(k=0; k < sp.n_shifts; k++)		// now extract the shifts 
			{
				sp.w[i+1, j+1] = sp.shifts[k].freq;

				if(Equals(sp.shifts[k].species, sp.shifts[k+1].species))
				{
					j++;
				}
				else
				{
					i++;  j=0;
				}
			}
		   
			if(sp.mu == false)
				sp.nconfig = i;
		 
			for(i=0; i < sp.nconfig; i++)
			{
				for(j=0; j < sp.nmes; j++)
				{
					for(k=0; k < sp.nmes; k++)
					{
						sp.aj[i+1, j+1, k+1] = 
							sp.j_coupling.value[j+(i*sp.nmes), k+(i*sp.nmes)];

					}			
				}
			}

			sp.ne = sp.nconfig;	// dupe variable ??
			sp.nen = sp.ne;

			if(sp.mu == true) sp.nen = 1;

			sp.nmesm = sp.nmes - 1;

		}*/

//-------------------------------------------------------------------------------------------------
// write out the Mexico mechanism file

		/*public static int mex_out(String file_name, global_par gp, SimPar sp, data_par q)
		{

			double k_equil;
			String spin_system;
			int i, j, k;				// handy counters

			StreamWriter sw;

			try
			{
				sw = new StreamWriter(file_name);
			}
			catch (Exception ex) 
			{
                util.show_and_log("mex_out(): " + file_name, ex.Message);
				return -1;
			}

			if(sp.mu == true) 
				k_equil = 1.0;
			else if(sp.pop[1] == 0.0 || sp.pop[2] == 0.0)
			{
                util.show_and_log("for non-mutual exchange, site populations cannot be 0");
				return -1;
			}
			else
				k_equil = sp.pop[2]/sp.pop[1];

			if(sp.nmes == 1) spin_system = "a";
			else if (sp.nmes == 2) spin_system = "ab";
			else if (sp.nmes == 3) spin_system = "abc";
			else if (sp.nmes == 4) spin_system = "abcd";
			else if (sp.nmes == 5) spin_system = "abcde";
			else spin_system = "invalid";

		//	1st line
			if (gp.plot_title.Length == 0)
				sw.WriteLine("Mexico mechanism file generated by SpinWorks 4");
			else
				sw.WriteLine("{0}", gp.plot_title);

		//  now some parameters in JCAMP-DX format
			sw.WriteLine("##$SI= {0}", q.SI);
			sw.WriteLine(String.Format(SW_main.num_format, "##$SW_p= {0:F4}", q.SW * q.SFO1));
			sw.WriteLine(String.Format(SW_main.num_format, "##$SF= {0:F7}", q.SF));
			sw.WriteLine(String.Format(SW_main.num_format, "##$OFFSET= {0:F6}", q.offset / q.SF));

		//	2nd line
			sw.WriteLine("#XwinNMR path here instead if using with XwinNMR");

			sw.WriteLine("{0}", gp.scratch_folder);					// for now  trailing \ needed??

		//	set up unity permutation vector for first site
			for(i = 1; i <= sp.nmes; i++)
				sp.ie[1, i] = i;

		//	produce duplicate second-site spin system for cases of
		//	mutual exchange

			if(sp.mu == true)
			{
				for(i = 1; i <= sp.nmes; i++)
				{
					sp.w[2, i] = sp.w[1, i];
					for(j = 1; j <= sp.nmes; j++)
					{
						sp.aj[2, i, j] = sp.aj[1, i, j];
					}
				}
			}

		//	3rd line
			sw.WriteLine(String.Format(SW_main.num_format, "#\n{0} {1} {2:F5}", sp.ne, sp.ne-1, k_equil));

			for(i = 1; i <= sp.ne; i++)						// repeat for each site (chem. config)
			{
		//  4th line
				sw.WriteLine("#site {0}: spin system\n{1}", i, spin_system);


		//	5th line	permutation vector
				sw.WriteLine("#permutation # {0}", i);
				for(j = 1; j <= sp.nmes; j++)
					sw.Write("{0} ", sp.ie[i, j] - 1);
				sw.WriteLine("");

		//	6th line	relaxation rates
				sw.WriteLine("#relaxation rates");
				for(j = 1; j <= sp.nmes; j++)
					sw.Write(String.Format(SW_main.num_format, "{0:F5} ", 1.0 / sp.t2));
				sw.WriteLine("\n#shifts and couplings");

		//	7th line spectral parameters
				for(j = 1; j <= sp.nmes; j++)
				{
					if(j > 1)
					{
						for(k = 1; k < j; k++)
						{
							sw.Write(String.Format(SW_main.num_format, "{0:F6} ", sp.aj[i, k, j]));
						}
					}
					sw.WriteLine(String.Format(SW_main.num_format, " {0:F5} ", sp.w[i, j]));
				}
			}

		//	last line
			sw.WriteLine(String.Format(SW_main.num_format, "#rate constant\n{0:F4}", sp.rc[1, 2]));

			sw.Close();
			return 1;
		}*/

//-------------------------------------------------------------------------------------------------
//	Write out the DNMR3 mechanism file

		/*public static int dnmr_out(String file_name, global_par gp, SimPar sp, data_par q)
		{
			int i, j ,k, n;						// handy counters 

			StreamWriter sw;

			try
			{
				sw = new StreamWriter(file_name);
			}
			catch (Exception ex) 
			{
                util.show_and_log("dnmr_out(): " + file_name, ex.Message);
				return -1;
			}
	     
			 record 1 
//	nrun n ne mu nmes nsym   <- nrun and nsym are fixed

			int mu_int;	// get int equivalent of bool mu value
			if (sp.mu) mu_int = 1;
			else mu_int = 0;

			sw.WriteLine(String.Format(SW_main.num_format, "0001 {0} {1} {2} {3} 0", sp.nspins, sp.nconfig, mu_int, sp.nmes));
		    
			 record 2 
//	the number of nuclei in each set of chemically equivalent nuclei
//	not used if each group consists of only one spin
			if(sp.nmes != sp.nspins)
			{
				for(i=1; i <= sp.nmes; i++)
				{
					sw.Write("{0} ", sp.nnmes[i]);
				}
				sw.WriteLine("");
			}
		     
			  record 3 
//	just some text
			if (gp.plot_title.Length == 0)
				sw.WriteLine(String.Format(SW_main.num_format, "DNMR3 input file generated by SpinWorks 4"));
			else
				sw.WriteLine(String.Format(SW_main.num_format, "{0}", gp.plot_title));
		   
			 records 4 and 5, repeated for each chemical config.  
//	The shifts and couplings - only one set for mutual exchange
		    
			if(sp.mu == true)
			   n = 1;
			else 
			   n = sp.nconfig;
		       
			for(i=1; i <= n; i++)
			{
				for(j=1; j <= sp.nmes; j++)							 shifts 
				{
				  sw.Write(String.Format(SW_main.num_format, "{0:F5} ", sp.w[i, j]));
				}
				sw.WriteLine("");
		          
				for(j=1; j <= sp.nmes; j++)							 couplings 
				{
					for(k=(j+1); k <= sp.nmes; k++)
					{
						sw.Write(String.Format(SW_main.num_format, "{0:F5} ", sp.aj[i, j, k]));
					}
					if(j <= (sp.nmes-1))
						sw.WriteLine("");
				}
			}														// for i  
		       
			 record 6 
//	The permutation vectors for mutual exchange
			if(sp.mu == true)
			{
				for(i = 2; i <= sp.nconfig; i++)
				{
					for(j = 1; j <= sp.nmes; j++)
						sw.Write("{0} ", sp.ie[i, j]);
					sw.WriteLine("");
				}
			}
		       
			 record 7 
//	parameters for generating the output file to be used by SpinWorks
			sw.WriteLine(String.Format(SW_main.num_format, "01 01 {0:F3}  {1:F3}  {2} ", q.offset, q.SFO1*q.SW, q.SI));
		    
			 record 8, populations, only for non-mutual exchange 
			if(sp.mu == false)
			{
				for(i=1; i <= sp.nconfig; i++)
				{
				  sw.Write(String.Format(SW_main.num_format, "{0:F4} ", sp.pop[i]));
				}
				sw.WriteLine("");
			}
		       
			 record 9, relaxation time  
//	This gives the effective linewidth for all nuclei in all configurations
//	Use MEXICO if you want a more realistic treatment of relaxation.
			sw.WriteLine(String.Format(SW_main.num_format, "{0:F5} ", sp.t2));
		   
			 record 10, rate constants 
//	for mutual exchange there can only be one rate constant even if there 
//	are more than two chemical configurations
			if(sp.mu == true)
			{
				sw.WriteLine(String.Format(SW_main.num_format, "{0:F5} ", sp.rc[1, 2]));
			}
			else
			{
				for(i=1; i <= sp.nconfig; i++)
				{
					for(j=(i+1); j <= sp.nconfig; j++)
					{
						sw.WriteLine(String.Format(SW_main.num_format, "{0:F5} ", sp.rc[i, j])); 
					}
				} 
			}
		  
			sw.Close();
			return 1;
		}*/

//-------------------------------------------------------------------------------------------------
//	get an array of simulated x,y data points from an external prog 
//	returns number of points read, -1 if file could not be opened 
//	usually used to get the DNMR3 output file

		/*public static int get_sim(String FileName, global_par gp, SimPar sp, data_par p, data_par q)
		{
			double a = 0.0, b = 0.0;
			int n_points = 0;
			int i;

			String instring;
			char[] delim = new char[] { ' ', '=' };
			String[] token;

// In some countries a "," is used as the decimal separator.  Override this,
// otherwise parameter sets, etc with decimal numbers can't be read without the user having
// to change their machine settings
			NumberFormatInfo format = new NumberFormatInfo();
			format.NumberDecimalSeparator = ".";

			n_points = 0;

			StreamReader sr;
			try { sr = new StreamReader(FileName); }
			catch (Exception ex)
			{
				util.show_and_log("get_sim(): cannot open simulation output file: " + FileName, 
                    ex.Message);
				return -1;
			}

//	count the number of data points  (is there some better way to do this???)
			i=0;
			while ((instring = sr.ReadLine()) != null)
			{
				i++;
			}
			sr.Dispose();

//	free any existing allocated memory for calculated spectrum array 
			util.free_calc(gp);							// not really necessary in .NET


//	allocate some memory for calculated spectrum 
			if(gp.data_array != null) 
				q.SI = i;								// set default size, no expt data
			else
			{
				if(i != q.SI)
					util.show_and_log("Warning: number of simulated points\n differs from experimental data");			
			}
			util.alloc_calc(gp, p, q);

			try { sr = new StreamReader(FileName); }	// re-open input string
			catch (Exception ex)
			{
				util.show_and_log("get_sim(): cannot open simulation output file: " + FileName, 
                    ex.Message);
				return -1;
			}

			i=0;
			while (i < q.SI && (instring = sr.ReadLine()) != null)
			{
				token = instring.Split(delim, StringSplitOptions.RemoveEmptyEntries);

				try { a = Convert.ToDouble(token[0], format); }
				catch (Exception) { }

				try { b = Convert.ToDouble(token[1], format); }
				catch (Exception) { }

				if (token[0] != null && token[1] != null)
				{
					if(i==0) sp.offset = a;
					gp.calc_array[i]=(int)(b * SimPar.DNMR_SCALE);
					i++;
				}
			}

			n_points = i;					// record the number of found data points 
			sp.swidth = sp.offset - a;

			sr.Dispose();

			if(n_points > 0)				// update the status shifts and availability
			{
				for(i=0; i < SimPar.MAX_SHIFTS; i++)
				{
					sp.status_shifts[i].freq = sp.shifts[i].freq;
					sp.status_shifts[i].label = sp.shifts[i].label;
				}	
			}  
			return n_points;
		}*/

//-------------------------------------------------------------------------------------------------
//	Read Mexico output file (or output from any other routine that outputs a spectrum
//	file in the same format as Bruker's 1r file)
//	TO DO:  array should be passed in argument list

		/*public static int get_1r(String file_name, global_par gp, data_par p, data_par q)
		{		
			int bytes_read = 0;

			FileStream fs;
			try { fs = new FileStream(file_name, FileMode.Open, FileAccess.Read); }
			catch (Exception ex)
			{
				util.show_and_log( ex.Message + ":  " + file_name);
				return -1;
			}

			BinaryReader br = new BinaryReader(fs);

			util.free_calc(gp);				// not really necessary in .NET
			util.alloc_calc(gp, p, q);

			for (int i = 0; i < q.SI; i++)
			{
				try
				{
					gp.calc_array[i] = br.ReadInt32();
					bytes_read = bytes_read + 4;
				}
				catch (Exception ex)
				{
                    util.show_and_log("Error reading Mexico output file: " + file_name,
                        ex.Message);
					break;
				}
			}

			fs.Close();
			br.Close();
			return bytes_read;
		}*/

//-------------------------------------------------------------------------------------------------
//	write an assigned transition file to disk.  Returns total number of lines written
//	-1 if an error

		public static int putTrans(String fileName, global_par gp, SimPar sp)
		{
			int n = 0;
			PrintWriter pw;

			//	open file
			try { pw = new PrintWriter(fileName); }
			catch (Exception ex)
			{
				Util.show_and_log("putTrans(): " + fileName, ex.getMessage());
				return -1;
			}
		 
			int i = 0;

			try
			{
				for (i = 0; i < sp.n_trans; i++)
				{
					if ((gp.calc_peaks[i].obs_freq > 0.0001 
                        || gp.calc_peaks[i].obs_freq < (-0.0001))
						&& gp.calc_peaks[i].trans_no != 0)
						pw.println(String.format("%.5f   line=%d \n", gp.calc_peaks[i].obs_freq,
							gp.calc_peaks[i].trans_no));
					n++;
				}

				for (i = 0; i < gp.n_peaks; i++)
				{
					pw.println(String.format("%.5f", gp.peaks[i].freq));
					n++;
				}

				pw.println("");
			}
			catch (Exception ex)
			{
                Util.show_and_log("putTrans has raised the following exception: ", ex.getMessage());
			}

			pw.close();
			return n;
		}

//-------------------------------------------------------------------------------------------------
//get any assigned transitions

		public static int getAssignedTransitionFile(String fileName, global_par gp, SimPar sp)
		{
			int j, k = 0, n;
			
			double a;

			String instring;
			String delim = "[=\t ]+";
			String[] token;

			BufferedReader br;
			try { br = new BufferedReader(new FileReader(fileName)); }
			catch (Exception ex)
			{
				Util.log_error("getAssignedTransitionFile(), cannot open: " 
			         + fileName + ",  maybe no transitions yet assigned ", ex.getMessage());
				return -1;
			}

			for(;;)
			{
				a = 0.0;	n = 0;
				try {instring = br.readLine();} 
				 catch (IOException ex) 
				 {
					 ex.printStackTrace();
					 Util.show_and_log("getAssignedTransitionFile(), read error: " + fileName, ex.getMessage());
					 try { br.close(); } 
					 catch (IOException e) { e.printStackTrace(); }
					 return -2;
	     		 }
	        	 
	        	if(instring == null) break;
	        	instring = instring.trim();
	        	
				token = instring.split(delim);
				
				if(instring.length() > 3  && token.length == 3)
				{
					try { a = Double.parseDouble(token[0]); }
					catch (Exception ex)
                    { Util.show_and_log("get_at_file(): " + instring, ex.getMessage());   }

					try { n = Integer.parseInt(token[2]); }
					catch (Exception ex)
                    { Util.show_and_log("get_at_file(): " + instring, ex.getMessage());   }

					if (a != 0.0 && n != 0)
					{
						for (j = 0; j < sp.n_trans; j++)
						{
							if (gp.calc_peaks[j].trans_no == n)
							{
								gp.calc_peaks[j].obs_freq = a;
								k++;
							}
						}
					}
				}
			 }  // end for(;;) 

			try {br.close();} 
			catch (IOException ex) {  ex.printStackTrace(); }	// close the assigned transition file 

			return k;
		}

//-------------------------------------------------------------------------------------------------
//	initialize spin system objects, for a system of up to n groups

	public static void init_sim(int n, SimPar sp)
	{
		n = 2 * n;      // because of multiple config in DNMR3
		sp.shifts = new ShiftInfo[n];
		sp.status_shifts = new ShiftInfo[n];

		for (int i = 0; i < n; i++)
		{
			sp.shifts[i] = new ShiftInfo();
			sp.status_shifts[i] = new ShiftInfo();
		}

		sp.j_coupling = new Coupling(n);
		sp.d_coupling = new Coupling(n);
	}

//-------------------------------------------------------------------------------------------------
//	load any optimized parameters appended to the end of the transition file
//	returns number of parameters found

		public static int getOpt(String fileName, data_par q, SimPar sp)
		{
			int i, j, count = 0;

			String instring;
			String delim = "[=\t ]+";
			String[] token;

			BufferedReader br;
			try { br = new BufferedReader(new FileReader(fileName)); }
			catch (Exception ex)
			{
				Util.show_and_log("getOpt(), cannot open: " + fileName, ex.getMessage());
				return -1;
			}

			for(;;)
			{
				try {instring = br.readLine();} 
				catch (IOException ex) 
				{
					ex.printStackTrace();
					Util.show_and_log("getTrans.read(), read error: " + fileName, ex.getMessage());
					try { br.close(); } 
					catch (IOException e) { e.printStackTrace(); }
					return -2;
				}

				if(instring == null) break;
				instring = instring.trim();
				token = instring.split(delim);	

				if (token[0].equals("v"))
				{
					try 
					{ 
						i = Integer.parseInt(token[1]);

                        if (sp.shift_unit == SW_main.units.PPM)
                            sp.shifts[i-1].freq = Double.parseDouble(token[2]) / q.SFO1;
                        else
                            sp.shifts[i-1].freq = Double.parseDouble(token[2]);
					}
					catch (Exception ex)
					{   Util.show_and_log("getOpt(): bad shift conversion: ", ex.getMessage()); }

					count++;
				}

				if (token[0].equals("j"))
				{
					try
					{
						i = Integer.parseInt(token[1]);
						j = Integer.parseInt(token[2]);
						sp.j_coupling.value[i-1][j-1] = Double.parseDouble(token[3]);
					}
					catch (Exception ex)
                    { Util.show_and_log("getOpt(): bad j conversion: ", ex.getMessage()); }

					count++;
				}

				if (token[0].equals("d"))
				{
					try
					{
						i = Integer.parseInt(token[1]);
						j = Integer.parseInt(token[2]);
						sp.d_coupling.value[i - 1][j - 1] = Double.parseDouble(token[3]);
					}
					catch (Exception ex)
                    { Util.show_and_log("getOpt(): d conversion: ", ex.getMessage()); }

					count++;
				}
			}

			try {	br.close(); } 
			catch (IOException ex) {	ex.printStackTrace();	}
			
			return count;
		}

//-------------------------------------------------------------------------------------------------

    public static int old_index(String label, SimPar sp)
    {
        for(int i = 0; i < SimPar.MAX_SHIFTS; i++)
            if(label.equals(sp.temp_label[i])) 
                return i;
        return -1;
    }
        
//--------------------------------------------------------------------------------------------------
//    	This function generates a synthetic spectrum from the values 
//    	in the peaks structure and puts it into the calc_array 

    public static void genspec(global_par gp, SimPar sp, data_par q)
    {
    	int i, j;						// indices 
    	int k;							// the peak reconstruction range in pts 
    	int start, end;
    	double w, deltav;
    	double a;						// intensity scaled to computer word size 
    	double factor;					// scaling factor
    	double total_intensity = 0.0;   // total intensity of all transitions
    	int total_spins = 0;            // total number of spins
    	double width = sp.linewidth;	// local copy of linewidth (changed in this routine)

    	if (gp.calc_array == null || sp.calc_mode != SimPar.HIRES) return;

    	try
    	{
    		if (sp.lineshape == SimPar.LOREN)
    			k = (int) (40 * Math.round(width / q.hzperpt));	// 40 * linewidth for each peak 
    		else // Gaussian 
    			k = (int) (80 * Math.round(width / q.hzperpt));	// bigger for Gaussian peaks 
    	}
    	catch (Exception ex)
    	{
    		Util.show_and_log("genspec() bad k " + ex.getMessage());
    		return;
    	}

    	// zero the calculated spectrum array 
    	for (i = 0; i < q.SI; i++)
    		gp.calc_array[i] = 0;

    	if (sp.lineshape == SimPar.GAUSS)
    		width = width / 2.0;			

    	for (i = 0; i < sp.n_trans; i++)
    		total_intensity = total_intensity + gp.calc_peaks[i].intensity;

    	//  Spin normalization code
    	for (i = 0; i < sp.n_shifts; i++)
    	{
    		if (sp.shifts[i].type.equals("1"))
    			total_spins = total_spins + 1;
    		else if (sp.shifts[i].type.equals("2"))
    			total_spins = total_spins + 2;
    		else if (sp.shifts[i].type.equals("3"))
    			total_spins = total_spins + 3;
    		else if (sp.shifts[i].type.equals("4"))
    			total_spins = total_spins + 4;
    		else if (sp.shifts[i].type.equals("2*1"))
    			total_spins = total_spins + 2;
    		else if (sp.shifts[i].type.equals("2*2"))
    			total_spins = total_spins + 4;
    		else if (sp.shifts[i].type.equals("2*3"))
    			total_spins = total_spins + 6;
    	}

    	if (Math.abs(total_intensity) > 0.1)
    		factor = 1.0e06 * (double)total_spins / total_intensity;
    	else
    		factor = 1.0e06;

    	i = 0;
    	while (i < sp.n_trans)
    	{
    		w = 1.0 / (width * width * gp.calc_peaks[i].rel_width);

    		try
    		{
    			start = (int) (Math.round((q.offset - gp.calc_peaks[i].calc_freq)
    					/ q.hzperpt) - k);
    		}
    		catch (Exception ex)
    		{
    			Util.show_and_log("genspec() bad k " + ex.getMessage());
    			return;
    		}

    		if (start < 0)
    			start = 0;

    		end = start + 2 * k;
    		if (end >= q.SI)
    			end = q.SI - 1;

    		a = gp.calc_peaks[i].intensity * factor;

    		if (sp.lineshape == SimPar.LOREN)
    		{
    			for (j = start; j <= end; j++)
    			{
    				deltav = (q.offset - j * q.hzperpt) - gp.calc_peaks[i].calc_freq;
    				gp.calc_array[j] = gp.calc_array[j] +
    						(a / (1.0 + 4.0 * deltav * deltav * w));
    			}
    		}
    		else // Gaussian 
    		{
    			for (j = start; j <= end; j++)
    			{
    				deltav = (q.offset - j * q.hzperpt) - gp.calc_peaks[i].calc_freq;
    				gp.calc_array[j] = gp.calc_array[j] +
    						(a * Math.exp(-M_LN2 * deltav * deltav * w));
    			}
    		}
    		i++;
    	}
    }       
 //------------------------------------------------------------------------------------------------       
}//	end class
