
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>
package simulation;

import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.wb.swt.SWTResourceManager;

import spinworks.SW_main;
import spinworks.Util;

import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Group;

/**
 * @author marat
 *
 */
public class ShiftEditor extends Dialog 
{
	protected Object result;
	protected Shell shell;
	
	private Label[] labelGroup;
	private Text[] textSpins;	
	private Text[] textLabel;	
	private Text[] textSpecies;
	private Combo[] comboSpin;
	private Text[] textShift;
	private Button[] buttonIterate;

	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public ShiftEditor(Shell parent, int style) 
	{
		super(parent, style);
		setText("Edit Chemical Shifts");
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() 
	{
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) 
		{
			if (!display.readAndDispatch()) 
			{
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() 
	{
		shell = new Shell(getParent(), getStyle());
		shell.setSize(451, 506);
		shell.setText("Edit Chemical Shifts");
		
		labelGroup = new Label[SimPar.MAX_SHIFTS];
		textSpins = new Text[SimPar.MAX_SHIFTS];
		textLabel = new Text[SimPar.MAX_SHIFTS];
		textSpecies = new Text[SimPar.MAX_SHIFTS];
		comboSpin = new Combo[SimPar.MAX_SHIFTS];
		textShift = new Text[SimPar.MAX_SHIFTS];
		buttonIterate = new Button[SimPar.MAX_SHIFTS];
		
		Button buttonOK = new Button(shell, SWT.NONE);	
		buttonOK.setBounds(283, 444, 75, 25);
		buttonOK.setFont(SWTResourceManager.getBoldFont(buttonOK.getFont()));
		buttonOK.setBackground(shell.getDisplay().getSystemColor(SWT.COLOR_GREEN));
		buttonOK.setText("OK");
		shell.setDefaultButton(buttonOK);
		
		Button buttonCancel = new Button(shell, SWT.NONE);		
		buttonCancel.setBounds(360, 444, 75, 25);
		buttonCancel.setFont(SWTResourceManager.getBoldFont(buttonCancel.getFont()));
		buttonCancel.setBackground(shell.getDisplay().getSystemColor(SWT.COLOR_RED));
		buttonCancel.setText("Cancel");	
		
				
		Label lblSpins = new Label(shell, SWT.NONE);
		lblSpins.setBounds(45, 10, 38, 15);
		lblSpins.setText("Spins");
		
		Label lblLabel = new Label(shell, SWT.NONE);
		lblLabel.setBounds(87, 10, 47, 15);
		lblLabel.setText("Label");
		
		Label lblSpecies = new Label(shell, SWT.NONE);
		lblSpecies.setBounds(154, 10, 61, 15);
		lblSpecies.setText("Species");
			
		Label lblSpin = new Label(shell, SWT.NONE);
		lblSpin.setBounds(221, 10, 38, 15);
		lblSpin.setText("Spin");
		
		Label lblNewLabel = new Label(shell, SWT.NONE);
		lblNewLabel.setBounds(283, 10, 71, 15);
		lblNewLabel.setText("Shift (Hz)");
		
		Group groupShiftUnit = new Group(shell, SWT.NONE);
		groupShiftUnit.setText("Shift Unit");
		groupShiftUnit.setBounds(10, 376, 425, 62);
		
		Button buttonHz = new Button(groupShiftUnit, SWT.RADIO);		
		buttonHz.setBounds(10, 29, 57, 16);
		buttonHz.setText("Hz.");
		if(SW_main.sp.shift_unit == SW_main.units.HZ)
			buttonHz.setSelection(true);
		buttonHz.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.sp.shift_unit = SW_main.units.HZ;
				for(int i = 0; i < SimPar.MAX_SHIFTS; i++)
					textShift[i].setText(String.format("%.4f", SW_main.sp.shifts[i].freq));
			}
		});
		
		Button buttonPPM = new Button(groupShiftUnit, SWT.RADIO);	
		buttonPPM.setBounds(82, 29, 90, 16);
		buttonPPM.setText("PPM");
		if(SW_main.sp.shift_unit == SW_main.units.PPM)
			buttonPPM.setSelection(true);
		buttonPPM.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				SW_main.sp.shift_unit = SW_main.units.PPM;
				for(int i = 0; i < SimPar.MAX_SHIFTS; i++)
					textShift[i].setText(String.format("%.4f", SW_main.sp.shifts[i].freq 
							                                           / SW_main.q.SFO1));
			}
		});
		
		Label labelFreq = new Label(groupShiftUnit, SWT.NONE);
		labelFreq.setBounds(178, 30, 142, 15);
		labelFreq.setText(String.format("Freq. = %.4f  MHz", SW_main.q.SFO1));
		
		for(int i = 0; i < SimPar.MAX_SHIFTS; i++)
		{
			labelGroup[i] = new Label(shell, SWT.NONE);
			labelGroup[i].setBounds(20, 50 + i * 25, 19, 15);
			labelGroup[i].setText(String.format("%d", i + 1));
			
			textSpins[i] = new Text(shell, SWT.BORDER);
			textSpins[i].setBounds(45, 47 + i * 25, 36, 21);
			textSpins[i].setText(SW_main.sp.shifts[i].type);

			textLabel[i] = new Text(shell, SWT.BORDER);
			textLabel[i].setBounds(87, 47 + i * 25, 61, 21);
			textLabel[i].setText(SW_main.sp.shifts[i].label);

			textSpecies[i] = new Text(shell, SWT.BORDER);
			textSpecies[i].setBounds(154, 47 + i * 25, 61, 21);
			textSpecies[i].setText(SW_main.sp.shifts[i].species);
			
			comboSpin[i] = new Combo(shell, SWT.NONE);
			comboSpin[i].setBounds(221, 47 + i * 25, 56, 23);
			comboSpin[i].setItems(new String[] {"1/2", "1", "3/2", "2", "5/2", "3", "7/2"});
			comboSpin[i].select(SW_main.sp.shifts[i].spin - 1);

			textShift[i] = new Text(shell, SWT.BORDER);
			textShift[i].setBounds(283, 47 + i * 25, 93, 21);
			if(SW_main.sp.shift_unit == SW_main.units.HZ)
				textShift[i].setText(String.format("%.4f", SW_main.sp.shifts[i].freq));
			else if(SW_main.sp.shift_unit == SW_main.units.PPM)
				textShift[i].setText(String.format("%.4f", SW_main.sp.shifts[i].freq 
						                                      / SW_main.q.SFO1));
			
			buttonIterate[i] = new Button(shell, SWT.CHECK);
			buttonIterate[i].setBounds(382, 52 + i * 25, 93, 16);
			buttonIterate[i].setText("Iterate");
			buttonIterate[i].setSelection(SW_main.sp.shifts[i].iterate);
		}
		
		// Listeners
		buttonOK.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				getShifts();
				
				String target;
				if (SW_main.gp.data_array == null || SW_main.q.data_folder.length() < 2)
					target = SW_main.gp.scratch_folder + "/spin_system";
				else
					target = SW_main.q.data_folder + "spin_system";

	            if (SW_main.gp.log_on) Util.log_error("Edit Chemical Shifts: writing spin system to: " + target);	            
	            SS_util.write_spins(target, SW_main.gp, SW_main.sp, SW_main.q);
	            
				shell.dispose();
			}
		});

		buttonCancel.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				shell.dispose();
			}
		});
									
	}	// end createContents

//-------------------------------------------------------------------------------------------------
//	get shift values from the dialog

	private void getShifts()
	{
		int old_shifts = SW_main.sp.n_shifts;               // keep track of current number of shifts

		// TODO is this used???
		for (int i = 0; i < SW_main.sp.n_shifts; i++)
			SW_main.sp.temp_label[i] = SW_main.sp.shifts[i].label;  // also keep track of current labels

		SW_main.sp.n_shifts = 0;                            // re-set number of shifts

		for (int i = 0; i < SimPar.MAX_SHIFTS; i++)
		{
			SW_main.sp.shifts[i].type = "";
			SW_main.sp.shifts[i].label = "";
			SW_main.sp.shifts[i].species = "";
			SW_main.sp.shifts[i].freq = 0.0;
		}

//!String.IsNullOrEmpty(dlg.type_0)

		for(int i = 0; i < SimPar.MAX_SHIFTS; i++)
		{
			if (!textSpins[i].getText().isEmpty() && !textSpins[i].getText().equals("0"))
			{
				SW_main.sp.shifts[SW_main.sp.n_shifts].type = textSpins[i].getText();
				SW_main.sp.shifts[SW_main.sp.n_shifts].label = textLabel[i].getText();
			
				if(textLabel[i].getText().isEmpty())
					SW_main.sp.shifts[SW_main.sp.n_shifts].label = "_1";
				
				SW_main.sp.shifts[SW_main.sp.n_shifts].species = textSpecies[i].getText();
				SW_main.sp.shifts[SW_main.sp.n_shifts].spin = comboSpin[i].getSelectionIndex() + 1;
				
				double temp = 0.0;
				try { temp = Double.parseDouble(textShift[i].getText()); }
				catch (Exception ex) { Util.show_and_log(ex.getMessage());}
				
				if (SW_main.sp.shift_unit == SW_main.units.HZ)
					SW_main.sp.shifts[SW_main.sp.n_shifts].freq = temp;
				else if (SW_main.sp.shift_unit == SW_main.units.PPM)
					SW_main.sp.shifts[SW_main.sp.n_shifts].freq = temp * SW_main.q.SFO1;

				SW_main.sp.shifts[SW_main.sp.n_shifts].iterate = buttonIterate[i].getSelection();
				SW_main.sp.n_shifts++;
			}
		}

		//  rationalize couplings if a shift has been deleted
		if (SW_main.sp.n_shifts < old_shifts)
		{
			for(int i = 0; i < SW_main.sp.n_shifts; i++)
				for (int j = 0; j < SW_main.sp.n_shifts; j++)
				{
					int k = SS_util.old_index(SW_main.sp.shifts[i].label, SW_main.sp);
					int l = SS_util.old_index(SW_main.sp.shifts[j].label, SW_main.sp);
					if (k >= 0 && l >= 0)
					{
						SW_main.sp.j_coupling.value[i][j] = SW_main.sp.j_coupling.value[k][l];
						SW_main.sp.d_coupling.value[i][j] = SW_main.sp.d_coupling.value[k][l];
					}
				}
		}
	}	// getShifts()
//-------------------------------------------------------------------------------------------------
}	// end of class
