/**
 * 
 */
package simulation;

import spinworks.SW_main;
import spinworks.SW_main.units;

/**
 * @author marat
 * Internal simulation parameters for SpinWorks Java
 */
public class SimPar implements Cloneable
{
	public final static int MAX_TRANS = 80000;
	public final static int MAX_SHIFTS = 12;
	public final static int MAX_COUPLINGS = 66;
	public final static int MAX_SYSTEMS = 3;
	public final static double DNMR_SCALE = 10000000.0;

	public final static int HIRES = 0;					// change to enumeration!!!  TO DO!!!
	public final static int DNMR = 1;
	public final static int DECON = 2;

	public final static int LOREN = 0;
	public final static int GAUSS = 1;		

	public SimPar()							// constructor, initialize values
	{
		n_shifts = 0;
				
		nconfig = 2;
		w = new double[4][6];
		aj = new double[4][6][6];
		rc = new double[4][4];
		t2 = 1.0;
		ie = new int[4][6];

		for (int i = 0; i < 4; i++) 
			for (int j = 0; j < 6; j++)
				ie[i][j] = 0;

		mu = true;
		nnmes = new int[MAX_SHIFTS];
		nspins = 0;
		pop = new double[4];

		calc_mode = HIRES;	
		lineshape = LOREN;			// enumerate!!!

		linewidth = 1.0;
		group_linewidth = 0.025;
		trans_thresh = 0.01;
		n_trans = 0;

		swidth = 5000.0;		// default spectral width in Hz.
		offset = 5000.0;		// default offset in Hz.
        shift_unit = SW_main.units.HZ;

		optmiz = 0;
		autasn = 1;
		autdel = 1;

		itrlim = 30;
		rmslim = 0.02;
		asnrms = 0.10;
		rngfra = 0.25;
		asnlim = 1.92;
		ignlim = 2.4;
		asnmod = 2;

		time_out = 10;			// 10 sec

        
        temp_label = new String[MAX_SHIFTS];
	}
	
//------------------------------------------------------------------------------------------------
	public Object clone() throws CloneNotSupportedException 
	{
        return super.clone();
    }
	
//-------------------------------------------------------------------------------------------------

    public ShiftInfo[] shifts;
	public ShiftInfo[] status_shifts;
	public Coupling j_coupling;
	public Coupling d_coupling;

    public String[] temp_label;

	public int n_shifts;

	public double linewidth;
	public int lineshape;
	public double group_linewidth;
	public double trans_thresh;
	public int n_trans;

	public double swidth;
	public double offset;
    public units shift_unit;

	public int calc_mode;


//Original NUMMRIT Fortan (yetch!) common blocks, many pars are not needed

	// /optcom/
	public double rmscon, rmsdiv, rmslim, damp, asnlim,
					ignlim, itrfac, asnrms;
	public int itrlim, itrmin, asnmod;
	public int anyasn, wgtrms;

	// /flagcm/
	public int autasn, autdel, optmiz, prtptl,
					prtkey, prtdif, prtvcm, pracnd;
	// /shpcom/
	public double ampscl, shpfac;

	// /prmcom/
	public int prmsiz, prmnum, prmi,
					prmj, prmd, lowvar;
	public double rngdef, rngnit, rngmin, rngfra;


//TO DO  CLEAN UP THIS MESS!!!!!!

//DNMR parameters, names extracted from original FORTRAN "common blocks", if anyone remembers what
//they were!  Probably a lot of these can be scrapped, and many of those left have their
//names changed to something sane.

//		int nconfig;
	public int nconfig;

////common block "par"
	public double[][] w;
	public double[][][] aj;

////common block "dynpar"
	public double[][] rc;
	public double t2;

////common block "permt"
	public int[][] ie;
	public boolean mu;

////original DNMR blank common
	public int n, np, max, maxm, ne, nm, nem, nen, nmes, nspins,
		nmesm, nsymm, mrc;
	public int[] nnmes; 

	public double[] pop;

	public int time_out;							// simulation timeout value (100 ms units)
}
