
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Display;

import dialogs.CalibrateDialog;
import dialogs.ParEditor;

/**
 * Listeners for the Button Panel
 * @author marat
 */
public class ButtonListeners extends SW_main
{
	public static void addButtonListeners()
	{
// Display buttons	
		vertScaleDownButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(gp.display_mode == modes.MODE_1D)
					gp.expt_scale *= 0.8333;	//  1/1.20
				else if(gp.display_mode == modes.MODE_FID)
					gp.fid_scale *= 0.8333;
				else if(gp.display_mode == modes.MODE_2D)
				{
					Util_2D.rescale_floor_2D(0.8333, gp);
					Util_2D.rescale_ceiling_2D(0.8333, gp);
					gp.new_2D_view = true;
				}
				
			    update_spect = true;
			    canvas.redraw();
			}
		});
		
		vertScaleUpButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(gp.display_mode == modes.MODE_1D)
					gp.expt_scale *= 1.20;	
				else if(gp.display_mode == modes.MODE_FID)
					gp.fid_scale *= 1.20;
				else if(gp.display_mode == modes.MODE_2D)
				{
					Util_2D.rescale_floor_2D(1.20, gp);
					Util_2D.rescale_ceiling_2D(1.20, gp);
					gp.new_2D_view = true;
				}
				
			    update_spect = true;
			    canvas.redraw();
			}
		});
				
		rangeScaleDownButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(gp.display_mode == modes.MODE_2D)  // 2D/3D display
	            {
					Util_2D.rescale_ceiling_2D(0.8333, gp);
					gp.new_2D_view = true;
	            }
			    update_spect = true;
			    canvas.redraw();
			}
		});
		
		rangeScaleUpButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(gp.display_mode == modes.MODE_2D)  // 2D/3D display
	            {
					Util_2D.rescale_ceiling_2D(1.20, gp);
					gp.new_2D_view = true;
	            }
			    update_spect = true;
			    canvas.redraw();
			}
		});
			
		prevBlockButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DisplayButtons.prevBlockButtonClick();
			}
		});
		
		nextBlockButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DisplayButtons.nextBlockButtonClick();
			}
		});

		scale1D_DownButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (gp.f1_trace_scale_active)
					gp.f1_trace_scale = gp.f1_trace_scale / 1.20;
				else if(gp.f2_trace_scale_active)
					gp.f2_trace_scale = gp.f2_trace_scale / 1.20;
				else
					gp.row_scale[gp.active_slice] = gp.row_scale[gp.active_slice] / 1.20;
				
				update_spect = true;
			    canvas.redraw();
			}
		});
		
		scale1D_UpButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (gp.f1_trace_scale_active)
					gp.f1_trace_scale = gp.f1_trace_scale * 1.20;
				else if(gp.f2_trace_scale_active)
					gp.f2_trace_scale = gp.f2_trace_scale * 1.20;
				else
					gp.row_scale[gp.active_slice] = gp.row_scale[gp.active_slice] * 1.20;
				
				update_spect = true;
			    canvas.redraw();
			}
		});
		
		vertResetButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DisplayButtons.vResetButtonClick();
			}
		});

		fullButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DisplayButtons.fullButtonClick();
			}
		});

		zoomButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DisplayButtons.zoomButtonClick();
			}
		});

		lastButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DisplayButtons.lastButtonClick();
			}
		});
			       
		disp2DButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DisplayButtons.display2DButtonClick();
			}
		});
		
		posOnlyButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DisplayButtons.posOnlyButtonClick();
			}
		});
		
		contourButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DisplayButtons.contourButtonClick();
			}
		});
		
		colButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DisplayButtons.colButtonClick();
			}
		});
		
		clearButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				gp.nslices = 0;
				update_spect = true;
			    canvas.redraw();
			}
		});
			
		editParsButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ParEditor dlg 
				= new ParEditor(shell, 
						SWT.APPLICATION_MODAL | SWT.TITLE | SWT.BORDER );
				dlg.open();      
			}
		});
		
		processButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				ProcessButton.processButtonClick();
			}
		});
		
// soft buttons, change with context	
		softButton1.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (q.data_mode == modes.MODE_1D)			// no 1D assignment yet
				{

				}
				else   //2D  Process F2 only
				{
					statusStripLabel1.setText("");
					statusStripLabel2.setText("Processing Rows:");
					Proc_2D.proc_rows(gp, p, q, p_f1, q_f1);			// FFT process the rows
					statusStripLabel2.setText("2D Processing Complete");
					statusStripLabel1.setText("Ready");
					gp.block_2D = 0;									// re-set block (row)counter
					Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);	// load first row 
					Util_2D.calc_scale_factors_2D(gp, q, q_f1);			// calculate scaling factors, etc.
					gp.display_mode = modes.MODE_2D;					// switch to 2D display
					gp.new_2D_view = true;								// must re-contour

					Util_2D.calc_default_levels_2D(gp);					// calculate def.display levels
				}
				update_spect = true;
				canvas.redraw();
			}
		});
		
		softButton2.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (q.data_mode == modes.MODE_1D)			// no 1D assignment yet
				{
				}
				else   //2D  Process F1 only
				{
					statusStripLabel1.setText("");
					statusStripLabel2.setText("Processing Columns:");
					Proc_2D.proc_columns(gp, p, q, p_f1, q_f1);	// FFT process the columns
					statusStripLabel2.setText("2D Processing Complete");
					statusStripLabel1.setText("Ready");
					gp.block_2D = 0;							// re-set block (row)counter
					Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);	// load first row 
					Util_2D.calc_scale_factors_2D(gp, q, q_f1);				// calculate scaling factors
					gp.display_mode = modes.MODE_2D;			// switch to 2D display
					gp.new_2D_view = true;						// must re-contour				
					Util_2D.calc_default_levels_2D(gp);			// calculate default display levels
				}
				update_spect = true;
				canvas.redraw();
			}
		});
		
		softButton3.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (q.data_mode == modes.MODE_1D)
				{
					if (gp.data_array != null)
	                {
						Fft.do_phase(gp.data_array, gp.i_data_array, q.SI, p.PHC0, p.PHC1);
	                    q.PHC0 = p.PHC0 + q.PHC0;       // update status parameters 	
	                    q.PHC1 = p.PHC1 + q.PHC1;
	                    p.PHC0 = q.PHC0;
	                    p.PHC1 = q.PHC1;
	                }

					if (gp.auto_save)
					{
						FileUtil.saveProc1D(gp, q);
	                    String jdx_file;
	                    jdx_file = q.data_folder + "spectrum.dx";
	                    DX_Util.write_jdx(jdx_file, gp, q);
					}
				}

				else   //2D  Apply Phase F2
				{
					if (gp.rows_real == null)
					{
						Util.show_and_log("No processed data!");
						return;
					}
					if (p_f1.MC2 == det_modes_IND.QF && gp.dimensions_processed == 2)
					{
						Util.show_and_log("Can't phase magnitude mode data!");
						return;
					}
					if(gp.rows_imag == null)
					{
						Util.show_and_log("No imaginary data!");
						return;
					}

					statusStripLabel1.setText("");
					statusStripLabel2.setText("Phasing rows");

					shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
	                Proc_2D.phase_rows(gp, p, q, q_f1);
					Util_2D.calc_scale_factors_2D(gp, q, q_f1);			// calculate scaling factors, etc.
					gp.block_2D = 0;
					Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);
					statusStripLabel2.setText("F2 phasing complete");
					statusStripLabel1.setText("Ready");
					gp.new_2D_view = true;

					if (gp.auto_save)						
						FileUtil.saveProc2D(false, gp, q, q_f1);
				}

				Util.rescale(gp, q);						// generate scaling paramaters
				gp.v_rescale = false;						// don't rescale on subsequent processing
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
				gp.cursor_flag = 0;						    // erase any spurious cursor lines
				update_spect = true;
				canvas.redraw();
			}
		});
		
		softButton4.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (q.data_mode == modes.MODE_1D)		// 1D AutoPhase
				{
					shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
					AutoPhase.lrAutoPhase(gp, p, q);	// linear regression autophase
					/*if(q.SFO1 > 299.0)
						AutoPhase.blfAutoPhase(gp, p, q);*/
					shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
				}
				else   									//2D  Apply Phase F1
				{
					if (gp.rows_real == null)
					{
						Util.show_and_log("No processed data!");
						return;
					}
					if (gp.dimensions_processed == 1)
					{
						Util.show_and_log("F1 is not yet processed!");
						return;
					}
					if (p_f1.MC2 == det_modes_IND.QF)
					{
						Util.show_and_log("Can't phase magnitude mode data!", "SpinWorks 4");
						return;
					}
					if (gp.irows_imag == null)
					{
						Util.show_and_log("No imaginary data");
						return;
					}
					
					statusStripLabel1.setText("");
					statusStripLabel2.setText("Phasing columns");

	                Proc_2D.phase_columns(gp, q, p_f1, q_f1);
					Util_2D.calc_scale_factors_2D(gp, q, q_f1);			// calculate scaling factors, etc.
					gp.block_2D = 0;
					Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);
					statusStripLabel2.setText("F1 phasing complete");
					statusStripLabel1.setText("Ready");
					gp.new_2D_view = true;

					if (gp.auto_save)
						FileUtil.saveProc2D(false, gp, q, q_f1);
				}
				update_spect = true;
				canvas.redraw();
			}
		});
		
		softButton5.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (q.data_mode == modes.MODE_1D)
				{
					if (gp.data_array != null)
						Fft.do_mc(gp.data_array, gp.i_data_array, q.SI);

					if (gp.auto_save)							
						FileUtil.saveProc1D(gp, q);
				}

				else   //2D
				{
					if (gp.rows_real == null)
					{
						Util.show_and_log("No processed data!");
						return;
					}
					if (gp.irows_real == null)
					{
						Util.show_and_log("No imaginary data!");
						return;
					}

					statusStripLabel1.setText("");
					statusStripLabel2.setText("Doing magnitude calculation of F1");

					shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
	                Proc_2D.mc_columns(gp, q, q_f1);
					Util_2D.calc_scale_factors_2D(gp, q, q_f1);			// calculate scaling factors, etc.
					gp.block_2D = 0;
					Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);
					statusStripLabel2.setText("F1 MC complete");
					statusStripLabel1.setText("Ready");
					gp.new_2D_view = true;

					if (gp.auto_save)							
						FileUtil.saveProc2D(false, gp, q, q_f1);
				}

				Util.rescale(gp, q);						// generate scaling paramaters
				gp.v_rescale = false;						// don't rescale on subsequent processing
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
				gp.cursor_flag = 0;						    // erase any spurious cursor lines
				update_spect = true;
				canvas.redraw();
			}
		});
		
		//TODO probably should rename this, as the context never changes
		softButton6.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(gp.display_mode == modes.MODE_1D && gp.data_array != null)
				{
					if (gp.adjust_mi == false)
					{
						gp.adjust_mi = true;
						softButton6.setText("Return");
						softButton6.setBackground(gp.colour_red);
						statusStripLabel1.setText("Click at desired PP minimum");						
					}
					else
					{
						gp.adjust_mi = false;
						softButton6.setText("PP Min.");
						softButton6.setBackground(gp.colour_green);
						statusStripLabel1.setText("Ready");
	                    statusStripLabel2.setText(String.format("Peak Pick Minimum = %.4f", 
							gp.pp_thresh));
					}
					update_spect = true;
					canvas.redraw();
				}
				else
				{
					// no 2D assignment for this button yet
				}
			}
		});
		
		blPointButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(gp.display_mode == modes.MODE_1D && gp.data_array != null)
				{
					if (gp.bpt_mode == false)
					{
						gp.bpt_mode = true;
						blPointButton.setText("Return");
						blPointButton.setBackground(gp.colour_red);
						statusStripLabel1.setText("Click to define baseline points");						
					}
					else
					{
						gp.bpt_mode = false;
						blPointButton.setText("BL Points");
						blPointButton.setBackground(gp.colour_green);
						statusStripLabel1.setText("Ready");
	                    statusStripLabel2.setText(String.format("Number of basline points = %d", 
							gp.n_bl_pts));
					}
					update_spect = true;
					canvas.redraw();
				}
				else
				{
					// no 2D assignment for this button yet
				}
			}
		});
		
// Yellow (dialog) buttons
		phaseButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(!SW_main.gp.phase_mode)
					PhaseHandler.PhaseButton_Click();
			}
		});

		integrateButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(!gp.integ_mode)
					IntegrateHandler.IntegrateButtonClick();
			}
		});
		
		calibrateButton.addSelectionListener(new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				CalibrateDialog dlg = new CalibrateDialog(shell, 
						SWT.APPLICATION_MODAL | SWT.TITLE | SWT.BORDER );
				int result = (int) dlg.open();
				if(result == SWT.OK)
				{
					if (gp.auto_save)
					{
						if(q.data_mode == modes.MODE_1D)
							FileUtil.saveProc1D(gp, q);
						else if(q.data_mode == modes.MODE_2D)
							FileUtil.saveProc2D(true, gp, q, q_f1);	// only save parameters
					}	
					DisplayButtons.fullButtonClick();
				}
				else	// Cancel button pressed
				{
					gp.cursor_flag = 0;
					update_spect = true;
					canvas.redraw();
				}
			}
		});
	}
//-------------------------------------------------------------------------------------------------
}
