
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

/**
 * contouring routines for SpinWorks Java
 * @author marat
 */
public class Contour 
{	
	public static int MAX_SEGMENTS = 300000;			// maximum number of contour segments

//-------------------------------------------------------------------------------------------------
	/**
	 * Generate a set of 2D contour segments from a 2D data set.  Regular version
	 * @param gp
	 * @param q
	 * @param q_ind
	 */
	public static void do_contours(global_par gp, data_par q, data_par q_ind)
	{
		int i;
		float[][] buff_mat = new float[1][1];			// Should be better way  TO DO
		int point1, point2, point1_f1, point2_f1;

		boolean interp_flag = true;					    // true if bi-cubic spline 
		// interpolation is used
		try
		{
			point1 = (int) (Math.round((q.offset - gp.freq1) / q.hzperpt) - 1);
			point2 = (int) (Math.round((q.offset - gp.freq2) / q.hzperpt) + 1);
			point1_f1 = (int) (Math.round((q_ind.offset - gp.freq1_indirect) / q_ind.hzperpt) - 1);
			point2_f1 = (int) (Math.round((q_ind.offset - gp.freq2_indirect) / q_ind.hzperpt) + 1);
		}
		catch (Exception ex)
		{
			Util.show_and_log("do_contours: " + ex.getMessage());
			return;
		}		

		// array bounds check  (perhaps put this into function along with order check???
		if (point1 < 0) point1 = 0;
		if (point2 >= q.SI) point2 = q.SI - 1;

		if (point1_f1 < 0) point1_f1 = 0;
		if (point2_f1 >= q_ind.SI) point2_f1 = q_ind.SI - 1;

		//		turn bi-cubic spline interpolation off for big matrices
		if ((point2 - point1) * (point2_f1 - point1_f1) > 65536) interp_flag = false;

		if (interp_flag) // apply interpolation
		{
			buff_mat = new float[2 * (point2_f1 - point1_f1) + 1][2 * (point2 - point1) + 1];

			interpolate_2D(gp.rows_real, buff_mat,
					point1, point2, point1_f1, point2_f1);
		}

		gp.n_segments = 0;
		gp.n_segments_neg = 0;

		for (i = 0; i < gp.n_levels; i = i + gp.contour_spacing)
		{
			if (gp.n_segments >= MAX_SEGMENTS - 1 || gp.n_segments_neg >= MAX_SEGMENTS - 1)
				return;

			if (interp_flag)	// interpolated
			{
				gp.n_segments = scan_for_segments_interp(buff_mat, 2.0F, 2 * point1, 2 * point2,
						2 * point1_f1, 2 * point2_f1, i, gp.n_segments,
						(float)gp.pos_level[i].value, gp.seg_pos);

				if (!gp.pos_only)
					gp.n_segments_neg = scan_for_segments_neg_interp(buff_mat, 2.0F, 
							2 * point1, 2 * point2, 2 * point1_f1, 2 * point2_f1, i, 
							gp.n_segments_neg, (float)gp.neg_level[i].value, gp.seg_neg);
			}
			else				// non-interpolated
			{
				gp.n_segments = scan_for_segments(gp.rows_real, point1, point2,
						point1_f1, point2_f1, i, gp.n_segments, 
						(float)gp.pos_level[i].value, gp.seg_pos);

				if (!gp.pos_only)
					gp.n_segments_neg = scan_for_segments_neg(gp.rows_real, point1, point2,
							point1_f1, point2_f1, i, gp.n_segments_neg,
							(float)gp.neg_level[i].value, gp.seg_neg);
			}
		}        
	}

//-------------------------------------------------------------------------------------------------
	/**
	 * Do a cubic spline interpolation of the dispay region
	 * Public since it is also used in image mode
	 * @param rows_real the matrix to interpolate
	 * @param buff_mat the interpolated matrix
	 * @param point1 first point in acquisition dimension
	 * @param point2 last point in f2 acquisition dimension
	 * @param point1_f1 first point in current indirect dimension
	 * @param point2_f1 last point in current indirect dimension
	 * @author marat
	 */
	public static void interpolate_2D(float[][] rows_real, float[][] buff_mat,
			int point1, int point2,
			int point1_f1, int point2_f1)
	{
		int i, j, k;										// handy counters
		int n;												// number of points for spline

		double[] x, y, y2;									// arrays for cubic spline

		n = point2 - point1 + 1;

		x = new double[n + 1];
		y = new double[n + 1];
		y2 = new double[n + 1];

		for (i = point1_f1; i <= point2_f1; i++)				   // loop over rows
		{
			k = 1;
			for (j = point1; j <= point2; j++)					   // loop over columns
			{   // load the buffer matrix
				buff_mat[(i - point1_f1) * 2][(j - point1) * 2] = rows_real[i][j];	
				x[k] = (double)((j - point1) * 2);				   // generate values for spline
				y[k] = (double)rows_real[i][j];
				k++;
			}
			Cspline.spline(x, y, n, 1.0e30, 1.0e30, y2);		   // generate a cubic spline 

			for (j = (2 * point1) + 1; j <= 2 * point2; j = j + 2) // even rows odd cols
				buff_mat[(i - point1_f1) * 2][j - (2 * point1)]
						= (float)Cspline.splint(x, y, y2, n, (double)j - (2 * point1));
		}

		n = point2_f1 - point1_f1 + 1;

		x = new double[n + 1];
		y = new double[n + 1];
		y2 = new double[n + 1];

		for (j = 2 * point1; j <= 2 * point2; j++)			// loop over columns
		{
			k = 1;
			for (i = point1_f1; i <= point2_f1; i++)		// loop over rows
			{
				x[k] = (double)((i - point1_f1) * 2);		// generate values for spline
				y[k] = (double)buff_mat[(i - point1_f1) * 2][j - (2 * point1)];
				k++;
			}
			Cspline.spline(x, y, n, 0.0, 0.0, y2);	        // generate a cubic spline 

			for (i = (2 * point1_f1) + 1; i <= 2 * point2_f1; i = i + 2) // odd rows all cols
			{
				buff_mat[i - (2 * point1_f1)][j - (2 * point1)]
						= (float)Cspline.splint(x, y, y2, n, (double)i - (2 * point1_f1));
			}
		}
	}

//-------------------------------------------------------------------------------------------------
	/**
	 * Scan for positive level contour segments
	 * @param mat
	 * @param p1
	 * @param p2
	 * @param p1_f1
	 * @param p2_f1
	 * @param level
	 * @param total
	 * @param c_level
	 * @param seg_pos
	 * @return the number of segments found
	 * @author marat
	 */
	private static int scan_for_segments(float[][] mat, int p1, int p2,
			int p1_f1, int p2_f1, int level, int total, float c_level,
			Segment[] seg_pos)
	{
		int i, j;
		int n_segments = total;

		float j_factor = 0.0F, i_factor = 0.0F,
				j_factor2 = 0.0F, i_factor2 = 0.0F;

		for (i = p1_f1; i < p2_f1 - 1; i++)
		{
			for (j = p1; j < p2 - 1; j++)
			{
				if (n_segments >= MAX_SEGMENTS - 1)
				{
					Util.show_and_log(
							"Maximum number of contour segments exceeded\nraise minimum level (floor)");
					return n_segments;
				}

				if ((mat[i][j] > c_level
						&& mat[i + 1][j] < c_level
						&& mat[i][j + 1] < c_level
						&& mat[i + 1][j + 1] < c_level)
						||
						(mat[i][j] < c_level
								&& mat[i + 1][j] > c_level
								&& mat[i][j + 1] > c_level
								&& mat[i + 1][j + 1] > c_level))
				{
					if (mat[i][j] != mat[i][j + 1])
					{
						j_factor =
								(c_level - mat[i][j])
								/ (mat[i][j + 1] - mat[i][j]);
					}
					if (mat[i][j] != mat[i + 1][j])
					{
						i_factor =
								(c_level - mat[i][j])
								/ (mat[i + 1][j] - mat[i][j]);
					}

					seg_pos[n_segments] = new Segment();		// efficient???
					seg_pos[n_segments].x1 = (float)j + j_factor;
					seg_pos[n_segments].y1 = (float)i;
					seg_pos[n_segments].x2 = (float)j;
					seg_pos[n_segments].y2 = (float)i + i_factor;
					seg_pos[n_segments].level = level;

					n_segments++;
				}

				else if ((mat[i][j] < c_level
						&& mat[i + 1][j] > c_level
						&& mat[i][j + 1] < c_level
						&& mat[i + 1][j + 1] < c_level)
						||
						(mat[i][j] > c_level
								&& mat[i + 1][j] < c_level
								&& mat[i][j + 1] > c_level
								&& mat[i + 1][j + 1] > c_level))
				{

					if (mat[i][j] != mat[i + 1][j])
					{
						j_factor =
								(c_level - mat[i + 1][j])
								/ (mat[i + 1][j + 1] - mat[i + 1][j]);
					}
					if (mat[i][j] != mat[i + 1][j])
					{
						i_factor = 1.0F
								- (c_level - mat[i + 1][j])
								/ (mat[i][j] - mat[i + 1][j]);
					}

					seg_pos[n_segments] = new Segment();
					seg_pos[n_segments].x1 = (float)j;
					seg_pos[n_segments].y1 = (float)i + i_factor;
					seg_pos[n_segments].x2 = (float)j + j_factor;
					seg_pos[n_segments].y2 = (float)i + 1.0F;
					seg_pos[n_segments].level = level;

					n_segments++;
				}
				else if ((mat[i][j] < c_level
						&& mat[i + 1][j] < c_level
						&& mat[i][j + 1] < c_level
						&& mat[i + 1][j + 1] > c_level)
						||
						(mat[i][j] > c_level
								&& mat[i + 1][j] > c_level
								&& mat[i][j + 1] > c_level
								&& mat[i + 1][j + 1] < c_level))
				{
					if (mat[i + 1][j] != mat[i + 1][j + 1])
					{
						j_factor = 1.0F
								- (c_level - mat[i + 1][j + 1])
								/ (mat[i + 1][j] - mat[i + 1][j + 1]);
					}
					if (mat[i][j + 1] != mat[i + 1][j + 1])
					{
						i_factor = 1.0F
								- (c_level - mat[i + 1][j + 1])
								/ (mat[i][j + 1] - mat[i + 1][j + 1]);
					}

					seg_pos[n_segments] = new Segment();
					seg_pos[n_segments].x1 = (float)j + 1.0F;
					seg_pos[n_segments].y1 = (float)i + i_factor;
					seg_pos[n_segments].x2 = (float)j + j_factor;
					seg_pos[n_segments].y2 = (float)i + 1.0F;
					seg_pos[n_segments].level = level;

					n_segments++;
				}
				else if ((mat[i][j] < c_level
						&& mat[i + 1][j] < c_level
						&& mat[i][j + 1] > c_level
						&& mat[i + 1][j + 1] < c_level)
						||
						(mat[i][j] > c_level
								&& mat[i + 1][j] > c_level
								&& mat[i][j + 1] < c_level
								&& mat[i + 1][j + 1] > c_level))
				{
					if (mat[i][j] != mat[i][j + 1])
					{
						j_factor = 1.0F
								- (c_level - mat[i][j + 1])
								/ (mat[i][j] - mat[i][j + 1]);
					}
					if (mat[i][j + 1] != mat[i + 1][j + 1])
					{
						i_factor =
								(c_level - mat[i][j + 1])
								/ (mat[i + 1][j + 1] - mat[i][j + 1]);
					}

					seg_pos[n_segments] = new Segment();
					seg_pos[n_segments].x1 = (float)j + j_factor;
					seg_pos[n_segments].y1 = (float)i;
					seg_pos[n_segments].x2 = (float)j + 1.0F;
					seg_pos[n_segments].y2 = (float)i + i_factor;
					seg_pos[n_segments].level = level;

					n_segments++;
				}

				else if ((mat[i][j] > c_level
						&& mat[i][j + 1] > c_level
						&& mat[i + 1][j] < c_level
						&& mat[i + 1][j + 1] < c_level)
						||
						(mat[i][j] < c_level
								&& mat[i][j + 1] < c_level
								&& mat[i + 1][j] > c_level
								&& mat[i + 1][j + 1] > c_level))
				{
					if (mat[i][j] != mat[i + 1][j])
					{
						i_factor =
								(c_level - mat[i][j])
								/ (mat[i + 1][j] - mat[i][j]);
					}
					if (mat[i][j + 1] != mat[i + 1][j + 1])
					{
						i_factor2 =
								(c_level - mat[i][j + 1])
								/ (mat[i + 1][j + 1] - mat[i][j + 1]);
					}

					seg_pos[n_segments] = new Segment();
					seg_pos[n_segments].x1 = (float)j;
					seg_pos[n_segments].y1 = (float)i + i_factor;
					seg_pos[n_segments].x2 = (float)j + 1.0F;
					seg_pos[n_segments].y2 = (float)i + i_factor2;
					seg_pos[n_segments].level = level;

					n_segments++;
				}
				else if ((mat[i][j + 1] > c_level
						&& mat[i + 1][j + 1] > c_level
						&& mat[i][j] < c_level
						&& mat[i + 1][j] < c_level)
						||
						(mat[i][j + 1] < c_level
								&& mat[i + 1][j + 1] < c_level
								&& mat[i][j] > c_level
								&& mat[i + 1][j] > c_level))
				{
					if (mat[i][j] != mat[i][j + 1])
					{
						j_factor =
								(c_level - mat[i][j])
								/ (mat[i][j + 1] - mat[i][j]);
					}
					if (mat[i + 1][j] != mat[i + 1][j + 1])
					{
						j_factor2 =
								(c_level - mat[i + 1][j])
								/ (mat[i + 1][j + 1] - mat[i + 1][j]);
					}

					seg_pos[n_segments] = new Segment();
					seg_pos[n_segments].x1 = (float)j + j_factor;
					seg_pos[n_segments].y1 = (float)i;
					seg_pos[n_segments].x2 = (float)j + j_factor2;
					seg_pos[n_segments].y2 = (float)i + 1.0F;
					seg_pos[n_segments].level = level;

					n_segments++;
				}
			}
		}
		return n_segments;
	}

//-------------------------------------------------------------------------------------------------
	/**
	 * Scan for negative level contour segments
	 * @param mat
	 * @param p1
	 * @param p2
	 * @param p1_f1
	 * @param p2_f1
	 * @param level
	 * @param total
	 * @param c_level
	 * @param seg_neg
	 * @return the number of segments found
	 * @author marat 
	 */
	private static int scan_for_segments_neg(float[][] mat, int p1, int p2,
			int p1_f1, int p2_f1, int level, int total, float c_level,
			Segment[] seg_neg)
	{
		int i, j;
		int n_segments_neg = total;

		float j_factor = 0.0F, i_factor = 0.0F,
				j_factor2 = 0.0F, i_factor2 = 0.0F;

		for (i = p1_f1; i < p2_f1 - 1; i++)
		{
			for (j = p1; j < p2 - 1; j++)
			{
				if (n_segments_neg >= MAX_SEGMENTS - 1)
				{
					Util.show_and_log(
							"Maximum number of negative contour segments exceeded\nraise minimum level (floor)");                     
					return n_segments_neg;
				}

				if ((mat[i][j] > c_level
						&& mat[i + 1][j] < c_level
						&& mat[i][j + 1] < c_level
						&& mat[i + 1][j + 1] < c_level)
						||
						(mat[i][j] < c_level
								&& mat[i + 1][j] > c_level
								&& mat[i][j + 1] > c_level
								&& mat[i + 1][j + 1] > c_level))
				{
					if (mat[i][j] != mat[i][j + 1])
					{
						j_factor =
								(c_level - mat[i][j])
								/ (mat[i][j + 1] - mat[i][j]);
					}
					if (mat[i][j] != mat[i + 1][j])
					{
						i_factor =
								(c_level - mat[i][j])
								/ (mat[i + 1][j] - mat[i][j]);
					}

					seg_neg[n_segments_neg] = new Segment();
					seg_neg[n_segments_neg].x1 = (float)j + j_factor;
					seg_neg[n_segments_neg].y1 = (float)i;
					seg_neg[n_segments_neg].x2 = (float)j;
					seg_neg[n_segments_neg].y2 = (float)i + i_factor;
					seg_neg[n_segments_neg].level = level;

					n_segments_neg++;
				}

				else if ((mat[i][j] < c_level
						&& mat[i + 1][j] > c_level
						&& mat[i][j + 1] < c_level
						&& mat[i + 1][j + 1] < c_level)
						||
						(mat[i][j] > c_level
								&& mat[i + 1][j] < c_level
								&& mat[i][j + 1] > c_level
								&& mat[i + 1][j + 1] > c_level))
				{

					if (mat[i][j] != mat[i + 1][j])
					{
						j_factor =
								(c_level - mat[i + 1][j])
								/ (mat[i + 1][j + 1] - mat[i + 1][j]);
					}
					if (mat[i][j] != mat[i + 1][j])
					{
						i_factor = 1.0F
								- (c_level - mat[i + 1][j])
								/ (mat[i][j] - mat[i + 1][j]);
					}

					seg_neg[n_segments_neg] = new Segment();
					seg_neg[n_segments_neg].x1 = (float)j;
					seg_neg[n_segments_neg].y1 = (float)i + i_factor;
					seg_neg[n_segments_neg].x2 = (float)j + j_factor;
					seg_neg[n_segments_neg].y2 = (float)i + 1.0F;
					seg_neg[n_segments_neg].level = level;

					n_segments_neg++;
				}
				else if ((mat[i][j] < c_level
						&& mat[i + 1][j] < c_level
						&& mat[i][j + 1] < c_level
						&& mat[i + 1][j + 1] > c_level)
						||
						(mat[i][j] > c_level
								&& mat[i + 1][j] > c_level
								&& mat[i][j + 1] > c_level
								&& mat[i + 1][j + 1] < c_level))
				{
					if (mat[i + 1][j] != mat[i + 1][j + 1])
					{
						j_factor = 1.0F
								- (c_level - mat[i + 1][j + 1])
								/ (mat[i + 1][j] - mat[i + 1][j + 1]);
					}
					if (mat[i][j + 1] != mat[i + 1][j + 1])
					{
						i_factor = 1.0F
								- (c_level - mat[i + 1][j + 1])
								/ (mat[i][j + 1] - mat[i + 1][j + 1]);
					}

					seg_neg[n_segments_neg] = new Segment();
					seg_neg[n_segments_neg].x1 = (float)j + 1.0F;
					seg_neg[n_segments_neg].y1 = (float)i + i_factor;
					seg_neg[n_segments_neg].x2 = (float)j + j_factor;
					seg_neg[n_segments_neg].y2 = (float)i + 1.0F;
					seg_neg[n_segments_neg].level = level;

					n_segments_neg++;
				}
				else if ((mat[i][j] < c_level
						&& mat[i + 1][j] < c_level
						&& mat[i][j + 1] > c_level
						&& mat[i + 1][j + 1] < c_level)
						||
						(mat[i][j] > c_level
								&& mat[i + 1][j] > c_level
								&& mat[i][j + 1] < c_level
								&& mat[i + 1][j + 1] > c_level))
				{
					if (mat[i][j] != mat[i][j + 1])
					{
						j_factor = 1.0F
								- (c_level - mat[i][j + 1])
								/ (mat[i][j] - mat[i][j + 1]);
					}
					if (mat[i][j + 1] != mat[i + 1][j + 1])
					{
						i_factor =
								(c_level - mat[i][j + 1])
								/ (mat[i + 1][j + 1] - mat[i][j + 1]);
					}

					seg_neg[n_segments_neg] = new Segment();
					seg_neg[n_segments_neg].x1 = (float)j + j_factor;
					seg_neg[n_segments_neg].y1 = (float)i;
					seg_neg[n_segments_neg].x2 = (float)j + 1.0F;
					seg_neg[n_segments_neg].y2 = (float)i + i_factor;
					seg_neg[n_segments_neg].level = level;

					n_segments_neg++;
				}

				else if ((mat[i][j] > c_level
						&& mat[i][j + 1] > c_level
						&& mat[i + 1][j] < c_level
						&& mat[i + 1][j + 1] < c_level)
						||
						(mat[i][j] < c_level
								&& mat[i][j + 1] < c_level
								&& mat[i + 1][j] > c_level
								&& mat[i + 1][j + 1] > c_level))
				{
					if (mat[i][j] != mat[i + 1][j])
					{
						i_factor =
								(c_level - mat[i][j])
								/ (mat[i + 1][j] - mat[i][j]);
					}
					if (mat[i][j + 1] != mat[i + 1][j + 1])
					{
						i_factor2 =
								(c_level - mat[i][j + 1])
								/ (mat[i + 1][j + 1] - mat[i][j + 1]);
					}

					seg_neg[n_segments_neg] = new Segment();
					seg_neg[n_segments_neg].x1 = (float)j;
					seg_neg[n_segments_neg].y1 = (float)i + i_factor;
					seg_neg[n_segments_neg].x2 = (float)j + 1.0F;
					seg_neg[n_segments_neg].y2 = (float)i + i_factor2;
					seg_neg[n_segments_neg].level = level;

					n_segments_neg++;
				}
				else if ((mat[i][j + 1] > c_level
						&& mat[i + 1][j + 1] > c_level
						&& mat[i][j] < c_level
						&& mat[i + 1][j] < c_level)
						||
						(mat[i][j + 1] < c_level
								&& mat[i + 1][j + 1] < c_level
								&& mat[i][j] > c_level
								&& mat[i + 1][j] > c_level))
				{
					if (mat[i][j] != mat[i][j + 1])
					{
						j_factor =
								(c_level - mat[i][j])
								/ (mat[i][j + 1] - mat[i][j]);
					}
					if (mat[i + 1][j] != mat[i + 1][j + 1])
					{
						j_factor2 =
								(c_level - mat[i + 1][j])
								/ (mat[i + 1][j + 1] - mat[i + 1][j]);
					}

					seg_neg[n_segments_neg] = new Segment();
					seg_neg[n_segments_neg].x1 = (float)j + j_factor;
					seg_neg[n_segments_neg].y1 = (float)i;
					seg_neg[n_segments_neg].x2 = (float)j + j_factor2;
					seg_neg[n_segments_neg].y2 = (float)i + 1.0F;
					seg_neg[n_segments_neg].level = level;

					n_segments_neg++;
				}
			}
		}
		return n_segments_neg;
	}

//-------------------------------------------------------------------------------------------------
//	Interpolated contour routines follow.  Because of the index arithmetic required, it is easier 
//	to separate routines for the interpolated case.  Also makes the non-interpolated case more
//  efficient
//-------------------------------------------------------------------------------------------------

	/**
	 * Scan for positive level contour segments, interpolated version
	 * @param mat
	 * @param int_fact
	 * @param p1
	 * @param p2
	 * @param p1_f1
	 * @param p2_f1
	 * @param level
	 * @param total
	 * @param c_level
	 * @param seg_pos
	 * @return number of segments found
	 * @author marat
	 */
	private static int scan_for_segments_interp(float[][] mat, float int_fact, int p1, int p2,
			int p1_f1, int p2_f1, int level, int total, float c_level,
			Segment[] seg_pos)
	{
		int i, j;
		int n_segments = total;

		float j_factor = 0.0F, i_factor = 0.0F,
				j_factor2 = 0.0F, i_factor2 = 0.0F;

		for (i = p1_f1; i < p2_f1 - 1; i++)
		{
			for (j = p1; j < p2 - 1; j++)
			{
				if (n_segments >= MAX_SEGMENTS - 1)
				{
					Util.show_and_log(
							"Maximum number of contour segments exceeded\nraise minimum level (floor)");
					return n_segments;
				}

				int ii = i - p1_f1;
				int jj = j - p1;

				if ((mat[ii][jj] > c_level
						&& mat[ii + 1][jj] < c_level
						&& mat[ii][jj + 1] < c_level
						&& mat[ii + 1][jj + 1] < c_level)
						||
						(mat[ii][jj] < c_level
								&& mat[ii + 1][jj] > c_level
								&& mat[ii][jj + 1] > c_level
								&& mat[ii + 1][jj + 1] > c_level))
				{
					if (mat[ii][jj] != mat[ii][jj + 1])
					{
						j_factor =
								(c_level - mat[ii][jj])
								/ (mat[ii][jj + 1] - mat[ii][jj]);
					}
					if (mat[ii][jj] != mat[ii + 1][jj])
					{
						i_factor =
								(c_level - mat[ii][jj])
								/ (mat[ii + 1][jj] - mat[ii][jj]);
					}

					seg_pos[n_segments] = new Segment();		// efficient???
					seg_pos[n_segments].x1 = ((float)j + j_factor) / int_fact;
					seg_pos[n_segments].y1 = ((float)i) / int_fact;
					seg_pos[n_segments].x2 = ((float)j) / int_fact;
					seg_pos[n_segments].y2 = ((float)i + i_factor) / int_fact;
					seg_pos[n_segments].level = level;

					n_segments++;
				}

				else if ((mat[ii][jj] < c_level
						&& mat[ii + 1][jj] > c_level
						&& mat[ii][jj + 1] < c_level
						&& mat[ii + 1][jj + 1] < c_level)
						||
						(mat[ii][jj] > c_level
								&& mat[ii + 1][jj] < c_level
								&& mat[ii][jj + 1] > c_level
								&& mat[ii + 1][jj + 1] > c_level))
				{

					if (mat[ii][jj] != mat[ii + 1][jj])
					{
						j_factor =
								(c_level - mat[ii + 1][jj])
								/ (mat[ii + 1][jj + 1] - mat[ii + 1][jj]);
					}
					if (mat[ii][jj] != mat[ii + 1][jj])
					{
						i_factor = 1.0F
								- (c_level - mat[ii + 1][jj])
								/ (mat[ii][jj] - mat[ii + 1][jj]);
					}

					seg_pos[n_segments] = new Segment();
					seg_pos[n_segments].x1 = ((float)j) / int_fact;
					seg_pos[n_segments].y1 = ((float)i + i_factor) / int_fact;
					seg_pos[n_segments].x2 = ((float)j + j_factor) / int_fact;
					seg_pos[n_segments].y2 = ((float)i + 1.0F) / int_fact;
					seg_pos[n_segments].level = level;

					n_segments++;
				}
				else if ((mat[ii][jj] < c_level
						&& mat[ii + 1][jj] < c_level
						&& mat[ii][jj + 1] < c_level
						&& mat[ii + 1][jj + 1] > c_level)
						||
						(mat[ii][jj] > c_level
								&& mat[ii + 1][jj] > c_level
								&& mat[ii][jj + 1] > c_level
								&& mat[ii + 1][jj + 1] < c_level))
				{
					if (mat[ii + 1][jj] != mat[ii + 1][jj + 1])
					{
						j_factor = 1.0F
								- (c_level - mat[ii + 1][jj + 1])
								/ (mat[ii + 1][jj] - mat[ii + 1][jj + 1]);
					}
					if (mat[ii][jj + 1] != mat[ii + 1][jj + 1])
					{
						i_factor = 1.0F
								- (c_level - mat[ii + 1][jj + 1])
								/ (mat[ii][jj + 1] - mat[ii + 1][jj + 1]);
					}

					seg_pos[n_segments] = new Segment();
					seg_pos[n_segments].x1 = ((float)j + 1.0F) / int_fact;
					seg_pos[n_segments].y1 = ((float)i + i_factor) / int_fact;
					seg_pos[n_segments].x2 = ((float)j + j_factor) / int_fact;
					seg_pos[n_segments].y2 = ((float)i + 1.0F) / int_fact;
					seg_pos[n_segments].level = level;

					n_segments++;
				}
				else if ((mat[ii][jj] < c_level
						&& mat[ii + 1][jj] < c_level
						&& mat[ii][jj + 1] > c_level
						&& mat[ii + 1][jj + 1] < c_level)
						||
						(mat[ii][jj] > c_level
								&& mat[ii + 1][jj] > c_level
								&& mat[ii][jj + 1] < c_level
								&& mat[ii + 1][jj + 1] > c_level))
				{
					if (mat[ii][jj] != mat[ii][jj + 1])
					{
						j_factor = 1.0F
								- (c_level - mat[ii][jj + 1])
								/ (mat[ii][jj] - mat[ii][jj + 1]);
					}
					if (mat[ii][jj + 1] != mat[ii + 1][jj + 1])
					{
						i_factor =
								(c_level - mat[ii][jj + 1])
								/ (mat[ii + 1][jj + 1] - mat[ii][jj + 1]);
					}

					seg_pos[n_segments] = new Segment();
					seg_pos[n_segments].x1 = ((float)j + j_factor) / int_fact;
					seg_pos[n_segments].y1 = ((float)i) / int_fact;
					seg_pos[n_segments].x2 = ((float)j + 1.0F) / int_fact;
					seg_pos[n_segments].y2 = ((float)i + i_factor) / int_fact;
					seg_pos[n_segments].level = level;

					n_segments++;
				}

				else if ((mat[ii][jj] > c_level
						&& mat[ii][jj + 1] > c_level
						&& mat[ii + 1][jj] < c_level
						&& mat[ii + 1][jj + 1] < c_level)
						||
						(mat[ii][jj] < c_level
								&& mat[ii][jj + 1] < c_level
								&& mat[ii + 1][jj] > c_level
								&& mat[ii + 1][jj + 1] > c_level))
				{
					if (mat[ii][jj] != mat[ii + 1][jj])
					{
						i_factor =
								(c_level - mat[ii][jj])
								/ (mat[ii + 1][jj] - mat[ii][jj]);
					}
					if (mat[ii][jj + 1] != mat[ii + 1][jj + 1])
					{
						i_factor2 =
								(c_level - mat[ii][jj + 1])
								/ (mat[ii + 1][jj + 1] - mat[ii][jj + 1]);
					}

					seg_pos[n_segments] = new Segment();
					seg_pos[n_segments].x1 = ((float)j) / int_fact;
					seg_pos[n_segments].y1 = ((float)i + i_factor) / int_fact;
					seg_pos[n_segments].x2 = ((float)j + 1.0F) / int_fact;
					seg_pos[n_segments].y2 = ((float)i + i_factor2) / int_fact;
					seg_pos[n_segments].level = level;

					n_segments++;
				}
				else if ((mat[ii][jj + 1] > c_level
						&& mat[ii + 1][jj + 1] > c_level
						&& mat[ii][jj] < c_level
						&& mat[ii + 1][jj] < c_level)
						||
						(mat[ii][jj + 1] < c_level
								&& mat[ii + 1][jj + 1] < c_level
								&& mat[ii][jj] > c_level
								&& mat[ii + 1][jj] > c_level))
				{
					if (mat[ii][jj] != mat[ii][jj + 1])
					{
						j_factor =
								(c_level - mat[ii][jj])
								/ (mat[ii][jj + 1] - mat[ii][jj]);
					}
					if (mat[ii + 1][jj] != mat[ii + 1][jj + 1])
					{
						j_factor2 =
								(c_level - mat[ii + 1][jj])
								/ (mat[ii + 1][jj + 1] - mat[ii + 1][jj]);
					}

					seg_pos[n_segments] = new Segment();
					seg_pos[n_segments].x1 = ((float)j + j_factor) / int_fact;
					seg_pos[n_segments].y1 = ((float)i) / int_fact;
					seg_pos[n_segments].x2 = ((float)j + j_factor2) / int_fact;
					seg_pos[n_segments].y2 = ((float)i + 1.0F) / int_fact;
					seg_pos[n_segments].level = level;

					n_segments++;
				}
			}
		}
		return n_segments;
	}

//-------------------------------------------------------------------------------------------------
	/**
	 * Scan for negative level contour segments, interpolated version
	 * @param mat
	 * @param int_fact
	 * @param p1
	 * @param p2
	 * @param p1_f1
	 * @param p2_f1
	 * @param level
	 * @param total
	 * @param c_level
	 * @param seg_neg
	 * @return the number of contour segments found
	 * @author marat
	 */
	private static int scan_for_segments_neg_interp(float[][] mat, float int_fact, int p1, int p2,
			int p1_f1, int p2_f1, int level, int total, float c_level,
			Segment[] seg_neg)
	{
		int i, j;
		int n_segments_neg = total;

		float	j_factor = 0.0F, i_factor = 0.0F,
				j_factor2 = 0.0F, i_factor2 = 0.0F;

		for (i = p1_f1; i < p2_f1 - 1; i++)
		{
			for (j = p1; j < p2 - 1; j++)
			{
				if (n_segments_neg >= MAX_SEGMENTS - 1)
				{
					Util.show_and_log(
							"Maximum number of negative contour segments exceeded\nraise minimum level (floor)");                       
					return n_segments_neg;
				}

				int ii = i - p1_f1;
				int jj = j - p1;

				if ((mat[ii][jj] > c_level
						&& mat[ii + 1][jj] < c_level
						&& mat[ii][jj + 1] < c_level
						&& mat[ii + 1][jj + 1] < c_level)
						||
						(mat[ii][jj] < c_level
								&& mat[ii + 1][jj] > c_level
								&& mat[ii][jj + 1] > c_level
								&& mat[ii + 1][jj + 1] > c_level))
				{
					if (mat[ii][jj] != mat[ii][jj + 1])
					{
						j_factor =
								(c_level - mat[ii][jj])
								/ (mat[ii][jj + 1] - mat[ii][jj]);
					}
					if (mat[ii][jj] != mat[ii + 1][jj])
					{
						i_factor =
								(c_level - mat[ii][jj])
								/ (mat[ii + 1][jj] - mat[ii][jj]);
					}

					seg_neg[n_segments_neg] = new Segment();
					seg_neg[n_segments_neg].x1 = ((float)j + j_factor) / int_fact;
					seg_neg[n_segments_neg].y1 = ((float)i) / int_fact;
					seg_neg[n_segments_neg].x2 = ((float)j) / int_fact;
					seg_neg[n_segments_neg].y2 = ((float)i + i_factor) / int_fact;
					seg_neg[n_segments_neg].level = level;

					n_segments_neg++;
				}

				else if ((mat[ii][jj] < c_level
						&& mat[ii + 1][jj] > c_level
						&& mat[ii][jj + 1] < c_level
						&& mat[ii + 1][jj + 1] < c_level)
						||
						(mat[ii][jj] > c_level
								&& mat[ii + 1][jj] < c_level
								&& mat[ii][jj + 1] > c_level
								&& mat[ii + 1][jj + 1] > c_level))
				{

					if (mat[ii][jj] != mat[ii + 1][jj])
					{
						j_factor =
								(c_level - mat[ii + 1][jj])
								/ (mat[ii + 1][jj + 1] - mat[ii + 1][jj]);
					}
					if (mat[ii][jj] != mat[ii + 1][jj])
					{
						i_factor = 1.0F
								- (c_level - mat[ii + 1][jj])
								/ (mat[ii][jj] - mat[ii + 1][jj]);
					}

					seg_neg[n_segments_neg] = new Segment();
					seg_neg[n_segments_neg].x1 = ((float)j) / int_fact;
					seg_neg[n_segments_neg].y1 = ((float)i + i_factor) / int_fact;
					seg_neg[n_segments_neg].x2 = ((float)j + j_factor) / int_fact;
					seg_neg[n_segments_neg].y2 = ((float)i + 1.0F) / int_fact;
					seg_neg[n_segments_neg].level = level;

					n_segments_neg++;
				}
				else if ((mat[ii][jj] < c_level
						&& mat[ii + 1][jj] < c_level
						&& mat[ii][jj + 1] < c_level
						&& mat[ii + 1][jj + 1] > c_level)
						||
						(mat[ii][jj] > c_level
								&& mat[ii + 1][jj] > c_level
								&& mat[ii][jj + 1] > c_level
								&& mat[ii + 1][jj + 1] < c_level))
				{
					if (mat[ii + 1][jj] != mat[ii + 1][jj + 1])
					{
						j_factor = 1.0F
								- (c_level - mat[ii + 1][jj + 1])
								/ (mat[ii + 1][jj] - mat[ii + 1][jj + 1]);
					}
					if (mat[ii][jj + 1] != mat[ii + 1][jj + 1])
					{
						i_factor = 1.0F
								- (c_level - mat[ii + 1][jj + 1])
								/ (mat[ii][jj + 1] - mat[ii + 1][jj + 1]);
					}

					seg_neg[n_segments_neg] = new Segment();
					seg_neg[n_segments_neg].x1 = ((float)j + 1.0F) / int_fact;
					seg_neg[n_segments_neg].y1 = ((float)i + i_factor) / int_fact;
					seg_neg[n_segments_neg].x2 = ((float)j + j_factor) / int_fact;
					seg_neg[n_segments_neg].y2 = ((float)i + 1.0F) / int_fact;
					seg_neg[n_segments_neg].level = level;

					n_segments_neg++;
				}
				else if ((mat[ii][jj] < c_level
						&& mat[ii + 1][jj] < c_level
						&& mat[ii][jj + 1] > c_level
						&& mat[ii + 1][jj + 1] < c_level)
						||
						(mat[ii][jj] > c_level
								&& mat[ii + 1][jj] > c_level
								&& mat[ii][jj + 1] < c_level
								&& mat[ii + 1][jj + 1] > c_level))
				{
					if (mat[ii][jj] != mat[ii][jj + 1])
					{
						j_factor = 1.0F
								- (c_level - mat[ii][jj + 1])
								/ (mat[ii][jj] - mat[ii][jj + 1]);
					}
					if (mat[ii][jj + 1] != mat[ii + 1][jj + 1])
					{
						i_factor =
								(c_level - mat[ii][jj + 1])
								/ (mat[ii + 1][jj + 1] - mat[ii][jj + 1]);
					}

					seg_neg[n_segments_neg] = new Segment();
					seg_neg[n_segments_neg].x1 = ((float)j + j_factor) / int_fact;
					seg_neg[n_segments_neg].y1 = ((float)i) / int_fact;
					seg_neg[n_segments_neg].x2 = ((float)j + 1.0F) / int_fact;
					seg_neg[n_segments_neg].y2 = ((float)i + i_factor) / int_fact;
					seg_neg[n_segments_neg].level = level;

					n_segments_neg++;
				}
				else if ((mat[ii][jj] > c_level
						&& mat[ii][jj + 1] > c_level
						&& mat[ii + 1][jj] < c_level
						&& mat[ii + 1][jj + 1] < c_level)
						||
						(mat[ii][jj] < c_level
								&& mat[ii][jj + 1] < c_level
								&& mat[ii + 1][jj] > c_level
								&& mat[ii + 1][jj + 1] > c_level))
				{
					if (mat[ii][jj] != mat[ii + 1][jj])
					{
						i_factor =
								(c_level - mat[ii][jj])
								/ (mat[ii + 1][jj] - mat[ii][jj]);
					}
					if (mat[ii][jj + 1] != mat[ii + 1][jj + 1])
					{
						i_factor2 =
								(c_level - mat[ii][jj + 1])
								/ (mat[ii + 1][jj + 1] - mat[ii][jj + 1]);
					}

					seg_neg[n_segments_neg] = new Segment();
					seg_neg[n_segments_neg].x1 = ((float)j) / int_fact;
					seg_neg[n_segments_neg].y1 = ((float)i + i_factor) / int_fact;
					seg_neg[n_segments_neg].x2 = ((float)j + 1.0F) / int_fact;
					seg_neg[n_segments_neg].y2 = ((float)i + i_factor2) / int_fact;
					seg_neg[n_segments_neg].level = level;

					n_segments_neg++;
				}
				else if ((mat[ii][jj + 1] > c_level
						&& mat[ii + 1][jj + 1] > c_level
						&& mat[ii][jj] < c_level
						&& mat[ii + 1][jj] < c_level)
						||
						(mat[ii][jj + 1] < c_level
								&& mat[ii + 1][jj + 1] < c_level
								&& mat[ii][jj] > c_level
								&& mat[ii + 1][jj] > c_level))
				{
					if (mat[ii][jj] != mat[ii][jj + 1])
					{
						j_factor =
								(c_level - mat[ii][jj])
								/ (mat[ii][jj + 1] - mat[ii][jj]);
					}
					if (mat[ii + 1][jj] != mat[ii + 1][jj + 1])
					{
						j_factor2 =
								(c_level - mat[ii + 1][jj])
								/ (mat[ii + 1][jj + 1] - mat[ii + 1][jj]);
					}

					seg_neg[n_segments_neg] = new Segment();
					seg_neg[n_segments_neg].x1 = ((float)j + j_factor) / int_fact;
					seg_neg[n_segments_neg].y1 = ((float)i) / int_fact;
					seg_neg[n_segments_neg].x2 = ((float)j + j_factor2) / int_fact;
					seg_neg[n_segments_neg].y2 = ((float)i + 1.0F) / int_fact;
					seg_neg[n_segments_neg].level = level;

					n_segments_neg++;
				}
			}
		}
		return n_segments_neg;
	}

//-------------------------------------------------------------------------------------------------
}
