
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;

import spinworks.SW_main.det_modes;
import spinworks.SW_main.windows;
import spinworks.SW_main.modes;

public class DX_Util 
{
	// TODO  There are just too many functions here! some should be combined!!! 
	//       eg combine rows and columns with projections.
	
	// TODO change to enumeration
	protected final static int DATA_REAL = 0;		// current read mode, either parameters, real data
	protected final static int DATA_IMAG = 1;		// or imaginary data
	protected final static int PARAM = 2;

	protected final static int SPECT = 3;			// type of data set, the two options are 
	protected final static int SPECT_RI = 4;        // SPECT for spectra
	protected final static int FID = 5;				// and FID for FID data

    public enum comp_modes { FIX, SQZ, DIFF_DUP };

    
    public static int read_jdx_pars(String fileName, global_par gp, data_par p, data_par q)
    {

        int type = -1;						    // default is spectra
        String instring;
        String delim = "[ =,\t]+";
        CharSequence[] sqz_char = new CharSequence[] {"@", "A", "B", "C", "D", "E", "F", "G", "H", "I",
                "a", "b", "c", "d", "e", "f","g", "h", "i"};
       
        CharSequence[] dif_char = new CharSequence[] {"%", "J", "K", "L", "M", "N", "O", "P", "Q", "R",
                "j", "k", "l", "m", "n", "o","p", "q", "r"};

        //char[] dup_char = new char[] { 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 's' };	// used?

        String[] token;

        double firstx = 5000.0, lastx = 0.0;		// local parameters
        double sf = 0.0, sw = 0.0;
        
        int npoints = 0, size = 0;
        boolean has_sf = false;                    // true if Bruker specific referencing 
        boolean has_sw = false;                    // found

        int var_dim_x = 0, var_dim_r = 0, var_dim_i = 0;
        double factor_x = 0.0002;

        if(gp.log_on) Util.log_error("*** JCAMP DX File :" + fileName);

        // try to open the jdx parameter file 
        BufferedReader br;
        try { br = new BufferedReader(new FileReader(fileName)); }
        catch (Exception ex)
        {
            Util.show_and_log("read_jdx_pars(), cannot open: " + fileName, ex.getMessage());
            return -1;
        }
        
        File fi = new File(fileName);
        q.data_folder = fi.getParent() + "/";
        //System.out.println(q.data_folder);

        //	start off assuming 1D processed data
        gp.display_mode = modes.MODE_1D;

        for(;;)
        {
        	try {instring = br.readLine();} 
            catch (IOException ex) 
            {
    			ex.printStackTrace();
    			Util.show_and_log("read_jdx_pars(), read error: " + fileName, ex.getMessage());
    			try { br.close(); } 
    			catch (IOException e) { e.printStackTrace(); }
    			return -1;
    		}
        	
        	//{0}
        	if(instring == null) break;
        	
            if (instring.length() > 3)
            {
                token = instring.split(delim);
                if (token[0].equals("##END")) break;		// end of file
                
                try
                {
                    if (token[0].equals("##.OBSERVE")
                        && token[1].equals("FREQUENCY"))
                    {
                        q.SFO1 = Double.parseDouble(token[2]); p.SFO1 = q.SFO1;
                        if (gp.log_on) Util.log_error(String.format("SFO1 = %f", q.SFO1));
                        if (q.SFO1 > 10000.0 || q.SFO1 < 0.1)
                            throw new Exception("spectrometer freq. out of range");
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

               
                if (token[0].equals("##.OBSERVE")
                		&& token[1].equals("NUCLEUS") && token.length > 2)
                {
                	if (gp.log_on) Util.log_error("NUCLEUS = " + token[2]);
                	q.NUC1 = token[2];
                }
               
                try
                {
                    if (token[0].equals("##.AVERAGE"))
                    {
                        if(gp.log_on)
                            Util.log_error("NS = " + token[1]);
                        q.NS = Integer.parseInt(token[1]);
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##NPOINTS"))
                    {
                        if(gp.log_on)
                            Util.log_error("NPOINTS = " + token[1]);
                        npoints = Integer.parseInt(token[1]);

                        if (npoints < 2 || npoints > 1000000)
                            throw new Exception("NPOINTS out of range");

                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##$SI"))
                    {
                        size = Integer.parseInt(token[1]);
                        if (gp.log_on) Util.log_error("SI= " + token[1]);
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##FIRST")
                        || token[0].equals("##FIRSTX"))
                    {
                        if (gp.log_on) Util.log_error("FIRSTX = " + token[1]);
                        firstx = Double.parseDouble(token[1]);
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##LAST")
                        || token[0].equals("##LASTX"))
                    {
                        if (gp.log_on) Util.log_error("LASTX = " + token[1]);
                        lastx = Double.parseDouble(token[1]);
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##$SF"))
                    {
                        if (gp.log_on) Util.log_error("SF = " + token[1]);
                        sf = Double.parseDouble(token[1]);
                        if (sf > 10000.0 || sf < 0.1)
                            throw new Exception("sf out of range");
                        has_sf = true;          // valid Bruker SF referencing parameter
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##$SW"))      // Bruker Specific
                    {
                        if (gp.log_on) Util.log_error("SW =  " + token[1] + " ppm");
                        sw = Double.parseDouble(token[1]);
                        has_sw = true;
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##$SWEEP")    // Nanalysis specific
                        && token[1].equals("WIDTH"))
                    {
                        if (gp.log_on) Util.log_error("SWEEP WIDTH ppm: " +  token[2]);
                        sw = Double.parseDouble(token[2]);
                        has_sw = true;
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##$OFFSET"))
                    {
                        if (gp.log_on) Util.log_error("OFFSET = " + token[1]);
                        q.OFFSET = Double.parseDouble(token[1]); p.OFFSET = q.OFFSET;
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

               
                if (token[0].equals("##$PULPROG"))
                {
                	if (gp.log_on) Util.log_error("PULPROG = " + token[1]);
                	q.pulse_sequence = token[1];
                }
   

                try
                {
                    if (token[0].equals("##$TD"))
                    {
                        if (gp.log_on) Util.log_error("TD = " + token[1]);
                        q.TD = Integer.parseInt(token[1]);
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }
                
                try
                {
                    if (token[0].equals("##$REVERSE"))
                    {
                        if(token[1].equalsIgnoreCase("yes"))
                        	SW_main.p.REVERSE = true;
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##VAR_DIM"))
                    {                           
                        var_dim_x = Integer.parseInt(token[1]);
                        var_dim_r = Integer.parseInt(token[2]);
                        if(token[3] != null)
                            var_dim_i = Integer.parseInt(token[3]);
                        if (gp.log_on) Util.log_error(String.format("VAR_DIM =  = %d  %d  %d",
                            var_dim_x, var_dim_r, var_dim_i));
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log(instring, ex.getMessage());
                }

                try
                {
                    if (token[0].equals("##FACTOR"))
                    {
                        if (gp.log_on) Util.log_error("FACTOR = " + token[1]);
                        factor_x = Double.parseDouble(token[1]);
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##$DECIM"))
                    {
                        if (gp.log_on) Util.log_error("DECIM = " + token[1]);
                        q.DECIM = Integer.parseInt(token[1]);
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##$DSPFVS"))
                    {
                        if (gp.log_on) Util.log_error("DECIM = " + token[1]);
                        q.DSPFVS = Integer.parseInt(token[1]);
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##$GRPDLY"))
                    {
                        if (gp.log_on) Util.log_error("GRPDLY = " + token[1]);
                        q.GRPDLY = Double.parseDouble(token[1]);
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                
                if (token[0].equals("##TITLE"))
                {
                	if (gp.log_on) Util.log_error("TITLE = " + token[1]);
                	gp.plot_title = instring.substring(8);
                }
                

                try
                {
                    if (token[0].equals("##$WDW"))					// window function type
                    {
                        if (gp.log_on) Util.log_error("WDW = " + token[1]);
                        int temp = Integer.parseInt(token[1]);                       
                        p.WDW = windows.values()[temp];	q.WDW = p.WDW;	// TODO check whether q.WDW should be NONE for FIDs
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##$LB"))
                    {
                        if (gp.log_on) Util.log_error("LB = " + token[1]);
                        p.LB = Double.parseDouble(token[1]);
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                try
                {
                    if (token[0].equals("##$GB"))
                    {
                        if (gp.log_on) Util.log_error("GB = " + token[1]);
                        p.GB = Double.parseDouble(token[1]);
                    }
                }
                catch (Exception ex)
                {
                    Util.show_and_log("read_jdx_pars(): " + instring, ex.getMessage());
                    try {	br.close();	} 
                    catch (IOException e) {	e.printStackTrace();	}
                    return -2;
                }

                //	Determine type, either the data set is an FID or  an FID
                
                if (token[0].equals("##NTUPLES"))
                	for (int j = 1; j < token.length; j++)
                		if (token[j].equals("FID"))
                		{
                			type = FID;			// FID data
                			if (gp.log_on) Util.log_error("type is FID");
                		}

                //	Determine type, either the data set is an FID or a spectrum;  
                //  determin whether the data is an FID (alternate method)
                if (token[0].equals("##DATA"))
                	for (int j = 1; j < token.length; j++)
                		if (token[j].equals("FID"))
                		{
                			type = FID;			// FID data
                			if (gp.log_on) Util.log_error("type is FID (alternate)");
                		}                     
                

                // determine whether the data is a SPECTRUM
                if (token[0].equals("##NTUPLES"))
                	for (int j = 1; j < token.length; j++)
                		if (token[j].equals("SPECTRUM")
                				|| token[j].equals("Spectrum"))
                		{
                			type = SPECT;			// processed spectrum
                			if (gp.log_on) Util.log_error("type is SPECT");
                		}
                            
             	// determine whether the data is a SPECTRUM (alternate)           
                if (token[0].equals("##DATA"))
                	for (int j = 1; j < token.length; j++)
                		if (token[j].equals("SPECTRUM")
                				|| token[j].equals("Spectrum"))
                		{
                			type = SPECT;			// processed spectrum
                			if (gp.log_on) Util.log_error("type is SPECT (alternate)");
                		}                
               
            }	// end if(instring.length > 3)
        }		// end while


        if (type == SPECT)
        {
            if (var_dim_r != 0 && (var_dim_r != size))
            {
                q.SI = var_dim_r;
                if (gp.log_on) Util.log_error("changing SI to var_dim_r");
            }
            else
            {
                if(size != 0)
                    q.SI = size;
                else
                    q.SI = npoints;
            }

            p.SI = q.SI;
            q.TD = q.SI;  // should be overwritten by Bruker specific parameter.

            if (q.SI > 2)
            {
                q.hzperpt = (firstx - lastx) / (q.SI);
            }

            if (!has_sw)
            {
                if (q.SFO1 > 0.1)
                    q.SW = (firstx - lastx) / q.SFO1;
 
                q.SW = Math.abs(q.SW);
            }
            else
                q.SW = sw;


            if (has_sf) // valid Bruker referencing parameter
            {
                q.SF = sf; p.SF = q.SF;
                q.offset = (q.SFO1 - p.SF) * 1.0e06 + (q.SW * q.SFO1) / 2.0;
            }
            else
            {
                if (firstx < lastx)
                    q.offset = lastx;
                else
                    q.offset = firstx;

                q.SF = q.SFO1 + ((q.SFO1 * q.SW) / 2.0 - q.offset) / 1000000.0;
                p.SF = q.SF;
            }
        }

        else if (type == FID)			// FID
        {
            if(var_dim_r != 0 && (2 * var_dim_r != q.TD))
            {
                q.TD = 2 * var_dim_r;
                if (gp.log_on) Util.log_error("changing TD to 2* var_dim_r");
            }

            if (!has_sw)        // no Bruker/Nanalysis specific SW parameter
            {
                q.SW = (1.0 / factor_x) / q.SFO1;
            }

            if (has_sf)
            {
                q.SF = sf; p.SF = q.SF;
                q.SW = sw; 
                q.hzperpt = q.SW * q.SFO1 / (double)(q.SI);
                q.offset = (q.SFO1 - p.SF) * 1.0e06 + (q.SW * q.SFO1) / 2.0;
            }
            else
            {
                // ???
            }           
        }

        else
        {
            Util.show_and_log("Unknown JCAMP-DX data type", "SpinWorks 4");
            return -1;
        }


//perhaps split out into separate function here
        q.jcamp_dx_comp = comp_modes.FIX;

        /*sr.Close();
        sr = new StreamReader(fileName);    // reset file to beginning */
        
        // reset file to beginning
        try {	br.close();	} 
        catch (IOException ex) {	ex.printStackTrace();	}
        
        try { br = new BufferedReader(new FileReader(fileName)); }
        catch (Exception ex)
        {
            Util.show_and_log("get_par(), cannot open: " + fileName, ex.getMessage());
            return -1;
        }
        

        for(;;)          // move to start of real data
        {
        	try {instring = br.readLine();} 
            catch (IOException ex) 
            {
    			ex.printStackTrace();
    			Util.show_and_log("read_jdx_pars(), read error: " + fileName, ex.getMessage());
    			try { br.close(); } 
    			catch (IOException e) { e.printStackTrace(); }
    			return -1;
    		}
        	if(instring == null) break;
        	
            if (instring.length() > 3)
            {
                token = instring.split(delim);
                try
                {
                    if (token.length > 2)
                        if (token[0].equals("##DATA")
                            && token[1].equals("TABLE")
                            && token[2].equals("(X++(R..R))"))
                        {
                            break;
                        }

                    if (token.length > 1)
                        if (token[0].equals("##XYDATA")        // modify this a bit??
                            && token[1].equals("(X++(Y..Y))"))
                        {
                            break;
                        }
                }
                catch (Exception ex)
                {
                    Util.show_and_log(instring, ex.getMessage());
                    break;
                }
            }
        }

        boolean is_sqz = false, is_dif = false;
        int i = 0;

        try 
        {
			while ((instring = br.readLine()) != null           // scan the points for SQZ
			    && i < 3 )
			{
			
			    for (int j = 0; j < sqz_char.length; j++)
			    {           	
			        if (instring.contains(sqz_char[j]))
			        {
			            is_sqz = true;
			            break;
			        }
			    }
			    if (is_sqz) break;
			    i++;
			}
		} 
        catch (IOException ex) 
        {
			ex.printStackTrace();
			return -1;
		}

        i = 0;
        
        try 
        {
			while ((instring = br.readLine()) != null           // scan the points for DIF
			    && i < 3)
			{
			    
			    for (int j = 0; j < dif_char.length; j++)
			    {
			        if (instring.contains(dif_char[j]))
			        {
			            is_dif = true;
			            break;
			        }
			    }
			    if (is_dif) break;
			    i++;
			}
		} 
        catch (IOException ex) 
        {
			ex.printStackTrace();
			return -1;
		}

        if (gp.log_on) Util.log_error(String.format("sqz = %b   dif = %b", is_sqz, is_dif));

        if (is_sqz && !is_dif)
            q.jcamp_dx_comp = comp_modes.SQZ;
        else if (is_dif)
            q.jcamp_dx_comp = comp_modes.DIFF_DUP;
     

        //	set detection mode based on DSP settings.  Sequential sampling not supported!
        if (q.DECIM != 0 && q.DSPFVS != 0)
            q.aq_mode = det_modes.DSP;
        else
            q.aq_mode = det_modes.SIM;

        if (gp.log_on)
        {
            Util.log_error(String.format("SF = %f   offset = %f   SW = %f",
                q.SF, q.offset, q.SW));

            Util.log_error(String.format("SI = %d   hzperpt = %f   SFO1 = %f",
                q.SI, q.hzperpt, q.SFO1));
        }

        try {	br.close();	} 
        catch (IOException ex) {	ex.printStackTrace();	}
        
        return type;
    }
    
//-------------------------------------------------------------------------------------------------
    public static int read_jdx_fid(String fileName, global_par gp, data_par p, data_par q)
    {
        // a possible last point problem??
        int points_read = 0;
        String instring;
        //char[] delim = new char[] { ' ', '=', ',', '\t' };
        String delim = "[ =,\t]+";
        String[] token;
        int i, j;
        
        BufferedReader br;
        try { br = new BufferedReader(new FileReader(fileName)); }
        catch (Exception ex)
        {
            Util.show_and_log("read_jdx_fid(), cannot open: " + fileName, ex.getMessage());
            return -1;
        }

        if (q.SI < q.TD / 2) q.SI = q.TD / 2;
        p.SI = q.SI;

        Util.alloc_proc(gp, p, q);

        try 
        {
			while ((instring = br.readLine()) != null)          // move to start of real data
			{
			    if (instring.length() > 3)
			    {
			        token = instring.split(delim);
			        try			
			        {
			            if(token.length > 2)
			                if (token[0].equals("##DATA")
			                    && token[1].equals("TABLE")
			                    && token[2].equals("(X++(R..R))"))
			                {
			                    if (gp.log_on) Util.log_error("real fid data found");
			                    break;
			                }                       
			        }
			        catch (Exception ex)
			        {   Util.show_and_log(instring, ex.getMessage());  }
			    }
			}
		} 
        catch (IOException e1) 
        {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}           

        i = 0;
        try 
        {
			while ((instring = br.readLine()) != null           // read the real fid points
			    && i < q.TD/2)
			{		
			    if (instring.length() > 3)
			    {
			        if (q.jcamp_dx_comp == comp_modes.SQZ)
			            instring = un_squeeze(instring);
			        else if (q.jcamp_dx_comp == comp_modes.DIFF_DUP)
			            instring = un_diff(instring);		       

			        instring = instring.trim();
			        token = instring.split(delim);
			        
			        /*System.out.println(instring);
			        System.out.println(String.format("%d \n", token.length));*/
			        
			        if (token.length <= 2) break;

			        for (j = 1; j < token.length && i < q.TD/2; j++)
			        {

			            try { gp.data_array[i] = Double.parseDouble(token[j]); }
			            catch (Exception ex)
			            {
			                Util.show_and_log("converting real FID data points caused the following exception:",
			                		ex.getMessage());
			                Util.show_and_log("input was: ", instring);
			                Util.show_and_log("JDX read aborted");
			                Util.log_error(instring + ex.getMessage());
			                return -1;
			            }
			            i++;
			            points_read++;
			        }
			    }
			}
		} 
        catch (IOException e1) 
        {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

        if (gp.log_on) Util.log_error(String.format("  real points found = %d", i));

        try {	br.close();	} 
        catch (IOException e) { e.printStackTrace(); }       
        
        try { br = new BufferedReader(new FileReader(fileName)); }
        catch (Exception ex)
        {
            Util.show_and_log("read_jdx_fid(), cannot open: " + fileName, ex.getMessage());
            return -1;
        }

        try 
        {
			while ((instring = br.readLine()) != null)          // move to start of imag data
			{				
			    if (instring.length() > 3)
			    {
			        token = instring.split(delim);
			        try			
			        {
			            if (token.length > 2)
			                if (token[0].equals("##DATA")
			                    && token[1].equals("TABLE")
			                    && token[2].equals("(X++(I..I))"))
			                {
			                    if (gp.log_on) Util.log_error("imag fid data found");
			                    break;
			                }
			        }
			        catch (Exception ex)
			        {
			        	Util.show_and_log(instring, ex.getMessage());
			        }
			    }
			}
		} catch (IOException e1) 
        {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}


        i = 0;
        try 
        {
			while ((instring = br.readLine()) != null           // read imag fid points
			    && i < q.TD / 2)
			{			
			    if (instring.length() > 3)
			    {
			        if (q.jcamp_dx_comp == comp_modes.SQZ)
			            instring = un_squeeze(instring);
			        else if (q.jcamp_dx_comp == comp_modes.DIFF_DUP)
			            instring = un_diff(instring);

			        instring = instring.trim();
			        token = instring.split(delim);
			        if (token.length <= 2) break;

			        for (j = 1; j < token.length && i < q.TD/2; j++)
			        {
			            try { gp.i_data_array[i] = Double.parseDouble(token[j]); }
			            catch (Exception ex)
			            {
			            	Util.show_and_log("converting imag FID data points caused the following exception:", 
			                	ex.getMessage());
			                Util.show_and_log("input was: ", instring);
			                Util.show_and_log("JDX read aborted");
			                return -1;
			            }
			            i++;
			            points_read++;
			        }
			    }
			}
		} 
        catch (IOException e1) 
        {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

        if (gp.log_on) Util.log_error(String.format("  imag points found = %d", i));

        try {	br.close();	} 
        catch (IOException ex) {	ex.printStackTrace();	}

        try { gp.fid = new double[q.TD]; }               // allocate fid space
        catch (Exception ex)
        {
            Util.show_and_log("read_jdx(): cannot allocate space for fid", ex.getMessage());
            try {	br.close();	} 
            catch (IOException e) {	e.printStackTrace();	}
            return -1;
        }

        if(Math.abs(p.LB) < 0.0001) p.LB = 1.0;			// default reasonable LB parameter
        
        gp.display_mode = modes.MODE_FID;  				// should this be here???
                                                        // or in calling function???
        for (j = 0; j < q.TD; j = j + 2)                // transfer data to fid array
        {                                               // reals and imag alternate
            gp.fid[j] = gp.data_array[j / 2];
            gp.fid[j + 1] = gp.i_data_array[j / 2];
        }
   
        return points_read;
    }
    
//-------------------------------------------------------------------------------------------------
    public static int read_jdx_spect(String fileName, global_par gp, data_par p, data_par q)
    {
        // a possible last point problem??
        int points_read = 0;
        String instring;
        //char[] delim = new char[] { ' ', '=', ',', '\t' };
        String delim = "[ =,\t]+";
        String[] token;
        int i, j;
        boolean has_imag = false;

        // try to open the jdx file 
        BufferedReader br;
        try { br = new BufferedReader(new FileReader(fileName)); }
        catch (Exception ex)
        {
            Util.show_and_log("read_jdx_spect(), cannot open: " + fileName, ex.getMessage());
            return -1;
        }
        
        Util.alloc_proc(gp, p, q);

        try 
        {
			while ((instring = br.readLine()) != null)          // move to start of real data
			{
			    if (instring.length() > 3)
			    {
			    	instring = instring.trim();
			    	token = instring.split(delim);
			        try			
			        {
			            if (token.length > 2)
			                if (token[0].equals("##DATA")
			                    && token[1].equals("TABLE")
			                    && token[2].equals("(X++(R..R))"))
			                {
			                    if (gp.log_on) Util.log_error("##DATA TABLE real spect data found");
			                    break;
			                }

			            if (token.length > 1)
			                if (token[0].equals("##XYDATA")        // modify this a bit??
			                    && (token[1].equals("(X++(Y..Y))") || token[1].equals("(X++(R..R))")))
			                {
			                    if (gp.log_on) Util.log_error("##XYDATA real spect data found");
			                    break;
			                }
			        }
			        catch (Exception ex)
			        {
			            Util.show_and_log(instring, ex.getMessage());
			        }
			    }
			}
		} 
        catch (IOException e) 
        {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

        if (q.hzperpt < 0.0)
            i = q.SI - 1;
        else
            i = 0;

        try 
        {
			while ((instring = br.readLine()) != null           // read the real spect points
			    && i < q.SI && i >= 0)
			{								
			    if (instring.length() > 3)
			    {
			        if (q.jcamp_dx_comp == comp_modes.SQZ)
			            instring = un_squeeze(instring);
			        else if (q.jcamp_dx_comp == comp_modes.DIFF_DUP)
			            instring = un_diff(instring);

			        instring = instring.trim();
			        token = instring.split(delim);
			        
			        if (token.length <= 2) break;

			        if (q.hzperpt < 0.0) // data are in reverse order!
			        {
			            for (j = 1; j < token.length; j++)
			            {

			                try { gp.data_array[i] = Double.parseDouble(token[j]); }
			                catch (Exception ex)
			                {
			                    Util.show_and_log(
			                        "converting real SPECT data points (reverse order) caused the following exception: ",
			                        ex.getMessage());
			                    return -1;
			                }
			                i--;
			                points_read++;
			            }                   
			        }
			        else  // "normal" order // TODO  is this right???
			        {
			            for (j = 1; j < token.length; j++)
			            {

			                try 
			                { 
			                	gp.data_array[i] = Double.parseDouble(token[j]); 
			                }			                
			                catch (Exception ex)
			                {
			                	System.out.println(String.format("q.SI = %d    i = %d", q.SI, i));
			                    Util.show_and_log("converting real SPECT data points (normal order) caused the following exception: ",
			                        ex.getMessage());
			                    return -1;
			                }
			                i++;
			                points_read++;
			            }
			        }
			    }
			}
		} 
        catch (IOException e) 
        {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

        q.hzperpt = Math.abs(q.hzperpt);

        if(gp.log_on) Util.log_error(String.format("  real points found = %d", i));

        try {	br.close();	} 
        catch (IOException e) {	e.printStackTrace();	}
        
        try {	br = new BufferedReader(new FileReader(fileName));	} 
        catch (FileNotFoundException e) {	e.printStackTrace();	}  // reset file to beginning
        															   // is there a better way??

        try 
        {
			while ((instring = br.readLine()) != null)          // move to start of imag data
			{
				
			    if (instring.length() > 3)
			    {
			    	 /*if (q.jcamp_dx_comp == comp_modes.SQZ)
				            instring = un_squeeze(instring);
				        else if (q.jcamp_dx_comp == comp_modes.DIFF_DUP)
				            instring = un_diff(instring);*/
			    	 
			    	instring = instring.trim();	
			    	token = instring.split(delim);
			        
			        try		
			        {
			            if (token.length > 2)
			                if (token[0].equals("##DATA")
			                    && token[1].equals("TABLE")
			                    && token[2].equals("(X++(I..I))"))
			                {
			                    if (gp.log_on) Util.log_error("##DATA TABLE imag. spect data found");
			                    has_imag = true;
			                    break;
			                }

			            if (token.length > 1)
			                if (token[0].equals("##XYDATA")        // modify this a bit??
			                    &&  token[1].equals("(X++(I..RI))"))
			                {
			                    if (gp.log_on) Util.log_error("##XYDATA imag. spect data found");
			                    has_imag = true;
			                    break;
			                }
			        }
			        catch (Exception ex)
			        {
			            Util.show_and_log(instring, ex.getMessage());
			        }
			    }
			}
		} 
        catch (IOException e) 
        {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

        if (!has_imag) gp.i_data_array = null;              // no imaginary data

        i = 0;
        try 
        {
			while ((instring = br.readLine()) != null           // read imag spect. points
			    && i < q.SI  && gp.i_data_array != null)
			{
			    if (instring.length() > 3)
			    {
			        if (q.jcamp_dx_comp == comp_modes.SQZ)
			            instring = un_squeeze(instring);
			        else if (q.jcamp_dx_comp == comp_modes.DIFF_DUP)
			            instring = un_diff(instring);

			        instring = instring.trim();
			        token = instring.split(delim);
			        if (token.length <= 2) break;

			        for (j = 1; j < token.length; j++)
			        {
			            try { gp.i_data_array[i] = Double.parseDouble(token[j]); }
			            catch (Exception ex)
			            {
			                Util.show_and_log(
			                    "converting imag spect data points caused the following exception: ",
			                    ex.getMessage());
			                return -1;
			            }
			            i++;
			            points_read++;
			        }
			    }
			}
		} 
        catch (IOException e) 
        {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

        if(gp.log_on) Util.log_error(String.format("  imag points found = %d", i));

        try {	br.close();	} 
        catch (IOException e) {	e.printStackTrace();	}

        return points_read;
    }

//-------------------------------------------------------------------------------------------------
// Read a JCAMP DX file into the F2 trace for 2D
// The F1 and F2 routines should be collapsed into one routine

 		public static int read_jdx_f2(String fileName, global_par gp)
 		{
 			int mode = PARAM;
 			boolean space_allocated = false;
 			String instring;
 			String delim = "[ =,\t]+";
 			String[] token;
 			double firstx = 5000.0, lastx = 0.0 /*, SF = 500.13 */;
 			int i = 0;
 			gp.size_f2_trace = 0;

 		// try to open the jdx parameter file 
 	        BufferedReader br;
 	        try { br = new BufferedReader(new FileReader(fileName)); }
 	        catch (Exception ex)
 	        {
 	            Util.show_and_log("read_jdx_f2(), cannot open: " + fileName, ex.getMessage());
 	            return -1;
 	        }
 	        			
 			for(;;)
 			{
 				try {instring = br.readLine();} 
 	            catch (IOException ex) 
 	            {
 	    			ex.printStackTrace();
 	    			Util.show_and_log("read_jdx_F2(), read error: " + fileName, ex.getMessage());
 	    			try { br.close(); } 
 	    			catch (IOException e) { e.printStackTrace(); }
 	    			return -1;
 	    		}
 				
 				instring = instring.trim();
 				
 				if(instring == null) break;
 				
 				if (instring.length() > 3)
 				{
 					token = instring.split(delim);
 	                if (token[0].equals("##END")) break;		// end of file

 					if(mode == PARAM)							// reading parameters
 					{

 						try
 						{
 							if (token[0].equals("##NPOINTS"))
 							{
 								gp.size_f2_trace = Integer.parseInt(token[1]); 
 							}
 						}
 						catch (Exception ex)
 						{
                             Util.show_and_log(instring + ex.getMessage());
                        }


 						try
 						{
 							if (token[0].equals("##FIRSTX"))
 							{
 								firstx = Double.parseDouble(token[1]);
 							}
 						}
 						catch (Exception ex)
 						{ 
                             Util.show_and_log(instring + ex.getMessage());
 					    }

 						try
 						{
 							if (token[0].equals("##LASTX"))
 							{
                                 lastx = Double.parseDouble(token[1]);
 							}
 						}
 						catch (Exception ex)
 						{ 
                             Util.show_and_log(instring, ex.getMessage());
                        }

 						try
 						{
 							if (token[0].equals("##$SF"))
 							{
                                 //SF = Double.parseDouble(token[1]); 
 							}
 						}
 						catch (Exception ex)
 						{ 
                             Util.show_and_log(instring + ex.getMessage());
 					    }

 					}
 					else if (mode == DATA_REAL && gp.size_f2_trace  > 0)		// finished parameters, now read data
 	                {
 						if (space_allocated == false)
 						{
 							gp.f2_trace = new double[gp.size_f2_trace];
 							space_allocated = true;
 						}

 	                    if(i + 4 > gp.size_f2_trace)			// check that we don't go over
 	                    {										// array end
 	                        Util.show_and_log(String.format("read_jdx_f2(): size = , but points read = ",
 	                            gp.size_f2_trace, i));
 	                        break;
 	                    }
 						try
 						{
                             gp.f2_trace[i] = Double.parseDouble(token[1]);
                             gp.f2_trace[i + 1] = Double.parseDouble(token[2]);
                             gp.f2_trace[i + 2] = Double.parseDouble(token[3]);
                             gp.f2_trace[i + 3] = Double.parseDouble(token[4]);
 						}
 						catch (Exception ex) 
 						{ 
 							Util.show_and_log(instring + ex.getMessage());
 						}
 	                    i = i + 4;
 					}

 					if (token[0].equals("##XYDATA")) mode = DATA_REAL;
 				}
             }

 			//	set some internal parameters, complete with paranoid divide by 0 checks
 			if (gp.size_f2_trace > 2)
 				gp.hzperpt_f2_trace = (firstx - lastx) / (gp.size_f2_trace - 1);

 			gp.offset_f2_trace = firstx;
 			gp.f2_trace_scale = ((double)gp.ysize_2D * 0.80) / 32000.0;

 			try {	br.close();	} 
 	        catch (IOException ex) {	ex.printStackTrace();	}
 			return i;
         }

 ////-------------------------------------------------------------------------------------------------
 //// Read a JCAMP DX file int the F1 trace for 2D

 		public static int read_jdx_f1(String fileName, global_par gp)
 		{
 			int mode = PARAM;
 			boolean space_allocated = false;
 			String instring;
 			String delim = "[ =,\t]+";
 			String[] token;

 			double firstx = 5000.0, lastx = 0.0 /*, SF = 125.8*/;
 			int i = 0;
 			gp.size_f1_trace = 0;

 		// try to open the jdx parameter file 
 	        BufferedReader br;
 	        try { br = new BufferedReader(new FileReader(fileName)); }
 	        catch (Exception ex)
 	        {
 	            Util.show_and_log("read_jdx_f1(), cannot open: " + fileName, ex.getMessage());
 	            return -1;
 	        }

 			for(;;)
 			{
 				try {instring = br.readLine();} 
 	            catch (IOException ex) 
 	            {
 	    			ex.printStackTrace();
 	    			Util.show_and_log("read_jdx_F1(), read error: " + fileName, ex.getMessage());
 	    			try { br.close(); } 
 	    			catch (IOException e) { e.printStackTrace(); }
 	    			return -1;
 	    		}
 				
 				instring = instring.trim();
 				
 				if(instring == null) break;
 				
 				if (instring.length() > 3)
 				{
 					token = instring.split(delim);
 	                if (token[0].equals("##END")) break;		// end of file

 					if (mode == PARAM)							// reading parameters
 					{

 						try
 						{
 							if (token[0].equals("##NPOINTS"))
 							{
 								gp.size_f1_trace = Integer.parseInt(token[1]);
 							}
 						}
 						catch (Exception ex)
 						{ 
                             Util.show_and_log(instring + ex.getMessage());
 					    }


 						try
 						{
 							if (token[0].equals("##FIRSTX"))
 							{
                                firstx = Double.parseDouble(token[1]);
 							}
 						}
 						catch (Exception ex)
 						{ 
                             Util.show_and_log(instring + ex.getMessage());
 					    }

 						try
 						{
 							if (token[0].equals("##LASTX"))
 							{
                                 lastx = Double.parseDouble(token[1]);
 							}
 						}
 						catch (Exception ex)
 						{ 
                             Util.show_and_log(instring + ex.getMessage());
 					    }

 						try
 						{
 							if (token[0].equals("##$SF"))		// unused???
 							{
                                 //SF = Double.parseDouble(token[1]);
 							}
 						}
 						catch (Exception ex)
 						{ 
                             Util.show_and_log(instring + ex.getMessage());
 					    }

 					}
 					else if (mode == DATA_REAL && gp.size_f1_trace > 0)		// finished parameters, now read data
 					{
 						if (space_allocated == false)
 						{
 							gp.f1_trace = new double[gp.size_f1_trace];
 							space_allocated = true;
 						}

 						if (i + 4 > gp.size_f1_trace)			// check that we don't go over
 						{										// array end
 							Util.show_and_log(String.format("read_jdx_f1(): size = , but points read = ",
 								gp.size_f1_trace, i));
 							break;
 						}
 						try
 						{
 							gp.f1_trace[i]     = Double.parseDouble(token[1]);
 							gp.f1_trace[i + 1] = Double.parseDouble(token[2]);
 							gp.f1_trace[i + 2] = Double.parseDouble(token[3]);
 							gp.f1_trace[i + 3] = Double.parseDouble(token[4]);
 						}
 						catch (Exception ex) 
 						{
 							Util.show_and_log(instring + ex.getMessage());
 						}
 						i = i + 4;
 					}

 					if (token[0].equals("##XYDATA")) mode = DATA_REAL;
 				}
 			}

 			//	set some internal parameters, complete with paranoid divide by 0 checks
 			if (gp.size_f1_trace > 2)
 				gp.hzperpt_f1_trace = (firstx - lastx) / (gp.size_f1_trace - 1);

 			gp.offset_f1_trace = firstx;
 			gp.f1_trace_scale = ((double)gp.ysize_2D * 0.80) / 32000.0;

 			try {	br.close();	} 
 	        catch (IOException ex) {	ex.printStackTrace();	}
 			return i;
 		}


 		

//-------------------------------------------------------------------------------------------------
//	write_jdx.cpp   utility function to write processed data in JCAMP DX format
//	returns number of data points written, -1 if file could not be opened, 0
//	if file could be opened but no data points written

		public static int write_jdx(String fileName, global_par gp, data_par q)
		{
			int i;
			PrintWriter pw;

			//	open file
			try { pw = new PrintWriter(fileName); }
			catch (Exception ex)
			{
				Util.show_and_log("write_jdx(): " + fileName, ex.getMessage());
				return -1;
			}

			double scale_factor = 32760.0/gp.data_max;				// scaling factor 
			pw.println("##TITLE= " + gp.plot_title);
			pw.println("##JCAMPDX= 5.0");
			pw.println("##DATA TYPE= NMR Spectrum");
			pw.println("##DATA CLASS= XYDATA");
			pw.println("##ORIGIN= SpinWorksJ 0.0, Kirk Marat, U of Manitoba");
			pw.println("##OWNER= undefined");
			pw.println(String.format("##.OBSERVE FREQUENCY= %.8f", q.SFO1));
			pw.println("##.OBSERVE NUCLEUS= " + q.NUC1);
			pw.println(String.format("##.AVERAGES= %d", q.NS));

			if (q.aq_mode == SW_main.det_modes.SEQ)			// acquisition mode
				pw.println("##.ACQUISITION MODE= SEQUENTIAL");
			else
				pw.println("##.ACQUISITION MODE= SIMULTANEOUS");

			pw.println("##.DIGITISER RES= 16");

            if (q.data_type == SW_main.data_types.BRUKER)		// data origin
                pw.println("##SPECTROMETER/DATA SYSTEM= Bruker " + q.INSTRUM);
            else if (q.data_type == SW_main.data_types.VNMR)
                pw.println("##SPECTROMETER/DATA SYSTEM= VARIAN Unity/Inova/VNMRS");
            else if (q.data_type == SW_main.data_types.JEOL)
                pw.println("##SPECTROMETER/DATA SYSTEM= JEOL 1D");
            else if (q.data_type == SW_main.data_types.TECMAG)
                pw.println("##SPECTROMETER/DATA SYSTEM= TECMAG");
            else if (q.data_type == SW_main.data_types.CDFF)
                pw.println("##SPECTROMETER/DATA SYSTEM= CDFF");
            else if (q.data_type == SW_main.data_types.SIMPSON)
                pw.println("##SPECTROMETER/DATA SYSTEM= SIMPSON simulation");
            else
                pw.println("##SPECTROMETER/DATA SYSTEM= UNKNOWN");

			pw.println("$$ Bruker Specific parameters");
			pw.println("$$ for XwinNMR compatibility");
			pw.println(String.format("##$SFO1= %.8f", q.SFO1));
			pw.println(String.format("##$SF= %.10f", q.SF));
			pw.println(String.format("##$OFFSET = %.8f", q.OFFSET));
			pw.println(String.format("##$SI= %d", q.SI));
			pw.println(String.format("##$TD= %d", q.TD));
			pw.println(String.format("##$SW= %.6f", q.SW));
		    pw.println("##$PULPROG= " + q.pulse_sequence);
			pw.println("$$ End of Bruker specific parameters");

			pw.println("##XUNITS= Hz");
			pw.println("##YUNITS= ARBITRARY UNITS");
			pw.println("##XFACTOR= 1");
			pw.println("##YFACTOR= 1");

			pw.println(String.format("##FIRSTX= %.10f", q.offset));
			pw.println(String.format("##LASTX= %.10f", 
                q.offset - (q.SW * q.SFO1)));
			pw.println(String.format("##DELTAX= %.14f", -(q.hzperpt)));
			pw.println(String.format("##MAXY= %d", (int)(scale_factor * gp.data_max)));
			pw.println(String.format("##MINY= %d", (int)(scale_factor * gp.data_array[0])));		//TO DO Fix!
			pw.println(String.format("##NPOINTS= %d", q.SI));
			pw.println(String.format("##FIRSTY= %d", (int)(scale_factor * gp.data_array[0])));
			pw.println("##XYDATA=(X++(Y..Y))");


			for(i = 0; i < q.SI; i = i+4)							// now the actual data points
			{
				pw.println(String.format(" %f %d %d %d %d", 
                            q.offset - ((double)i * q.hzperpt),
							(int)(scale_factor * gp.data_array[i]),
							(int)(scale_factor * gp.data_array[i + 1]),
							(int)(scale_factor * gp.data_array[i + 2]),
							(int)(scale_factor * gp.data_array[i + 3])));
			}

			pw.println("##END=");

			pw.close();
			return i;		// return data points written
		}
	
//-------------------------------------------------------------------------------------------------
//		Version for  columns (F1) of a 2D data set

		public static int write_jdx_f1(String fileName, int column, 
				global_par gp, data_par q, data_par q_f1)
		{
			int i;
			PrintWriter pw;

			//	open file
			try { pw = new PrintWriter(fileName); }
			catch (Exception ex)
			{
				Util.show_and_log("write_jdx_f1(): " + fileName, ex.getMessage());
				return -1;
			}
			
			double data_max = -5.0e10, data_min = 5.0e10;
			
			for (i = 0; i < q_f1.SI; i++)
			{
				if (gp.rows_real[i][column] > data_max) data_max = gp.rows_real[i][column];
				if (gp.rows_real[i][column] < data_min) data_min = gp.rows_real[i][column];
			}

			double scale_factor = 32760.0/data_max;				// scaling factor to 16 bit

			String title_string = String.format("%s column at %.4f ppm", gp.plot_title.replace("\n", " "),
					(q.offset - column * q.hzperpt) / q.SF);

			pw.println("##TITLE= " + title_string);		
			pw.println("##JCAMPDX= 5.0");		
			pw.println("##DATA TYPE= NMR Spectrum");
			pw.println("##DATA CLASS= XYDATA");
			pw.println("##ORIGIN= SpinWorksJ 0.0 , Kirk Marat, U of Manitoba");
			pw.println("##OWNER= undefined");
			pw.println(String.format("##.OBSERVE FREQUENCY= %.8f", q_f1.SFO1));
			pw.println("##.OBSERVE NUCLEUS= " + q.NUC1);
			pw.println(String.format("##.AVERAGES= %d", q.NS));

			if (q.aq_mode == SW_main.det_modes.SEQ)			// acquisition mode
				pw.println("##.ACQUISITION MODE= SEQUENTIAL");
			else
				pw.println("##.ACQUISITION MODE= SIMULTANEOUS");

			pw.println("##.DIGITISER RES= 16");

			if (q.data_type == SW_main.data_types.BRUKER)									// data origin
				pw.println("##SPECTROMETER/DATA SYSTEM= Bruker " + q.INSTRUM);
			else if (q.data_type == SW_main.data_types.VNMR)
				pw.println("##SPECTROMETER/DATA SYSTEM= VARIAN Unity/Inova/VNMRS");
			else
				pw.println("##SPECTROMETER/DATA SYSTEM= Other or UNKNOWN");

			pw.println("$$ Bruker Specific parameters");
			pw.println("$$ for XwinNMR compatibility");
			pw.println(String.format("##$SFO1= %.8f", q_f1.SFO1));
			pw.println(String.format("##$SF= %.10f", q_f1.SF));
			pw.println(String.format("##$OFFSET = %.8f", q_f1.OFFSET));
			pw.println(String.format("##$SI= %d", q_f1.SI));
			pw.println(String.format("##$TD= %d", q_f1.TD));
			pw.println(String.format("##$SW= %.6f", q_f1.SW));
			pw.println("##$PULPROG= " + q.pulse_sequence);
			pw.println("$$ End of Bruker specific parameters");
			pw.println("##XUNITS= Hz");
			pw.println("##YUNITS= ARBITRARY UNITS");
			pw.println("##XFACTOR= 1");
			pw.println("##YFACTOR= 1");
			pw.println(String.format("##FIRSTX= %.10f", q_f1.offset));
			pw.println(String.format("##LASTX= %.10f", 
					q_f1.offset - (q_f1.SW * q_f1.SFO1)));
			pw.println(String.format("##DELTAX= %.14f", -(q_f1.hzperpt)));
			pw.println(String.format("##MAXY= %d", (int)(scale_factor * data_max)));
			pw.println(String.format("##MINY= %d", (int)(scale_factor * data_min)));		//TO DO Fix!
			pw.println(String.format("##NPOINTS= %d", q_f1.SI));
			pw.println(String.format("##FIRSTY= %d", (int)(scale_factor * gp.rows_real[0][column])));
			pw.println("##XYDATA=(X++(Y..Y))");

			for(i = 0; i < q_f1.SI; i = i+4)							// now the actual data points
			{
				pw.println(String.format(" %f %d %d %d %d",
						q_f1.offset - ((double)i * q_f1.hzperpt),
						(int)(scale_factor * (double)gp.rows_real[i][column]),
						(int)(scale_factor * (double)gp.rows_real[i + 1][column]),
						(int)(scale_factor * (double)gp.rows_real[i + 2][column]),
						(int)(scale_factor * (double)gp.rows_real[i + 3][column])));
			}

			pw.println("##END=");
			pw.close();
			return i;		// return data points written
		}

//-------------------------------------------------------------------------------------------------
//		Version for F2 (rows) of a 2D data set
		public static int write_jdx_f2(String fileName, int row, global_par gp, 
				data_par q, data_par q_f1)
		{
			int i;
			PrintWriter pw;

			//				open file
			try { pw = new PrintWriter(fileName); }
			catch (Exception ex)
			{
				Util.show_and_log("write_jdx_f2(): " + fileName, ex.getMessage());
				return -1;
			}


			double data_max = -5.0e10, data_min = 5.0e10;
			for (i = 0; i < q.SI; i++)
			{
				if (gp.rows_real[row][i] > data_max) data_max = gp.rows_real[row][i];
				if (gp.rows_real[row][i] < data_min) data_min = gp.rows_real[row][i];
			}

			double scale_factor = 32760.0 / data_max;				// scaling factor to 16 bit

			String title_string = String.format("%s row at %.4f ppm", gp.plot_title.replace("\n", " "),
					(q_f1.offset - row * q_f1.hzperpt) / q_f1.SF);

			pw.println("##TITLE= " + title_string);
			pw.println("##JCAMPDX= 5.0");
			pw.println("##DATA TYPE= NMR Spectrum");
			pw.println("##DATA CLASS= XYDATA");
			pw.println("##ORIGIN= SpinWorksJ 0.0, Kirk Marat, U of Manitoba");
			pw.println("##OWNER= undefined");
			pw.println(String.format("##.OBSERVE FREQUENCY= %.8f", q.SFO1));
			pw.println("##.OBSERVE NUCLEUS= " + q.NUC1);
			pw.println(String.format("##.AVERAGES= %d", q.NS));

			if (q.aq_mode == SW_main.det_modes.SEQ)			// acquisition mode
				pw.println("##.ACQUISITION MODE= SEQUENTIAL");
			else
				pw.println("##.ACQUISITION MODE= SIMULTANEOUS");

			pw.println("##.DIGITISER RES= 16");

			if (q.data_type == SW_main.data_types.BRUKER)									// data origin
				pw.println("##SPECTROMETER/DATA SYSTEM= Bruker " + q.INSTRUM);
			else if (q.data_type == SW_main.data_types.VNMR)
				pw.println("##SPECTROMETER/DATA SYSTEM= VARIAN Unity/Inova");
			else
				pw.println("##SPECTROMETER/DATA SYSTEM= UNKNOWN");

			pw.println("$$ Bruker Specific parameters");
			pw.println("$$ for XwinNMR compatibility");
			pw.println(String.format("##$SFO1= %.8f", q.SFO1));
			pw.println(String.format("##$SF= %.10f", q.SF));
			pw.println(String.format("##$OFFSET = %.8f", q.OFFSET));
			pw.println(String.format("##$SI= %d", q.SI));
			pw.println(String.format("##$TD= %d", q.TD));
			pw.println(String.format("##$SW= %.6f", q.SW));
			pw.println("##$PULPROG= " + q.pulse_sequence);
			pw.println("$$ End of Bruker specific parameters");
			pw.println("##XUNITS= Hz");
			pw.println("##YUNITS= ARBITRARY UNITS");
			pw.println("##XFACTOR= 1");
			pw.println("##YFACTOR= 1");
			pw.println(String.format("##FIRSTX= %.10f", q.offset));
			pw.println(String.format("##LASTX= %.10f", 
					q.offset - (q.SW * q.SFO1)));
			pw.println(String.format("##DELTAX= %.14f", -(q.hzperpt)));
			pw.println(String.format("##MAXY= %d", (int)(scale_factor * data_max)));
			pw.println(String.format("##MINY= %d", (int)(scale_factor * data_min)));		//TO DO Fix!
			pw.println(String.format("##NPOINTS= %d", q.SI));
			pw.println(String.format("##FIRSTY= %d", (int)(scale_factor * gp.rows_real[row][0])));
			pw.println("##XYDATA=(X++(Y..Y))");

			for (i = 0; i < q.SI; i = i + 4)							// now the actual data points
			{

				pw.println(String.format(" %f %d %d %d %d",
						q.offset - ((double)i * q.hzperpt),
						(int)(scale_factor * (double)gp.rows_real[row][i]),
						(int)(scale_factor * (double)gp.rows_real[row][i + 1]),
						(int)(scale_factor * (double)gp.rows_real[row][i + 2]),
						(int)(scale_factor * (double)gp.rows_real[row][i + 3])));
			}

			pw.println("##END=");
			pw.close();
			return i;		// return data points written
		}
		
		
//-------------------------------------------------------------------------------------------------
//		Write the F2 projection as JDX file

			public static int write_jdx_f2_proj(String fileName, global_par gp,
						data_par q)
			{
				int i;
				PrintWriter pw;

				//	open file
				try { pw = new PrintWriter(fileName); }
				catch (Exception ex)
				{
					Util.show_and_log("write_jdx_f2(): " + fileName, ex.getMessage());
					return -1;
				}


				double data_max = -5.0e10, data_min = 5.0e10;
				try
				{
					for (i = 0; i < q.SI; i++)
					{
						if (gp.f2_trace[i] > data_max) data_max = gp.f2_trace[i];
						if (gp.f2_trace[i] < data_min) data_min = gp.f2_trace[i];
					}
				}
				catch (Exception ex)
				{
					Util.show_and_log("write_jdx_f2(): ", ex.getMessage());
					pw.close();
					return -1;
				}

				double scale_factor = 32760.0 / data_max;				// scaling factor to 16 bit

	            String title_string = String.format("F2 projection" + gp.plot_title);


				pw.println("##TITLE= " + title_string);
				pw.println("##JCAMPDX= 5.0");
				pw.println("##DATA TYPE= NMR Spectrum");
				pw.println("##DATA CLASS= XYDATA");
				pw.println("##ORIGIN= SpinWorksJ 0.0, Kirk Marat, U of Manitoba");
				pw.println("##OWNER= undefined");
				pw.println(String.format("##.OBSERVE FREQUENCY= %.8f", q.SFO1));
				pw.println("##.OBSERVE NUCLEUS= " + q.NUC1);
				pw.println(String.format("##.AVERAGES= %d", q.NS));

				if (q.aq_mode == SW_main.det_modes.SEQ)				// acquisition mode
					pw.println("##.ACQUISITION MODE= SEQUENTIAL");
				else
					pw.println("##.ACQUISITION MODE= SIMULTANEOUS");

				pw.println("##.DIGITISER RES= 16");

				if (q.data_type == SW_main.data_types.BRUKER)		// data origin
					pw.println("##SPECTROMETER/DATA SYSTEM= Bruker " + q.INSTRUM);
				else if (q.data_type == SW_main.data_types.VNMR)
					pw.println("##SPECTROMETER/DATA SYSTEM= VARIAN Unity/Inova/VNMRS");

				else
					pw.println("##SPECTROMETER/DATA SYSTEM= UNKNOWN");

				pw.println("$$ Bruker Specific parameters");
				pw.println("$$ for XwinNMR compatibility");
				pw.println(String.format("##$SFO1= %.8f", q.SFO1));
				pw.println(String.format("##$SF= %.10f", q.SF));
				pw.println(String.format("##$OFFSET = %.8f", q.OFFSET));
				pw.println(String.format("##$SI= %d", q.SI));
				pw.println(String.format("##$TD= %d", q.TD));
				pw.println(String.format("##$SW= %.6f", q.SW));
				pw.println("##$PULPROG= " + q.pulse_sequence);
				pw.println("$$ End of Bruker specific parameters");
				pw.println("##XUNITS= Hz");
				pw.println("##YUNITS= ARBITRARY UNITS");
				pw.println("##XFACTOR= 1");
				pw.println("##YFACTOR= 1");
				pw.println(String.format("##FIRSTX= %.10f", q.offset));
				pw.println(String.format("##LASTX= %.10f", 
	                q.offset - (q.SW * q.SFO1)));
				pw.println(String.format("##DELTAX= %.14f", -(q.hzperpt)));
				pw.println(String.format("##MAXY= %d", (int)(scale_factor * data_max)));
				pw.println(String.format("##MINY= %d", (int)(scale_factor * data_min)));		//TO DO Fix!
				pw.println(String.format("##NPOINTS= %d", q.SI));
				pw.println(String.format("##FIRSTY= %d", (int)(scale_factor * gp.f2_trace[0])));
				pw.println("##XYDATA=(X++(Y..Y))");

				for (i = 0; i < q.SI; i = i + 4)							// now the actual data points
				{
					try
					{
						pw.println(String.format(" %f %d %d %d %d",
							q.offset - ((double)i * q.hzperpt),
												 (int)(scale_factor * (double)gp.f2_trace[i]),
												 (int)(scale_factor * (double)gp.f2_trace[i + 1]),
												 (int)(scale_factor * (double)gp.f2_trace[i + 2]),
												 (int)(scale_factor * (double)gp.f2_trace[i + 3])));
					}
					catch (Exception ex)
					{
						Util.show_and_log("write_jdx_f2_proj(): ", ex.getMessage());
						break;
					}
				}


				pw.println("##END=");
				pw.close();
				return i;		// return data points written

			}

//-------------------------------------------------------------------------------------------------
//		Write the F1 projection as JDX file

			public static int write_jdx_f1_proj(String fileName, global_par gp,
						data_par q, data_par q_f1)
			{
				int i;
				PrintWriter pw;

				//				open file
				try { pw = new PrintWriter(fileName); }
				catch (Exception ex)
				{
					Util.show_and_log("write_jdx_f2(): " + fileName, ex.getMessage());
					return -1;
				}


				double data_max = -5.0e10, data_min = 5.0e10;
				
				try
				{
					for (i = 0; i < q_f1.SI; i++)
					{
						if (gp.f1_trace[i] > data_max) data_max = gp.f1_trace[i];
						if (gp.f1_trace[i] < data_min) data_min = gp.f1_trace[i];
					}
				}
				catch (Exception ex)
				{
					Util.show_and_log("write_jdx_f1_proj(): ", ex.getMessage());
					pw.close();
					return -1;
				}

				double scale_factor = 32760.0 / data_max;				// scaling factor to 16 bit

	            String title_string = String.format("F1 projection" + gp.plot_title);


				pw.println("##TITLE= " + title_string);
				pw.println("##JCAMPDX= 5.0");
				pw.println("##DATA TYPE= NMR Spectrum");
				pw.println("##DATA CLASS= XYDATA");
				pw.println("##ORIGIN= SpinWorksJ 0.0, Kirk Marat, U of Manitoba");
				pw.println("##OWNER= undefined");
				pw.println(String.format("##.OBSERVE FREQUENCY= %.8f", q_f1.SFO1));
				pw.println("##.OBSERVE NUCLEUS= " + q.NUC1);
				pw.println(String.format("##.AVERAGES= %d", q.NS));

				if (q.aq_mode == SW_main.det_modes.SEQ)				// acquisition mode
					pw.println("##.ACQUISITION MODE= SEQUENTIAL");
				else
					pw.println("##.ACQUISITION MODE= SIMULTANEOUS");

				pw.println("##.DIGITISER RES= 16");

				if (q.data_type == SW_main.data_types.BRUKER)			// data origin
					pw.println("##SPECTROMETER/DATA SYSTEM= Bruker " + q.INSTRUM);
				else if (q.data_type == SW_main.data_types.VNMR)
					pw.println("##SPECTROMETER/DATA SYSTEM= VARIAN Unity/Inova/VNMRS");

				else
					pw.println("##SPECTROMETER/DATA SYSTEM= UNKNOWN");

				pw.println("$$ Bruker Specific parameters");
				pw.println("$$ for XwinNMR compatibility");
				pw.println(String.format("##$SFO1= %.8f", q_f1.SFO1));
				pw.println(String.format("##$SF= %.10f", q_f1.SF));
				pw.println(String.format("##$OFFSET = %.8f", q_f1.OFFSET));
				pw.println(String.format("##$SI= %d", q_f1.SI));
				pw.println(String.format("##$TD= %d", q_f1.TD));
				pw.println(String.format("##$SW= %.6f", q_f1.SW));
				pw.println("##$PULPROG= " + q.pulse_sequence);
				pw.println("$$ End of Bruker specific parameters");
				pw.println("##XUNITS= Hz");
				pw.println("##YUNITS= ARBITRARY UNITS");
				pw.println("##XFACTOR= 1");
				pw.println("##YFACTOR= 1");
				pw.println(String.format("##FIRSTX= %.10f", q_f1.offset));
				pw.println(String.format("##LASTX= %.10f",
	                q_f1.offset - (q_f1.SW * q_f1.SFO1)));
				pw.println(String.format("##DELTAX= %.14f", -(q_f1.hzperpt)));
				pw.println(String.format("##MAXY= %d", (int)(scale_factor * data_max)));
				pw.println(String.format("##MINY= %d", (int)(scale_factor * data_min)));		//TO DO Fix!
				pw.println(String.format("##NPOINTS= %d", q_f1.SI));
				pw.println(String.format("##FIRSTY= %d", (int)(scale_factor * gp.f1_trace[0])));
				pw.println("##XYDATA=(X++(Y..Y))");

				for (i = 0; i < q_f1.SI; i = i + 4)							// now the actual data points
				{
					try
					{
						pw.println(String.format(" %f %d %d %d %d",
							q_f1.offset - ((double)i * q_f1.hzperpt),
												 (int)(scale_factor * (double)gp.f1_trace[i]),
												 (int)(scale_factor * (double)gp.f1_trace[i + 1]),
												 (int)(scale_factor * (double)gp.f1_trace[i + 2]),
												 (int)(scale_factor * (double)gp.f1_trace[i + 3])));
					}
					catch (Exception ex)
					{
						Util.show_and_log("write_jdx_f1_proj(): ", ex.getMessage());
						break;
					}
				}


				pw.println("##END=");
				pw.close();
				return i;		// return data points written
			}
			

//-------------------------------------------------------------------------------------------------
/// Undo "DIFF" type JCAMP-DX compression
	private static String un_diff(String instring )
    {
    	int i, j;
    	char[] c_array = new char[instring.length()];

    	//char[] delim = new char[] { ' ' };
    	String delim = "[ ]+";
    	String[] token;

    	char[] ch = new char[] {'%', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R',
    			'j', 'k', 'l', 'm', 'n', 'o','p', 'q', 'r'};

    	char[] i_ch = new char[] {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
    			'1', '2', '3', '4', '5', '6', '7', '8', '9'};

    	String new_string = "";
    	boolean is_char = false;

    	instring = un_squeeze(instring);
    	c_array = instring.toCharArray();

    	for (i = 0; i < instring.length(); i++)
    	{
    		is_char = false;
    		for (j = 0; j < ch.length; j++)
    		{
    			if (c_array[i] == ch[j])
    			{
    				is_char = true;
    				break;
    			}
    		}
    		if (is_char)
    		{ 
    			new_string = new_string + String.format(" %c", ch[j]); //<<<<<<<
    		}
    		else 
    			new_string = new_string + c_array[i];
    	}

    	token = new_string.split(delim);

    	int[] y = new int[token.length];

    	y[0] = Integer.parseInt(token[0]);
    	y[1] = Integer.parseInt(token[1]);

    	try
    	{
    		for (i = 2; i < token.length; i++)
    		{
    			char[] ch_array = new char[token[i].length()];
    			ch_array = token[i].toCharArray();

    			is_char = false;
    			for (j = 0; j < ch.length; j++)
    			{
    				if (ch_array[0] == ch[j])
    				{
    					is_char = true;
    					break;
    				}
    			}

    			if (is_char)
    			{                  
    				ch_array[0] = i_ch[j];

    				String temp = "";

    				for (int k = 0; k < ch_array.length; k++)
    					temp = temp + ch_array[k];

    				if(j <= 9)
    				{
    					y[i] = y[i-1] + Integer.parseInt((temp));
    				}
    				else if (j > 9)
    				{
    					y[i] = y[i - 1] - Integer.parseInt((temp));
    				}                       
    			}
    		}
    	}
    	catch(Exception ex)
    	{
    		Util.show_and_log("exception in un_diff()", ex.getMessage());
    		return "";
    	}

    	new_string = "";
    	for (i = 0; i < token.length -1; i++)
    		new_string = new_string + String.format(" %d", y[i]);

    	return new_string;
    }
        
 //------------------------------------------------------------------------------------------------
 /// Un-do "Squeeze" type JCAMP-DX compression
      private static String un_squeeze(String instring)
      {
      	int i, j;
      	char[] c_array = new char[instring.length()];
      	char[] ch = new char[] {'@', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I',
      			'a', 'b', 'c', 'd', 'e', 'f','g', 'h', 'i'};
      	String new_string = "";
      	boolean is_char = false;

      	c_array = instring.toCharArray();

      	try
      	{
      		for (i = 0; i < instring.length(); i++)
      		{
      			is_char = false;
      			for (j = 0; j < ch.length; j++)
      			{
      				if (c_array[i] == ch[j])
      				{
      					is_char = true;
      					break;
      				}
      			}
      			if (is_char)
      			{
      				if (j <= 9)
      					new_string = new_string + String.format(" %d", j);
      				else if (j > 9) // necessary
      					new_string = new_string + String.format(" -%d", j - 9);
      			}
      			else
      				//new_string = new_string + c_array[i].ToString();
      				new_string = new_string + c_array[i];
      		}
      	}
      	catch(Exception ex)
      	{
      		Util.show_and_log("exception in un_squeeze()", ex.getMessage());
      		return "";
      	}

      	return new_string;
      }
      
//-------------------------------------------------------------------------------------------------
}
