
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

//	Perhaps merge this into ButtonListeners??
public class DisplayButtons extends SW_main
{
//-------------------------------------------------------------------------------------------------
//  Do a vertical reset of the spectrum
	
	public static void vResetButtonClick()
	{
		if (gp.display_mode == modes.MODE_1D)          // 1D
		{
			gp.data_max = Math.abs(Util.dscan(gp.data_array, q.SI, 0, q.SI));
			if (gp.data_max > 0.0)                          // paranoia
				gp.expt_scale = (double)gp.dysize * 0.8 / gp.data_max;
		}
		else if (gp.display_mode == modes.MODE_2D)     // 2D
		{
			if(q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XY)
				gp.data_max_2D = Util_2D.max_2D(gp.rows_real, q.SI, q_f2.SI);	// find global maximum
			else
				gp.data_max_2D = Util_2D.max_2D(gp.rows_real, q.SI, q_f1.SI);
			
			Util_2D.calc_default_levels_2D(gp);
			gp.new_2D_view = true;
		}

		gp.cursor_flag = 0;
		update_spect = true;
		canvas.redraw();
	}
	
//-------------------------------------------------------------------------------------------------
//  display full spectrum, 1D and 2D

	public static void fullButtonClick()
	{
		if (gp.display_mode == modes.MODE_1D
				&& gp.data_array == null && gp.calc_array == null) 
			return;


		if (gp.display_mode == modes.MODE_FID)
		{
			gp.t1 = 0.0;    
			gp.t2 = (double)(q.TD / 2) * q.dwell;
		}
		
		else if(gp.display_mode == modes.MODE_1D)
		{
			gp.last_freq1 = gp.freq1;
			gp.last_freq2 = gp.freq2;
			gp.freq1 = q.offset;
			gp.freq2 = q.offset - (q.SFO1 * q.SW);
		}
		
		else if (gp.display_mode == modes.MODE_2D)
		{
			gp.last_freq1 = gp.freq1;
			gp.last_freq2 = gp.freq2;
			gp.last_freq1_indirect = gp.freq1_indirect;
			gp.last_freq2_indirect = gp.freq2_indirect;
			gp.freq1 = q.offset;
			gp.freq2 = q.offset - (q.SFO1 * q.SW);

			if (q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XY)
			{
				gp.freq1_indirect = q_f2.offset;
				gp.freq2_indirect = q_f2.offset - q_f2.SW * q_f2.SFO1;
			}
			else
			{
				gp.freq1_indirect = q_f1.offset;
				gp.freq2_indirect = q_f1.offset - (q_f1.SFO1 * q_f1.SW);
			}
			gp.new_2D_view = true;
		}

		gp.cursor_flag = 0;
		update_spect = true;
		canvas.redraw();
		slider.setSelection(50);       
	}
	
//-------------------------------------------------------------------------------------------------
//  zoom region between cursors, 1D and 2D

	public static void zoomButtonClick()
	{
		if (gp.display_mode == modes.MODE_1D
				&& gp.data_array == null && gp.calc_array == null) return;

		if(gp.cursor_flag != 2) 
		{
			Util.show_and_log("You must define region with two cursors first");
			return;
		}

		if (gp.display_mode == modes.MODE_FID)
		{
			gp.t1 = gp.new_t1;
			gp.t2 = gp.new_t2;
			slider.setSelection((int)(100.0 * (gp.t1 + gp.t2)/2.0 / (q.dwell * q.TD /2)));
		}
		
		else if (gp.display_mode == modes.MODE_1D)			// scroll bar only used in 1D
		{
			gp.last_freq1 = gp.freq1;
			gp.last_freq2 = gp.freq2;
			gp.freq1 = gp.new_freq1;
			gp.freq2 = gp.new_freq2;
			slider.setSelection((int)(100.0 * (q.offset - (gp.freq1 + gp.freq2) / 2.0) /
					(q.SFO1 * q.SW)));
		}
		
		else if (gp.display_mode == modes.MODE_2D)
		{
			gp.last_freq1 = gp.freq1;
			gp.last_freq2 = gp.freq2;
			gp.last_freq1_indirect = gp.freq1_indirect;
			gp.last_freq2_indirect = gp.freq2_indirect;

			gp.freq1 = gp.new_freq1;
			gp.freq2 = gp.new_freq2;
			gp.freq1_indirect = gp.new_freq1_indirect;
			gp.freq2_indirect = gp.new_freq2_indirect;
			gp.new_2D_view = true;
		}       		

		gp.cursor_flag = 0;
		update_spect = true;
		canvas.redraw();
	}

//-------------------------------------------------------------------------------------------------
// restore last expansion
	
	public static void lastButtonClick()
    {
        if (gp.display_mode == modes.MODE_1D
               && gp.data_array == null && gp.calc_array == null) return;

        gp.freq1 = gp.last_freq1;				            // F2
        gp.freq2 = gp.last_freq2;
        gp.freq1_indirect = gp.last_freq1_indirect;		    // F1
        gp.freq2_indirect = gp.last_freq2_indirect;

        slider.setSelection ((int)(100.0 * (q.offset - (gp.freq1 + gp.freq2) / 2.0) /
				(q.SFO1 * q.SW)));
        
        gp.new_2D_view = true;
		gp.cursor_flag = 0;
		update_spect = true;		
		canvas.redraw();     
    }

//-------------------------------------------------------------------------------------------------
// switch between 1D and 2D display modes
	
	public static void display2DButtonClick()
	{
		if(gp.display_mode != modes.MODE_2D)		// switch from 1D to 2D display mode
        {
	        gp.display_mode = modes.MODE_2D;
            disp2DButton.setText("1D Display");
            InterfaceUtil.set2D();					// set correct button panel

	        update_spect = true;		
			canvas.redraw(); 
	        SW_main.slider.setVisible(false);
        }
        else if(gp.display_mode != modes.MODE_1D)	// switch from 2D to 1D display mode
        {
	        gp.display_mode = modes.MODE_1D;
            disp2DButton.setText("2D Display");
            InterfaceUtil.set1D();					// set correct button panel
            disp2DButton.setVisible(true);			// so we can return to 2D display mode

	        if(gp.slice_no[gp.active_slice] > 0 && gp.slice_no[gp.active_slice] < q_f1.SI)
		        gp.block_2D = gp.slice_no[gp.active_slice];
	        Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);

	        update_spect = true;		
			canvas.redraw(); 
			SW_main.slider.setVisible(true);
        }
	}
//-------------------------------------------------------------------------------------------------
	public static void prevBlockButtonClick()
	{
		if (q.data_mode == modes.MODE_2D && gp.display_mode == modes.MODE_FID
				&& gp.block_2D > 0)
		{
			gp.block_2D--;
		}

		// Bruker stacked 2D mode (T1, diffusion, etc)
		else if (q.data_type == data_types.BRUKER && gp.block_2D > 0
				&& gp.display_mode == modes.MODE_1D && gp.n_traces > 0)
		{
			gp.block_2D--;
			for (int i = 0; i < q.SI; i++)
			{
				gp.data_array[i] = gp.stack_trace[gp.block_2D].data[i];
				gp.i_data_array[i] = gp.stack_trace[gp.block_2D].idata[i];
			}
		}

		if ((q.data_mode == modes.MODE_2D || q.data_mode == modes.MODE_3D)
				&& gp.n_traces == 0)
		{
			// in 1D display mode, step through individual rows of 2D matrix
			if (gp.block_2D > 0 && gp.rows_real != null && gp.display_mode == modes.MODE_1D)
			{
				gp.block_2D--;
				Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);
			}
			// in 2D dispay mode step through displayed slices
			else
			{
				if (gp.active_slice > 0)
					gp.active_slice = gp.active_slice - 1;
			}
		}

		// Varian, Anasazi multifid (T1, DEPT, etc.)
		else if ((q.data_type == data_types.VNMR) || (q.data_type == data_types.CDFF)
				&& gp.vblock > 1)
		{
			gp.vblock--;
			p.PH_mod = phasing.CONSTANTS;				// we assume that the we want 
			Proc_1D.proc_1D(gp, p, q );					// phasing the same as the first block
		}

		update_spect = true;		
		canvas.redraw(); 
	}

//-------------------------------------------------------------------------------------------------
	public static void nextBlockButtonClick()
	{
		// Display individual 2D FIDs  not working???
		if (q.data_mode == modes.MODE_2D && gp.display_mode == modes.MODE_FID
				&& gp.block_2D < q_f1.TD-1)
		{
			gp.block_2D++;
		}

		// Bruker stacked 2D mode (T1, diffusion, etc)
		else if (q.data_type == data_types.BRUKER && gp.block_2D < gp.n_traces - 1
				&& gp.display_mode == modes.MODE_1D && gp.n_traces > 0)
		{
			gp.block_2D++;
			for (int i = 0; i < q.SI; i++)
			{
				gp.data_array[i] = gp.stack_trace[gp.block_2D].data[i];
				gp.i_data_array[i] = gp.stack_trace[gp.block_2D].idata[i];
			}
		}

		else if ((q.data_mode == modes.MODE_2D || q.data_mode == modes.MODE_3D)
				&& gp.n_traces == 0)
		{
			// in 1D display mode, step through individual rows of 2D matrix
			if (gp.block_2D < q_f1.SI - 1 && gp.rows_real != null
					&& gp.display_mode == modes.MODE_1D)
			{
				gp.block_2D++;
				Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);
			}

			// in 2D dispay mode step through displayed slices
			else
			{
				if (gp.active_slice < gp.nslices - 1)
					gp.active_slice = gp.active_slice + 1;
			}
		}          

		// Varian, Anasazi multifid (T1, DEPT, etc.)
		else if ((q.data_type == data_types.VNMR) || (q.data_type == data_types.CDFF) 
				&& gp.vblock < q.nblocks)
		{
			gp.vblock++;
			p.PH_mod =phasing.CONSTANTS;							// we assume that the we want 				
			Proc_1D.proc_1D(gp, p, q);                              // as the first block
		}

		update_spect = true;		
		canvas.redraw(); 
	}
	
//-------------------------------------------------------------------------------------------------
	public static void posOnlyButtonClick()
	{
		if(gp.pos_only)				                // switch from positive only to pos and neg
		{
            gp.pos_only = false;
            posOnlyButton.setText("Pos Only");
            posOnlyButton.setToolTipText("Switch to positive only contours");
			Util_2D.calc_default_colours(gp);
		}
		else							            // switch from pos and neg to positive only
		{
			gp.pos_only = true;
            posOnlyButton.setText("Pos/Neg");
            posOnlyButton.setToolTipText("Switch to positive and negative contours");
			Util_2D.calc_pos_only_colors(gp);
		}
		
		gp.new_2D_view = true;			            // regenerate 2D display
		update_spect = true;
	    canvas.redraw();
	}
	
//-------------------------------------------------------------------------------------------------
	public static void contourButtonClick()
	{
		if (gp.display_2D == SW_main.display_modes.IMAGE)
		{
			gp.display_2D = SW_main.display_modes.CONTOUR;
			contourButton.setText("Image");
			contourButton.setToolTipText("Switch to 2D image mode");
		}
		else if (gp.display_2D == SW_main.display_modes.CONTOUR)
		{
			gp.display_2D = SW_main.display_modes.IMAGE;
			contourButton.setText("Contour");
			contourButton.setToolTipText("Switch to 2D contour mode");
		}
		gp.new_2D_view = true;
		update_spect = true;
	    canvas.redraw();	
	}
	
	public static void colButtonClick()
    {  

        if (gp.row_col == trace_modes.ROWS)     //  now a three state button
        {
        	gp.row_col = trace_modes.COLUMNS;
            colButton.setText("Label");
            colButton.setToolTipText("Switch to\n 2D peak label mode");
            if(gp.nslices < 1)
            	statusStripLabel4.setText("  Mouse:  (set cursor,  zoom,  extract column)");
            else
            	statusStripLabel4.setText("  Mouse:  (set cursor / select trace,  zoom,  extract column)");
        }
        
        else if (gp.row_col == trace_modes.COLUMNS)
        {
        	gp.row_col = trace_modes.LABEL;
            colButton.setText("Rows");
            colButton.setToolTipText("Switch to rows mode");
            if(gp.nslices < 1)
            	statusStripLabel4.setText("  Mouse:  (set cursor,  zoom,  Label Peak)");
            else
            	statusStripLabel4.setText("  Mouse:  (set cursor / select trace,  zoom,  Label Peak)");
        }
        
        else if (gp.row_col == trace_modes.LABEL)
        {
        	gp.row_col = trace_modes.ROWS;
            colButton.setText("Col");
            colButton.setToolTipText("Switch to columns mode");
            if(gp.nslices < 1)
            	statusStripLabel4.setText("  Mouse:  (set cursor,  zoom,  extract row)");
            else
            	statusStripLabel4.setText("  Mouse:  (set cursor / select trace,  zoom,  extract row)");
        }
        
	    gp.nslices = 0;				                // re-set slice counter
	    gp.cursor_flag = 0;			                // clear any cursors
	    update_spect = true;
	    canvas.redraw();			           	 	// refresh screen
	    //set_tool_tips();				            // tool tips may have changed
    }
	
//-------------------------------------------------------------------------------------------------
}	// end class
