
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

import java.awt.Desktop;
import java.io.File;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MenuAdapter;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.FileDialog;

import dialogs.DisplayColourDialog;
import dialogs.ParEditor;
import dialogs.PlotParEditor;
import dialogs.PlotTitleEditor;

public class EditMenuListeners extends SW_main
{
	public static void addEditMenuListeners()
	{       
		// edit Menu, update menu items when dropped				
		editMenu.addMenuListener  (new MenuAdapter() 
		{
			public void menuShown(MenuEvent e) 
			{			
				if((gp.display_mode == modes.MODE_1D && gp.data_array != null)
						|| (gp.display_mode == modes.MODE_2D && gp.rows_real != null))
				{
					editCopyToFilePNG.setEnabled(true);
					editCopyToFilePNG_As.setEnabled(true);
					editCopyToFileJPG.setEnabled(true);
				}	
				else
				{
					editCopyToFilePNG.setEnabled(false);
					editCopyToFilePNG_As.setEnabled(false);
					editCopyToFileJPG.setEnabled(false);
				}
			}
		});
				
         // Proc Pars
         editProcParsItem.addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
 				ParEditor dlg 
 				= new ParEditor(shell, 
 						SWT.APPLICATION_MODAL | SWT.TITLE | SWT.BORDER );

 				dlg.open();
        	 }
         });

         // Plot Pars 
         editPlotParsItem.addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 PlotParEditor dlg 
  				= new PlotParEditor(shell, 
  						SWT.APPLICATION_MODAL | SWT.TITLE | SWT.BORDER );

  				dlg.open();
        	 }
         });         
         
         // Display Colours 
         editDisplayColoursItem.addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		DisplayColourDialog dlg 
        		= new DisplayColourDialog(shell, SWT.APPLICATION_MODAL | SWT.TITLE | SWT.BORDER);
        		
  				dlg.open();
  				
  				update_spect = true;
  				canvas.redraw();
        	 }
         });
         
         // Plot title
         editPlotTitleItem.addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 PlotTitleEditor dlg 
        		 = new PlotTitleEditor(shell, 
        				 SWT.APPLICATION_MODAL | SWT.TITLE | SWT.BORDER );

        		 dlg.open(); 
        		 update_spect = true;
        		 canvas.redraw();
        	 }
         });
         
         
         editCopyToFilePNG.addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 if((gp.display_mode == modes.MODE_1D && gp.data_array == null)
	    					|| (gp.display_mode == modes.MODE_2D && gp.rows_real == null))
	    				return;
        		 
        		 PNG_Util.doPrintToFile(SW_main.q.data_folder + "spect.png", SWT.IMAGE_PNG);
        		       		 
        		 if (Desktop.isDesktopSupported())
        		 {
        			 try 
        			 {
        				 Desktop.getDesktop().open(new File(SW_main.q.data_folder + "spect.png"));
        			 }  
        			 catch (Exception ex) // IO Exception and nullPointerException??
        			 {
        				 Util.show_and_log("no application for this file type?? " +
        						 SW_main.q.data_folder + "spect.png", ex.getMessage());
        			 }			    
        		 }
        		 else
        			 Util.show_and_log("Current desktop not supported");      		 
        	 }
         });
         
         editCopyToFilePNG_As.addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 if((gp.display_mode == modes.MODE_1D && gp.data_array == null)
	    					|| (gp.display_mode == modes.MODE_2D && gp.rows_real == null))
	    				return;
        		 
        		 FileDialog fd = new FileDialog(shell, SWT.SAVE);
	             fd.setText("Save spectrum graphics as a PNG file");
	             if((q.data_type == data_types.BRUKER || q.data_type == data_types.VNMR) 
	            		 && q.data_folder != null)
            		 fd.setFilterPath(q.data_folder);
            	 else if(gp.default_folder != null)
            		 fd.setFilterPath(gp.default_folder);	             
	             String[] filterExt = {"*.png"};
	             fd.setFilterExtensions(filterExt);

	             String result = fd.open();
	             
        		 PNG_Util.doPrintToFile(result, SWT.IMAGE_PNG);
        		       		 
        		 if (Desktop.isDesktopSupported())
        		 {
        			 try 
        			 {
        				 Desktop.getDesktop().open(new File(result));
        			 }  
        			 catch (Exception ex) // IO Exception and nullPointerException??
        			 {
        				 Util.show_and_log("no application for this file type?? " +
        						 SW_main.q.data_folder + "spect.png", ex.getMessage());
        			 }			    
        		 }
        		 else
        			 Util.show_and_log("Current desktop not supported");      		 
        	 }
         });
         
         editCopyToFileJPG.addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 if((gp.display_mode == modes.MODE_1D && gp.data_array == null)
	    					|| (gp.display_mode == modes.MODE_2D && gp.rows_real == null))
	    				return;
        		 
        		 PNG_Util.doPrintToFile(SW_main.q.data_folder + "spect.jpg", SWT.IMAGE_JPEG);
        	 
        		 if (Desktop.isDesktopSupported())
        		 {
        			 try 
        			 {
        				 Desktop.getDesktop().open(new File(SW_main.q.data_folder + "spect.jpg"));
        			 }  
        			 catch (Exception ex) // IO Exception and nullPointerException??
        			 {
        				 Util.show_and_log("no application for this file type?? " +
        						 SW_main.q.data_folder + "spect.jpg", ex.getMessage());
        			 }			    
        		 }
        		 else
        			 Util.show_and_log("Current desktop not supported");  
        	 }
         });
	}
//-------------------------------------------------------------------------------------------------
}
