
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MenuAdapter;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.MessageBox;

import dialogs.PrintPreviewDialog;
import dialogs.RecentFilesDialog;

public class FileMenuListeners extends SW_main
{
	public static void addFileMenuListeners()
	{

		// file Menu, update menu items when dropped				
		fileMenu.addMenuListener  (new MenuAdapter() // menuListener??  Listener??
		{
			public void menuShown(MenuEvent e) 
			{
				// TODO
				
				if(gp.display_mode == modes.MODE_2D && gp.rows_real != null)
				{
					fileReadF1Item.setEnabled(true);
					fileReadF2Item.setEnabled(true);
				}
				else
				{
					fileReadF1Item.setEnabled(false);
					fileReadF2Item.setEnabled(false);
				}
				
				if(gp.display_mode == modes.MODE_1D && gp.data_array != null)
				{
					fileSaveJDX_Item.setEnabled(true);
					fileSaveAsJDX_Item.setEnabled(true);
				}
				else
				{
					fileSaveJDX_Item.setEnabled(false);
					fileSaveAsJDX_Item.setEnabled(false);
				}			
				
				if(gp.display_mode == modes.MODE_2D && gp.nslices > 0)
				{
					fileSaveColumnsAsJDX_Item.setEnabled(true);
					fileSaveRowsAsJDX_Item.setEnabled(true);
				}
				else
				{
					fileSaveColumnsAsJDX_Item.setEnabled(false);
					fileSaveRowsAsJDX_Item.setEnabled(false);
				}
				
				if((gp.display_mode == modes.MODE_1D && gp.data_array != null)
				|| (gp.display_mode == modes.MODE_2D && gp.rows_real != null))
				{
					fileSaveItem.setEnabled(true);
					filePrintItem.setEnabled(true);
					filePrintPreviewItem.setEnabled(true);
				}
				else
				{
					fileSaveItem.setEnabled(false);
					filePrintItem.setEnabled(false);
					filePrintPreviewItem.setEnabled(false);
				}
				
				if((gp.display_mode == modes.MODE_1D && gp.data_array != null)
						|| (gp.display_mode == modes.MODE_FID && gp.fid != null))
					fileExportXY_Item.setEnabled(true);
				else
					fileExportXY_Item.setEnabled(false);					
					
				if(gp.expt_read)
					fileReadItem.setEnabled(true);
				else
					fileReadItem.setEnabled(false);
			}
		});
				
		// File Open
	     fileOpenItem.addSelectionListener  (new SelectionAdapter() 
	     {
	         @Override
	          public void widgetSelected(SelectionEvent e) 
	          {
	        	 FileDialog fd = new FileDialog(shell, SWT.OPEN);
	             fd.setText("Open an NMR FID file");
	             
	             
	             try	// Just so File Open doesn't fail if something is strange
	             {
	            	 if(q.data_type == data_types.BRUKER && q.data_folder != null)
	            		 fd.setFilterPath(q.data_folder);
	            	 else if(gp.default_folder != null)
	            		 fd.setFilterPath(gp.default_folder);
	             }
	             catch(Exception ex) { ex.printStackTrace();}
	             
	             String selected = fd.open();
	             if(selected != null)
	             {
	            	 q.data_mode = modes.MODE_1D;
	            	 FileOpen.openSelectedFile(selected);
	            	 update_spect = true;
	         		 canvas.redraw();					// temporary!
	             }
	          }
	     });
       
	     // File Read F1 trace
	     fileReadF1Item.addSelectionListener  (new SelectionAdapter() 
	     {
	    	 @Override
	    	 public void widgetSelected(SelectionEvent e) 
	    	 {
	    		 FileDialog fd = new FileDialog(shell, SWT.OPEN);
	             fd.setText("Open a JCAMP-DX file for F1 trace");
	             
	             try	// Just so File Open doesn't fail if something is strange
	             {
	            	 if(q.data_type == data_types.BRUKER && q.data_folder != null)
	            	 {
	            		 fd.setFilterPath(q.data_folder);
	            	 }
	            	 else if(gp.default_folder != null)
	            	 {
	            		 fd.setFilterPath(gp.default_folder);
	            	 }
	             }
	             catch(Exception ex) { ex.printStackTrace();}
	             
	             String selected = fd.open();
	             if(selected == null) return; 	// possibly cancel button selected
	                          
	             String[] name = selected.split("[.]+");
	     		 String extension = name[name.length - 1];
	     		 
	     		 if (!(extension.equals("dx") ||
	     				 extension.equals("jdx")))
	     		 {
	     			 Util.show_and_log("This does not seem to be a JCAMP DX file");
	     			 return;
	     		 }

	    		 if (DX_Util.read_jdx_f1(selected, gp) <= 0)
	    		 {
	    			 update_spect = true;
	         		 canvas.redraw();
	    			 return;     // could not read file, bail-out.
	    		 }
	    		 
	    		 gp.f1_trace_path = selected;
	    		 gp.f1_trace_on = true;
	    		 gp.f1_from_scan = false;
	    		 //TODO F1_trace.SelectedIndex = 1;

	    		 //	 	update display
	    		 statusStripLabel1.setText("Ready");
	    		 DisplayButtons.fullButtonClick();
	    	 }
	     });
	     
	     // File Read F2 trace
	     fileReadF2Item.addSelectionListener  (new SelectionAdapter() 
	     {
	    	 @Override
	    	 public void widgetSelected(SelectionEvent e) 
	    	 {
	    		 FileDialog fd = new FileDialog(shell, SWT.OPEN);
	             fd.setText("Open a JCAMP-DX file for F2 trace");
	             
	             try	// Just so File Open doesn't fail if something is strange
	             {
	            	 if(q.data_type == data_types.BRUKER && q.data_folder != null)
	            	 {
	            		 //System.out.println("setting data Folder" + q.data_folder);
	            		 fd.setFilterPath(q.data_folder);
	            	 }
	            	 else if(gp.default_folder != null)
	            	 {
	            		 //System.out.println("setting default Folder" + gp.default_folder);
	            		 fd.setFilterPath(gp.default_folder);
	            	 }
	             }
	             catch(Exception ex) { ex.printStackTrace();}
	             
	             String selected = fd.open();
	             if(selected == null) return;		// possibly cancel selected
	                          
	             String[] name = selected.split("[.]+");
	     		 String extension = name[name.length - 1];
	     		 
	     		 if (!(extension.equals("dx") ||
	     				 extension.equals("jdx")))
	     		 {
	     			 Util.show_and_log("This does not seem to be a JCAMP DX file");
	     			 return;
	     		 }

	    		 if (DX_Util.read_jdx_f2(selected, gp) <= 0)
	    		 {
	    			 update_spect = true;
	         		 canvas.redraw();
	    			 return;     // could not read file, bail-out.
	    		 }
	    		 
	    		 gp.f2_trace_path = selected;
	    		 gp.f2_trace_on = true;
	    		 gp.f2_from_scan = false;
	    		 //TODO F2_trace.SelectedIndex = 1;

	    		 //	 	update display
	    		 statusStripLabel1.setText("Ready");
	    		 DisplayButtons.fullButtonClick();
	    	 }
	     });     
	     
	     // File Save JDX
	     fileSaveJDX_Item.addSelectionListener  (new SelectionAdapter() 
	     {
	    	 @Override
	    	 public void widgetSelected(SelectionEvent e) 
	    	 {
	    		 shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
	    		 if (q.data_mode == modes.MODE_1D)
	    			 DX_Util.write_jdx(q.data_folder + "spectrum.dx", gp, q);
	    		 shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
	    		 statusStripLabel1.setText("Ready");
	             statusStripLabel2.setText("Data saved as JCAMP-DX");
	    	 }
	     });	
	     
	     // File Save As JDX
	     fileSaveAsJDX_Item.addSelectionListener  (new SelectionAdapter() 
	     {
	    	 @Override
	    	 public void widgetSelected(SelectionEvent e) 
	    	 {
	    		 shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
	    		 
	    		 FileDialog fd = new FileDialog(shell, SWT.SAVE);
	             fd.setText("Save procesed NMR data as a named JCAMP-DX file");
	             
	             
	             try	// Just so File Open doesn't fail if something is strange
	             {
	            	 if(q.data_type == data_types.BRUKER && q.data_folder != null)
	            		 fd.setFilterPath(q.data_folder);
	            	 else if(gp.default_folder != null)
	            		 fd.setFilterPath(gp.default_folder);
	             }
	             catch(Exception ex) { ex.printStackTrace();}
	             
	             String selected = fd.open();
	             DX_Util.write_jdx(selected, gp, q);
	             
	             statusStripLabel1.setText("Ready");
	             statusStripLabel2.setText("Data saved as spectrum.dx");	             
	    		 shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
	    	 }
	     });	
	          
	     // File Save 2D columns as JDX
	     fileSaveColumnsAsJDX_Item.addSelectionListener  (new SelectionAdapter() 
	     {
	    	 @Override
	    	 public void widgetSelected(SelectionEvent e) 
	    	 {
	    		 shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));

	    		 for(int i = 0; i < gp.nslices && i < 32; i++)
	    		 {
	    			 String fname = String.format("%scolumn%d.dx", q.data_folder, gp.slice_no[i]);

	    			 if (DX_Util.write_jdx_f1(fname, gp.slice_no[i], gp, q, q_f1) <= 0)
	    			 {
	    				 Util.show_and_log("write_jdx_f1(): error writing columns");
	    				 break;
	    			 }
	    		 }
	    		 statusStripLabel1.setText("Ready");
	             statusStripLabel2.setText("2D columns saved as spectrum.dx");
	    		 shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
	    	 }
	     });	 
	     
	  // File Save 2D rows as JDX
	     fileSaveRowsAsJDX_Item.addSelectionListener  (new SelectionAdapter() 
	     {
	    	 @Override
	    	 public void widgetSelected(SelectionEvent e) 
	    	 {
	    		 shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));

	    		 for(int i = 0; i < gp.nslices && i < 32; i++)
	    		 {
	    			 String fname = String.format("%srow%d.dx", q.data_folder, gp.slice_no[i]);

	    			 if (DX_Util.write_jdx_f2(fname, gp.slice_no[i], gp, q, q_f1) <= 0)
	    			 {
	    				 Util.show_and_log("write_jdx_f1(): error writing rows");
	    				 break;
	    			 }
	    		 }
	    		 statusStripLabel1.setText("Ready");
	             statusStripLabel2.setText("2D columns saved as spectrum.dx");
	    		 shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
	    	 }
	     });	 
	     	     
	     // File Save: SpinWorks format (also saves as JDX in 1D mode
	     fileSaveItem.addSelectionListener  (new SelectionAdapter() 
	     {
	    	 @Override
	    	 public void widgetSelected(SelectionEvent e) 
	    	 {
	    		 shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));

	 			if (q.data_mode == modes.MODE_1D)
	 			{
	 				FileUtil.saveProc1D(gp, q);
	 				statusStripLabel1.setText("Ready");
	                DX_Util.write_jdx(q.data_folder + "spectrum.dx", gp, q);
	 			}
	 			else if (q.data_mode == modes.MODE_2D)
	 			{
	 				FileUtil.saveProc2D(false, gp, q, q_f1);
	 				statusStripLabel1.setText("Ready");
	 			}
	 			statusStripLabel1.setText("Ready");
	            statusStripLabel2.setText("Processed data saved");
	 			shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
	    	 }
	     });	// end fileSaveItem.addSelectionListener 
	     
	  // Read processed data (SpinWorks format)
	     fileReadItem.addSelectionListener  (new SelectionAdapter() 
	     {
	    	 @Override
	    	 public void widgetSelected(SelectionEvent e) 
	    	 {
	    		 FileOpen.readProcessed();
	    	 }
	     });	// end fileSaveItem.addSelectionListener 
	     
	  // Export data as XY pairs
	     fileExportXY_Item.addSelectionListener  (new SelectionAdapter() 
	     {
	    	 @Override
	    	 public void widgetSelected(SelectionEvent e) 
	    	 {
	    		 FileDialog fd = new FileDialog(shell, SWT.SAVE);
	             fd.setText("Save Data as XY pairs");
	             fd.setFilterPath(q.data_folder);
	          	             
	             String result = fd.open();
	             
	             if(result != null)
	            		 FileUtil.exportXY(result);
	    	 }
	     });	// end fileExportXY_Item.addSelectionListener 
	     
	  // File Print
	     filePrintItem.addSelectionListener  (new SelectionAdapter() 
	     {
	    	 @Override
	    	 public void widgetSelected(SelectionEvent e) 
	    	 {
	    		if((SW_main.gp.display_mode == modes.MODE_1D && SW_main.gp.data_array == null)
	    					|| (SW_main.gp.display_mode == modes.MODE_2D && SW_main.gp.rows_real == null))
	    				return;
	    		PrintHandler.doPrint();
	    	 }
	     }); // end filePrintItem.addSelectionListener 
	     
	   // File Print Preview
	     filePrintPreviewItem.addSelectionListener  (new SelectionAdapter() 
	     {
	    	 @Override
	    	 public void widgetSelected(SelectionEvent e) 
	    	 {
	    		 if((gp.display_mode == modes.MODE_1D && gp.data_array == null)
	    					|| (gp.display_mode == modes.MODE_2D && gp.rows_real == null))
	    				return;
	        	 PrintPreviewDialog dlg = new PrintPreviewDialog(shell, 
	        			 SWT.APPLICATION_MODAL | SWT.TITLE | SWT.BORDER);
	             dlg.setText("Print Preview");
	             dlg.open();	    		
	    	 }
	     }); // end filePrintPreviewItem.addSelectionListener 
	     
	   // File Recent Files dialog
	     fileRecentItem.addSelectionListener  (new SelectionAdapter() 
	     {
	    	 @Override
	    	 public void widgetSelected(SelectionEvent e) 
	    	 {
	    		 
	    		 RecentFilesDialog dlg = new RecentFilesDialog(shell, SWT.APPLICATION_MODAL | SWT.TITLE | SWT.BORDER);
	    		 Object result = dlg.open();
	    		 if(result == null)
	    			 return;
	    		 
	    		 if((int)result >= 0 && (int)result < 8)
	    		 {
	    			 q.data_mode = modes.MODE_1D;
	            	 FileOpen.openSelectedFile(file_obj.file_path[(int)result]);
	            	 update_spect = true;
	         		 canvas.redraw();
	    		 }
	  
	    	 }
	     }); // end fileRecentItem
	     
	  // File Exit, with confirmation
	     fileExitItem.addSelectionListener  (new SelectionAdapter() 
	     {
	    	 @Override
	    	 public void widgetSelected(SelectionEvent e) 
	    	 {
	    		 MessageBox messageBox = new MessageBox(shell, SWT.ICON_QUESTION
	    				 | SWT.YES | SWT.NO);
	    		 messageBox.setMessage("Do you really want to exit?");
	    		 messageBox.setText("SpinWorks Java Exiting Application");
	    		 int response = messageBox.open();
	    		 if (response == SWT.YES)
	    		 {
	    			 PreferenceUtil.setPrefs();
	    			 PreferenceUtil.setMru();
	    			 System.exit(0);
	    		 }
	    	 }
	     });
	}
//-------------------------------------------------------------------------------------------------
}
