
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
public class FileUtil extends SW_main
{
//-------------------------------------------------------------------------------------------------	
/**
 * save integral regions to disk
 */
	public static void saveInteg1D()
	{
		if (SW_main.gp.n_integrals < 1) return;

		int i = 0;

		PrintWriter pw;
		
		String fileName = null;
		
		if(q.data_type != SW_main.data_types.JEOL)	//TODO clean this up
			fileName = q.data_folder + "1D_integ";	
		else if(q.data_type == SW_main.data_types.JEOL
				 || q.data_type == SW_main.data_types.JCAMP_DX)
			fileName = q.data_path + "_SW_proc/1D_integ";
		
		//	open file
		try { pw = new PrintWriter(fileName); }
		catch (Exception ex)
		{
			Util.show_and_log("save_1D_integ(): " + fileName, ex.getMessage());
			return;
		}

		//	now write the integral regions
		for (i = 0; i < SW_main.gp.n_integrals; i++)
			if (SW_main.gp.integrals[i].start > -1.0e30)
			{
				pw.println(String.format("%.3f  %.3f  %.1f  %.1f",
						SW_main.gp.integrals[i].start, SW_main.gp.integrals[i].end, 
						SW_main.gp.integrals[i].bias, SW_main.gp.integrals[i].slope));
			}

		pw.close(); ;
	}

//-------------------------------------------------------------------------------------------------
/**
 * get 1D integral regions
 * @return true if OK, false otherwise
 */
	public static boolean readInteg1D()
	{
		String instring;
		String delim = "[ =]+";
		String[] token;
		String fileName = null;
		
		if(q.data_type != SW_main.data_types.JEOL)	//TODO clean this up
			fileName = q.data_folder + "1D_integ";	
		else if(q.data_type == SW_main.data_types.JEOL
				|| q.data_type == SW_main.data_types.JCAMP_DX)
			fileName = q.data_path + "_SW_proc/1D_integ";

		File fi = new File(fileName);
		if(!fi.exists())
			return false;
		
		// try to open the integral file 
		BufferedReader br;
		try { br = new BufferedReader(new FileReader(fileName)); }
		catch (Exception ex)
		{
			//util.show_and_log("read1DInteg(), cannot open: " + fileName, ex.getMessage());
			return false;
		}

		gp.n_integrals = 0;

		for(;;)
		{
			try {instring = br.readLine();} 
			catch (IOException ex) 
			{
				ex.printStackTrace();
				Util.show_and_log("read1DInteg(), read error: " + fileName, ex.getMessage());
				try { br.close(); } 
				catch (IOException e) { e.printStackTrace(); }
				return false;
			}
			
			if(instring == null) break;
			
			if (instring.length() > 3)
			{
				token = instring.split(delim);       
				if (token[0].equals("##END")) break; // necessary??

				if (token.length == 2)      // old format integ regions file
				{
					try
					{
						gp.integrals[gp.n_integrals] = new IntegInfo();	// TODO necessaary??
						gp.integrals[gp.n_integrals].start = Double.parseDouble(token[0]);
						gp.integrals[gp.n_integrals].end = Double.parseDouble(token[1]);
						gp.integrals[gp.n_integrals].bias = 0.0;
						gp.integrals[gp.n_integrals].slope = 0.0;
						gp.n_integrals++;
					}
					catch (Exception ex)
					{ 
						Util.show_and_log("read_1D_integ(): " + instring,  ex.getMessage());
					}
				}
				else if (token.length == 4)     //  new format integ regions file
				{
					try
					{
						gp.integrals[gp.n_integrals] = new IntegInfo();	// TODO necessaary??
						gp.integrals[gp.n_integrals].start = Double.parseDouble(token[0]);
						gp.integrals[gp.n_integrals].end = Double.parseDouble(token[1]);
						gp.integrals[gp.n_integrals].bias = Double.parseDouble(token[2]);
						gp.integrals[gp.n_integrals].slope = Double.parseDouble(token[3]);
						gp.n_integrals++;
					}
					catch (Exception ex)
					{
						Util.show_and_log("read_1D_integ(): " + instring, ex.getMessage());
					}
				}
			}
		}
		
		try{br.close();}
		catch (Exception ex) {ex.printStackTrace();}
		
		// now calculate integral values
		for (int i = 0; i < gp.n_integrals; i++)
        {
            double value = 0.0;
            int point1, point2;
           
            point1 = (int) Math.round((q.offset - gp.integrals[i].start) / q.hzperpt);
            point2 = (int) Math.round((q.offset - gp.integrals[i].end) / q.hzperpt);
        
            for (int j = point1; j <= point2; j++)
                value = value + gp.data_array[j]
                    + gp.integrals[i].bias
                    + gp.integrals[i].slope * ((double)j - (double)point1);
           
            if (i == 0)
                gp.integ_cal = value;

            gp.integrals[i].value = value;
            gp.integrals[i].scaled = value / gp.integ_cal;
        }
		return true;
	}
	
//-------------------------------------------------------------------------------------------------
/**
 * Read 1D peak file
 * @param gp a SpinWorks global parameter file
 * @param q status acquisition parameters in the direct dimension
 */
	public static int readPeaks1D(global_par gp, data_par q)
	{
		String instring;
		String delim = "[ =]+";
		String[] token;
		String fileName = null;

		if(q.data_type != SW_main.data_types.JEOL)	//TODO clean this up
			fileName = q.data_folder + "peaks.txt";	
		else if(q.data_type == SW_main.data_types.JEOL
				|| q.data_type == SW_main.data_types.JCAMP_DX)
			fileName = q.data_path + "_SW_proc/peaks.txt";
		
		// try to open the peaks file 
		BufferedReader br;
		try { br = new BufferedReader(new FileReader(fileName)); }
		catch (Exception ex)
		{
			return -1;	// silent exit
		}

		try
		{
			instring = br.readLine();   // read and discard the 3 header lines 
			instring = br.readLine();
			instring = br.readLine();
		}
		catch(Exception ex)
		{
			try{br.close();}
			catch (Exception e) {e.printStackTrace();}
			return -1;
		}

		gp.n_peaks = 0;

		for(;;)
		{
			try {instring = br.readLine();} 
			catch (IOException ex) 
			{
				ex.printStackTrace();
				Util.show_and_log("readPeaks(), read error: " + fileName, ex.getMessage());
				try{br.close();}
				catch (Exception e) {e.printStackTrace();}
				return gp.n_peaks;
			}

			if(instring == null) 		
				break;

			
			instring = instring.trim();
			if (instring.length() > 3)
			{
				token = instring.split(delim); 

				if (token[0].length() >= 1)
				{
					try
					{
						gp.peaks[gp.n_peaks] = new PeakInfo();
						gp.peaks[gp.n_peaks].freq = Double.parseDouble(token[1]);
						gp.peaks[gp.n_peaks].intensity = Double.parseDouble(token[4]);	
						if(token.length > 5)
							gp.peaks[gp.n_peaks].label = new String(token[5]);
						gp.n_peaks++;
					}
					catch (Exception ex)
					{
						Util.show_and_log("readPeaks1D(): " + instring, ex.getMessage());
						break;
					}					
				}
			}
		}
		try{br.close();}
		catch (Exception ex) {ex.printStackTrace();}
		
		return gp.n_peaks;
	}
	
//-------------------------------------------------------------------------------------------------
/**
 * Save 2D peak assignments to a file
 * @param gp A SpinWorks glbal parameter object
 * @param q status data parameters
 */
	 public static void savePeaks2D(global_par gp, data_par q)
     {
         if (gp.n_peaks < 1) return;

         int i = 0;

         PrintWriter pw;
         String fileName = q.data_folder + "2D_peaks";	// data folder already has trailing "/"

         //	write the peaks list
         try { pw = new PrintWriter(fileName); }
         catch (Exception ex)
         {
             Util.show_and_log("save_2D_peaks(): " + fileName, ex.getMessage());
             return;
         }

//	now write the peaks
         for (i = 0; i < gp.n_peaks; i++)
             if (gp.peaks[i].freq > -1.0e9)
             {
                 pw.println(String.format("PEAK %.4f  %.4f  %.4f  %s",
                 gp.peaks[i].freq, gp.peaks[i].freq_f1, 
                 gp.peaks[i].intensity, gp.peaks[i].label));

                 pw.print("COSY ");
                 for (int j = 0; j < gp.peaks[i].n_cosy; j++)
                     pw.print(String.format(" %.5f", gp.peaks[i].cosy[j]));
                 pw.println();
                 
                 pw.print("COSY_F1 ");
                 for (int j = 0; j < gp.peaks[i].n_cosy; j++)
                     pw.print(String.format(" %.5f", gp.peaks[i].cosy_f1[j]));
                 pw.println();

                 pw.print("TOCSY ");
                 for (int j = 0; j < gp.peaks[i].n_tocsy; j++)
                     pw.print(String.format(" %.5f", gp.peaks[i].tocsy[j]));
                 pw.println();
                 
                 pw.print("TOCSY_F1 ");
                 for (int j = 0; j < gp.peaks[i].n_tocsy; j++)
                     pw.print(String.format(" %.5f", gp.peaks[i].tocsy_f1[j]));
                 pw.println();

                 pw.print("NOESY ");
                 for (int j = 0; j < gp.peaks[i].n_noesy; j++)
                     pw.print(String.format(" %.5f", gp.peaks[i].noesy[j]));
                 pw.println();
                 
                 pw.print("NOESY_F1 ");
                 for (int j = 0; j < gp.peaks[i].n_noesy; j++)
                     pw.print(String.format(" %.5f", gp.peaks[i].noesy_f1[j]));
                 pw.println();

                 pw.print("HMBC ");
                 for (int j = 0; j < gp.peaks[i].n_hmbc; j++)
                     pw.print(String.format(" %.5f", gp.peaks[i].hmbc[j]));
                 pw.println();
                 
                 pw.print("HMBC_F1 ");
                 for (int j = 0; j < gp.peaks[i].n_hmbc; j++)
                     pw.print(String.format(" %.5f", gp.peaks[i].hmbc_f1[j]));
                 pw.println();
                 pw.println("#");
             }

         pw.close();
     }

//-------------------------------------------------------------------------------------------------
 /**
  * loads any 2D peak assignments
  * @param gp global parameter object
  * @param q status parameters in acquisition dimension
  */
	 public static void readPeaks2D(global_par gp, data_par q)
	 {
		 String instring;
		 String delim = "[ =]+";
		 String[] token;
		 String fileName = q.data_folder + "2D_peaks";

		 //try to open the 2D peaks file 
		 BufferedReader br;
		 try { br = new BufferedReader(new FileReader(fileName)); }
		 catch (Exception ex)
		 {
			 Util.log_error("cannot open " + fileName, ex.getMessage());
			 return;
		 }


		 gp.n_peaks = 0;

		 for (;;)
		 {
			 try {instring = br.readLine();} 
			 catch (IOException ex) 
			 {
				 ex.printStackTrace();
				 Util.show_and_log("get_par(), read error: " + fileName, ex.getMessage());
				 try { br.close(); } 
				 catch (IOException e) { e.printStackTrace(); }
				 return;
			 }

			 if(instring == null) break;

			 if (instring.length() > 3)
			 {
				 token = instring.split(delim);

				 if (token[0].length() >= 3)
				 {
					 try
					 {
						 if (token[0].equals("PEAK"))
						 {
							 gp.n_peaks++;
							 gp.peaks[gp.n_peaks-1] = new PeakInfo();

							 gp.peaks[gp.n_peaks-1].freq =
									 Double.parseDouble(token[1]);

							 gp.peaks[gp.n_peaks-1].freq_f1 =
									 Double.parseDouble(token[2]);

							 gp.peaks[gp.n_peaks-1].intensity =
									 Double.parseDouble(token[3]);

							 for (int i = 4; i < token.length; i++)
								 gp.peaks[gp.n_peaks-1].label
								 = gp.peaks[gp.n_peaks-1].label + " " + token[i];                              
						 }
						 
						 else if (token[0].equals("COSY"))
						 {
							 for (int i = 1; i < token.length; i++)
							 {
								 gp.peaks[gp.n_peaks-1].cosy[i - 1]
										 = Double.parseDouble(token[i]);
								 gp.peaks[gp.n_peaks-1].n_cosy++;
							 }
						 }
						 
						 else if (token[0].equals("COSY_F1"))
						 {
							 for (int i = 1; i < token.length; i++)
							 {
								 try
								 {
									 gp.peaks[gp.n_peaks-1].cosy_f1[i - 1]
											 = Double.parseDouble(token[i]);
								 }
								 catch(Exception ex1)
								 {
									 System.out.println(String.format("ex1 i = %d  n_peaks = %d token = %s", i, gp.n_peaks, token[i]));
									 System.out.println(ex1.getMessage() + lineSep);
								 }
							 }
						 }
						 
						 else if (token[0].equals("TOCSY"))
						 {
							 for (int i = 1; i < token.length; i++)
							 {
								 gp.peaks[gp.n_peaks-1].tocsy[i - 1]
										 = Double.parseDouble(token[i]);
								 gp.peaks[gp.n_peaks-1].n_tocsy++;
							 }
						 } 
						 
						 else if (token[0].equals("TOCSY_F1"))
						 {
							 for (int i = 1; i < token.length; i++)
							 {
								 try
								 {
									 gp.peaks[gp.n_peaks-1].tocsy_f1[i - 1]
											 = Double.parseDouble(token[i]);
								 }
								 catch(Exception ex1)
								 {
									 System.out.println(String.format("ex1 i = %d  n_peaks = %d token = %s", i, gp.n_peaks, token[i]));
									 System.out.println(ex1.getMessage() + lineSep);
								 }
							 }
						 }
						 
						 else if (token[0].equals("NOESY"))
						 {
							 for (int i = 1; i < token.length; i++)
							 {
								 gp.peaks[gp.n_peaks-1].noesy[i - 1]
										 = Double.parseDouble(token[i]);
								 gp.peaks[gp.n_peaks-1].n_noesy++;
							 }
						 }
						 
						 else if (token[0].equals("NOESY_F1"))
						 {
							 for (int i = 1; i < token.length; i++)
							 {
								 try
								 {
									 gp.peaks[gp.n_peaks-1].noesy_f1[i - 1]
											 = Double.parseDouble(token[i]);
								 }
								 catch(Exception ex1)
								 {
									 System.out.println(String.format("ex1 i = %d  n_peaks = %d token = %s", i, gp.n_peaks, token[i]));
									 System.out.println(ex1.getMessage() + lineSep);
								 }
							 }
						 }
						 
						 else if (token[0].equals("HMBC"))
						 {
							 for (int i = 1; i < token.length; i++)
							 {
								 gp.peaks[gp.n_peaks-1].hmbc[i - 1]
										 = Double.parseDouble(token[i]);
								 gp.peaks[gp.n_peaks-1].n_hmbc++;
							 }
						 } 
						 
						 else if (token[0].equals("HMBC_F1"))
						 {
							 for (int i = 1; i < token.length; i++)
							 {
								 try
								 {
									 gp.peaks[gp.n_peaks-1].hmbc_f1[i - 1]
											 = Double.parseDouble(token[i]);
								 }
								 catch(Exception ex1)
								 {
									 System.out.println(String.format("ex1 i = %d  n_peaks = %d token = %s", i, gp.n_peaks, token[i]));
									 System.out.println(ex1.getMessage() + lineSep);
								 }
							 }
						 }
						 
					 }
					 catch (Exception ex)
					 {
						 Util.show_and_log("read_2D_peaks(): " + instring, ex.getMessage());
					 }
				 }	// if(token.length...
			 }	// if(insttring.length...
		 }	// for(;;)

		 try {	br.close();	} 
		 catch (IOException e) {	e.printStackTrace();	}
	 }
	 
//-------------------------------------------------------------------------------------------------
/**
 * Save processed 1D data
 * @param gp global paramater object
 * @param q status parameters, acquisition dimension
 * @return bytes written, -1 if wrong type of data, -2 if reals couldn't be written,
 * -3 if imaginaries couldn't be written, 0 if files were opened but no data could be written
 */
	public static int saveProc1D(global_par gp, data_par q)
	{
		/*if (q.data_type == SW_main.data_types.JCAMP_DX)
			return -1;*/
		
		int i;
		int bytes_written = 0;
		double scale_factor = 1.0;

		String file_name_real;
		String file_name_imag;
		String file_name_par;
		
		if(q.data_type == SW_main.data_types.BRUKER 	// TODO allow at least JEOL??
				|| q.data_type == SW_main.data_types.VNMR)
		{
			file_name_real = q.data_folder + "1D_real";
			file_name_imag = q.data_folder + "1D_imag";
			file_name_par = q.data_folder + "1D_par";
		}
		else if(q.data_type == SW_main.data_types.JEOL
				|| q.data_type == SW_main.data_types.JCAMP_DX)
		{
			File fi = new File(q.data_path + "_SW_proc");
			if(!fi.exists())
				fi.mkdir();
			
			file_name_real = q.data_path + "_SW_proc/1D_real";
			file_name_imag = q.data_path + "_SW_proc/1D_imag";
			file_name_par = q.data_path + "_SW_proc/1D_par";
		}
		else
			return -1;

//	write the parameter file
		PrintWriter pw;

		//	open file
		try { pw = new PrintWriter(file_name_par); }
		catch (Exception ex)
		{
			Util.show_and_log("save_prod_1D(): " + file_name_par, ex.getMessage());
			return -4;
		}

		DateFormat df = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
    	Date dateobj = new Date();

		try
		{
			pw.println(String.format("## SpinWorksJ 0.0 1D parameters for: %s", q.data_folder));          
			pw.println("## saved: " + df.format(dateobj) + lineSep );
			/*String temp = gp.plot_title.replace('\n', ';'); // replace CR LF pair
			temp = temp.replace('\r', ';');
			pw.println("title= " + temp);*/
			
			gp.plot_title.replaceAll(lineSep, ";");
			pw.println("title= " + gp.plot_title.replaceAll(lineSep, ";"));

			pw.println(String.format("size= %d", q.SI));
			pw.println(String.format("sf= %.8f", q.SF));
			pw.println(String.format( "sf_trans= %.8f", q.SFO1));
			pw.println(String.format( "sw= %.3f", q.SFO1 * q.SW));

			pw.println(String.format( "scale_factor= %.4f", scale_factor));
			pw.println(String.format( "vscale= %.12f", gp.expt_scale));
			pw.println(String.format( "ymax= %.3f", gp.data_max));
			pw.println(String.format( "data_max= %.2f", gp.data_max));
			pw.println(String.format( "freq1= %.4f", gp.freq1));
			pw.println(String.format( "freq2= %.4f", gp.freq2));
			pw.println(String.format( "integ_scale= %.12f", gp.integ_scale));
			pw.println(String.format( "integ_cal= %.12f", gp.integ_cal));

			pw.println(String.format( "roi1_freq1= %.4f", gp.roi_1_f2_left));
			pw.println(String.format( "roi1_freq2= %.4f", gp.roi_1_f2_right));

			pw.println(String.format( "roi2_freq1= %.4f", gp.roi_2_f2_left));
			pw.println(String.format( "roi2_freq2= %.4f", gp.roi_2_f2_right));

			pw.println(String.format( "roi3_freq1= %.4f", gp.roi_3_f2_left));
			pw.println(String.format( "roi3_freq2= %.4f", gp.roi_3_f2_right));

			pw.println(String.format( "phase0= %.4f", q.PHC0));
			pw.println(String.format( "phase1= %.4f", q.PHC1));

			pw.println(String.format( "wdw= %d", q.WDW.ordinal()));
			pw.println(String.format( "lb= %.4f", q.LB));
			pw.println(String.format( "gf= %.4f", q.GB));
			pw.println(String.format( "ssb= %.4f", q.ssb));

			pw.println(String.format( "x_size= %.1f", gp.plot_xsize));
			pw.println(String.format( "y_size= %.1f", gp.plot_ysize));

			pw.println(String.format("plot_axis= %b", gp.plot_axis));
			pw.println(String.format("plot_frame= %b", gp.plot_frame));
			pw.println(String.format("plot_integrals= %b", gp.plot_integrals));
			pw.println(String.format("plot_parameters= %b", gp.plot_parameters));
			pw.println(String.format("plot_peak_labels= %b", gp.plot_peak_labels));
			pw.println(String.format("plot_simulated= %b", gp.plot_simulated));
			pw.println(String.format("plot_spectrum= %b", gp.plot_spectrum));
			pw.println(String.format("plot_grid= %b", gp.plot_grid));

			pw.println(String.format( "axis_font_height= %.1f",
					gp.axis_font_height));
			pw.println(String.format( "peak_font_height= %.1f",
					gp.peak_font_height));
			pw.println(String.format( "integ_font_height= %.1f",
					gp.integ_font_height));
			pw.println(String.format( "title_font_height= %.1f",
					gp.title_font_height));
			pw.println(String.format( "par_font_height= %.1f",
					gp.par_font_height));
			pw.println(String.format( "spect_pen_thickness= %.2f",
					gp.spect_pen_thickness));
			pw.println("double_data= true");       // for back compatability     
		}
		catch (Exception ex)
		{                    
			Util.show_and_log("save_proc_1D() raised the following exception: ", ex.getMessage());
			pw.close();
			return -3;
		}

		pw.close();		
		
//	now the real data
		FileOutputStream bw;
		try
		{
			bw = new FileOutputStream(file_name_real);
		}
		catch (Exception ex)
		{
			Util.show_and_log("save_proc_1D(): ", ex.getMessage());
			return -3;
		}

		byte[] buffer = new byte[8];
		
		if(gp.data_array != null)
			for (i = 0; i < q.SI; i++)
			{
				buffer = revDoubleToByteArray(gp.data_array[i]);
				try { bw.write(buffer); } 
				catch (IOException ex) 
				{
					Util.show_and_log("save_proc_1D(): Error writing real data", ex.getMessage());
					ex.printStackTrace();
					break;
				}
				bytes_written += 8;
			}

		try { bw.close(); } 
		catch (IOException e) { e.printStackTrace(); }

//	and the imaginary
		try
		{
	        bw = new FileOutputStream(file_name_imag);
		}
		catch (Exception ex)
		{
			Util.show_and_log("save_proc_1D(): ", ex.getMessage());
			return -2;
		}

		if (gp.i_data_array != null)
			for (i = 0; i < q.SI; i++)
			{
				buffer = revDoubleToByteArray(gp.i_data_array[i]);
				try { bw.write(buffer); } 
				catch (IOException ex) 
				{
					Util.show_and_log("save_proc_1D(): Error writing imag data", ex.getMessage());
					ex.printStackTrace();
					break;
				}
				bytes_written += 8;
			}

		try { bw.close(); } 
		catch (IOException e) { e.printStackTrace(); }

		if (gp.log_on)
			Util.log_error(String.format("save_proc_1D(): bytes written = %d", bytes_written));

		return bytes_written;
	}
	
//-------------------------------------------------------------------------------------------------	
/**
 * Read 1D processed data (SpinWorks format)
 * @param gp global parameter object
 * @param p set data parameters
 * @param q status data parameters
 * @return bytes read, -1 on error
 */
	public static int readProc1D(global_par gp, data_par p, data_par q)
	{				
		String instring;
		String delim = "[ =]+";
		String[] token;
		double scale_factor = 1.0;
		boolean double_data = false;       // true if data are 8 byte floating point

		int i, size = 0;
		int bytes_read = 0;
		double sf = 500.0;
		double sw = 0.0;

		String file_name_real;
		String file_name_imag;
		String file_name_par;
		
		if(q.data_type == SW_main.data_types.BRUKER 	// TODO allow at least JEOL??
				|| q.data_type == SW_main.data_types.VNMR)
		{
			file_name_real = q.data_folder + "1D_real";
			file_name_imag = q.data_folder + "1D_imag";
			file_name_par = q.data_folder + "1D_par";
		}
		else if(q.data_type == SW_main.data_types.JEOL || q.data_type == SW_main.data_types.JCAMP_DX)
		{
			file_name_real = q.data_path + "_SW_proc/1D_real";
			file_name_imag = q.data_path + "_SW_proc/1D_imag";
			file_name_par = q.data_path + "_SW_proc/1D_par";
		}
		else
			return -1;

		//	read the parameter file 
		BufferedReader br;
		try { br = new BufferedReader(new FileReader(file_name_par)); }
		catch (Exception ex)
		{
			Util.log_error("cannot open " + file_name_par, ex.getMessage());
			return -1;
		}

		scale_factor = 1.0;							// re-set global scale factor

		//	loop through all lines in parameter file looking for desired parameters
		for(;;)
		{
			try {instring = br.readLine();} 
			catch (IOException ex) 
			{
				ex.printStackTrace();
				Util.show_and_log("get_par(), read error: " + file_name_par, ex.getMessage());
				try { br.close(); } 
				catch (IOException e) { e.printStackTrace(); }
				return -1;
			}

			if(instring == null) break;

			if (instring.length() > 3)
			{
				token = instring.split(delim);

				if (token[0].equals("title"))				// title
				{
					gp.plot_title =  instring.substring(7);
					gp.plot_title = gp.plot_title.replace(";;", lineSep);
					gp.plot_title = gp.plot_title.replace(";", lineSep);
				}

				if (token[0].equals("size"))				// get the size
					size = Integer.parseInt(token[1]);
				try
				{
					if (token[0].equals("sf"))				// get SF (reference frequency) 
					{
						sf = Double.parseDouble(token[1]);
					}
					if (sf > 10000.0 || sf < 0.1)
						throw new Exception("sf out of range");
				}
				catch(Exception ex)
				{
					Util.show_and_log("error getting SF" + ex.getMessage());
					try {  br.close();  } 
					catch (IOException e) {  e.printStackTrace();  }
					return -3;
				}

				if (token[0].equals("sf_trans"))			   // get SFO1 (trans. frequency) 
				{
					q.SFO1 = Double.parseDouble(token[1]);
					p.SFO1 = q.SFO1;
				}

				try
				{
					if (token[0].equals("sw"))				   // get sw in F2 in Hz 
					{
						sw = Double.parseDouble(token[1]);
						if (sw > 1.0e8 || sf < 0.001)
							throw new Exception("sw out of range");
					}
				}
				catch(Exception ex)
				{
					Util.show_and_log("error getting SW" + ex.getMessage());
					try {  br.close();  } 
					catch (IOException e) {  e.printStackTrace();  }
					return -3;
				}

				if (token[0].equals("scale_factor"))		// get global scaling factor 
					scale_factor = Double.parseDouble(token[1]);

				if (token[0].equals("vscale"))				// get vertical scaling
					gp.expt_scale = Double.parseDouble(token[1]);

				if (token[0].equals("data_max"))			// get maximum of data 
					gp.data_max = Double.parseDouble(token[1]);

				if (token[0].equals("integ_scale"))			// get integral scaling
					gp.spect_scale = Double.parseDouble(token[1]);

				if (token[0].equals("integ_cal"))			// get integral calibration value
					gp.integ_cal = Double.parseDouble(token[1]);

				if (token[0].equals("freq1"))
					gp.freq1 = Double.parseDouble(token[1]);

				if (token[0].equals("freq2"))
					gp.freq2 = Double.parseDouble(token[1]);

				if (token[0].equals("roi1_freq1"))
					gp.roi_1_f2_left = Double.parseDouble(token[1]);

				if (token[0].equals("roi1_freq2"))
					gp.roi_1_f2_right = Double.parseDouble(token[1]);

				if (token[0].equals("roi2_freq1"))
					gp.roi_2_f2_left = Double.parseDouble(token[1]);				

				if (token[0].equals("roi2_freq2"))
					gp.roi_2_f2_right = Double.parseDouble(token[1]);			

				if (token[0].equals("roi3_freq1"))
					gp.roi_3_f2_left = Double.parseDouble(token[1]);				

				if (token[0].equals("roi3_freq2"))
					gp.roi_3_f2_right = Double.parseDouble(token[1]);

				if (token[0].equals("phase0"))
				{
					p.PHC0 = Double.parseDouble(token[1]);
					q.PHC0 = p.PHC0;					// since this is processed data
				}										// status reflects state of data

				if (token[0].equals("phase1"))
				{
					p.PHC1 = Double.parseDouble(token[1]);
					q.PHC1 = p.PHC1;					// since this is processed data
				}										// status reflects state of data

				if (token[0].equals("wdw"))				// get the window function used
				{
					q.WDW = SW_main.windows.values()[Integer.parseInt(token[1])];
					p.WDW = q.WDW;
				}

				if (token[0].equals("lb"))
					q.LB = Double.parseDouble(token[1]);

				if (token[0].equals("gf"))
					q.GB = Double.parseDouble(token[1]);				

				if (token[0].equals("ssb"))
					q.ssb = Double.parseDouble(token[1]);

				if (token[0].equals("x_size"))
					gp.plot_xsize = Double.parseDouble(token[1]);

				if (token[0].equals("y_size"))
					gp.plot_ysize = Double.parseDouble(token[1]);

				if (token[0].equals("plot_axis"))
					gp.plot_axis = Boolean.parseBoolean(token[1]);

				if (token[0].equals("plot_frame"))
					gp.plot_frame = Boolean.parseBoolean(token[1]);

				if (token[0].equals("plot_integrals"))
					gp.plot_integrals = Boolean.parseBoolean(token[1]);

				if (token[0].equals("plot_parameters"))
					gp.plot_parameters = Boolean.parseBoolean(token[1]);				

				if (token[0].equals("plot_peak_labels"))
					gp.plot_peak_labels = Boolean.parseBoolean(token[1]);

				if (token[0].equals("plot_simulated"))
					gp.plot_simulated = Boolean.parseBoolean(token[1]);

				if (token[0].equals("plot_spectrum"))
					gp.plot_spectrum= Boolean.parseBoolean(token[1]);

				if (token[0].equals("plot_title_flag"))
					gp.plot_title_flag = Boolean.parseBoolean(token[1]);

				if (token[0].equals("plot_grid"))
					gp.plot_grid = Boolean.parseBoolean(token[1]);

				if (token[0].equals("axis_font_height"))
					gp.axis_font_height = Float.parseFloat(token[1]);

				if (token[0].equals("peak_font_height"))
					gp.peak_font_height = Float.parseFloat(token[1]);

				if (token[0].equals("integ_font_height"))
					gp.integ_font_height = Float.parseFloat(token[1]);

				if (token[0].equals("title_font_height"))
					gp.title_font_height = Float.parseFloat(token[1]);				

				if (token[0].equals("par_font_height"))
					gp.par_font_height = Float.parseFloat(token[1]);

				if (token[0].equals("spect_pen_thickness"))
					gp.spect_pen_thickness = Float.parseFloat(token[1]);

				if (token[0].equals("double_data"))
					double_data = Boolean.parseBoolean(token[1]);
			}
		}
	
		try {  br.close();  } 
		catch (IOException e1) {  e1.printStackTrace();  }

		if(size != q.SI)
		{
			p.SI = size; q.SI = p.SI;
			q.hzperpt = q.SW * q.SFO1 / (double)(q.SI);
		}

		if(sf != q.SF)
		{
			q.SF = sf;	p.SF = q.SF;
			q.offset = (q.SFO1 - p.SF) * 1.0e06 + (q.SW*q.SFO1)/2.0;
		}

		if (sw != 0.0)
		{
			q.SW = sw / q.SFO1;
			q.hzperpt = sw / (double)(q.SI);
			q.offset = (q.SFO1 - p.SF) * 1.0e06 + (q.SW * q.SFO1) / 2.0;
			gp.freq1 = q.offset; gp.freq2 = q.offset - sw;
		}

		FileInputStream fs;

// first the reals
		File file = new File(file_name_real);

		try { fs = new FileInputStream(file); }
		catch (IOException ex)
		{
			Util.show_and_log("read_proc_1D cannot open: " + file_name_real, ex.getMessage());
			return -1;
		}

		Util.alloc_proc(gp, q.SI);
		
		byte[] b = new byte[(int)file.length()];
		
		try { fs.read(b); } 
		catch (IOException ex) 
		{
			ex.printStackTrace();
			try { fs.close(); } 
			catch (IOException e) { e.printStackTrace(); }
			return -3;
		}
		try { fs.close(); } 
		catch (IOException e) { e.printStackTrace(); }
		
		ByteBuffer bb = ByteBuffer.wrap(b);
		//SpinWorks produced processed data always LITTLE ENDIAN
		bb.order(ByteOrder.LITTLE_ENDIAN);	
				
		try
		{			
			for (i = 0; i < q.SI; i++)
			{
				if (double_data)
				{
					gp.data_array[i] = bb.getDouble();
					bytes_read += 8;
				}
				else
				{
					gp.data_array[i] = (double)bb.getInt() * scale_factor;
					bytes_read += 4;
				}
			}
		}
		catch (Exception ex)
		{
			Util.show_and_log("read_proc_1D(): ",  ex.getMessage());
			return -4;
		}			
		
//  now imaginaries
		file = new File(file_name_imag);

		try { fs = new FileInputStream(file); }
		catch (IOException ex)
		{
			Util.show_and_log("read_proc_1D cannot open: " + file_name_real, ex.getMessage());
			return -1;
		}
		
		b = new byte[(int)file.length()];

		try { fs.read(b); } 
		catch (IOException ex) 
		{
			ex.printStackTrace();
			try { fs.close(); } 
			catch (IOException e) { e.printStackTrace(); }
			return -3;
		}
		try { fs.close(); } 		
		catch (IOException e) { e.printStackTrace(); }
		
		bb = ByteBuffer.wrap(b);
		//SpinWorks produced processed data always LITTLE ENDIAN (Intel)
		bb.order(ByteOrder.LITTLE_ENDIAN);
		try
		{
			for (i = 0; i < q.SI; i++)
			{
				if (double_data)
				{
					gp.i_data_array[i] = bb.getDouble();
					bytes_read += 8;
				}
				else
				{
					gp.i_data_array[i] = bb.getInt() * scale_factor;
					bytes_read += 4;
				}
			}
		}
		catch (Exception ex)
		{
			Util.show_and_log("read_proc_1D() exception: ",  ex.getMessage());
			return -6;
		}			

		if (bytes_read > 0) gp.display_mode = SW_main.modes.MODE_1D;

		statusStripLabel2.setText("Processed 1D data saved");
		if (gp.log_on)
			Util.log_error(String.format("read_proc_1D(): bytes read = %d", bytes_read));
		
		return bytes_read;
	}

//-------------------------------------------------------------------------------------------------
/**
 * Save processed 2D data
 * @param par_only true to only update the parameters (no data written)
 * @param gp global parameter object
 * @param q status parameters, acquisition dimension
 * @param q_f1 status parameters, indirect dimension
 * @return bytes written, -1 if wrong data type, -2, -3 if data can't be written
 */
	public static int saveProc2D(boolean par_only, global_par gp, data_par q, data_par q_f1)
	{				
		int i, j;
		int bytes_written = 0;
		double scale_factor = 1.0;

		if (gp.data_max_2D > 2.0e9)
			scale_factor = gp.data_max_2D / 2.0e9;
	
		DateFormat df = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
    	Date dateobj = new Date();

    	
    	String fileNameRR;
		String fileNameRI;
		String fileNameIR;
		String fileNameII;
		String fileNamePar;
		
    	if(q.data_type == SW_main.data_types.BRUKER 	// TODO allow at least JEOL??
				|| q.data_type == SW_main.data_types.VNMR)
		{
    		fileNameRR = q.data_folder + "2D_rr";
    		fileNameRI= q.data_folder + "2D_ri";
    		fileNameIR = q.data_folder + "2D_ir";
    		fileNameII = q.data_folder + "2D_ii";
    		fileNamePar = q.data_folder + "2D_par";
		}
		else if(q.data_type == SW_main.data_types.JEOL)
		{
			File fi = new File(q.data_path + "_SW_proc");
			if(!fi.exists())
				fi.mkdir();
			
			fileNameRR = q.data_path + "_SW_proc/2D_rr";
    		fileNameRI= q.data_path + "_SW_proc/2D_ri";
    		fileNameIR = q.data_path + "_SW_proc/2D_ir";
    		fileNameII = q.data_path + "_SW_proc/2D_ii";
    		fileNamePar = q.data_path + "_SW_proc/2D_par";
		}
		else
			return -1;
    			
		//	write the parameter file
		//			write the parameter file
		PrintWriter pw;

		//	open file
		try { pw = new PrintWriter(fileNamePar); }
		catch (Exception ex)
		{			
			Util.show_and_log("save_prod_1D(): " + fileNamePar, ex.getMessage());
			return -4;
		}

		try
		{ 
			if(q.data_type == SW_main.data_types.JEOL)
				pw.println("## SpinWorksJ 0.0 2D parameters for: " + q.data_path);
			else
				pw.println("## SpinWorksJ 0.0 2D parameters for: " + q.data_folder);
			pw.println("## saved: " + df.format(dateobj) + lineSep );
			
			if (gp.plot_title == null)
				gp.plot_title = " ";

			/*String temp = gp.plot_title.replace('\n', ';'); // replace CR LF pair
			temp = temp.replace('\r', ';');
			pw.println("title= " + temp);*/

			gp.plot_title.replaceAll(lineSep, ";");
			pw.println("title= " + gp.plot_title.replaceAll(lineSep, ";"));
			
			pw.println(String.format("size= %d", q.SI));
			pw.println(String.format("size_f1= %d", q_f1.SI));

			pw.println(String.format("sf= %.8f", q.SF));
			pw.println(String.format("sf_f1= %.8f", q_f1.SF));
			pw.println(String.format("sf_trans= %.8f", q.SFO1));
			pw.println(String.format("sf_trans_f1= %.8f", q_f1.SFO1));
			pw.println(String.format("sw= %.3f", q.SFO1 * q.SW));
			pw.println(String.format("sw_f1= %.3f", q_f1.SFO1 * q_f1.SW));

			pw.println(String.format("scale_factor= %.8f", scale_factor));

			pw.println(String.format("roi1_freq1= %.4f", gp.roi_1_f2_left));
			pw.println(String.format("roi1_freq2= %.4f", gp.roi_1_f2_right));
			pw.println(String.format("roi1_freq1_indirect= %.4f", gp.roi_1_f1_left));
			pw.println(String.format("roi1_freq2_indirect= %.4f", gp.roi_1_f1_right));

			pw.println(String.format("roi2_freq1= %.4f", gp.roi_2_f2_left));
			pw.println(String.format("roi2_freq2= %.4f", gp.roi_2_f2_right));
			pw.println(String.format("roi2_freq1_indirect= %.4f", gp.roi_2_f1_left));
			pw.println(String.format("roi2_freq2_indirect= %.4f", gp.roi_2_f1_right));

			pw.println(String.format("roi3_freq1= %.4f", gp.roi_3_f2_left));
			pw.println(String.format("roi3_freq2= %.4f", gp.roi_3_f2_right));
			pw.println(String.format("roi3_freq1_indirect= %.4f", gp.roi_3_f1_left));
			pw.println(String.format("roi3_freq2_indirect= %.4f", gp.roi_3_f1_right));

			pw.println(String.format("phase0= %.4f", q.PHC0));
			pw.println(String.format("phase1= %.4f", q.PHC1));
			pw.println(String.format("phase0_f1= %.4f", q_f1.PHC0));
			pw.println(String.format("phase1_f1= %.4f", q_f1.PHC1));

			pw.println(String.format("lp_f1= %d", q_f1.lp.ordinal()));
			pw.println(String.format("lp_coef_f1= %d", q_f1.lp_coef));
			pw.println(String.format("lp_cutoff_f1= %.4f", q_f1.lp_cutoff));
			pw.println(String.format("lp_input_f1= %d", q_f1.lp_input));
			pw.println(String.format("lp_pred_f1= %d", q_f1.lp_pred));

			pw.println(String.format("wdw= %d", q.WDW.ordinal()));
			pw.println(String.format("wdw_f1= %d", q_f1.WDW.ordinal()));
			pw.println(String.format("lb= %.4f", q.LB));
			pw.println(String.format("lb_f1= %.4f", q_f1.LB));
			pw.println(String.format("gf= %.4f", q.GB));
			pw.println(String.format("gf_f1= %.4f", q_f1.GB));
			pw.println(String.format("ssb= %.4f", q.ssb));
			pw.println(String.format("ssb_f1= %.4f", q_f1.ssb));

			pw.println(String.format("f2_path= %s", gp.f2_trace_path));
			pw.println(String.format("f1_path= %s", gp.f1_trace_path));

			pw.println(String.format("x_size= %.1f", gp.plot_xsize));
			pw.println(String.format("y_size= %.1f", gp.plot_ysize));

			pw.println(String.format("plot_axis= %b", gp.plot_axis));
			pw.println(String.format("plot_frame= %b", gp.plot_frame));
			pw.println(String.format("plot_integrals= %b", gp.plot_integrals));
			pw.println(String.format("plot_parameters= %b", gp.plot_parameters));
			pw.println(String.format("plot_peak_labels= %b", gp.plot_peak_labels));
			pw.println(String.format("plot_simulated= %b", gp.plot_simulated));
			pw.println(String.format("plot_spectrum= %b", gp.plot_spectrum));
			pw.println(String.format("plot_grid= %b", gp.plot_grid));
			pw.println(String.format("parameters_right= %b", gp.parameters_right));

			pw.println(String.format("axis_font_height= %.1f", gp.axis_font_height));
			pw.println(String.format("peak_font_height= %.1f", gp.peak_font_height));
			pw.println(String.format("integ_font_height= %.1f", gp.integ_font_height));
			pw.println(String.format("title_font_height= %.1f", gp.title_font_height));
			pw.println(String.format("par_font_height= %.1f",  gp.par_font_height));
			pw.println(String.format("spect_pen_thickness= %.2f", gp.spect_pen_thickness));
			pw.println("float_data= true");
		}
		catch (Exception ex)
		{
			Util.show_and_log("save_proc_2D() raised the following exception: ", ex.getMessage());
			pw.close();
			return -3;
		}

		pw.close();

		if (par_only) return 1;

		//	now the rr data
		if (gp.rows_real != null)
		{
			FileOutputStream bw;
			try
			{
				bw = new FileOutputStream(fileNameRR);
			}
			catch (Exception ex)
			{
				Util.show_and_log("save_proc_2D() cannot open rr: ", ex.getMessage());
				return -3;
			}

			byte[] buffer = new byte[4 * q.SI * q_f1.SI];

			try
			{
				int index = 0;
				for (i = 0; i < q_f1.SI; i++)
					for (j = 0; j < q.SI; j++)
					{
						revFloatToByteArray(gp.rows_real[i][j], buffer, index);
						index += 4;
					}
				bytes_written += index;
				bw.write(buffer);
			}
			catch (Exception ex)
			{
				Util.show_and_log("save_proc_2D() writing rr failed: ", ex.getMessage());
				try { bw.close(); } 
				catch (IOException e) { e.printStackTrace(); }
				return -3;
			}

			try { bw.close(); } 
			catch (IOException e) { e.printStackTrace(); }
		}

		//	and the ri data
		if (gp.rows_imag != null)
		{
			FileOutputStream bw;
			try
			{
				bw = new FileOutputStream(fileNameRI);
			}
			catch (Exception ex)
			{
				Util.show_and_log("save_proc_2D() cannot open ri: ", ex.getMessage());
				return -3;
			}

			byte[] buffer = new byte[4 * q.SI * q_f1.SI];

			try
			{
				int index = 0;
				for (i = 0; i < q_f1.SI; i++)
					for (j = 0; j < q.SI; j++)
					{
						revFloatToByteArray(gp.rows_imag[i][j], buffer, index);																		
						index += 4;
					}
				bytes_written +=  index;
				bw.write(buffer);
			}
			catch (Exception ex)
			{
				Util.show_and_log("save_proc_2D(): ", ex.getMessage());
				try { bw.close(); } 
				catch (IOException e) { e.printStackTrace(); }
				return -3;
			}

			try { bw.close(); } 
			catch (IOException e) { e.printStackTrace(); }
		}

		//	and the ir data
		if (gp.irows_real != null)
		{
			FileOutputStream bw;
			try
			{
				bw = new FileOutputStream(fileNameIR);
			}
			catch (Exception ex)
			{
				Util.show_and_log("save_proc_2D(): cannot open ir", ex.getMessage());
				return -3;
			}

			byte[] buffer = new byte[4 * q.SI * q_f1.SI];

			try
			{
				int index = 0;
				for (i = 0; i < q_f1.SI; i++)
					for (j = 0; j < q.SI; j++)
					{
						revFloatToByteArray(gp.irows_real[i][j], buffer, index);												
						index += 4;
					}
				bytes_written += index;
				bw.write(buffer);
			}
			catch (Exception ex)
			{
				Util.show_and_log("save_proc_2D(): ", ex.getMessage());
				try { bw.close(); } 
				catch (IOException e) { e.printStackTrace(); }
				return -3;
			}

			try { bw.close(); } 
			catch (IOException e) { e.printStackTrace(); }
		}

		//	and the ii data
		if (gp.irows_imag!= null)
		{
			FileOutputStream bw;
			try
			{
				bw = new FileOutputStream(fileNameII);
			}
			catch (Exception ex)
			{
				Util.show_and_log("save_proc_2D(): cannot open ii", ex.getMessage());
				return -3;
			}

			byte[] buffer = new byte[4 * q.SI * q_f1.SI];

			try
			{
				int index = 0;
				for (i = 0; i < q_f1.SI; i++)
					for (j = 0; j < q.SI; j++)
					{
						revFloatToByteArray(gp.irows_imag[i][j], buffer, index);							
						index += 4;
					}
				bytes_written += index;
				bw.write(buffer);
			}
			catch (Exception ex)
			{
				Util.show_and_log("save_proc_2D(): ", ex.getMessage());
				try { bw.close(); } 
				catch (IOException e) { e.printStackTrace(); }
				return -3;
			}

			try { bw.close(); } 
			catch (IOException e) { e.printStackTrace(); }
		}

		statusStripLabel2.setText("Processed 2D data saved");
		if (gp.log_on)
			Util.log_error(String.format("save_proc_2D(): bytes written = {0}", bytes_written));

		return bytes_written;
	}
	
//-------------------------------------------------------------------------------------------------
/**
 * Read 2D processed data (SpinWorks format)
 * @param gp global parameter object
 * @param p	 set parameters, acquisition dimension
 * @param p_f1 set parameters, indirect dimension
 * @param q status parameters, acquisition dimension
 * @param q_f1 p_f1 status parameters, indirect dimension
 * @return bytes read, -1 on error
 */
	public static int readProc2D(global_par gp,
			data_par p, data_par p_f1, data_par q, data_par q_f1)
	{
		String instring;
		String delim = "[ =]+";
		String[] token;

		// In some countries a "," is used as the decimal separator.  Override this,
		// otherwise parameter sets, etc. with decimal numbers can't be read without the user having
		// to change their machine settings

		int i, j, size = 0, size_f1 = 0;
		int bytes_read = 0;
		double sf = 500.0,      sf_f1 = 500.0;
		double sw = 0.0,        sw_f1 = 0.0;

		boolean float_data = true;

		gp.f2_trace_path = null;        gp.f1_trace_path = null;

		String fileNameRR, fileNameRI, fileNameIR, fileNameII, fileNamePar;
		
		if(q.data_type == SW_main.data_types.BRUKER 	// TODO allow at least JEOL??
				|| q.data_type == SW_main.data_types.VNMR)
		{
    		fileNameRR = q.data_folder + "2D_rr";
    		fileNameRI = q.data_folder + "2D_ri";
    		fileNameIR = q.data_folder + "2D_ir";
    		fileNameII = q.data_folder + "2D_ii";
    		fileNamePar = q.data_folder + "2D_par";
		}
		else if(q.data_type == SW_main.data_types.JEOL)
		{
			File fi = new File(q.data_path + "_SW_proc");
			if(!fi.exists())
				fi.mkdir();
			
			fileNameRR = q.data_path + "_SW_proc/2D_rr";
    		fileNameRI = q.data_path + "_SW_proc/2D_ri";
    		fileNameIR = q.data_path + "_SW_proc/2D_ir";
    		fileNameII = q.data_path + "_SW_proc/2D_ii";
    		fileNamePar = q.data_path + "_SW_proc/2D_par";
		}
		else
			return -1;

		//	read the parameter file
		BufferedReader br;
		try { br = new BufferedReader(new FileReader(fileNamePar)); }
		catch (Exception ex)
		{
			if(!gp.auto_load)
				Util.show_and_log("read_proc_2D():" + ex.getMessage());
			return -4;
		}

		//	loop through all lines in parameter file looking for desired parameters

		try
		{
			for (;;)
			{
				try {instring = br.readLine();} 
				catch (IOException ex) 
				{
					ex.printStackTrace();
					Util.show_and_log("get_par(), read error: " + fileNamePar, ex.getMessage());
					try { br.close(); } 
					catch (IOException e) { e.printStackTrace(); }
					return -1;
				}

				if(instring == null) break;

				if (instring.length() > 3)
				{
					token = instring.split(delim);

					if (token[0].equals("title"))				// title
					{
						String temp =  instring.substring(7);
						gp.plot_title = temp.replace(";;", lineSep);
						gp.plot_title = temp.replace(";", lineSep);
					}

					if (token[0].equals("size"))				// get the size
						size = Integer.parseInt(token[1]);

					if (token[0].equals("size_f1"))				// get the size in F1
						size_f1 = Integer.parseInt(token[1]);

					if (token[0].equals("sf"))					// get SF (reference frequency) 
						sf = Double.parseDouble(token[1]);


					if (token[0].equals("sf_f1"))				// get SF (reference frequency) 
						sf_f1 = Double.parseDouble(token[1]);

					if (token[0].equals("sf_trans"))			// get SFO1 (trans. frequency) 
					{
						q.SFO1 = Double.parseDouble(token[1]);
						p.SFO1 = q.SFO1;
					}

					if (token[0].equals("sf_trans_f1"))			// get SFO1 (f1) (trans. frequency) 
					{
						q_f1.SFO1 = Double.parseDouble(token[1]);
						p_f1.SFO1 = q.SFO1;
					}

					if (token[0].equals("sw"))				    // get sw in F2 in Hz 
					{
						sw = Double.parseDouble(token[1]);
					}

					if (token[0].equals("sw_f1"))				// get sw in F2 in Hz 
					{
						sw_f1 = Double.parseDouble(token[1]);
					}

					if (token[0].equals("roi1_freq1"))
						gp.roi_1_f2_left = Double.parseDouble(token[1]);

					if (token[0].equals("roi1_freq2"))
						gp.roi_1_f2_right = Double.parseDouble(token[1]);

					if (token[0].equals("roi1_freq1_indirect"))
						gp.roi_1_f1_left = Double.parseDouble(token[1]);

					if (token[0].equals("roi1_freq2_indirect"))
						gp.roi_1_f1_right = Double.parseDouble(token[1]);

					if (token[0].equals("roi2_freq1"))
						gp.roi_2_f2_left = Double.parseDouble(token[1]);

					if (token[0].equals("roi2_freq2"))
						gp.roi_2_f2_right = Double.parseDouble(token[1]);

					if (token[0].equals("roi2_freq1_indirect"))
						gp.roi_2_f1_left = Double.parseDouble(token[1]);

					if (token[0].equals("roi2_freq2_indirect"))
						gp.roi_2_f1_right = Double.parseDouble(token[1]);

					if (token[0].equals("roi3_freq1"))
						gp.roi_3_f2_left = Double.parseDouble(token[1]);

					if (token[0].equals("roi3_freq2"))
						gp.roi_3_f2_right = Double.parseDouble(token[1]);

					if (token[0].equals("roi3_freq1_indirect"))
						gp.roi_3_f1_left = Double.parseDouble(token[1]);

					if (token[0].equals("roi3_freq2_indirect"))
						gp.roi_3_f1_right = Double.parseDouble(token[1]);

					if (token[0].equals("phase0"))
						p.PHC0 = Double.parseDouble(token[1]);

					if (token[0].equals("phase1"))
						p.PHC1 = Double.parseDouble(token[1]);

					if (token[0].equals("phase0_f1"))
						p_f1.PHC0 = Double.parseDouble(token[1]);

					if (token[0].equals("phase1_f1"))
						p_f1.PHC1 = Double.parseDouble(token[1]);

					if (token[0].equals("wdw"))				// get the window function used
						q.WDW = SW_main.windows.values()[Integer.parseInt(token[1])];

					if (token[0].equals("wdw_f1"))			// get the window function used
						q_f1.WDW = SW_main.windows.values()[Integer.parseInt(token[1])];

					if (token[0].equals("lb"))
						q.LB = Double.parseDouble(token[1]);

					if (token[0].equals("lb_f1"))
						q_f1.LB = Double.parseDouble(token[1]);

					if (token[0].equals("gf"))
						q.GB = Double.parseDouble(token[1]);

					if (token[0].equals("gf_f1"))
						q_f1.GB = Double.parseDouble(token[1]);

					if (token[0].equals("ssb"))
						q.ssb = Double.parseDouble(token[1]);

					if (token[0].equals("ssb_f1"))
						q_f1.ssb = Double.parseDouble(token[1]);

					if (token[0].equals("lp_f1"))
					{
						q_f1.lp = SW_main.LP_modes.values()[Integer.parseInt(token[1])];
						p_f1.lp = q_f1.lp;
					}

					if (token[0].equals("lp_coef_f1"))
					{
						q_f1.lp_coef = Integer.parseInt(token[1]);
						p_f1.lp_coef = q_f1.lp_coef;
					}

					if (token[0].equals("lp_cutoff_f1"))
					{
						q_f1.lp_cutoff = Double.parseDouble(token[1]);
						p_f1.lp_cutoff = q_f1.lp_cutoff;
					}

					if (token[0].equals("lp_input_f1"))
					{
						q_f1.lp_input = Integer.parseInt(token[1]);
						p_f1.lp_input = q_f1.lp_input;
					}

					if (token[0].equals("lp_pred_f1"))
					{
						q_f1.lp_pred = Integer.parseInt(token[1]);
						p_f1.lp_pred = q_f1.lp_pred;
					}

					if (token[0].equals("f2_path") && token.length > 1)   	 // F2 path
					{
						gp.f2_trace_path = token[1];
					}

					if (token[0].equals("f1_path") && token.length > 1)		// F1 path
					{
						gp.f1_trace_path = token[1];
					}

					if (token[0].equals("x_size"))
						gp.plot_xsize = Double.parseDouble(token[1]);

					if (token[0].equals("y_size"))
						gp.plot_ysize = Double.parseDouble(token[1]);

					if (token[0].equals("plot_axis"))
						gp.plot_axis = Boolean.parseBoolean(token[1]);

					if (token[0].equals("plot_frame"))
						gp.plot_frame = Boolean.parseBoolean(token[1]);

					if (token[0].equals("plot_integrals"))
						gp.plot_integrals = Boolean.parseBoolean(token[1]);

					if (token[0].equals("plot_parameters"))
						gp.plot_parameters = Boolean.parseBoolean(token[1]);

					if (token[0].equals("plot_peak_labels"))
						gp.plot_peak_labels = Boolean.parseBoolean(token[1]);

					if (token[0].equals("plot_simulated"))
						gp.plot_simulated = Boolean.parseBoolean(token[1]);

					if (token[0].equals("plot_spectrum"))
						gp.plot_spectrum = Boolean.parseBoolean(token[1]);

					if (token[0].equals("plot_title_flag"))
						gp.plot_title_flag = Boolean.parseBoolean(token[1]);

					if (token[0].equals("plot_grid"))
						gp.plot_grid = Boolean.parseBoolean(token[1]);

					if (token[0].equals("parameters_right"))
						gp.parameters_right = Boolean.parseBoolean(token[1]);

					if (token[0].equals("axis_font_height"))
						gp.axis_font_height = Float.parseFloat(token[1]);

					if (token[0].equals("peak_font_height"))
						gp.peak_font_height = Float.parseFloat(token[1]);

					if (token[0].equals("integ_font_height"))
						gp.integ_font_height = Float.parseFloat(token[1]);

					if (token[0].equals("title_font_height"))
						gp.title_font_height = Float.parseFloat(token[1]);

					if (token[0].equals("par_font_height"))
						gp.par_font_height = Float.parseFloat(token[1]);

					if (token[0].equals("spect_pen_thickness"))
						gp.spect_pen_thickness = Float.parseFloat(token[1]);

					if (token[0].equals("float_data"))
						float_data = Boolean.parseBoolean(token[1]);										
				}
			}
		}	// end parameter search
		
		catch(Exception ex)
		{
			try {  br.close(); } 
			catch (IOException e1) 
			{e1.printStackTrace();}
			return -3;
		}		
		
		try {  br.close(); } 
		catch (IOException e1) 
		{e1.printStackTrace();}

		if (size != q.SI)
		{
			p.SI = size; q.SI = p.SI;
			q.hzperpt = q.SW * q.SFO1 / (double)(q.SI);
		}

		if (sf != q.SF)
		{
			q.SF = sf; p.SF = q.SF;
			q.offset = (q.SFO1 - p.SF) * 1.0e06 + (q.SW * q.SFO1) / 2.0;
		}

		if (size_f1 != q_f1.SI)
		{
			p_f1.SI = size_f1; q_f1.SI = p_f1.SI;
			q_f1.hzperpt = q_f1.SW * q_f1.SFO1 / (double)(q_f1.SI);
		}

		if (sf_f1 != q_f1.SF)
		{
			q_f1.SF = sf_f1; p_f1.SF = q_f1.SF;
			q_f1.offset = (q_f1.SFO1 - p_f1.SF) * 1.0e06 + (q_f1.SW * q_f1.SFO1) / 2.0;
		}

		if (sw != 0.0)
		{
			q.SW = sw / q.SFO1;
			q.hzperpt = sw / (double)(q.SI);
			q.offset = (q.SFO1 - p.SF) * 1.0e06 + (q.SW * q.SFO1) / 2.0;
			gp.freq1 = q.offset;      gp.freq2 = q.offset - sw;
		}

		if (sw_f1 != 0.0)
		{
			q_f1.SW = sw_f1 / q_f1.SFO1;
			q_f1.hzperpt = sw_f1 / (double)(q_f1.SI);
			q_f1.offset = (q_f1.SFO1 - p_f1.SF) * 1.0e06 + (q_f1.SW * q_f1.SFO1) / 2.0;
			gp.freq1_indirect = q_f1.offset;      gp.freq2_indirect = q_f1.offset - sw_f1;
		}

		//	allocate space for all quadrants
		Util_2D.alloc_rows(q_f1.SI, q.SI, gp, q, q_f1);

		FileInputStream fs;
		File file = new File(fileNameRR);

		try { fs = new FileInputStream(file); }
		catch (IOException ex)
		{
			Util.show_and_log("no real-real data???: " + fileNameRR, ex.getMessage());
			return -1;
		}

		byte[] b = new byte[(int)file.length()];

		try { fs.read(b); } 
		catch (IOException ex) 
		{
			ex.printStackTrace();
			try { fs.close(); } 
			catch (IOException e) { e.printStackTrace(); }
			return -3;
		}

		ByteBuffer bb = ByteBuffer.wrap(b);

		//SpinWorks produced processed data always LITTLE ENDIAN
		bb.order(ByteOrder.LITTLE_ENDIAN);

		if (fs != null && file.length() > 0)
		{		
			try
			{

				for (i = 0; i < q_f1.SI; i++)
					for (j = 0; j < q.SI; j++)
					{
						if(float_data)
							gp.rows_real[i][j] = bb.getFloat();
						else
							gp.rows_real[i][j] = (float)bb.getInt();

						bytes_read = bytes_read + 4;
					}
			}
			catch (Exception ex)
			{
				Util.show_and_log("read_proc_2D():", ex.getMessage());
				try {  fs.close(); } 
				catch (IOException e1) 
				{e1.printStackTrace();}
				return -3;
			}

			try {  fs.close(); } 
			catch (IOException e1) 
			{e1.printStackTrace();}
		}		

		file = new File(fileNameRI);

		try { fs = new FileInputStream(file); }
		catch (IOException ex)
		{
			//util.show_and_log("read_proc_2D cannot open: " + fileNameRI, ex.getMessage());
			fs = null;
		}		

		if (fs != null && file.length() > 0)
		{
			b = new byte[(int)file.length()];
			try { fs.read(b); } 
			catch (IOException ex) 
			{
				ex.printStackTrace();
				try { fs.close(); } 
				catch (IOException e) { e.printStackTrace(); }
				return -3;
			}

			bb = ByteBuffer.wrap(b);

			//SpinWorks produced processed data always LITTLE ENDIAN
			bb.order(ByteOrder.LITTLE_ENDIAN);

			try
			{

				for (i = 0; i < q_f1.SI; i++)
					for (j = 0; j < q.SI; j++)
					{
						if(float_data)
							gp.rows_imag[i][j] = bb.getFloat();
						else
							gp.rows_imag[i][j] = (float)bb.getInt();

						bytes_read = bytes_read + 4;
					}
			}
			catch (Exception ex)
			{
				Util.show_and_log("read_proc_2D():", ex.getMessage());
				try {  fs.close(); } 
				catch (IOException e1) 
				{e1.printStackTrace();}
				return -3;
			}

			try {  fs.close(); } 
			catch (IOException e1) 
			{e1.printStackTrace();}
		}

		file = new File(fileNameIR);

		try { fs = new FileInputStream(file); }
		catch (IOException ex)
		{
			//util.show_and_log("read_proc_2D cannot open: " + fileNameIR, ex.getMessage());
			fs = null;
		}

		if (fs != null && file.length() > 0)
		{
			b = new byte[(int)file.length()];

			try { fs.read(b); } 
			catch (IOException ex) 
			{
				ex.printStackTrace();
				try { fs.close(); } 
				catch (IOException e) { e.printStackTrace(); }
				return -3;
			}

			bb = ByteBuffer.wrap(b);

			//SpinWorks produced processed data always LITTLE ENDIAN
			bb.order(ByteOrder.LITTLE_ENDIAN);

			try
			{

				for (i = 0; i < q_f1.SI; i++)
					for (j = 0; j < q.SI; j++)
					{
						if(float_data)
							gp.irows_real[i][j] = bb.getFloat();
						else
							gp.irows_real[i][j] = (float)bb.getInt();

						bytes_read = bytes_read + 4;
					}
			}
			catch (Exception ex)
			{
				Util.show_and_log("read_proc_2D():", ex.getMessage());
				try {  fs.close(); } 
				catch (IOException e1) 
				{e1.printStackTrace();}
				return -3;
			}

			try {  fs.close(); } 
			catch (IOException e1) 
			{e1.printStackTrace();}
		}

		file = new File(fileNameII);

		try { fs = new FileInputStream(file); }
		catch (IOException ex)
		{
			fs = null;
		}

		if (fs != null && file.length() > 0)
		{
			b = new byte[(int)file.length()];

			try { fs.read(b); } 
			catch (IOException ex) 
			{
				ex.printStackTrace();
				try { fs.close(); } 
				catch (IOException e) { e.printStackTrace(); }
				return -3;
			}

			bb = ByteBuffer.wrap(b);

			//SpinWorks produced processed data always LITTLE ENDIAN
			bb.order(ByteOrder.LITTLE_ENDIAN);

			try
			{

				for (i = 0; i < q_f1.SI; i++)
					for (j = 0; j < q.SI; j++)
					{
						if(float_data)
							gp.irows_imag[i][j] = bb.getFloat();
						else
							gp.irows_imag[i][j] = (float)bb.getInt();

						bytes_read = bytes_read + 4;
					}
			}
			catch (Exception ex)
			{
				Util.show_and_log("read_proc_2D():", ex.getMessage());
				try {  fs.close(); } 
				catch (IOException e1) 
				{e1.printStackTrace();}
				return -3;
			}

			try {  fs.close(); } 
			catch (IOException e1) 
			{e1.printStackTrace();}
		}

		// calculate scaling factors, etc.
		Util_2D.calc_scale_factors_2D(gp, q, q_f1);			
		gp.expt_scale = (double)gp.dysize * 0.80 / gp.data_max_2D;

		if (bytes_read > 0) gp.display_mode = SW_main.modes.MODE_2D;

		if (gp.log_on)
			Util.log_error(String.format("read_proc_2D(): bytes read = %d", bytes_read));

		return bytes_read;
	}
	
//-------------------------------------------------------------------------------------------------
/**
 * Writes current data as an ASCII text set of x, y pairs
 * @param fileName
 * @return number of points written
 */
	public static int exportXY(String fileName)
	{
		int i = 0;
		PrintWriter pw;

		//	open file
		try { pw = new PrintWriter(fileName); }
		catch (Exception ex)
		{
			Util.show_and_log("exportXY(): " + fileName, ex.getMessage());
			return -1;
		}

		pw.println("ti: " + gp.plot_title);

		if (gp.display_mode == modes.MODE_1D)
		{
			pw.println(String.format("##frequency %.7f", q.SFO1));
			Util.check_order(gp);

			int point1 = (int)((q.offset - gp.freq1) / q.hzperpt);
			int point2 = (int)((q.offset - gp.freq2) / q.hzperpt);
			if (point1 < 0) point1 = 0;
			if (point2 >= q.SI) point2 = q.SI - 1;

			for (i = point1; i <= point2; i++)
			{
				if (gp.axis_unit == SW_main.units.PPM)
					pw.println(String.format("%.5f  %.3f",
							(q.offset - (double)i * q.hzperpt) / q.SF,
							gp.data_array[i] / 1000.0));
				else
					pw.println(String.format("%.3f  %.3f",
							(q.offset - (double)i * q.hzperpt),
							gp.data_array[i] / 1000.0));
			}
		}

		else if (gp.display_mode == modes.MODE_FID)
		{
			double aqtime = 0.5 / (q.SW * q.SF) * q.TD;

			pw.println(String.format("acquisition time = %.6f", aqtime));

			for (i = 0; i <= q.TD - 1; i = i + 2)
			{
				pw.println(String.format("%.8f   %.3f   %.3f",
						((double)(i) / (double) q.TD) * aqtime, gp.fid[i], gp.fid[i + 1]));
			}
		}

		pw.close();   
		return i;
	}
//-------------------------------------------------------------------------------------------------
/**
 * converts a double into an array of 8 bytes
 * @param value  the double value
 * @return the byte array
 */
	public static byte[] toByteArray(double value) 
	{
	    byte[] bytes = new byte[8];
	    ByteBuffer.wrap(bytes).putDouble(value);
	    return bytes;
	}

//-------------------------------------------------------------------------------------------------
/**
 * converts a float into an array of 4 bytes
 * @param value the float value to convert
 * @return the byte array
 */
	public static byte[] toByteArray(float value) 
	{
		byte[] bytes = new byte[4];
		ByteBuffer.wrap(bytes).putFloat(value);
		return bytes;
	}

//-------------------------------------------------------------------------------------------------
/**
 * converts a double into an array of 8 bytes in reverse order,
 * necessary because the JVM has the opposite byte order to the Intel processors.
 * @param value the double to convert
 * @return the reversed byte array
 */
	public static byte[] revDoubleToByteArray(double value)
	{
		byte[] bytes = new byte[8];
		byte[] temp = new byte[8];
	    ByteBuffer.wrap(bytes).putDouble(value);
	    temp[0] = bytes[7];
	    temp[1] = bytes[6];
	    temp[2] = bytes[5];
	    temp[3] = bytes[4];
	    temp[4] = bytes[3];
	    temp[5] = bytes[2];
	    temp[6] = bytes[1];
	    temp[7] = bytes[0];
	    
	    return temp;
	}

//-------------------------------------------------------------------------------------------------
/**
 * converts a float into an array of 4 bytes in reverse order,
 * necessary because the JVM has the opposite byte order to the Intel processors.
 * @param value the float to convert
 * @return the reversed byte array
 */
	public static byte[] revFloatToByteArray(float value)
	{
		byte[] bytes = new byte[4];
		byte[] temp = new byte[4];
		ByteBuffer.wrap(bytes).putFloat(value);
		temp[0] = bytes[3];	
		temp[1] = bytes[2];
		temp[2] = bytes[1];
		temp[3] = bytes[0];

		return temp;
	}

//-------------------------------------------------------------------------------------------------
/**
 * inserts a float (4 bytes) into a byte array in reverse order, at the position given by index
 * necessary because the JVM has the opposite byte order to the Intel processors.
 * @param value the float to convert
 * @param buffer the byte array
 * @param index the location in the byte array
 */
	public static void revFloatToByteArray(float value, byte[] buffer, int index)
	{
		byte[] bytes = new byte[4];			
		ByteBuffer.wrap(bytes).putFloat(value);
		buffer[0 + index] = bytes[3];
		buffer[1 + index] = bytes[2];
		buffer[2 + index] = bytes[1];
		buffer[3 + index] = bytes[0];
	}
	
//-------------------------------------------------------------------------------------------------
/**
 * Get the experiment number from a Bruker path
 * @param path The fully qualified path to the data (fid file)
 * @return expt. number if successful, 0 if unable, -1 if an exception was thrown
 */
	public static int getExNo(String path)
	{
		int i = 1, j;
		int n = 0;

		try
		{               
			String temp = path.substring(0, path.length() - 1);

			for (j = temp.length() - 1; j > 0; j--)
			{
				if(temp.charAt(j) == '/' || temp.charAt(j) == '\\') break;
				n = n + Integer.parseInt(String.format("%c", temp.charAt(j))) * i;     
				i = i * 10;
			}
		}
		catch (Exception ex)
		{
			Util.show_and_log("get_ex_no():  " + path, ex.getMessage());
			return -1;
		}

		return n;
	}

//-------------------------------------------------------------------------------------------------
/**
 * Generates the correct paths for a new Bruker experiment number, 1D only.
 * Updates q.data_path and q.data_folder
 * @param exno the new experiment number
 * @param q status data parameters
 * @param gp SpinWorks "global parameter" object
 * @return true if valid data set found, false otherwise
 */
	public static boolean setExno(int exno, data_par q, global_par gp)
	{
		int j = 0, k = 0;

		String temp = q.data_folder.substring(0, q.data_folder.length() - 1);

		for (j = temp.length() - 1; j > 0; j--)
		{
			if(temp.charAt(j) == '/' || temp.charAt(j) == '\\') break;
		}
		k = temp.length() - j - 1;			     // length of expno #field

		temp = temp.substring(0, temp.length() - k);

		String new_folder = String.format("%s%d%c", temp, exno, '/');
		String new_path = new_folder + "fid";
		File fi = new File(new_path);           // check if fid file exists

		if (!fi.exists()) return false;         // nope!  so bail out

		q.data_path = new_path;			        // update data path
		q.data_folder = new_folder;		        // and data folder

		return true;
	}
	
//-------------------------------------------------------------------------------------------------
}
