
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.nio.ByteBuffer;

import spinworks.SW_main.byte_orders;
import spinworks.SW_main.det_modes;
import spinworks.SW_main.det_modes_IND;
import spinworks.SW_main.phasing;
import spinworks.SW_main.windows;
import spinworks.SW_main.filters;
import spinworks.SW_main.modes;

public class FileUtilVarian 
{
	public static boolean f2_as_2D_indirect = false;  // for using alternate axis for indirect in Varian

//-------------------------------------------------------------------------------------------------
//		Does a trial read of the Varian "procpar" file looking for a non zero value for SW1 and SW2.  
//		A non-zero value indicates that this is a 2D or 3D experiment and not just an "arrayed" 
//  	experiment.
//		This needs to be updated!! 

	public static int checkModeVarian(String fileName)
	{
		String instring;
		String delim = "[ =]+";
		String[] token;
		double sw1 = 0.0;       double sw2 = 0.0;
		int ni = 0;             int ni2 = 0;

		// try to open the parameter file 
        BufferedReader br;
        try { br = new BufferedReader(new FileReader(fileName)); }
        catch (Exception ex)
        {
            Util.show_and_log("checkModeVarian(), cannot open: " + fileName, ex.getMessage());
            return -1;
        }

		//		now scan each line looking for the parameters that we need

		for (;;)
		{
			 try {instring = br.readLine();} 
             catch (IOException ex) 
             {
     			ex.printStackTrace();
     			Util.show_and_log("checkModeVarian(), read error: " + fileName, ex.getMessage());
     			try { br.close(); } 
     			catch (IOException e) { e.printStackTrace(); }
     			return -1;
     		}
			 
			if(instring == null) break;
			
			token = instring.split(delim);

			try
			{
				if (token[0].equals("sw1"))						// a desired parameter
				{
					instring = br.readLine();                   // so read next line
					token = instring.split(delim);
					sw1 = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{   Util.show_and_log("checkModeVarian():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("sw2"))						// a desired parameter
				{
					instring = br.readLine();;                  // so read next line
					token = instring.split(delim);
					sw2 = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{   Util.show_and_log("checkModeVarian():  " + instring + ex.getMessage());    }

			try
			{
				if (token[0].equals("ni"))						// a desired parameter
				{
					instring = br.readLine();                   // so read next line
					token = instring.split(delim);
					ni = Integer.parseInt(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("checkModeVarian():  " + instring + ex.getMessage()); }

			try
			{
				if (token[0].equals("ni2"))						// a desired parameter
				{
					instring = br.readLine();                   // so read next line
					token = instring.split(delim);
					ni2 = Integer.parseInt(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("checkModeVarian():  " + instring + ex.getMessage()); }


		}
		
		try {	br.close();	} 
		catch (IOException e) {	e.printStackTrace(); }
		
		if (sw1 > 0.0 && sw2 > 0.0 && ni2 > 0)  return 3;                   // 3D
		if (sw1 > 0.0  && ni > 0 && ni2 == 0) return 2;		// 2D
		else return 1;								                        // 1D
	}
		
//-------------------------------------------------------------------------------------------------
//	Read the Varian "procpar" file and extract necessary parameters
//  updated for 3D

	public static boolean getVarianPars(String fileName, data_par p, data_par p_f1, data_par p_f2,
			data_par q, data_par q_f1, data_par q_f2)
	{

		String instring;
		String delim = "[ =]+";
		String[] token;
		String pulse_sequence = "undefined";		        // pulse sequence as string
		//int[] phase = new int[3];
		String f1coef = "";
		boolean f1_procmode_set = false;

		double rfl = 0.0, rfp = 0.0,                        // Varian style ref. parameters 
				sfrq = 500.0, /* reffrq = 500.0, tof = 0.0, */
				dfrq = 500.0, dfrq2 = 500.0;

		int ni = 0, ni2 = 0;
		double rfl1 = 0.0, rfp1 = 0.0,                      // 2D, 3D: F1
				/* reffrq1 = 0.0, */ swidth1 = 0.0;

		double rfl2 = 0.0, rfp2 = 0.0,                      // 3D: F2
				/* reffrq2 = 0.0, */ swidth2 = 0.0;

		//String refsource1 = "undefined";                  // reference source  ID
		//String refsource2 = "undefined";                  // reference source  ID
		String axis = "undefined";                          // axis labelling parameter

		double rp = 0.0, lp = 0.0;							// Varian style phase constants
		double swidth = 500.0;								// spectral width in Hz (internal)

		// try to open the parameter file 
		BufferedReader br;
		try { br = new BufferedReader(new FileReader(fileName)); }
		catch (Exception ex)
		{
			Util.show_and_log("getVarianPars(), cannot open: " + fileName, ex.getMessage());
			return false;
		}

		//	now scan each line looking for the parameters thast we need

		for(;;)	// infinite loop
		{
			 try {instring = br.readLine();} 
             catch (IOException ex) 
             {
     			ex.printStackTrace();
     			Util.show_and_log("getVarianPars(), read error: " + fileName, ex.getMessage());
     			try { br.close(); } 
     			catch (IOException e) { e.printStackTrace(); }
     			return false;
     		}
			 
			if(instring == null) break;
			
			token = instring.split(delim);

			try
			{
				if (token[0].equals("sfrq"))                               // spectrometer frequency
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					sfrq = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{   Util.show_and_log("getVarianPars():  " + instring, ex.getMessage());    }

			try
			{
				if (token[0].equals("dfrq"))                               // decoupler frequency
				{
					instring = br.readLine();
					token = instring.split(delim);
					dfrq = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("dfrq2"))                              // second dec. freq.
				{
					instring = br.readLine();
					token = instring.split(delim);
					//MessageBox.Show(String.Format("dfrq2 = {0}", token[1]));
					dfrq2 = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("tn"))									// nucleus
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("refsource1 = {0}", token[1]));
					q.NUC1 = token[1];
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			/*try
			{
				if (token[0].equals("reffrq"))                             // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("reffrq = {0}", token[1]));
					reffrq = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }*/

			/*try
			{
				if (token[0].equals("reffrq1"))                            // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("reffrq1 = {0}", token[1]));
					reffrq1 = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }*/

			/*try
			{
				if (token[0].equals("reffrq2"))                            // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("reffrq2 = {0}", token[1]));
					reffrq2 = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }*/

			/*try
			{
				if (token[0].equals("tof"))                                // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("tof = {0}", token[1]));
					tof = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }*/

			try
			{
				if (token[0].equals("np"))                                 // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("np = {0}", token[1]));
					q.TD = Integer.parseInt(token[1]); p.TD = q.TD;
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("ni"))                                 // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("ni = {0}", token[1]));
					ni = Integer.parseInt(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("ni2"))                                // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("ni2 = {0}", token[1]));
					ni2 = Integer.parseInt(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("ct"))                                 // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("ct = {0}", token[1]));
					q.NS = Integer.parseInt(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("fn"))                                 // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("fn = {0}", token[1]));
					p.SI = Integer.parseInt(token[1]); q.SI = p.SI;
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("fn1"))                                // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("fn1 = {0}", token[1]));
					q_f1.SI = Integer.parseInt(token[1]); p_f1.SI = q_f1.SI;
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("fn2"))                                // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("fn1 = {0}", token[1]));
					q_f2.SI = Integer.parseInt(token[1]); p_f2.SI = q_f2.SI;
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("sw"))                                 // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("sw = {0}", token[1]));
					swidth = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("sw1"))                                // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("sw1 = {0}", token[1]));
					swidth1 = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("sw2"))                                // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("sw1 = {0}", token[1]));
					swidth2 = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("rfp"))                                // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("rfp = {0}", token[1]));
					rfp = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("rfp1"))                               // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("rfp1 = {0}", token[1]));
					rfp1 = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("rfp2"))                               // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("rfp1 = {0}", token[1]));
					rfp2 = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("rfl"))                                // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("rfl = {0}", token[1]));
					rfl = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }


			try
			{
				if (token[0].equals("rfl1"))                               // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("rfl1 = {0}", token[1]));
					rfl1 = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("rfl2"))                               // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("rfl = {0}", token[1]));
					rfl2 = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			/*try
			{
				if (token[0].equals("refsource1"))                         // F1 or "Z"
				{
					instring = br.readLine();// read line 2
					token = instring.split(delim);
					//MessageBox.Show(String.Format("refsource1 = {0}", token[1]));
					refsource1 = token[1];
				}
			}
			catch (Exception ex)
			{ util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }*/

			/*try
			{
				if (token[0].equals("refsource2"))                         // a desired parameter
				{
					instring = br.readLine();                               // F2 or "y"
					token = instring.split(delim);
					//MessageBox.Show(String.Format("refsource1 = {0}", token[1]));
					refsource2 = token[1];
				}
			}
			catch (Exception ex)
			{ util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }*/


			try
			{
				if (token[0].equals("axis"))                         // axis parameter
				{
					instring = br.readLine();// read line 2
					token = instring.split(delim);
					axis = token[1];
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("seqfil"))                         // a desired parameter
				{
					instring = br.readLine();                           // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("seqfil = {0}", token[1]));
					pulse_sequence = token[1];
					q.pulse_sequence = pulse_sequence;
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("rp"))                                 // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("rp = {0}", token[1]));
					rp = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("lp"))                                 // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("lp = {0}", token[1]));
					lp = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("lb"))                                 // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
					//MessageBox.Show(String.Format("lb = {0}", token[1]));
					p.LB = Double.parseDouble(token[1]);
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("phase"))                              // a desired parameter
				{
					instring = br.readLine();                               // so read next line
					token = instring.split(delim);
				}
			}
			catch (Exception ex) // currently null catch block
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("f1coef"))                              // a desired parameter
				{
					instring = br.readLine();                                // so read next line
					token = instring.split(delim);
					for(int i = 1; i < token.length; i++)
						f1coef = f1coef + token[i];
					f1coef = f1coef.replace('"', ' ');
					f1coef = f1coef.trim();
										
					if(SW_main.gp.log_on)
						Util.log_error("Varian/Agilent f1coef = " + f1coef);
				}
			}
			catch (Exception ex) // currently null catch block
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }

			try
			{
				if (token[0].equals("array"))                         // a desired parameter
				{
					instring = br.readLine();                         // so read next line
					token = instring.split(delim);    
					if(token[1].contains("phase,phase2")) q.varian_array = 1;
					else if(token[1].contains("phase2,phase")) q.varian_array = 2;
				}
			}
			catch (Exception ex)
			{ Util.show_and_log("getVarianPars():  " + instring, ex.getMessage()); }
		}       // end while

		//	First try to guess at some processing parameters based on pulse sequence 

		//  name for common pulse sequences
		//if (pulse_sequence.contains("COSY")           // homonuclear magnitude
		//|| pulse_sequence.contains("cosy"))
		//{
		//    // F1                                           // F2
		//    p_f1.WDW =  windows.SINE; p.WDW =  windows.SINE;
		//    p_f1.ssb = 0.0; p.ssb = 0.0;
		//    p_f1.PH_mod =  phasing.MAGNITUDE; p.PH_mod =  phasing.NONE;
		//    p_f1.MC2 =  det_modes_IND.QF;
		//    p_f1.REVERSE = false;
		//    f1_procmode_set = true;
		//}

		//else if (pulse_sequence.contains("NOESY")          // homonuclear phase sensitive
		//    || pulse_sequence.contains("noesy")
		//    || pulse_sequence.contains("ROESY")
		//    || pulse_sequence.contains("roesy"))
		//{
		//    // F1                                      // F2
		//    p_f1.WDW =  windows.QSINE; p.WDW =  windows.QSINE;
		//    p_f1.ssb = 90.0; p.ssb = 90.0;
		//    p_f1.PHC0 = 0.0; p_f1.PHC1 = 0.0;
		//    p_f1.PH_mod =  phasing.CONSTANTS; p.PH_mod =  phasing.CONSTANTS;
		//    p_f1.MC2 =  det_modes_IND.STATES;
		//    p_f1.REVERSE = true;
		//    f1_procmode_set = true;
		//}

		//else if (pulse_sequence.contains("HSQC")       // typical phase sensitive 2D
		//    || pulse_sequence.contains("hsqc")
		//    || pulse_sequence.contains("TOCSY")
		//    || pulse_sequence.contains("tocsy")
		//    || pulse_sequence.contains("gDQCOSY")      // overvides simple "COSY"
		//    || pulse_sequence.contains("HMQC")
		//    || pulse_sequence.contains("hmqc"))
		//{
		//    // F1                                       // F2
		//    p_f1.WDW =  windows.QSINE; p.WDW =  windows.QSINE;
		//    p_f1.ssb = 90.0; p.ssb = 90.0;
		//    p_f1.PH_mod =  phasing.NONE; p.PH_mod =  phasing.NONE;
		//    p_f1.REVERSE = true;

		//    if (Equals(pulse_sequence, "\"gHSQC\"")      // gradient enhanced
		//        || Equals(pulse_sequence, "\"gNhsqc\"")
		//        || Equals(pulse_sequence, "\"gHMQC\"")
		//        || Equals(pulse_sequence, "\"gChsqc\""))
		//        p_f1.MC2 =  det_modes_IND.ECHO_ANTIECHO;
		//    else                                         // ordinary States
		//        p_f1.MC2 =  det_modes_IND.STATES;

		//    if (Equals(pulse_sequence, "\"gNhsqc\"")  // from "BioPack"
		//        || Equals(pulse_sequence, "\"gChsqc\""))
		//    {
		//        p_f1.REVERSE = false;
		//        p_f1.PHC0 = 90.0;
		//        p_f1.PH_mod =  phasing.CONSTANTS;
		//    }
		//    f1_procmode_set = true;
		//}

		//else if (pulse_sequence.contains("HMBC")              // HMBC experiments (add cigar)
		//    || pulse_sequence.contains("hmbc"))
		//{
		//    // F1                                           // F2
		//    p_f1.WDW =  windows.SINE; p.WDW =  windows.SINE;
		//    p_f1.ssb = 45.0; p.ssb = 45.0;
		//    p_f1.PH_mod =  phasing.MAGNITUDE; p.PH_mod =  phasing.NONE;
		//    p_f1.MC2 =  det_modes_IND.QF;
		//    p_f1.REVERSE = false;
		//    f1_procmode_set = true;
		//}

		// for 3D for now, base detection mode on pulse sequence name
		/*else*/ if (pulse_sequence.contains("ghn_co") && q.data_mode == SW_main.modes.MODE_3D)
		{
			p.WDW = p_f1.WDW = p_f2.WDW =
					SW_main.windows.QSINE; p.WDW = SW_main.windows.QSINE;
					p.PH_mod = SW_main.phasing.CONSTANTS;   //p.PHC0 = p.PHC1 = 0.0;
					p.ssb = p_f1.ssb = p_f2.ssb = 90.0;
					p_f2.PHC0 = -90.0; p_f2.PH_mod = SW_main.phasing.CONSTANTS;
					p_f2.MC2 = SW_main.det_modes_IND.ECHO_ANTIECHO;
					p_f1.MC2 = SW_main.det_modes_IND.STATES;
					p.filter_type = filters.FILT_GAUSS;
					p_f1.REVERSE = true;
					f1_procmode_set = true;
		}

		else if (pulse_sequence.contains("gnoesyChsqc") && q.data_mode ==  modes.MODE_3D)
		{
			p.WDW = p_f1.WDW = p_f2.WDW =
					SW_main.windows.QSINE; p.WDW = SW_main.windows.QSINE;
					p.PH_mod = SW_main.phasing.CONSTANTS;   //p.PHC0 = p.PHC1 = 0.0;
					p.ssb = p_f1.ssb = p_f2.ssb = 90.0;
					p_f2.MC2 = SW_main.det_modes_IND.STATES;
					p_f1.MC2 = SW_main.det_modes_IND.STATES;
					p_f2.REVERSE = true; p_f1.REVERSE = true;
					p.filter_type = filters.FILT_GAUSS;
					f1_procmode_set = true;
		}

		else if (pulse_sequence.contains("gnoesyNhsqc") && q.data_mode ==  modes.MODE_3D)
		{
			p.WDW = p_f1.WDW = p_f2.WDW =
					SW_main.windows.QSINE; p.WDW = SW_main.windows.QSINE;
					p.PH_mod = SW_main.phasing.CONSTANTS; //p.PHC0 = p.PHC1 = 0.0;
					p_f2.PHC0 = -90.0; p_f2.PH_mod = SW_main.phasing.CONSTANTS;
					p.ssb = p_f1.ssb = p_f2.ssb = 90.0;
					p_f2.MC2 = SW_main.det_modes_IND.STATES;
					p_f1.MC2 = SW_main.det_modes_IND.STATES;
					p_f1.REVERSE = true;
					p.filter_type = filters.FILT_GAUSS;
					f1_procmode_set = true;
		}

		// Not common sequence, check f1coef to determine f1 detection mode
		if (f1_procmode_set == false)
		{
			if (f1coef.equals("100000-10") && 	
					q.data_mode ==  modes.MODE_2D)  // typical States
			{
				p_f1.MC2 = SW_main.det_modes_IND.STATES;
				p_f1.REVERSE = true;
				p_f1.WDW = SW_main.windows.QSINE; p.WDW = SW_main.windows.QSINE;
				p_f1.ssb = 90.0; p.ssb = 90.0;
				p_f1.PHC0 = 0.0; p_f1.PHC1 = 0.0;
				p_f1.PH_mod = SW_main.phasing.CONSTANTS; p.PH_mod = SW_main.phasing.CONSTANTS;
				f1_procmode_set = true;
			}
			else if (f1coef.equals("1001") || f1coef.length() < 4
					&& q.data_mode == modes.MODE_2D)
				// typical COSY/HMBC type
			{
				p_f1.MC2 =  det_modes_IND.QF;
				p_f1.WDW =  windows.SINE; p.WDW =  windows.SINE;
				if (pulse_sequence.contains("HMBC")
						|| pulse_sequence.contains("hmbc"))
				{
					p_f1.ssb = 45.0; p.ssb = 45.0;  // better S:N for HMBC
				}
				else
				{
					p_f1.ssb = 0.0; p.ssb = 0.0;    // probably COSY like
				}
				p_f1.PH_mod =  phasing.MAGNITUDE; p.PH_mod =  phasing.NONE;
				p_f1.REVERSE = false;
				f1_procmode_set = true;
			}
			else if (f1coef.equals("1010010-1")
					&& q.data_mode ==  modes.MODE_2D)  // typical echo-antiecho
			{
				p_f1.WDW =  windows.QSINE; p.WDW =  windows.QSINE;
				p_f1.ssb = 90.0; p.ssb = 90.0;
				p_f1.REVERSE = true;
				p_f1.PHC0 = 0.0; p_f1.PHC1 = 0.0;
				p_f1.PH_mod =  phasing.CONSTANTS;
				p.PH_mod =  phasing.CONSTANTS;
				p_f1.MC2 =  det_modes_IND.ECHO_ANTIECHO;
				f1_procmode_set = true;
			}
			else if (f1coef.equals("10-100101")    // alternate EA (relax disp)
					&& q.data_mode ==  modes.MODE_2D)  
			{
				p_f1.WDW =  windows.QSINE; p.WDW =  windows.QSINE;
				p_f1.ssb = 90.0; p.ssb = 90.0;
				p_f1.REVERSE = true;
				p_f1.PHC0 = 90; p_f1.PHC1 = 0.0;
				p_f1.MC2 =  det_modes_IND.ECHO_ANTIECHO;
				p_f1.PH_mod =  phasing.CONSTANTS; 
				p.PH_mod =  phasing.CONSTANTS;
				f1_procmode_set = true;
			}
			else if (f1coef.equals("10-100-10-1")    // alternate EA 
					&& q.data_mode ==  modes.MODE_2D)
			{
				p_f1.WDW =  windows.QSINE; p.WDW =  windows.QSINE;
				p_f1.ssb = 90.0; p.ssb = 90.0;
				p_f1.PH_mod =  phasing.NONE; p.PH_mod =  phasing.NONE;
				//p_f1.REVERSE = true;
				p_f1.PHC0 = -90; p_f1.PHC1 = 0.0;
				p_f1.MC2 =  det_modes_IND.ECHO_ANTIECHO;
				p_f1.PH_mod =  phasing.CONSTANTS;
				p.PH_mod =  phasing.CONSTANTS;
				f1_procmode_set = true;
			}
		}

		if (q.data_mode ==  modes.MODE_2D
				&& f1_procmode_set == false)
		{
			Util.show_and_log("Cannot determine F1 detection mode, must be set manually");
		}

		//TO DO:  need to add checking of phase parameter and processing coefficients?

		q.SFO1 = sfrq;										// sfrq is the actual freq.
		q.SFO2 = dfrq;                                      // other channels...
		q.SFO3 = dfrq2;

		// lets try using "axis" parameter instead

		axis = axis.replace('"', ' ');   	// remove " char at beginning and end of string
		axis = axis.trim();
		if(axis.length() == 1)        	// pad out to 3 char for 1D and 2D
			axis = axis + "  ";
		else if (axis.length() == 2)
			axis = axis + " ";
		
		//System.out.println("axis = " + axis);

		if (axis.charAt(1) == 'p') 
			q_f1.SFO1 = sfrq;
		else if (axis.charAt(1) == 'd' || axis.charAt(1) == '1')
			q_f1.SFO1 = dfrq;
		else if (axis.charAt(1) == '2')
			q_f1.SFO1 = dfrq2;

		if (axis.charAt(2) == 'p')
			q_f2.SFO1 = sfrq;
		else if (axis.charAt(2) == 'd' || axis.charAt(1) == '1')
			q_f2.SFO1 = dfrq;
		else if (axis.charAt(2) == '2')
			q_f2.SFO1 = dfrq2;

		q.hzperpt = swidth / (q.SI);                // direct or "X"

		if (q_f1.SI != 0)
			q_f1.hzperpt = swidth1 / (q_f1.SI);     // F1 or "Z" in 3D

		if (q_f2.SI != 0)
			q_f2.hzperpt = swidth2 / (q_f2.SI);     // F2 or "Y" on 3D

		q.offset = -rfl + rfp + swidth;
		q_f1.offset = -rfl1 + rfp1 + swidth1;
		q_f2.offset = -rfl2 + rfp2 + swidth2;

		//	apply referencing in direct dimension
		q.SF = sfrq - (swidth / 2.0 - rfl + rfp) / 1.0e6;
		p.SF = q.SF;

		//	apply referencing in F1, depending on "refsource" parameter
		//  turn stupid Varian type referencing to something more reasonable
		//  lets try using "axis" parameter instead

		if (axis.charAt(1) == 'p')
			q_f1.SF = sfrq - (swidth1 / 2.0 - rfl1 + rfp1) / 1.0e6;
		else if (axis.charAt(1) == 'd' || axis.charAt(1) == '1')
			q_f1.SF = dfrq - (swidth1 / 2.0 - rfl1 + rfp1) / 1.0e6;
		else if (axis.charAt(1) == '2')
			q_f1.SF = dfrq2 - (swidth1 / 2.0 - rfl1 + rfp1) / 1.0e6;
		else
			q_f2.SF = q.SF;				// if nothing else, assume homonuclear correlation

		p_f1.SF = q_f1.SF;				// necessary ????

		//	apply referencing in F2, depending on "refsource" parameter
		//  turn stupid Varian type referencing to something more reasonable
		//  lets try using "axis" parameter instead

		if (axis.charAt(2) == 'p')
			q_f2.SF = sfrq - (swidth2 / 2.0 - rfl2 + rfp2) / 1.0e6;
		else if (axis.charAt(2) == 'd' || axis.charAt(2) == '1')
			q_f2.SF = dfrq - (swidth2 / 2.0 - rfl2 + rfp2) / 1.0e6;
		else if (axis.charAt(2) == '2')
			q_f2.SF = dfrq2 - (swidth2 / 2.0 - rfl2 + rfp2) / 1.0e6;
		else
			q_f2.SF = q.SF;				// if nothing else, assume homonuclear correlation

		p_f2.SF = q_f2.SF;				// necessary ????


		q.OFFSET = (float)(q.offset / p.SFO1); p.OFFSET = q.OFFSET;

		if (q_f1.SFO1 > 0.1)
		{
			q_f1.OFFSET = (float)(q_f1.offset / q_f1.SFO1);
			p_f1.OFFSET = q_f1.OFFSET;
		}

		if (q_f2.SFO1 > 0.1)
		{
			q_f2.OFFSET = (float)(q_f2.offset / q_f2.SFO1);
			p_f2.OFFSET = q_f2.OFFSET;
		}

		p.SW = swidth / p.SFO1;                 // direct
		q.SW = swidth / q.SFO1;

		if (q_f1.SFO1 > 0.1)                    // first indirect, F1 or "Z"
			q_f1.SW = swidth1 / q_f1.SFO1;

		if (q_f2.SFO1 > 0.1)                    // second indirect, F2 or "Y"
			q_f2.SW = swidth2 / q_f2.SFO1;

		p.PHC0 = -rp;                           // TO DO update phasing for 
		p.PHC1 = -lp;                           // indirect dimensions


		if (q.data_mode ==  modes.MODE_2D)
		{
			if (q_f1.MC2 ==  det_modes_IND.QF)
				q_f1.TD = ni;
			else
				q_f1.TD = ni * 2;
		}
		else if (q.data_mode ==  modes.MODE_3D)
		{
			q_f1.TD = ni * 2;       // F1 or "Z"
			q_f2.TD = ni2 * 2;      // F2 or "Y"
		}

		if(f2_as_2D_indirect)       // clean up!!
		{
			q_f1.TD = ni2 * 2;
			q_f2.SW = swidth2 / q_f1.SFO1;
		}

		q.aq_mode =  det_modes.SIM;			// Varian has simultaneous sampling only 
		q.BYTORDP =  byte_orders.BIG;		// Varian uses Power PC in Acq proc.

		if (SW_main.gp.log_on)                          // log some info if requested
		{
			Util.log_error(String.format(
					"get_varian_pars(%s): SFO1 = %.7f", fileName, q.SFO1));

			Util.log_error(String.format(
					"get_varian_pars({%s}): SFO2 = %.7f", fileName, q.SFO2));

			Util.log_error(String.format(
					"get_varian_pars(%s}): SFO3 = %.7f", fileName, q.SFO3));

			Util.log_error(String.format(
					"get_varian_pars({%s}): SF in direct, F1,   F2 = %.7f  %.7f  %.7f", 
					fileName, q.SF, q_f1.SF, q_f2.SF));

			Util.log_error(String.format(
					"get_varian_pars({%s}): SW in direct, F1,   F2 = %.7f  %.7f  %.7f",
					fileName, q.SW, q_f1.SW, q_f2.SW));

			Util.log_error("getVarianPars(): " + pulse_sequence);
		}

		q.dwell = 1.0 / (q.SW * q.SFO1);
		
		try {	br.close();	} 
		catch (IOException e) {	e.printStackTrace(); }

		return (true);
	}
	
//-------------------------------------------------------------------------------------------------
//	read Varian fid file
	public static int getVarianFid(String fileName, global_par gp, data_par q)
	{
		
		DataFileHeader fh = new DataFileHeader();
		DataBlockHeader bh = new DataBlockHeader();
		
		FileInputStream fs;
		File file = new File(fileName);

		// try to open the data file, bail out if fails
		try { fs = new FileInputStream(file); }
		catch (IOException ex)
		{
			Util.show_and_log("getVarianFid(): cannot open: " + fileName, ex.getMessage());
			return -1;
		}

		byte[] fh_byte = new byte[32];
		byte[] bh_byte = new byte[28];
		int bytes_read = 0;
		
		// read entire file into byte array
		byte[] b = new byte[(int)file.length()];

		try { fs.read(b); } 
		catch (IOException ex) 
		{
			ex.printStackTrace();
			try { fs.close(); } 
			catch (IOException e) { e.printStackTrace(); }
			return -3;
		}

		// get file header portion		
		 for (int i = 0; i < 32; i++)
				fh_byte[i] = b[i];
			
		 ByteBuffer bb_fh = ByteBuffer.wrap(fh_byte);
		
		fh.nblocks = bb_fh.getInt(0);
		fh.np = bb_fh.getInt(8);
		fh.status = bb_fh.getShort(26);
		fh.nbheaders = bb_fh.getInt(28);
		q.nblocks = fh.nblocks;
		
		//System.out.println(String.format("file header status: %d", fh.status));

		//  don't go past last block!
		if (gp.vblock > fh.nblocks)
		{
			Util.show_and_log("requested block number greater than \n blocks acquired \n reset to 1");
			gp.vblock = 1;
		}

		//  loop through to get desired block
		for (int j = 1; (j <= gp.vblock) && (j <= fh.nblocks); j++)
		{
			//  read block header, and get status field
			for (int i = 0; i < 28; i++)
				bh_byte[i] = b[i + 32 + (j-1) * (4 * fh.np + 28)];
			
			ByteBuffer bb_bh = ByteBuffer.wrap(bh_byte);
			bh.status = bb_bh.getShort(2);			

			//  now read the data
			if ((bh.status & 0x8) != 0)				// 32 bit float	
			{
				if (gp.log_on) Util.log_error("getVarianFid(): 32 bit float");
				try { gp.fid = new double[q.TD]; }
				catch (Exception ex)
				{
					Util.show_and_log("getVarianFid():  " + fileName + ex.getMessage());
					try { fs.close(); } 
				    catch (IOException ex1) { ex1.printStackTrace(); }
					return -1;
				}

				byte[] fid_buffer = new byte[fh.np * 4];
				for(int i = 0; i < fh.np * 4; i++)
					fid_buffer[i] = b[i + 60 + (j-1) * (28 + 4 * fh.np)];
				
				ByteBuffer bb_fid = ByteBuffer.wrap(fid_buffer);

				for (int i = 0; i < q.TD; i++)
				{
					try
					{
						bytes_read = bytes_read + 4;
						gp.fid[i] = (double)bb_fid.getFloat(i * 4);
					}
					catch (Exception ex)
					{
						Util.show_and_log("getVarianFid():  " + fileName + ex.getMessage());
						break;
					}
				}
			}

			if ((bh.status & 0x4) != 0)			//most common, 32 bit ints
			{
				if (gp.log_on) Util.log_error("getVarianFid(): 32 bit integer");
				try { gp.fid = new double[q.TD]; }
				catch (Exception ex)
				{
					Util.show_and_log("getVarianFid():  " + fileName + ex.getMessage());
					try { fs.close(); } 
				    catch (IOException e) { e.printStackTrace(); }
					return -1;
				}

				byte[] fid_buffer = new byte[fh.np * 4];
				for(int i = 0; i < fh.np * 4; i++)
					fid_buffer[i] = b[i + 60 + (j-1) * (28 + 4 * fh.np)];
				
				ByteBuffer bb_fid = ByteBuffer.wrap(fid_buffer);

				for (int i = 0; i < q.TD; i++)
				{
					try
					{
						bytes_read = bytes_read + 4;
						gp.fid[i] = (double)bb_fid.getInt(i * 4);
					}
					catch (Exception ex)
					{
						Util.show_and_log("getVarianFid():  " + fileName + ex.getMessage());
						break;
					}
				}
			}               

			else if ((bh.status & 0x4) == 0)      // 16 bit to save space 
			{
				if (gp.log_on) Util.log_error("getVarianFid(): 16 bit integer");
				try { gp.fid = new double[q.TD]; }
				catch (Exception ex)
				{
					Util.show_and_log("getVarianFid():  " + fileName + ex.getMessage());
					try { fs.close(); } 
				    catch (IOException e1) { e1.printStackTrace(); }
					return -1;
				}

				byte[] fid_buffer = new byte[fh.np * 2];
				for(int i = 0; i < fh.np * 2; i++)
					fid_buffer[i] = b[i + 60 + (j-1) * (28 + 2 * fh.np)];
				
				ByteBuffer bb_fid = ByteBuffer.wrap(fid_buffer);

				for (int i = 0; i < q.TD; i++)
				{
					try
					{
						bytes_read = bytes_read + 2;
						gp.fid[i] = (double)bb_fid.getShort(i * 2);
					}
					catch (Exception ex)
					{
						Util.show_and_log("getVarianFid():  " + fileName + ex.getMessage());
						break;
					}
				}
			}
		}

		//  clean-up    
		
		try { fs.close(); } 
	    catch (IOException ex) { ex.printStackTrace(); }

		if (gp.log_on)
			Util.log_error(String.format("getVarianFid(): bytes read = %d", bytes_read));

		return (bytes_read);
	}
	
//-------------------------------------------------------------------------------------------------
//	read Varian 2D fid file
//  Note ths the Java virtual machine (JVM) has BIGENDIAN byte order the same as Varian/Agilent
//	data.

	public static int getVarianSer(String fileName, global_par gp,
			data_par q,  data_par q_f1)
	{

		DataFileHeader fh = new DataFileHeader();
		DataBlockHeader bh = new DataBlockHeader();

		FileInputStream fs;
		File file = new File(fileName);

		// try to open the data file, bail out if it fails
		try { fs = new FileInputStream(file); }
		catch (IOException ex)
		{
			Util.show_and_log("getVarianFid(): cannot open: " + fileName, ex.getMessage());
			return -1;
		}

		byte[] fh_byte = new byte[32];
		byte[] bh_byte = new byte[28];

		int bytes_read = 0;
		boolean error = false;     // for breaking nested loops

		// read entire file into byte array
		byte[] b = new byte[(int)file.length()];

		try { fs.read(b); } 
		catch (IOException ex) 
		{
			ex.printStackTrace();
			try { fs.close(); } 
			catch (IOException e) { e.printStackTrace(); }
			return -3;
		}

		// get file header portion		
		for (int i = 0; i < 32; i++)
			fh_byte[i] = b[i];

		ByteBuffer bb_fh = ByteBuffer.wrap(fh_byte);

		fh.nblocks = bb_fh.getInt(0);
		fh.np = bb_fh.getInt(8);
		fh.status = bb_fh.getShort(26);
		fh.nbheaders = bb_fh.getInt(28);
		q.nblocks = fh.nblocks;

		try { gp.ser = new double[q_f1.TD][q.TD]; }
		catch (Exception ex)
		{
			Util.show_and_log("getVarianSer():  " + fileName + ex.getMessage());
			try { fs.close(); } 
			catch (IOException e) { e.printStackTrace(); }
			return -1;
		}

		//  loop through all increments
		for (int i = 0; i < q_f1.TD; i++)
		{          
			//  read block header, and get status field
			for (int k = 0; k < 28; k++)
				bh_byte[k] = b[k + 32 + (i) * (4 * fh.np + 28)];

			ByteBuffer bb_bh = ByteBuffer.wrap(bh_byte);
			bh.status = bb_bh.getShort(2);			
			bytes_read = bytes_read + 28;

			//  now read the data
			if ((bh.status & 0x8) != 0)				// TODO 32 bit float, untested!!! 	
			{
				if (gp.log_on) Util.log_error(
						String.format("getVarianSer(): 32 bit float, i = %d ", i));

				byte[] fid_buffer = new byte[fh.np * 4];
				for(int k = 0; k < fh.np * 4; k++)
					fid_buffer[k] = b[k + 60 + i * (28 + 4 * fh.np)];

				ByteBuffer bb_fid = ByteBuffer.wrap(fid_buffer);

				for (int j = 0; j < q.TD; j++)
				{
					try
					{
						bytes_read = bytes_read + 4;
						gp.ser[i][j] = (double)bb_fid.getFloat(j * 4);
					}
					catch (Exception ex)
					{
						Util.show_and_log("getVarianFid():  " + fileName + ex.getMessage());
						break;
					}
				}
			}

			else if ((bh.status & 0x4) != 0)			//most common, 32 bit ints
			{
				if (gp.log_on) Util.log_error(
						String.format("get_varian_ser(): 32 bit integer, i = %d ", i));

				byte[] fid_buffer = new byte[fh.np * 4];
				for(int k = 0; k < fh.np * 4; k++)
					fid_buffer[k] = b[k + 60 + i * (28 + 4 * fh.np)];

				ByteBuffer bb_fid = ByteBuffer.wrap(fid_buffer);

				for (int j = 0; j < q.TD; j++)
				{
					try
					{
						bytes_read = bytes_read + 4;
						gp.ser[i][j] = bb_fid.getInt(j * 4);
					}
					catch (Exception ex)
					{
						Util.show_and_log("getVarianFid():  " + fileName + ex.getMessage());
						break;
					}
				}
			}

			else if ((bh.status & 0x4) == 0)             // TODO test: 16 bit to save space 
			{
				if (gp.log_on) Util.log_error("get_varian_ser(): 16 bit integer");

				byte[] fid_buffer = new byte[fh.np * 2];
				for(int k = 0; k < fh.np * 2; k++)
					fid_buffer[k] = b[k + 60 + i * (28 + 2 * fh.np)];

				ByteBuffer bb_fid = ByteBuffer.wrap(fid_buffer);

				for (int j = 0; j < q.TD; j++)
				{
					try
					{
						bytes_read = bytes_read + 4;
						gp.ser[i][j] = bb_fid.getInt(j * 2);
					}
					catch (Exception ex)
					{
						Util.show_and_log("getVarianFid():  " + fileName + ex.getMessage());
						break;
					}
				}

			}
			if (error) break;   // break outer loop
		}

		//  clean-up    
		try { fs.close(); } 
		catch (IOException ex) { ex.printStackTrace(); }

		if (gp.log_on)
			Util.log_error(String.format("getVarianSer(): bytes read = {0}", bytes_read));

		return (bytes_read);
	}
        
//-------------------------------------------------------------------------------------------------
}
