
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;

/**
 * 2D Graphics routines for SpinWorks J
 * @author marat, 
 * University of Manitoba
 */
public class GraphUtil2D 
{
	public static int MAX_DISP_BUFF = 80000;
	public static int MAX_PIXELS = 300000;

    public static Pixel_2D[] pix = new Pixel_2D[MAX_PIXELS];
    public static Pixel_2D[] ovl_pix = new Pixel_2D[MAX_PIXELS];

	public static int pix_count; 
	
//-------------------------------------------------------------------------------------------------
/**
 * Draw the 2D display.  If gp.new_2D_view is true, then the data will be re-contoured,
 * else, the pix-map will be refreshed.  Drawing of overlayed traces, projections, or
 * external 1D traces is also handled here.  Normally, the graphics object passed to 
 * this routine is a pixmap, which will be mapped to the screen by canvas.redraw()
 * @param gr The current graphics context
 * @param gp A SpinWorks global parameters object
 * @param q data parameters in direct dimension
 * @param q_ind data parameters in indirect dimension
 * @param update_only if true, only data areaa will be re-drawn, not axes, projections etc. (reduces flicker)
 */
	public static void draw_2D_view(GC gr, global_par gp, 
			data_par q, data_par q_ind, boolean update_only)
	{
		//	check the order of the frequencies
		Util.check_order(gp);

		//	update sizes for 1D "projections" etc.	
		if(gp.rotate_2D)
		{
			if(gp.f2_trace_on) gp.right_2D = 250;
			else gp.right_2D = 100;

			if(gp.f1_trace_on) gp.top_2D = 200;
			else gp.top_2D = 50;
		}
		else
		{
			if(gp.f1_trace_on) gp.right_2D = 250;
			else gp.right_2D = 100;

			if(gp.f2_trace_on) gp.top_2D = 200;
			else gp.top_2D = 50;
		}
		
		gp.xsize_2D = gp.dxsize - gp.left_2D - gp.right_2D;		// room for y axis at left
		gp.ysize_2D = gp.dysize - gp.top_2D - gp.bottom_2D;		// room for axis and title

		//	paint solid background colour first
		gr.setBackground(gp.back_colour);
		gr.fillRectangle(0, 0, gp.dxsize, gp.dysize);

		//	update scaling factors  // TO DO perhaps this can be moved???
		if(!gp.rotate_2D)			// normal (F2 horiz.) 2D view
		{
			gp.hzperpix = (gp.freq1 - gp.freq2)/(double)(gp.xsize_2D-1);
			gp.hzperpix_f1 = (gp.freq1_indirect - gp.freq2_indirect)/(double)(gp.ysize_2D-1);
		}
		else								// rotated (F horiz.) 2D view
		{
			gp.hzperpix = (gp.freq1 - gp.freq2)/(double)(gp.ysize_2D-1);
			gp.hzperpix_f1 = (gp.freq1_indirect - gp.freq2_indirect)/(double)(gp.xsize_2D-1);
		}

		//	draw 2D spectrum          
		if(gp.display_2D == SW_main.display_modes.IMAGE)            // image mode
		{
			if (Math.abs(Math.round
					((gp.freq1 - gp.freq2) / q.hzperpt)) < 512      // use bicubic spline
					&& Math.abs(Math.round ((gp.freq1_indirect - 
						gp.freq2_indirect) / q_ind.hzperpt)) < 256) // interpolation if 
			{                                                       // less that 256 x 512
				if (!gp.rotate_2D)
					draw_2D_image_spline(gr, gp, q, q_ind, 0);
				else
					draw_2D_image_spline_rotated(gr, gp, q, q_ind, 0);
			}
			else		// regular version
			{
				if (!gp.rotate_2D)
					draw_2D_image(gr, gp, q, q_ind, 0);
				else
					draw_2D_image_rotated(gr, gp, q, q_ind, 0);            
			}
		}   // end of image draw

		else if (gp.display_2D == SW_main.display_modes.CONTOUR)   // contour mode	
		{														 
			if (!gp.rotate_2D)
				draw_2D_contour(gr, gp, q, q_ind, 0);
			else
				draw_2D_contour_rotated(gr, gp, q, q_ind, 0);
		}   // end of contour draw

		for (int j = 0; j < gp.nslices; j++)
		{
			if (gp.row_col == SW_main.trace_modes.ROWS)				// draw rows
			{
				if (!gp.rotate_2D)									// normal version	
					drawRowOn2D(gr, gp, q, q_ind, j, 50 + (j * 50), gp.row_scale[j]);
				else												// rotated version
					drawRowOn2D_rotated(gr, gp, q, q_ind, j, 50 + (j * 50), gp.row_scale[j]);
			}
			else if (gp.row_col == SW_main.trace_modes.COLUMNS)		// draw columns
			{
				if (!gp.rotate_2D)									// normal version
					drawColumnOn2D(gr, gp, q, q_ind, j, 40 + (j * 80), gp.row_scale[j]);					
				else												// rotated version
					drawColumnOn2D_rotated(gr, gp, q, q_ind, j, 40 + (j * 80), gp.row_scale[j]);				
			}
		}		
		
		if (update_only)    // only updating data area, so can clean-up and return
		{
			return;
		}

		//  display limits, etc. may have changed, so draw axes, etc.
		//  Due to rounding issues, contours may overlap into border areas.  clear these areas for 
		//  neat display    Really necessary???? YES
		
		gr.setBackground(gp.back_colour);
		gr.fillRectangle(0, 0, gp.left_2D, gp.dysize);      // left
		gr.fillRectangle(0, 0, gp.dxsize, gp.top_2D);       // top
		gr.fillRectangle(0, gp.dysize - gp.bottom_2D,       // bottom
				gp.dxsize, gp.bottom_2D);
		gr.fillRectangle(gp.dxsize - gp.right_2D, 0,        // right
				gp.right_2D, gp.dysize);
				
		//	draw F1 trace, if requested, real-time scanning traces are done in mouse handlers
		if (gp.f1_trace_on && !gp.f1_from_scan && gp.f1_trace != null)
		{
			if (!gp.rotate_2D)
			{
				drawF1On2D(gr, gp.xsize_2D + 20, gp);
				/*if (gp.f1_trace_scale_active)
					gr.DrawString("+/-", font, Brushes.Magenta,
							gp.xsize_2D + gp.left_2D + 30, gp.top_2D - 20);*/
			}
			else
			{
				drawF1On2D_rotated(gr, gp.ysize_2D + 20, gp);
			}
		}
		
		//	draw F2 trace, if requested, real-time scanning traces are done in mouse handlers
		if (gp.f2_trace_on && !gp.f2_from_scan && gp.f2_trace != null)
		{
			if (!gp.rotate_2D)
			{
				drawF2On2D(gr, gp.ysize_2D + 20, gp);
				/*if (gp.f2_trace_scale_active)
					gr.DrawString("+/-", font, Brushes.Magenta,
							gp.xsize_2D + gp.left_2D, gp.top_2D - 40);*/ // TODO
			}
			else
			{
				drawF2On2D_rotated(gr, gp.xsize_2D + 20, gp);
			}
		}
				
		//	draw the axes
		if (!gp.rotate_2D)
		{
			GraphUtil.draw_axis(gr, gp.freq1, gp.freq2, 0, gp.axis_offset, 
					gp.xsize_2D, gp, q, q_ind);
			draw_y_axis_2D(gp, q, q_ind, gr, 
					gp.freq1_indirect, gp.freq2_indirect, gp.left_2D, gp.ysize_2D);
		}
		else
		{
			GraphUtil.draw_axis(gr, gp.freq1_indirect, gp.freq2_indirect, 0, gp.axis_offset, 
					gp.xsize_2D, gp, q, q_ind);
			draw_y_axis_2D(gp, q, q_ind, gr, gp.freq1, gp.freq2, gp.left_2D, gp.ysize_2D);
		}		
					
		//	draw the intensity scale and label buttons
		draw_scale(gr, gp);
		draw_scale_button_labels(gr, gp);

		// refresh any cursor lines
		Device device = Display.getCurrent();
		gr.setForeground(new Color(device, 255, 0, 0));	// Red
		
		if(gp.cursor_flag == 1)
		{
			gr.drawLine(SW_main.cursor1_x, gp.top_2D, SW_main.cursor1_x, 
					gp.dysize - gp.bottom_2D);
			gr.drawLine(gp.left_2D, SW_main.cursor1_y, 
					gp.xsize_2D + gp.left_2D, 
					SW_main.cursor1_y);
		}

		else if(gp.cursor_flag == 2)
		{
			gr.drawLine(SW_main.cursor1_x, gp.top_2D, SW_main.cursor1_x, 
					gp.dysize - gp.bottom_2D);
			gr.drawLine(gp.left_2D, SW_main.cursor1_y,
					gp.xsize_2D + gp.left_2D,
					SW_main.cursor1_y);

			gr.drawLine(SW_main.cursor2_x, gp.top_2D, SW_main.cursor2_x, 
					gp.dysize- gp.bottom_2D);
			gr.drawLine(gp.left_2D, SW_main.cursor2_y, 
					gp.xsize_2D + gp.left_2D,
					SW_main.cursor2_y);
		}
		
		// draw title, if desired
		if(gp.show_title)
		{
			gr.setForeground(gp.axis_colour);
			gr.setFont(new Font(SW_main.shell.getDisplay(), 
					"Arial", SW_main.MEDIUM_FONT, SWT.NORMAL));	// set font   	
			gr.drawText(gp.plot_title, 3, 10);
		}
				
		if(gp.n_peaks > 0 && !gp.rotate_2D)	// show any 2D peak assignments
		{		
			draw2D_Assignments(gr, gp, q, q_ind);
			draw2D_Peaks(gr, gp, q, q_ind);
		}			
	}
	
//-------------------------------------------------------------------------------------------------	
// TODO, perhaps get rid of unused x_offset from these routines
//-------------------------------------------------------------------------------------------------	
/**
 * Draw a 2D image type display
 * @param gr a graphics object
 * @param gp a SpionWorks global parameter object
 * @param q status data parameters, acquisition direction
 * @param q_ind status data parameters, indirect dimension
 * @param x_offset x offset, in pixels
 */
	private static void draw_2D_image(GC gr, global_par gp, data_par q, data_par q_ind,
			int x_offset)
	{
		if (gp.rows_real == null) return;
		int i, j, k, l, x, y, lev;
		int old_x, old_y, old_lev;
		int point1, point2, point1_f1, point2_f1;

		Device device = Display.getCurrent();

		if(gp.new_2D_view == false)		// just refresh display
		{
			old_y = pix[0].y;

			for(i = 0; (i < pix_count) && (i < (MAX_PIXELS)); i++)
			{
				
				if(pix[i].l == 1 && pix[i].k == 1 && Math.abs(pix[i].y - old_y) == 1)
				{	
					gr.setForeground(new Color(device, pix[i].red, pix[i].green, pix[i].blue));
					gr.drawPoint(pix[i].x, pix[i].y);
				}
				else
				{	
					gr.setBackground(new Color(device, pix[i].red, pix[i].green, pix[i].blue));
					gr.fillRectangle(pix[i].x-pix[i].l, pix[i].y-pix[i].k, 
							2*pix[i].l, 2*pix[i].k);
				}
				old_y = pix[i].y;
			}
			return;		// pixels are re-drawn can quit
		}				// end of simple pixel re-draw        

		//	a new 2D view is required
		//	calculate index points in the 2D array from the frequencies
		
		point1 = (int) Math.round((q.offset - gp.freq1) / q.hzperpt);
		point2 = (int) Math.round((q.offset - gp.freq2) / q.hzperpt);
		point1_f1 = (int) Math.round((q_ind.offset - gp.freq1_indirect) / q_ind.hzperpt);
		point2_f1 = (int) Math.round((q_ind.offset - gp.freq2_indirect) / q_ind.hzperpt);
		
		// make sure array bounds are not exceeded
		if(point2 >= q.SI) point2 = q.SI-1;
		if(point1 < 0 || point1 >= q.SI) point1 = 0;

		if(point2_f1 >= q_ind.SI) point2_f1 = q_ind.SI-1;
		if(point1_f1 < 0 || point1_f1 >= q_ind.SI) point1_f1 = 0;

		old_x = gp.left_2D + x_offset;	
		old_y = gp.dysize - gp.bottom_2D;	
		old_lev = 0;


		k = (int) Math.round((double)(gp.ysize_2D) / (double)(point2_f1 - point1_f1));
		l = (int) Math.round((double)(gp.xsize_2D) / (double)(point2 - point1));

		if(k == 0) k = 1;
		if(l == 0) l = 1;

		//	initialize counter and clear pix map to background colour
		for(i = 0; i < pix_count && i < MAX_PIXELS; i++)
		{		
			pix[i].blue = gp.back_colour.getBlue();	
			pix[i].red = gp.back_colour.getRed();		
			pix[i].green = gp.back_colour.getGreen();
			pix[i].x = 0;		    pix[i].y = 0;
			pix[i].k = 0;		    pix[i].l = 0;
		}

		pix_count = 0;

		for(i = point1_f1; i < point2_f1; i++)
			for(j = point1; j < point2; j++)
			{

				x = (int) Math.round
						((gp.freq1 - q.offset + j * q.hzperpt) / gp.hzperpix) 
						+ gp.left_2D + x_offset;

				y = gp.dysize - gp.bottom_2D
						- (int) Math.round
						((gp.freq1_indirect - q_ind.offset + i * q_ind.hzperpt) / gp.hzperpix_f1);

				
				lev = Util_2D.calc_level(gp.rows_real[i][j], gp);

				if((x != old_x || y != old_y || lev > old_lev) && lev != -1)
				{
					if(k == 1 && l == 1 && Math.abs(y - old_y) == 1)
					{
						if(gp.rows_real[i][j] > 0)
						{
							gr.setForeground(new Color(device, gp.pos_level[lev].red, 
									gp.pos_level[lev].green, gp.pos_level[lev].blue));
						}
						else if(!gp.pos_only)
						{
							gr.setForeground(new Color(device, gp.neg_level[lev].red, 
									gp.neg_level[lev].green, gp.neg_level[lev].blue));
						}
						gr.drawPoint(x, y);
					}
					else
					{	
						if(gp.rows_real[i][j] > 0)
						{
							gr.setBackground(new Color(device, gp.pos_level[lev].red, 
									gp.pos_level[lev].green, gp.pos_level[lev].blue));
						}
						else if(!gp.pos_only)
						{
							gr.setBackground(new Color(device, gp.neg_level[lev].red, 
									gp.neg_level[lev].green, gp.neg_level[lev].blue));
						}
						gr.fillRectangle(x-l, y-k, 2*l, 2*k);
					}
					
					old_x = x;	old_y = y; old_lev = lev;
					if (pix_count < (MAX_PIXELS))
					{
						pix[pix_count].x = x;
						pix[pix_count].y = y;  
						pix[pix_count].l = l;
						pix[pix_count].k = k;
						
						if (gp.rows_real[i][j] >= 0)
						{
							pix[pix_count].red = gp.pos_level[lev].red;
							pix[pix_count].blue = gp.pos_level[lev].blue;
							pix[pix_count].green = gp.pos_level[lev].green;
							pix_count++;
						}
						else if (gp.rows_real[i][j] <= 0 && !gp.pos_only)
						{
							pix[pix_count].red = gp.neg_level[lev].red;
							pix[pix_count].blue = gp.neg_level[lev].blue;
							pix[pix_count].green = gp.neg_level[lev].green;
							pix_count++;
						}
					}
					else
					{
						break;
					}
				}
			}

		if (pix_count >= MAX_PIXELS)
			Util.show_and_log("Display buffer exceeded, Raise the floor!");

		gp.new_2D_view = false;
	}

//-------------------------------------------------------------------------------------------------	
/**
 * Draw a 2D image type display, rotated (F1 horizontal)
 * @param gr a graphics object
 * @param gp a SpionWorks global parameter object
 * @param q status data parameters, acquisition direction
 * @param q_ind status data parameters, indirect dimension
 * @param x_offset x offset, in pixels
 */
	private static void draw_2D_image_rotated(GC gr, global_par gp, 
			data_par q, data_par q_ind, int x_offset)
	{
		if (gp.rows_real == null) return;

		int i, j, x, y, lev;
		int old_x, old_y, old_lev;
		Device device = Display.getCurrent();

		if (gp.new_2D_view == false)		// just refresh display
		{
			old_y = pix[0].y;

			for (i = 0; (i < pix_count) && (i < (MAX_PIXELS)); i++)
			{					
				
				if(pix[i].l == 1 && pix[i].k == 1 && Math.abs(pix[i].y - old_y) == 1)
				{	
					gr.setForeground(new Color(device, pix[i].red, pix[i].green, pix[i].blue));
					gr.drawPoint(pix[i].x, pix[i].y);
				}
				else
				{	
					gr.setBackground(new Color(device, pix[i].red, pix[i].green, pix[i].blue));
					gr.fillRectangle(pix[i].x-pix[i].l, pix[i].y-pix[i].l, 
							2 * pix[i].k, 2 * pix[i].l);
				}

				old_y = pix[i].y;
			}
			return;		// pixels are re-drawn can quit
		}				// end of simple pixel re-draw

		//	a new 2D view is required
		//	calculate index points in the 2D array from the frequencies
		int point1, point2, point1_f1, point2_f1;

		point1 = (int) Math.round((q.offset - gp.freq1) / q.hzperpt);
		point2 = (int) Math.round((q.offset - gp.freq2) / q.hzperpt);
		point1_f1 = (int) Math.round((q_ind.offset - gp.freq1_indirect) / q_ind.hzperpt);
		point2_f1 = (int) Math.round((q_ind.offset - gp.freq2_indirect) / q_ind.hzperpt);

		// make sure array bounds are not exceeded
		if (point2 >= q.SI) point2 = q.SI - 1;
		if (point1 < 0 || point1 >= q.SI) point1 = 0;

		if (point2_f1 >= q_ind.SI) point2_f1 = q_ind.SI - 1;
		if (point1_f1 < 0 || point1_f1 >= q_ind.SI) point1_f1 = 0;

		old_x = gp.left_2D + x_offset; 
		old_y = gp.dysize - gp.bottom_2D;
		old_lev = 0;

		int k, l;

		k = (int) Math.round((double)(gp.xsize_2D) / (double)(point2_f1 - point1_f1));
		l = (int) Math.round((double)(gp.ysize_2D) / (double)(point2 - point1));

		if (k == 0) k = 1;
		if (l == 0) l = 1;

		//	initialize counter and clear pix map to background
		for (i = 0; i < pix_count && i < MAX_PIXELS; i++)
		{
			pix[i].blue = gp.back_colour.getBlue();
			pix[i].red = gp.back_colour.getRed();
			pix[i].green = gp.back_colour.getGreen();
			pix[i].x = 0; pix[i].y = 0;
			pix[i].k = 0; pix[i].l = 0;
		}

		pix_count = 0;

		for (i = point1_f1; i < point2_f1; i++)
			for (j = point1; j < point2; j++)
			{
				x = (int) Math.round
						((gp.freq1_indirect - q_ind.offset + i * q_ind.hzperpt) / gp.hzperpix_f1) 
						+ gp.left_2D + x_offset;

				y = gp.dysize - gp.bottom_2D
						- (int) Math.round
						((gp.freq1 - q.offset + j * q.hzperpt) / gp.hzperpix);

				lev = Util_2D.calc_level(gp.rows_real[i][j], gp);

				if ((x != old_x || y != old_y || lev > old_lev) && lev != -1)
				{
					if (k == 1 && l == 1 && Math.abs(y - old_y) == 1)
					{
						if (gp.rows_real[i][j] > 0)
						{
							gr.setForeground(new Color(device, gp.pos_level[lev].red, 
									gp.pos_level[lev].green, gp.pos_level[lev].blue));
						}
						else if (!gp.pos_only)
						{
							gr.setForeground(new Color(device, gp.neg_level[lev].red, 
									gp.neg_level[lev].green, gp.neg_level[lev].blue));
						}
						gr.drawPoint(x,  y);
					}
					else
					{
						if (gp.rows_real[i][j] > 0)
						{
							gr.setBackground(new Color(device, gp.pos_level[lev].red, 
									gp.pos_level[lev].green, gp.pos_level[lev].blue));
						}
						else if (!gp.pos_only)
						{
							gr.setBackground(new Color(device, gp.neg_level[lev].red, 
									gp.neg_level[lev].green, gp.neg_level[lev].blue));
						}
						gr.fillRectangle(x - k, y - l, 2 * k, 2 * l);
					}
					old_x = x; old_y = y; old_lev = lev;
					if (pix_count < (MAX_PIXELS))
					{
						pix[pix_count].x = x; pix[pix_count].y = y;
						pix[pix_count].l = l; pix[pix_count].k = k;
						if (gp.rows_real[i][j] >= 0)
						{
							pix[pix_count].red = gp.pos_level[lev].red;
							pix[pix_count].blue = gp.pos_level[lev].blue;
							pix[pix_count].green = gp.pos_level[lev].green;
							pix_count++;
						}
						else if (gp.rows_real[i][j] <= 0 && !gp.pos_only)
						{
							pix[pix_count].red = gp.neg_level[lev].red;
							pix[pix_count].blue = gp.neg_level[lev].blue;
							pix[pix_count].green = gp.neg_level[lev].green;
							pix_count++;
						}
					}
					//pix_count++;
					else
					{
						break;
					}
				}
			}
		if (pix_count >= (MAX_PIXELS))
			Util.show_and_log("Display buffer exceeded (2)\nRaise the floor!");

		gp.new_2D_view = false;
	}
   
//-------------------------------------------------------------------------------------------------
/**
 * Draw an image type 2D with bicubic spline interpolation
 * Gives smoother images at high expansion
 * @param gr a graphics object
 * @param gp A SpinWorks global parameter object
 * @param q status data parameters, acquisition dimension
 * @param q_ind status data parameters, indirect dimension
 * @param x_offset x screen offset in pixels
 */
    private static void draw_2D_image_spline(GC gr, global_par gp, 
    		data_par q, data_par q_ind, int x_offset)
    {
    	if (gp.rows_real == null) return;
    	int i, j, x, y, lev;
    	int old_x, old_y, old_lev;

    	Device device = Display.getCurrent();

    	if(gp.new_2D_view == false)		// just refresh display
    	{
    		old_y = pix[0].y;

    		for(i = 0; (i < pix_count) && (i < (MAX_PIXELS)); i++)
    		{
    			if(pix[i].l == 1 && pix[i].k == 1 && Math.abs(pix[i].y - old_y) == 1)
    			{	
    				gr.setForeground(new Color(device, pix[i].red, pix[i].green, pix[i].blue));
    				gr.drawPoint(pix[i].x, pix[i].y);
    			}
    			else
    			{	
    				gr.setBackground(new Color(device, pix[i].red, pix[i].green, pix[i].blue));
    				gr.fillRectangle(pix[i].x-pix[i].l, pix[i].y-pix[i].k, 
    						2*pix[i].l, 2*pix[i].k);
    			}
    			old_y = pix[i].y;
    		}
    		return;		// pixels are re-drawn can quit
    	}				// end of simple pixel re-draw    

    	//	initialize counter and clear pix map to background colour
    	for(i = 0; i < pix_count && i < MAX_PIXELS; i++)
    	{
    		pix[i].blue = gp.back_colour.getBlue();	
    		pix[i].red = gp.back_colour.getRed();		
    		pix[i].green = gp.back_colour.getGreen();
    		pix[i].x = 0;		    pix[i].y = 0;
    		pix[i].k = 0;		    pix[i].l = 0;
    	}

    	pix_count = 0;

    	int point1, point2, point1_f1, point2_f1;

    	point1 = (int) Math.round((q.offset - gp.freq1) / q.hzperpt) - 1;
    	point2 = (int) Math.round((q.offset - gp.freq2) / q.hzperpt) + 1;
    	point1_f1 = (int) Math.round((q_ind.offset - gp.freq1_indirect) / q_ind.hzperpt) - 1;
    	point2_f1 = (int) Math.round((q_ind.offset - gp.freq2_indirect) / q_ind.hzperpt) + 1;

    	// make sure array bounds are not exceeded
    	if(point2 >= q.SI) point2 = q.SI-1;
    	if(point1 < 0 || point1 >= q.SI) point1 = 0;

    	if(point2_f1 >= q_ind.SI) point2_f1 = q_ind.SI-1;
    	if(point1_f1 < 0 || point1_f1 >= q_ind.SI) point1_f1 = 0;

    	old_x = gp.left_2D + x_offset;	
    	old_y = gp.dysize - gp.bottom_2D;	
    	old_lev = 0;

    	int k, l;

    	k = (int)Math.round(0.5 * (double)(gp.xsize_2D) / (double)(point2_f1 - point1_f1));
    	l = (int)Math.round(0.5 * (double)(gp.ysize_2D) / (double)(point2 - point1));

    	if(k == 0) k = 1;
    	if(l == 0) l = 1;

    	float[][] buff_mat = new float[2 * (point2_f1 - point1_f1 + 1)][ 2 * (point2 - point1 + 1)];
    	Contour.interpolate_2D(gp.rows_real, buff_mat, point1, point2, point1_f1, point2_f1);

    	for(i = 2*point1_f1; i < 2*point2_f1; i++)
    		for(j = 2*point1; j < 2*point2; j++)
    		{

    			x = (int)Math.round
    					((gp.freq1 - q.offset + j * q.hzperpt / 2.0) / gp.hzperpix) 
    					+ gp.left_2D + x_offset;

    			y = gp.dysize - gp.bottom_2D - (int)Math.round((gp.freq1_indirect - q_ind.offset
    					+ i * q_ind.hzperpt / 2.0) / gp.hzperpix_f1);


    			lev = Util_2D.calc_level(buff_mat[i - (2*point1_f1)][ j - (2*point1)], gp);

    			if((x != old_x || y != old_y || lev > old_lev) && lev != -1)
    			{
    				if(k == 1 && l == 1 && Math.abs(y - old_y) == 1)
    				{
    					if (buff_mat[i - (2*point1_f1)][ j - (2*point1)] > 0)
    					{
    						gr.setForeground(new Color(device, gp.pos_level[lev].red, 
    								gp.pos_level[lev].green, gp.pos_level[lev].blue));
    					}
    					else if(!gp.pos_only)
    					{

    						gr.setForeground(new Color(device, gp.neg_level[lev].red, 
    								gp.neg_level[lev].green, gp.neg_level[lev].blue));
    					}
    					gr.drawPoint(x,  y);
    				}
    				else
    				{
    					if (buff_mat[i - (2*point1_f1)][ j - (2*point1)] > 0)
    					{
    						gr.setBackground(new Color(device, gp.pos_level[lev].red, 
    								gp.pos_level[lev].green, gp.pos_level[lev].blue));
    					}
    					else if(!gp.pos_only)
    					{
    						gr.setBackground(new Color(device, gp.neg_level[lev].red, 
    								gp.neg_level[lev].green, gp.neg_level[lev].blue));
    					}
    					gr.fillRectangle(x-l, y-k, 2*l, 2*k);
    				}
    				old_x = x;	old_y = y; old_lev = lev;
    				if (pix_count < (MAX_PIXELS))
    				{
    					pix[pix_count].x = x; pix[pix_count].y = y;
    					pix[pix_count].l = l; pix[pix_count].k = k;
    					if (buff_mat[i - (2 * point1_f1)][ j - (2 * point1)] >= 0)
    					{
    						pix[pix_count].red = gp.pos_level[lev].red;
    						pix[pix_count].blue = gp.pos_level[lev].blue;
    						pix[pix_count].green = gp.pos_level[lev].green;
    						pix_count++;
    					}
    					else if (buff_mat[i - (2 * point1_f1)][ j - (2 * point1)] <= 0 && !gp.pos_only)
    					{
    						pix[pix_count].red = gp.neg_level[lev].red;
    						pix[pix_count].blue = gp.neg_level[lev].blue;
    						pix[pix_count].green = gp.neg_level[lev].green;
    						pix_count++;
    					}
    				}
    				else
    				{
    					break;
    				}
    			}
    		}

    	if(pix_count >= (MAX_PIXELS))
    		Util.show_and_log("Display buffer exceeded (3)\nRaise the floor!");

    	gp.new_2D_view = false;
    }

//-------------------------------------------------------------------------------------------------
/**
 * Draw an image type 2D with bicubic spline interpolation, 
 * rotated (indirect horizontal) version.  
 * Gives smoother images at high expansion
 * @param gr a graphics object
 * @param gp A SpinWorks global parameter object
 * @param q status data parameters, acquisition dimension
 * @param q_ind status data parameters, indirect dimension
 * @param x_offset x screen offset in pixels
 */
    private static void draw_2D_image_spline_rotated(GC gr, global_par gp, 
    		data_par q, data_par q_ind, int x_offset)
    {
    	if (gp.rows_real == null) return;
    	int i, j, k, l, x, y, lev;
    	int point1, point2, point1_f1, point2_f1;
    	int old_x, old_y, old_lev;

    	Device device = Display.getCurrent();

    	if (gp.new_2D_view == false)					    // just refresh display
    	{
    		old_y = pix[0].y;

    		for (i = 0; (i < pix_count) && (i < (MAX_PIXELS)); i++)
    		{
    			if (pix[i].l == 1 && pix[i].k == 1 && Math.abs(pix[i].y - old_y) == 1)
    			{
    				gr.setForeground(new Color(device, pix[i].red, pix[i].green, pix[i].blue));
    				gr.drawPoint(pix[i].x, pix[i].y);
    			}
    			else
    			{
    				gr.setBackground(new Color(device, pix[i].red, pix[i].green, pix[i].blue));
    				gr.fillRectangle(pix[i].x - pix[i].k, pix[i].y - pix[i].l,
    						2 * pix[i].k, 2 * pix[i].l);
    			}
    			old_y = pix[i].y;
    		}
    		return;		// pixels are re-drawn can quit
    	}				// end of simple pixel re-draw


    	for (i = 0; i < pix_count; i++)
    	{
    		pix[i].blue = gp.back_colour.getBlue();
    		pix[i].red = gp.back_colour.getRed();
    		pix[i].green = gp.back_colour.getGreen();
    		pix[i].x = 0; pix[i].y = 0;
    		pix[i].k = 0; pix[i].l = 0;
    	}

    	pix_count = 0;

    	point1 = (int) Math.round((q.offset - gp.freq1) / q.hzperpt) - 1;
    	point2 = (int) Math.round((q.offset - gp.freq2) / q.hzperpt) + 1;
    	point1_f1 = (int) Math.round((q_ind.offset - gp.freq1_indirect) / q_ind.hzperpt) - 1;
    	point2_f1 = (int) Math.round((q_ind.offset - gp.freq2_indirect) / q_ind.hzperpt) + 1;

    	// make sure array bounds are not exceeded
    	if (point2 >= q.SI) point2 = q.SI - 1;
    	if (point1 < 0 || point1 >= q.SI) point1 = 0;

    	if (point2_f1 >= q_ind.SI) point2_f1 = q_ind.SI - 1;
    	if (point1_f1 < 0 || point1_f1 >= q_ind.SI) point1_f1 = 0;

    	old_x = gp.left_2D + x_offset; 
    	old_y = gp.dysize - gp.bottom_2D;
    	old_lev = 0;

    	k = (int) Math.round(0.5 * (double)(gp.xsize_2D) / (double)(point2_f1 - point1_f1));
    	l = (int) Math.round(0.5 * (double)(gp.ysize_2D) / (double)(point2 - point1));

    	if (k == 0) k = 1;
    	if (l == 0) l = 1;

    	float[][] buff_mat = new float[2 * (point2_f1 - point1_f1 + 1)][2 * (point2 - point1 + 1)];
    	Contour.interpolate_2D(gp.rows_real, buff_mat, point1, point2, point1_f1, point2_f1);

    	for (i = 2 * point1_f1; i < 2 * point2_f1; i++)
    		for (j = 2 * point1; j < 2 * point2; j++)
    		{

    			x = (int) Math.round
    					((gp.freq1_indirect - q_ind.offset + i * q_ind.hzperpt /
    							2.0) / gp.hzperpix_f1) + gp.left_2D + x_offset;

    			y = gp.dysize - gp.bottom_2D - (int) Math.round
    					((gp.freq1 - q.offset
    							+ j * q.hzperpt / 2.0) / gp.hzperpix);


    			lev = Util_2D.calc_level(buff_mat[i - (2 * point1_f1)][j - (2 * point1)], gp);

    			if ((x != old_x || y != old_y || lev > old_lev) && lev != -1)
    			{
    				if (k == 1 && l == 1 && Math.abs(y - old_y) == 1)
    				{
    					if (buff_mat[i - (2 * point1_f1)][j - (2 * point1)] > 0)
    					{
    						gr.setForeground(new Color(device, gp.pos_level[lev].red, 
    								gp.pos_level[lev].green, gp.pos_level[lev].blue));
    					}
    					else if (!gp.pos_only)
    					{
    						gr.setForeground(new Color(device, gp.neg_level[lev].red, 
    								gp.neg_level[lev].green, gp.neg_level[lev].blue));
    					}
    					gr.drawPoint(x,  y);
    				}
    				else
    				{
    					if (buff_mat[i - (2 * point1_f1)][j - (2 * point1)] > 0)
    					{
    						gr.setBackground(new Color(device, gp.pos_level[lev].red, 
    								gp.pos_level[lev].green, gp.pos_level[lev].blue));
    					}
    					else if (!gp.pos_only)
    					{
    						gr.setBackground(new Color(device, gp.neg_level[lev].red, 
    								gp.neg_level[lev].green, gp.neg_level[lev].blue));
    					}
    					gr.fillRectangle(x - k, y - l, 2 * k, 2 * l);
    				}
    				old_x = x; old_y = y; old_lev = lev;
    				if (pix_count < (MAX_PIXELS))
    				{
    					pix[pix_count].x = x; pix[pix_count].y = y;
    					pix[pix_count].l = l; pix[pix_count].k = k;
    					if (buff_mat[i - (2 * point1_f1)][j - (2 * point1)] >= 0)
    					{
    						pix[pix_count].red = gp.pos_level[lev].red;
    						pix[pix_count].blue = gp.pos_level[lev].blue;
    						pix[pix_count].green = gp.pos_level[lev].green;
    						pix_count++;
    					}
    					else if (buff_mat[i - (2 * point1_f1)][j - (2 * point1)] <= 0
    							&& !gp.pos_only)
    					{
    						pix[pix_count].red = gp.neg_level[lev].red;
    						pix[pix_count].blue = gp.neg_level[lev].blue;
    						pix[pix_count].green = gp.neg_level[lev].green;
    						pix_count++;
    					}
    				}
    				else
    				{
    					break;
    				}
    			}
    		}

    	if (pix_count >= (MAX_PIXELS))
    		Util.show_and_log("Display buffer exceeded \nRaise the floor!");

    	gp.new_2D_view = false;
    }
       
//-------------------------------------------------------------------------------------------------
/**
 * Draw a 2D contour type display
 * @param gr a graphics object
 * @param gp a SpinWorks global parameter object
 * @param q status data parameters, acquisition dimension
 * @param q_ind status acquisition parameters, indirect dimension
 * @param x_offset screen x offset in pixels
 */
    private static void draw_2D_contour(GC gr, global_par gp, data_par q, data_par q_ind,
			int x_offset)
	{          
		int i;											// handy counters
		int x_off, y_off;								// screen offsets
		int x1, y1, x2, y2;								// line segment coordinates
		Device device = Display.getCurrent();

		//	find screen offset in pixels

		x_off = gp.left_2D
				- (int)Math.round((q.offset - gp.freq1) / gp.hzperpix)
				+ x_offset;
		y_off = (int)Math.round((q_ind.offset - gp.freq1_indirect) / gp.hzperpix_f1);

		//	save arithmetic in loop
		double ratio = q.hzperpt / gp.hzperpix;				// pix per pt in F2
		double ratio_f1 = q_ind.hzperpt / gp.hzperpix_f1;	// pix per pt in F1
		int y_orig = gp.dysize - gp.bottom_2D + y_off;

		if(gp.new_2D_view)							// threshold or limits, etc. have changed					
			Contour.do_contours(gp, q, q_ind);		// so generate a new set of contours         

		//	currently, positive level contours are always drawn
		for(i = 0; i < gp.n_segments; i++)
		{			
			gr.setForeground(new Color(device,
					gp.pos_level[gp.seg_pos[i].level].red,
					gp.pos_level[gp.seg_pos[i].level].green,
					gp.pos_level[gp.seg_pos[i].level].blue));

			x1 = (int)Math.round((double)gp.seg_pos[i].x1 * ratio) + x_off;
			y1 = y_orig - (int)Math.round((double)gp.seg_pos[i].y1 * ratio_f1);
			x2 = (int)Math.round((double)gp.seg_pos[i].x2 * ratio) + x_off;
			y2 = y_orig - (int)Math.round((double)gp.seg_pos[i].y2 * ratio_f1);
			
			gr.drawLine(x1, y1, x2, y2);
		}          

		//	negative contours are only drawn if requested
		for(i = 0; i < gp.n_segments_neg; i++)
		{				
			gr.setForeground(new Color(device,
					gp.neg_level[gp.seg_neg[i].level].red,
					gp.neg_level[gp.seg_neg[i].level].green,
					gp.neg_level[gp.seg_neg[i].level].blue));

			x1 = (int)Math.round((double)gp.seg_neg[i].x1 * ratio) + x_off;
			y1 = y_orig - (int)Math.round((double)gp.seg_neg[i].y1 * ratio_f1);
			x2 = (int)Math.round((double)gp.seg_neg[i].x2 * ratio) + x_off;
			y2 = y_orig - (int)Math.round((double)gp.seg_neg[i].y2 * ratio_f1);
			
			gr.drawLine(x1, y1, x2, y2);
		}

		gp.new_2D_view = false;
	}
        
//-------------------------------------------------------------------------------------------------
/**
 * Draw a 2D contour type display, rotated (indirect horizontal)
 * @param gr a graphics object
 * @param gp a SpinWorks global parameter object
 * @param q status data parameters, acquisition dimension
 * @param q_ind status acquisition parameters, indirect dimension
 * @param x_offset screen x offset in pixels
 */
    private static void draw_2D_contour_rotated(GC gr, global_par gp, 
			data_par q, data_par q_ind, int x_offset)
	{
		int i;											// handy counters
		int x_off, y_off;								// screen offsets
		int x1, y1, x2, y2;								// line segment coordinates
		Device device = Display.getCurrent();

		//	find screen offset in pixels
		
		x_off = gp.left_2D
				- (int) Math.round((q_ind.offset - gp.freq1_indirect) / gp.hzperpix_f1)
				+ x_offset;
		y_off = (int) Math.round((q.offset - gp.freq1) / gp.hzperpix);
		
		//	save arithmetic in loop
		double ratio = q.hzperpt / gp.hzperpix;				// pix per pt in F2
		double ratio_f1 = q_ind.hzperpt / gp.hzperpix_f1;	// pix per pt in F1
		int y_orig = gp.dysize - gp.bottom_2D + y_off;

		if (gp.new_2D_view)							// threshold or limits, etc. have changed					
			Contour.do_contours(gp, q, q_ind);		// so generate a new set of contours

		//	currently, positive level contours are always drawn
		for (i = 0; i < gp.n_segments; i++)
		{
			gr.setForeground(new Color(device,
					gp.pos_level[gp.seg_pos[i].level].red,
					gp.pos_level[gp.seg_pos[i].level].green,
					gp.pos_level[gp.seg_pos[i].level].blue));

			x1 = (int)Math.round((double)gp.seg_pos[i].y1 * ratio_f1) + x_off;
			y1 = y_orig - (int)Math.round((double)gp.seg_pos[i].x1 * ratio);
			x2 = (int)Math.round((double)gp.seg_pos[i].y2 * ratio_f1) + x_off;
			y2 = y_orig - (int)Math.round((double)gp.seg_pos[i].x2 * ratio);
			
			gr.drawLine(x1, y1, x2, y2);
		}

		//	negative contours are only drawn if requested
		for (i = 0; i < gp.n_segments_neg; i++)
		{
			gr.setForeground(new Color(device,
					gp.neg_level[gp.seg_neg[i].level].red,
					gp.neg_level[gp.seg_neg[i].level].green,
					gp.neg_level[gp.seg_neg[i].level].blue));


				x1 = (int)Math.round((double)gp.seg_neg[i].y1 * ratio_f1) + x_off;
				y1 = y_orig - (int)Math.round((double)gp.seg_neg[i].x1 * ratio);
				x2 = (int)Math.round((double)gp.seg_neg[i].y2 * ratio_f1) + x_off;
				y2 = y_orig - (int)Math.round((double)gp.seg_neg[i].x2 * ratio);
			
			gr.drawLine(x1, y1, x2, y2);
		}

		gp.new_2D_view = false;
	}
        
//-------------------------------------------------------------------------------------------------
/**
 * Draws an intensity scale on the right-hand side of the 2D display
 * @param gr a graphics object
 * @param gp a SpinWorks global parameter object
 */
	private  static void draw_scale(GC gr, global_par gp)
	{
		int y = gp.dysize - gp.bottom_2D - 60;
		int x = gp.dxsize  - 80;
		int h = 15;
		int w = 25;
		int i;
		boolean anycor;
		String text;
		Device device = Display.getCurrent();

		// first annotate the correlations
		gr.setFont(new Font(SW_main.shell.getDisplay(), "Arial", SW_main.MEDIUM_FONT, SWT.NORMAL));
		
		anycor = false;
		for(i = 0; i < gp.n_peaks; i++)
			if(gp.peaks[i].n_cosy > 0)
				anycor = true;
		if(gp.showCOSYcorr && anycor)
		{
			gr.setForeground(gp.cosy_colour);
			gr.drawText("COSY", x, y);
		}
		y += 20;
		
		anycor = false;
		for(i = 0; i < gp.n_peaks; i++)
			if(gp.peaks[i].n_tocsy > 0)
				anycor = true;
		if(gp.showTOCSYcorr && anycor)
		{
			gr.setForeground(gp.tocsy_colour);
			gr.drawText("TOCSY", x, y);
		}
		y += 20;
		
		anycor = false;
		for(i = 0; i < gp.n_peaks; i++)
			if(gp.peaks[i].n_noesy > 0)
				anycor = true;
		if(gp.showNOESYcorr && anycor)
		{
			gr.setForeground(gp.noesy_colour);
			gr.drawText("NOESY", x, y);
		}
		
		anycor = false;
		for(i = 0; i < gp.n_peaks; i++)
			if(gp.peaks[i].n_hmbc > 0)
				anycor = true;
		y += 20;
		if(gp.showHMBCcorr && anycor)
		{
			gr.setForeground(gp.hmbc_colour);
			gr.drawText("HMBC", x, y);
		}
		
		// now the contours levelling
		gr.setFont(new Font(SW_main.shell.getDisplay(), "Arial", SW_main.SMALL_FONT, SWT.NORMAL));
		
		y = gp.ysize_2D/2 + gp.top_2D;

		for( i = 0; i < gp.n_levels; i++)
		{
			if(i == 0 || i == gp.n_levels -1)
			{
				gr.setForeground(gp.axis_colour);
				text = String.format("%.0f", gp.pos_level[i].value / 1000);
				text = text.trim();
				Point size = gr.textExtent(text);
				if(!gp.pos_only)
					gr.drawText(text, gp.dxsize - 40 - size.x, y-h);
				else
					gr.drawText(text, gp.dxsize - 40 - size.x, y);
			}
			
			gr.setBackground(new Color(device, gp.pos_level[i].red, gp.pos_level[i].green, gp.pos_level[i].blue ));
			gr.fillRectangle(gp.dxsize -35, y, w, h);
			gr.setBackground(gp.back_colour);
			y = y - h;
		}

		if(gp.pos_only) // is only positive volours (e.g. COSY) we can return
			return;

		y = gp.ysize_2D / 2 + gp.top_2D;

		for(i = 0; i < gp.n_levels; i++)
		{
			if(i == 0 || i == gp.n_levels -1)
			{
				gr.setForeground(gp.axis_colour);
				text = String.format("%.0f", gp.neg_level[i].value / 1000);
				text = text.trim();
				Point size = gr.textExtent(text);				// and centre number
				gr.drawText(text, gp.dxsize -40 - size.x, y);
			}

			gr.setBackground(new Color(device, gp.neg_level[i].red, gp.neg_level[i].green, 
					gp.neg_level[i].blue ));
			gr.fillRectangle(gp.dxsize - 35, y, w, h);
			gr.setBackground(gp.back_colour);
			y = y + h;
		}
				
	}

//-------------------------------------------------------------------------------------------------
/**
 * Draws the scale button labels at the right of the 2D display
 * @param gr a graphics object
 * @param gp a SpinWorks global parameter object
 */
	private static void draw_scale_button_labels(GC gr, global_par gp)
	{
		int x = gp.dxsize - 10;
		int y = 5;
		int spacing = 25;

		gr.setForeground(gp.axis_colour);
		gr.setFont(new Font(SW_main.shell.getDisplay(), "Arial", SW_main.SMALL_FONT, SWT.NORMAL));
		
		String msg = "Floor:";
		Point size = gr.textExtent(msg);	
		gr.drawText(msg, x - size.x, y);
		
		y += spacing;
		msg = "Range:";
		size = gr.textExtent(msg);
		gr.drawText(msg, x - size.x, y);
		
		y += spacing;
		msg = "Trace:";
		size = gr.textExtent(msg);
		gr.drawText(msg, x - size.x, y);
		
		y += spacing;
		msg = "1D Scale:";
		size = gr.textExtent(msg);
		gr.drawText(msg, x - size.x, y);       
	}	
	
//------------------------------------------------------------------------------------------------
/**
 * Draw the indirect axis for 2D (Y) or 3D (Y or Z)
 * @param gp a SpinWorks global parameter object
 * @param q status parameters, acquisitikon dimension
 * @param q_ind status parameters, indirect dimension
 * @param gr a graphics object
 * @param f1 start frequency (Hz)
 * @param f2 end frequency (Hz)
 * @param x_offset x offset in pixels
 * @param ysize y size in pixels
 */
	private static void draw_y_axis_2D(global_par gp, data_par q, data_par q_ind, 
			GC gr, double f1, double f2, int x_offset, int ysize)
	{
		double range, tic_space = 0.1, first_tic, tic, pow10;
		int x, y, old_y;
		int i, iexp;
		String number;
		double unitperpix;
		
		gr.setFont(new Font(SW_main.shell.getDisplay(), "Arial", SW_main.MEDIUM_FONT, SWT.NORMAL));
		gr.setForeground(gp.axis_colour);

		if(gp.axis_unit_indirect == SW_main.units.PPM)
		{
			if(!gp.rotate_2D)
			{	f1 = f1/q_ind.SF;	f2 = f2/q_ind.SF;	}	// normal orientation
			else
			{	f1 = f1/q.SF;		f2 = f2/q.SF;		}	// rotated orientation
		}

		iexp = 0;
		range = f1 - f2;
		unitperpix = (f1 - f2)/(double)(ysize - 1);

		// scale range to between 1 to 10, keep track of power of 10 

		if(range >= 1.0)
			while(range >= 10.0)
			{
				range = range/10.0;
				iexp++;
			}
		else		//(range < 1.0)
			while(range < 1.0)
			{
				range = range*10.0;
				iexp--;
			}

		if(Math.abs(f1) < 10000.0)
		{
			if(range < 1.0 && range >= 0.5)	tic_space = 0.02;			
			else if(range < 2.0 && range >= 1.0)	tic_space = 0.05;	
			else if(range < 5.0 && range >= 2.0)	tic_space = 0.1;	
			else if(range < 10.0 && range >= 5.0)	tic_space = 0.2;	
		}
		else		// big numbers, coarser spacing
		{
			if(range < 1.0 && range >= 0.5)	tic_space = 0.05;			
			else if(range < 2.0 && range >= 1.0)	tic_space = 0.1;	
			else if(range < 5.0 && range >= 2.0)	tic_space = 0.2;	
			else if(range < 10.0 && range >= 5.0)	tic_space = 0.5;	
		}

		pow10 = Math.pow(10.0, (double)iexp);

		// calculate real tic spacing and rational first tic position 
		double ts = tic_space;
		tic_space = tic_space * pow10;

		first_tic = tic_space * (Math.round((f1 - tic_space) / tic_space));

		double ft = first_tic/pow10;

		// set up graphics, initial location
		x = x_offset;
		y = gp.dysize - gp.bottom_2D;

		String str = "";

		if(gp.rotate_2D)
		{
			if (q.data_mode == SW_main.modes.MODE_3D)
			{
				if (gp.axis_unit_indirect == SW_main.units.HZ)
					str = "Hz \nX(F3)";

				else if (gp.axis_unit_indirect == SW_main.units.PPM)
					str = "PPM \nX(F3)";
			}
			else
			{
				if (gp.axis_unit_indirect == SW_main.units.HZ)
					str = "Hz \n(F2)";

				else if (gp.axis_unit_indirect == SW_main.units.PPM)
					str = "PPM \n(F2)";
			}
		}
		else
		{
			if (q.data_mode == SW_main.modes.MODE_3D && 
					gp.mode_3D == SW_main.modes_3D.XY)
			{
				if (gp.axis_unit_indirect == SW_main.units.HZ)
					str = "Hz Y\n(F2)";

				else if (gp.axis_unit_indirect == SW_main.units.PPM)
					str = "PPM Y\n(F2)";
			}
			else if (q.data_mode == SW_main.modes.MODE_3D &&
					gp.mode_3D == SW_main.modes_3D.XZ)
			{
				if (gp.axis_unit_indirect == SW_main.units.HZ)
					str = "Hz Z\n(F1)";

				else if (gp.axis_unit_indirect == SW_main.units.PPM)
					str = "PPM Z\n(F1)";
			}
			else
			{
				if (gp.axis_unit_indirect == SW_main.units.HZ)
					str = "Hz \n(F1)";

				else if (gp.axis_unit_indirect == SW_main.units.PPM)
					str = "PPM \n(F1)";
			}
		}

		gr.drawText(str, 1, gp.top_2D);

		tic = first_tic;

		if (tic_space < 0.001 || tic_space > 10000.0)
		{
			if ((ft / (4.0 * ts) - Math.round(ft / (4.0 * ts))) > 0.001)
				i = 1;
			else
				i = 0;
		}
		else
		{
			if ((ft / (2.0 * ts) - Math.round(ft / (2.0 * ts))) > 0.001)
				i = 1;
			else
				i = 0;
		}


		while((int)((f1-tic)/unitperpix) <= ysize)
		{
			old_y = y;
			y = gp.dysize - gp.bottom_2D - (int)Math.round((f1 - tic) / unitperpix); 

			gr.drawLine(x, old_y, x, y);

			if(i == 0)
			{
				gr.drawLine(x, y, x-10, y);
				if(y > gp.top_2D + 30 &&  y < gp.dysize - 20 - gp.bottom_2D)
				{
					// decide on number format
					if(tic_space < 0.001)
						number = String.format("%.4f", tic);
					else if(tic_space < 0.01)
						number = String.format("%.3f", tic);
					else if(tic_space <0.1)
						number = String.format("%.2f", tic);
					else if(tic_space < 1.0 )
						number = String.format("%.1f", tic);
					else if(tic_space <10.0 )
						number = String.format("%.0f", tic);
					else if(tic_space < 100.0 )
						number = String.format("%.0f", tic);
					else if(tic_space < 1000.0 )
						number = String.format("%.0f", tic);
					else if(tic_space < 10000.0 )
						number = String.format("%.0f", tic);
					else if(tic_space < 100000.0 )
						number = String.format("%.0f", tic);
					else
						number = String.format("%.0f", tic);

					number = number.trim();							// remove white space
					Point size = gr.textExtent(number);				// and centre number   				
					gr.drawText(number, x - 15 - size.x, y - size.y / 2);
				}
				i=1;
			}
			else
			{ 
				gr.drawLine(x, y, x-5, y);
				i=0;
			}
			tic = tic - tic_space;
		}		// end while

		gr.drawLine(x, y, x, gp.top_2D);	// now clean up 
		gr.drawLine(x_offset, gp.top_2D, gp.xsize_2D + x_offset, gp.top_2D);
		gr.drawLine(gp.xsize_2D + x_offset, gp.top_2D, 
				gp.xsize_2D + x_offset, gp.dysize - gp.bottom_2D);

		// show phase pivot point if in phase mode 
		if (gp.phase_mode)
		{
			if (gp.axis_unit_indirect == SW_main.units.HZ)
				y = gp.dysize - gp.bottom_2D -
				(int)Math.round((f1 - (q_ind.offset
						- (double)gp.pivot_ind * q_ind.hzperpt)) / unitperpix);

			else if (gp.axis_unit_indirect == SW_main.units.PPM)
				y = gp.dysize - gp.bottom_2D -
				(int)Math.round((f1 - (q_ind.offset
						- (double)gp.pivot_ind * q_ind.hzperpt) / q_ind.SF) / unitperpix);

			gr.setForeground(new Color(Display.getCurrent(), 255, 100, 100));
			gr.setLineWidth(3);
			gr.drawLine(0, y, x_offset, y);
		}
	} // End of draw_y_axis()
		
//-------------------------------------------------------------------------------------------------
/**
 * Draw 1D overlay on 2D: acquisition dimension
 * @param gr a graphics object
 * @param gp a SpinWorks global parameter object
 * @param q status data parameters, acquisition dimension
 * @param q_ind status data parameters, indirect dimension
 * @param n index of the row to be displayed
 * @param yoffset y offset of the trace in pixels
 * @param scale vertical scaling of the 1D trace
 */
	private static void drawRowOn2D(GC gr, global_par gp, data_par q, data_par q_ind,
			int n, int yoffset, double scale)
	{ 
		int point1, point2;					// start and stop points in data array 
		int x, y, oldx, oldy;
		double a;							// for phasing 
		int[] xvalues = new int[MAX_DISP_BUFF];
		int[] yvalues = new int[MAX_DISP_BUFF];
		int npoints = 0;					//	initialize point counter 

		String freq_label;

		if(gp.slice_no[n] < 0 || gp.slice_no[n] >= q_ind.SI) return;	// outside array!

		double hzperpix = (gp.freq1-gp.freq2)/(double)(gp.xsize_2D-1);
		point1 = (int) Math.round((q.offset - gp.freq1) / q.hzperpt);
		point2 = (int) Math.round((q.offset - gp.freq2) / q.hzperpt);

		//	make sure array bounds are not exceeded TO DO  necessary??? done already??
		if(point1 < 0) point1 = 0;
		if(point2 >= q.SI) point2 = q.SI-1;

		y = gp.dysize - (int)(gp.rows_real[gp.slice_no[n]][point1] * scale)
				- gp.bottom_2D
				- yoffset;

		if (y > (gp.dysize - gp.bottom_2D - 2))
			y = gp.dysize - gp.bottom_2D - 2;
		if (y < (gp.top_2D + 2))
			y = gp.top_2D + 2;

		oldx = gp.left_2D;
		oldy = y;

		if(gp.axis_unit == SW_main.units.PPM)
			freq_label = String.format("%.2f ppm", 
					(q_ind.offset - ((double)gp.slice_no[n] *q_ind.hzperpt))/q_ind.SF);
		else
			freq_label = String.format("%.2f Hz.",
					q_ind.offset - ((double)gp.slice_no[n] *q_ind.hzperpt));

		gr.setFont(new Font(SW_main.shell.getDisplay(), "Arial", SW_main.SMALL_FONT, SWT.NORMAL));
		gr.setBackground(gp.back_colour);
		gr.setForeground(gp.axis_colour);
		gr.drawText(freq_label,gp.left_2D+5, y-20);

		for(int i = point1; i <= point2; i++)
		{
			x = gp.left_2D
					+ ((int) Math.round((gp.freq1 - q.offset + i * q.hzperpt) / hzperpix));

			if(gp.phase_mode)
			{
				a = 0.0174533 * (gp.ph0 + ((double)i/(double)q.SI) * gp.ph1);
				y = gp.dysize
						- (int)(scale 
								* (gp.rows_real[gp.slice_no[n]][i]*Math.cos(a) 
										- gp.rows_imag[gp.slice_no[n]][i]*Math.sin(a)))
						- gp.bottom_2D
						- yoffset;
			}
			else
				y = gp.dysize 
				- (int)(gp.rows_real[gp.slice_no[n]][i] * scale) 
				- gp.bottom_2D
				- yoffset;

			if (y > (gp.dysize - gp.bottom_2D - 2))
				y = gp.dysize - gp.bottom_2D - 2;
			if (y < (gp.top_2D + 2))
				y = gp.top_2D + 2;

			if(((x != oldx) || (Math.abs(y-oldy) > 4)) &&
					npoints < MAX_DISP_BUFF)
			{
				xvalues[npoints] = x;
				yvalues[npoints] = y;
				npoints++;
				oldx = x;
				oldy = y;
			}
		}

		if(n == gp.active_slice)
			gr.setForeground(gp.spect_colour);
		else
			gr.setForeground(gp.sim_colour);
		
//  	draw Spectrum 
    	int[] pts = new int[2 * npoints];

    	for(int i = 0; i < npoints; i++) 
    	{
    		pts[2 * i] = xvalues[i];
    		pts[2 * i + 1] = yvalues[i];
    	}
    	
    	if(npoints > 2)
    		gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values
	}

//-------------------------------------------------------------------------------------------------
/**
 * Draw 1D overlay on 2D: indirect dimension, rotated (indirect hohizontal)
 * @param gr a graphics object
 * @param gp a SpinWorks global parameter object
 * @param q status data parameters, acquisition dimension
 * @param q_ind status data parameters, indirect dimension
 * @param n index of the row to be displayed
 * @param yoffset y offset of the trace in pixels
 * @param scale vertical scaling of the 1D trace
 */
	private static void drawRowOn2D_rotated(GC gr, global_par gp, data_par q, 
			data_par q_ind, int n, int yoffset, double scale)
	{ 
		int point1, point2;					// start and stop points in data array 
		int x, y, oldx, oldy;
		double a;							// for phasing 
		int[] xvalues = new int[MAX_DISP_BUFF];
		int[] yvalues = new int[MAX_DISP_BUFF];
		int npoints = 0;					//	initialize point counter 

		String freq_label;

		if(gp.slice_no[n] < 0 || gp.slice_no[n] >= q.SI) return;	// outside array!

		double hzperpix = (gp.freq1_indirect-gp.freq2_indirect)/(double)(gp.xsize_2D-1);

		point1 = (int) Math.round((q_ind.offset - gp.freq1_indirect) / q_ind.hzperpt);
		point2 = (int) Math.round((q_ind.offset - gp.freq2_indirect) / q_ind.hzperpt);
		
		//		make sure array bounds are not exceeded 
		if(point1 < 0) point1 = 0;
		if(point2 >= q_ind.SI) point2 = q_ind.SI-1;

		y = gp.dysize - (int)(gp.rows_real[point1][gp.slice_no[n]] * scale)
				- gp.bottom_2D
				- yoffset;

		if (y > (gp.dysize - gp.bottom_2D - 2))
			y = gp.dysize - gp.bottom_2D - 2;
		if (y < (gp.top_2D + 2))
			y = gp.top_2D + 2;

		oldx = gp.left_2D;
		oldy = y;

		if(gp.axis_unit == SW_main.units.PPM)
			freq_label = String.format("%.3f ppm",
					(q.offset - ((double)gp.slice_no[n] * q.hzperpt))/q.SF);
		else
			freq_label = String.format("%.2f Hz.",
					(q.offset - (double)gp.slice_no[n] * q.hzperpt));
		
		gr.setFont(new Font(SW_main.shell.getDisplay(), "Arial", SW_main.SMALL_FONT, SWT.NORMAL));
		gr.setBackground(gp.back_colour);
		gr.setForeground(gp.axis_colour);
		gr.drawText(freq_label, gp.left_2D+5, y-20);

		for(int i=point1; i<=point2; i++)
		{
			x = gp.left_2D
					+ ((int) Math.round((gp.freq1_indirect - q_ind.offset + i * q_ind.hzperpt) 
							/ hzperpix));
			
			if(gp.phase_mode)
			{
				a = 0.0174533 * (gp.ph0 + ((double)i/(double)q_ind.SI) * gp.ph1);
				y = gp.dysize
						- (int)(scale 
								* (gp.rows_real[i][gp.slice_no[n]]*Math.cos(a) 
										- gp.rows_imag[i][gp.slice_no[n]]*Math.sin(a)))
						- gp.bottom_2D
						- yoffset;
			}
			else
				y = gp.dysize 
				- (int)(gp.rows_real[i][gp.slice_no[n]] * scale) 
				- gp.bottom_2D
				- yoffset;

			if (y > (gp.dysize - gp.bottom_2D - 2))
				y = gp.dysize - gp.bottom_2D - 2;
			if (y < (gp.top_2D + 2))
				y = gp.top_2D + 2;

			if(((x != oldx) || (Math.abs(y-oldy) > 4)) &&
					npoints < MAX_DISP_BUFF)
			{
				xvalues[npoints] = x;
				yvalues[npoints] = y;
				npoints++;
				oldx = x;
				oldy = y;
			}
		}

		if(n == gp.active_slice)
			gr.setForeground(gp.spect_colour);
		else
			gr.setForeground(gp.sim_colour);
		
//  	draw Spectrum 
    	int[] pts = new int[2 * npoints];

    	for(int i = 0; i < npoints; i++) 
    	{
    		pts[2 * i] = xvalues[i];
    		pts[2 * i + 1] = yvalues[i];
    	}
    	
    	if(npoints > 2)
    		gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values
	}
			
//-------------------------------------------------------------------------------------------------
 /**
 * Draw 1D overlay on 2D: indirect dimension
 * @param gr a graphics object
 * @param gp a SpinWorks global parameter object
 * @param q status data parameters, acquisition dimension
 * @param q_ind status data parameters, indirect dimension
 * @param n index of the row to be displayed
 * @param yoffset y offset of the trace from y axis in pixels
 * @param scale vertical scaling of the 1D trace
 */
	private static void drawColumnOn2D(GC gr, global_par gp, data_par q, data_par q_ind,
			int n, int yoffset, double scale)
	{ 
		int point1, point2;					// start and stop points in data array 
		double a;							// for phasing 

		int[] xvalues = new int[MAX_DISP_BUFF];
		int[] yvalues = new int[MAX_DISP_BUFF];
		int npoints = 0;

		String freq_label;

		if(gp.slice_no[n] < 0 || gp.slice_no[n] >= q.SI) return;	// outside array!

		double hzperpix_f1 = (gp.freq1_indirect-gp.freq2_indirect)/(double)(gp.ysize_2D-1);

		point1 = (int) Math.round((q_ind.offset - gp.freq1_indirect) / q_ind.hzperpt);
		point2 = (int) Math.round((q_ind.offset - gp.freq2_indirect) / q_ind.hzperpt);

		//		make sure array bounds are not exceeded 
		if(point1 < 0) point1 = 0;
		if(point2 >= q_ind.SI) point2 = q_ind.SI-1;

		yvalues[0] = (int) (gp.rows_real[point1][gp.slice_no[n]] * scale)
				+ gp.left_2D
				+ yoffset;

		if(gp.axis_unit == SW_main.units.PPM)
			freq_label = String.format("%.3f ppm",
					(q.offset - ((double)gp.slice_no[n] * q.hzperpt))/q.SF);
		else
			freq_label = String.format("%.3f Hz.",
					(q.offset - (double)gp.slice_no[n] * q.hzperpt));

		gr.setFont(new Font(SW_main.shell.getDisplay(), "Arial", SW_main.SMALL_FONT, SWT.NORMAL));
		gr.setForeground(gp.axis_colour);
		gr.setBackground(gp.back_colour);
		gr.drawText(freq_label, yvalues[0], gp.top_2D + 5);

		for(int i=point1; i<=point2; i++)
		{ 
			xvalues[npoints] = gp.dysize
					- ((int) Math.round((gp.freq1_indirect
							- q_ind.offset + i * q_ind.hzperpt) / hzperpix_f1))
					- gp.bottom_2D;

			if(gp.phase_mode)
			{
				a = 0.0174533 * (gp.ph0 + ((double)i/(double)q_ind.SI) * gp.ph1);
				yvalues[npoints] = (int)(scale 
						* (gp.rows_real[i][gp.slice_no[n]] * Math.cos(a) 
								- gp.irows_real[i][gp.slice_no[n]] * Math.sin(a)))
						+ gp.left_2D
						+ yoffset;
			}
			else
			{
				yvalues[npoints] = (int)(gp.rows_real[i][gp.slice_no[n]] * scale) 
						+ gp.left_2D
						+ yoffset;
			}

			if (yvalues[npoints] > (/*gp.dxsize - gp.right_2D*/gp.xsize_2D + gp.left_2D - 2))
				yvalues[npoints] = /*gp.dxsize - gp.right_2D*/gp.xsize_2D + gp.left_2D - 2;
			if (yvalues[npoints] < (gp.left_2D + 2))
				yvalues[npoints] = gp.left_2D + 2;
			npoints++;
		}

		if(n == gp.active_slice)
			gr.setForeground(gp.spect_colour);
		else
			gr.setForeground(gp.sim_colour);

		//	      	draw Spectrum 
		int[] pts = new int[2 * npoints];

		for(int i = 0; i < npoints; i++) 
		{
			pts[2 * i] = yvalues[i];		// note coordinate rotation
			pts[2 * i + 1] = xvalues[i];
		}

		if(npoints > 2)
			gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values
	}

//-------------------------------------------------------------------------------------------------
/**
 * Draw 1D overlay on 2D: axquisition dimension, rotated
 * @param gr a graphics object
 * @param gp a SpinWorks global parameter object
 * @param q status data parameters, acquisition dimension
 * @param q_ind status data parameters, indirect dimension
 * @param n index of the row to be displayed
 * @param yoffset y offset of the trace from axis in pixels
 * @param scale vertical scaling of the 1D trace
 */
	private static void drawColumnOn2D_rotated(GC gr, global_par gp, 
			data_par q, data_par q_ind, int n, int yoffset, double scale)
	{ 
		int point1, point2;					// start and stop points in data array 
		double a;							// for phasing 

		int[] xvalues = new int[MAX_DISP_BUFF];
		int[] yvalues = new int[MAX_DISP_BUFF];
		int npoints = 0;

		String freq_label;

		if(gp.slice_no[n] < 0 || gp.slice_no[n] >= q_ind.SI) return;	// outside array!

		double hzperpix = (gp.freq1-gp.freq2)/(double)(gp.ysize_2D-1);
		
		point1 = (int) Math.round((q.offset - gp.freq1) / q.hzperpt);
		point2 = (int) Math.round((q.offset - gp.freq2) / q.hzperpt);
		
		//		make sure array bounds are not exceeded 
		if(point1 < 0) point1 = 0;
		if(point2 >= q.SI) point2 = q.SI-1;

		yvalues[0] = (int)(gp.rows_real[gp.slice_no[n]][point1] * scale)
				+ gp.left_2D
				+ yoffset;

		if(gp.axis_unit == SW_main.units.PPM)
			freq_label = String.format("%.3f ppm",
					(q_ind.offset - ((double)gp.slice_no[n] * q_ind.hzperpt))/q_ind.SF);
		else
			freq_label = String.format("%.2f Hz.",
					(q_ind.offset - (double)gp.slice_no[n] * q_ind.hzperpt));
		
		gr.setFont(new Font(SW_main.shell.getDisplay(), "Arial", SW_main.SMALL_FONT, SWT.NORMAL));
		gr.setForeground(gp.axis_colour);
		gr.setBackground(gp.back_colour);
		gr.drawText(freq_label, yvalues[0], gp.top_2D + 5);

		for(int i=point1; i<=point2; i++)
		{
			xvalues[npoints] = gp.dysize
					- ((int) Math.round((gp.freq1 - q.offset + i * q.hzperpt) / hzperpix))
					- gp.bottom_2D;
			

			if(gp.phase_mode)
			{
				a = 0.0174533 * (gp.ph0 + ((double)i/(double)q_ind.SI) * gp.ph1);
				yvalues[npoints] = (int)(scale 
						* (gp.rows_real[gp.slice_no[n]][i]*Math.cos(a) 
								- gp.irows_real[gp.slice_no[n]][i]*Math.sin(a)))
						+ gp.left_2D
						+ yoffset;
			}
			else
			{
				yvalues[npoints] = (int)(gp.rows_real[gp.slice_no[n]][i] * scale) 
						+ gp.left_2D
						+ yoffset;
			}

			if (yvalues[npoints] > (/*gp.dxsize - gp.right_2D*/gp.xsize_2D + gp.left_2D - 2))
				yvalues[npoints] = /*gp.dxsize - gp.right_2D*/gp.xsize_2D + gp.left_2D - 2;
			if (yvalues[npoints] < (gp.left_2D + 2))
				yvalues[npoints] = gp.left_2D + 2;
			npoints++;
		}

		if(n == gp.active_slice)
			gr.setForeground(gp.spect_colour);
		else
			gr.setForeground(gp.sim_colour);
		
//  draw Spectrum 
	int[] pts = new int[2 * npoints];

	for(int i = 0; i < npoints; i++) 
	{
		pts[2 * i] = yvalues[i];		// note coordinate rotation
		pts[2 * i + 1] = xvalues[i];
	}

	if(npoints > 2)
		gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values
	}


//-------------------------------------------------------------------------------------------------
/**
 * Draw 1D external, projection, etc. on 2D: aquisition dimension
 * @param gr graphics context
 * @param yoffset in pixels
 * @param gp SpinWorks global parameter object
 */
	static void drawF2On2D(GC gr,  int yoffset, global_par gp)
	{ 
		int point1, point2;					// start and stop points in data array 
		int x, y, oldx, oldy;							
		int[] xvalues = new int[MAX_DISP_BUFF];
		int[] yvalues = new int[MAX_DISP_BUFF];
		int npoints = 0;					//	initialize point counter 

		double hzperpix = (gp.freq1 - gp.freq2)/(double)(gp.xsize_2D-1);

		point1 = (int) Math.round((gp.offset_f2_trace - gp.freq1) 
				/ gp.hzperpt_f2_trace);
		point2 = (int) Math.round((gp.offset_f2_trace - gp.freq2) 
				/ gp.hzperpt_f2_trace);

		//		make sure array bounds are not exceeded 
		if(point1 < 0) point1 = 0;
		if(point2 >= gp.size_f2_trace) point2 = gp.size_f2_trace;

		y = gp.dysize - (int)(gp.f2_trace[point1] * gp.f2_trace_scale)
				- gp.bottom_2D
				- yoffset;

		if (y > (gp.ysize_2D - gp.bottom_2D - 2))
			y = gp.ysize_2D - gp.bottom_2D - 2;
		if (y < 24)
			y = 24;

		oldx = gp.left_2D;
		oldy = y;

		for(int i=point1; i < point2; i++)
		{
			x = gp.left_2D
					+ ((int) Math.round((gp.freq1 - gp.offset_f2_trace
							+ i * gp.hzperpt_f2_trace) / hzperpix));

			y = gp.dysize - (int)(gp.f2_trace[i] * gp.f2_trace_scale) 
					- gp.bottom_2D
					- yoffset;

			if (y > (gp.top_2D - 2))
				y = gp.top_2D - 2;
			if (y < 24)
				y = 24;

			if(((x != oldx) || (Math.abs(y-oldy) > 4)) &&
					npoints < MAX_DISP_BUFF)
			{
				xvalues[npoints] = x;
				yvalues[npoints] = y;
				npoints++;
				oldx = x;
				oldy = y;
			}
		}

//  	draw Spectrum 
		gr.setForeground(gp.spect_colour);	
    	int[] pts = new int[2 * npoints];

    	for(int i = 0; i < npoints; i++) 
    	{
    		pts[2 * i] = xvalues[i];
    		pts[2 * i + 1] = yvalues[i];
    	}
    	
    	if(npoints > 2)
    		gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values
	}
	
//-------------------------------------------------------------------------------------------------
/**
 *  draw 1D external, projection etc.  on rotated 2D display: aquisition dimension
 * @param gr graphics context
 * @param yoffset in pixels
 * @param gp SpinWorks global parameter object
 */
	static void drawF2On2D_rotated(GC gr, int yoffset, global_par gp)
	{ 
		int point1, point2;					// start and stop points in data array 
		int x, y, oldx, oldy;							
		int[] xvalues = new int[MAX_DISP_BUFF];
		int[] yvalues = new int[MAX_DISP_BUFF];
		int npoints = 0;					//	initialize point counter 

		double hzperpix = (gp.freq1 - gp.freq2)/(double)(gp.ysize_2D-1);

		point1 = (int) Math.round((gp.offset_f2_trace - gp.freq1) 
				/ gp.hzperpt_f2_trace);
		point2 = (int) Math.round((gp.offset_f2_trace - gp.freq2)
				/ gp.hzperpt_f2_trace);


		//		make sure array bounds are not exceeded 
		if(point1 < 0) point1 = 0;
		if(point2 >= gp.size_f2_trace) point2 = gp.size_f2_trace;

		y = (int)(gp.f2_trace[point1] * gp.f2_trace_scale)
				+ gp.left_2D
				+ yoffset;

		x = gp.dysize - gp.bottom_2D;

		if (y > (gp.dxsize - 100))
			y = gp.dxsize - 100;
		if (y < gp.xsize_2D + gp.left_2D)
			y = gp.xsize_2D + gp.left_2D;

		oldx = x;
		oldy = y;


		for(int i=point1; i < point2; i++)
		{
			x = gp.dysize
					- (int) Math.round((gp.freq1 - gp.offset_f2_trace
							+ i * gp.hzperpt_f2_trace) / hzperpix)
					- gp.bottom_2D;

			y = (int)(gp.f2_trace[i] * gp.f2_trace_scale) 
					+ gp.left_2D
					+ yoffset;

			if (y > (gp.dxsize - 100))
				y = gp.dxsize - 100;
			if (y < gp.xsize_2D + gp.left_2D + 2)
				y = gp.xsize_2D + gp.left_2D + 2;

			if(((x != oldx) || (Math.abs(y-oldy) > 4)) &&
					npoints < MAX_DISP_BUFF)
			{
				xvalues[npoints] = x;
				yvalues[npoints] = y;
				npoints++;
				oldx = x;
				oldy = y;
			}
		}
		
//  	draw Spectrum 
		gr.setForeground(gp.spect_colour);	
    	int[] pts = new int[2 * npoints];

    	for(int i = 0; i < npoints; i++) 
    	{
    		pts[2 * i] = yvalues[i];
    		pts[2 * i + 1] = xvalues[i];
    	}
    	
    	if(npoints > 2)
    		gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values
	}
						
//-------------------------------------------------------------------------------------------------
/**
 * draw 1D  external, projection etc. on 2D: indirect dimensikon
 * @param gr graphics context
 * @param yoffset in pixels
 * @param gp a SpinWorks global parameter object
 */
	static void drawF1On2D(GC gr, int yoffset, global_par gp)
	{ 
		int point1, point2;					// start and stop points in data array 
		int x, y, oldx, oldy;						
		int[] xvalues = new int[MAX_DISP_BUFF];
		int[] yvalues = new int[MAX_DISP_BUFF];
		int npoints = 0;					//	initialize point counter 

		double hzperpix = (gp.freq1_indirect - gp.freq2_indirect)/(double)(gp.ysize_2D-1);

		point1 = (int) Math.round((gp.offset_f1_trace - gp.freq1_indirect) 
				/ gp.hzperpt_f1_trace);
		point2 = (int) Math.round((gp.offset_f1_trace - gp.freq2_indirect) 
				/ gp.hzperpt_f1_trace);
		

		//		make sure array bounds are not exceeded 
		if(point1 < 0) point1 = 0;
		if(point2 >= gp.size_f1_trace) point2 = gp.size_f1_trace;

		y = (int)(gp.f1_trace[point1] * gp.f1_trace_scale)
				+ gp.left_2D
				+ yoffset;

		x = gp.dysize - gp.bottom_2D;

		if (y > (gp.dxsize - 100))
			y = gp.dxsize - 100;
		if (y < gp.xsize_2D + gp.left_2D)
			y = gp.xsize_2D + gp.left_2D;

		oldx = x;
		oldy = y;

		for(int i=point1; i < point2; i++)
		{
			x = gp.dysize
					- (int) Math.round((gp.freq1_indirect - gp.offset_f1_trace
							+ i * gp.hzperpt_f1_trace) / hzperpix)
					- gp.bottom_2D;
			
			y = (int)(gp.f1_trace[i] * gp.f1_trace_scale) 
					+ gp.left_2D
					+ yoffset;

			if (y > (gp.dxsize - 100))
				y = gp.dxsize - 100;
			if (y < gp.xsize_2D + gp.left_2D + 2)
				y = gp.xsize_2D + gp.left_2D + 2;

			if(((x != oldx) || (Math.abs(y-oldy) > 4)) &&
					npoints < MAX_DISP_BUFF)
			{
				xvalues[npoints] = x;
				yvalues[npoints] = y;
				npoints++;
				oldx = x;
				oldy = y;
			}
		}

	//	draw Spectrum 
	int[] pts = new int[2 * npoints];
	gr.setForeground(gp.spect_colour);

	for(int i = 0; i < npoints; i++) 
	{
		pts[2 * i] = yvalues[i];		// note coordinate rotation
		pts[2 * i + 1] = xvalues[i];
	}

	if(npoints > 2)
		gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values
	}
	
	
//-------------------------------------------------------------------------------------------------
/**
 * Draw 1D external, projection, etc. on 2D: indirect dimension, rotated
 * @param gr graphics context
 * @param yoffset in pixels
 * @param gp SpinWorks global parameter object
 */
	static void drawF1On2D_rotated(GC gr, int yoffset, global_par gp)
	{ 
		int point1, point2;					// start and stop points in data array 
		int x, y, oldx, oldy;							
		int[] xvalues = new int[MAX_DISP_BUFF];
		int[] yvalues = new int[MAX_DISP_BUFF];
		int npoints = 0;					//	initialize point counter 

		double hzperpix = (gp.freq1_indirect-gp.freq2_indirect)/(double)(gp.xsize_2D-1);

		point1 = (int) Math.round((gp.offset_f1_trace - gp.freq1_indirect) 
				/ gp.hzperpt_f1_trace);
		point2 = (int) Math.round((gp.offset_f1_trace - gp.freq2_indirect) 
				/ gp.hzperpt_f1_trace);

		//	make sure array bounds are not exceeded 
		if(point1 < 0) point1 = 0;
		if(point2 >= gp.size_f1_trace) point2 = gp.size_f1_trace;

		y = gp.dysize - (int)(gp.f1_trace[point1] * gp.f1_trace_scale)
				- gp.bottom_2D
				- yoffset;

		if (y > (gp.ysize_2D - gp.bottom_2D - 2))
			y = gp.ysize_2D - gp.bottom_2D - 2;
		if (y < 24)
			y = 24;

		oldx = gp.left_2D;
		oldy = y;


		for(int i=point1; i < point2; i++)
		{
			x = gp.left_2D + ((int) Math.round((gp.freq1_indirect
					- gp.offset_f1_trace + i * gp.hzperpt_f1_trace) / hzperpix));

			y = gp.dysize - (int)(gp.f1_trace[i] * gp.f1_trace_scale) 
					- gp.bottom_2D
					- yoffset;

			if (y > (gp.top_2D - 2))
				y = gp.top_2D - 2;
			if (y < 24)
				y = 24;

			if(((x != oldx) || (Math.abs(y-oldy) > 2)) &&
					npoints < MAX_DISP_BUFF)
			{
				xvalues[npoints] = x;
				yvalues[npoints] = y;
				npoints++;
				oldx = x;
				oldy = y;
			}
		} 

		//	draw Spectrum 
		int[] pts = new int[2 * npoints];
		gr.setForeground(gp.spect_colour);

		for(int i = 0; i < npoints; i++) 
		{
			pts[2 * i] = xvalues[i];
			pts[2 * i + 1] = yvalues[i];
		}

		if(npoints > 2)
			gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values			
	}	
	
//-------------------------------------------------------------------------------------------------	
/**
 * Draw any 2D peak assignments
 * @param gr the current graphice object
 * @param gp the global parameter block
 * @param q status acquisition parameters, acquisition dimension
 * @param q_ind status acquisition parameters current indirect dimension
 */
	private static void draw2D_Peaks(GC gr, global_par gp, data_par q, data_par q_ind)
    {
        int x, y;
        
        gr.setForeground(gp.axis_colour);
        gr.setFont(new Font(SW_main.shell.getDisplay(), 
        		"Arial", SW_main.MEDIUM_FONT, SWT.NORMAL));	// set font

        for (int i = 0; i < gp.n_peaks; i++)
        {
            if (gp.peaks[i].freq > -1.0e9)
            {
              
                x =  ((int)Math.round((gp.freq1 - gp.peaks[i].freq) / gp.hzperpix) + gp.left_2D);
                y =  (gp.dysize - gp.bottom_2D
                    - (int)Math.round((gp.freq1_indirect - gp.peaks[i].freq_f1) / gp.hzperpix_f1));
               
                
                if (x > gp.left_2D && x < gp.dxsize - gp.left_2D
                    && y > gp.top_2D && y < gp.dysize - gp.bottom_2D)
                {
                    gr.drawLine(x - 10, y, x + 10, y);
                    gr.drawLine(x, y - 10, x, y + 10);
                    gr.drawText(gp.peaks[i].label, (x-10), (y + 10));
                    if (i == gp.active_peak)
                        gr.drawRectangle(x - 10, y - 10, 20, 20);
                }
            }
        }
    }
	
//-------------------------------------------------------------------------------------------------	
/**
 * Draw any 2D peak assignments
 * @param gr the current graphics object
 * @param gp the global parameter block
 * @param q status acquisition parameters, acquisition dimension
 * @param q_ind status acquisition parameters current indirect dimension
 */
	private static void draw2D_Assignments(GC gr, global_par gp, data_par q, data_par q_ind)
    {
		int x, y, x_end, y_end;
				
		for (int i = 0; i < gp.n_peaks; i++)
        {
            if (gp.peaks[i].freq > -1.0e9)
            {
            	gr.setForeground(gp.cosy_colour);
            	for(int j = 0; j < gp.peaks[i].n_cosy && gp.showCOSYcorr; j++)
            	{
            		x =  ((int)Math.round((gp.freq1 - gp.peaks[i].freq) 
            				/ gp.hzperpix) + gp.left_2D);
            		y =  (gp.dysize - gp.bottom_2D
            				- (int)Math.round((gp.freq1_indirect - gp.peaks[i].freq_f1) 
            				      / gp.hzperpix_f1));          		

            		if(x < gp.left_2D) x = gp.left_2D + 2;
            		if(x > gp.dxsize - gp.right_2D) x = gp.dxsize - gp.right_2D - 2;
            		if(y < gp.top_2D) y = gp.top_2D + 2;
            		if(y > gp.dysize - gp.bottom_2D) y = gp.dysize - gp.bottom_2D - 2;         		

            		if (gp.peaks[i].cosy_f1[j] > 0.0)	// has an associated F1 correlation
            		{
            			gr.setLineStyle(SWT.LINE_SOLID);
            			x_end = ((int)Math.round((gp.freq1 - (gp.peaks[i].cosy[j] * q.SF)) 
          				      / gp.hzperpix) + gp.left_2D);           		
            			y_end = (gp.dysize - gp.bottom_2D
          				- (int)Math.round((gp.freq1_indirect 
          						- (gp.peaks[i].cosy_f1[j]) * q_ind.SF) / gp.hzperpix_f1));           			            			            		    			
            		}
            		else		// doesn't have an associated F1 correlation
            		{
            			gr.setLineStyle(SWT.LINE_DASH);
            			y = y - 2;
            			x_end = ((int)Math.round((gp.freq1 - (gp.peaks[i].cosy[j] * q.SF)) 
            				      / gp.hzperpix) + gp.left_2D);               		
                		y_end = 0;              		               		            			
            		}
            		
            		if(x_end < gp.left_2D) x_end = gp.left_2D + 2;
            		if(x_end > gp.dxsize - gp.right_2D) x_end = gp.dxsize - gp.right_2D - 2;
            		if(y_end < gp.top_2D) y_end = gp.top_2D + 2;
            		if(y_end > gp.dysize - gp.bottom_2D) y_end = gp.dysize - gp.bottom_2D - 2; 
            		
            		gr.drawLine(x, y, x_end, y);
        			gr.drawLine(x_end, y, x_end, y_end);  
            	}	// for(int j...
            	
            	gr.setForeground(gp.tocsy_colour);
            	for(int j = 0; j < gp.peaks[i].n_tocsy && gp.showTOCSYcorr; j++)
            	{
            		x =  ((int)Math.round((gp.freq1 - gp.peaks[i].freq) 
            				/ gp.hzperpix) + gp.left_2D);
            		y =  (gp.dysize - gp.bottom_2D
            				- (int)Math.round((gp.freq1_indirect - gp.peaks[i].freq_f1) 
            				      / gp.hzperpix_f1));          		

            		if(x < gp.left_2D) x = gp.left_2D + 2;
            		if(x > gp.dxsize - gp.right_2D) x = gp.dxsize - gp.right_2D - 2;
            		if(y < gp.top_2D) y = gp.top_2D + 2;
            		if(y > gp.dysize - gp.bottom_2D) y = gp.dysize - gp.bottom_2D - 2;         		

            		if (gp.peaks[i].tocsy_f1[j] > 0.0)	// has an associated F1 correlation
            		{
            			gr.setLineStyle(SWT.LINE_SOLID);
            			x_end = ((int)Math.round((gp.freq1 - (gp.peaks[i].tocsy[j] * q.SF)) 
          				      / gp.hzperpix) + gp.left_2D);           		
            			y_end = (gp.dysize - gp.bottom_2D
          				- (int)Math.round((gp.freq1_indirect 
          						- (gp.peaks[i].tocsy_f1[j]) * q_ind.SF) / gp.hzperpix_f1));           			            			            		    			
            		}
            		else		// doesn't have an associated F1 correlation
            		{
            			gr.setLineStyle(SWT.LINE_DASH);
            			y = y - 2;
            			x_end = ((int)Math.round((gp.freq1 - (gp.peaks[i].tocsy[j] * q.SF)) 
            				      / gp.hzperpix) + gp.left_2D);               		
                		y_end = 0;              		               		            			
            		}
            		
            		if(x_end < gp.left_2D) x_end = gp.left_2D + 2;
            		if(x_end > gp.dxsize - gp.right_2D) x_end = gp.dxsize - gp.right_2D - 2;
            		if(y_end < gp.top_2D) y_end = gp.top_2D + 2;
            		if(y_end > gp.dysize - gp.bottom_2D) y_end = gp.dysize - gp.bottom_2D - 2; 
            		
            		gr.drawLine(x, y, x_end, y);
        			gr.drawLine(x_end, y, x_end, y_end);  
            	}	// for(int j...
            	
            	gr.setForeground(gp.noesy_colour);
            	for(int j = 0; j < gp.peaks[i].n_noesy && gp.showNOESYcorr; j++)
            	{
            		x =  ((int)Math.round((gp.freq1 - gp.peaks[i].freq) 
            				/ gp.hzperpix) + gp.left_2D);
            		y =  (gp.dysize - gp.bottom_2D
            				- (int)Math.round((gp.freq1_indirect - gp.peaks[i].freq_f1) 
            				      / gp.hzperpix_f1));          		

            		if(x < gp.left_2D) x = gp.left_2D + 2;
            		if(x > gp.dxsize - gp.right_2D) x = gp.dxsize - gp.right_2D - 2;
            		if(y < gp.top_2D) y = gp.top_2D + 2;
            		if(y > gp.dysize - gp.bottom_2D) y = gp.dysize - gp.bottom_2D - 2;         		

            		if (gp.peaks[i].noesy_f1[j] > 0.0)	// has an associated F1 correlation
            		{
            			gr.setLineStyle(SWT.LINE_SOLID);
            			x_end = ((int)Math.round((gp.freq1 - (gp.peaks[i].noesy[j] * q.SF)) 
          				      / gp.hzperpix) + gp.left_2D);           		
            			y_end = (gp.dysize - gp.bottom_2D
          				- (int)Math.round((gp.freq1_indirect 
          						- (gp.peaks[i].noesy_f1[j]) * q_ind.SF) / gp.hzperpix_f1));           			            			            		    			
            		}
            		else		// doesn't have an associated F1 correlation
            		{
            			gr.setLineStyle(SWT.LINE_DASH);
            			y = y - 2;
            			x_end = ((int)Math.round((gp.freq1 - (gp.peaks[i].noesy[j] * q.SF)) 
            				      / gp.hzperpix) + gp.left_2D);               		
                		y_end = 0;              		               		            			
            		}
            		
            		if(x_end < gp.left_2D) x_end = gp.left_2D + 2;
            		if(x_end > gp.dxsize - gp.right_2D) x_end = gp.dxsize - gp.right_2D - 2;
            		if(y_end < gp.top_2D) y_end = gp.top_2D + 2;
            		if(y_end > gp.dysize - gp.bottom_2D) y_end = gp.dysize - gp.bottom_2D - 2; 
            		
            		gr.drawLine(x, y, x_end, y);
        			gr.drawLine(x_end, y, x_end, y_end);  
            	}	// for(int j...
            	
            	gr.setForeground(gp.hmbc_colour);
            	for(int j = 0; j < gp.peaks[i].n_hmbc && gp.showHMBCcorr; j++)
            	{
            		x =  ((int)Math.round((gp.freq1 - gp.peaks[i].freq) 
            				/ gp.hzperpix) + gp.left_2D);
            		y =  (gp.dysize - gp.bottom_2D
            				- (int)Math.round((gp.freq1_indirect - gp.peaks[i].freq_f1) 
            				      / gp.hzperpix_f1));          		

            		if(x < gp.left_2D) x = gp.left_2D + 2;
            		if(x > gp.dxsize - gp.right_2D) x = gp.dxsize - gp.right_2D - 2;
            		if(y < gp.top_2D) y = gp.top_2D + 2;
            		if(y > gp.dysize - gp.bottom_2D) y = gp.dysize - gp.bottom_2D - 2;         		

            		if (gp.peaks[i].hmbc[j] > 0.0)	// has an associated F2 correlation
            		{
            			gr.setLineStyle(SWT.LINE_SOLID);
            			x_end = ((int)Math.round((gp.freq1 - (gp.peaks[i].hmbc[j] * q.SF)) 
          				      / gp.hzperpix) + gp.left_2D);           		
            			y_end = (gp.dysize - gp.bottom_2D
          				- (int)Math.round((gp.freq1_indirect 
          						- (gp.peaks[i].hmbc_f1[j]) * q_ind.SF) / gp.hzperpix_f1));           			            			            		    			
            		}
            		else		// doesn't have an associated F2 correlation
            		{
            			gr.setLineStyle(SWT.LINE_DASH);
            			y = y - 2;
            			y_end = (gp.dysize - gp.bottom_2D
                  				- (int)Math.round((gp.freq1_indirect 
                  						- (gp.peaks[i].hmbc_f1[j]) * q_ind.SF) / gp.hzperpix_f1));              		
                		x_end = gp.dxsize;              		               		            			
            		}
            		
            		if(x_end < gp.left_2D) x_end = gp.left_2D + 2;
            		if(x_end > gp.dxsize - gp.right_2D) x_end = gp.dxsize - gp.right_2D - 2;
            		if(y_end < gp.top_2D) y_end = gp.top_2D + 2;
            		if(y_end > gp.dysize - gp.bottom_2D) y_end = gp.dysize - gp.bottom_2D - 2; 
            		
            		gr.drawLine(x, y, x, y_end);
        			gr.drawLine(x, y_end, x_end, y_end);  
            	}	// for(int j...
            	
            	
            }
        }	// for (int i...
    }
//-------------------------------------------------------------------------------------------------
}	// end class

