package spinworks;

import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import spinworks.SW_main.modes;
import spinworks.SW_main.modes_3D;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseWheelListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Canvas;

/**
 * Dialog for inset boxes
 * @author marat
 */
public class InsetBox extends Dialog 
{
	protected Object result;
	protected Shell shell;
	
	private global_par gp;			// local global parameter structure
	private data_par q;				// link (pointer) to data parameters	
	private data_par q_ind;			// data parameters in the current indirect dimension
	private  int n;					// the box number of this box
	private boolean update_spect;
	public  IntegInfo[] integrals;	// link (pointer)to integrals

	static Image image;
	static GC gcImage;
	    
	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public InsetBox(Shell parent, int style, int boxNumber) 	// constructor
	{
		super(parent, style);
		setText("Inset Box");
		
		n = boxNumber;
		
//		somewhat wasteful, but create a separate global parameter structure for each
//		inset box
		try 
		{
			gp = (global_par) SW_main.gp.clone();
		} 
		catch (CloneNotSupportedException e1) {	 e1.printStackTrace(); 	}
		
		try 
		{
			q = (data_par) SW_main.q.clone();
		} 
		catch (CloneNotSupportedException e1) {	 e1.printStackTrace(); 	}
		
		try	// TODO handle indirect dimension properly for 3D
		{
			if(q.data_mode == SW_main.modes.MODE_3D && SW_main.gp.mode_3D == modes_3D.XY )
				q_ind = (data_par) SW_main.q_f2.clone();	// 3D in XY mode
			else
				q_ind = (data_par) SW_main.q_f1.clone();	// 1D, 2D, or 3D in XZ mode
		}		
		catch (CloneNotSupportedException e) {	e.printStackTrace();	}
		
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() 
	{
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) 
		{
			if (!display.readAndDispatch()) 
			{
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() 
	{
		shell = new Shell(getParent(), SWT.DIALOG_TRIM | SWT.RESIZE);
		shell.setSize(632, 555);
		shell.setText(getText());
		
		final Canvas canvas = new Canvas(shell, SWT.BORDER);
		canvas.setBounds(0, 0, 616, 481);		
		
		gp.freq1 = SW_main.gp.new_freq1;
		gp.freq2 = SW_main.gp.new_freq2;
		Util.check_order(gp);
		gp.hzperpix = (gp.freq1 - gp.freq2) / (double)(gp.dxsize - 1);
		
//		store some values in print_inset_box to save for plotting
		SW_main.gp.printInsetBox[n].yscale = gp.expt_scale;
		SW_main.gp.printInsetBox[n].calc_yscale = gp.calc_scale;
		SW_main.gp.printInsetBox[n].freq1 = gp.new_freq1;
		SW_main.gp.printInsetBox[n].freq2 = gp.new_freq2;
		SW_main.gp.printInsetBox[n].freq1_indirect = gp.new_freq1_indirect;
		SW_main.gp.printInsetBox[n].freq1_indirect = gp.new_freq1_indirect;
				
		// resize listener
        shell.addListener (SWT.Resize,  new Listener () 
        {   
        	/**
        	 * @param arg0
        	 */
        	@Override
        	public void handleEvent(Event arg0)
        	{ 
        		Rectangle rect = shell.getClientArea();     		
        		gp.dxsize = rect.width; gp.dysize = rect.height;
        		gp.hzperpix = (gp.freq1 - gp.freq2) / (double)(gp.dxsize - 1);
        		
        		canvas.setBounds(0, 0, gp.dxsize, gp.dysize);
    			
    			SW_main.gp.printInsetBox[n].calc_sizes(shell.getBounds().x, shell.getBounds().y,
    					shell.getBounds().width, shell.getBounds().height);   	
    			
    			/*System.out.println(String.format("Resized: window id = %d  window location  =  %d  %d", 
    					n, SW_main.windowLocation.x, SW_main.windowLocation.y));
    			
    			System.out.println(String.format("   page x = %f  page y = %f  width = %f  height = %f", 
    					SW_main.gp.printInsetBox[n].page_x, SW_main.gp.printInsetBox[n].page_y,
    					SW_main.gp.printInsetBox[n].page_width, SW_main.gp.printInsetBox[n].page_height));*/
    			
        		gp.new_2D_view = true;
        		update_spect = true;
        		canvas.redraw();  		// triggers paint listener
        	}
        });
              
        shell.addListener (SWT.Move,  new Listener () 
        {   
        	/**
        	 * @param arg0
        	 */
        	@Override
        	public void handleEvent(Event arg0)
        	{    			
    			SW_main.gp.printInsetBox[n].calc_sizes(shell.getBounds().x, shell.getBounds().y,
    					shell.getBounds().width, shell.getBounds().height);   	
    			
    			/*System.out.println(String.format("  Moved: Window id = %d  window location  =  %d  %d", 
    					n, SW_main.windowLocation.x, SW_main.windowLocation.y));
    			
    			System.out.println(String.format("   page x = %f  page y = %f  width = %f  height = %f", 
    					SW_main.gp.printInsetBox[n].page_x, SW_main.gp.printInsetBox[n].page_y,
    					SW_main.gp.printInsetBox[n].page_width, SW_main.gp.printInsetBox[n].page_height));*/
        	}
        });      
        
        // this box has been closed, remove from list of print bozes
        shell.addListener (SWT.Close,  new Listener () 
        {   
        	/**
        	 * @param arg0
        	 */
        	@Override
        	public void handleEvent(Event arg0)
        	{  
        		SW_main.gp.printInsetBox[n].active = false;
        	}
        });      
        
        // paint listener
        canvas.addPaintListener(new PaintListener()
        { 
            public void paintControl(PaintEvent e)
            {                   
                if(update_spect)
                {  
                	//	Fix for MacOS Graphics bug! //////////////
                	//	necessary to dispose and re-create spectral bitmap every time spectrum is updated.
                	if(image != null) image.dispose();
            		if(gcImage != null)gcImage.dispose();
            		image = new Image(shell.getDisplay(), canvas.getBounds());
            		gcImage = new GC(image);
            		///////////////////
            		
                	// drawing routines here
            		
            		gcImage.setBackground(gp.back_colour);
            		gcImage.fillRectangle(0, 0, gp.dxsize, gp.dysize);
                	
            		if(gp.display_mode == modes.MODE_1D)
            		{
            			GraphUtil.draw_spect(gcImage, gp.freq1, gp.freq2,
            					gp.expt_scale, 0, gp.expt_y_offset,
            					gp.dxsize, gp.dysize,
            					gp.data_array, 2,
            					gp.spect_colour, gp, q);
            			
            			//draw integrals if desired
            			if ( gp.show_integrals && gp.n_integrals > 0)
            				GraphUtil.draw_integ(gcImage, gp.freq1, gp.freq2, gp.integ_scale, 120, gp, q);

            			GraphUtil.draw_axis(gcImage, gp.freq1, gp.freq2, 0, gp.axis_offset, gp.dxsize, 
            					gp, q, q_ind);
            			
            		}
            		else if(gp.display_mode == modes.MODE_2D)
            		{
            			// TODO 2D display for inset boxes
            		}
            		
                	update_spect = false;
                }
   
                e.gc.drawImage(image, 0, 0);				// map bitmap to screen
            }     
        }); 
        
        canvas.addMouseWheelListener(new MouseWheelListener() 
        {
			@Override
			public void mouseScrolled(MouseEvent e) 
			{       		
				if(e.count > 0)
					gp.expt_scale *= 1.10;
				else
					gp.expt_scale /= 1.10;
				
				SW_main.gp.printInsetBox[n].yscale = gp.expt_scale;
				gp.new_2D_view = true;
        		update_spect = true;
        		canvas.redraw();
			}
        });       
	}
}
