package spinworks;

import java.awt.Desktop;
import java.io.File;
import java.io.PrintWriter;
import org.eclipse.swt.SWT;

import dialogs.IntegDialog;

public class IntegrateHandler extends SW_main
{
//-------------------------------------------------------------------------------------------------
//	start integration dialog

	public static void IntegrateButtonClick()
	{
		if (gp.integ_mode == true) return;	// already running!
		if (gp.display_mode == modes.MODE_1D &&
				gp.data_array == null)
		{
			Util.show_and_log("No experimental data to integrate");
			return;
		}

		integDialog = new IntegDialog(shell, SWT.NONE | SWT.TITLE | SWT.BORDER );
		gp.integ_mode = true;						// flag integration mode
		gp.cursor_flag = 0;                         // clear any cursors
		integDialog.open();

		update_spect = true;						// refresh screen
		canvas.redraw();
	}
		
//-------------------------------------------------------------------------------------------------
	/**
	 * serial peak pick (1 cursor) or integrate (2 cursors) of a peak in a stack
	 */
	public static void peakPickStack()
	{
		if(gp.n_traces == 0)
		{
			Util.show_and_log("You must be in stack trace mode");
			return;
		}
		
		if (gp.cursor_flag == 0)
		{
			Util.show_and_log("You must define peak or region with one or two cursors first");
			return;
		}
		int j;

		String peaksFileName;
		PrintWriter pw;

		peaksFileName = q.data_folder + "array_peaks.txt";

		try
		{
			pw = new PrintWriter(peaksFileName);
		}
		catch (Exception ex)
		{
			Util.show_and_log("list_peaks():" + peaksFileName, ex.getMessage());
			return;
		}

		if (gp.cursor_flag == 1)
		{
			pw.println("Array peaklist for: " + q.data_folder);
			pw.println(String.format("   at %.3f ppm", gp.new_freq1 / q.SF));
			pw.println("normalized to first trace");
			pw.println();
		} 
		else if (gp.cursor_flag == 2)
		{
			pw.println("Array integral list for: " + q.data_folder);
			pw.println(String.format("   from %.3f ppm to %.3f ppm",
					gp.new_freq1 / q.SF, gp.new_freq2 / q.SF));
			pw.println("normalized to first trace");
			pw.println();
		}

		if (gp.cursor_flag == 1)
		{
			for (j = 0; j < gp.n_traces; j++)
			{
				double freq = PeakPick.findPeak(gp.new_freq1, 8.0, gp, q, sp);
				double intensity = gp.stack_trace[j].data[Util.freq_to_index(freq, q)];
				pw.println(String.format("%3d   %8.5f", j + 1, intensity / gp.data_max));
			}
		}
		else if (gp.cursor_flag == 2)
		{
			int start = Util.freq_to_index(gp.new_freq1, q);
			int end = Util.freq_to_index(gp.new_freq2, q);

			double integ_max = 0.0;
			for (int i = 0; i < q.SI; i++)
				integ_max = integ_max + gp.stack_trace[0].data[i];

			double sum;
			for (j = 0; j < gp.n_traces; j++)
			{
				sum = 0.0;
				for (int i = start; i <= end; i++)
					sum = sum + gp.stack_trace[j].data[i];
				pw.println(String.format("%3d   %10.5f", j + 1, sum / integ_max));
			}
		}

		pw.close();

		//	List with whatever editor is associated with the ".txt" file type
		if (Desktop.isDesktopSupported()) // new way!  portable??
		{
			try 
			{
				Desktop.getDesktop().open(new File(peaksFileName));
			} 
			catch (Exception ex) 
			{
				Util.show_and_log("no application for this file type?? " +
						peaksFileName, ex.getMessage());
			}
		}
		else
			Util.show_and_log("Desktop not supported");
	}
		
//-------------------------------------------------------------------------------------------------
	public static void integrateStack()
	{
		if(gp.n_traces == 0)
		{
			Util.show_and_log("You must be in stack trace mode");
			return;
		}
		
		String integFileName =  q.data_folder  + "integ_series.txt";
		PrintWriter pw;

		try
		{
			pw = new PrintWriter(integFileName);
		}
		catch (Exception ex)
		{
			Util.show_and_log("integrateStack(): " + integFileName, ex.getMessage());
			return;
		}

		pw.println("SpinWorks integral list for:  " + q.data_folder);
		pw.println("normalized to total integral in first trace");
		pw.println();
		pw.println("   #        start(ppm)        end(ppm)       integral");
		pw.println();

		double max = 0.0;
		for (int k = 0; k < q.SI; k++)
			max += gp.stack_trace[0].data[k];
		
		for (int j = 0; j < gp.n_traces; j++)
		{
			pw.println(String.format("trace # %d", j + 1));
			for (int i = 0; i < gp.n_integrals; i++)
			{
				if (gp.integrals[i].start > -1.0e40)
				{
					int start = Util.freq_to_index(gp.integrals[i].start, q);
					int end = Util.freq_to_index(gp.integrals[i].end, q);
					if (start < 0) start = 0;
					if (end >= q.SI - 1) end = q.SI - 1;
					double value = 0.0;

					for (int k = start; k <= end; k++)
						value += gp.stack_trace[j].data[k];

					pw.println(String.format(
							"   %3d      %10.3f      %10.3f      %12.8f",
							i + 1,
							gp.integrals[i].start / q.SF, gp.integrals[i].end / q.SF,
							value / max));
				}
			}
			pw.println();
		}

		pw.println("\nAlternate format");
		pw.println(
				"Peak # Start (ppm)         End (ppm)       Integral[trace 1]...Integral[trace n]");
		pw.println();

		for (int j = 0; j < gp.n_integrals; j++)
		{
			int start = Util.freq_to_index(gp.integrals[j].start, q);
			int end = Util.freq_to_index(gp.integrals[j].end, q);
			if (start < 0) start = 0;
			if (end >= q.SI - 1) end = q.SI - 1;

			pw.print(String.format("   %3d  %10.3f        %10.3f    ",
					j + 1, gp.integrals[j].start / q.SF, gp.integrals[j].end / q.SF));

			for (int i = 0; i < gp.n_traces; i++)
			{
				double value = 0.0;
				for (int k = start; k <= end; k++)
					value += gp.stack_trace[i].data[k];
				pw.print(String.format("   %.8f", value / max));
			}
			pw.println();
		}

		pw.close();

		//	List with whatever editor is associated with the ".txt" file type
		if (Desktop.isDesktopSupported()) // new way!  portable??
		{
			try 
			{
				Desktop.getDesktop().open(new File(integFileName));
			} 
			catch (Exception ex) 
			{
				Util.show_and_log("no application for this file type?? " +
						integFileName, ex.getMessage());
			}
		}
		else
			Util.show_and_log("Desktop not supported");
	}
	
//-------------------------------------------------------------------------------------------------		
}
