package spinworks;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import spinworks.SW_main.byte_orders;

public class JDF_Util 
{
	//TODO  check whether byte order info is being properly extracted from the JEOL parameters
	 final static int MAX_PAR_SIZE = 8192;
	 final static byte REAL = 1, TPPI = 2, COMPLEX = 3, REAL_COMPLEX = 4, ENVELOPE = 5;
	 final static int ONE_D = 1, TWO_D = 2, THREE_D = 3, SMALL_TWO_D = 12;
	 final static int ENDIAN = 8, DATA_DIMENSION_NUMBER = 12, DATA_DIMENSION_EXISTS = 13,
			 			DATA_TYPE = 14, TRANSLATE = 16, DATA_AXIS_TYPE = 24, DATA_UNITS = 32, TITLE = 48,
			 			DATA_POINTS = 176, DATA_OFFSET_START = 208, DATA_OFFSET_STOP = 240,
			 			DATA_AXIS_START = 272, DATA_AXIS_STOP = 336,
			 			COMMENT = 680, BASE_FREQ = 1064, REVERSED = 1192,
			 			PARAM_START = 1212, PARAM_LENGTH = 1216,
			 			ZERO_POINT = 1128, DATA_START = 1284, DATA_LENGTH = 1288;
	 
	 private static JDF_Header hd = new JDF_Header();
	 
//-------------------------------------------------------------------------------------------------
/// method to read parameters from a JEOL jdf file

	 public static int get_jdf_pars(String fileName, global_par gp, data_par p,
			 data_par p_f1, data_par q, data_par q_f1)
	 {
		 
		 int index;
		 byte[] byte_array;

		 JDF_Parameter[] param = new JDF_Parameter[MAX_PAR_SIZE];
			 	
		 String delim = "[ =,\t]+";
		 String[] token;

		 double x_freq = 0.0, x_sweep = 0.0, x_offset = 0.0;
		 double y_freq = 0.0, y_sweep = 0.0, y_offset = 0.0;

		 int x_points = 0, y_points = 0;
		 boolean[] axis_exists = new boolean[8];
		 boolean digital_filter = false;
		 int n_stages = 0;
		 double[] orders = new double[8];      // number of stages and order at each stage
		 double[] factors = new double[8];     // decimation factor at each stage
		 
		 FileInputStream fs;
		 File file = new File(fileName);

		 // try to open the data file, bail out if fails
		 try { fs = new FileInputStream(file); }
		 catch (IOException ex)
		 {
			Util.show_and_log("ge_jdf_pars(): cannot open: " + fileName, ex.getMessage());
			return -1;
		 }

		 byte_array = new byte[(int)file.length()];
		 
		 // read into byte array
		 try { fs.read(byte_array); } 
		 catch (IOException ex) 
		 {
			 ex.printStackTrace();
			 try { fs.close(); } 
			 catch (IOException e) { e.printStackTrace(); }
			 return -3;
		 }

		 ByteBuffer bb = ByteBuffer.wrap(byte_array);

		 // Note:  all fields in the header (and the list and the history) sections are in
		 // BIG endian.  The byte order of the parameter and data sections is given by 
		 // the ENDIAN values in the header (most likely LITTLE - Intel these days).
		 // Note that Java JVM is Big endian

		 hd.file_identifier = get_string_from_jdf(byte_array, 0, 8);

		 index = ENDIAN;
		 hd.endian = byte_array[index];
		 //System.out.println(String.format("ENDIAN = %d", (int)hd.endian));

		 index = DATA_DIMENSION_NUMBER;
		 hd.data_dimension_number = byte_array[index];

		 index = DATA_DIMENSION_EXISTS;
		 hd.data_dimension_exist = byte_array[index];
		 if ((hd.data_dimension_exist & 0x80) != 0) axis_exists[0] = true;
		 if ((hd.data_dimension_exist & 0x40) != 0) axis_exists[1] = true;

		 index = DATA_TYPE;
		 hd.data_type = byte_array[index];

		 index = TRANSLATE;
		 for(int i = 0; i < 8; i++)
		 {
			 hd.translate[i] = byte_array[index + i];
		 }

		 if ((hd.data_type & 0x3F) == 1)
			 q.data_mode = SW_main.modes.MODE_1D;

		 else if ((hd.data_type & 0x3F) == 2)
			 q.data_mode = SW_main.modes.MODE_2D;

		 else
		 {         
			 Util.show_and_log(String.format("get jdf file(): unsupported dimension #: %d",
					 (int)(hd.data_type & 0x3F)));
			 try { 	fs.close();	} 
			 catch (IOException ex) {	ex.printStackTrace();	}
			 return -1;
		 }

		 index = DATA_AXIS_TYPE;

		 for (int i = 0; i < 8; i++)
		 {
			 hd.data_axis_type[i] = byte_array[index + i];
			 if (gp.log_on) Util.log_error(String.format("   hd.data_axis_type[%d] = {%d}",
					 i, (int)hd.data_axis_type[i]));
		 }

		 hd.title = get_string_from_jdf(byte_array, TITLE, 124);
		 if (gp.log_on) Util.log_error("   title = " + hd.title);

		 index = DATA_UNITS;

		 hd.data_units[0] = bb.getShort(index);		 
		 hd.data_units[1] = bb.getShort(index);
		 
		 // looks like 284 is FID and 282 is processed ???

		 if (gp.log_on)
		 {
			 Util.log_error(String.format("   header units[0] = %d", hd.data_units[0]));
			 Util.log_error(String.format("   header units[1] = %d", hd.data_units[1]));
		 }

		 index = DATA_POINTS;

		 for (int i = 0; i < 8; i++)
		 {
			 hd.data_points[i] = bb.getInt(index);
			 index = index + 4;
			 if(gp.log_on)
				 Util.log_error(String.format("   hd.data_points[%d] = %d  ", i, hd.data_points[i]));
		 }

		 index = DATA_OFFSET_START;

		 for (int i = 0; i < 8; i++)
		 {
			 hd.data_offset_start[i] = bb.getInt(index);
			 index = index + 4;
			 if(gp.log_on)
				 Util.log_error(String.format("   hd.data_offset_start[%d] = %d  ", 
				 i, hd.data_offset_start[i]));
		 }

		 index = DATA_OFFSET_STOP;

		 for (int i = 0; i < 8; i++)
		 {;
			 hd.data_offset_stop[i] = bb.getInt(index);
			 index = index + 4;
			 if (gp.log_on)
				 Util.log_error(String.format("   hd.data_offset_stop[%d] = %d  ",
				 i, hd.data_offset_stop[i]));
		 }

		 index = DATA_AXIS_START;

		 for (int i = 0; i < 8; i++)
		 {
			 hd.data_axis_start[i] = bb.getDouble(index);
			 index = index + 8;
			 if (gp.log_on)
				 Util.log_error(String.format("   hd.data_axis_start[%d] = %f  ",
				 i, hd.data_axis_start[i]));
		 }

		 index = DATA_AXIS_STOP;

		 for (int i = 0; i < 8; i++)
		 {
			 hd.data_axis_stop[i] = bb.getDouble(index);
			 index = index + 8;
			 if (gp.log_on)
				 Util.log_error(String.format("   hd.data_axis_stop[%d] = %f  ",
				 i, hd.data_axis_stop[i]));
		 }

		 hd.comment = get_string_from_jdf(byte_array, COMMENT, 128);
		 if (gp.log_on) Util.log_error("   comment = " + hd.comment);

		 index = BASE_FREQ;

		 for (int i = 0; i < 8; i++)
		 {
			 hd.base_freq[i] = bb.getDouble(index);
			 index = index + 8;
			 if (gp.log_on)
				 Util.log_error(String.format("   hd.base_freq[%d] = %f  ",
				 i, hd.base_freq[i]));
		 }

		 index = REVERSED;

		 for (int i = 0; i < 8; i++)
		 {
			 if (byte_array[index] == (byte)1)
				 hd.reversed[i] = true;
			 else
				 hd.reversed[i] = false;

			 index = index + 1;
			 if (gp.log_on)
				 Util.log_error(String.format("   hd.reversed[%d] = %b  ",
				 i, hd.reversed[i]));
		 }

		 index = PARAM_START;
		 hd.param_start = bb.getInt(index);
		 if(gp.log_on) 
			 Util.log_error(String.format("   Parameter start = %d", hd.param_start));

		 index = PARAM_LENGTH;
		 hd.param_length = bb.getInt(index);
		 if(gp.log_on)
			 Util.log_error(String.format("   Parameter length = %d", hd.param_length));

		 index = ZERO_POINT;

		 for (int i = 0; i < 8; i++)
		 {
			 hd.base_freq[i] = bb.getDouble(index);
			 index = index + 8;
			 if (gp.log_on)
				 Util.log_error(String.format("   hd.zero_point[%d] = %f  ",
				 i, hd.zero_point[i]));
		 }

		 index = DATA_START;
		 hd.data_start = bb.getInt(index);
		 if(gp.log_on)
			 Util.log_error(String.format("   data start = %d   ", hd.data_start));

		 index = DATA_LENGTH;
		 hd.data_length = bb.getLong(index);
		 if(gp.log_on)
			 Util.log_error(String.format("   hd.data length = %d   ", hd.data_length));
		 
		 int j = 0;  // j counts number of parameters, i is index (in bytes) into buffer array
		             // from the beginning of the array

		 //TODO need better decoding of units!
		 if(gp.log_on) Util.log_error("***  JEOL parameter block read starting ***");			 
		 
		 
		 for (int i = (int)hd.param_start + 16;
				 i < hd.param_start + 16 + hd.param_length && j < MAX_PAR_SIZE;
				 i = i + 64)      // load parameters into parameter array
		 {                
			 param[j] = get_parameter_from_jdf(bb, i);   // read parameter structure
			 
			 ByteBuffer par_buffer = ByteBuffer.wrap(param[j].unit_value);			 	 
		     par_buffer.order(ByteOrder.LITTLE_ENDIAN);	// handle byte order issues here!

			 if(gp.log_on)   // dump parameters to debug file if requested.
			 {
				 Util.log_error("");
				 Util.log_error("new parameter >>");
				 Util.log_error(String.format("  name = %s", param[j].name));

				 Util.log_error(String.format("  unit scaler = %d", param[j].unit_scaler));

				 String temp_str = "";
				 for (int k = 0; k < 10; k++)
					 temp_str = temp_str + String.format("   units = %d   ", (int)param[j].units[k]);

				 Util.log_error(temp_str);

				 temp_str = "";
				 
				 for (int k = 0; k < 16; k++)
					 temp_str = temp_str + String.format("   value = %d   ", (int)param[j].unit_value[k]);
				 Util.log_error(temp_str);

				 Util.log_error(String.format("value type = %d", param[j].value_type));

				 if (param[j].value_type == 0)
					 Util.log_error(String.format("string value = %s",
							 get_string_from_jdf(param[j].unit_value, 0, 16)));

				 if (param[j].value_type == 1)
					 Util.log_error(String.format("integer value = %d", par_buffer.getInt(0)));

				 if (param[j].value_type == 2)
					 Util.log_error(String.format("float value = %f", par_buffer.getDouble(0)));
			 }

			 if (param[j].name.trim().equalsIgnoreCase("SCANS"))
			 {
				 q.NS = par_buffer.getInt(0);
			 }

			 if (param[j].name.trim().equalsIgnoreCase("X_FREQ"))
			 {
				 x_freq = par_buffer.getDouble(0)
						 * Math.pow(10.0, (int)param[j].unit_scaler);
				 /*System.out.println(String.format("X_FREQ value units = %d  %d  scaler = %d", 
						 (int)param[j].units[0], (int)param[j].units[1], (int)param[j].unit_scaler));*/

				 if (/*param[j].units[0] == 3329  && */ param[j].units[1] == 13)
					 x_freq = x_freq / 1.0e6;				 			 
			 }

			 if (param[j].name.trim().equalsIgnoreCase("Y_FREQ"))
			 {
				 y_freq = par_buffer.getDouble(0)
						 * Math.pow(10.0, (int)param[j].unit_scaler);

				 if (/* param[j].units[0] == 3329 && */ param[j].units[1] == 13)
					 y_freq = y_freq / 1.0e6;
			 }


			 if (param[j].name.trim().equalsIgnoreCase("X_SWEEP"))
			 {
				 x_sweep = par_buffer.getDouble(0)
						 * Math.pow(10.0, (int)param[j].unit_scaler);
				 
				 /*System.out.println(String.format("X_SWEEP units = %d  %d  scaler = %d", 
						 param[j].units[0], param[j].units[1], (int)param[j].unit_scaler));*/
				 //  TO DO: check for other possible units for x_sweep (e.g. ppm or MHz)
			 }

			 if (param[j].name.trim().equalsIgnoreCase("Y_SWEEP"))
			 {
				 y_sweep = par_buffer.getDouble(0)
						 * Math.pow(10.0, (int)param[j].unit_scaler);
				 //  TO DO: check for other possible units for x_sweep (e.g. ppm or MHz)
			 }

			 if (param[j].name.trim().equalsIgnoreCase("X_OFFSET"))
			 {
				 x_offset = par_buffer.getDouble(0)
						 * Math.pow(10.0, (int)param[j].unit_scaler); // other units??
			 }

			 if (param[j].name.trim().equalsIgnoreCase("Y_OFFSET"))
			 {			 
				 y_offset = par_buffer.getDouble(0)
						 * Math.pow(10.0, (int)param[j].unit_scaler); // other units??
			 }

			 if (param[j].name.trim().equalsIgnoreCase("X_POINTS"))
			 {
				 x_points = par_buffer.getInt(0);
			 }

			 if (param[j].name.trim().equalsIgnoreCase("Y_POINTS"))
			 {
				 y_points = par_buffer.getInt(0);
			 }

			 if (param[j].name.trim().equalsIgnoreCase("EXPERIMENT") && (param[j].value_type == 0))
			 {
				 q.pulse_sequence = get_string_from_jdf(param[j].unit_value, 0, 16);
			 }

			 // digital filter parameters

			 if (param[j].name.trim().equalsIgnoreCase("DIGITAL_FILTER"))
			 {
				 String filter = get_string_from_jdf(param[j].unit_value, 0, 16);
				 filter = filter.trim();

				 //System.out.println("DIGITAL FILTER");
				 if (filter.contains("TRUE"))
				 {
					 digital_filter = true;
					 //System.out.println("   IS TRUE");
				 }
			 }

			 if (param[j].name.trim().equalsIgnoreCase("ORDERS"))
			 {
				 String temp = get_string_from_jdf(param[j].unit_value, 0, 16);				 
				 temp = temp.trim(); 				 
				 token = temp.split(delim);
				 				 
				 if (token.length != 0)
				 {
					 try { n_stages = Integer.parseInt(token[0]);	}
					 catch(NumberFormatException ex){ ex.printStackTrace();	}
				 }

				 for (int k = 1; k < token.length && k < orders.length; k++)
				 {
					 try{	orders[k - 1] = Double.parseDouble(token[k]);	}
					 catch(NumberFormatException ex){	ex.printStackTrace();	}
					 //System.out.println(String.format("  orders[%d] = %f", k, orders[k]));
				 }

			 }

			 if (param[j].name.trim().equalsIgnoreCase("FACTORS"))
			 {
				 String temp = get_string_from_jdf(param[j].unit_value, 0, 16);
				 temp = temp.trim();
				 token = temp.split(delim);

				 for (int k = 0; k < token.length && k < factors.length; k++)
				 {
					 try{	factors[k] = Double.parseDouble(token[k]);	}
					 catch(NumberFormatException ex){	ex.printStackTrace(); }
					 //System.out.println(String.format("  factors[%d] = %f", k, factors[k]));
				 }
			 }
			 
			 // TODO may need this!  shows whether the FID has been "time corrected"
			 if (param[j].name.trim().equalsIgnoreCase("digital_filter_status"))
			 {
				 String temp = get_string_from_jdf(param[j].unit_value, 0, 16);
				 if(gp.log_on)
					 Util.log_error("digital filter status = " + temp);
				 temp = temp.trim();
			 }

			 j++;
		 }   // end for i (parameter scan)

		 //  convert to SpinWorks internal parameters	 

		 // status pars
		 q.SFO1 = x_freq;                        p.SFO1 = q.SFO1;
		 q.SW = x_sweep / q.SFO1;                p.SW = q.SW;
		 q.TD = 2 * x_points;                    p.TD = q.TD;

		 p.SI = q.TD;

		 if (q.data_mode == SW_main.modes.MODE_2D)
		 {
			 q_f1.SFO1 = y_freq;                 p_f1.SFO1 = q_f1.SFO1;
			 q_f1.SW = y_sweep / q_f1.SFO1;      p_f1.SW = q_f1.SW;

			 if (hd.data_axis_type[1] == (byte)4 || hd.data_axis_type[1] == (byte)1)
			 {
				 q_f1.TD = y_points;
			 }

			 // try to decode various indirect detection methods
			 if (hd.data_axis_type[1] == REAL_COMPLEX && hd.data_axis_type[1] == REAL_COMPLEX)
			 { 
				 // magnitude (COSY/HMBC type)
				 p_f1.MC2 = SW_main.det_modes_IND.QF;
				 p.WDW = SW_main.windows.SINE;
				 p_f1.WDW = SW_main.windows.SINE;
				 p_f1.PH_mod = SW_main.phasing.MAGNITUDE;
				 hd.n_sections = 2;
			 }

			 if (hd.data_axis_type[1] == COMPLEX && hd.data_axis_type[1] == COMPLEX)
			 { 
				 // hypercomplex (e.g States, EA, etc.)
				 q_f1.TD = 2 * y_points;
				 p_f1.MC2 = SW_main.det_modes_IND.STATES;
				 p.WDW = SW_main.windows.QSINE;
				 p_f1.WDW = SW_main.windows.QSINE;
				 p.ssb = 90.0;
				 p_f1.ssb = 90.0;
				 hd.n_sections = 4;
				 if (hd.reversed[1])
					 p_f1.REVERSE = true;
			 }

			 p_f1.SI = q_f1.TD * 2;
		 }

		 int ii = 64;							         // minimum reasonable size 
		 while ((p.SI - 1) / ii > 0) ii = ii * 2;		 // force size to be a power of 2 (for FFT) 
		 p.SI = ii;
		 q.SI = p.SI;                                    // necessary? set at transform?

		 q.SF = q.SFO1 - (x_offset * x_freq) / 1.0e6; p.SF = q.SF;
		 q.hzperpt = x_sweep / (double)(q.SI); p.hzperpt = q.hzperpt;
		 q.offset = (q.SFO1 - q.SF) * 1.0e06 + x_sweep / 2.0; p.offset = q.offset;

		 if (q.data_mode == SW_main.modes.MODE_2D)
		 {
			 ii = 32;                                    // minimum reasonable size
			 while ((p_f1.SI - 1) / ii > 0) ii = ii * 2; // force size to be a power of 2 (for FFT) 
			 p_f1.SI = ii;
			 q_f1.SI = p_f1.SI;                          // necessary? set at transform?

			 q_f1.SF = q_f1.SFO1 - (y_offset * y_freq) / 1.0e6;          p_f1.SF = q_f1.SF;
			 q_f1.hzperpt = y_sweep / (double)(q_f1.SI);                 p_f1.hzperpt = q_f1.hzperpt;
			 q_f1.offset = (q_f1.SFO1 - q_f1.SF) * 1.0e06 + y_sweep / 2.0;  p_f1.offset = q_f1.offset;
			 p_f1.lp_pred = q_f1.TD;
		 }   

		 if (digital_filter)
		 {
			 q.GRPDLY = calc_JEOL_group_delay(orders, factors, n_stages);
			 p.GRPDLY = q.GRPDLY;
		 }

		 p.LB = 2 * x_sweep / q.TD;      // default LB = 1/AT

		 gp.freq1 = q.offset;
		 gp.freq2 = q.offset - x_sweep;

		 if (q.data_mode == SW_main.modes.MODE_2D)
		 {
			 gp.freq1_indirect = q_f1.offset;
			 gp.freq2_indirect = q_f1.offset - y_sweep;
		 }

		 gp.plot_title = hd.comment;
		 gp.plot_title.trim();

		 if (hd.endian == 0)
			 q.BYTORDA = SW_main.byte_orders.BIG;
		 else
			 q.BYTORDA = SW_main.byte_orders.LITTLE;
		 p.BYTORDA = q.BYTORDA;

		 if (digital_filter)
			 q.aq_mode = SW_main.det_modes.CUSTOM_DSP;  // change to DSP once I'm
			 else                                       // sure I have it right
				 q.aq_mode = SW_main.det_modes.SIM;

		 if (hd.data_axis_type[1] == (byte)4)
			 q_f1.MC2 = SW_main.det_modes_IND.QF;

		 p.aq_mode = q.aq_mode;
		 q.dwell = 1.0 / (q.SW * q.SFO1);

		 if (gp.log_on)
		 {
			 Util.log_error(String.format("***  NS =   %d", q.NS));
			 Util.log_error(String.format("***  SFO1 = %f", q.SFO1));
			 Util.log_error(String.format("***  SW =   %f", q.SW));
			 Util.log_error(String.format("***  TD =   %d", q.TD));

			 if (q.data_mode == SW_main.modes.MODE_2D)
			 {
				 Util.log_error(String.format("***  SFO1 (F1) = %f", q_f1.SFO1));
				 Util.log_error(String.format("***  SW (F1) =   %f", q_f1.SW));
				 Util.log_error(String.format("***  TD (F1)   = %d", q_f1.TD));
			 }
		 }
		 
		 if (hd.data_units[0] == (short)(282))     // spectrum  TODO  Check!!!
		 {
			 q.SI = x_points;         p.SI = q.SI;
			 q.hzperpt = x_sweep / (double)(q.SI);   p.hzperpt = q.hzperpt;
			 try { 	fs.close();	} 
			 catch (IOException e) {	e.printStackTrace();	}
			 return 1;
		 }
		 else
		 {
			 try { 	fs.close();	} 
			 catch (IOException e) {	e.printStackTrace();	}
			 return 2;                                       // fid	
		 }
	 }
	 
//-------------------------------------------------------------------------------------------------
//   Reads a JEOL 1D fid

	 public static int get_jdf_fid_1D(String fileName, global_par gp, data_par q)
	 {
		 int bytes_read = 0;
		 int j, data_start;

		 FileInputStream fs;
	
	      File file = new File(fileName);

	      try { fs = new FileInputStream(file); }
	      catch (IOException ex)
	      {
	          Util.show_and_log("get_jdf_fid_1D(): cannot open: " + fileName, ex.getMessage());
	          return -1;
	      }
		      
	      byte[] b = new byte[(int)file.length()];
	      
	      try { fs.read(b); } 
	      catch (IOException ex) 
	      {
			  ex.printStackTrace();
			  try { fs.close(); } 
		      catch (IOException e) { e.printStackTrace(); }
			  return -3;
		  }
		      
	      ByteBuffer bb = ByteBuffer.wrap(b);
	      
	      // handle byte order issues here!
	      if (q.BYTORDA == byte_orders.LITTLE)
	    	  bb.order(ByteOrder.LITTLE_ENDIAN);
	      
		 data_start = hd.data_start;

		 try { gp.fid = new double[q.TD]; }
		 catch (Exception ex)
		 {
			 Util.show_and_log("get_jdf_file(): cannot allocate space for fid "
					 + fileName + " " + ex.getMessage());
			 try { 	fs.close();	} 
			 catch (IOException e) {	e.printStackTrace();	}
			 return -2;
		 }

		 j = 0;      // read the real points
		 int start_index = (int)data_start;
		 for (int i = start_index; i < start_index + 8 * q.TD/2; i = i + 8)
		 {
			 gp.fid[j] = bb.getDouble(i);
			 j = j + 2;
			 bytes_read += 8;
		 }

		 j = 1;      // read the imaginary points
		 start_index = (int)data_start + 8 * q.TD/2;
		 for (int i = start_index; i < start_index + 8 * q.TD/2; i = i + 8)
		 {
			 gp.fid[j] = bb.getDouble(i);
			 j = j + 2;
			 bytes_read += 8;
		 }

		 try { 	fs.close();	} 
		 catch (IOException e) {	e.printStackTrace();	}	
		 
		 return bytes_read;
	 }
	  
//-------------------------------------------------------------------------------------------------
// reads a JEOL 2D fid
	 public static int get_jdf_fid_2D(String fileName, global_par gp, 
			 data_par q, data_par q_f1)
	 {
		 int bytes_read = 0;

		 FileInputStream fs;
	
	      File file = new File(fileName);

	      try { fs = new FileInputStream(file); }
	      catch (IOException ex)
	      {
	          Util.show_and_log("get_ser(): cannot open: " + fileName, ex.getMessage());
	          return -1;
	      }
	      
	      byte[] b = new byte[(int)file.length()];

		 // read into byte array
	      try { fs.read(b); } 
	      catch (IOException ex) 
	      {
			  ex.printStackTrace();
			  try { fs.close(); } 
		      catch (IOException e) { e.printStackTrace(); }
			  return -3;
		  }
		      
		 try { gp.ser = new double[q_f1.TD][q.TD]; }
		 catch (Exception ex)
		 {
			 Util.show_and_log("get_jdf_fid_2D(): cannot allocate space for fid "
					 + fileName + " " + ex.getMessage());
			 try { fs.close(); } 
		     catch (IOException e) { e.printStackTrace(); }
			 return -2;
		 }
		 
		 if (hd.n_sections == 2)
			 bytes_read = 4 * convert_jeol_data_mag(b, gp, q, q_f1);
		 else if (hd.n_sections == 4)
			 bytes_read = 4 * convert_jeol_data_hypercomplex(b, gp, q, q_f1);


		 try { fs.close(); } 
	     catch (IOException e) { e.printStackTrace(); }
		 return bytes_read;  // perhaps something more meaningful
	 }
 
//-------------------------------------------------------------------------------------------------
     private static int convert_jeol_data_mag(byte[] buffer, global_par gp, data_par q, data_par q_f1)
     {
         double[] jeol_fid_real, jeol_fid_imag;
         double[] bruker_fid;	    
         int i, j, k;		        // general loop counters

         double[] jeol_sort_real, jeol_sort_imag; 
         int index;                  // counter into array
         int sub_posi0, sub_posi1;   // submatrix positions
         int data_length = (int)hd.data_length / 8 /*sizeof(double)*/; // size if data block in points
         int points_converted = 0;

         final int SUBMATRIX_DIM_2D = 32;

         try
         {
             bruker_fid = new double[data_length];
             jeol_fid_real = new double[data_length / 2];  // real section
             jeol_fid_imag = new double[data_length / 2];  // imag section
             jeol_sort_real = new double[data_length / 2];
             jeol_sort_imag = new double[data_length / 2];
         }
         catch (Exception ex)
         {
             Util.show_and_log("convert_jeol_data_mag(): cannot allocate space",
                 ex.getMessage());
             return -1;
         }

         ByteBuffer bb = ByteBuffer.wrap(buffer);
	      
	      // handle byte order issues here!
	      if (q.BYTORDA == byte_orders.LITTLE)
	    	  bb.order(ByteOrder.LITTLE_ENDIAN);
	      
         // extract data from JEOL file
         j = 0;
         for (i = (int)hd.data_start; 
             i < (int)hd.data_start + (int)hd.data_length / 2; 
             i = i + 8)
         {
             jeol_fid_real[j] = bb.getDouble(i);
             j++;
         }

         j = 0;
         for (i = (int)hd.data_start + (int)hd.data_length / 2; 
             i < (int)hd.data_start + (int)hd.data_length; 
             i = i + 8)
         {
        	 jeol_fid_imag[j] = bb.getDouble(i);
             j++;
         }

         index = 0;
         for (k = 0; k < hd.data_points[1] / SUBMATRIX_DIM_2D; k++)
         {
             for (i = 0; i < SUBMATRIX_DIM_2D; i++) 
             {
                 for (sub_posi1 = 0; sub_posi1 < hd.data_points[0] / (SUBMATRIX_DIM_2D); sub_posi1++)
                 {
                     for (sub_posi0 = 0; sub_posi0 < SUBMATRIX_DIM_2D; sub_posi0++)
                     {
                         jeol_sort_real[index] = jeol_fid_real[k * hd.data_points[0] * SUBMATRIX_DIM_2D
                                             + i * SUBMATRIX_DIM_2D 
                                             + sub_posi1 * SUBMATRIX_DIM_2D * SUBMATRIX_DIM_2D
                                             + sub_posi0];
                         jeol_sort_imag[index] = jeol_fid_imag[k * hd.data_points[0] * SUBMATRIX_DIM_2D
                                             + i * SUBMATRIX_DIM_2D 
                                             + sub_posi1 * SUBMATRIX_DIM_2D * SUBMATRIX_DIM_2D
                                             + sub_posi0];
                         index++;
                     }
                 }
             }
         }
       
         for (i = 0; i < data_length / 2; i++)
         {
             bruker_fid[i * 2] = (jeol_sort_real[i]);    // Real section 

             bruker_fid[(i * 2) + 1]
                 = jeol_sort_imag[i];                    // Imag section
         }

         k = 0;

         // copy into internal "ser" file (TO DO combine with above item)
         for (i = 0; i < q_f1.TD; i++)
             for (j = 0; j < q.TD; j++)
             {
                 gp.ser[i][j] = bruker_fid[k];
                 k++;
                 points_converted++;
             }

         return (points_converted);

     }
 
 //------------------------------------------------------------------------------------------------
 // Converts a JEOL file with hyercomplex 2D data into the internal format required by SpinWorks
   
     private static int convert_jeol_data_hypercomplex(
    		 byte[] buffer, global_par gp, data_par q, data_par q_f1)
     {
    	 double[] jeol_fid_rr, jeol_fid_ri,
    	 jeol_fid_ir, jeol_fid_ii;	        // separated hypercomplex quadrants 

    	 double[] jeol_sort_rr, jeol_sort_ri,
    	 jeol_sort_ir, jeol_sort_ii;	        // separated sorted JEOL data

    	 int i, j, k;		                        // general loop counters
    	 int points_converted = 0;                   // number of points actually converted


    	 int index;                                  // index into array
    	 int sub_posi0, sub_posi1;

    	 int data_length = (int)hd.data_length / 8 /*sizeof(double)*/; // size of data block in points

    	 final int SUBMATRIX_DIM_2D = 32;

    	 try
    	 {
    		 jeol_fid_rr = new double[data_length / 4];  // rr section
    		 jeol_fid_ri = new double[data_length / 4];  // ri section
    		 jeol_fid_ir = new double[data_length / 4];  // ir section
    		 jeol_fid_ii = new double[data_length / 4];  // ii section              
    	 }
    	 catch (Exception ex)
    	 {
    		 Util.show_and_log(
    				 "convert_jeol_data_hypercomplex(): cannot create space for JEOL fid",
    				 ex.getMessage());
    		 return (-1);
    	 }
    	 
    	 ByteBuffer bb = ByteBuffer.wrap(buffer);
	      
	      // handle byte order issues here!
	      if (q.BYTORDA == byte_orders.LITTLE)
	    	  bb.order(ByteOrder.LITTLE_ENDIAN);
	             

    	 // extract data from JEOL ".jdf" file
    	 // hd (header) is currently static to the class (should be changed).
    	 j = 0;                                                          // extract rr
    	 for (i = (int)hd.data_start;
    			 i < (int)hd.data_start + (int)hd.data_length / 4;
    			 i = i + 8)
    	 {
    		 jeol_fid_rr[j] = bb.getDouble(i);
    		 j++;
    	 }

    	 j = 0;                                                          // extract ri
    	 for (i = (int)hd.data_start + (int)hd.data_length / 4;
    			 i < (int)hd.data_start + (int)hd.data_length / 2;
    			 i = i + 8)
    	 {
    		 jeol_fid_ri[j]  = bb.getDouble(i);
    		 j++;
    	 }

    	 j = 0;                                                          // extract ir
    	 for (i = (int)hd.data_start + (int)hd.data_length / 2;
    			 i < (int)hd.data_start + 3 * (int)hd.data_length / 4;
    			 i = i + 8)
    	 {
    		 jeol_fid_ir[j] = bb.getDouble(i);
    		 j++;
    	 }

    	 j = 0;                                                          // extract ii
    	 for (i = (int)hd.data_start + 3 * (int)hd.data_length / 4;
    			 i < (int)hd.data_start + (int)hd.data_length;
    			 i = i + 8)
    	 {
    		 jeol_fid_ii[j] = bb.getDouble(i);
    		 j++;
    	 }

    	 try
    	 {
    		 jeol_sort_rr = new double[data_length / 4];
    		 jeol_sort_ri = new double[data_length / 4];
    		 jeol_sort_ir = new double[data_length / 4];
    		 jeol_sort_ii = new double[data_length / 4];
    	 }
    	 catch (Exception ex)
    	 {
    		 Util.show_and_log(
    				 "convert_jeol_data_hypercomplex(): cannot create space for sorted JEOL fid",
    				 ex.getMessage());
    		 return (-1);
    	 }

    	 index = 0;
    	 for (k = 0; k < hd.data_points[1] / SUBMATRIX_DIM_2D; k++)
    	 {
    		 for (i = 0; i < SUBMATRIX_DIM_2D; i++)
    		 {
    			 for (sub_posi1 = 0; sub_posi1 < hd.data_points[0] / (SUBMATRIX_DIM_2D); sub_posi1++)
    			 {
    				 for (sub_posi0 = 0; sub_posi0 < SUBMATRIX_DIM_2D; sub_posi0++)
    				 {
    					 jeol_sort_rr[index] = jeol_fid_rr[k * hd.data_points[0] * SUBMATRIX_DIM_2D
    					                                   + i * SUBMATRIX_DIM_2D 
    					                                   + sub_posi1 * SUBMATRIX_DIM_2D * SUBMATRIX_DIM_2D
    					                                   + sub_posi0];
    					 jeol_sort_ri[index] = jeol_fid_ri[k * hd.data_points[0] * SUBMATRIX_DIM_2D
    					                                   + i * SUBMATRIX_DIM_2D 
    					                                   + sub_posi1 * SUBMATRIX_DIM_2D * SUBMATRIX_DIM_2D
    					                                   + sub_posi0];
    					 jeol_sort_ir[index] = jeol_fid_ir[k * hd.data_points[0] * SUBMATRIX_DIM_2D
    					                                   + i * SUBMATRIX_DIM_2D 
    					                                   + sub_posi1 * SUBMATRIX_DIM_2D * SUBMATRIX_DIM_2D
    					                                   + sub_posi0];
    					 jeol_sort_ii[index] = jeol_fid_ii[k * hd.data_points[0] * SUBMATRIX_DIM_2D
    					                                   + i * SUBMATRIX_DIM_2D 
    					                                   + sub_posi1 * SUBMATRIX_DIM_2D * SUBMATRIX_DIM_2D
    					                                   + sub_posi0];
    					 index++;
    				 }
    			 }
    		 }
    	 }

    	 // copy into internal "ser" file
    	 for (i = 0; i < q_f1.TD / 2; i++)       // indirect dimension
    	 {
    		 for (j = 0; j < q.TD / 2; j++)      // acquisition dimension
    		 {
    			 gp.ser[i * 2][j * 2] = jeol_sort_rr[j + i * (q.TD / 2)];
    			 gp.ser[i * 2][j * 2 + 1] = jeol_sort_ri[j + i * (q.TD / 2)];
    			 gp.ser[i * 2 + 1][j * 2] = jeol_sort_ir[j + i * (q.TD / 2)];
    			 gp.ser[i * 2 + 1][j * 2 + 1] = jeol_sort_ii[j + i * (q.TD / 2)];
    			 points_converted += 4;
    		 }
    	 }
    	 return (points_converted);
     }

           
           
//-------------------------------------------------------------------------------------------------
//   Reads a processed JEOL 1D spectrum

	 public static int get_jdf_spect_1D(String fileName, global_par gp, data_par q)
	 {
		 int bytes_read = 0;
		 int j, data_start;

		 FileInputStream fs;
	
	      File file = new File(fileName);

	      try { fs = new FileInputStream(file); }
	      catch (IOException ex)
	      {
	          Util.show_and_log("get_jdf_fid_1D(): cannot open: " + fileName, ex.getMessage());
	          return -1;
	      }
		      
	      byte[] b = new byte[(int)file.length()];
	      
	      try { fs.read(b); } 
	      catch (IOException ex) 
	      {
			  ex.printStackTrace();
			  try { fs.close(); } 
		      catch (IOException e) { e.printStackTrace(); }
			  return -3;
		  }
		      
	      ByteBuffer bb = ByteBuffer.wrap(b);
	      
	      // handle byte order issues here!
	      if (q.BYTORDA == byte_orders.LITTLE)
	    	  bb.order(ByteOrder.LITTLE_ENDIAN);
	     
	     // generate internal SpinWorks parameters
	     // note that data processed by JEOL "Delta" software may be trimmed 
	     // at the ends of the spectrum because SW > digital filter bandwidth

	     q.SI = (int)hd.data_offset_stop[0] - (int)hd.data_offset_start[0];
	     q.SW = hd.data_axis_start[0] - hd.data_axis_stop[0];
	     q.hzperpt = q.SW * q.SFO1 / (double)(q.SI - 1); // is -1 correct??
	     q.offset = hd.data_axis_start[0] * q.SF;
	     q.SF = q.SFO1 + ((q.SFO1 * q.SW) / 2.0 - q.offset) / 1000000.0;
          
		 data_start = hd.data_start;

		 try { Util.alloc_proc(gp, q.SI); }
		 catch (Exception ex)
		 {
			
			 Util.show_and_log("get_jdf_spect(): cannot allocate space for spectrum "
					 + fileName,  ex.getMessage());
			 try { 	fs.close();	} 
			 catch (IOException e) {	e.printStackTrace();	}
			 return -2;
		 }

		 j = 0;      // read the real points
		 int start_index = (int)data_start + 8 * (int)hd.data_offset_start[0];
		 try
		 {
			 for (int i = start_index; i < start_index + 8 * q.SI; i = i + 8)
			 {
				 gp.data_array[j] = bb.getDouble(i);
				 j++;
				 bytes_read = bytes_read + 8;
			 }
		 }
		 catch(Exception ex)
		 {
			 Util.show_and_log("error reading real data points", ex.getMessage());
			 ex.printStackTrace();
		 }

		 j = 0;      // read the imaginary points
		 start_index = (int)data_start
                 + 8 * (int)hd.data_points[0]
                 + 8 * (int)hd.data_offset_start[0];
		 try
		 {
			 for (int i = start_index; i < start_index + 8 * q.SI; i = i + 8)
			 {
				 gp.i_data_array[j] = bb.getDouble(i);
				 j++;
				 bytes_read = bytes_read + 8;
			 }
		 }
		 catch(Exception ex)
		 {
			 Util.show_and_log("error reading imaginary data points", ex.getMessage());
			 ex.printStackTrace();
		 }

		 try { 	fs.close();	} 
		 catch (IOException e) {	e.printStackTrace();	}

		 return bytes_read;
	 }

	 
//-------------------------------------------------------------------------------------------------	 
//   This method extracts a string from a JEOL jdf format file (or any byte array).

	 private static String get_string_from_jdf(byte[] buffer, int index, int length)
	 {
		 char[] ch = new char[length];

		 for (int i = 0; i < length; i++)
			 ch[i] = (char)buffer[index + i];

		 return new String(ch, 0, length);          
	 }
	 
//   overload for ByteBuffer, as opposed to byte[].  Are both methods necessary? 
	 private static String get_string_from_jdf(ByteBuffer buffer, int index, int length)
	 {
		 char[] ch = new char[length];

		 for (int i = 0; i < length; i++)
			 ch[i] = (char) buffer.get(index + i);

		 return new String(ch, 0, length);          
	 }
	 
//-------------------------------------------------------------------------------------------------
//   Extracts a parameter "structure" from a jdf byte array

	 private static JDF_Parameter get_parameter_from_jdf(ByteBuffer buffer, int index)
	 {
		 JDF_Parameter par = new JDF_Parameter();
		 int i;
		 byte[] b;
		 ByteBuffer bb;

		 par.unit_scaler = buffer.getShort(index + 4);
		 par.unit_scaler = Short.reverseBytes(par.unit_scaler);

		 for (i = 0; i < 10; i++)
		 {
		 	par.units[i] = buffer.get(index + 6 + i);
		 }

		 for (i = 0; i < 16; i++)
		 	par.unit_value[i] = buffer.get(index + 16 + i);	 

		 b = new byte[8];
		 
		 // TODO do this byte reversal a better way??
		 for (i = 0; i < 8; i++)
			 b[i] = buffer.get(index + 32 + i);
		 
		 bb = ByteBuffer.wrap(b);
		 bb.order(ByteOrder.LITTLE_ENDIAN);			 
		 par.value_type = bb.getInt(0);

		 par.name = get_string_from_jdf(buffer, index + 36, 28);

		 return par;
	 }
	 
//-------------------------------------------------------------------------------------------------
//   calculates the digital filter group delay (in dwell periods) for JEOL jdf data. 
//   The first order phase correction (degrees) will be 360 times the group delay.

	 private static double calc_JEOL_group_delay(double[] orders, double[] factors, 
			 int n_stages)
	 {          
		 double sum = 0.0;

		 try
		 {
			 for (int i = 0; i < n_stages; i++)
			 {
				 double denom = factors[i];
				 for (int j = i + 1; j < n_stages; j++)
					 denom *= factors[j];
				 sum += (orders[i] - 1.0) / denom;
			 }
		 }
		 catch (Exception ex)
		 {
			 Util.log_error("Exception during calculation JEOL group delay" + ex.getMessage());
			 return 0.0;
		 }
		 return 0.5 * sum;
	 }

//-------------------------------------------------------------------------------------------------	        
	        	        
}
