package spinworks;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;

import dialogs.PeakAssignmentDialog;
import dialogs.PeakLabelDialog;

public class MouseHandler extends SW_main
{
	 private static int mouse_down_x, mouse_down_y;		// cursor coordinates when mouse is down
	 private static boolean drag_zoom;					// in drag-zoom mode
	 private static boolean move_expt, move_calc;		// moving baseline of spectra
	 private static int cur_peak;						// current peak being assigned (2D)
	 public static boolean assignCOSY = false, assignTOCSY = false, 
			               assignNOESY = false, assignHMBC = false;
	 
	 public static PeakAssignmentDialog dlg;
	 // so we can't start a second peak assignment dialog.  need a better way to do this
	 
	// mouse is up, effectively a click
	public static void MouseUp(MouseEvent e)
	{
		mouse_down = false;
				
		if (e.button == 1 && gp.display_mode == modes.MODE_1D)  				// sets/clears cursors
            LeftMouseClick_1D(e);              									// if mouse has been dragged
                                                    							// does a drag-zoom  between two cursors
		else if (e.button == 1 && gp.display_mode == modes.MODE_FID)
			LeftMouseClick_FID(e);
		
		else if (e.button == 1 && gp.display_mode == modes.MODE_2D)
			LeftMouseClick_2D();
		
		else if (e.button == 2)
        	MiddleMouseClick();
		
        else if (e.button == 3 && gp.display_mode == modes.MODE_1D)			
        	RightMouseClick_1D(e);	// picks peak in normal 1D
	
        else if (e.button == 3 && gp.display_mode == modes.MODE_2D)
        {                                           
            if (q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XY)
                RightMouseClick_2D(q_f2);   		// selects row or column in 2D
            else
                RightMouseClick_2D(q_f1);
        }      
	}

//-------------------------------------------------------------------------------------------------
	public static void MouseDown(MouseEvent e)
	{
		if(e.button != 1) return;		// only for left mouse button
		if(gp.bpt_mode) return;
		mouse_down = true;
		
		if (Math.abs(e.y - (gp.dysize - gp.expt_y_offset)) < 5
				&& gp.data_array != null)
			move_expt = true;

		else if (Math.abs(e.y - (gp.dysize - gp.calc_y_offset)) < 5
				&& gp.calc_array != null)
			move_calc = true;
        else	// doing a drag-zoom
        {
        	mouse_down_x = e.x;
        	mouse_down_y = e.y;
        	drag_zoom = true;
        }
	}

//-------------------------------------------------------------------------------------------------
	public static void LeftMouseClick_1D(MouseEvent e)
	{     
		mouse_down = false;		// mouse is no longer down

		// Util.selectTrace(cursor_x, cursor_y, 0);
		
		
		if(drag_zoom && Math.abs(mouse_down_x - cursor_x) > 10 && ! gp.integ_mode  && ! gp.bpt_mode)
		{		
			do_drag_zoom_1D();
			return;
		}
			
		 if (gp.bpt_mode)
         {
             // Define a baseline point based on cursor 1 frequency
             int i, point1, point2, count;
             double sum;

             if (gp.n_bl_pts >= 199)
             {
                 Util.show_and_log("Too many baseline points");
                 return;
             }
             gp.new_freq1 = gp.freq1 - ((double)e.x * gp.hzperpix);
             gp.baseline_x[gp.n_bl_pts] = (q.offset - gp.new_freq1) / q.hzperpt;
             gp.baseline_sig[gp.n_bl_pts] = 1.0;

             int range = q.SI / 1024;
             if (range < 8) range = 8;

                 point1 = (int) (Math.round(gp.baseline_x[gp.n_bl_pts]) - range);
                 if (point1 < 0) point1 = 0;

                 point2 = (int) (Math.round(gp.baseline_x[gp.n_bl_pts]) + range);
                 if (point2 > q.SI - 1) point2 = q.SI - 1;

             count = 0; sum = 0.0;

             for (i = point1; i <= point2; i++)
             {
                 sum = sum + gp.data_array[i];
                 count++;
             }
             gp.baseline_y[gp.n_bl_pts] = sum / (double)count;
                          
             gp.n_bl_pts++;
                         
             statusStripLabel2.setText(
                 String.format("%d baseline points defined", gp.n_bl_pts));
             update_spect = true;
             canvas.redraw();

             gp.cursor_flag = 0;
             return;
         }	// end gp.bpt_mode
		 		 
		 if (move_expt)				// zero line move of expt. finished, set no cursors,
         {							// refresh and return
             move_expt = false;
             gp.cursor_flag = 0;
             update_spect = true;
             canvas.redraw();
             return;
         }
         if (move_calc)				// zero line move of calc. finished, set no cursors, 
         {							// refresh and return
             move_calc = false;
             gp.cursor_flag = 0;
             update_spect = true;
             canvas.redraw();
             return;
         }
         
         if(gp.adjust_mi)
         {
        	 gp.pp_thresh = (double)(cursor_y - gp.dysize + gp.expt_y_offset)
        			 / -(gp.expt_scale * gp.data_max);
        	 if (gp.pp_thresh > 1.0) gp.pp_thresh = 1.0;

        	 drag_zoom = false;
        	 gp.cursor_flag = 0;
        	 update_spect = true;
        	 canvas.redraw();
        	 return;
         }        
         		
		if(gp.cursor_flag == 0)
		{		
			if (gp.integ_mode)
            {
                int i;

                cursor1_x = e.x;
                gp.new_freq1 = gp.freq1
                    - ((double)e.x * gp.hzperpix);

                for (i = 0; i < gp.n_integrals; i++)
                {
                    if (gp.new_freq1 < gp.integrals[i].start
                        && gp.new_freq1 > gp.integrals[i].end)
                        break;
                }

                if (i == gp.current_integral)
                {
                    gp.current_integral = -1;
                    gp.cursor_flag = 0;
                }
                else if (i < gp.n_integrals)
                {
                    gp.current_integral = i;
                    gp.current_bias = gp.integrals[i].bias;
                    gp.current_slope = gp.integrals[i].slope;
                    gp.cursor_flag = 0;
                    
                    /*if (IntegDialog != null)	// resets sliders to middle on new integral
                    {							// necessary ???
                        SW_main.integ_dialog.BiasTrackBar.Value = 0; // TODO  How???!!!
                        SW_main.integ_dialog.SlopeTrackBar.Value = 0;
                    }*/               
                }
                else
                    gp.cursor_flag = 1;

            }  // end if(gp.integ_mode) 
			else
			{		
				cursor1_x = e.x;
				cursor1_y = e.y;
				gp.new_freq1 = gp.freq1 - ((double)e.x * gp.hzperpix);
				gp.cursor_flag = 1;
			}
		}
		
		else if(gp.cursor_flag == 1)
		{
			if (gp.integ_mode)			// if integrating this defines integral region
            {
                gp.new_freq2 = gp.freq1 - ((double)e.x * gp.hzperpix);
                if (gp.new_freq1 < gp.new_freq2)
                {
                    double temp = gp.new_freq1;
                    gp.new_freq1 = gp.new_freq2;
                    gp.new_freq2 = temp;
                }
                gp.integrals[gp.n_integrals] = new IntegInfo();
                gp.integrals[gp.n_integrals].start = gp.new_freq1;
                gp.integrals[gp.n_integrals].end = gp.new_freq2;
                gp.n_integrals++;
                gp.cursor_flag = 0;
                update_spect = true;
                canvas.redraw();
            }
			else	// normal mode
			{
				cursor2_x = e.x;
				cursor2_y = e.y;
				gp.new_freq2 = gp.freq1 - ((double)e.x * gp.hzperpix);
				gp.cursor_flag = 2;
			}
		}
		
		else if(gp.cursor_flag == 2)
		{
			gp.cursor_flag = 0;
		}
		drag_zoom = false;
	}
	
//-------------------------------------------------------------------------------------------------
	
	public static void LeftMouseClick_FID(MouseEvent e)
	{
		if(drag_zoom && Math.abs(mouse_down_x - cursor_x) > 10)
		{
			mouse_down = false;		// mouse is no longer down
			do_drag_zoom_FID();
			return;
		}
        
		double secperpix = (gp.t2 - gp.t1) / (double)(gp.dxsize - 1);
        if (gp.cursor_flag == 0)
        {
            cursor1_x = e.x;
            gp.new_t1 = gp.t1
                + ((double)e.x * secperpix);
            gp.cursor_flag = 1;
        }

        else if (gp.cursor_flag == 1)
        {
            cursor2_x = e.x;
            gp.new_t2 = gp.t1
                + ((double)e.x * secperpix);
            gp.cursor_flag = 2;
        }
		
		else if(gp.cursor_flag == 2)
		{
			gp.cursor_flag = 0;
		}
        drag_zoom = false;
	}

//-------------------------------------------------------------------------------------------------
	public static void LeftMouseClick_2D()
	{     
		mouse_down = false;		// mouse is no longer down

		if(drag_zoom && Math.abs(mouse_down_x - cursor_x) > 10 
				     && Math.abs(mouse_down_y - cursor_y) > 10)
		{		
			do_drag_zoom_2D();
			return;
		}
		
		// trap any clicks on overlayed traces.  Selects the active trace
		int selection = -1;
		
		if(gp.row_col == SW_main.trace_modes.ROWS)
			selection = Util_2D.selectTrace_direct(gp, q, cursor_x, cursor_y, 10);
		else if(gp.row_col == SW_main.trace_modes.COLUMNS)
			selection = Util_2D.selectTrace_indirect(gp, q_f1, cursor_x, cursor_y, 10);
		
		if(selection != -1 && selection < gp.nslices)
		{
			gp.active_slice = selection;
			gp.cursor_flag = 0;
			drag_zoom = false;
			update_spect = true;
            canvas.redraw();         
            return;
		}
		
//		see if we are outside the data area, but in one of the "projection areas"  if so toggle on
        //	scaling of the projections.  Primarily for those that don't have a mouse wheel
        //  returns true if outside data area, false if within
        if (cursor_x < gp.left_2D
            || cursor_x > gp.xsize_2D + gp.left_2D
            || cursor_y < gp.top_2D
            || cursor_y > gp.dysize - gp.bottom_2D)
        {
            if (gp.rotate_2D)	// silly Varian F1 horizontal mode
            {
                if (gp.f1_trace_on && cursor_x< gp.top_2D)
                {
                    gp.f1_trace_scale_active = true;		// click near F2 trace
                    gp.f2_trace_scale_active = false;
                }
                if (gp.f2_trace_on && cursor_x > gp.xsize_2D + gp.left_2D)
                {
                    gp.f2_trace_scale_active = true;		// near F1 trace
                    gp.f1_trace_scale_active = false;
                }
                update_spect = true;
                canvas.redraw();
            }
            else				// normal F2 horizontal mode
            {
                if (gp.f2_trace_on && cursor_x < gp.top_2D)
                {
                    gp.f2_trace_scale_active = true;		// click near F2 trace
                    gp.f1_trace_scale_active = false;
                }
                if (gp.f1_trace_on && cursor_x > gp.xsize_2D + gp.left_2D)
                {
                    gp.f1_trace_scale_active = true;		// near F1 trace
                    gp.f2_trace_scale_active = false;
                }
            }
            update_spect = true;
            canvas.redraw();
            return;  // outside of data area, so no need to go further
        }

        // if we get here, we are inside data area
        gp.f1_trace_scale_active = false;		// turn off trace scaling	
        gp.f2_trace_scale_active = false;

        if (gp.cursor_flag == 0)				// no cursors on screen
        {
            cursor1_x = cursor_x;
            cursor1_y = cursor_y;

            if (gp.rotate_2D)
            {
                gp.new_freq1_indirect = gp.freq1_indirect
                    - ((double)(cursor_x - gp.left_2D) * gp.hzperpix_f1);
                gp.new_freq1 = gp.freq1
                    - ((double)(gp.dysize - gp.bottom_2D - cursor_y) * gp.hzperpix);
            }
            else
            {
                gp.new_freq1 = gp.freq1
                    - ((double)(cursor_x - gp.left_2D) * gp.hzperpix);
                gp.new_freq1_indirect = gp.freq1_indirect
                    - ((double)(gp.dysize - gp.bottom_2D - cursor_y) * gp.hzperpix_f1);
            }

            gp.cursor_flag = 1;
        }
        
        else if (gp.cursor_flag == 1)			// 1 cursor on screen			
        {
            cursor2_x = cursor_x;
            cursor2_y = cursor_y;

            if (gp.rotate_2D)
            {
                gp.new_freq2_indirect = gp.freq1_indirect
                    - ((double)(cursor_x - gp.left_2D) * gp.hzperpix_f1);
                gp.new_freq2 = gp.freq1
                    - ((double)(gp.dysize - gp.bottom_2D - cursor_y) * gp.hzperpix);
            }
            else
            {
                gp.new_freq2 = gp.freq1
                    - ((double)(cursor_x - gp.left_2D) * gp.hzperpix);
                gp.new_freq2_indirect = gp.freq1_indirect
                    - ((double)(gp.dysize - gp.bottom_2D - cursor_y) * gp.hzperpix_f1);
            }

            gp.cursor_flag = 2;
        }
        
        else if (gp.cursor_flag == 2)			  // 2 cursors on screen
            gp.cursor_flag = 0;					  // so re-set to 0 cursors
        
        drag_zoom = false;
        update_spect = true;
        canvas.redraw();
	}
	
//-------------------------------------------------------------------------------------------------
	public static void MiddleMouseClick()	// 1D and 2D  zoom between cursors
	{		
		if (gp.display_mode == modes.MODE_1D
                && gp.data_array == null && gp.calc_array == null) return;

            if (gp.cursor_flag != 2)
                return;

            if (gp.display_mode == modes.MODE_2D)
            {
            	gp.last_freq1 = gp.freq1;
            	gp.last_freq2 = gp.freq2;
            	gp.last_freq1_indirect = gp.freq1_indirect;
            	gp.last_freq2_indirect = gp.freq2_indirect;

            	gp.freq1 = gp.new_freq1;
            	gp.freq2 = gp.new_freq2;
            	gp.freq1_indirect = gp.new_freq1_indirect;
            	gp.freq2_indirect = gp.new_freq2_indirect;
            	gp.new_2D_view = true;
            }

            else if (gp.display_mode == modes.MODE_1D)	
            {
            	gp.last_freq1 = gp.freq1;
            	gp.last_freq2 = gp.freq2;
            	gp.freq1 = gp.new_freq1;
            	gp.freq2 = gp.new_freq2;
            	slider.setSelection(((int)(100.0 * (q.offset - (gp.freq1 + gp.freq2) / 2.0) /
                        (q.SFO1 * q.SW))));
            }

            else if (gp.display_mode == modes.MODE_FID)
            {
            	gp.t1 = gp.new_t1;
            	gp.t2 = gp.new_t2;
            }

            gp.cursor_flag = 0;
	}

//-------------------------------------------------------------------------------------------------
	public static void RightMouseClick_1D(MouseEvent e)
	{
        if (gp.cursor_flag == 0 && gp.calc_array != null
            && gp.data_array != null			//  right button, assign line	
            && gp.display_mode == SW_main.modes.MODE_1D
            && sp.n_trans != 0)
        {
            if (gp.interp_on == false)
            {
                gp.calc_peaks[gp.trans_index].obs_freq = gp.freq1 - ((double)e.x * gp.hzperpix);
            }
            else
            {
                gp.calc_peaks[gp.trans_index].obs_freq
                = PeakPick.findPeak(gp.freq1 - ((double)e.x * gp.hzperpix), 4.0, gp, q, sp);
            }

            update_spect = true;
            canvas.redraw();         
        }

        else if (gp.cursor_flag == 0 && sp.n_trans == 0
            && gp.data_array != null			//  right button, pick peak	
            && gp.display_mode == SW_main.modes.MODE_1D)
        {      	        	
        	double freq = gp.freq1 - (double)cursor_x * gp.hzperpix;
        	int peak = Util.closestPeak(gp, freq, 5);    
        	
        	if(peak != -1)	// peak exists, so add label
        	{
        		//System.out.println(String.format("right click, freq = %f   peak = %d", freq, peak));

        		PeakLabelDialog dlg = new PeakLabelDialog(shell, SWT.APPLICATION_MODAL, peak);
        		dlg.open();

        		update_spect = true;
        		gp.cursor_flag = 0;
        		canvas.redraw();
        		return;
        	}
                	
            gp.peaks[gp.n_peaks] = new PeakInfo();

            if (gp.interp_on == false)
            {
                gp.peaks[gp.n_peaks].freq = gp.freq1 - ((double)e.x * gp.hzperpix);
                gp.peaks[gp.n_peaks].intensity =
                    gp.data_array[Util.freq_to_index(gp.peaks[gp.n_peaks].freq, q)];
            }
            else
            {
                gp.peaks[gp.n_peaks].freq
                = PeakPick.findPeak(gp.freq1 - ((double)e.x * gp.hzperpix), 4.0, gp, q , sp);
                gp.peaks[gp.n_peaks].intensity =
                    gp.data_array[Util.freq_to_index(gp.peaks[gp.n_peaks].freq, q)];
            }

            gp.n_peaks++;
            PeakPick.savePeaks(gp, q);
            update_spect = true;
            canvas.redraw();
        }
	}
	
//-------------------------------------------------------------------------------------------------
	public static void RightMouseClick_2D(data_par q_ind)
	{
		if (gp.cursor_flag != 0				// Right Click: 2D mode select row or column
				|| gp.rows_real == null		// or label/assign 2D peak
				|| gp.display_mode != SW_main.modes.MODE_2D
				|| cursor_x < gp.left_2D
				|| cursor_x > gp.xsize_2D + gp.left_2D
				|| cursor_y < gp.top_2D
				|| cursor_y > gp.dysize - gp.bottom_2D)
		{
			return;
		}
		
		int index = 0;

		if (gp.row_col == SW_main.trace_modes.ROWS)	// rows
		{

			if (gp.rotate_2D)			// rotated (indirect horizontal) mode
			{
				double freq = gp.freq1 - ((double)(gp.dysize - cursor_y - gp.bottom_2D) 
						* gp.hzperpix);
				index = (int) Math.round((q.offset - freq) / q.hzperpt);
			}
			else						// normal mode
			{
				double freq = gp.freq1_indirect - ((double)(gp.dysize - cursor_y - gp.bottom_2D) 
						* gp.hzperpix_f1);
				index = (int) Math.round((q_ind.offset - freq) / q_ind.hzperpt);
			}
		}

		else if (gp.row_col == SW_main.trace_modes.COLUMNS)	// columns
		{

			if (gp.rotate_2D)			// rotated (indirect horizontal) mode
			{
				double freq = gp.freq1_indirect - ((double)(cursor_x - gp.left_2D) 
						* gp.hzperpix_f1);
				index = (int) Math.round((q_ind.offset - freq) / q_ind.hzperpt);
			}
			else						// normal mode
			{
				double freq = gp.freq1 - ((double)(cursor_x - gp.left_2D) * gp.hzperpix);
				index = (int) Math.round((q.offset - freq) / q.hzperpt) + 1; // why off by 1 
			}
		}
		
		else if (gp.row_col == SW_main.trace_modes.LABEL &&  !gp.rotate_2D
				&& !(assignCOSY || assignTOCSY || assignNOESY || assignHMBC))	// label/assign peak
		{
			double freq = gp.freq1 - ((double)(cursor_x - gp.left_2D) * gp.hzperpix);
			double freq_f1 = gp.freq1_indirect
					- ((double)(gp.dysize - cursor_y - gp.bottom_2D) * gp.hzperpix_f1);

			cur_peak = Util_2D.closestPeak(gp, freq, freq_f1, 5);   // index of peak to edit

			//Dialog creation here
			if (dlg == null)
			{
				dlg = new PeakAssignmentDialog(shell, 
						SWT.NONE | SWT.TITLE | SWT.BORDER, cur_peak, freq, freq_f1);  // create dialog object      
				dlg.open();
			}
		}
		
		else if (gp.row_col == SW_main.trace_modes.LABEL && assignCOSY && !gp.rotate_2D )	// add a COSY correlation
		{
			double freq = gp.freq1 - ((double)(cursor_x - gp.left_2D) * gp.hzperpix);
            double freq_f1 = gp.freq1_indirect
                      - ((double)(gp.dysize - cursor_y - gp.bottom_2D) * gp.hzperpix_f1);
               
            // update peak assignment dialog
            dlg.textCOSY.setText(dlg.textCOSY.getText() + String.format("%.4f, %.4f\r\n", 
            		freq / q.SF, freq_f1 / q_f1.SF));
            
            // parse assignments here so they can be seen immediately
            if(cur_peak == -1)		// a new peak
	            PeakPick.parseAssignments(dlg.textCOSY.getText(), 
						SW_main.gp.peaks[SW_main.gp.n_peaks - 1], corr.COSY);
            else					// editing an old peak
	            PeakPick.parseAssignments(dlg.textCOSY.getText(), 
						SW_main.gp.peaks[cur_peak], corr.COSY);
            
            assignCOSY = false;
		}
		
		else if (gp.row_col == SW_main.trace_modes.LABEL && assignTOCSY && !gp.rotate_2D )	// add a TOCSY correlation
		{
			double freq = gp.freq1 - ((double)(cursor_x - gp.left_2D) * gp.hzperpix);
            double freq_f1 = gp.freq1_indirect
                      - ((double)(gp.dysize - cursor_y - gp.bottom_2D) * gp.hzperpix_f1);
               
            // update peak assignment dialog
            dlg.textTOCSY.setText(dlg.textTOCSY.getText() + String.format("%.4f, %.4f\r\n", 
            		freq / q.SF, freq_f1 / q_f1.SF));
            
            // parse assignments here so they can be seen immediately
            if(cur_peak == -1)		// a new peak
	            PeakPick.parseAssignments(dlg.textTOCSY.getText(), 
						SW_main.gp.peaks[SW_main.gp.n_peaks - 1], corr.TOCSY);
            else					// editing an old peak
	            PeakPick.parseAssignments(dlg.textTOCSY.getText(), 
						SW_main.gp.peaks[cur_peak], corr.TOCSY);
            
            assignTOCSY = false;
		}
		
		else if (gp.row_col == SW_main.trace_modes.LABEL && assignNOESY && !gp.rotate_2D )	// add a NOESY correlation
		{
			double freq = gp.freq1 - ((double)(cursor_x - gp.left_2D) * gp.hzperpix);
            double freq_f1 = gp.freq1_indirect
                      - ((double)(gp.dysize - cursor_y - gp.bottom_2D) * gp.hzperpix_f1);
               
            // update peak assignment dialog
            dlg.textNOESY.setText(dlg.textNOESY.getText() + String.format("%.4f, %.4f\r\n", 
            		freq / q.SF, freq_f1 / q_f1.SF));
            
            // parse assignments here so they can be seen immediately
            if(cur_peak == -1)		// a new peak
	            PeakPick.parseAssignments(dlg.textNOESY.getText(), 
						SW_main.gp.peaks[SW_main.gp.n_peaks - 1], corr.NOESY);
            else					// editing an old peak
	            PeakPick.parseAssignments(dlg.textNOESY.getText(), 
						SW_main.gp.peaks[cur_peak], corr.NOESY);
            
            assignNOESY = false;
		}
		
		else if (gp.row_col == SW_main.trace_modes.LABEL && assignHMBC && !gp.rotate_2D )	// add a NOESY correlation
		{
			double freq = gp.freq1 - ((double)(cursor_x - gp.left_2D) * gp.hzperpix);
            double freq_f1 = gp.freq1_indirect
                      - ((double)(gp.dysize - cursor_y - gp.bottom_2D) * gp.hzperpix_f1);
               
            // update peak assignment dialog
            dlg.textHMBC.setText(dlg.textHMBC.getText() + String.format("%.4f, %.4f\r\n", 
            		freq / q.SF, freq_f1 / q_f1.SF));
            
            // parse assignments here so they can be seen immediately
            if(cur_peak == -1)		// a new peak
	            PeakPick.parseAssignments(dlg.textHMBC.getText(), 
						SW_main.gp.peaks[SW_main.gp.n_peaks - 1], corr.HMBC);
            else					// editing an old peak
	            PeakPick.parseAssignments(dlg.textHMBC.getText(), 
						SW_main.gp.peaks[cur_peak], corr.HMBC);
            
            assignHMBC = false;
		}
						
		if (gp.row_col != SW_main.trace_modes.LABEL)	// not in label mode, update traces
        {
            gp.row_scale[gp.nslices] = gp.expt_scale;
            gp.active_slice = gp.nslices;

            if (gp.nslices < 32)
            {
                gp.slice_no[gp.nslices] = index;
                gp.nslices++;
            }
        }
		
		update_spect = true;
        canvas.redraw();
	}

//-------------------------------------------------------------------------------------------------
	public static void MouseMove(MouseEvent e) 
	{	
		if (!(gp.data_array != null || gp.calc_array != null || gp.fid != null) 
                || gp.phase_mode || gp.bpt_mode) 
		        return;

        if (gp.rows_real == null && gp.display_mode == modes.MODE_2D)
                return;
            
		if(gp.display_mode == modes.MODE_1D)
		{	 
			cursor_x = e.x; cursor_y = e.y;
			canvas.setFocus();		// to enable mouse wheel events
			canvas.redraw();
		}
		
		else if (gp.display_mode == modes.MODE_FID)
		{
			cursor_x = e.x; cursor_y = e.y;
			canvas.setFocus();		// to enable mouse wheel events
			canvas.redraw();		
		}
		
		else if (gp.display_mode == modes.MODE_2D)
		{
			cursor_x = e.x; cursor_y = e.y;
			canvas.setFocus();		// to enable mouse wheel events
			canvas.redraw();		
		}
	}
	
//-------------------------------------------------------------------------------------------------
	public static void do_mouse_wheel_1D(MouseEvent e)
	{
		if (gp.data_array == null && gp.calc_array == null) return;
		if (gp.show_spectrum == false) return;

		if (e.y < gp.dysize / 2 && gp.calc_array != null && gp.show_simulated)
		{
			/*if (sp.calc_mode == sim_par.DECON)
			{
				if (e.count > 0) gp.expt_scale *= 1.10;
				else gp.expt_scale /= 1.10;
			}*/
			//else
			//{
				if (e.count > 0) gp.calc_scale *= 1.10;
				else gp.calc_scale /= 1.10;
			//}
		}
		else
		{
			if (e.count > 0) gp.expt_scale *= 1.10;
			else gp.expt_scale /= 1.10;
		}

		if (gp.phase_mode || gp.integ_mode)  // if in phase or integration modes 
		{
			update_spect = true;
			canvas.redraw();       			 // complete re-draw         
		}

		else                             	 // otherwise, write directly to screen
		{
			update_spect = true;
			canvas.redraw();
		}
	}

//-------------------------------------------------------------------------------------------------
    public static void do_mouse_wheel_FID(MouseEvent e)
    {
        if (e.count > 0) gp.fid_scale *= 1.10;
        else gp.fid_scale /= 1.10;

        update_spect = true;
        canvas.redraw();
    }

//-------------------------------------------------------------------------------------------------
    public static void do_mouse_wheel_2D(MouseEvent e)
    {
// in 2D data area, adjust vertical scaling (contour levels)
        if (e.y > gp.top_2D && e.y < (gp.dysize - gp.top_2D)
            && e.x > gp.left_2D && e.x < (gp.dxsize - gp.right_2D))
        {
            if (e.count < 0)
            {
                Util_2D.rescale_floor_2D(1.20, gp);
                Util_2D.rescale_ceiling_2D(1.20, gp);
            }
            else
            {
                Util_2D.rescale_floor_2D(0.8333, gp);
                Util_2D.rescale_ceiling_2D(0.8333, gp);
            }

            gp.new_2D_view = true;	// need to re-contour
            update_spect = true;
            canvas.redraw();
            return;
        }  
        
//  outside data area, adjust 1D traces if present
//	re-draw just the 1D trace area of the spectrum
        if (!gp.rotate_2D)	// normal F2 horizontal mode
        {       	
        	if (gp.f2_trace_on && cursor_y < (gp.top_2D + 150)
        			&& cursor_x < (gp.xsize_2D + gp.left_2D))
        	{
        		if (e.count < 0)
        			gp.f2_trace_scale = gp.f2_trace_scale / 1.10;
        		else if (e.count > 0)
        			gp.f2_trace_scale = gp.f2_trace_scale * 1.10;

        		update_spect = true;
                canvas.redraw();
        	}
        	else if (gp.f1_trace_on && cursor_y > gp.top_2D && cursor_x > (gp.xsize_2D + gp.left_2D))
        	{
        		if (e.count < 0)
        			gp.f1_trace_scale = gp.f1_trace_scale / 1.10;
        		else if (e.count > 0)
        			gp.f1_trace_scale = gp.f1_trace_scale * 1.10;

        		update_spect = true;
                canvas.redraw();
        	}

        	else				// rotated (F1 horizontal) mode
        	{
        		if (gp.f1_trace_on && cursor_y < (gp.top_2D + 150)
        				&& cursor_x < (gp.xsize_2D + gp.left_2D))
        		{
        			if (e.count < 0)
        				gp.f1_trace_scale = gp.f1_trace_scale / 1.10;
        			else if (e.count > 0)
        				gp.f1_trace_scale = gp.f1_trace_scale * 1.10;

        			update_spect = true;
                    canvas.redraw();
        		}
        		else if (gp.f2_trace_on && cursor_y > gp.top_2D && cursor_x > (gp.xsize_2D + gp.left_2D))
        		{
        			if (e.count < 0)
        				gp.f2_trace_scale = gp.f2_trace_scale / 1.10;
        			else if (e.count > 0)
        				gp.f2_trace_scale = gp.f2_trace_scale * 1.10;

        			update_spect = true;
                    canvas.redraw();
        		}
        	}
        }
     }
   
 //-------------------------------------------------------------------------------------------------
    public static void do_mouse_double_click(MouseEvent e)
    {        	   	
    	// curently unused
    }
//-------------------------------------------------------------------------------------------------
//	This is ugly and should be split up
	public static void drawCursors(PaintEvent e, data_par q_ind)
	{
		int i = 0, j = 0;
		
		// set-up font and colours
		Font font = new Font(shell.getDisplay(), "Arial", SW_main.SMALL_FONT, SWT.NORMAL);						
		e.gc.setForeground(gp.cursor_colour); 						
		e.gc.setBackground(gp.back_colour);	
		e.gc.setFont(font);
		
		if (gp.display_mode == modes.MODE_1D && 
		   (gp.data_array != null || gp.calc_array != null))
		{
			if(drag_zoom && !gp.integ_mode) // drag zoom mode
			{			
				e.gc.drawLine(mouse_down_x, 2, mouse_down_x, gp.dysize - gp.axis_offset - 2);
				e.gc.drawLine(cursor_x, 2, cursor_x, gp.dysize - gp.axis_offset - 2);
				e.gc.drawLine( cursor_x, gp.dysize - gp.axis_offset - 2,
						mouse_down_x, gp.dysize - gp.axis_offset - 2);
				e.gc.drawLine(cursor_x, 2, mouse_down_x, 2);
				String msg = String.format("delta = %.3f Hz  (= %.4f ppm)",
						Util.pt_to_freq(cursor_x, gp) - Util.pt_to_freq(mouse_down_x, gp), 
						(Util.pt_to_freq(cursor_x, gp) - Util.pt_to_freq(mouse_down_x, gp)) / q.SF);
				
				Point size = e.gc.textExtent(msg);
								
				int x  = (mouse_down_x + cursor_x) / 2 - size.x / 2;
				if (x < 154) x = 154;
				e.gc.drawText(msg,  x, 150);
			}	// end 1D drag zoom
			
			else if(move_expt && !gp.integ_mode)	// move experimental
			{
				gp.expt_y_offset = (gp.dysize - cursor_y);
                e.gc.drawLine(0, cursor_y, gp.dxsize, cursor_y);
			}
			
			else if(move_calc && !gp.integ_mode)	// move simulated
			{
				gp.calc_y_offset = (gp.dysize - cursor_y);
                e.gc.drawLine(0, cursor_y, gp.dxsize, cursor_y);
			}
			
			else	// normal cursor mode, 1D
			{
				i = (int) Math.round((q.offset
						- (gp.freq1 - (double)cursor_x * gp.hzperpix)) / q.hzperpt);
				if(i < 0) i = 0;
				else if(i >= q.SI) i = q.SI -1;

				e.gc.drawLine(cursor_x, 0, cursor_x, gp.dysize);
				String msg;
				
				if(gp.calc_array != null && gp.data_array == null)	// TODO cleaner way to do this??
				{ 	// calculated data only
					if (gp.axis_unit == SW_main.units.HZ)
						msg = String.format("shift = %.3f Hz   inten. = %6.4f   abs. = %d",
								Util.pt_to_freq(cursor_x, gp), gp.calc_array[i] / gp.calc_max,
								(int)gp.calc_array[i]);
					else
						msg = String.format("shift = %.4f ppm   inten. = %6.4f   abs. = %d",
								(Util.pt_to_freq(cursor_x, gp) / q.SF), (gp.calc_array[i] / gp.calc_max),
								(int)gp.calc_array[i]);
				}
				else
				{	// experimental data (and possibly calculated data) are present
					if (gp.axis_unit == SW_main.units.HZ)
						msg = String.format("shift = %.3f Hz   inten. = %6.4f   abs. = %d",
								Util.pt_to_freq(cursor_x, gp), gp.data_array[i] / gp.data_max,
								(int)gp.data_array[i]);
					else
						msg = String.format("shift = %.4f ppm   inten. = %6.4f   abs. = %d",
								(Util.pt_to_freq(cursor_x, gp) / q.SF), (gp.data_array[i] / gp.data_max),
								(int)gp.data_array[i]);
				}
				
				e.gc.drawText(msg, 2, 80);

				if(gp.cursor_flag == 0)
					e.gc.drawText(String.format("frequency = %f MHz", Util.pt_to_freq(cursor_x, gp) / 1.0E6 + q.SF), 
							2, 100);
				
				for(int k = 0; k < MAX_CHILD_WINDOWS; k++)
				{
					if(childWindow[k] != null && childWindowActive[k])
					{
						try{ childWindow[k].updateCursors(Util.pt_to_freq(cursor_x, gp), 0.0); }
						catch(Exception ex){ex.printStackTrace();}
					}
				}
			}
		}	// end mode 1D
		
		// now FID mode
		else if (gp.display_mode == modes.MODE_FID)
		{
			if(drag_zoom) // drag zoom mode
			{
				e.gc.drawLine(mouse_down_x, 2, mouse_down_x, gp.dysize - gp.axis_offset - 2);
				e.gc.drawLine(cursor_x, 2, cursor_x, gp.dysize - gp.axis_offset - 2);
				e.gc.drawLine( cursor_x, gp.dysize - gp.axis_offset - 2,
						mouse_down_x, gp.dysize - gp.axis_offset - 2);
				e.gc.drawLine(cursor_x, 2, mouse_down_x, 2);
			}	//	end drag zoom
			else	// normal cursor mode
			{
				e.gc.drawLine(cursor_x, 0, cursor_x, gp.dysize);
				double secperpix = (gp.t2 - gp.t1) / (double) (gp.dxsize - 1);
				String msg = String.format("Time = %f Sec.", (gp.t1 + (double) cursor_x * secperpix));
				e.gc.drawText(msg, 2, 80);
			}
		}	// end mode FID
		
		
		// now 2D mode
		else if (gp.display_mode == modes.MODE_2D && gp.rows_real != null
				&& cursor_x > gp.left_2D && cursor_x < gp.dxsize - gp.right_2D
				&& cursor_y > gp.top_2D && cursor_y < gp.dysize - gp.bottom_2D)
		{
			if(drag_zoom)
			{
				e.gc.drawLine(mouse_down_x, gp.top_2D, mouse_down_x, gp.dysize - gp.bottom_2D);
				e.gc.drawLine(gp.left_2D, mouse_down_y, gp.dxsize - gp.right_2D, mouse_down_y);
				
				e.gc.drawLine(cursor_x, gp.top_2D, cursor_x, gp.dysize - gp.bottom_2D);
				e.gc.drawLine(gp.left_2D, cursor_y, gp.dxsize - gp.right_2D, cursor_y);
			}
			else
			{
				if(!gp.rotate_2D)  // normal orientation
				{
					i = (int) Math.round((q.offset
							- (gp.freq1 - (double)(cursor_x - gp.left_2D) * gp.hzperpix)) / q.hzperpt);
					if (i < 0) i = 0; if (i >= q.SI) i = q.SI - 1;

					j = (int) Math.round((q_ind.offset
							- (gp.freq1_indirect - (double)(gp.dysize - gp.bottom_2D - cursor_y) 
									* gp.hzperpix_f1)) / q_ind.hzperpt);
					if (j < 0) j = 0; if (j >= q_ind.SI) j = q_ind.SI - 1;		
					
					

		            //	draw F1 and/or F2 traces
		            if(q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XY)
		                drawTraces(e.gc, i, j, q_f2);
		            else	// 2D or 3D XZ
		                drawTraces(e.gc, i, j, q_f1);
		            
				}
				else				// rotated orientation
				{
					i = (int) Math.round((q_ind.offset
	                        - (gp.freq1_indirect - (double)(cursor_x - gp.left_2D) * gp.hzperpix_f1)) / q_ind.hzperpt);
	                    if (i < 0) i = 0; if (i >= q_ind.SI) i = q_ind.SI - 1;

                    j = (int) Math.round((q.offset
                        - (gp.freq1 - (double)(gp.dysize - gp.bottom_2D - cursor_y) 
                        * gp.hzperpix)) / q.hzperpt);

                    if (j < 0) j = 0; if (j >= q.SI) j = q.SI - 1;
                    
//                	draw F1 and/or F2 traces
		            if(q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XY)
		                drawTraces(e.gc, i, j, q_f2);
		            else	// 2D or 3D XZ
		                drawTraces(e.gc, i, j, q_f1);
				}
				
				e.gc.setForeground(gp.cursor_colour);
				e.gc.drawLine(cursor_x, gp.top_2D, cursor_x, gp.dysize - gp.bottom_2D);
				e.gc.drawLine(gp.left_2D, cursor_y, gp.dxsize - gp.right_2D, cursor_y);
				
				String msg = "";
				
				// decide on correct cursor labeling depending on mode
				// this really needs to be cleaned-up
				if(!gp.rotate_2D)  // normal orientation
				{
					if (gp.axis_unit == SW_main.units.HZ)
					{
						if(q.data_mode == modes.MODE_2D)
							msg = String.format(
									"F2 (Hz) = %.3f   F1 (Hz) = %.3f   inten. = %.3f  index: F2 = %d  F1 = %d",
									(gp.freq1 - (double)(cursor_x - gp.left_2D) * gp.hzperpix),
									(gp.freq1_indirect - (double)(gp.dysize - gp.bottom_2D - cursor_y) 
											* gp.hzperpix_f1),
									(gp.rows_real[j][i] / 1000.0), i, j);
						
						else if(q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XY)
							msg = String.format(
									"F3 (Hz) = %.3f   F2 (Hz) = %.3f   inten. = %.3f  index: F3 = %d  F2 = %d",
									(gp.freq1 - (double)(cursor_x - gp.left_2D) * gp.hzperpix),
									(gp.freq1_indirect - (double)(gp.dysize - gp.bottom_2D - cursor_y) 
											* gp.hzperpix_f1),
									(gp.rows_real[j][i] / 1000.0), i, j);
						
						else if(q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XZ)
							msg = String.format(
									"F3 (Hz) = %.3f   F1 (Hz) = %.3f   inten. = %.3f  index: F2 = %d  F1 = %d",
									(gp.freq1 - (double)(cursor_x - gp.left_2D) * gp.hzperpix),
									(gp.freq1_indirect - (double)(gp.dysize - gp.bottom_2D - cursor_y) 
											* gp.hzperpix_f1),
									(gp.rows_real[j][i] / 1000.0), i, j);
					}	// Hz
					else	// PPM
					{
						if(q.data_mode == modes.MODE_2D)
							msg = String.format(
									"F2 (ppm) = %.4f   F1 (ppm) = %.4f  inten. = %.3f  index: F2 = %d  F1 = %d",
									((gp.freq1 - (double)(cursor_x- gp.left_2D) * gp.hzperpix) / q.SF),
									((gp.freq1_indirect - (double)(gp.dysize - gp.bottom_2D - cursor_y) 
											* gp.hzperpix_f1) / q_ind.SF),
									(gp.rows_real[j][i] / 1000.0), i, j);
						
						else if(q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XY)
							msg = String.format(
									"F3 (ppm) = %.4f   F2 (ppm) = %.4f  inten. = %.3f  index: F3 = %d  F2 = %d",
									((gp.freq1 - (double)(cursor_x- gp.left_2D) * gp.hzperpix) / q.SF),
									((gp.freq1_indirect - (double)(gp.dysize - gp.bottom_2D - cursor_y) 
											* gp.hzperpix_f1) / q_ind.SF),
									(gp.rows_real[j][i] / 1000.0), i, j);
						
						else if(q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XZ)
							msg = String.format(
									"F3 (ppm) = %.4f   F1 (ppm) = %.4f  inten. = %.3f  index: F3 = %d  F1 = %d",
									((gp.freq1 - (double)(cursor_x- gp.left_2D) * gp.hzperpix) / q.SF),
									((gp.freq1_indirect - (double)(gp.dysize - gp.bottom_2D - cursor_y) 
											* gp.hzperpix_f1) / q_ind.SF),
									(gp.rows_real[j][i] / 1000.0), i, j);
					}	// PPM
				}
				else				// (indirect horizontal) orientation
				{
					 if (gp.axis_unit == SW_main.units.HZ)
					 {
						 if(q.data_mode == modes.MODE_2D)
		                    msg = String.format("F1 (Hz) = %.3f   F2 (Hz) = %.3f   inten. = %.3f    index: F1 = %d F2 = %d",
		                        (gp.freq1_indirect - (double)(cursor_x- gp.left_2D) * gp.hzperpix_f1),
		                        (gp.freq1 - (double)(gp.dysize - gp.bottom_2D - cursor_y) * gp.hzperpix),
		                        (gp.rows_real[i][j] / 1000.0), i, j);
						 
						 else if(q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XY)
							 msg = String.format("F2 (Hz) = %.3f   F3 (Hz) = %.3f   inten. = %.3f    index: F2 = %d F3 = %d",
		                        (gp.freq1_indirect - (double)(cursor_x- gp.left_2D) * gp.hzperpix_f1),
		                        (gp.freq1 - (double)(gp.dysize - gp.bottom_2D - cursor_y) * gp.hzperpix),
		                        (gp.rows_real[i][j] / 1000.0), i, j);
						 
						 else if(q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XZ)
							 msg = String.format("F1 (Hz) = %.3f   F3 (Hz) = %.3f   inten. = %.3f    index: F1 = %d F3 = %d",
		                        (gp.freq1_indirect - (double)(cursor_x- gp.left_2D) * gp.hzperpix_f1),
		                        (gp.freq1 - (double)(gp.dysize - gp.bottom_2D - cursor_y) * gp.hzperpix),
		                        (gp.rows_real[i][j] / 1000.0), i, j);
					 }	// Hz
	                 else	//PPM
	                 {
	                	 if(q.data_mode == modes.MODE_2D)
		                    msg = String.format("F1 (ppm) = %.4f   F2 (ppm) = %.4f  inten. = %.3f  index: F1 = %d  F2 = %d",
		                        ((gp.freq1_indirect - (double)(cursor_x - gp.left_2D) * gp.hzperpix_f1) / q_ind.SF),
		                        ((gp.freq1 - (double)(gp.dysize - gp.bottom_2D - cursor_y) * gp.hzperpix) / q.SF),
		                        (gp.rows_real[i][j] / 1000.0), i, j);
	                	 
	                	 else if(q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XY)
	                		 msg = String.format("F2 (ppm) = %.4f   F3 (ppm) = %.4f  inten. = %.3f  index: F2 = %d  F3 = %d",
 		                        ((gp.freq1_indirect - (double)(cursor_x - gp.left_2D) * gp.hzperpix_f1) / q_ind.SF),
 		                        ((gp.freq1 - (double)(gp.dysize - gp.bottom_2D - cursor_y) * gp.hzperpix) / q.SF),
 		                        (gp.rows_real[i][j] / 1000.0), i, j);
	                	 
                		 else if(q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XZ)
                			 msg = String.format("F1 (ppm) = %.4f   F3 (ppm) = %.4f  inten. = %.3f  index: F1 = %d  F3 = %d",
 		                        ((gp.freq1_indirect - (double)(cursor_x - gp.left_2D) * gp.hzperpix_f1) / q_ind.SF),
 		                        ((gp.freq1 - (double)(gp.dysize - gp.bottom_2D - cursor_y) * gp.hzperpix) / q.SF),
 		                        (gp.rows_real[i][j] / 1000.0), i, j);
	                 }
				}
				//	finally, draw the information				
				e.gc.drawText(msg, gp.left_2D + 5, 60);	
				if(assignCOSY || assignTOCSY || assignNOESY || assignHMBC)
					e.gc.drawText("right click on \ncorrrelated peak", cursor_x + 10, cursor_y + 10);	
				
				for(int k = 0; k < MAX_CHILD_WINDOWS; k++)
				{
					if(childWindow[k] != null && childWindowActive[k])  // TODO move within if(rotate2D)...   block
					{	
						try{ childWindow[k].updateCursors((gp.freq1 - (double)(cursor_x - gp.left_2D) * gp.hzperpix), 
								(gp.freq1_indirect - (double)(gp.dysize - gp.bottom_2D - cursor_y) * gp.hzperpix_f1)); }
						catch(Exception ex){ex.printStackTrace();}
					}
				}	// end updating child windows
			}	// end normal cursor mode
		}	// end 2D
		
		//  2D mode, but in trace area, give hint on scaling trace
		else if (gp.display_mode == modes.MODE_2D && gp.rows_real != null
				&& gp.f2_trace_on && gp.f2_from_scan
				&& cursor_x < gp.dxsize - gp.right_2D
				&& cursor_y < gp.top_2D)
		{
			e.gc.drawText("use mouse wheel to scale trace", gp.left_2D + 5, 60);
		}
		
		else if (gp.display_mode == modes.MODE_2D && gp.rows_real != null
				&& gp.f1_trace_on && gp.f1_from_scan
				&& cursor_x > (gp.dxsize - gp.right_2D)
				&& cursor_y > gp.top_2D && cursor_y < gp.dysize - gp.bottom_2D)
		{
			e.gc.drawText("use mouse wheel\nto scale trace", 
					gp.dxsize - gp.right_2D + 5, gp.top_2D);
		}

		if (gp.cursor_flag == 1)
		{
			if (gp.display_mode == modes.MODE_1D)
			{
				String msg = String.format( "delta = %.4f Hz  (= %.4f ppm)",
						gp.new_freq1 - Util.pt_to_freq(cursor_x, gp), 
						(gp.new_freq1 - Util.pt_to_freq(cursor_x, gp)) / q.SF);
				e.gc.drawText(msg, 2, 100);
			}
			else if (gp.display_mode == modes.MODE_FID)
			{
				double secperpix = (gp.t2 - gp.t1) / (double)(gp.dxsize - 1);

				double delta = (gp.t1 + (double)cursor_x * secperpix) - gp.new_t1;
				String msg;

				if (Math.abs(delta) > 0.1)
					msg = String.format("delta = %.5f Sec.  1/delta = %.3f Hz",
							delta, Math.abs(1.0 / delta));

				else if (Math.abs(delta) <= 0.1 && delta > 0.001)
					msg = String.format("delta = %.3f ms.  1/delta = %.1f Hz",
							1000.0 * delta, Math.abs(1.0 / delta));

				else
					msg = String.format("delta = %.2f us.  1/delta = %.1f} Hz",
							1000000.0 * delta, Math.abs(1.0 / delta));

				e.gc.drawText(msg, 2, 100);
			}
		}  // if cursor_flag == 1
		
		else if (gp.cursor_flag == 2 && gp.display_mode == modes.MODE_1D)
		{
			String msg = String.format("delta = %.3f Hz  (= %.4f ppm)",
					gp.new_freq1 - gp.new_freq2, (gp.new_freq1 - gp.new_freq2) / q.SF);
			e.gc.drawText(msg, 2, 100);
		} // if cursor_flag == 2		

		else if(gp.display_mode == modes.MODE_2D)
		{
			// TODO: 2D/3D cursor display here  necessary??
		}
		
		// clean-up
		font.dispose();
	}

//-------------------------------------------------------------------------------------------------
	private static void do_drag_zoom_1D()
    {
		if(gp.data_array == null && gp.calc_array == null)
			return;
		
        gp.cursor_flag = 0;             // no cursors on screen 
        gp.last_freq1 = gp.freq1;       // remember current limits for
        gp.last_freq2 = gp.freq2;       // "last" function
        double left = gp.freq1;
        gp.freq1 = left - (double)mouse_down_x * gp.hzperpix;
        gp.freq2 = left - (double)cursor_x * gp.hzperpix;
        slider.setSelection((int)(100.0 * (q.offset - (gp.freq1 + gp.freq2) / 2.0) /
				(q.SFO1 * q.SW)));

        drag_zoom = false;
        update_spect = true;
        canvas.redraw();
    }

//-------------------------------------------------------------------------------------------------
	private static void do_drag_zoom_FID()
    {
		double secperpix = (gp.t2 - gp.t1) / (double)(gp.dxsize - 1);
        gp.cursor_flag = 0;             // no cursors on screen 

        gp.new_t1 = gp.t1 + ((double)mouse_down_x * secperpix);
        gp.t2 = gp.t1 + ((double)cursor_x * secperpix);
        gp.t1 = gp.new_t1;

        slider.setSelection((int)(100.0 * (gp.t1 + gp.t2) / 2.0 / (q.dwell * q.TD / 2)));

        drag_zoom = false;
        update_spect = true;
        canvas.redraw(); 
    }
	
//-------------------------------------------------------------------------------------------------
	private static void do_drag_zoom_2D()
	{	
		 // bail out if user tries to drag-zoom outside of data area
        if (cursor_x < gp.left_2D || cursor_x > (gp.dxsize - gp.right_2D)
            || cursor_y < gp.top_2D || cursor_y > (gp.dysize - gp.bottom_2D)
            || mouse_down_x < gp.left_2D || mouse_down_x > (gp.dxsize - gp.right_2D)
            || mouse_down_y < gp.top_2D || mouse_down_y > (gp.dysize - gp.bottom_2D))
            {
                return;       
            }
        
		 gp.cursor_flag = 0;                          // no cursors on screen 
         gp.last_freq1 = gp.freq1;                    // remember current limits for
         gp.last_freq2 = gp.freq2;                    // "last" function
         gp.last_freq1_indirect = gp.freq1_indirect;  // remember current limits for
         gp.last_freq2_indirect = gp.freq2_indirect;  // "last" function
         
		if(!gp.rotate_2D)	// normal orientation
		{
			double left = gp.freq1;
			double bottom = gp.freq1_indirect;
			
			gp.freq1 = left - ((double)mouse_down_x - gp.left_2D)
					* gp.hzperpix;
			gp.freq2 = left - ((double)cursor_x - gp.left_2D)
					* gp.hzperpix;

			gp.freq1_indirect = bottom - (double)(gp.dysize - gp.bottom_2D - mouse_down_y)
					* gp.hzperpix_f1;
			gp.freq2_indirect = bottom - (double)(gp.dysize - gp.bottom_2D - cursor_y)
					* gp.hzperpix_f1;
		}
		else	// rotated (silly Varian F1 horizontal mode)
		{
			 double left = gp.freq1_indirect;
             double bottom = gp.freq1;
             gp.freq1_indirect = left - ((double)mouse_down_x - gp.left_2D)
                 * gp.hzperpix_f1;
             gp.freq2_indirect = left - ((double)cursor_x - gp.left_2D)
                 * gp.hzperpix_f1;

             gp.freq1 = bottom - (double)(gp.dysize - gp.bottom_2D - mouse_down_y)
                 * gp.hzperpix;
             gp.freq2 = bottom - (double)(gp.dysize - gp.bottom_2D - cursor_y)
                 * gp.hzperpix;
		}

		drag_zoom = false;
		gp.new_2D_view = true;
		update_spect = true;
		canvas.redraw();
	}
	
//-------------------------------------------------------------------------------------------------
//	Draw real time rows or columns along F1 or F2 of 2D spectrum

	private static void drawTraces(GC gr, int i, int j, data_par q_ind)
	{
		if(gp.f2_trace_on && gp.f2_from_scan)
		{
			
			if(!gp.rotate_2D)  // normal orientation
			{
				Util_2D.copy_F2_to_trace(j, gp, q);
				GraphUtil2D.drawF2On2D(gr, gp.ysize_2D + 20, gp);
			}
			else 			 // indirect horizontal
			{
				Util_2D.copy_F2_to_trace(i, gp, q);		// TO DO, etc.
				GraphUtil2D.drawF2On2D_rotated(gr, gp.xsize_2D + 20, gp);
			}
		}

		if(gp.f1_trace_on && gp.f1_from_scan)
		{
			if(!gp.rotate_2D)  	// normal orientation
			{
				Util_2D.copy_F1_to_trace(i, gp, q_ind);
				GraphUtil2D.drawF1On2D(gr, gp.xsize_2D + 20, gp);
			}
			else  				// indirect horizontal
			{				
				Util_2D.copy_F1_to_trace(j, gp, q_ind);
				GraphUtil2D.drawF1On2D_rotated(gr, gp.ysize_2D + 20, gp);
			}		
		}
	}
	
//-------------------------------------------------------------------------------------------------
}	// end class
