package spinworks;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.widgets.Display;

import spinworks.SW_main.modes;
import spinworks.SW_main.modes_3D;

/**
 * provides utilities to save a spectrum as PNG or JPEG
 * @author marat
 *
 */
public class PNG_Util 
{
	static int png_dpi = 600;
	static double printScaleFactor;
	static double fontScaleFactor;
	
	/**
	 * Print the spectrum graphics to a file PNG or JPEG according to type
	 * @param type SWT.IMAGE_PNG or SWT.IMAGE_JPEG
	 */
	public static void doPrintToFile(String fileName, int type) 
	{
		Display display = Display.getDefault();
		Image image = new Image(display, 6300, 4800);
		GC gc = new GC(image);
		gc.setBackground(display.getSystemColor(SWT.COLOR_WHITE));	
		gc.fillRectangle(image.getBounds());	// necessary??

		printScaleFactor = png_dpi / 25.4;
		fontScaleFactor = printScaleFactor * 0.78;  // bit of a kluge

		// default colours if user hasn't chosen others
		//	TODO initialize elsewhere (in gp!)
		if(SW_main.gp.print_spect_colour == null)
			SW_main.gp.print_spect_colour = new Color(gc.getDevice(), 0, 0, 0);

		if(SW_main.gp.print_integ_colour == null)
			SW_main.gp.print_integ_colour = new Color(gc.getDevice(), 0, 0, 0);

		if(SW_main.gp.print_axis_colour == null)
			SW_main.gp.print_axis_colour = new Color(gc.getDevice(), 0, 0, 0);

		 if(SW_main.gp.display_mode == modes.MODE_1D)
		 {
			 doPrint1D(gc);
		 }
		 
		 else if (SW_main.gp.display_mode == modes.MODE_FID)
		 {
			 // TODO
		 }
		 
		 else  // must be 2D or 3D
		 {
			 doPrint2D(gc);
		 }
		 		
		// create the image file
		ImageLoader loader = new ImageLoader();
		loader.data = new ImageData[] {image.getImageData()};		
			
		try	{	loader.save(fileName, type);	}
		catch(Exception ex)
		{
			Util.show_and_log("Error saving PNG/JPEG file: " + fileName, ex.getMessage());
			ex.printStackTrace();
		}

		System.out.println("PNG file saved to: " + fileName);
		
		gc.dispose();
		image.dispose();
	}
	
//-------------------------------------------------------------------------------------------------
	private static void doPrint1D(GC gc)
	{
		// some useful values
		double scale_factor = SW_main.gp.plot_ysize/((double)SW_main.gp.dysize);
        
        int points = 
				Math.abs((int)((SW_main.q.offset - SW_main.gp.freq1) / SW_main.q.hzperpt) 
				       - (int)((SW_main.q.offset - SW_main.gp.freq2) / SW_main.q.hzperpt));     
		
        double thresh;
        if (points > 40000)
       	 thresh = 0.1;
        else
       	 thresh = 0.02;
        
        double int_off;

        if (SW_main.gp.plot_integral_values && SW_main.gp.n_integrals != 0)	// add space for integral
        	int_off = 10.0;								    // values if necessary
        else
        	int_off = 0.0;
			
//    	print any stacked traces
// 		do this first, so any whitewashing won't cover up other objects
        for (int i = SW_main.gp.n_traces -1; i >= 0 ; i--)
        {
        	int j; 
        	
        	if (SW_main.gp.plot_spectrum == false)
        		j = i;
        	else
        		j = i + 1;

        	points =
        			Math.abs((int)((SW_main.gp.stack_trace[i].offset - SW_main.gp.freq1) 
        					       / SW_main.gp.stack_trace[i].hzperpt)
        					- (int)((SW_main.gp.stack_trace[i].offset - SW_main.gp.freq2) 
        							/ SW_main.gp.stack_trace[i].hzperpt));

        	if (points > 40000)
        		thresh = 0.1;
        	else
        		thresh = 0.02;

        	double xsize;

        	if (SW_main.gp.stack_x_size_auto)
        		xsize = SW_main.gp.plot_xsize;
        	else
        		xsize = SW_main.gp.stack_x_size * scale_factor;
        	
        	gc.setLineWidth((int)(SW_main.gp.spect_pen_thickness * printScaleFactor * 1.7));
        	
        	PrintUtil.print_spect(gc, SW_main.gp.stack_trace[i].print_colour,
        			SW_main.gp.freq1, SW_main.gp.freq2,
        			SW_main.gp.stack_trace[i].yscale * scale_factor,
        			(SW_main.gp.expt_y_offset + SW_main.gp.stack_y_offset * (j) 
        					- SW_main.gp.axis_offset) * scale_factor + int_off,
        			SW_main.gp.stack_trace[i].data, thresh,
        			xsize, SW_main.gp.plot_ysize,
        			SW_main.gp.plot_xoffset + (SW_main.gp.stack_x_offset * (j)) * scale_factor, 
        			SW_main.gp.plot_yoffset, 
        			printScaleFactor, SW_main.gp, SW_main.q);
        	      	
        	double off = SW_main.gp.stack_y_offset * j * scale_factor;       	       				
        	String label;

        	if (SW_main.gp.stack_label_type == 0)
        		label = "";
        	else if (SW_main.gp.stack_label_type == 1)
        		label = String.format("%d", i + 1);
        	else if (SW_main.gp.stack_label_type == 2)
        		label = SW_main.gp.stack_trace[i].title;
        	else if (SW_main.gp.stack_label_type == 3)
        		label = SW_main.gp.stack_trace[i].folder;
        	else
        		label = "";
        	
        	gc.drawText(label, 
        			(int)((SW_main.gp.plot_xoffset + (SW_main.gp.stack_x_offset * j * scale_factor)) 
        					* printScaleFactor),
        			(int)((SW_main.gp.plot_ysize /*- SW_main.gp.plot_yoffset*/ - off) * printScaleFactor));
        	
        }	// for i     
        
		 if(SW_main.gp.plot_axis)			// Plot the axis
		 {
			 Font axisFont = new Font(gc.getDevice(), "Arial", 
					 (int)(SW_main.gp.axis_font_height * fontScaleFactor), 
					 SWT.NORMAL);  	
			 gc.setLineWidth(4);
			 PrintUtil.print_axis(gc, axisFont, SW_main.gp.print_axis_colour, 
					 SW_main.gp.freq1, SW_main.gp.freq2, 
					 SW_main.gp.plot_xsize, SW_main.gp.plot_ysize,
					 SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset,
					 printScaleFactor, SW_main.gp, SW_main.q, SW_main.q_f1);	// TODO change to q_ind
			 axisFont.dispose();
		 }
	          
        if(SW_main.gp.plot_spectrum) 		// Plot the spectrum
        {
        	gc.setLineWidth((int)(SW_main.gp.spect_pen_thickness * printScaleFactor * 1.7));
        	
        	PrintUtil.print_spect(gc, SW_main.gp.print_spect_colour,
        			SW_main.gp.freq1, SW_main.gp.freq2, 
        			SW_main.gp.expt_scale * scale_factor,
        			(SW_main.gp.expt_y_offset - SW_main.gp.axis_offset) * scale_factor + int_off,
        			SW_main.gp.data_array, thresh,
        			SW_main.gp.plot_xsize, SW_main.gp.plot_ysize, 
        			SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset,
        			printScaleFactor, SW_main.gp, SW_main.q);
        }
        
        if(SW_main.gp.plot_simulated) 		// Plot the spectrum
        {
        	gc.setLineWidth((int)(SW_main.gp.spect_pen_thickness * printScaleFactor * 1.7));
        	
        	PrintUtil.print_spect(gc, SW_main.gp.print_sim_colour,
        			SW_main.gp.freq1, SW_main.gp.freq2, 
        			SW_main.gp.calc_scale * scale_factor,
        			(SW_main.gp.calc_y_offset - SW_main.gp.axis_offset) * scale_factor + int_off,
        			SW_main.gp.calc_array, thresh,
        			SW_main.gp.plot_xsize, SW_main.gp.plot_ysize, 
        			SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset,
        			printScaleFactor, SW_main.gp, SW_main.q);
        }
                
  
                              
        if(SW_main.gp.plot_integrals)
        {
        	gc.setLineWidth(4);
        	Font integFont = new Font(gc.getDevice(), "Arial", 
        			(int)(SW_main.gp.integ_font_height * fontScaleFactor), 
        			SWT.NORMAL);  
        	PrintUtil.print_integ(gc, integFont, SW_main.gp.print_integ_colour,
        			SW_main.gp.freq1, SW_main.gp.freq2, 
        			SW_main.gp.integ_scale * scale_factor, SW_main.gp.expt_y_offset * scale_factor, 
        			SW_main.gp.plot_xsize, SW_main.gp.plot_ysize, 
        			SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset, printScaleFactor, SW_main.gp, SW_main.q);
        	integFont.dispose();
        }
        
//    	print any inset windows (boxes)including integrals within the inset box
        for (int i = 0; i < SW_main.gp.n_boxes; i++)
        {
        	if (SW_main.gp.printInsetBox[i].active)
        	{
        		/*gc.drawRectangle((int)((SW_main.gp.plot_xoffset 
        				+ SW_main.gp.printInsetBox[i].page_x) * printScaleFactor), 
        				(int)((SW_main.gp.plot_yoffset
        						+ SW_main.gp.printInsetBox[i].page_y) * printScaleFactor),
        				(int)(SW_main.gp.printInsetBox[i].page_width * printScaleFactor), 
        				(int)(SW_main.gp.printInsetBox[i].page_height * printScaleFactor));*/

        		Font axisFont = new Font(gc.getDevice(), "Arial", 
        				(int)(SW_main.gp.axis_font_height * fontScaleFactor), 
        				SWT.NORMAL);  // font sizes are in points	
        		gc.setLineWidth(4);
        		PrintUtil.print_axis(gc, axisFont, SW_main.gp.print_axis_colour, 
        				SW_main.gp.printInsetBox[i].freq1, SW_main.gp.printInsetBox[i].freq2, 
        				SW_main.gp.printInsetBox[i].page_width, 
        				SW_main.gp.printInsetBox[i].page_height,
        				SW_main.gp.plot_xoffset + SW_main.gp.printInsetBox[i].page_x, 
        				SW_main.gp.plot_yoffset + SW_main.gp.printInsetBox[i].page_y,
        				printScaleFactor, SW_main.gp, SW_main.q, SW_main.q_f1);	// TODO change to q_ind
        		axisFont.dispose();

        		gc.setLineWidth((int)(SW_main.gp.spect_pen_thickness * printScaleFactor * 1.7));
        		PrintUtil.print_spect(gc, SW_main.gp.print_spect_colour,
        				SW_main.gp.printInsetBox[i].freq1, SW_main.gp.printInsetBox[i].freq2, 
        				SW_main.gp.printInsetBox[i].yscale * scale_factor,
        				(SW_main.gp.expt_y_offset - SW_main.gp.axis_offset) * scale_factor + int_off,
        				SW_main.gp.data_array, thresh,
        				SW_main.gp.printInsetBox[i].page_width, 
        				SW_main.gp.printInsetBox[i].page_height, 
        				SW_main.gp.plot_xoffset + SW_main.gp.printInsetBox[i].page_x, 
        				SW_main.gp.plot_yoffset + SW_main.gp.printInsetBox[i].page_y,
        				printScaleFactor, SW_main.gp, SW_main.q);	
        		
        		if(SW_main.gp.plot_integrals && SW_main.gp.plot_integrals_inset)
                {
                	gc.setLineWidth(4);
                	Font integFont = new Font(gc.getDevice(), "Arial", 
                			(int)(SW_main.gp.integ_font_height * fontScaleFactor), 
                			SWT.NORMAL);  
                	PrintUtil.print_integ(gc, integFont, SW_main.gp.print_integ_colour,
                			SW_main.gp.printInsetBox[i].freq1, SW_main.gp.printInsetBox[i].freq2,
                			SW_main.gp.integ_scale * scale_factor, SW_main.gp.expt_y_offset * scale_factor, 
                			SW_main.gp.printInsetBox[i].page_width, 
                			SW_main.gp.printInsetBox[i].page_height, 
                			SW_main.gp.plot_xoffset + SW_main.gp.printInsetBox[i].page_x, 
                			SW_main.gp.plot_yoffset + SW_main.gp.printInsetBox[i].page_y, 
                			printScaleFactor, SW_main.gp, SW_main.q);
                	integFont.dispose();
                }
        	}                   
        }
        
        if(SW_main.gp.plot_peak_labels)
        {
        	gc.setLineWidth(4);
        	Font peakFont = new Font(gc.getDevice(), "Arial", 
        			(int)(SW_main.gp.peak_font_height * fontScaleFactor), 
        			SWT.NORMAL);         	
        	PrintUtil.print_peaks(gc, peakFont, SW_main.gp.print_integ_colour, // peaks same colour as integrals for now
        			SW_main.gp.freq1, SW_main.gp.freq2, 
        			SW_main.gp.plot_xsize, SW_main.gp.plot_ysize, SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset, 
        			printScaleFactor, 
        			SW_main.gp.expt_scale * scale_factor, 
        			(SW_main.gp.expt_y_offset - SW_main.gp.axis_offset) * scale_factor + int_off,
        			SW_main.gp, SW_main.q);
        	peakFont.dispose();
        }
        
        if(SW_main.gp.plot_frame)			// draw a nice frame around it, if desired
        {
        	gc.setLineWidth(4);
	        gc.drawRectangle((int)(SW_main.gp.plot_xoffset * printScaleFactor), 
	        		         (int)(SW_main.gp.plot_yoffset * printScaleFactor), 
	                         (int)(SW_main.gp.plot_xsize * printScaleFactor), 
	                         (int)(SW_main.gp.plot_ysize * printScaleFactor));
        }
        
        if(SW_main.gp.plot_title_flag)		// print the plot title
        {
        	Font titleFont = new Font(gc.getDevice(), "Arial", 
        			(int)(SW_main.gp.title_font_height * fontScaleFactor), 
        			SWT.NORMAL);  
        	gc.setFont(titleFont);
        	gc.setForeground(new Color(gc.getDevice(), 0, 0, 0));
	        gc.drawText(String.format("SpinWorks Java: %s", SW_main.gp.plot_title),  
		                   (int)((SW_main.gp.plot_xoffset + 1.0) * printScaleFactor), 
		                   (int)((SW_main.gp.plot_yoffset - SW_main.gp.title_font_height + 1.0) * printScaleFactor));
	        titleFont.dispose();
        }
        
        if(SW_main.gp.plot_parameters)	// plot the parameters, if desired
        {
        	Font parFont = new Font(gc.getDevice(), "Arial", 
        			(int)(SW_main.gp.par_font_height * fontScaleFactor), 
        			SWT.NORMAL);  
	        PrintUtil.print_parameters(gc, parFont, new Color(gc.getDevice(), 0, 0, 0),
				          SW_main.gp.plot_xsize, SW_main.gp.plot_ysize, 
				          SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset, printScaleFactor,
				          SW_main.gp, SW_main.q);
	        parFont.dispose();
        }
        //black.dispose();
	}
//------------------------------------------------------------------------------------------------
	private static void doPrint2D(GC gc)
	{
		double top_space, left_space;

		Device device = gc.getDevice();
     

		if (!SW_main.gp.rotate_2D)					// normal (F2 horizontal) orientation
		{
			if (SW_main.gp.f2_trace_on)
				top_space = 40.0;
			else
				top_space = 0.0;

			if (SW_main.gp.f1_trace_on)
				left_space = 40.0;
			else
				left_space = 0.0;
		}
		else								// rotated (F1 horizontal) orientation
		{		
			if (SW_main.gp.f2_trace_on)
				left_space = 40.0;
			else
				left_space = 0.0;

			if (SW_main.gp.f1_trace_on)
				top_space = 40.0;
			else
				top_space = 0.0;
		}

		//adjust sizes to allow for "projections"
		double plot_xsize_2D = SW_main.gp.plot_xsize  - left_space;
		double plot_ysize_2D = SW_main.gp.plot_ysize  - top_space;

		// select correct parameters for indirect dimension
		data_par q_ind = new data_par();
		if (SW_main.q.data_mode == modes.MODE_3D && SW_main.gp.mode_3D == modes_3D.XY)
			q_ind = (data_par)SW_main.q_f2;
		else
			q_ind = (data_par)SW_main.q_f1;

		gc.setLineWidth(4);
		
		// print the 2D contour map
		if(!SW_main.gp.rotate_2D)	// normal orientatikon
			PrintUtil2D.print_2D_contour(gc, device,							// graphics object and pens
					plot_xsize_2D, plot_ysize_2D,								// sizes in mm
					SW_main.gp.plot_xoffset + left_space, SW_main.gp.plot_yoffset + top_space,	// offsets from UL in mm
					printScaleFactor, SW_main.gp, SW_main.q, q_ind);							// parameter structures

		else				// rotated (indrect horizonral) orientation
			PrintUtil2D.print_2D_contour_rotated(gc, device,					// graphics object and pens
					plot_xsize_2D, plot_ysize_2D,								// sizes in mm
					SW_main.gp.plot_xoffset + left_space, SW_main.gp.plot_yoffset + top_space,	// offsets from UL in mm
					printScaleFactor, SW_main.gp, SW_main.q, q_ind);							// parameter structures

		if(SW_main.gp.plot_axis)			// Plot the axis
		{
			Font axisFont = new Font(device, "Arial", (int)(SW_main.gp.axis_font_height * fontScaleFactor), SWT.NORMAL);  // font sizes are in points					 
			gc.setLineWidth((int)(SW_main.gp.spect_pen_thickness * printScaleFactor));
			gc.setForeground(new Color(device, 0, 0, 0)); // TODO proper colour handling
			gc.setLineWidth(4);

			//  print axes
			if (!SW_main.gp.rotate_2D)	// normal orientation
			{
				//		print the x axis
				PrintUtil.print_axis(gc, axisFont, SW_main.gp.print_axis_colour,
						SW_main.gp.freq1, SW_main.gp.freq2,
						plot_xsize_2D, plot_ysize_2D,
						SW_main.gp.plot_xoffset + left_space, SW_main.gp.plot_yoffset + top_space,
						printScaleFactor, SW_main.gp, SW_main.q, q_ind);

				//		print the y axis
				PrintUtil2D.print_y_axis(gc, axisFont, SW_main.gp.print_axis_colour,
						SW_main.gp.freq1_indirect, SW_main.gp.freq2_indirect,
						plot_xsize_2D, plot_ysize_2D,
						SW_main.gp.plot_xoffset + left_space, SW_main.gp.plot_yoffset + top_space,
						printScaleFactor, SW_main.gp, SW_main.q, q_ind);
			}
			else				// rotated orientation
			{
				//		print the x axis
				PrintUtil.print_axis(gc, axisFont, SW_main.gp.print_axis_colour,
						SW_main.gp.freq1_indirect, SW_main.gp.freq2_indirect,
						plot_xsize_2D, plot_ysize_2D,
						SW_main.gp.plot_xoffset + left_space, SW_main.gp.plot_yoffset + top_space,
						printScaleFactor, SW_main.gp, SW_main.q, q_ind);

				//		print the y axis
				PrintUtil2D.print_y_axis(gc, axisFont, SW_main.gp.print_axis_colour,
						SW_main.gp.freq1, SW_main.gp.freq2,
						plot_xsize_2D, plot_ysize_2D,
						SW_main.gp.plot_xoffset + left_space, SW_main.gp.plot_yoffset + top_space,
						printScaleFactor, SW_main.gp, SW_main.q, q_ind);
			}
			axisFont.dispose();
		}


		//  print the F2 trace
		if (SW_main.gp.f2_trace_on)
		{
			gc.setLineWidth(4);
			if (!SW_main.gp.rotate_2D)			// normal orientation
				PrintUtil2D.print_spect_f2(gc, SW_main.gp.print_spect_colour,			// print objects
						SW_main.gp.freq1, SW_main.gp.freq2, SW_main.gp.f2_trace_scale * 4.2,			// freq and scaling
						SW_main.gp.f2_trace, 0.1,										// array and threshold
						plot_xsize_2D, top_space,								// sizes
						SW_main.gp.plot_xoffset + left_space,                   // offsets
						SW_main.gp.plot_yoffset - SW_main.gp.trace_offset,						
						printScaleFactor, 										// printer scale factor
						SW_main.gp);											// global parameters
			else						// rotated orientation
			{
				PrintUtil2D.print_spect_f2_rotated(gc, SW_main.gp.print_spect_colour,
						SW_main.gp.freq1, SW_main.gp.freq2, SW_main.gp.f2_trace_scale * 4.2,
						SW_main.gp.f2_trace, 0.1,
						plot_ysize_2D, left_space,
						SW_main.gp.plot_yoffset + top_space,
						SW_main.gp.plot_xoffset - SW_main.gp.trace_offset,
						printScaleFactor, SW_main.gp);
			}
		}

		//  print F1 trace
		if(SW_main.gp.f1_trace_on)
		{
			gc.setLineWidth(4);
			if (!SW_main.gp.rotate_2D)			// normal orientation
				PrintUtil2D.print_spect_f1(gc, SW_main.gp.print_spect_colour,
						SW_main.gp.freq1_indirect, SW_main.gp.freq2_indirect, SW_main.gp.f1_trace_scale * 4.2,
						SW_main.gp.f1_trace, 0.1,
						plot_ysize_2D, left_space,
						top_space + SW_main.gp.plot_yoffset,
						SW_main.gp.plot_xoffset - SW_main.gp.trace_offset,
						printScaleFactor, SW_main.gp);
			else						// rotated orientation
			{
				PrintUtil2D.print_spect_f1_rotated(gc, SW_main.gp.print_spect_colour,
						SW_main.gp.freq1_indirect, SW_main.gp.freq2_indirect, SW_main.gp.f1_trace_scale * 4.2,
						SW_main.gp.f1_trace, 0.1,
						plot_xsize_2D, top_space,
						left_space + SW_main.gp.plot_xoffset,
						SW_main.gp.plot_yoffset - SW_main.gp.trace_offset,
						printScaleFactor, SW_main.gp);
			}
		}

		//	draw a nice frame around it, if desired
		if(SW_main.gp.plot_frame)
		{
			gc.setLineWidth(4);
			gc.setForeground(new Color(gc.getDevice(), 0, 0, 0)); // TODO proper colour handling
			gc.drawRectangle((int)((SW_main.gp.plot_xoffset + left_space) * printScaleFactor), 
					(int)((SW_main.gp.plot_yoffset + top_space) * printScaleFactor), 
					(int)(plot_xsize_2D * printScaleFactor), (int)(plot_ysize_2D * printScaleFactor));
		}

		//	print the plot title
		if(SW_main.gp.plot_title_flag)
		{
			gc.setFont(new Font(device, "Arial", (int)(SW_main.gp.title_font_height * fontScaleFactor), SWT.NORMAL));  // font sizes are in points
			gc.setForeground(new Color(gc.getDevice(), 0, 0, 0)); 
			gc.drawText("SpinWorks Java: " + SW_main.gp.plot_title, 
					(int)((SW_main.gp.plot_xoffset + 1.0) * printScaleFactor), 
					(int)((SW_main.gp.plot_yoffset - SW_main.gp.title_font_height + 1.0) * printScaleFactor));
		}

		//	plot the parameters, if desired
		if(SW_main.gp.plot_parameters)	
		{
			Font parFont = new Font(device, "Arial", (int)(SW_main.gp.par_font_height * fontScaleFactor), 
					SWT.NORMAL);  // font sizes are in points
			gc.setFont(parFont);
			PrintUtil2D.print_parameters_2D(gc, parFont, new Color(gc.getDevice(), 0, 0, 0),
					SW_main.gp.plot_xsize, SW_main.gp.plot_ysize, 
					SW_main.gp.plot_xoffset, SW_main.gp.plot_yoffset, printScaleFactor,
					SW_main.gp, SW_main.q, q_ind);
			parFont.dispose();
		}
	}
//------------------------------------------------------------------------------------------------

}
