package spinworks;

/**
 * The class holding the picked peaks information
 * @author Kirk Marat, 
 * University of Manitoba
 */
public class PeakInfo  implements Cloneable
{
/**
 * basic constructor
 */
    public PeakInfo()
    {
        freq = 0.0;     freq_f1 = 0.0;	intensity = 0.0;
		width = 1.0;	gauss = 0.0;
		label = "";
        n_cosy = 0; n_tocsy = 0; n_noesy = 0; n_hmbc = 0;
        cosy = new double[10];
        cosy_f1 = new double[10];
        tocsy = new double[10];
        tocsy_f1= new double[10];
        noesy = new double[10];
        noesy_f1 = new double[10];
        hmbc = new double[10];
        hmbc_f1 = new double[10];
    }

//------------------------------------------------------------------------------------------------
/**
 * peak constructor (overload)for normal peak picking
 * @param _freq frequency in Hz of peak in F2
 * @param _freq_f1 frequency in Hz of peak in F
 * @param _intensity intensity in absolute units
 * @param _label text label for peak
 */
    public PeakInfo(double _freq, double _freq_f1, double _intensity, String _label)
    {
        freq = _freq; 
        freq_f1 = _freq_f1;
        intensity = _intensity;
		width = 1.0;	gauss = 0.0;
        label = _label;
        n_cosy = 0; n_tocsy = 0; n_noesy = 0; n_hmbc = 0;
        cosy = new double[10];
        cosy_f1 = new double[10];
        tocsy = new double[10];
        tocsy_f1 = new double[10];
        noesy = new double[10];
        noesy_f1 = new double[10];
        hmbc = new double[10];
        hmbc_f1 = new double[10];
        
        for(int i = 0; i < 10; i++)
        {
        	cosy_f1[i] = 0.0;
        	tocsy_f1[i] = 0.0;
        	noesy_f1[i] = 0.0;
        	hmbc_f1[i] = 0.0;
        }
    }

//-------------------------------------------------------------------------------------------------
/**
 * peak constructor (overload) for normal peak picking
 * @param _freq frequency in Hz of peak in F2
 * @param _freq_f1 frequency in Hz of peak in F1
 * @param _row 
 * @param _col
 * @param _intensity intensity in absolute units
 * @param _label text label for peak
 */
    public PeakInfo(double _freq, double _freq_f1, int _row, int _col,
                     double _intensity, String _label)
    {
        freq = _freq;
        freq_f1 = _freq_f1;
        intensity = _intensity;
        width = 1.0; gauss = 0.0;
        label = _label;
        row = _row;
        col = _col;
        n_cosy = 0; n_tocsy = 0; n_noesy = 0; n_hmbc = 0;
        cosy = new double[10];
        cosy_f1 = new double[10];
        tocsy = new double[10];
        tocsy_f1 = new double[10];
        noesy = new double[10];
        noesy_f1 = new double[10];
        hmbc = new double[10];
        hmbc_f1 = new double[10];
        
        for(int i = 0; i < 10; i++)
        {
        	cosy_f1[i] = 0.0;
        	tocsy_f1[i] = 0.0;
        	noesy_f1[i] = 0.0;
        	hmbc_f1[i] = 0.0;
        }       
    }
 
//-------------------------------------------------------------------------------------------------
/**
 * peak constructor (overload)for deconvolution
 * @param _freq frequency in Hz of peak in F
 * @param _intensity intensity in absolute units
 * @param _width peak width in Hz.
 * @param _gauss Gaussian for the peak
 * @param _label text label for peak
 */
	public PeakInfo(double _freq,  double _intensity, double _width, double _gauss, String _label)
	{
		freq = _freq;
		intensity = _intensity;
		width = _width;
		gauss = _gauss;
		label = _label;
        freq_iterate = true;
        intensity_iterate = true;
        width_iterate = true;
        gauss_iterate = false;
        n_cosy = 0; n_tocsy = 0; n_noesy = 0; n_hmbc = 0;
	}

//-------------------------------------------------------------------------------------------------
	protected Object clone() throws CloneNotSupportedException 
	{
		return super.clone();
	}
	
//-------------------------------------------------------------------------------------------------		
    public void dispose()			// TODO
    {
    }
    
//-------------------------------------------------------------------------------------------------
// data members
	public double freq;				// frequency of peak in Hz.
    public double freq_f1;			// frequency of peak in Hz in F1. (2D only)
	public double intensity;		// intensity in raw spectral units
	public double width;			// width in Hz.
	public double gauss;			// % Gaussian 
    public String label;			// a text label for the peak
    public boolean freq_iterate;       // whether we should iterate on this freq (decon)
    public boolean intensity_iterate;  // whether we should iterate on this intensity(decon)
    public boolean width_iterate;      // whether we should iterate on this width (decon)
    public boolean gauss_iterate;      // on the % Gaussian character
    public double[] cosy;           // list of COSY correlations
    public double[] cosy_f1;        // HSQC F1 freq of COSY correlation
    public double[] tocsy;          // list of TOCSY correlations
    public double[] tocsy_f1;       // HSQC F1 freq of TOCSY correlation
    public double[] noesy;          // list of NOESY correlations
    public double[] noesy_f1;       // HSQC F1 freq of NOESY correlation
    public double[] hmbc;           // list of HMBC correlations to this proton
    public double[] hmbc_f1;        // HSQC F1 freq of HMBC correlation
    public int n_cosy, n_tocsy,
               n_noesy, n_hmbc;     // number of each type of correlation
    public int row, col;            // row and column addresses for 2D
}
