package spinworks;

import java.awt.Desktop;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MenuAdapter;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Display;

import dialogs.BinningParametersDialog;
import dialogs.DecimalDigitsDialog;

public class PeaksMenuListeners extends SW_main
{
	public static void addPeaksMenuListeners()
	{
		// Peaks and Integrals menu
		// update menu items when dropped				
		peakPickMenu.addMenuListener  (new MenuAdapter() 
		{
			public void menuShown(MenuEvent e) 
			{	
				if((gp.display_mode == modes.MODE_1D && gp.data_array != null))
				{
					pickPeaksItem.setEnabled(true);
					binDataItem.setEnabled(true);
				}
				else
				{
					pickPeaksItem.setEnabled(false);
					binDataItem.setEnabled(false);
				}

				if(gp.n_peaks != 0)
				{
					clearPeaksItem.setEnabled(true);
					clearRegionPeaksItem.setEnabled(true);
				}
				else
				{
					clearPeaksItem.setEnabled(false);
					clearRegionPeaksItem.setEnabled(false);
				}
				
				if(gp.n_integrals != 0)
				{
					clearIntegralRegionsItem.setEnabled(true);
					retainIntegralRegionsItem.setEnabled(true);
				}
				else
				{
					clearIntegralRegionsItem.setEnabled(false);
					retainIntegralRegionsItem.setEnabled(false);;
				}

				if(gp.n_traces != 0)
				{
					peakPickStackItem.setEnabled(true);
					integrateStackItem.setEnabled(true);
					binStackedArrayItem.setEnabled(true);
				}
				else
				{
					peakPickStackItem.setEnabled(false);
					integrateStackItem.setEnabled(false);
					binStackedArrayItem.setEnabled(false);
				}

				if(gp.n_peaks != 0)
					listPeaksItem.setEnabled(true);
				else
					listPeaksItem.setEnabled(false);
			}
		});
		        
        pickPeaksItem.addSelectionListener  (new SelectionAdapter() // pick peaks
        {
            @Override
             public void widgetSelected(SelectionEvent e) 
             {
           	 	PeakHandler.PickPeaks();
             }
        });
        
        clearPeaksItem.addSelectionListener  (new SelectionAdapter() // clear all peaks
        {
            @Override
             public void widgetSelected(SelectionEvent e) 
             {
            	PeakPick.clearPeaks(gp); 
            	update_spect = true;
            	canvas.redraw();	
             }
        });
        
        clearRegionPeaksItem.addSelectionListener  (new SelectionAdapter() // clear peaks in region
        {
            @Override
             public void widgetSelected(SelectionEvent e) 
             {
            	PeakPick.deletePeaks(gp);

                if(gp.display_mode != modes.MODE_2D)   // leave 2D peaks in order selected
    			    gp.n_peaks = PeakPick.sortPeaks(gp.n_peaks, gp.peaks);   // by user for now

                update_spect = true;
                canvas.redraw();
             }
        });
        
        peakPickStackItem.addSelectionListener  (new SelectionAdapter()
        {
        	@Override
        	public void widgetSelected(SelectionEvent e) 
        	{
        		IntegrateHandler.peakPickStack();
        		gp.cursor_flag = 0;
        		update_spect = true;
                canvas.redraw();
        	}
        });

        integrateStackItem.addSelectionListener  (new SelectionAdapter()
        {
        	@Override
        	public void widgetSelected(SelectionEvent e) 
        	{
        		shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
        		 
        		IntegrateHandler.integrateStack();
        		
        		gp.cursor_flag = 0;
        		update_spect = true;
                canvas.redraw();
                shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
        	}
        });
        
        listPeaksItem.addSelectionListener  (new SelectionAdapter()
        {
            @Override
             public void widgetSelected(SelectionEvent e) 
             {
            	if(gp.display_mode == modes.MODE_1D)
           	 		PeakPick.listPeaks(q);
            	else if (gp.display_mode == modes.MODE_2D)
            	{
            		if(q.data_mode == modes.MODE_2D 
            		   || q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XZ)
            			PeakPick.listPeaks2D(gp, q, q_f1);		// 2D or 3D XZ
            		else if(q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XY)
            			PeakPick.listPeaks2D(gp, q, q_f2);		// 3D XY
            	}
             }
        });
        
        loadPeaksItem.addSelectionListener  (new SelectionAdapter()
        {
            @Override
             public void widgetSelected(SelectionEvent e) 
             {
            	if (gp.display_mode == modes.MODE_1D)
            		FileUtil.readPeaks1D(gp, q);
            	else if(gp.display_mode == modes.MODE_2D)
            		FileUtil.readPeaks2D(gp, q);
            	     
            	update_spect = true;
            	canvas.redraw();
             }
        });
        
        interpolateItem.addSelectionListener  (new SelectionAdapter() // toggle interpolation
        {															  // for peak picking and 
        	@Override												  // calibration
        	public void widgetSelected(SelectionEvent e) 
        	{
        		if(gp.interp_on)
        		{
        			gp.interp_on = false;
        			interpolateItem.setSelection(false);
        		}
        		else
        		{
        			gp.interp_on = true;
        			interpolateItem.setSelection(true);
        		}
        	}
        });
        
        posOnlyPeaks.addSelectionListener  (new SelectionAdapter() // only pos peaks
        {
            @Override
             public void widgetSelected(SelectionEvent e) 
             {
	           	 gp.pp_sign = 1;
	           	 bothPeaks.setSelection(false);
	           	 negOnlyPeaks.setSelection(false);
             }
        });
        
        negOnlyPeaks.addSelectionListener  (new SelectionAdapter() // only nehative peaks
        {
            @Override
             public void widgetSelected(SelectionEvent e) 
             {
	           	 gp.pp_sign = -1;
	           	 bothPeaks.setSelection(false);
	           	 posOnlyPeaks.setSelection(false);
             }
        });
        
        bothPeaks.addSelectionListener  (new SelectionAdapter() // both pos and neg
        {
            @Override
             public void widgetSelected(SelectionEvent e) 
             {
	           	 gp.pp_sign = 0;
	           	 negOnlyPeaks.setSelection(false);
	           	 posOnlyPeaks.setSelection(false);
             }
        });
        
        unitsHz.addSelectionListener  (new SelectionAdapter() //pp units to Hz.
        {
            @Override
             public void widgetSelected(SelectionEvent e) 
             {
	           	 unitsPPM.setSelection(false);
	           	 gp.peak_unit = units.HZ;
	           	 update_spect = true;
	           	 canvas.redraw();
             }
        });
       
        unitsPPM.addSelectionListener  (new SelectionAdapter() // pp unis to PPM
        {
            @Override
             public void widgetSelected(SelectionEvent e) 
             {
	           	 unitsHz.setSelection(false);
	           	 gp.peak_unit = units.PPM;
	           	 update_spect = true;
	           	 canvas.redraw();
             }
        });
        
        decimalDigitsItem.addSelectionListener  (new SelectionAdapter() // set number of decimal digits for 
        {																// peaks and integrals
        	@Override
        	public void widgetSelected(SelectionEvent e) 
        	{
        		DecimalDigitsDialog dlg = new DecimalDigitsDialog(shell, 
 						SWT.APPLICATION_MODAL | SWT.TITLE | SWT.BORDER );
        		dlg.open();
        	}
        });
        
        clearIntegralRegionsItem.addSelectionListener  (new SelectionAdapter() // clear all integral regions
        {
        	@Override
        	public void widgetSelected(SelectionEvent e) 
        	{
        		SW_main.gp.n_integrals = 0;
				SW_main.gp.current_integral = -1;
	            SW_main.update_spect = true;
				SW_main.canvas.redraw();
        	}
        });
        
        retainIntegralRegionsItem.addSelectionListener  (new SelectionAdapter() // toggle retain 
        {																		// integrals feature
        	@Override
        	public void widgetSelected(SelectionEvent e) 
        	{
        		if(gp.retain_integral_regions)
				{
					gp.retain_integral_regions = false;
					retainIntegralRegionsItem.setSelection(false);
				}
				else
				{
					gp.retain_integral_regions = true;
					retainIntegralRegionsItem.setSelection(true);
					//gp.auto_load = false;
				}
        	}
        });
        
        binDataItem.addSelectionListener  (new SelectionAdapter() // bin the data
        {
        	@Override
        	public void widgetSelected(SelectionEvent e) 
        	{       
        		 PeakPick.binData(gp.data_array, gp.bin_start, gp.bin_end, gp.bin_width, 
        	                gp.bin_method, q, gp.data_max);
        	}
        });
        
        binStackedArrayItem.addSelectionListener  (new SelectionAdapter() // bin a stacked array
        {
        	@Override
        	public void widgetSelected(SelectionEvent e) 
        	{             		
        		String binFileName = q.data_folder + "bin_file.txt";
        		PrintWriter pw;
        		try
        		{
        			pw = new PrintWriter(binFileName);
        		}
        		catch (Exception ex)
        		{
        			Util.show_and_log("list_peaks(): " + binFileName,  ex.getMessage());
        			return;
        		}

        		for (int i = 0; i < gp.n_traces; i++)
        		{
        			pw.println();
        			pw.println(String.format("trace # %d:", i + 1));
        			PeakPick.binData(gp.stack_trace[i].data, gp.bin_start, gp.bin_end, gp.bin_width,
        					gp.bin_method, q, gp.data_max, pw);
        		}

        		pw.close();
        		
        		if (Desktop.isDesktopSupported())
        		{
        		    try 
        		    {
        		        Desktop.getDesktop().open(new File(binFileName));
        		    } 
        		    catch (IOException ex) 
        		    {
        		        Util.show_and_log("no application for this file type?? " +
        		             binFileName, ex.getMessage());
        		    }
        		}                 
        	}
        });
        
        editBinningParametersItem.addSelectionListener  (new SelectionAdapter() // bin a stacked array
        {
        	@Override
        	public void widgetSelected(SelectionEvent e) 
        	{
        		BinningParametersDialog dlg = new BinningParametersDialog(shell, 
 						SWT.APPLICATION_MODAL | SWT.TITLE | SWT.BORDER );
        		dlg.open();
        	}
        });
              
	}	// end addPeaksMenuListeners()
//-------------------------------------------------------------------------------------------------
}	// end class
