package spinworks;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.printing.PrintDialog;
import org.eclipse.swt.printing.Printer;
import org.eclipse.swt.printing.PrinterData;

public class PrintHandler extends SW_main
{
	final public static double mmToPoint = 2.835;  // converts mm to points
		
//-------------------------------------------------------------------------------------------------
	public static void doPrint()
	{
		 PrintDialog dlg = new PrintDialog(shell);
		 PrinterData printerData = dlg.open();
			 
		 if (printerData == null) 
			 return;
		 
		 printerData.orientation = PrinterData.LANDSCAPE;
		 
		 try
		 {
			 Printer printer = new Printer(printerData);	 
			 Point printerDPI = printer.getDPI();		 			 
			 printScaleFactor = (double)printerDPI.x / 25.4;	// assume same DPI x and y??
			 
			 GC gc = new GC(printer);	// get the GC for the printer		 
			 Color black = new Color(gc.getDevice(), 0, 0, 0);
	
			 // default colours if user hasn't chosen others
			 if(SW_main.gp.print_spect_colour == null)
				 SW_main.gp.print_spect_colour = black;
	
			 if(SW_main.gp.print_integ_colour == null)
				 SW_main.gp.print_integ_colour = black;
	
			 if(SW_main.gp.print_axis_colour == null)
				 SW_main.gp.print_axis_colour = black;
			 			 
			 if (printer.startJob("Print Plot")) 
			 {
				 if (printer.startPage()) 
				 {
								 					 
					 if(gp.display_mode == modes.MODE_1D)
						 doPrint1D(gc);
					 
					 else if (gp.display_mode == modes.MODE_FID)
					 {
						 // TODO
					 }
					 
					 else  // must be 2D or 3D
						 doPrint2D(gc);
					 
					 printer.endPage();
				 }
			 }
			 
			 printer.endJob();
			 printer.dispose();
			 gc.dispose();
		 }
		 catch(Exception ex)
		{
			Util.show_and_log("Exception in doPrint()", ex.getMessage());
			ex.printStackTrace();
		}
		 
	}

//-------------------------------------------------------------------------------------------------	
	private static void doPrint1D(GC gc)
	{
		// some useful values
		double scale_factor_y = gp.plot_ysize/((double)gp.dysize);
		double scale_factor_x = gp.plot_xsize / ((double)gp.dxsize);
        
        int points = 
				Math.abs((int)((q.offset - gp.freq1) / q.hzperpt) 
				       - (int)((q.offset - gp.freq2) / q.hzperpt));
      
        Color black = new Color(gc.getDevice(), 0, 0, 0);
        
        if(SW_main.gp.print_spect_colour == null)
			SW_main.gp.print_spect_colour = black;
		
		if(SW_main.gp.print_integ_colour == null)
			SW_main.gp.print_integ_colour = black;
		
		if(SW_main.gp.print_axis_colour == null)
			SW_main.gp.print_axis_colour = black;
        
        double thresh;
        if (points > 40000)
       	 thresh = 0.1;
        else
       	 thresh = 0.02;
        
        double int_off;

        if (gp.plot_integral_values && gp.n_integrals != 0)	// add space for integral
        	int_off = 10.0;								    // values if necessary
        else
        	int_off = 0.0;
			
		 if(gp.plot_axis)			// Plot the axis
		 {
			 Font axisFont = new Font(gc.getDevice(), "Arial", (int)(gp.axis_font_height * mmToPoint), 
					 SWT.NORMAL);  // font sizes are in points					 
			 PrintUtil.print_axis(gc, axisFont, gp.print_axis_colour, 
					 gp.freq1, gp.freq2, 
					 gp.plot_xsize, gp.plot_ysize,
					 gp.plot_xoffset, gp.plot_yoffset,
					 printScaleFactor, gp, q, q_f1);	// TODO change to q_ind
			 axisFont.dispose();
		 }
	          
        if(gp.plot_spectrum) 		// Plot the spectrum
        {
        	gc.setLineWidth((int)(gp.spect_pen_thickness * printScaleFactor));
        	PrintUtil.print_spect(gc, gp.print_spect_colour,
        			gp.freq1, gp.freq2, 
        			gp.expt_scale * scale_factor_y,
        			(gp.expt_y_offset - gp.axis_offset) * scale_factor_y + int_off,
        			gp.data_array, thresh,
        			gp.plot_xsize, gp.plot_ysize, 
        			gp.plot_xoffset, gp.plot_yoffset,
        			printScaleFactor, gp, q);
        }    
        
        if(gp.plot_simulated) 		// Plot the simulated spectrum
        {
        	gc.setLineWidth((int)(gp.spect_pen_thickness * printScaleFactor));
        	PrintUtil.print_spect(gc, gp.print_spect_colour,
        			gp.freq1, gp.freq2, 
        			gp.calc_scale * scale_factor_y,
        			(gp.calc_y_offset - gp.axis_offset) * scale_factor_y + int_off,
        			gp.calc_array, thresh,
        			gp.plot_xsize, gp.plot_ysize, 
        			gp.plot_xoffset, gp.plot_yoffset,
        			printScaleFactor, gp, q);
        }     
        
//    	print any inset windows (boxes)including integrals within the inset box
    			for (int i = 0; i < gp.n_boxes; i++)
    			{
    				if (gp.printInsetBox[i].active)
    				{
    					/*gc.drawRectangle((int)((gp.plot_xoffset 
    							                  + gp.printInsetBox[i].page_x) * printScaleFactor), 
   	        		                     (int)((gp.plot_yoffset
   	        		                              + gp.printInsetBox[i].page_y) * printScaleFactor),
   	                         (int)(gp.printInsetBox[i].page_width * printScaleFactor), 
   	                         (int)(gp.printInsetBox[i].page_height * printScaleFactor));*/
    					
    					 Font axisFont = new Font(gc.getDevice(), "Arial", (int)(gp.axis_font_height * mmToPoint), 
    							 SWT.NORMAL);  // font sizes are in points					 
    					 PrintUtil.print_axis(gc, axisFont, gp.print_axis_colour, 
    							 gp.printInsetBox[i].freq1, gp.printInsetBox[i].freq2, 
    							 gp.printInsetBox[i].page_width, gp.printInsetBox[i].page_height,
    							 gp.plot_xoffset + gp.printInsetBox[i].page_x, 
    							 gp.plot_yoffset + gp.printInsetBox[i].page_y,
    							 printScaleFactor, gp, q, q_f1);	// TODO change to q_ind
    					 axisFont.dispose();
    					
    					gc.setLineWidth((int)(gp.spect_pen_thickness * printScaleFactor));
    		        	PrintUtil.print_spect(gc, gp.print_spect_colour,
    		        			gp.printInsetBox[i].freq1, gp.printInsetBox[i].freq2, 
    		        			gp.printInsetBox[i].yscale * scale_factor_y,
    		        			(gp.expt_y_offset - gp.axis_offset) * scale_factor_y + int_off,
    		        			gp.data_array, thresh,
    		        			gp.printInsetBox[i].page_width, gp.printInsetBox[i].page_height, 
    		        			gp.plot_xoffset + gp.printInsetBox[i].page_x, 
    		        			gp.plot_yoffset + gp.printInsetBox[i].page_y,
    		        			printScaleFactor, gp, q);
    		        	
    		        	if(gp.plot_integrals && gp.plot_integrals_inset)
    		            {
    		            	Font integFont = new Font(gc.getDevice(), "Arial", (int)(gp.integ_font_height * mmToPoint), 
    		            			SWT.NORMAL);  // font sizes are in points
    		            	PrintUtil.print_integ(gc, integFont, gp.print_integ_colour,
    		            			gp.printInsetBox[i].freq1, gp.printInsetBox[i].freq2, 
    		            			gp.integ_scale * scale_factor_y, 
    		            			gp.expt_y_offset * scale_factor_y, 
    		            			gp.printInsetBox[i].page_width, gp.printInsetBox[i].page_height, 
    		            			gp.plot_xoffset + gp.printInsetBox[i].page_x, 
    		            			gp.plot_yoffset + gp.printInsetBox[i].page_y, 
    		            			printScaleFactor, gp, q);
    		            	integFont.dispose();
    		            }
    				}                   
    			}
    			    			
    			
//    	print any stacked traces
        for (int i = gp.n_traces -1; i >= 0; i--)
        {
        	int j; 
        	
        	if (gp.plot_spectrum == false)
        		j = i;
        	else
        		j = i + 1;

        	points =
        			Math.abs((int)((gp.stack_trace[i].offset - gp.freq1) / gp.stack_trace[i].hzperpt)
        					- (int)((gp.stack_trace[i].offset - gp.freq2) / gp.stack_trace[i].hzperpt));

        	if (points > 40000)
        		thresh = 0.1;
        	else
        		thresh = 0.02;

        	double xsize;

        	if (gp.stack_x_size_auto)
        		xsize = gp.plot_xsize;
        	else
        		xsize = gp.stack_x_size * scale_factor_x;
        	
        	PrintUtil.print_spect(gc, gp.stack_trace[i].print_colour,
        			gp.freq1, gp.freq2,
        			gp.stack_trace[i].yscale * scale_factor_y,
        			(gp.expt_y_offset + gp.stack_y_offset * (j) - gp.axis_offset) * scale_factor_y + int_off,
        			gp.stack_trace[i].data, thresh,
        			xsize, gp.plot_ysize,
        			gp.plot_xoffset + (gp.stack_x_offset * (j)) * scale_factor_y, gp.expt_scale * scale_factor_y, 
        			printScaleFactor, gp, q);
        	      	
        	double off = SW_main.gp.stack_y_offset * j * scale_factor_y;       	       				
        	String label;

        	if (gp.stack_label_type == 0)
        		label = "";
        	else if (gp.stack_label_type == 1)
        		label = String.format("%d", i + 1);
        	else if (gp.stack_label_type == 2)
        		label = gp.stack_trace[i].title;
        	else if (gp.stack_label_type == 3)
        		label = gp.stack_trace[i].folder;
        	else
        		label = "";
        	
        	gc.drawText(label, 
        			(int)((SW_main.gp.plot_xoffset + (SW_main.gp.stack_x_offset * j * scale_factor_x)) 
        					* printScaleFactor),
        			(int)((SW_main.gp.plot_ysize /*- SW_main.gp.plot_yoffset*/ - off) * printScaleFactor));
        	
        }	// for i
    			   			   			
        if(gp.plot_frame)			// draw a nice frame around it, if desired
	        gc.drawRectangle((int)(gp.plot_xoffset * printScaleFactor), 
	        		         (int)(gp.plot_yoffset * printScaleFactor), 
	                         (int)(gp.plot_xsize * printScaleFactor), 
	                         (int)(gp.plot_ysize * printScaleFactor));
        

        if(gp.plot_title_flag)		// print the plot title
        {
        	Font titleFont = new Font(gc.getDevice(), "Arial", (int)(gp.title_font_height * mmToPoint), 
        			SWT.NORMAL);  // font sizes are in points
        	gc.setFont(titleFont);
        	gc.setForeground(black);
	        gc.drawText(String.format("SpinWorks Java: %s", gp.plot_title),  
		                   (int)((gp.plot_xoffset + 1.0) * printScaleFactor), 
		                   (int)((gp.plot_yoffset - gp.title_font_height - 1.0) * printScaleFactor));
	        titleFont.dispose();
        }
        
        if(gp.plot_integrals)
        {
        	Font integFont = new Font(gc.getDevice(), "Arial", (int)(gp.integ_font_height * mmToPoint), 
        			SWT.NORMAL);  // font sizes are in points
        	PrintUtil.print_integ(gc, integFont, gp.print_integ_colour,
        			gp.freq1, gp.freq2, 
        			gp.integ_scale * scale_factor_y, gp.expt_y_offset * scale_factor_y, 
        			gp.plot_xsize, gp.plot_ysize, 
        			gp.plot_xoffset, gp.plot_yoffset, printScaleFactor, gp, q);
        	integFont.dispose();
        }
        
        if(gp.plot_peak_labels)
        {
        	Font peakFont = new Font(gc.getDevice(), "Arial", (int)(gp.peak_font_height * mmToPoint), 
        			SWT.NORMAL);  // font sizes are in points       	
        	PrintUtil.print_peaks(gc, peakFont, gp.print_integ_colour, // peaks same colour as integrals for now
        			gp.freq1, gp.freq2, 
        			gp.plot_xsize, gp.plot_ysize, gp.plot_xoffset, gp.plot_yoffset, 
        			printScaleFactor, 
        			gp.expt_scale * scale_factor_y, 
        			(gp.expt_y_offset - gp.axis_offset) * scale_factor_y + int_off,
        			gp, q);
        	peakFont.dispose();
        }
        
        if(gp.plot_parameters)	// plot the parameters, if desired
        {
        	Font parFont = new Font(gc.getDevice(), "Arial", (int)(gp.par_font_height * mmToPoint), 
        			SWT.NORMAL);  // font sizes are in points
	        PrintUtil.print_parameters(gc, parFont, black,
				          gp.plot_xsize, gp.plot_ysize, 
				          gp.plot_xoffset, gp.plot_yoffset, printScaleFactor,
				          gp, q);
	        parFont.dispose();
        }
	}
	
//-------------------------------------------------------------------------------------------------
	private static void doPrint2D(GC gc)
	{				        
		double top_space, left_space;
		
		Device device = gc.getDevice();
        Color black = new Color(device, 0, 0, 0);      
        
		if (!gp.rotate_2D)					// normal (F2 horizontal) orientation
		{
			if (gp.f2_trace_on)
				top_space = 40.0;
			else
				top_space = 0.0;

			if (gp.f1_trace_on)
				left_space = 40.0;
			else
				left_space = 0.0;
		}
		else								// rotated (F1 horizontal) orientation
		{		
			if (gp.f2_trace_on)
				left_space = 40.0;
			else
				left_space = 0.0;

			if (gp.f1_trace_on)
				top_space = 40.0;
			else
				top_space = 0.0;
		}

//adjust sizes to allow for "projections"
		double plot_xsize_2D = gp.plot_xsize  - left_space;
		double plot_ysize_2D = gp.plot_ysize  - top_space;
		
// select correct parameters for indirect dimension
        data_par q_ind = new data_par();
        if (q.data_mode == modes.MODE_3D && gp.mode_3D == modes_3D.XY)
            q_ind = (data_par)q_f2;
        else
            q_ind = (data_par)q_f1;
        
// print the 2D contour map
        if(!gp.rotate_2D)	// normal orientatikon
        	PrintUtil2D.print_2D_contour(gc, device,							// graphics object and pens
        			plot_xsize_2D, plot_ysize_2D,								// sizes in mm
        			gp.plot_xoffset + left_space, gp.plot_yoffset + top_space,	// offsets from UL in mm
        			printScaleFactor, gp, q, q_ind);							// parameter structures
        	
        else				// rotated (indrect horizonral) orientation
        	PrintUtil2D.print_2D_contour_rotated(gc, device,					// graphics object and pens
        			plot_xsize_2D, plot_ysize_2D,								// sizes in mm
        			gp.plot_xoffset + left_space, gp.plot_yoffset + top_space,	// offsets from UL in mm
        			printScaleFactor, gp, q, q_ind);							// parameter structures
        
        if(gp.plot_axis)			// Plot the axis
		{
        	Font axisFont = new Font(device, "Arial", (int)(gp.axis_font_height * mmToPoint), SWT.NORMAL);  // font sizes are in points					 
        	gc.setLineWidth((int)(gp.spect_pen_thickness * printScaleFactor));
        	//gc.setForeground(new Color(device, 0, 0, 0)); 
        	gc.setForeground(gp.print_axis_colour);

//  print axes
        	if (!gp.rotate_2D)	// normal orientation
        	{
        		//		print the x axis
        		PrintUtil.print_axis(gc, axisFont, gp.print_axis_colour,
        				gp.freq1, gp.freq2,
        				plot_xsize_2D, plot_ysize_2D,
        				gp.plot_xoffset + left_space, gp.plot_yoffset + top_space,
        				printScaleFactor, gp, q, q_ind);

        		//		print the y axis
        		PrintUtil2D.print_y_axis(gc, axisFont, gp.print_axis_colour,
        				gp.freq1_indirect, gp.freq2_indirect,
        				plot_xsize_2D, plot_ysize_2D,
        				gp.plot_xoffset + left_space, gp.plot_yoffset + top_space,
        				printScaleFactor, gp, q, q_ind);
        	}
        	else				// rotated orientation
        	{
        		//		print the x axis
        		PrintUtil.print_axis(gc, axisFont, gp.print_axis_colour,
            			gp.freq1_indirect, gp.freq2_indirect,
            			plot_xsize_2D, plot_ysize_2D,
            			gp.plot_xoffset + left_space, gp.plot_yoffset + top_space,
            			printScaleFactor, gp, q, q_ind);

            	//		print the y axis
        		PrintUtil2D.print_y_axis(gc, axisFont, gp.print_axis_colour,
            			gp.freq1, gp.freq2,
            			plot_xsize_2D, plot_ysize_2D,
            			gp.plot_xoffset + left_space, gp.plot_yoffset + top_space,
            			printScaleFactor, gp, q, q_ind);
        	}
        	axisFont.dispose();
		 }
        
        
//  print the F2 trace
        if (gp.f2_trace_on)
        {
        	if (!gp.rotate_2D)			// normal orientation
        		PrintUtil2D.print_spect_f2(gc, gp.print_spect_colour,			// print objects
        				gp.freq1, gp.freq2, gp.f2_trace_scale * 4.2,			// freq and scaling
        				gp.f2_trace, 0.1,										// array and threshold
        				plot_xsize_2D, top_space,								// sizes
        				gp.plot_xoffset + left_space,                           // offsets
        				gp.plot_yoffset - gp.trace_offset,						
        				printScaleFactor, 										// printer scale factor
        				gp);													// global parameters
        	else						// rotated orientation
        	{
        		PrintUtil2D.print_spect_f2_rotated(gc, gp.print_spect_colour,
        				gp.freq1, gp.freq2, gp.f2_trace_scale * 4.2,
        				gp.f2_trace, 0.1,
        				plot_ysize_2D, left_space,
        				gp.plot_yoffset + top_space,
        				gp.plot_xoffset - gp.trace_offset,
        				printScaleFactor, gp);
        	}
        }

//  print F1 trace
        if(gp.f1_trace_on)
        {
        	if (!gp.rotate_2D)			// normal orientation
        		PrintUtil2D.print_spect_f1(gc, gp.print_spect_colour,
        				gp.freq1_indirect, gp.freq2_indirect, gp.f1_trace_scale * 4.2,
        				gp.f1_trace, 0.1,
        				plot_ysize_2D, left_space,
        				top_space + gp.plot_yoffset,
        				gp.plot_xoffset - gp.trace_offset,
        				printScaleFactor, gp);
        	else						// rotated orientation
        	{
        		PrintUtil2D.print_spect_f1_rotated(gc, gp.print_spect_colour,
        				gp.freq1_indirect, gp.freq2_indirect, gp.f1_trace_scale * 4.2,
        				gp.f1_trace, 0.1,
        				plot_xsize_2D, top_space,
        				left_space + gp.plot_xoffset,
        				gp.plot_yoffset - gp.trace_offset,
        				printScaleFactor, gp);
        	}
        }
        
//	draw a nice frame around it, if desired
        if(gp.plot_frame)
        {
        	gc.setForeground(black); // TODO proper colour handling
        	gc.drawRectangle((int)((gp.plot_xoffset + left_space) * printScaleFactor), 
        			(int)((gp.plot_yoffset + top_space) * printScaleFactor), 
        			(int)(plot_xsize_2D * printScaleFactor), (int)(plot_ysize_2D * printScaleFactor));
        }

//	print the plot title
        if(gp.plot_title_flag)
        {
        	gc.setFont(new Font(device, "Arial", (int)(gp.title_font_height * mmToPoint), SWT.NORMAL));  // font sizes are in points
        	gc.setForeground(black); 
        	gc.drawText("SpinWorks Java: " + gp.plot_title, 
        			(int)((gp.plot_xoffset + 1.0) * printScaleFactor), 
        			(int)((gp.plot_yoffset - gp.title_font_height - 1.0) * printScaleFactor));
        }
       
//	plot the parameters, if desired
        if(gp.plot_parameters)	
        {
        	Font parFont = new Font(device, "Arial", (int)(gp.par_font_height * mmToPoint), 
        			SWT.NORMAL);  // font sizes are in points
        	gc.setFont(parFont);
	        PrintUtil2D.print_parameters_2D(gc, parFont, black,
				          gp.plot_xsize, gp.plot_ysize, 
				          gp.plot_xoffset, gp.plot_yoffset, printScaleFactor,
				          gp, q, q_ind);
	        parFont.dispose();
        }     
	}
        
//-------------------------------------------------------------------------------------------------
}
