package spinworks;

/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;

public class PrintUtil 
{

//	print_xoffset and print_yoffset are the locations on the page of the whole print
//	measured from the upper left corner, with the +Y direction being DOWN.  
//	Sorry folks, that's just the way it is.
	
/**
 * print a 1D spectrum
 * @param gr
 * @param colour
 * @param f1
 * @param f2
 * @param yscale
 * @param yoffset
 * @param array
 * @param thresh
 * @param xsize
 * @param ysize
 * @param print_xoffset location of print window in mm from upper left corner
 * @param print_yoffset location of print window in mm from upper left corner
 * @param scale
 * @param gp
 * @param q
 */
        public static void print_spect(GC gr, Color colour,
        		        double f1, double f2, 
        		        double yscale, double yoffset, double[] array, double thresh,
				        double xsize, double ysize, double print_xoffset, double print_yoffset,
				        double scale, global_par gp, data_par q)
        {
        	if(array == null)	// probably no simulated spectrum
        		return;
        	
        	try{ gr.setForeground(colour); }
        	catch(Exception ex) {gr.setForeground(new Color(gr.getDevice(), 0, 0, 0));}
        	
	        int point1, point2; // start and stop points in data array 
	        double x, y;        // plot x and y coord. 
	        double oldx, oldy;  // old x and y values 
	        int i, count = 0;
            final int MAX_POINTS = 1024;    // can this be bigger???
            double[] xarray = new double[MAX_POINTS];
            double[] yarray = new double[MAX_POINTS];
	       
            xsize = xsize * scale;			// scale from internal units (mm) to device units
            ysize = ysize * scale;
            print_xoffset = print_xoffset * scale;
            print_yoffset = print_yoffset * scale;

            double hzperpix = (f1 - f2) / (double)(xsize - 1);		// local            

        //	find start/stop points in array
	        point1 = (int) ((q.offset - f1)/q.hzperpt);
	        point2 = (int) ((q.offset - f2)/q.hzperpt);

        //  Make sure array bounds are not exceeded 
	        if(point1 < 0) point1 = 0;
	        if(point2 >= q.SI) point2 = q.SI - 1;

			double label_space = 0.0;
			if (gp.plot_peak_labels && gp.n_peaks != 0) 
				label_space = 17.0 * scale;

        //	Set initial pen location (is this necessary???
	        y = ysize - (double)array[point1] * yscale * scale - yoffset + print_yoffset;

            oldx = (f1 - q.offset + (double)point1 * q.hzperpt) / hzperpix + print_xoffset;
	        oldy = y;

			if(point2 - point1 < 80000)
				for(i = point1; i <= point2; i++)
				{
					x = (f1 - q.offset + (double)i*q.hzperpt)/hzperpix + print_xoffset;
					y = ysize -  array[i] * yscale * scale - yoffset * scale + print_yoffset;
	            
					if (y > (ysize + print_yoffset  - 1.0 * scale)) 
						y = ysize + print_yoffset - 1.0 * scale;
					
					if (y < print_yoffset + label_space) 
						y = print_yoffset + label_space;
					
					if (Math.abs(x - oldx) > thresh || (Math.abs(y - oldy) > thresh))	// significant move
					{
                        xarray[count] = x;       		yarray[count] = y;
						oldx = x;                       oldy = y;
                        count++;
					}

                    if ((count >= MAX_POINTS || i >= point2-1) && count >= 2)
                    {              	
                    	// draw Spectrum 
                    	int[] pts = new int[2 * count];

                    	for(int j = 0; j < count; j++) 
                    	{
                    		pts[2 * j] = (int)xarray[j];
                    		pts[2 * j + 1] = (int)yarray[j];
                    	}
                    	
                    	if(gp.stack_whitewash)
                    	{
                    		int[] vertices = new int[2 * count + 4];
                    		int min = 0;	// actually a maximum since origin is upper left and + y is down
                    		//System.out.println(String.format("Starting vertex creation, count = %d", count));
                    		for(int j = 0; j < count; j++)
                    		{
                    			vertices[2 * j] = (int) xarray[j];
                    			vertices[2 * j + 1] = (int) yarray[j];
                    			if(min < (int) yarray[j]) min = (int) yarray[j];
                    		} 
                    		
                    		vertices[2 * count] = vertices[2 * count - 2];
                        	vertices[2 * count + 1] = min;
                        	vertices[2 * count + 2] = vertices[0];
                        	vertices[2 * count + 3] = min; 
                	    	//gr.setBackground(gp.colour_blue);
                        	//System.out.println("drawing polygon");
                	    	gr.fillPolygon(vertices);
                	    	//gr.setBackground(new Color(gr.getDevice(), 255, 255, 255));
                    	}
                    	
                    	gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values
                    	
                        count = 0;
                    }
				}
            else		// skip points for big spectra, but look for max y deflection (necessary??)
                for (i = point1+1; i <= point2-1; i = i+3)
                {
                    x = (f1 - q.offset + (double)i * q.hzperpt) / hzperpix + print_xoffset;

                    double intensity = array[i];
                    if (Math.abs(array[i - 1]) > Math.abs(intensity)) intensity = array[i - 1];
                    if (Math.abs(array[i + 1]) > Math.abs(intensity)) intensity = array[i + 1];

                    y = ysize - intensity * yscale * scale - yoffset * scale + print_yoffset;

                    if (y > (ysize + print_yoffset - 1.0)) y = ysize + print_yoffset - 1.0;
                    if (y < print_yoffset + label_space) y = print_yoffset + label_space;

                    if (Math.abs(x - oldx) > thresh || (Math.abs(y - oldy) > thresh))	// significant move
                    {
                        xarray[count] = (int)x;       yarray[count] = (int)y;
                        oldx = x;                     oldy = y;
                        count++;
                    }

                    if ((count >= MAX_POINTS || i >= point2 - 4) && count >= 2)
                    {                   	
                      // draw Spectrum 
                    	int[] pts = new int[2 * count];

                    	for(int j = 0; j < count; j++) 
                    	{
                    		pts[2 * j] = (int)xarray[j];
                    		pts[2 * j + 1] = (int)yarray[j];
                    	}
                    	
                    	gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values                   	
                        count = 0;
                    }
                }	// end for i
            // end else
        }

//-------------------------------------------------------------------------------------------------
/**
 * Draw a 1D "X" axis used in 1D and 2D
 * @param gr
 * @param font
 * @param colour
 * @param f1
 * @param f2
 * @param xsize
 * @param ysize
 * @param print_xoffset
 * @param print_yoffset
 * @param scale
 * @param gp
 * @param q
 * @param q_ind
 */
        public static void print_axis(GC gr, Font font, Color colour,
        		double f1, double f2, 
        		double xsize, double ysize, 
        		double print_xoffset, double print_yoffset,
        		double scale, global_par gp, data_par q, data_par q_ind)
        {
         // scale should scale from mm to whatever units are used by device (typically printScaleFactor)
        	double range, tic_space = 0.1, first_tic, tic, pow10;
        	double x, y, oldx;
        	int i, iexp;
        	String number;
        	double unitperpix;

        	xsize = xsize * scale;
        	ysize = ysize * scale;
        	print_xoffset = print_xoffset * scale;
        	print_yoffset = print_yoffset * scale;
        	
        	gr.setFont(font);
        	try{ gr.setForeground(colour); }
        	catch(Exception ex) {gr.setForeground(new Color(gr.getDevice(), 0, 0, 0));}

          //TODO create a dashed pen for the grid, same colour as axis but 1/2 the thickness
       
         // decide whether labelling is in Hz or PPM, and whether F1 or F2
        	if(gp.axis_unit == SW_main.units.PPM)
        	{
        		if(gp.rotate_2D && (gp.display_mode == SW_main.modes.MODE_2D))
        		{
        			f1 = f1/q_ind.SF; 
        			f2 = f2/q_ind.SF;
        		}
        		else
        		{
        			f1 = f1/q.SF; 
        			f2 = f2/q.SF;
        		}
        	}

        	iexp = 0;
        	range = f1 - f2;
        	unitperpix = (f1 - f2)/(double)(xsize);

        //  scale range to between 1 to 10, keep track of power of 10 
        	if(range >= 1.0)
        		while(range >= 10.0)
        		{
        			range=range/10.0;
        			iexp++;
        		}

        	else	//(range < 1.0)
        		while(range < 1.0)
        		{
        			range=range*10.0;
        			iexp--;
        		}      

        //  decide tic spacing using standard 1,2,5 sequence 
        //  coarser spacing for big numbers, short axis or large fonts
        	if(Math.abs(f1) > 10000.0 || xsize < 180.00 || gp.axis_font_height * scale > 6.0)	
        	{																
        		if(range < 1.0 && range >= 0.5) tic_space = 0.05;	
        		if(range < 2.0 && range >= 1.0) tic_space = 0.1;
        		if(range < 5.0 && range >= 2.0) tic_space = 0.2;	
        		if(range < 10.0 && range >= 5.0) tic_space = 0.5;	
        	}
        	else		
        	{
        		if(range < 1.0 && range >= 0.5) tic_space = 0.02;	
        		if(range < 2.0 && range >= 1.0) tic_space = 0.05;	
        		if(range < 5.0 && range >= 2.0) tic_space = 0.1;	
        		if(range < 10.0 && range >= 5.0) tic_space = 0.2;	
        	}

        //  calculate real tic spacing and rational first tic position
        	pow10 = Math.pow(10.0, (double)iexp); 
        	double ts = tic_space;
        	tic_space = tic_space * pow10;
        	first_tic = tic_space * ((int)((f1-tic_space)/tic_space));
        	double ft = first_tic/pow10;

        	x = print_xoffset;
        	y = ysize + print_yoffset;

        	if(gp.display_mode == SW_main.modes.MODE_1D)
        	{
        		if(gp.axis_unit == SW_main.units.HZ)
        			gr.drawText("Hz", (int)(x + 0.2 * scale), (int)(y + 4.0 * scale));   			
        		else if(gp.axis_unit == SW_main.units.PPM)
        			gr.drawText("PPM", (int)(x + 0.2 * scale), (int)(y + 4.0 * scale));
        	}
        	else
        	{
        		if(gp.rotate_2D)
        		{
        			if(gp.axis_unit == SW_main.units.HZ)
        				gr.drawText("Hz (F1)", (int)(x + 0.2 * scale), (int)(y + 4.0 * scale));       				
        			else if(gp.axis_unit == SW_main.units.PPM)
        				gr.drawText("PPM (F1)", (int)(x + 0.2 * scale), (int)(y + 4.0 * scale));      				
        		}
        		else
        		{
        			if(gp.axis_unit == SW_main.units.HZ)
        				gr.drawText("Hz (F2)", (int)(x + 0.2 * scale), (int)(y + 4.0 * scale));
        			else if(gp.axis_unit == SW_main.units.PPM)
        				gr.drawText("PPM (F2)", (int)(x + 0.2 * scale), (int)(y + 4.0 * scale));
        				
        		}
        	}

        	tic = first_tic;

        //  now draw the axis, every second tic mark labelled 
        	if((ft/(2.0*ts) - (int)(ft/(2.0*ts))) > 0.001)
        		i=1;
        	else
        		i=0;

        	while((int)((f1-tic)/unitperpix) <= xsize)
        	{
        		oldx = x;
        		x = (int)((f1-tic)/unitperpix) + print_xoffset;

        		gr.drawLine((int)oldx, (int)y, (int)x, (int)y);
        		
        		if(i == 0)
        		{
        			gr.drawLine((int)x, (int)y, (int)x, (int)(y + 3.0 * scale));
        			
        		//  TODO  plat a grid
        			/*if (gp.plot_grid)
        				//gr.DrawLine(dashed_pen, (float)x, (float)y, (float)x, (float)(y - ysize));*/   // TODO
        			double label_space;
        			if(gp.display_mode == SW_main.modes.MODE_2D)
        				label_space = 20.0 * scale;
        			else
        				label_space = 14.0 * scale;

        			if((x > (label_space + print_xoffset)
        					&& x < (xsize - 2.50 * scale + print_xoffset))) 
        			{
        				//decide on amount of right padding to centre number under tic  				
        				if(tic_space < 0.001)
        					number = String.format("%.4f" , tic);
        				else if(tic_space < 0.01)
        					number = String.format("%.3f" , tic);
        				else if(tic_space <0.1)
        					number = String.format("%.2f" , tic);
        				else if(tic_space < 1.0 )
        					number = String.format("%.1f" , tic);
        				else if(tic_space <10.0 )
        					number = String.format("%.0f" , tic);
        				else if(tic_space < 100.0 )
        					number = String.format("%.0f" , tic);
        				else if(tic_space < 1000.0 )
        					number = String.format("%.0f" , tic);
        				else if(tic_space < 10000.0 )
        					number = String.format("%.0f" , tic);
        				else if(tic_space < 100000.0 )
        					number = String.format("%.0f" , tic);
        				else
        					number = String.format("%.0f" , tic);
        				
        				number = number.trim();							// remove extra whitespace
        				Point size = gr.textExtent(number);				// and centre number        				
        				gr.drawText(number, (int)x - size.x / 2, (int)(y + 4 * scale));
        				
        			}
        			i=1;
        		}
        		else
        		{
        			gr.drawLine((int)x, (int)y, (int)x, (int)(y + 1.5 * scale));
        			i=0;
        		}
        		tic = tic - tic_space;
        	}	// while((int)((f1-tic)/unitperpix) <= xsize)

        	gr.drawLine((int)x, (int)y, (int)(xsize + print_xoffset), (int)y);

        } // End of print_axis()
        
//-------------------------------------------------------------------------------------------------
/**
 * print the parameters under the spectrum, 1D version
 * @param gr
 * @param font
 * @param colour
 * @param xsize in mm
 * @param ysize in mm
 * @param print_xoffset in mm
 * @param print_yoffset in mm
 * @param scale
 * @param gp
 * @param q
 */
        public static void print_parameters(GC gr, Font font, Color colour,
        		double xsize, double ysize, 
        		double print_xoffset, double print_yoffset, double scale,
        		global_par gp, data_par q)
        {
        	int x,y;
        	int x_move = (int)((xsize /2.0 ) * scale);
        	int	y_move = (int)(1.5 * gp.par_font_height * scale);
        	
        	try{ gr.setForeground(colour); }
        	catch(Exception ex) {gr.setForeground(new Color(gr.getDevice(), 0, 0, 0));}
        	gr.setFont(font);

        	x = (int)((print_xoffset + .50) * scale);				// set initial location
        	y = (int)((print_yoffset + ysize + 15.00) * scale);

        	String str = "";

        	if (q.data_path.length() > 34)
        		str = "..." + q.data_path.substring(q.data_path.length() - 34);
        	else
        		str = q.data_path;

        	if(q.data_type == SW_main.data_types.VNMR)
        	{       		
        		gr.drawText(String.format("file: %s  block# %d  expt: %s", 
        				str, gp.vblock, q.pulse_sequence), x, y);
        	}
        	else
        	{
        		gr.drawText(String.format("file: %s  expt: %s", 
        				str, q.pulse_sequence), x, y);
        	}
        	y = y + y_move;

        	gr.drawText(String.format("transmitter freq.: %12.6f MHz", 
        			q.SFO1), x, y);       	
        	y = y + y_move;

        	gr.drawText(String.format("time domain size: %d points", 
        			q.TD), x, y);       	
        	y = y + y_move;

        	if(q.TD != 0) 		
        		gr.drawText(String.format("width: %.2f Hz = %.4f ppm = %.6f Hz/pt", 
        				q.SFO1 * q.SW, q.SW, (q.SFO1 * q.SW) / (double)q.TD), 
        				x, y);
        	y = y + y_move;

        	gr.drawText(String.format("number of scans: %d", 
        			q.NS), x, y);

        	// start second column
        	y = (int)((print_yoffset + ysize + 15.00) * scale);		// re-set y
        	x = x + x_move;									        // move over

        	gr.drawText(String.format("freq of 0 ppm: %12.6f MHz", 
        			q.SF), x, y);       	
        	y = y + y_move;

        	gr.drawText(String.format("processed size: %d points", 
        			q.SI), x, y);       	
        	y = y + y_move;

        	gr.drawText(String.format("LB: %.3f    GF: %.4f", 
        			q.LB, q.GB), x, y);        	
        	y = y + y_move;       	

        	gr.drawText(String.format( "Hz/cm: %.3f    ppm/cm: %.5f", 
        			(gp.freq1 - gp.freq2) / (xsize / 10.0), 
        			((gp.freq1 - gp.freq2) / q.SFO1) / (xsize / 10.0)), x, y);
        }
        
//-------------------------------------------------------------------------------------------------
/**
 * print integrals
 * @param gr
 * @param font
 * @param colour
 * @param f1
 * @param f2
 * @param yscale
 * @param yoffset
 * @param xsize in mm
 * @param ysize in mm
 * @param print_xoffset
 * @param print_yoffset
 * @param scale
 * @param gp
 * @param q
 */
    public static void print_integ(GC gr, Font font, Color colour,	// graphics objects
    		double f1, double f2,									// display limits (Hz.)
    		double yscale, double yoffset,							// scaling and y offset
    		double xsize, double ysize,								// plot size (mm)
    		double print_xoffset, double print_yoffset,				// print offset (mm from UL)
    		double scale, global_par gp, data_par q)				// parameter structures
    {
    	if (gp.data_array == null)
    	{
    		Util.show_and_log(
    		"print_integ(...) integral regions defined\n but there are no experimental data");
    		return;
    	}

    	try{ gr.setForeground(colour); }
    	catch(Exception ex) {gr.setForeground(new Color(gr.getDevice(), 0, 0, 0));}
    	gr.setFont(font);
    	
    	double hzperpix;
    	double ratio;         // ratio of actual to display digital resolution 
    	int point1, point2;   // start and stop points in data array 
    	int point1a;          // for cases where left edge of paper may be outside of spectrum
    	int start, stop;      // start and stop points in integral 
    	int x = 0, y;         // plot x and y coord. 
    	int old_x, old_y;
    	int x_start, x_end;
    	int i, j;
    	double integral;
    	String number;

    	xsize = xsize * scale;
    	ysize = ysize * scale;
    	print_xoffset = print_xoffset * scale;
    	print_yoffset = print_yoffset * scale;      	
    	yoffset = yoffset * scale;
    	yscale = yscale * scale;

    	hzperpix =(f1-f2)/(double)(xsize-1);
    	ratio = q.hzperpt/hzperpix;						// pix per pt 

    	point1 = (int) ((q.offset-f1)/q.hzperpt);
    	point2 = (int) ((q.offset-f2)/q.hzperpt);
    	point1a = point1;

    	//    	make sure array bounds are not exceeded 
    	if(point1 < 0) point1 = 0;
    	if(point2 >= q.SI) point2 = q.SI-1;

    	for(i=0; i < gp.n_integrals; i++)
    	{
    		if(gp.integrals[i].start > -1.0e30)
    		{
    			start = (int)((q.offset-gp.integrals[i].start)/q.hzperpt);
    			if(start < 0) start = 0;
    			stop = (int)((q.offset-gp.integrals[i].end)/q.hzperpt);
    			if(stop > q.SI-1) stop = q.SI-1;

    			integral = 0.0; 

    			y = (int) (ysize - yoffset + print_yoffset);

    			old_x = (int) (((double)(start-point1a) * ratio) + print_xoffset);			        

    			x_start = old_x;

    			old_y = (int) (ysize - yoffset + print_yoffset);

    			for(j = start + 1; j < stop; j++)			// plot the integral
    			{
    				x = (int) (((double)(j-point1a)*ratio) + print_xoffset);

    				if (x <= (float)print_xoffset || x > (float)(xsize + print_xoffset))
    					break;  // outside of printed page, so skip this integral

    				integral = integral + gp.data_array[j]
    						+ gp.integrals[i].bias
    						+ gp.integrals[i].slope * ((double)j - (double)start);

    				y = (int)(ysize 
    						- integral*yscale 
    						- yoffset
    						+ print_yoffset);

    				if (y > (int)(ysize + print_yoffset  + 1.0 * scale))
    					y = (int)(ysize + print_yoffset + 1.0 * scale);

    				if(y < (int)(print_yoffset + 5.0 * scale))    
    					y = (int)(print_yoffset + 5.0 * scale);

    				if(gp.plot_integrals)
    					gr.drawLine(old_x, (int)(old_y - 5.0 * scale), x, (int)(y - 5.0 * scale));

    				old_y = y;		old_x = x;
    			}
    			x_end = x;

    			if(x_start >= (float)print_xoffset 
    					&& x_end <= (float)(xsize + print_xoffset))
    			{
    				//    	add this value to the integral list
    				gp.integrals[i].value = integral;

    				//    	paranoid divide by zero check 
    				if(gp.integ_cal > 1.0 || gp.integ_cal < -1.0)
    					gp.integrals[i].scaled = integral / gp.integ_cal;

    				y	= (int)(ysize 
    						+ print_yoffset 
    						- yoffset  
    						+ 5.0 * scale);

    				if (gp.plot_integral_values)
    				{
    					//	draw integral region lines
    					gr.drawLine(x_start, y, x_start, (int)(y + 2.0 * scale));
    					gr.drawLine(x_start, (int)(y + 2.0 * scale),
    							(int)((x_start + x_end) / 2), (int)(y + 4.0 * scale));
    					gr.drawLine((int)((x_start + x_end) / 2), (int)(y + 4.0 * scale),
    							(int)((x_start + x_end) / 2.0), (int)(y + 6.0 * scale));
    					gr.drawLine((int)((x_start + x_end) / 2.0), (int)(y + 4.0 * scale),
    							x_end, (int)(y + 2.0 * scale));
    					gr.drawLine(x_end, (int)(y + 2.0 * scale), x_end, y);

    					String int_digits_str = "%." + String.format("%d", gp.int_digits) + "f";

    					number = String.format(int_digits_str, (gp.integrals[i].scaled));

    					// TODO add proper color handling
    	    			Point size = gr.textExtent(number);
    	    			Image textImage = RotateText.createRotatedText(number, gr.getFont(), 
    	    					colour, 
    	    					new Color(gr.getDevice(), 255, 255, 255), 
    	    					SWT.NORMAL);
    	    			
    					gr.drawImage(textImage, 
    							(int)((((start - point1a) + (stop - point1a)) / 2 * ratio)
								+ print_xoffset - size.y / 2),
						        (int)(ysize + print_yoffset - 2.0 * scale) - size.x);
    					
    					textImage.dispose();	
    				}
    			}
    		}
    	}
    }
        
//-------------------------------------------------------------------------------------------------
/**
 * print picked peaks on plot
 * @param gr
 * @param font
 * @param colour
 * @param f1
 * @param f2
 * @param xsize
 * @param ysize
 * @param print_xoffset
 * @param print_yoffset
 * @param scale
 * @param yscale
 * @param yoffset
 * @param gp
 * @param q
 */
    public static void print_peaks(GC gr, Font  font, Color colour,	
    		double f1, double f2,
    		double xsize, double ysize, 
    		double print_xoffset, double print_yoffset, double scale,
    		double yscale, double yoffset,
    		global_par gp, data_par q)
    {
    	int i;
    	int y;
    	int datax; 
    	int data_index;
    	double hzperpix;
    	int[] label_x = new int[gp.n_peaks];
    	String str;

    	xsize = xsize * scale;
    	ysize = ysize * scale;
    	print_xoffset = print_xoffset * scale;
    	print_yoffset = print_yoffset * scale;

    	hzperpix = (f1 - f2)/(double)(xsize - 1);
    	
    	try{ gr.setForeground(colour); }
    	catch(Exception ex) {gr.setForeground(new Color(gr.getDevice(), 0, 0, 0));}
    	gr.setFont(font);

    	for(i = 0; i < gp.n_peaks; i++)
    		label_x[i] = (int)((f1 - gp.peaks[i].freq)/hzperpix + print_xoffset);

    	boolean changed = true; 
    	int count = 0;

    	// adjust spacing to prevent overlap of labels
    	while(changed == true && count < 1200)
    	{
    		changed = false;
    		for (i = 0; i < gp.n_peaks - 1; i++)
    		{
    			if (Math.abs(label_x[i + 1] - label_x[i]) < (int) (gp.peak_font_height * scale))
    			{
    				label_x[i] =  label_x[i] - (int) ((gp.peak_font_height * scale) / 10.0);
    				changed = true;
    				break;
    			}
    		}

    		for (i = gp.n_peaks - 1; i > 0; i--)
    		{
    			if (Math.abs(label_x[i - 1] - label_x[i]) < (int) (gp.peak_font_height * scale))
    			{
    				label_x[i] = label_x[i] + (int) ((gp.peak_font_height * scale) / 10.0);
    				changed = true;
    				break;
    			}
    		}
    		count++;
    	}

    	for(i=0; i < gp.n_peaks; i++)				// print any "picked peaks"
    	{
    		datax = (int)(((f1 - gp.peaks[i].freq)/hzperpix) + print_xoffset);
    		data_index = (int)((q.offset - gp.peaks[i].freq) / q.hzperpt);

    		if(data_index >= 0 
    				&& data_index < q.SI 
    				&& datax > (float)print_xoffset 
    				&& datax < (float)(xsize + print_xoffset) 
    				&& label_x[i] > print_xoffset + 2.0 * scale
    				&& label_x[i] < (float)(xsize + print_xoffset - 2.0 * scale)
    				&& gp.peaks[i].freq != 0.0)
    		{
    			if(gp.peak_font_height * scale < 2.9 * scale)  
    				y = (int)(17.0 * scale + print_yoffset);
    			
    			else if (gp.peak_font_height * scale >= (2.9 * scale) 
    					&& gp.peak_font_height * scale < (4.0 * scale))
    				y = (int)(19.0 * scale + print_yoffset);
    			
    			else
    				y = (int)(21.0 * scale + print_yoffset);

    			gr.drawLine(label_x[i], y + (int)(1.0 * scale), label_x[i], (int) (y + 3.0 * scale));
    			gr.drawLine(label_x[i], y + (int)(3.0 * scale), datax, (int) (y + 5.0 * scale));
    			gr.drawLine(datax, (int) (y + 5.0 * scale), datax, (int) (y + 7.0 * scale));

    			String pp_digits_str = "%." + String.format("%d", gp.pp_digits) + "f";

    			if(gp.peak_unit == SW_main.units.PPM)
    				str = String.format(pp_digits_str, gp.peaks[i].freq / q.SF);
    			else
    				str = String.format(pp_digits_str, gp.peaks[i].freq);
    			
    			// TODO add proper colour handling
    			Point size = gr.textExtent(str);
    			Image textImage = RotateText.createRotatedText(str, gr.getFont(), 
    					colour, 
    					new Color(gr.getDevice(), 255, 255, 255), 
    					SWT.NORMAL);
				gr.drawImage(textImage, label_x[i] - size.y / 2, y - size.x);
				textImage.dispose();	
				
				
				// add peak labels
				if(gp.peaks[i].label.length() > 0 &&  gp.peaks[i].label != null)
    			{
					y = (int)(ysize 
					    -  gp.peaks[i].intensity * yscale * scale 
					    - yoffset * scale 
					    + print_yoffset);
					size = gr.textExtent(gp.peaks[i].label);
    				try
    				{
    					gr.drawLine(datax, y - (int)(1.0 * scale), datax, y - (int)(3.0 * scale));
    					Image labelTextImage = RotateText.createRotatedText(gp.peaks[i].label, 
    							gr.getFont(), colour, new Color(gr.getDevice(), 255, 255, 255), 
    							SWT.NORMAL);
    					gr.drawImage(labelTextImage, datax - size.y / 2, 
    							y - size.x - (int)(4.0 * scale));
    					labelTextImage.dispose();	
    				}
    				catch(Exception ex) {	ex.printStackTrace();	}
    			}
				//////////////
    		}
    	}
    	
    	
    	
    //  label any peaks with labels 
    	/*for (i = 0; i < gp.n_peaks; i++)
    	{
    		datax[i] = (int) Math.round((gp.freq1 - gp.peaks[i].freq) / gp.hzperpix);
    		data_index = (int) Math.round((q.offset - gp.peaks[i].freq) / q.hzperpt);
    		
    		labelx[i] = datax[i];

    		if (data_index >= 0 && data_index < q.SI && datax[i] > 0 && datax[i] < gp.dxsize)
    		{
    			datay = gp.dysize
    					- (int)(gp.data_array[data_index] * gp.expt_scale)
    					- gp.expt_y_offset;

    			if (datay <= 100) datay = 101;
    			gr.drawLine(datax[i], datay - 15, datax[i], datay - 30);  
    			
    			if(gp.peaks[i].label.length() > 0 &&  gp.peaks[i].label != null)
    			{
    				try
    				{
    					Image textImage = RotateText.createRotatedText(gp.peaks[i].label, 
    							peaks_font, gp.axis_colour, gp.back_colour, SWT.NORMAL);
    					gr.drawImage(textImage, datax[i] - SW_main.SMALL_FONT / 2 - 2, 
    							datay - textImage.getBounds().height - 40);
    					textImage.dispose();	
    				}
    				catch(Exception ex) {	ex.printStackTrace();	}
    			}
    		}
    	}*/
    	
    	
    }
//-------------------------------------------------------------------------------------------------
}	// end class
