package spinworks;

/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;

public class PrintUtil2D 
{
//-------------------------------------------------------------------------------------------------
/**
 * Print a 2D contour type display, used in printing, print preview and print files 
 * @param gr
 * @param device
 * @param xsize
 * @param ysize
 * @param print_xoffset
 * @param print_yoffset
 * @param scale
 * @param gp
 * @param q
 * @param q_ind
 */
	public static void print_2D_contour(
			GC gr, 	Device device,    					// graphics object 
			double xsize, double ysize,					// sizes in mm
			double print_xoffset, double print_yoffset,	// offsets from UL in mm
			double scale,                               // scaling factor internal units (mm) to device units
			global_par gp, data_par q, data_par q_ind)	// parameter structures
	{
		int i;											// handy counters
		double x_off, y_off;							// screen offsets
		int x1, y1, x2, y2;

		xsize = xsize * scale;                          // adjust sizes for matafiles in
		ysize = ysize * scale;                          // pixels
		print_xoffset = print_xoffset * scale;
		print_yoffset = print_yoffset * scale;

		double hzperpix = (gp.freq1 - gp.freq2)/(double)(xsize-1);
		double hzperpix_f1 = (gp.freq1_indirect - gp.freq2_indirect)/(double)(ysize-1);

		double ratio = q.hzperpt / hzperpix;			// pix per pt in F2
		double ratio_f1 = q_ind.hzperpt / hzperpix_f1;	// pix per pt in F1

		//	find print offset in pixels
		x_off = ((q.offset - gp.freq1 )/ hzperpix);
		y_off = ((q_ind.offset - gp.freq1_indirect) / hzperpix_f1);

		//  generate contours
		Contour.do_contours(gp, q, q_ind);

		//	currently, positive level contours are always drawn
		//  colour contours look better if they are thickened up a bit
		for(i = 0; i < gp.n_segments; i++)
		{
			if (gp.colour_contour)
			{
				gr.setForeground(new Color(device,
						gp.pos_level[gp.seg_pos[i].level].red,
						gp.pos_level[gp.seg_pos[i].level].green,
						gp.pos_level[gp.seg_pos[i].level].blue));
				
				//pen_pos.Width = gp.spect_pen_thickness * 1.5F * scale;
			}
			else
				gr.setForeground(new Color(device, 0, 0, 0));

			x1 = (int) Math.round((double)gp.seg_pos[i].x1 * ratio + print_xoffset - x_off);
			y1 = (int) Math.round(ysize  - (double)gp.seg_pos[i].y1 * ratio_f1 
					+ print_yoffset + y_off);

			x2 = (int) Math.round((double)gp.seg_pos[i].x2 * ratio + print_xoffset - x_off);
			y2 = (int) Math.round(ysize  - (double)gp.seg_pos[i].y2 * ratio_f1 
					+ print_yoffset + y_off);

			gr.drawLine(x1, y1, x2, y2);
		}

		//	negative contours are only drawn if requested
		for(i = 0; i < gp.n_segments_neg; i++)
		{
			if (gp.colour_contour)
			{
				gr.setForeground(new Color(device,
						gp.neg_level[gp.seg_neg[i].level].red,
						gp.neg_level[gp.seg_neg[i].level].green,
						gp.neg_level[gp.seg_neg[i].level].blue));
				
				//pen_neg.Width = gp.spect_pen_thickness * 1.5F;
			}
			else
				gr.setForeground(new Color(device, 0, 0, 0));

			x1 = (int) Math.round((double)gp.seg_neg[i].x1 * ratio + (print_xoffset - (double)x_off));
			y1 = (int) Math.round(ysize  - (double)gp.seg_neg[i].y1 * ratio_f1 
					+ print_yoffset + (double)y_off);

			x2 = (int) Math.round((double)gp.seg_neg[i].x2 * ratio + (double)(print_xoffset - x_off));
			y2 = (int) Math.round(ysize  - gp.seg_neg[i].y2 * ratio_f1 
					+ (double)print_yoffset + (double)y_off);

			gr.drawLine(x1, y1, x2, y2);
		}
	}
	
//------------------------------------------------------------------------------------------------
/**
 * Print a 2D contour type display, used in printing, print preview and metafiles
 * rotated (Varian) version
 * @param gr
 * @param device
 * @param xsize
 * @param ysize
 * @param print_xoffset
 * @param print_yoffset
 * @param scale
 * @param gp
 * @param q
 * @param q_ind
 */
	public static void print_2D_contour_rotated(
			GC gr, Device device,						// graphics object and pens
			double xsize, double ysize,					// sizes in mm
			double print_xoffset, double print_yoffset,	// offsets from UL in mm
			double scale,
			global_par gp, data_par q, data_par q_ind)	// parameter structures
	{
		int i;											// handy counters
		double x_off, y_off;							// screen offsets
		int x1, y1, x2, y2;

		xsize = xsize * scale;                          // adjust sizes for matafiles in
		ysize = ysize * scale;                          // pixels
		print_xoffset = print_xoffset * scale;
		print_yoffset = print_yoffset * scale;

		double hzperpix = (gp.freq1 - gp.freq2) / (double)(ysize - 1);
		double hzperpix_f1 = (gp.freq1_indirect - gp.freq2_indirect) / (double)(xsize - 1);

		double ratio = q.hzperpt / hzperpix;				// pix per pt in F2
		double ratio_f1 = q_ind.hzperpt / hzperpix_f1;		// pix per pt in F1

		//	find print offset in pixels
		x_off = (float)((q_ind.offset - gp.freq1_indirect) / hzperpix_f1);
		y_off = (float)((q.offset - gp.freq1) / hzperpix);

		//  generate contours
		Contour.do_contours(gp, q, q_ind);

		//	currently, positive level contours are always drawn
		for (i = 0; i < gp.n_segments; i++)
		{
			if (gp.colour_contour)
			{
				gr.setForeground(new Color(device,
						gp.pos_level[gp.seg_pos[i].level].red,
						gp.pos_level[gp.seg_pos[i].level].green,
						gp.pos_level[gp.seg_pos[i].level].blue));
			}
			else
				gr.setForeground(new Color(device, 0, 0, 0));

			x1 = (int) Math.round((double)gp.seg_pos[i].y1 * ratio_f1 + print_xoffset - x_off);

			y1 = (int) Math.round(ysize - (double)gp.seg_pos[i].x1 * ratio
					+ print_yoffset + y_off);

			x2 = (int) Math.round((double)gp.seg_pos[i].y2 * ratio_f1 + print_xoffset - x_off);

			y2 = (int) Math.round(ysize - (double)gp.seg_pos[i].x2 * ratio
					+ print_yoffset + y_off);

			gr.drawLine(x1, y1, x2, y2);
		}

		//	negative contours are only drawn if requested
		for (i = 0; i < gp.n_segments_neg; i++)
		{
			if (gp.colour_contour)
			{
				gr.setForeground(new Color(device,
						gp.neg_level[gp.seg_neg[i].level].red,
						gp.neg_level[gp.seg_neg[i].level].green,
						gp.neg_level[gp.seg_neg[i].level].blue));
			}
			else
				gr.setForeground(new Color(device, 0, 0, 0));

			x1 = (int) Math.round((double)gp.seg_neg[i].y1 * ratio_f1 + (print_xoffset - (double)x_off));
			y1 = (int) Math.round(ysize - (double)gp.seg_neg[i].x1 * ratio
					+ print_yoffset + y_off);

			x2 = (int) Math.round((double)gp.seg_neg[i].y2 * ratio_f1 + (double)(print_xoffset - x_off));
			y2 = (int) Math.round(ysize - (double)gp.seg_neg[i].x2 * ratio
					+ (double)print_yoffset + y_off);

			gr.drawLine(x1, y1, x2, y2);
		}
	}
	
//-------------------------------------------------------------------------------------------------
/**
 * draw a y axis for 2D spectrum
 * @param gr
 * @param font
 * @param colour
 * @param f1
 * @param f2
 * @param xsize
 * @param ysize
 * @param print_xoffset
 * @param print_yoffset
 * @param scale
 * @param gp
 * @param q
 * @param q_ind
 */
	public static void print_y_axis(GC gr, Font font, Color colour,		// graphics objects
			double f1, double f2,										// frequency limits
			double xsize, double ysize,									// plot size in mm
			double print_xoffset, double print_yoffset,					// in mm from UL corner
			double scale, global_par gp, data_par q, data_par q_ind)
	{

		xsize = xsize * scale;                          // adjust sizes 
		ysize = ysize * scale;                          // mm to pixels
		print_xoffset = print_xoffset * scale;
		print_yoffset = print_yoffset * scale;

		double range, tic_space = 0.1, first_tic, tic, pow10;
		int x, y, old_y;
		int i, iexp;
		String number;
		double unitperpix;		
		
		gr.setFont(font);
		try{ gr.setForeground(colour); }
    	catch(Exception ex) {gr.setForeground(new Color(gr.getDevice(), 0, 0, 0));}
		
		if(gp.axis_unit_indirect == SW_main.units.PPM)
		{
			if(gp.rotate_2D)
			{f1 = f1/q.SF; f2 = f2/q.SF;}
			else
			{f1 = f1/q_ind.SF; f2 = f2/q_ind.SF;}
		}


		iexp = 0;
		range = f1-f2;
		unitperpix = (range)/(double)(ysize);

		// scale range to between 1 to 10, keep track of power of 10 

		if(range >= 1.0)
			while(range >= 10.0)
			{
				range = range/10.0;
				iexp++;
			}

		else		//(range < 1.0)
			while(range < 1.0)
			{
				range = range*10.0;
				iexp--;
			}

		if(range < 1.0 && range >= 0.5)	        tic_space = 0.02;			
		else if(range < 2.0 && range >= 1.0)	tic_space = 0.05;	
		else if(range < 5.0 && range >= 2.0)	tic_space = 0.1;	
		else if(range < 10.0 && range >= 5.0)	tic_space = 0.2;	

		pow10 = Math.pow(10.0, (double)iexp);

		// calculate real tic spacing and rational first tic position 
		double ts = tic_space;
		tic_space = tic_space * pow10;
		first_tic = tic_space * ((int)((f1-tic_space)/tic_space));
		double ft = first_tic/pow10;

		// set initial location
		x = (int)(xsize + print_xoffset);
		y = (int)(ysize + print_yoffset);

		// identify the axis
		if(!gp.rotate_2D)									// normal (F1 vertical) mode								
		{
			if (gp.axis_unit_indirect == SW_main.units.HZ)	
				gr.drawText("Hz (F1)",
						(int)(x + 4.0 * scale), (int)(y - gp.axis_font_height * scale));			
			else if (gp.axis_unit_indirect == SW_main.units.PPM)
				gr.drawText("PPM (F1)",
						(int)(x + 4.0 * scale), (int)(y - gp.axis_font_height * scale));
		}
		else												// rotated (F2 vertical) mode
		{
			if (gp.axis_unit_indirect == SW_main.units.HZ)
				gr.drawText("Hz (F2)", 
						(int)(x + 4.0 * scale), (int)(y - gp.axis_font_height * scale));		
			else if (gp.axis_unit_indirect == SW_main.units.PPM)
				gr.drawText("PPM (F2)",
						(int)(x + 4.0 * scale), (int)(y - gp.axis_font_height * scale));		
		}

		tic = first_tic;

		// now draw the axis, every second tic mark labelled 
		if(tic_space < 0.001 || tic_space > 10000.0)
		{
			if((ft/(4.0*ts) - (int)(ft/(4.0*ts))) > 0.001)
				i=1;
			else
				i=0;
		}
		else
		{
			if((ft/(2.0*ts) - (int)(ft/(2.0*ts))) > 0.001)
				i=1;
			else
				i=0;
		}

		while((ysize + print_yoffset - ((f1-tic)/unitperpix)) > print_yoffset)
		{
			old_y = y;
			y = (int)(ysize + print_yoffset - ((f1-tic)/unitperpix));

			gr.drawLine(x, old_y, x, y);

			if(i == 0)
			{	
				gr.drawLine(x, y, (int)(x + 3.5 * scale), y);
				if(gp.plot_grid)
					gr.drawLine(x, y, (int)(x - xsize), y);

				if(y < (ysize + print_yoffset) &&  y >  print_yoffset )
				{
					// decide on amount of right padding to centre number under tic
					if(tic_space < 0.001)
						number = String.format("%.4f", (tic));
					else if(tic_space < 0.01)
						number = String.format("%.3f", (tic));
					else if(tic_space <0.1)
						number = String.format("%.2f", (tic));
					else if(tic_space < 1.0 )
						number = String.format("%.1f", (tic));
					else if(tic_space <10.0 )
						number = String.format("%.0f", (tic));
					else if(tic_space < 100.0 )
						number = String.format("%.0f", (tic));
					else if(tic_space < 1000.0 )
						number = String.format("%.0f", (tic));
					else if(tic_space < 10000.0 )
						number = String.format("%.0f", (tic));
					else if(tic_space < 100000.0 )
						number = String.format("%.0f", (tic));
					else
						number = String.format("%.0f", (tic));

					if (y < ysize + print_yoffset - 10.0F)
					{
						number = number.trim();
						gr.drawText(number, 
								x + (int)(4.0 * scale), (int)(y - gp.axis_font_height * scale / 2.0));
					}
				}
				i=1;
			}
			else
			{ 
				gr.drawLine(x, y, (int)(x + 1.5 * scale), y);
				i=0;
			}
			tic = tic - tic_space;
		}							// end while
	}								// End of print_y_axis()
	
	
	
//-------------------------------------------------------------------------------------------------
//	routines for drawing F1 and F2 traces
//-------------------------------------------------------------------------------------------------

/**
 * draw the f2 trace (2D) spectrum on the printer
 * @param gr
 * @param colour
 * @param f1
 * @param f2
 * @param yscale
 * @param array
 * @param thresh
 * @param xsize
 * @param ysize
 * @param print_xoffset
 * @param print_yoffset
 * @param scale
 * @param gp
 */
	public static void print_spect_f2(GC gr, Color colour,
			double f1, double f2, double yscale, double[] array, double thresh,
			double xsize, double ysize, 
			double print_xoffset, double print_yoffset,
			double scale, global_par gp)
	{
		double hzperpix;
		double plot_factor;		// conversion factor screen to plot units
		int point1, point2;		// start and stop points in data array 
		double x,y;				// plot x and y coord. 
		double oldx, oldy;		// old x and y values 
		int i, count = 0;

		xsize = xsize * scale;                          // adjust sizes for matafiles
		ysize = ysize * scale;                          // and printer steps in pixels
		print_xoffset = print_xoffset * scale;
		print_yoffset = print_yoffset * scale;

		int MAX_POINTS = 1024;    // can this be bigger???
		int[] xarray = new int[MAX_POINTS];
		int[] yarray = new int[MAX_POINTS];

		plot_factor = ysize/((double)gp.dysize);
		hzperpix = (f1-f2)/(xsize-1.0);
		yscale = yscale * plot_factor;
		point1 = (int) ((gp.offset_f2_trace - f1) / gp.hzperpt_f2_trace);
		point2 = (int) ((gp.offset_f2_trace - f2) / gp.hzperpt_f2_trace);

		try{ gr.setForeground(colour); }
    	catch(Exception ex) {gr.setForeground(new Color(gr.getDevice(), 0, 0, 0));}

		// make sure array bounds are not exceeded 
		if(point1 < 0) point1 = 0;
		if(point2 >= gp.size_f2_trace) point2 = gp.size_f2_trace;

		y = (ysize - ((double)array[point1] * yscale)
				+ print_yoffset);

		oldx = print_xoffset;
		oldy = y;

		if (point2 - point1 < 40000)	// small number of points, print every point
		{
			for (i = point1; i < point2; i++)
			{
				x = ((f1 - gp.offset_f2_trace + i * gp.hzperpt_f2_trace) / hzperpix) + print_xoffset;

				y = ysize - (double)array[i] * yscale
						+ print_yoffset;
				if (y < print_yoffset) y = print_yoffset;

				if (Math.abs(x - oldx) > thresh || (Math.abs(y - oldy) > thresh))	// significant move
				{
					xarray[count] = (int)x; yarray[count] = (int)y;
					oldx = x; oldy = y;
					count++;
				}

				if ((count >= MAX_POINTS || i >= point2-1) && count >= 2)
				{

					//draw Spectrum 
					int[] pts = new int[2 * count];

					for(int j = 0; j < count; j++) 
					{
						pts[2 * j] = (int)xarray[j];
						pts[2 * j + 1] = (int)yarray[j];
					}

					gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values

					count = 0;
				}
			}
		}
		else	// skip  points but look for max y deflection
		{
			for (i = point1 + 1; i < point2 - 1; i = i + 3)
			{
				x = ((f1 - gp.offset_f2_trace + i * gp.hzperpt_f2_trace) / hzperpix) + print_xoffset;

				double intensity = array[i];
				if (Math.abs(array[i - 1]) > Math.abs(intensity)) intensity = array[i - 1];
				if (Math.abs(array[i + 1]) > Math.abs(intensity)) intensity = array[i + 1];

				y = ysize - (double)intensity * yscale
						+ print_yoffset;

				if (y < print_yoffset) y = print_yoffset;

				if (Math.abs(x - oldx) > thresh || (Math.abs(y - oldy) > thresh))	// significant move
				{
					xarray[count] = (int)x; yarray[count] = (int)y;
					oldx = x; oldy = y;
					count++;
				}

				if ((count >= MAX_POINTS || i >= point2 - 4) && count >= 2)
				{                   	
					//draw Spectrum 
					int[] pts = new int[2 * count];

					for(int j = 0; j < count; j++) 
					{
						pts[2 * j] = (int)xarray[j];
						pts[2 * j + 1] = (int)yarray[j];
					}

					gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values                   	
					count = 0;
				}
			}	//for(i=...
		}	// else
	}
				
//-------------------------------------------------------------------------------------------------
/**
 * draw the f2 trace (2D) spectrum on the printer, rotated version
 * @param gr
 * @param colour
 * @param f1
 * @param f2
 * @param yscale
 * @param array
 * @param thresh
 * @param xsize
 * @param ysize
 * @param print_xoffset
 * @param print_yoffset
 * @param scale
 * @param gp
 */
	public static void print_spect_f2_rotated(GC gr, Color colour,
			double f1, double f2, double yscale, double[] array, double thresh,
			double xsize, double ysize,
			double print_xoffset, double print_yoffset,
			double scale, global_par gp)
	{
		double hzperpix;
		double plot_factor;		// conversion factor screen to plot units
		int point1, point2;		// start and stop points in data array 
		double x,y;				// plot x and y coord. 
		double oldx, oldy;		// old x and y values 
		int i;

		xsize = xsize * scale;                          // adjust sizes for matafiles
		ysize = ysize * scale;                          // and printer steps pixels
		print_xoffset = print_xoffset * scale;
		print_yoffset = print_yoffset * scale;

		plot_factor = (double)ysize/((double)gp.dysize);
		hzperpix = -(f1-f2)/(double)(xsize-1);			// pix per pt 
		yscale = yscale * plot_factor;
		point1 = (int) ((gp.offset_f2_trace - f1) / gp.hzperpt_f2_trace);
		point2 = (int) ((gp.offset_f2_trace - f2) / gp.hzperpt_f2_trace);


		// make sure array bounds are not exceeded 
		if(point1 < 0) point1 = 0;
		if(point2 >= gp.size_f2_trace) point2 = gp.size_f2_trace;
		
		try{ gr.setForeground(colour); }
    	catch(Exception ex) {gr.setForeground(new Color(gr.getDevice(), 0, 0, 0));}

		y = ysize - (array[point1] * yscale)
				+ print_yoffset;

		oldx = xsize + print_xoffset;
		oldy = y;

		for(i = point1; i < point2; i++)
		{
			x = xsize 
					+ ((f1 - gp.offset_f2_trace + i*gp.hzperpt_f2_trace) / hzperpix) 
					+ print_xoffset;

			y = ysize - ((double)array[i]*yscale)  
					+ print_yoffset;

			if (y < 0) y = 0;

			if (Math.abs(x - oldx) > thresh || (Math.abs(y - oldy) > thresh))	// significant move
			{
				gr.drawLine((int)oldy, (int)oldx, (int)y, (int)x);
				oldx = x;
				oldy = y;
			}
		}
	}

//-------------------------------------------------------------------------------------------------
/**
 * draw the f1 trace (2D) spectrum on the printer
 * @param gr
 * @param colour
 * @param f1
 * @param f2
 * @param yscale
 * @param array
 * @param thresh
 * @param xsize
 * @param ysize
 * @param print_xoffset
 * @param print_yoffset
 * @param scale
 * @param gp
 */
	public static void print_spect_f1(GC gr, Color colour,
			double f1, double f2, double yscale, double[] array, double thresh,
			double xsize, double ysize, 
			double print_xoffset, double print_yoffset,
			double scale, global_par gp)
	{
		double hzperpix;
		double plot_factor;		// conversion factor screen to plot units
		int point1, point2;		// start and stop points in data array 
		double x,y;				// plot x and y coord. 
		double oldx, oldy;		// old x and y values 
		int i, count = 0;

		int MAX_POINTS = 1024;    // can this be bigger???
		int[] xarray = new int[MAX_POINTS];
		int[] yarray = new int[MAX_POINTS];

		xsize = xsize * scale;                          // adjust sizes for matafiles 
		ysize = ysize * scale;                          // and printer steps pixels
		print_xoffset = print_xoffset * scale;
		print_yoffset = print_yoffset * scale;

		plot_factor = ysize/((double)gp.dysize);
		hzperpix = -(f1 - f2)/(double)(xsize - 1);		// pix per pt 
		yscale = yscale * plot_factor;
		point1 = (int) ((gp.offset_f1_trace - f1) / gp.hzperpt_f1_trace);
		point2 = (int) ((gp.offset_f1_trace - f2) / gp.hzperpt_f1_trace);

		try{ gr.setForeground(colour); }
    	catch(Exception ex) {gr.setForeground(new Color(gr.getDevice(), 0, 0, 0));}

		// make sure array bounds are not exceeded 
		if(point1 < 0) point1 = 0;
		if(point2 >= gp.size_f1_trace) point2 = gp.size_f1_trace;


		y = ysize - (double)array[point1] * yscale + print_yoffset;

		oldx = xsize + print_xoffset;
		oldy = y;

		if (point2 - point1 < 20000)    // small number of points, print every point
		{
			for (i = point1; i < point2; i++)
			{
				x = xsize
						+ ((f1 - gp.offset_f1_trace + i * gp.hzperpt_f1_trace) / hzperpix)
						+ print_xoffset;

				y = ysize - (double)array[i] * yscale + print_yoffset;

				if (y < 0.0) y = 0.0;


				if (Math.abs(x - oldx) > thresh || (Math.abs(y - oldy) > thresh))	// significant move
				{
					xarray[count] = (int)x; yarray[count] = (int)y;
					oldx = x; oldy = y;
					count++;
				}

				if ((count >= MAX_POINTS || i >= point2-1) && count >= 2)
				{

					//draw Spectrum 
					int[] pts = new int[2 * count];

					for(int j = 0; j < count; j++) 
					{
						pts[2 * j] = (int)yarray[j];		// note coordinate rotation
						pts[2 * j + 1] = (int)xarray[j];
					}

					gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values

					count = 0;
				}
			}
		}
		else
		{
			for (i = point1 + 1; i < point2 - 1; i = i + 3)
			{
				x = xsize
						+ ((f1 - gp.offset_f1_trace + i * gp.hzperpt_f1_trace) / hzperpix)
						+ print_xoffset;

				double intensity = array[i];
				if (Math.abs(array[i - 1]) > Math.abs(intensity)) intensity = array[i - 1];
				if (Math.abs(array[i + 1]) > Math.abs(intensity)) intensity = array[i + 1];

				y = ysize - (double)intensity * yscale + print_yoffset;

				if (y < 0.0) y = 0.0;

				if (Math.abs(x - oldx) > thresh || (Math.abs(y - oldy) > thresh))	// significant move
				{
					xarray[count] = (int)x; yarray[count] = (int)y;
					oldx = x; oldy = y;
					count++;
				}

				if ((count >= MAX_POINTS || i >= point2 - 4) && count >= 2)
				{                   	
					//draw Spectrum 
					int[] pts = new int[2 * count];

					for(int j = 0; j < count; j++) 
					{
						pts[2 * j] = (int)yarray[j];		// note coordinate rotation
						pts[2 * j + 1] = (int)xarray[j];
					}

					gr.drawPolyline(pts);   // polyLine requires single array with alternating x, y values                   	
					count = 0;
				}
			}
		}
	}

//-------------------------------------------------------------------------------------------------
/**
 * draw the f1 trace (2D) spectrum on the printer (rotated version)
 * @param gr
 * @param colour
 * @param f1
 * @param f2
 * @param yscale
 * @param array
 * @param thresh
 * @param xsize
 * @param ysize
 * @param print_xoffset
 * @param print_yoffset
 * @param scale
 * @param gp
 */
	public static void print_spect_f1_rotated(GC gr, Color colour,
			double f1, double f2, double yscale, double[] array, double thresh,
			double xsize, double ysize,
			double print_xoffset, double print_yoffset,
			double scale, global_par gp)
	{
		double hzperpix;
		double plot_factor;		// conversion factor screen to plot units
		int point1, point2;		// start and stop points in data array 
		double x,y;				// plot x and y coord. 
		double oldx, oldy;		// old x and y values 
		int i;

		xsize = xsize * scale;                          // adjust sizes for matafiles 
		ysize = ysize * scale;                          // and printer steps in pixels
		print_xoffset = print_xoffset * scale;
		print_yoffset = print_yoffset * scale;

		plot_factor = -(double)ysize/((double)gp.dysize);
		hzperpix = (f1-f2)/(double)(xsize-1);
		yscale = yscale * plot_factor;
		point1 = (int) ((gp.offset_f1_trace - f1) / gp.hzperpt_f1_trace);
		point2 = (int) ((gp.offset_f1_trace - f2) / gp.hzperpt_f1_trace);

		try{ gr.setForeground(colour); }
    	catch(Exception ex) {gr.setForeground(new Color(gr.getDevice(), 0, 0, 0));}
		
		// make sure array bounds are not exceeded 
		if(point1 < 0) point1 = 0;
		if(point2 >= gp.size_f1_trace) point2 = gp.size_f1_trace;

		y = ysize + ((double)array[point1] * yscale)
				+ print_yoffset;

		oldx = print_xoffset;
		oldy = y;

		for(i=point1; i < point2; i++)
		{
			x = (((f1 - gp.offset_f1_trace + i*gp.hzperpt_f1_trace) / hzperpix) + print_xoffset);

			y = ysize + ((double)array[i] * yscale)  
					+ print_yoffset;

			if (y < print_yoffset) y = print_yoffset;

			if (Math.abs(x - oldx) > thresh || (Math.abs(y - oldy) > thresh))	// significant move
			{
				gr.drawLine((int)oldx, (int)oldy, (int)x, (int)y);
				oldx = x;
				oldy = y;
			}
		}
	}
			
//-------------------------------------------------------------------------------------------------
/**
 * print the parameters, 2D version, standard version, below spectrum
 * @param gr
 * @param font
 * @param colour
 * @param xsize
 * @param ysize
 * @param print_xoffset
 * @param print_yoffset
 * @param scale
 * @param gp
 * @param q
 * @param q_ind
 */
	public static void print_parameters_2D(GC gr, Font font, Color colour,
			double xsize, double ysize, 
			double print_xoffset, double print_yoffset,
			double scale, global_par gp, data_par q, data_par q_ind)
	{
		xsize = xsize * scale;                          // adjust sizes for matafiles 
		ysize = ysize * scale;                          // and printer steps in pixels
		print_xoffset = print_xoffset * scale;
		print_yoffset = print_yoffset * scale;

		int x_move = (int)((gp.plot_xsize/3.0 + 10.0) * scale), 
				y_move = (int)((1.5 * gp.par_font_height) * scale);
		String str = ""; 

		int x = (int)(print_xoffset + .50 * scale);		// set initial location
		int y = (int)(print_yoffset + (gp.plot_ysize + 12.00) * scale);
		
		gr.setFont(font);
		try{ gr.setForeground(colour); }
    	catch(Exception ex) {gr.setForeground(new Color(gr.getDevice(), 0, 0, 0));}

		if (q.data_path.length() > 34)
    		str = "..." + q.data_path.substring(q.data_path.length() - 34);
    	else
    		str = q.data_path;

    	if(q.data_type == SW_main.data_types.VNMR)
    	{       		
    		gr.drawText(String.format("file: %s  block# %d  expt: %s", 
    				str, gp.vblock, q.pulse_sequence), x, y);
    	}
    	else
    	{
    		gr.drawText(String.format("file: %s  expt: %s", 
    				str, q.pulse_sequence), x, y);
    	}
    	y = y + y_move;

    	gr.drawText(String.format("transmitter freq.: %12.6f MHz", 
				q.SFO1), x, y);       	
    	y = y + y_move;

    	gr.drawText(String.format("time domain size: %d by %d points", 
				q.TD, q_ind.TD), x, y);       	
    	y = y + y_move;

    	if(q.TD != 0) 		
    		gr.drawText(String.format("width: %.2f Hz = %.4f ppm = %.6f Hz/pt", 
    				q.SFO1 * q.SW, q.SW, (q.SFO1 * q.SW) / (double)q.TD), 
    				x, y);
    	y = y + y_move;

    	gr.drawText(String.format("number of scans: %d", 
				q.NS), x, y);


		// start second column
		y = (int)(print_yoffset + (gp.plot_ysize + 12.00) * scale);	// re-set y
		x = x + x_move;									// move over

		gr.drawText("F2:", (int)(x - 5.0 * scale), y);

		gr.drawText(String.format("freq. of 0 ppm: %.7f MHz", 
				(q.SF)), 
				x, y);
		y = y + y_move;

		gr.drawText(String.format("processed size: %d complex points", 
				(q.SI)), 
				x, y);
		y = y + y_move;

		switch(q.WDW)
		{
		case NONE:
			str = "window function: NONE";
		break;

		case EM:
			str = "window function: Exponential";
		break;

		case GM:
			str = "window function: Lorentz to Gauss";
		break;

		case SINE:
			str = "window function: Sine";
		break;

		case QSINE:	
			str = "window function: Sine Squared";
		break;

		case TRAF:
			str = "window function: TRAF";
		break;

		case TRAFS:
			str = "window function: TRAF";
		break;

		case GAUSSIAN:
			str = "window function: Gaussian";
		break;

		default:
			break;
		}
		gr.drawText(str, x, y);
		y = y + y_move;

		if (q.WDW == SW_main.windows.EM || q.WDW == SW_main.windows.GM
				|| q.WDW == SW_main.windows.TRAF || q.WDW == SW_main.windows.TRAFS
				|| q.WDW == SW_main.windows.GAUSSIAN)
			str = String.format("LB: %.3f    GF: %.3f}", (q.LB), (q.GB));
		else
			str = String.format("shift: %.1f degrees", (q.ssb));
		gr.drawText(str, x, y);
		y = y + y_move;


		if (!gp.rotate_2D)		
		{
			str = String.format("Hz/cm: %.3f    ppm/cm: %.5f",
					((gp.freq1 - gp.freq2) / (xsize / 10.0 / scale)),	// back to cm.
					(((gp.freq1 - gp.freq2) / q.SFO1) / (xsize / 10.0 / scale)));
			gr.drawText(str, x, y);
		}
		else	// Varian F1 horizontal mode
		{		
			str = String.format("Hz/cm: %.3f    ppm/cm: %.5f",
					((gp.freq1 - gp.freq2) / (ysize / 10.0 / scale)),
					(((gp.freq1 - gp.freq2) / q.SFO1) / (ysize / 10.0 / scale)));
			gr.drawText(str, x, y);
		}


		// start third column
		y = (int)(print_yoffset + (gp.plot_ysize + 12.00) * scale);		// re-set y
		x = x + x_move;													// move over

		gr.drawText("F1:", (int)(x - 5.0 * scale), y);

		gr.drawText(String.format("freq. of 0 ppm: %.7f MHz", 
				(q_ind.SF)), 
				x, y);
		y = y + y_move;

		gr.drawText(String.format("processed size: %d complex points", 
				(q_ind.SI)), 
				x, y);
		y = y + y_move;

		switch(q_ind.WDW)
		{
		case NONE:
			str = "window function: NONE";
		break;

		case EM:
			str = "window function: Exponential";
		break;

		case GM:
			str = "window function: Lorentz to Gauss";
		break;

		case SINE:
			str = "window function: Sine";
		break;

		case QSINE:	
			str = "window function: Sine Squared";
		break;

		case TRAF:
			str = "window function: TRAF";
		break;

		case TRAFS:
			str = "window function: TRAF";
		break;

		case GAUSSIAN:
			str = "window function: Gaussian";
		break;

		default:
			break;
		}
		gr.drawText(str, x, y);
		y = y + y_move;

		if (q_ind.WDW == SW_main.windows.EM || q_ind.WDW == SW_main.windows.GM
				|| q_ind.WDW == SW_main.windows.TRAF
				|| q_ind.WDW == SW_main.windows.TRAFS || q_ind.WDW == SW_main.windows.GAUSSIAN)
			str = String.format("LB: %.3f    GF: %.3f", (q_ind.LB), (q_ind.GB));
		else
			str = String.format("shift: %.1f degrees", (q_ind.ssb));
		gr.drawText(str, x, y);
		y = y + y_move;

	
		if (!gp.rotate_2D)		
		{
			str = String.format("Hz/cm: %.3f    ppm/cm: %.5f",
					((gp.freq1_indirect - gp.freq2_indirect) / (ysize / 10.0 / scale)),
					(((gp.freq1_indirect - gp.freq2_indirect) / q_ind.SFO1) / (ysize / 10.0 / scale)));
			gr.drawText(str, x, y);
		}
		else	// Varian F1 horizontal mode
		{		
			str = String.format("Hz/cm: %.3f    ppm/cm: %.5f",
					((gp.freq1_indirect - gp.freq2_indirect) / (xsize / 10.0 / scale)),
					(((gp.freq1_indirect - gp.freq2_indirect) / q_ind.SFO1) / (xsize / 10.0 / scale)));
			gr.drawText(str, x, y);
		}

	}
//-------------------------------------------------------------------------------------------------
}
