package spinworks;

/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

import lp.Lp;	// Linear prediction routines

/**
 * 1D processing
 * @author marat
 */
public class Proc_1D 
{
	/**
	 * Process a 1D data set
	 * @param gp a SpinWorks global parameter object
	 * @param p set parameters
	 * @param q status parameters
	 * @return true if OK, false otherwise
	 */
	public static boolean proc_1D(global_par gp, data_par p, data_par q)
	{	
		if (FileOpen.get_data(gp, p, q) == false)	// get data 
		{											   
            Util.show_and_log("get_data() failed");
			return false;							    // bail out if load failed
		}
		
//TODO Back LP needs testing
//Do any requested back LP.  Complex and real are equivalent in this case
		if (p.lp == SW_main.LP_modes.BACK_REAL || p.lp == SW_main.LP_modes.BACK_COMPLEX)
			Lp.do_back_lp_f2(gp, p, q);

//forward Zhu-Bax LP now supported! (mainly for demonstration)
		else if (p.lp == SW_main.LP_modes.ZHU_BAX && p.lp_pred <= 2048
            && q.TD <= 2048)
				Lp.do_ZhuBax_LP(gp, p, q);

        Fft.do_fp_and_bias(gp, p, q);                            // first point and bias correction 

        boolean real_data = false;                               // if data are real
        if (q.aq_mode == SW_main.det_modes.SEQ) real_data = true;

        if (p.WDW == SW_main.windows.EM)					// window the data  // change to case??
            Wfcn.do_em(gp.proc_buffer, q.td_eff, p.LB, q.SF * q.SW, real_data);
        else if (p.WDW == SW_main.windows.GM)				// GM
        {
            if (p.GB > 0.0 && p.LB < 0.0)
                Wfcn.do_gm(gp.proc_buffer, q.td_eff, p.LB, p.GB, q.SW * q.SF, real_data);
            else
            {
                Util.show_and_log("GB cannot be 0\nLB must be < 0, continuing with no window");
            }
        }
        else if (p.WDW == SW_main.windows.TRAF)
            Wfcn.do_traf(gp.proc_buffer, q.td_eff, p.LB, q.SF * q.SW, real_data);
        else if (p.WDW == SW_main.windows.TRAFS)
            Wfcn.do_trafs(gp.proc_buffer, q.td_eff, p.LB, q.SF * q.SW, real_data);
        else if (p.WDW == SW_main.windows.SINE)
            Wfcn.do_sine(gp.proc_buffer, q.td_eff, p.ssb);
        else if (p.WDW == SW_main.windows.QSINE)
            Wfcn.do_qsine(gp.proc_buffer, q.td_eff, p.ssb);
        else if (p.WDW == SW_main.windows.GAUSSIAN)        
            Wfcn.do_gaussian(gp.proc_buffer, q.td_eff, p.LB, q.SW * q.SF, real_data);

		q.WDW = p.WDW;

        if (p.WDW == SW_main.windows.TRAF  // update appropriate status parameter
            || p.WDW == SW_main.windows.TRAFS
            || p.WDW == SW_main.windows.EM
            || p.WDW == SW_main.windows.GAUSSIAN)
            q.LB = p.LB;
        else if
            (p.WDW == SW_main.windows.GM)
        {
            q.GB = p.GB;
            q.LB = p.LB;
        }
        else if (p.WDW == SW_main.windows.SINE
            || p.WDW == SW_main.windows.QSINE)
            q.ssb = p.ssb;

		Filter.do_filter(p.filter_type, p.filter_points,
			q.TD, gp.proc_buffer);					        	// apply solvent filter

		if (p.cls != 0)                                         // circular left shift if
			Fft.circular_ls(gp, p, q);                          // requested

        Fft.do_fft(gp.proc_buffer, q.td_eff, q.SI, q.aq_mode); // do the transform  

        Util.free_proc(gp);									    // free and allocate any
        Util.alloc_proc(gp, p, q);                              // existing space

        Fft.extract_data(gp.proc_buffer, gp.data_array, gp.i_data_array,
            q.SI, q.aq_mode);

//apply correction for DSP group delay
        if (q.aq_mode == SW_main.det_modes.DSP
            || q.aq_mode == SW_main.det_modes.CUSTOM_DSP)
            Fft.dsp_phase(-1, gp, q);
      
		if (p.PH_mod == SW_main.phasing.CONSTANTS)		    // now any requested phasing
        {
			Fft.do_phase(gp.data_array, gp.i_data_array, q.SI, p.PHC0, p.PHC1);
            q.PHC0 = p.PHC0 + q.PHC0;       // update status parameters 	
            q.PHC1 = p.PHC1 + q.PHC1;
            p.PHC0 = q.PHC0;
            p.PHC1 = q.PHC1;
        }

//autophase routine currently really sucks!
		/*else if (p.PH_mod == SW_main.phasing.AUTO)		// autophasing TODO
		{
			auto_phase.do_auto_phase(gp, p, q);	// only do baseline flattening version
			if (q.SFO1 > 298.0)					// for 1H or 19F (assuming 1H >= 300 MHz)
				auto_phase.do_auto_phase_2(gp, p, q);
            if (gp.log_on) util.log_error("autophase finished");
		}*/

        else if (p.PH_mod == SW_main.phasing.MAGNITUDE)	// magnitude calculation
        {
            Fft.do_mc(gp.data_array, gp.i_data_array, q.SI);
            if (gp.log_on) Util.log_error("magnitude calculation finished");
        }

        else if (p.PH_mod == SW_main.phasing.POWER)
        {
            Fft.do_ps(gp.data_array, gp.i_data_array, q.SI);
            if (gp.log_on) Util.log_error("power spectrum finished");
        }

        gp.data_max = Math.abs(Util.dscan(gp.data_array, q.SI, 0, q.SI));
        if (gp.data_max > 0.0  && gp.v_rescale)                          // paranoia
            gp.expt_scale = (double)gp.dysize * 0.8 / gp.data_max;

//housekeeping
        q.hzperpt = q.SW * q.SFO1 / (double)(q.SI);	        // update digital resolution
        gp.cursor_flag = 0;                                 // no cursors on screen
        gp.display_mode = SW_main.modes.MODE_1D;
        gp.v_rescale = false;

		return true;    // something more meaningful later
	}
//-------------------------------------------------------------------------------------------------
}
