
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import lp.Lp;

/**
 * @author Kirk Marat,
 * University of Manitoba
 */
public class Proc_3D 
{
	 public static float[][][] XYZ;   	// 3D cubes, rrr
     public static float[] temp_i;  	// temporary imaginaries
	
//-------------------------------------------------------------------------------------------------
 /**
  * load planes processed in X and Y but still time domain in Z
  * @param q status parameters in acquisition dimension (X)
  * @param q_f2 status parameters in F2 (Y)
  * @param q_f1 status parameters in F1 (Z)
  * @return bytes read if OK, -1 if file can't be opened, -3 on IO error
  */
     public static int load_XY_t(data_par q, data_par q_f2, data_par q_f1)
     //                                   X              Y              Z
     {
         String fileNameRR;
         FileInputStream fs;
         int bytes_read = 0;
         int i, j, k;

         for (k = 0; k < q_f1.TD; k++)              // loop over F1 (Z)
         {
        	 fileNameRR = String.format("%sSW_3D/3D_XYt_rr.%d", q.data_folder , k);
        	 File file = new File(fileNameRR);

        	 try { fs = new FileInputStream(file); }
        	 catch (IOException ex)
        	 {
        		 Util.show_and_log("load_XY_t(): cannot open: " + fileNameRR, ex.getMessage());
        		 return -1;
        	 }

        	 byte[] b = new byte[(int)file.length()];
        	 try { fs.read(b); } 
        	 catch (IOException ex) 
        	 {
        		 ex.printStackTrace();
        		 try { fs.close(); } 
        		 catch (IOException e) { e.printStackTrace(); }
        		 return -3;
        	 }

        	 ByteBuffer bb = ByteBuffer.wrap(b);		
        	 bb.order(ByteOrder.LITTLE_ENDIAN); //SpinWorks processed data always LITTLE ENDIAN

        	 try
        	 {
        		 for (i = 0; i < q_f2.SI; i++)       // loop over F2 (Y)
        			 for (j = 0; j < q.SI; j++)      // loop over F3 (X)
        			 {
        				 Proc_3D.XYZ[j][i][k] = bb.getFloat();
        				 bytes_read = bytes_read + 4;
        			 }
        	 }
        	 catch (Exception ex)
        	 {
        		 Util.show_and_log(String.format("load_plane_3D():\n  {0}:  " + ex.getMessage()));
        		 try { fs.close(); } 
        		 catch (IOException e) { e.printStackTrace(); }
        		 return -2;  // file opened but read error
        	 }

        	 try { fs.close(); } 
    		 catch (IOException e) { e.printStackTrace(); }
         }   // for k

         return bytes_read;       
     }

//-------------------------------------------------------------------------------------------------
 /**
  * load planes processed in X and Z but still time domain in Y
  *@param q status parameters in acquisition dimension (X)
  * @param q_f2 status parameters in F2 (Y)
  * @param q_f1 status parameters in F1 (Z)
  * @return bytes read if OK, -1 if file can't be opened, -3 on IO error
  */
     public static int load_XZ_t(data_par q, data_par q_f2, data_par q_f1)
     {
    	 String fileNameRR;
         FileInputStream fs;
         int bytes_read = 0;
         int i, j, k;

         for (k = 0;  k < q_f2.TD; k++)              // loop over F2 (Y)
         {
        	 fileNameRR = String.format("%sSW_3D/3D_XZt_rr.%d", q.data_folder , k);
        	 File file = new File(fileNameRR);

        	 try { fs = new FileInputStream(file); }
        	 catch (IOException ex)
        	 {
        		 Util.show_and_log("load_XZ_t(): cannot open: " + fileNameRR, ex.getMessage());
        		 return -1;
        	 }

        	 byte[] b = new byte[(int)file.length()];
        	 try { fs.read(b); } 
        	 catch (IOException ex) 
        	 {
        		 ex.printStackTrace();
        		 try { fs.close(); } 
        		 catch (IOException e) { e.printStackTrace(); }
        		 return -3;
        	 }

        	 ByteBuffer bb = ByteBuffer.wrap(b);		
        	 bb.order(ByteOrder.LITTLE_ENDIAN); //SpinWorks processed data always LITTLE ENDIAN

             try
             {

                 for (i = 0; i < q_f1.SI; i++)       // loop over F1 (Z)
                     for (j = 0; j < q.SI; j++)      // loop over F3 (X)
                     {
                         Proc_3D.XYZ[j][k][i] = bb.getFloat();
                         bytes_read = bytes_read + 4;
                     }
             }
             catch (Exception ex)
             {
                 Util.show_and_log(String.format("load_plane_3D():\n  %d:  " + ex.getMessage()));
                 try { fs.close(); } 
        		 catch (IOException e) { e.printStackTrace(); }
                 return -2;  // file opened but read error
             }

             try { fs.close(); } 
    		 catch (IOException e) { e.printStackTrace(); } 
         }   // for k

         return bytes_read;
     }
     
//-------------------------------------------------------------------------------------------------
 /**
  * Process the columns of a 3D file (F1 or Z transforms transforms of XY planes)
  * @param gp a SpinWorks global parameter object
  * @param p set pars in acuisition dimension
  * @param q status parameters in acuisition dimension
  * @param p_f2 set parameters in F2 (Y)
  * @param q_f2 status parameters in F2 (Y)
  * @param p_f1 set parameters in F1 (Z)
  * @param q_f1 status parameters in F1 (Z)
  * @return true if OK, false on error
  */
     public static boolean proc_Z(global_par gp, data_par p, data_par q,
         data_par p_f2, data_par q_f2, data_par p_f1, data_par q_f1)
     {
         int i, j;

         switch (p_f1.MC2)
         {
             case QF:
             case STATES:
             case STATES_TPPI:
             case ECHO_ANTIECHO:
                 Util.alloc_proc_buffer(2 * q_f1.SI, gp);
                 break;

             case TPPI:
             case SEQUENTIAL:
                 Util.alloc_proc_buffer(4 * q_f1.SI, gp);
                 break;

             default:
                 Util.show_and_log("proc_Z(): unknown detection mode");
                 return false;
         }

         if (p_f1.MC2 == SW_main.det_modes_IND.QF)							// two quadrant transform
         {
            Util.show_and_log(String.format(
                 "Magnitude mode not supported in 3D"));
             return false;
         }

         
         q_f1.td_eff = q_f1.TD;
         
         SW_main.progressBar.setMaximum(SW_main.q_f2.TD);

         // must loop over all points in plane
         for (i = 0; i < q_f2.SI; i++)	    // loop over all F2 (Y) points
         {
             for (j = 0; j < q.SI; j++)          // loop over all F3 (X detection) points
             {
                 if (load_fid_3D_Z(j, i, XYZ, gp, p_f1, q_f1) == false)	// real	
                 {
                     Util.log_error(String.format("Cannot load Z column # %d, %d", i, j) + " fqt");
                     return false;
                 }

                 SW_main.current_column = (i+1) * (j+1);

                 if (p_f1.lp == SW_main.LP_modes.FORWARD_REAL
                     || p_f1.lp == SW_main.LP_modes.FORWARD_COMPLEX)// FORWARD LP
                 {

                     if (Lp.do_ZhuBax_LP(SW_main.direction.INDIRECT, gp, p, q, p_f1, q_f1) == false)
                         break;
                 }
                 else if (p_f1.lp == SW_main.LP_modes.ZHU_BAX)
                 {

                     if (Lp.do_ZhuBax_LP(SW_main.direction.INDIRECT, gp, p, q, p_f1, q_f1) == false)
                         break;
                 }

                 if (p_f1.MC2 == SW_main.det_modes_IND.TPPI
                     || p_f1.MC2 == SW_main.det_modes_IND.SEQUENTIAL) 		// fp correction for
                     gp.proc_buffer[0] = gp.proc_buffer[0] * p_f1.fcor;	  	// real transform
                 else
                 {
                     gp.proc_buffer[0] = gp.proc_buffer[0] * p_f1.fcor;    	// first point correction
                     gp.proc_buffer[1] = gp.proc_buffer[1] * p_f1.fcor;	  	// for complex transform
                 }

                 if (gp.log_on && SW_main.current_column == 1) Util.log_error(String.format(
                     "Windowing Z  4QT real wdw_fcn = {0}", p_f1.WDW));

                 // the direction definitions need to be cleaned up a bit.
                 Proc_2D.window_2D(SW_main.direction.INDIRECT, gp, p, q, p_f1, q_f1);

                 Proc_2D.do_fft_ind(gp, p_f1, q_f1);
                 post_proc_3D_Z(j, i, XYZ,  temp_i, gp, p_f1, q_f1);  // extract into  arrays

                 if (p_f1.PH_mod == SW_main.phasing.CONSTANTS)   // apply any requested phasing
                     do_pk_3D_Z(j, i, XYZ, temp_i, q_f1.SI, p_f1.PHC0, p_f1.PHC1);

         // Imaginary quadrants here if necessary

             }  // end for j             
             //ProgressBarPerformStepWithText(new Object(), new EventArgs());
             SW_main.statusStripLabel3.setText(String.format("processing col %d  of %d", 
            		 SW_main.current_column, q.SI * q_f2.SI));
             SW_main.progressBar.setSelection(i);
         }  // end for i

         // do magnitude correction, if requested... possibly later
         
         q_f1.WDW = p_f1.WDW;							    // update status prameters	
         q_f1.PH_mod = p_f1.PH_mod;
         q_f1.MC2 = p_f1.MC2;

         if (q_f1.PH_mod == SW_main.phasing.CONSTANTS)
         {
             q_f1.PHC0 = p_f1.PHC0;
             q_f1.PHC1 = p_f1.PHC1;
         }

         Util.free_proc_buffer(gp);                          // legacy call
       
         Util_2D.free_rows(false, true, true, true, gp);     // free space used 
                                                             // by imaginaries         

         gp.data_max_2D = Util_2D.max_2D(gp.rows_real, q.SI, q_f1.SI);		// TODO need 3D equivalents
         gp.data_noise = Util_2D.noise_2D(gp, q, q_f1);
         
         SW_main.progressBar.setSelection(0);
         return true;
     }

//-------------------------------------------------------------------------------------------------
 /**
  * Process the columns of a 2D file (F2 or Y transforms of processed XZ)
  * @param gp a SpinWorks global parameter object
  * @param p set pars in acuisition dimension
  * @param q status parameters in acuisition dimension
  * @param p_f2 set parameters in F2 (Y)
  * @param q_f2 status parameters in F2 (Y)
  * @param p_f1 set parameters in F1 (Z)
  * @param q_f1 status parameters in F1 (Z)
  * @return true if OK, false on error
  */
     public static boolean proc_Y(global_par gp, data_par p, data_par q,
         data_par p_f2, data_par q_f2, data_par p_f1, data_par q_f1)
     {
         int i, j;

         switch (p_f2.MC2)
         {
             case QF:
             case STATES:
             case STATES_TPPI:
             case ECHO_ANTIECHO:
                 Util.alloc_proc_buffer(2 * q_f2.SI, gp);
                 break;

             case TPPI:
             case SEQUENTIAL:
                 Util.alloc_proc_buffer(4 * q_f2.SI, gp);
                 break;

             default:
                 Util.show_and_log("proc_Y(): unknown detection mode");
                 return false;
         }

         if (p_f2.MC2 == SW_main.det_modes_IND.QF)							// two quadrant transform
         {
            Util.show_and_log("Magnitude mode not supported in 3D");
             return false;
         }


         q_f2.td_eff = q_f2.TD;

         SW_main.progressBar.setMaximum(q_f1.SI);

         // must loop over all points in plane
         for (i = 0; i < q_f1.SI; i++)	    // loop over all F2 (Z) points
         {
             for (j = 0; j < q.SI; j++)          // loop over all F3 (X detection) points
             {
                 if (load_fid_3D_Y(j, i, XYZ, gp, p_f2, q_f2) == false)	// real	
                 {
                     Util.show_and_log(String.format("Cannot load Y column # %d, %d", i, j) + " fqt");
                     return false;
                 }

                 SW_main.current_column = (i + 1) * (j + 1);

                 if (p_f2.lp == SW_main.LP_modes.FORWARD_REAL
                     || p_f2.lp == SW_main.LP_modes.FORWARD_COMPLEX)// FORWARD LP
                 {

                     if (Lp.do_ZhuBax_LP(SW_main.direction.INDIRECT, gp, p, q, p_f2, q_f2) == false)//<<<< ?????????
                         break;  
                 }
                 else if (p_f2.lp == SW_main.LP_modes.ZHU_BAX)
                 {

                     if (Lp.do_ZhuBax_LP(SW_main.direction.INDIRECT, gp, p, q, p_f2, q_f2) == false)//<<<<<<<<< ??????
                         break;
                 }

                 if (p_f2.MC2 == SW_main.det_modes_IND.TPPI
                     || p_f2.MC2 == SW_main.det_modes_IND.SEQUENTIAL)	// fp correction for
                     gp.proc_buffer[0] = gp.proc_buffer[0] * p_f2.fcor;	// real transform
                 else
                 {
                     gp.proc_buffer[0] = gp.proc_buffer[0] * p_f2.fcor;	// first point correction
                     gp.proc_buffer[1] = gp.proc_buffer[1] * p_f2.fcor;	// for complex transform
                 }

                 if (gp.log_on && SW_main.current_column == 1) Util.log_error(String.format(
                     "Windowing Z  4QT real wdw_fcn = {0}", p_f2.WDW));

                 Proc_2D.window_2D(SW_main.direction.INDIRECT, gp, p, q, p_f2, q_f2); // <<< ???

                 Proc_2D.do_fft_ind(gp, p_f2, q_f2);
                 post_proc_3D_Y(j, i, XYZ, temp_i, gp, p_f2, q_f2);		// extract into  arrays

                 if (p_f2.PH_mod == SW_main.phasing.CONSTANTS)
                     do_pk_3D_Y(j, i, XYZ, temp_i, q_f2.SI, p_f2.PHC0, p_f2.PHC1);

                 // Inaginary quadrants here if necessary

             }  // end for j
             
             //ProgressBarPerformStepWithText(new Object(), new EventArgs());
             SW_main.progressBar.setSelection(i);
             
         }  // end for i


         // do magnitude correction, if requested... possibly later

         q_f2.WDW = p_f2.WDW;							// update status prameters	
         q_f2.PH_mod = p_f2.PH_mod;
         q_f2.MC2 = p_f2.MC2;

         if (q_f2.PH_mod == SW_main.phasing.CONSTANTS)
         {
             q_f2.PHC0 = p_f2.PHC0;
             q_f2.PHC1 = p_f2.PHC1;
         }

         Util.free_proc_buffer(gp);                      // legacy call

         Util_2D.free_rows(false, true, true, true, gp);

         gp.data_max_2D = Util_2D.max_2D(gp.rows_real, q.SI, q_f2.SI);	// TODO need 3D equivalents!!???
         gp.data_noise = Util_2D.noise_2D(gp, q, q_f2);

         //ProgressBarReset(new Object(), new EventArgs());
         SW_main.progressBar.setSelection(0);

         return true;
     }

//-------------------------------------------------------------------------------------------------
// extract an column (F1 or Z fid) from a processed "XYt" cube
     public static boolean load_fid_3D_Z(int m, int n, float[][][] cube, global_par gp,
                      data_par p_f1, data_par q_f1)
     {
         int i, td;

         if (cube == null || gp.proc_buffer == null)
         {
             Util.show_and_log(
                 "load_fid_3D_Z(): Space allocation error: cube or proc_buffer null");
             return false;
         }

         Util.zero_array(gp.proc_buffer);

         //	set effective td and extract data from column
         if (q_f1.TD <= (q_f1.SI))										// truncate if td longer 
             td = q_f1.TD;												// than size (rare case!)
         else
             td = q_f1.SI;

         for (i = 0; i < td; i = i + 2)									// load actual data 
         {
             gp.proc_buffer[i] = (double)cube[m][n][i];
             if (p_f1.REVERSE)                                      	// reverse data if
                 gp.proc_buffer[i + 1] = -(double)cube[m][n][i + 1]; 	// requested by inverting
             else                                                    	// alternate data points
                 gp.proc_buffer[i + 1] = (double)cube[m][n][i + 1];
         }

         if (p_f1.MC2 == SW_main.det_modes_IND.STATES_TPPI)
         {
             for (i = 0; i < td; i = i + 4)
             {
                 gp.proc_buffer[i + 2] = -(gp.proc_buffer[i + 2]);		//  convert to 
                 gp.proc_buffer[i + 3] = -(gp.proc_buffer[i + 3]);		//  equivalent to STATES
             }
         }

         return true;								// something meaningful later
     }

//-------------------------------------------------------------------------------------------------
// extract an column (F2 or Y fid) from a processed "XZt" cube
     public static boolean load_fid_3D_Y(int m, int n, float[][][] cube, global_par gp,
                      data_par p_f2, data_par q_f2)
     {
         int i, td;

         if (cube == null || gp.proc_buffer == null)
         {
             Util.show_and_log(
                 "load_fid_3D_Z(): Space allocation error: cube or proc_buffer null");
             return false;
         }

         Util.zero_array(gp.proc_buffer);

         //	set effective td and extract data from column
         if (q_f2.TD <= (q_f2.SI))									// truncate if td longer 
             td = q_f2.TD;											// than size (rare case!)
         else
             td = q_f2.SI;

         for (i = 0; i < td; i = i + 2)								 // load actual data 
         {
             gp.proc_buffer[i] = (double)cube[m][i][n];
             if (p_f2.REVERSE)                                       // reverse data if
                 gp.proc_buffer[i + 1] = -(double)cube[m][i + 1][n]; // requested by inverting
             else                                                    // alternate data points
                 gp.proc_buffer[i + 1] = (double)cube[m][i + 1][n];
         }

         if (p_f2.MC2 == SW_main.det_modes_IND.STATES_TPPI)
         {
             for (i = 0; i < td; i = i + 4)
             {
                 gp.proc_buffer[i + 2] = -(gp.proc_buffer[i + 2]);	  //  convert to 
                 gp.proc_buffer[i + 3] = -(gp.proc_buffer[i + 3]);	  //  equivalent to STATES
             }
         }

         return true;								// something meaningful later
     }

//-------------------------------------------------------------------------------------------------
// handle data extraction and ordering for a 3D column in Z direction
     public static void post_proc_3D_Z(int m, int n, float[][][] cube, 
         float[] imag, global_par gp, data_par p_ind, data_par q_ind)
     {
         int i;

         switch (p_ind.MC2)
         {
             case QF:
             case STATES:
             case STATES_TPPI:
             case ECHO_ANTIECHO:
                 
                 for (i = q_ind.SI / 2; i < q_ind.SI; i++)		// extract real part of data 
                     cube[m][n][i] = (float)(gp.proc_buffer[2 * (i - q_ind.SI / 2)]);
                 for (i = 0; i < q_ind.SI / 2; i++)
                     cube[m][n][i] = (float)(gp.proc_buffer[2 * (i + q_ind.SI / 2)]);

                 for (i = q_ind.SI / 2; i < q_ind.SI; i++)		// extract imag. part of data 
                     imag[i] = (float)(gp.proc_buffer[2 * (i - q_ind.SI / 2) + 1]);
                 for (i = 0; i < q_ind.SI / 2; i++)
                     imag[i] = (float)(gp.proc_buffer[2 * (i + q_ind.SI / 2) + 1]);     
        
                 break;

             case SEQUENTIAL:
             case TPPI:
                 
                 for (i = 0; i < q_ind.SI; i++)
                 {
                     cube[m][n][q_ind.SI - 1 - i]
                         = (float)(gp.proc_buffer[(2 * q_ind.SI - 1) - (2 * i + 1)]);
                     imag[q_ind.SI - 1 - i]
                         = (float)(gp.proc_buffer[(2 * q_ind.SI - 1) - (2 * i)]);
                 }

                 break;

             default:
                 break;
         }
     }

//-------------------------------------------------------------------------------------------------
// handle data extraction and ordering for a 3D column in Y direction
     public static void post_proc_3D_Y(int m, int n, float[][][] cube, float[] imag,
         global_par gp, data_par p_ind, data_par q_ind)
     {
         int i;

         switch (p_ind.MC2)
         {
             case QF:
             case STATES:
             case STATES_TPPI:
             case ECHO_ANTIECHO:
                
                 for (i = q_ind.SI / 2; i < q_ind.SI; i++)		// extract real part of data 
                     cube[m][i][n] = (float)(gp.proc_buffer[2 * (i - q_ind.SI / 2)]);
                 for (i = 0; i < q_ind.SI / 2; i++)
                     cube[m][i][n] = (float)(gp.proc_buffer[2 * (i + q_ind.SI / 2)]);

                 for (i = q_ind.SI / 2; i < q_ind.SI; i++)		// extract imag. part of data 
                     imag[i] = (float)(gp.proc_buffer[2 * (i - q_ind.SI / 2) + 1]);
                 for (i = 0; i < q_ind.SI / 2; i++)
                     imag[i] = (float)(gp.proc_buffer[2 * (i + q_ind.SI / 2) + 1]);

                 break;

             case SEQUENTIAL:
             case TPPI:
                 
                 for (i = 0; i < q_ind.SI; i++)
                 {
                     cube[m][q_ind.SI - 1 - i][n]
                         = (float)(gp.proc_buffer[(2 * q_ind.SI - 1) - (2 * i + 1)]);
                     imag[q_ind.SI - 1 - i] 
                         = (float)(gp.proc_buffer[(2 * q_ind.SI - 1) - (2 * i)]);
                 }

                 break;

             default:
                 break;
         }
     }

//-------------------------------------------------------------------------------------------------
// phase correct column in Z direction at x,y = m,n.  Since only the real, real, real cube is 
// saved, the imaginaries for the column are passed as a 1D vector.
     public static void do_pk_3D_Z(int m, int n, float[][][] cube, 
         float[] imag, int si, double phc0, double phc1)
     {
         int i;
         double a, t_r, t_i;

         for (i = 0; i < si; i++)   // now do the phasing 
         {
             a = 0.0174533 * (phc0 + ((double)i / (double)si) * phc1);
             t_r = (double)cube[m][n][i];
             t_i = (double)imag[i];
             cube[m][n][i] = (float)(t_r * Math.cos(a) - t_i * Math.sin(a));
             imag[i] = (float)(t_i * Math.cos(a) + t_r * Math.sin(a));
         }
     }

//-------------------------------------------------------------------------------------------------
// phase correct column in Y direction at x,z = m,n. Since only the real, real, real cube is 
// saved, the imaginaries for the column are passed as a 1D vector.

     public static void do_pk_3D_Y(int m, int n, float[][][] cube, 
         float[] imag, int si, double phc0, double phc1)
     {
         int i;
         double a, t_r, t_i;

         for (i = 0; i < si; i++)   // now do the phasing 
         {
             a = 0.0174533 * (phc0 + ((double)i / (double)si) * phc1);
             t_r = (double)cube[m][i][n];
             t_i = (double)imag[i];
             cube[m][i][n] = (float)(t_r * Math.cos(a) - t_i * Math.sin(a));
             imag[i] =  (float)(t_i * Math.cos(a) + t_r * Math.sin(a));
         }
     }
     
 //------------------------------------------------------------------------------------------------        
}
