package spinworks;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Display;

public class Processing2DMenuListeners extends SW_main
{
//	Processing 2D sub Menu listeners
	public static void addProcessingMenuListeners()
	{
		symmetrizeMenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (gp.rows_real == null)
				{
					Util.show_and_log("There are no processed 2D data");
					return;
				}

				if (q.SI != q_f1.SI)
				{
					Util.show_and_log(String.format(" F1 size %d does not equal F2 size %d, Symmitrization aborted",
							q_f1.SI, q.SI));
					return;
				}

				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
				statusStripLabel1.setText("Symmetrizing 2D Data");

	            Proc_2D.sym_2D(0, gp, q_f1);

				Util_2D.calc_scale_factors_2D(gp, q, q_f1);	// calculate scaling factors, etc.
				gp.block_2D = 0;
				Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);

				if (gp.auto_save)
					FileUtil.saveProc2D(false, gp, q, q_f1);

				statusStripLabel2.setText("2D Symmetrization Complete");
				statusStripLabel1.setText("Ready");
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));

				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}
		});
		
		smoothMenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (gp.rows_real == null)
				{
					Util.show_and_log("There are no processed 2D data");
					return;
				}

				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
				statusStripLabel1.setText("Smoothing 2D Data");

	            Proc_2D.smooth_2D(gp, q, q_f1);

				Util_2D.calc_scale_factors_2D(gp, q, q_f1);	// calculate scaling factors, etc.
				gp.block_2D = 0;
				Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);

				if (gp.auto_save)
					FileUtil.saveProc2D(false, gp, q, q_f1);
				
				statusStripLabel2.setText("2D Smooth Complete");
				statusStripLabel1.setText("Ready");
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));

				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();	
			}
		});
		
		baselineCorrectF2MenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (gp.rows_real == null)
	            {
	                Util.show_and_log("There are no processed 2D data");
	                return;
	            }

				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
				statusStripLabel1.setText("Performing Whittiker baseline correction in F2");

				for (int i = 0; i < q_f1.SI; i++)
				{
					Util_2D.load_display_2D_direct(i, gp, p, q);	// row by row correction
	                BaseLine.whittaker_bl(gp.data_array, gp.i_data_array,
	                    q.SI, p.baseline_filter, p.baseline_thresh, p.baseline_lambda);
					Util_2D.store_row_direct(i, gp, p, q);
				}

	            q.baseline_filter = p.baseline_filter;
	            q.baseline_thresh = p.baseline_thresh;
	            q.baseline_lambda = p.baseline_lambda;

				if (gp.auto_save)
					FileUtil.saveProc2D(false, gp, q, q_f1);

				statusStripLabel2.setText("Whittaker F2 baseline correction complete");
				statusStripLabel1.setText("Ready");
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
				
				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}
		});
		
		baselineCorrectF1MenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (gp.rows_real == null)
	            {
	                Util.show_and_log("There are no processed 2D data");
	                return;
	            }

	            if (p_f1.MC2 == det_modes_IND.QF && gp.dimensions_processed == 2)
	            {
	               Util.show_and_log("command cannot be used on magnitude mode data");
	                return;
	            }

	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
				statusStripLabel1.setText("Performing Whittiker baseline correction in F1");
	           
	            double[] array = new double[q_f1.SI];
	            double[] i_array = new double[q_f1.SI];

	            for (int i = 0; i < q.SI; i++)	// column by column correction
	            {
	                for (int j = 0; j < q_f1.SI; j++)
	                {
	                    array[j] = (double)gp.rows_real[j][i];
	                    i_array[j] = (double)gp.irows_real[j][i];
	                }

	                BaseLine.whittaker_bl(array, i_array, p_f1.SI,
	                    p_f1.baseline_filter, p_f1.baseline_thresh, q_f1.baseline_lambda);

	                for (int j = 0; j < q_f1.SI; j++)
	                {
	                    gp.rows_real[j][i]= (float)array[j]; 
	                    gp.irows_real[j][i] = (float)i_array[j];
	                }
	            }

	            q_f1.baseline_filter = p_f1.baseline_filter;
	            q_f1.baseline_thresh = p_f1.baseline_thresh;
	            q_f1.baseline_lambda = p_f1.baseline_lambda;

				if (gp.auto_save)	
					FileUtil.saveProc2D(false, gp, q, q_f1);

				statusStripLabel2.setText("Whittaker F1 baseline correction complete");
				statusStripLabel1.setText("Ready");
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
				
				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}
		});
		
		driftCorrectF2MenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (gp.rows_real == null)
				{
					Util.show_and_log("There are no processed 2D data");
					return;
				}

				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
				statusStripLabel1.setText("Performing drift correction correction in F2");

				for (int i = 0; i < q_f1.SI; i++)
				{
					Util_2D.load_display_2D_direct(i, gp, p, q);	// row by row correction
					BaseLine.drift_corr(gp.data_array, q.SI);		// uses 1D version
					Util_2D.store_row_direct(i, gp, p, q);			// store corrected row
				}

				statusStripLabel2.setText("drift correction (F2) finished");
				statusStripLabel1.setText("Ready");
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));

				if (gp.auto_save)
					FileUtil.saveProc2D(false, gp, q, q_f1);

				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}
		});
		
		driftCorrectF1MenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (gp.rows_real == null)	
				{
					Util.show_and_log("There are no processed 2D data");
					return;
				}

				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
				statusStripLabel1.setText("Performing drift correction correction in F1");

				for (int i = 0; i < q.SI; i++)	// column by column correction
					BaseLine.drift_corr(gp.rows_real, i, q_f1.SI);

				if (gp.auto_save)
					FileUtil.saveProc2D(false, gp, q, q_f1);

				statusStripLabel2.setText("drift correction (F1) finished");
				statusStripLabel1.setText("Ready");
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));

				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}
		});
		
		calculateF2ProjectionMenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				Util_2D.calc_F2_projection(gp, q, q_f1);
				String fname = q.data_folder + "f2_proj.dx";
				DX_Util.write_jdx_f2_proj(fname, gp, q);	
				update_spect = true;
				canvas.redraw();
			}
		});
		
		calculateF1ProjectionMenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				Util_2D.calc_F1_projection(gp, q, q_f1);
				String fname = q.data_folder + "f1_proj.dx";
				DX_Util.write_jdx_f1_proj(fname, gp, q, q_f1);
				update_spect = true;
				canvas.redraw();	
			}
		});
		
		calculateF2SumMenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				 Util_2D.calc_F2_sum(gp, q, q_f1);
				 String fname = q.data_folder + "f2_sum.dx";
				 DX_Util.write_jdx_f2_proj(fname, gp, q);
		         update_spect = true;
		         canvas.redraw();	
			}
		});
		
		calculateF1SumMenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				Util_2D.calc_F1_sum(gp, q, q_f1);
				String fname = q.data_folder + "f1_sum.dx";
				DX_Util.write_jdx_f1_proj(fname, gp, q, q_f1);
	            update_spect = true;
				canvas.redraw();	
			}
		});
		
		clearImagMenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				//MenuFileSaveProcessed_Click(); // TODO backup imaginary data
				gp.rows_imag = null;
				gp.irows_real = null;
				gp.irows_imag = null;
				System.gc();	//GC.Collect();		// available in Java??  use System.gc()
			}
		});
		
		hilbertTransformF2MenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				Util.alloc_proc_buffer(2*q.SI, gp);
	            if (gp.rows_imag == null) gp.rows_imag = new float[q_f1.SI][q.SI];
	            if (gp.irows_real == null) gp.irows_real = new float[q_f1.SI][q.SI];
	            if (gp.irows_imag == null) gp.irows_imag = new float[q_f1.SI][q.SI];
	            
	            for (int i = 0; i < q_f1.SI; i++)
	                Proc_2D.do_ht_2D_dir(i, gp, q);
			}
		});
		
		hilbertTransformF1MenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				Util.alloc_proc_buffer(2 * q_f1.SI, gp);
	            if (gp.rows_imag == null) gp.rows_imag = new float[q_f1.SI][q.SI];
	            if (gp.irows_real == null) gp.irows_real = new float[q_f1.SI][q.SI];
	            if (gp.irows_imag == null) gp.irows_imag = new float[q_f1.SI][q.SI];

	            for (int i = 0; i < q.SI; i++)
	                Proc_2D.do_ht_2D_ind(i, gp, q_f1);
			}
		});
		

		tiltJResolvedMenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				int shift = 0;
				float[] temp_array = new float[q.SI];
				double ratio = q_f1.hzperpt / q.hzperpt;

				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));

				for (int i = 0; i < q_f1.SI; i++)
				{
					for (int j = 0; j < q.SI; j++)
					{
						temp_array[j] = gp.rows_real[i][j];
						gp.rows_real[i][j] = (float)0.0;
					}

					for (int j = 0; j < q.SI; j++)
					{
						shift = (int) Math.round(ratio * (double)(i - q_f1.SI / 2));
						if(j - shift > 0 && j - shift < q.SI)

							gp.rows_real[i][j] = temp_array[j - shift];
					}
				}

				// clean up
				statusStripLabel2.setText("Tilt complete");
				statusStripLabel1.setText("Ready");
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));

				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}						
		});	
		
		symmetrizeJResolvedMenuItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));

				int i, j;

	            /*if (mode == 0)			            // magnitude data	
	            {*/
				for (i = 0; i < q.SI; i++)      // loop over all columns
				{
					for (j = 0; j < q_f1.SI / 2; j++)
					{
						if (gp.rows_real[j][i] > gp.rows_real[q_f1.SI - 1 - j][i])
							gp.rows_real[j][i] = gp.rows_real[q_f1.SI - 1 - j][i];
						else
							gp.rows_real[q_f1.SI - 1 - j][i] = gp.rows_real[j][i];
					}
				}    
	            //}

/*	            else if (mode == 1)		            // phase sensitive (necessary???)
	            {
	                
	            }*/

				// clean up
				statusStripLabel2.setText("Symmetrize complete");
				statusStripLabel1.setText("Ready");
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));

				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}						
		});	
		
		referenceF1_13CtoCDCl3.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				q_f1.SF = q.SF * 0.251450188;
				Util_2D.auto_ref_ind(gp, p_f1, q_f1);
				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}
		});
		
		referenceF1_13CtoD2O_TSP.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				q_f1.SF = q.SF * 0.251449530;
				Util_2D.auto_ref_ind(gp, p_f1, q_f1);
				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}
		});
		
		referenceF1_13CtoD2O_DSS.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				q_f1.SF = q.SF * 0.251449530;
				Util_2D.auto_ref_ind(gp, p_f1, q_f1);
				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}
		});
		
		referenceF1_13CtoAcetone.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				q_f1.SF = q.SF * 0.2514499771;
				Util_2D.auto_ref_ind(gp, p_f1, q_f1);
				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}
		});
		
		referenceF1_13CtoDMSO.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				q_f1.SF = q.SF * 0.2514503388;
				Util_2D.auto_ref_ind(gp, p_f1, q_f1);
				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}
		});
		
		referenceF1_13CtoBenzene.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				q_f1.SF = q.SF * 0.251450188;
				Util_2D.auto_ref_ind(gp, p_f1, q_f1);
				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}
		});
		
		referenceF1_15N_D2O.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				q_f1.SF = q.SF * 0.101329118;
				Util_2D.auto_ref_ind(gp, p_f1, q_f1);
				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}
		});
		
		referenceF1_Homonuclear.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				q_f1.SF = q.SF;
				Util_2D.auto_ref_ind(gp, p_f1, q_f1);
				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();
			}
		});
		
	}
//-------------------------------------------------------------------------------------------------
}
