/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

/**
 * 
 */
package spinworks;

import java.io.File;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Display;

/**
 * Menu listeners for 3D processing
 * @author Kirk Marat, 
 * University of Manitoba
 */
public class Processing3DMenuListeners extends SW_main
{
	public static void addProcessingMenuListeners()
	{
		// perhaps some of the details in these listeners should be split out
		// into another class
		processFirstXY_Item.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{			
				gp.mode_3D = modes_3D.XY;
	            statusStripLabel1.setText("Processing first XY Plane");
	            statusStripLabel2.setText("");
	            if (gp.log_on) Util.log_error("processing first XY Plane");
	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));

	            try { gp.ser = new double[q_f2.TD][q.TD]; }
	            catch (Exception ex)
	            {
	                Util.show_and_log("get_ser() allocation error", ex.getMessage());
	                return;
	            }

	            if (q.data_type == data_types.BRUKER)
	            {
	                if (q.AQORDER == 0)	//TODO Change AQORDER to enumeration??
	                {
	                    if (FileUtilBruker3D.getSerXY_321(0, gp, q, q_f1, q_f2) <= 0)
	                    {
	                        Util.show_and_log("cannot open: " + q.data_path);
	                        statusStripLabel1.setText("Ready");
	                        statusStripLabel2.setText("Error Processing 3D");
	                        return;
	                    }
	                }
	                else if (q.AQORDER == 1)
	                {
	                    if (FileUtilBruker3D.getSerXY(0, gp, q, q_f1, q_f2) <= 0)
	                    {
	                        Util.show_and_log("cannot open: " + q.data_path);
	                        statusStripLabel1.setText("Ready");
	                        statusStripLabel2.setText("Error Processing 3D");
	                        return;
	                    }
	                }
	                else
	                {
	                    Util.show_and_log("Invalid AQORDER " + q.data_path);
	                    statusStripLabel1.setText("Ready");
	                    statusStripLabel2.setText("Error Processing 3D");
	                    return;
	                }
	            }
	            
	            else if (q.data_type == data_types.VNMR)
	            {
	            	System.out.println(String.format("Varian 3D order = %d" , q.varian_array));
	                int ret_value = 0;

	                if (q.varian_array == 1)	//varian_array =
	                    ret_value = FileUtilVarian3D.get_varian_ser_3D_XY_order_1(0, gp, q, q_f1, q_f2);
	                else if (q.varian_array == 2)
	                    ret_value = FileUtilVarian3D.get_varian_ser_3D_XY_order_2(0, gp, q, q_f1, q_f2);

	                if (ret_value <= 0)
	                {
	                    Util.show_and_log("cannot open: " + q.data_path);
	                    statusStripLabel1.setText("Ready");
	                    statusStripLabel2.setText("Error Processing 3D");
	                    return;
	                }
	            }

	            Proc_2D.proc_rows(gp, p, q, p_f2, q_f2);			// FFT process the rows                          
	            Proc_2D.proc_columns(gp, p, q, p_f2, q_f2);			// FFT process the columns;                            // must handle EA better for 3D
	            FileUtil3D.savePlane3D(modes_3D.XY, 0, gp, q, q_f2);

	            gp.ser = null;
	            System.gc();	//GC.Collect();	// equivalent in Java???
	            gp.current_plane = 0;
	            gp.dimensions_processed = 2;

	            progressBar.setSelection(0);
	            statusStripLabel2.setText("First plane (X-Y) processing complete");
	            statusStripLabel1.setText("Ready");
	            if (gp.log_on) Util.log_error("First plane (X-Y) processing complete");

	            q_f1.SI = p_f1.SI;  // must update other dimension status parameter here
	            FileUtil3D.save_par_3D(gp, q, q_f2, q_f1); 
	            FileUtil3D.loadPlane_XYt(0, gp, q, q_f2);

	            gp.block_2D = 0;							// re-set block (row) counter
	            Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);	// load first row into 1D display	

	            // calculate scaling factors, etc.

	            Util_2D.calc_scale_factors_2D(gp, q, q_f2);	// calculate scaling factors, etc.
	            Util_2D.calc_default_levels_2D(gp);	        // calculate default display levels
	            gp.freq1_indirect = q_f2.offset;
	            gp.freq2_indirect = q_f2.offset - q_f2.SW * q_f2.SFO1;
	            gp.expt_scale = (double)gp.dysize * 0.80 / gp.data_max_2D;
	            gp.display_mode = modes.MODE_2D;			// switch to 2D display
	            gp.new_2D_view = true;						// must re-contour
	            InterfaceUtil.set3D();
	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
	            update_spect = true;
	            canvas.redraw();
			}
		});
		
		processXY_Item.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{				
				gp.mode_3D = SW_main.modes_3D.XY;
				progressBar.setMaximum(q_f1.TD);
				progressBar.setSelection(0);
				statusStripLabel1.setText("Processing XY Planes");
				statusStripLabel2.setText("");
				if (gp.log_on) Util.log_error("Processing XY Planes");

				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));

				try { gp.ser = new double[q_f2.TD][q.TD]; }
				catch (Exception ex)
				{
					Util.show_and_log("Process XY, ser() allocation error", ex.getMessage());
					return;
				}

				for (gp.current_plane = 0; gp.current_plane < q_f1.TD; gp.current_plane++)
				{
					if (q.data_type == data_types.BRUKER)
					{
						if (q.AQORDER == 0)
						{
							if (FileUtilBruker3D.getSerXY_321(gp.current_plane, gp, q, q_f1, q_f2) <= 0)
							{
								Util.show_and_log("cannot open: " + q.data_path);
								statusStripLabel1.setText("Ready");
								statusStripLabel2.setText("Error Processing 3D");
								return;
							}
						}
						else if (q.AQORDER == 1)
						{
							if (FileUtilBruker3D.getSerXY(gp.current_plane,
									gp, q, q_f1, q_f2) <= 0)
							{
								Util.show_and_log("cannot open: " + q.data_path);
								statusStripLabel1.setText("Ready");
								statusStripLabel2.setText("Error Processing 3D");
								return;
							}
						}
						else
						{
							Util.show_and_log(
									String.format("Invalid AQORDER %s:  %d", q.data_path, q.AQORDER));
							statusStripLabel1.setText("Ready");
							statusStripLabel2.setText("Error Processing 3D");
							return;
						}
					}

					else if (q.data_type == data_types.VNMR)
					{
						int ret_value = 0;

						if (q.varian_array == 1)
							ret_value = 
							    FileUtilVarian3D.get_varian_ser_3D_XY_order_1(gp.current_plane, 
									gp, q, q_f1, q_f2);
						else if (q.varian_array == 2)
							ret_value =
								FileUtilVarian3D.get_varian_ser_3D_XY_order_2(gp.current_plane,
									gp, q, q_f1, q_f2);

						if (ret_value <= 0)
						{
							Util.show_and_log("cannot open: " + q.data_path);
							statusStripLabel1.setText("Ready");
							statusStripLabel2.setText("Error Processing 3D");
							return;
						}
					}

					progressBar.setSelection(gp.current_plane);
					statusStripLabel3.setText(String.format(
							"Processing XY Plane %d of %d", gp.current_plane, q_f1.TD));

					Proc_2D.proc_rows(gp, p, q, p_f2, q_f2);			// FFT process the rows                          
					Proc_2D.proc_columns(gp, p, q, p_f2, q_f2);			// FFT process the columns;                            // must handle EA better for 3D
					FileUtil3D.savePlane3D(SW_main.modes_3D.XY, gp.current_plane, gp, q, q_f2);
				}

				gp.ser = null;
				gp.current_plane = 0;
				gp.dimensions_processed = 2;
				progressBar.setSelection(0);
				statusStripLabel1.setText("Ready");
				statusStripLabel2.setText("3D processing (X-Y planes) complete");
				statusStripLabel3.setText("");
				if (gp.log_on) Util.log_error("3D processing (X-Y planes) complete", gp.scratch_folder);

				q_f1.SI = p_f1.SI;  // must update other dimension status parameter here
				FileUtil3D.save_par_3D(gp, q, q_f2, q_f1); 
				FileUtil3D.loadPlane_XYt(0, gp, q, q_f2);

				gp.block_2D = 0;							// re-set block (row)counter
				Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);	// load first row into 1D display	

				// calculate scaling factors, etc.

				Util_2D.calc_scale_factors_2D(gp, q, q_f2);	// calculate scaling factors, etc.
				Util_2D.calc_default_levels_2D(gp);	        // calculate default display levels

				gp.freq1_indirect = q_f2.offset;
				gp.freq2_indirect = q_f2.offset - q_f2.SW * q_f2.SFO1;

				gp.expt_scale = (double)gp.dysize * 0.80 / gp.data_max_2D;

				gp.new_2D_view = true;					// must re-contour
				gp.display_mode = modes.MODE_2D;		// roll into set3D() ???
				InterfaceUtil.set3D();					// probably redundant
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
				update_spect = true;
				canvas.redraw();		            
			}
		});
		
		processZofXY_Item.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{				
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
	            statusStripLabel1.setText("Processing Z");
	            statusStripLabel2.setText("");
	            gp.mode_3D = SW_main.modes_3D.XY;

	            String par_file = q.data_folder + "3D_par";
	            FileUtil3D.getPar3D(par_file, gp, p, p_f2, p_f1, q, q_f2, q_f1);
	            q_f1.SI = p_f1.SI; // update status paramater

	            // Don't wait for system to do this, free memory now!
	            Proc_3D.XYZ = null;
	            System.gc();

	           /* long heapsize = Runtime.getRuntime().totalMemory();
	            System.out.println("heapsize is : " + heapsize);*/
	                        
	            try
	            {   Proc_3D.XYZ = new float[q.SI][q_f2.SI][p_f1.SI];    }
	            catch (Exception ex)
	            {
	                Util.show_and_log("XYZ cube allocation error, probable insufficient memory",
	                    ex.getMessage());
	                return;
	            }
	            
	            /*heapsize = Runtime.getRuntime().totalMemory();
	            System.out.println("heapsize after XYZ allocation  is : " + heapsize);*/

	            Proc_3D.temp_i = new float[p_f1.SI];           
	            Util_2D.alloc_rows(q_f2.SI, q.SI, gp, q, q_f2);

	            statusStripLabel1.setText("reading XYt planes from disk");           

	            int n = Proc_3D.load_XY_t(q, q_f2, q_f1);

	            if (gp.log_on)
	                Util.log_error(String.format("3D Processing Z dimension: bytes read = %d", n));

	            Proc_3D.proc_Z(gp, p, q, p_f2, q_f2, p_f1, q_f1);

	            statusStripLabel1.setText("Saving processed XYZ cube to disk ");
	            FileUtil3D.saveCube(q.data_folder, q.SI, q_f2.SI, q_f1.SI);

	            gp.dimensions_processed = 3;
	            gp.current_plane = 0;

	            gp.display_mode = modes.MODE_2D;			// switch to 2D display
	            Util_3D.load_XY_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, q.SI, q_f2.SI);
	            Util_2D.calc_scale_factors_2D(gp, q, q_f2);	// calculate scaling factors, etc.
	            
	            // find global maximum
	            gp.data_max_3D = Util_3D.max_3D(Proc_3D.XYZ, q.SI, q_f2.SI, q_f1.SI);	
	            gp.expt_scale = (double)gp.dysize * 0.80 / gp.data_max_3D;    
	            Util_2D.calc_default_levels_2D(gp);	        // calculate default display levels
	            
	            statusStripLabel2.setText("3D Z transform complete");
	            statusStripLabel3.setText("");
	            statusStripLabel1.setText("Ready");
	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));      
	            DisplayButtons.fullButtonClick();			// triggers re-draw
			}
		});
		//-----------------
		
		processFirstXZ_Item.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				System.out.println(String.format("Varian 3D order = %d" , q.varian_array));
				gp.mode_3D = SW_main.modes_3D.XZ;      // set 3D mode to XZ

				statusStripLabel1.setText("Processing first XZ Plane");
				statusStripLabel2.setText("");
	            if (gp.log_on) Util.log_error("processing first XZ Plane");
	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));

	            try { gp.ser = new double[q_f1.TD][q.TD]; }
	            catch (Exception ex)
	            {
	                Util.show_and_log("Proc first XZ,  allocation error", ex.getMessage());
	                return;
	            }

	            if (q.data_type == data_types.BRUKER)
	            {
	                if (q.AQORDER == 0)
	                {
	                    if (FileUtilBruker3D.getSerXZ_321(0, gp, q, q_f1, q_f2) <= 0)
	                    {
	                        Util.show_and_log("cannot open: " + q.data_path);
	                        statusStripLabel1.setText("Ready");
	                        statusStripLabel2.setText("Error Processing 3D");
	                        return;
	                    }
	                }
	                else if (q.AQORDER == 1)
	                {
	                    if (FileUtilBruker3D.getSerXZ(0, gp, q, q_f1, q_f2) <= 0)
	                    {
	                        Util.show_and_log(String.format("cannot open: ", q.data_path));
	                        statusStripLabel1.setText("Ready");
	                        statusStripLabel2.setText("Error Processing 3D");
	                        return;
	                    }
	                }
	                else
	                {
	                    Util.show_and_log(String.format("Invalid AQORDER: ", q.data_path));
	                    statusStripLabel1.setText("Ready");
                        statusStripLabel2.setText("Error Processing 3D");
	                    return;
	                }
	            }
	            else if (q.data_type == data_types.VNMR)
	            {
	                int ret_value = 0;
	                if (q.varian_array == 1)
	                    ret_value = FileUtilVarian3D.get_varian_ser_3D_XZ_order_1(0, gp, q, q_f1, q_f2);
	                else if (q.varian_array == 2)
	                    ret_value = FileUtilVarian3D.get_varian_ser_3D_XZ_order_2(0, gp, q, q_f1, q_f2);

	                else if (ret_value <= 0)
	                {               
	                    Util.show_and_log("cannot open: " + q.data_path);
	                    statusStripLabel1.setText("Ready");
	                    statusStripLabel2.setText("Error loading first XZ plane");
	                    return;                   
	                }
	            }

	            Proc_2D.proc_rows(gp, p, q, p_f1, q_f1);			// FFT process the rows                          
	            Proc_2D.proc_columns(gp, p, q, p_f1, q_f1);			// FFT process the columns

	            FileUtil3D.savePlane3D(SW_main.modes_3D.XZ, 0, gp, q, q_f1);

	            progressBar.setSelection(0);
	            statusStripLabel2.setText("First plane (X-Z) processing complete");
	            statusStripLabel1.setText("Ready");
	            if (gp.log_on) Util.log_error("First plane (X-Z) processing complete");

	            gp.ser = null;
	            System.gc();
	            gp.current_plane = 0;
	            gp.dimensions_processed = 2;

	            q_f2.SI = p_f2.SI;  // must update other dimension status parameter here
	            FileUtil3D.save_par_3D(gp, q, q_f2, q_f1);
	            FileUtil3D.loadPlane_XZt(0, gp, q, q_f1);

	            gp.block_2D = 0;							// re-set block (row)counter
	            Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);	// load first row into 1D display	

	            // calculate scaling factors, etc.

	            Util_2D.calc_scale_factors_2D(gp, q, q_f1);	// calculate scaling factors, etc.
	            Util_2D.calc_default_levels_2D(gp);	        // calculate default display levels

	            gp.freq1_indirect = q_f1.offset;
	            gp.freq2_indirect = q_f1.offset - q_f1.SW * q_f1.SFO1;

	            gp.expt_scale = (double)gp.dysize * 0.80 / gp.data_max_2D;
	            gp.display_mode = modes.MODE_2D;		// switch to 2D display
	            gp.new_2D_view = true;					// must re-contour
	            InterfaceUtil.set3D();
	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
	            update_spect = true;
	            canvas.redraw();	            
			}
		});
		
		processXZ_Item.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{				
				gp.mode_3D = SW_main.modes_3D.XZ;      // set 3D mode to XZ

				progressBar.setMaximum(q_f2.TD);
				statusStripLabel1.setText("Processing XZ Planes");
				statusStripLabel2.setText("");
				if (gp.log_on) Util.log_error("processing XZ Planes", gp.scratch_folder);

				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));

				try { gp.ser = new double[q_f1.TD][q.TD]; }
				catch (Exception ex)
				{
					Util.show_and_log("get_ser() allocation error", ex.getMessage());
					return;
				}

				for (gp.current_plane = 0; gp.current_plane < q_f2.TD; gp.current_plane++)
				{
					if (q.AQORDER == 0 && q.data_type == data_types.BRUKER)
					{
						if (FileUtilBruker3D.getSerXZ_321(gp.current_plane, gp, q, q_f1, q_f2) <= 0)
						{
							Util.show_and_log("cannot open: " + q.data_path);
							statusStripLabel1.setText("Ready");
							statusStripLabel2.setText("Error Processing 3D");
							return;
						} 
					} 

					else if (q.AQORDER == 1 && q.data_type == data_types.BRUKER)
					{
						if (FileUtilBruker3D.getSerXZ(gp.current_plane, gp, q, q_f1, q_f2) <= 0)
						{
							Util.show_and_log("cannot open: " + q.data_path);
							statusStripLabel1.setText("Ready");
							statusStripLabel2.setText("Error Processing 3D");
							return;
						}
					}
					else if (q.data_type == data_types.VNMR)
					{
						int ret_value = 0;

						if (q.varian_array == 1)
							ret_value = FileUtilVarian3D.get_varian_ser_3D_XZ_order_1(
									gp.current_plane, gp, q, q_f1, q_f2);
						else if (q.varian_array == 2)
							ret_value = FileUtilVarian3D.get_varian_ser_3D_XZ_order_2(
									gp.current_plane, gp, q, q_f1, q_f2);

						if(ret_value <= 0)
						{
							Util.show_and_log("cannot open: " + q.data_path);
							statusStripLabel1.setText("Ready");
							statusStripLabel2.setText("Error Processing 3D");
							return;
						}
					}
					else
					{
						Util.show_and_log("Invalid AQORDER: " + q.data_path);
						statusStripLabel1.setText("Ready");
						statusStripLabel2.setText("Error Processing 3D");
						return;
					}

					statusStripLabel3.setText(String.format("Processing XZ Plane %d of %d", gp.current_plane, q_f2.TD));
					progressBar.setSelection(gp.current_plane);

					Proc_2D.proc_rows(gp, p, q, p_f1, q_f1);			// FFT process the rows                          
					Proc_2D.proc_columns(gp, p, q, p_f1, q_f1);			// FFT process the columns

					FileUtil3D.savePlane3D(SW_main.modes_3D.XZ, gp.current_plane, gp, q, q_f1);

				}   // for gp.current_plane

				progressBar.setSelection(0);
				statusStripLabel2.setText("3D Processing (XZ planes) complete");
				statusStripLabel3.setText("");
				statusStripLabel1.setText("Ready");
				if (gp.log_on) Util.log_error("3D processing (XZ planes) complete", gp.scratch_folder);

				gp.ser = null;
				System.gc();	//GC.Collect();	// equivalent in Java???
				gp.current_plane = 0;
				gp.dimensions_processed = 2;

				q_f2.SI = p_f2.SI;  // must update other dimension status parameter here
				FileUtil3D.save_par_3D(gp, q, q_f2, q_f1);
				FileUtil3D.loadPlane_XZt(0, gp, q, q_f1);

				gp.block_2D = 0;							// re-set block (row)counter
				Util_2D.load_display_2D_direct(gp.block_2D, gp, p, q);	// load first row into 1D display	

				// calculate scaling factors, etc.

				Util_2D.calc_scale_factors_2D(gp, q, q_f1);	// calculate scaling factors, etc.
				Util_2D.calc_default_levels_2D(gp);	        // calculate default display levels

				gp.freq1_indirect = q_f1.offset;
				gp.freq2_indirect = q_f1.offset - q_f1.SW * q_f1.SFO1;

				gp.expt_scale = (double)gp.dysize * 0.80 / gp.data_max_2D;
				gp.new_2D_view = true;						// must re-contour
				InterfaceUtil.set3D();
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
				update_spect = true;
				canvas.redraw();	            		            		            		            
			}
		});
		
		processYofXZ_Item.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{				
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
				statusStripLabel1.setText("Processing Y Dimension");
				statusStripLabel2.setText("");
				gp.mode_3D = SW_main.modes_3D.XZ;

				String par_file = q.data_folder + "3D_par";
				FileUtil3D.getPar3D(par_file, gp, p, p_f2, p_f1, q, q_f2, q_f1);
				q_f2.SI = p_f2.SI; // update status paramater  Y dimension size

				// Don't wait for system to do this, free memory now!
				Proc_3D.XYZ = null;
				System.gc(); //GC.Collect();	// Java equivalent???

				try
				{   Proc_3D.XYZ = new float[q.SI][p_f2.SI][q_f1.SI]; }
				catch (Exception ex)
				{
					Util.show_and_log("XYZ cube allocation error, possible insufficient memory", 
							ex.getMessage());
					return;
				}
				Proc_3D.temp_i = new float[p_f2.SI];

				Util_2D.alloc_rows(q_f1.SI, q.SI, gp, q, q_f1);

				statusStripLabel1.setText("reading XZt planes from disk");

				int n = Proc_3D.load_XZ_t(q, q_f2, q_f1);

				if (gp.log_on)
					Util.log_error(String.format("3D Processing Y dimension: bytes read = %d", n));

				Proc_3D.proc_Y(gp, p, q, p_f2, q_f2, p_f1, q_f1);

				statusStripLabel1.setText("Saving processed XYZ cube to disk ");
				statusStripLabel2.setText("");
				FileUtil3D.saveCube(q.data_folder, q.SI, q_f2.SI, q_f1.SI);

				gp.dimensions_processed = 3;
				gp.current_plane = 0;

				gp.display_mode = modes.MODE_2D;			// switch to 2D display
				//InterfaceUtil.set3D(); instead??
				
	            Util_3D.load_XZ_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
	            Util_2D.calc_scale_factors_2D(gp, q, q_f1);	// calculate scaling factors, etc.
	            
	            // find global maximum
	            gp.data_max_3D = Util_3D.max_3D(Proc_3D.XYZ, q.SI, q_f2.SI, q_f1.SI);	
	            gp.expt_scale = (double)gp.dysize * 0.80 / gp.data_max_3D;    
	            Util_2D.calc_default_levels_2D(gp);	        // calculate default display levels
	            
	            statusStripLabel2.setText("3D Z transform complete");
	            statusStripLabel3.setText("");
	            statusStripLabel1.setText("Ready");
	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));      
	            DisplayButtons.fullButtonClick();			// triggers re-draw	            	            		            
			}
		});
		//--------------------
		
		bcPlanesDirectItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{								
				if (gp.rows_real == null)
	            {
	                Util.show_and_log("There are no processed 3D data planes loaded");
	                return;
	            }

	            if (gp.dimensions_processed != 3)
	            {
	                Util.show_and_log("The data must be processed in all three dimensions");
	                return;
	            }
	                    
	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
	            statusStripLabel1.setText("F3 baseline correction");
	            statusStripLabel2.setText("");

	            if (gp.mode_3D == modes_3D.XY)              
	            {  
	            	Util_2D.alloc_rows(q_f2.SI, q.SI, gp, q, q_f2);
	            	Util.alloc_proc_buffer(2 * q.SI, gp);
					
				    for (int i = 0; i < q_f1.SI; i++)   // loop over all planes
				    {
	                    Util_3D.load_XY_plane(i, Proc_3D.XYZ, gp.rows_real, q.SI, q_f2.SI);
	                    
	                    for (int j = 0; j < q_f2.SI; j++)  // Can this be combined into the next loop??
	                    {
	                        Proc_2D.do_ht_2D_dir(j, gp,  q);
	                    }
	                    
	                    for (int j = 0; j < q_f2.SI; j++)
	                    {
	                        Util_2D.load_display_2D_direct(j, gp, p, q);	// row by row correction
	                        BaseLine.whittaker_bl(gp.data_array, gp.i_data_array,
	                            q.SI, p.baseline_filter, p.baseline_thresh, p.baseline_lambda);
	                        Util_2D.store_row_direct(j, gp, p, q);
	                    }
	                    Util_3D.store_XY_plane(i, Proc_3D.XYZ, gp.rows_real, q.SI, q_f2.SI);
				    }
	                Util_3D.load_XY_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, q.SI, q_f2.SI);
	            }
	            else if (gp.mode_3D == modes_3D.XZ)
	            {
	            	Util_2D.alloc_rows(q_f1.SI, q.SI, gp, q, q_f1);
	            	Util.alloc_proc_buffer(2 * q.SI, gp);
					
	                for (int i = 0; i < q_f2.SI; i++)   // loop over all planes
	                {
	                    Util_3D.load_XZ_plane(i, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
	                    
	                    for (int j = 0; j < q_f1.SI; j++)	// Can this be combined into the next loop??
	                    {
	                        Proc_2D.do_ht_2D_dir(j, gp, q);
	                    }
	                    
	                    for (int j = 0; j < q_f1.SI; j++)
	                    {
	                        Util_2D.load_display_2D_direct(j, gp, p, q);	// row by row correction
	                        BaseLine.whittaker_bl(gp.data_array, gp.i_data_array,
	                            q.SI, p.baseline_filter, p.baseline_thresh, p.baseline_lambda);
	                        Util_2D.store_row_direct(j, gp, p, q);
	                    }
	                    Util_3D.store_XZ_plane(i, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
	                }
	                Util_3D.load_XZ_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
	            }
	            
	            //update status parameters
	            q.baseline_filter = p.baseline_filter;
	            q.baseline_thresh = p.baseline_thresh;
	            q.baseline_lambda = p.baseline_lambda;

	            // housekeeping
	            statusStripLabel3.setText("Saving XYZ cube to disk");
	            FileUtil3D.saveCube(q.data_folder, q.SI, q_f2.SI, q_f1.SI);
	            statusStripLabel2.setText("Baseline correct complete");
	            statusStripLabel1.setText("Ready");
	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));      
	            gp.new_2D_view = true;
	            update_spect = true;
	            canvas.redraw(); 											
			}
		});
		
		bcPlanesIndirectItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{							
				if (gp.rows_real == null)
				{
					Util.show_and_log("There are no processed 3D data planes loaded");
					return;
				}

				if (gp.dimensions_processed != 3)
				{
					Util.show_and_log("The data must be processed in all three dimensions");
					return;
				}

				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
				if(gp.mode_3D == modes_3D.XY)
					statusStripLabel1.setText("Performing F2 baseline correction");
				else if (gp.mode_3D == modes_3D.XZ)
					statusStripLabel1.setText("Performing F1 baseline correction");
				statusStripLabel2.setText("");

				if (gp.mode_3D == modes_3D.XY)
				{
					Util_2D.alloc_rows(q_f2.SI, q.SI, gp, q, q_f2);
					double[] array = new double[q_f2.SI];
					double[] i_array = new double[q_f2.SI];

					for (int k = 0; k < q_f1.SI; k++)   // loop over all planes
					{
						Util_3D.load_XY_plane(k, Proc_3D.XYZ, gp.rows_real, q.SI, q_f2.SI);

						for (int i = 0; i < q.SI; i++)	// column by column correction
						{
							for (int j = 0; j < q_f2.SI; j++)
							{
								array[j] = (double)gp.rows_real[j][i];
								i_array[j] = (double)gp.irows_real[j][i];
							}

							BaseLine.whittaker_bl(array, i_array, p_f2.SI,
									p_f2.baseline_filter, p_f2.baseline_thresh, q_f2.baseline_lambda);

							for (int j = 0; j < q_f2.SI; j++)
							{
								gp.rows_real[j][i] = (float)array[j];
								gp.irows_real[j][i] = (float)i_array[j];
							}							
						}
						Util_3D.store_XY_plane(k, Proc_3D.XYZ, gp.rows_real, q.SI, q_f2.SI);
					}
					Util_3D.load_XY_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, q.SI, q_f2.SI);
				}

				else if (gp.mode_3D == modes_3D.XZ)
				{
					Util_2D.alloc_rows(q_f1.SI, q.SI, gp, q, q_f1);
					double[] array = new double[q_f1.SI];
					double[] i_array = new double[q_f1.SI];

					for (int k = 0; k < q_f2.SI; k++)   // loop over all planes
					{
						Util_3D.load_XZ_plane(k, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);

						for (int i = 0; i < q.SI; i++)	// column by column correction
						{
							for (int j = 0; j < q_f1.SI; j++)
							{
								array[j] = (double)gp.rows_real[j][i];
								i_array[j] = (double)gp.irows_real[j][i];
							}

							BaseLine.whittaker_bl(array, i_array, p_f1.SI,
									p_f1.baseline_filter, p_f1.baseline_thresh, q_f1.baseline_lambda);

							for (int j = 0; j < q_f1.SI; j++)
							{
								gp.rows_real[j][i] = (float)array[j];
								gp.irows_real[j][i] = (float)i_array[j];
							}
						}
						Util_3D.store_XZ_plane(k, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
					}
					Util_3D.load_XZ_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
				}

				//  update status parameters
				if (gp.mode_3D == modes_3D.XY)
				{
					q_f2.baseline_filter = p_f2.baseline_filter;
					q_f2.baseline_thresh = p_f2.baseline_thresh;
					q_f2.baseline_lambda = p_f2.baseline_lambda;
				}
				else if (gp.mode_3D == modes_3D.XZ)
				{
					q_f1.baseline_filter = p_f1.baseline_filter;
					q_f1.baseline_thresh = p_f1.baseline_thresh;
					q_f1.baseline_lambda = p_f1.baseline_lambda;
				}

				// housekeeping
				statusStripLabel3.setText("Saving XYZ cube to disk");
				FileUtil3D.saveCube(q.data_folder, q.SI, q_f2.SI, q_f1.SI);
				statusStripLabel2.setText("Baseline correct complete");
				statusStripLabel1.setText("Ready");
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));      
				gp.new_2D_view = true;
				update_spect = true;	            		            		            
			}
		});
		//-----------------------
		
		phasePlanesDirectItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (gp.rows_real == null)
	            {
	                Util.show_and_log("There are no processed 3D data planes loaded");
	                return;
	            }

	            if (gp.dimensions_processed != 3)
	            {
	                Util.show_and_log("The data must be processed in all three dimensions");
	                return;
	            }

	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));
	            statusStripLabel1.setText("F3 phase correction");
	            statusStripLabel2.setText("");

	            if (gp.mode_3D == modes_3D.XY)
	            {	            	
	            	Util_2D.alloc_rows(q_f2.SI, q.SI, gp, q, q_f2);
	                for (int i = 0; i < q_f1.SI; i++)   // loop over all planes
	                {	                	
	                    Util_3D.load_XY_plane(i, Proc_3D.XYZ, gp.rows_real, q.SI, q_f2.SI);
	                    Util.alloc_proc_buffer(2 * q.SI, gp);

	                    for (int j = 0; j < q_f2.SI; j++)
	                    {
	                        Proc_2D.do_ht_2D_dir(j, gp,  q);
	                    }  

	                    Proc_2D.phase_rows(gp, p, q, q_f2);
	                    Util_3D.store_XY_plane(i, Proc_3D.XYZ, gp.rows_real, q.SI, q_f2.SI);
	                }
	                Util_3D.load_XY_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, q.SI, q_f2.SI);
	            }

	            else if (gp.mode_3D == modes_3D.XZ)
	            {	            	
	            	Util_2D.alloc_rows(q.SI, q_f1.SI, gp, q, q_f1);
	                for (int i = 0; i < q_f2.SI; i++)   // loop over all planes
	                {
	                    Util_3D.load_XZ_plane(i, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
	                    Util.alloc_proc_buffer(2 * q.SI, gp);

	                    for (int j = 0; j < q_f1.SI; j++)
	                    {
	                        Proc_2D.do_ht_2D_dir(j, gp, q);
	                    }

	                    Proc_2D.phase_rows(gp, p, q, q_f1);
	                    Util_3D.store_XZ_plane(i, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
	                }
	                Util_3D.load_XZ_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
	            }

	            q.PHC0 = p.PHC0;
	            q.PHC1 = p.PHC1;

	            // housekeeping
	            statusStripLabel3.setText("Saving XYZ cube to disk");
	            FileUtil3D.saveCube(q.data_folder, q.SI, q_f2.SI, q_f1.SI);
	            statusStripLabel2.setText("3D Phasing complete");
	            statusStripLabel1.setText("Ready");
	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));      
	            gp.new_2D_view = true;
	            update_spect = true;
	            canvas.redraw();                        
			}
		});
		
		phasePlanesIndirectItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (gp.rows_real == null)
				{
					Util.show_and_log("There are no processed 3D data planes loaded");
					return;
				}

				if (gp.dimensions_processed != 3)
				{
					Util.show_and_log("The data must be processed in all three dimensions");
					return;
				}

				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));

				if (gp.mode_3D == modes_3D.XY)
				{
					Util_2D.alloc_rows(q_f2.SI, q.SI, gp, q, q_f2);
					statusStripLabel1.setText("Performing Y (F2) phase correction");
					statusStripLabel2.setText("");

					for (int i = 0; i < q_f1.SI; i++)   // loop over all planes
					{
						Util_3D.load_XY_plane(i, Proc_3D.XYZ, gp.rows_real, q.SI, q_f2.SI);
						Util.alloc_proc_buffer(2 * q_f2.SI, gp);

						for (int j = 0; j < q.SI; j++)
						{
							Proc_2D.do_ht_2D_ind(j, gp, q_f2);
						}  

						Proc_2D.phase_columns(gp, q, p_f2, q_f2);
						Util_3D.store_XY_plane(i, Proc_3D.XYZ, gp.rows_real, q.SI, q_f2.SI);
					}
					Util_3D.load_XY_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, q.SI, q_f2.SI);
					q_f2.PHC0 = p_f2.PHC0;
					q_f2.PHC1 = p_f2.PHC1;
					statusStripLabel2.setText("Y (F2) phase correction complete");
				}

				else if (gp.mode_3D == modes_3D.XZ)
				{
					Util_2D.alloc_rows(q_f1.SI, q.SI, gp, q, q_f1);
					statusStripLabel1.setText("Performing Z (F1) phase correction");
					statusStripLabel2.setText("");

					for (int i = 0; i < q_f2.SI; i++)   // loop over all planes
					{
						Util_3D.load_XZ_plane(i, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
						Util.alloc_proc_buffer(2 * q_f1.SI, gp);

						for (int j = 0; j < q.SI; j++)
						{
							Proc_2D.do_ht_2D_ind(j, gp, q_f1);
						}

						Proc_2D.phase_columns(gp, q, p_f1, q_f1);
						Util_3D.store_XZ_plane(i, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
					}
					Util_3D.load_XZ_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
					q_f1.PHC0 = p_f1.PHC0;
					q_f1.PHC1 = p_f1.PHC1;
					statusStripLabel2.setText("Z (F1) phase correction complete");
				}

				// housekeeping
				statusStripLabel3.setText("Saving XYZ cube to disk");
				FileUtil3D.saveCube(q.data_folder, q.SI, q_f2.SI, q_f1.SI);
				statusStripLabel2.setText("3D Phasing complete");
				statusStripLabel1.setText("Ready");
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));      
				gp.new_2D_view = true;
				update_spect = true;
				canvas.redraw();          
			}
		});
		//---------------------------
		
		hilbertTransformDirectItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				Util.alloc_proc_buffer(2*q.SI, gp);
	            if (gp.rows_imag == null) gp.rows_imag = new float[q_f1.SI][q.SI];
	            if (gp.irows_real == null) gp.irows_real = new float[q_f1.SI][q.SI];
	            if (gp.irows_imag == null) gp.irows_imag = new float[q_f1.SI][q.SI];
	            
	            for (int i = 0; i < q_f1.SI; i++)
	                Proc_2D.do_ht_2D_dir(i, gp, q);
			}
		});
		
		hilbertTransformIndirectItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (gp.mode_3D == modes_3D.XY)
				{
					Util.alloc_proc_buffer(2 * q_f2.SI, gp);
					if (gp.rows_imag == null) gp.rows_imag = new float[q_f2.SI][q.SI];
					if (gp.irows_real == null) gp.irows_real = new float[q_f2.SI][q.SI];
					if (gp.irows_imag == null) gp.irows_imag = new float[q_f2.SI][q.SI];

					for (int i = 0; i < q.SI; i++)
						Proc_2D.do_ht_2D_ind(i, gp, q_f2);
				}
				else if (gp.mode_3D == modes_3D.XZ)
				{
					Util.alloc_proc_buffer(2 * q_f1.SI, gp);
					if (gp.rows_imag == null) gp.rows_imag = new float[q_f1.SI][q.SI];
					if (gp.irows_real == null) gp.irows_real = new float[q_f1.SI][q.SI];
					if (gp.irows_imag == null) gp.irows_imag = new float[q_f1.SI][q.SI];

					for (int i = 0; i < q.SI; i++)
						Proc_2D.do_ht_2D_ind(i, gp, q_f1);
				}
			}
		});
		//------------------------------
		// try to reclaim some space
		freeImaginariesItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				gp.rows_imag = null;
				gp.irows_real = null;
				gp.irows_imag = null;
				System.gc();	// just gives the system a hint
			}
		});
		
		freeImaginariesOnDiskItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{				
				 File file;

		            for (int i = 0; i < q_f1.TD; i++) // XYt planes
		            {
		                try
		                {		                    	                    
		                    file = new File(String.format("%sSW_3D/3D_XYt_ri.%d",
			                        q.data_folder, i));
		                    file.delete();
		               
		                    file = new File(String.format("%sSW_3D/3D_XYt_ir.%d",
		                            q.data_folder, i));
		                    file.delete();

		                    file = new File(String.format("%sSW_3D/3D_XYt_ii.%d",
		                            q.data_folder, i));
		                    file.delete();
		                }
		                catch (Exception ex) 
		                {
		                    if (gp.log_on)
		                        Util.log_error("freeImaginariesOnDiskItem:" + ex.getMessage());
		                }
		            }

		            for (int i = 0; i < q_f2.TD; i++) // XZt planes
		            {
		                try
		                {
		                	 file = new File(String.format("%sSW_3D/3D_XZt_ri.%d",
				                        q.data_folder, i));
		                	 file.delete();

		                	 file = new File(String.format("%sSW_3D/3D_XZt_ir.%d",
		                			 q.data_folder, i));
		                	 file.delete();

		                	 file = new File(String.format("%sSW_3D/3D_XZt_ii.%d",
		                			 q.data_folder, i));
		                	 file.delete();
		                }
		                catch (Exception ex)
		                {
		                    if (gp.log_on)
		                        Util.log_error("freeImaginariesOnDiskItem: " + ex.getMessage());
		                }
		            }
		            statusStripLabel2.setText("Imaginary planes on disk deleted");		            		            
			}
		});
		
		referenceIndirect15N.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{				
				if (gp.mode_3D == modes_3D.XZ)
				{
					q_f1.SF = q.SF * 0.101329118;
					Util_2D.auto_ref_ind(gp, p_f1, q_f1);
					update_spect = true;
					canvas.redraw();
				}
				else if (gp.mode_3D == modes_3D.XY)
				{
					q_f2.SF = q.SF * 0.101329118;
					Util_2D.auto_ref_ind(gp, p_f2, q_f2);
					update_spect = true;
					canvas.redraw();
				}
			}
		});
		
		referenceIndirect13C.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (gp.mode_3D == modes_3D.XZ)
				{
					q_f1.SF = q.SF * 0.251449530;
					Util_2D.auto_ref_ind(gp, p_f1, q_f1);
					update_spect = true;
					canvas.redraw();
				}
				else if (gp.mode_3D == modes_3D.XY)
				{
					q_f2.SF = q.SF * 0.251449530;
					Util_2D.auto_ref_ind(gp, p_f2, q_f2);
					update_spect = true;
					canvas.redraw();
				}
			}
		});
		
		referenceIndirectHomonuclear.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if (gp.mode_3D == modes_3D.XZ)
				{
					q_f1.SF = q.SF;
					Util_2D.auto_ref_ind(gp, p_f1, q_f1);
					update_spect = true;
					canvas.redraw();
				}
				else if (gp.mode_3D == modes_3D.XY)
				{
					q_f2.SF = q.SF;
					Util_2D.auto_ref_ind(gp, p_f2, q_f2);
					update_spect = true;
					canvas.redraw();
				}
			}
		});		
		
//-------------------------------------------------------------------------------------------------
	}	// end addProcessing3DMenuListeners
//-------------------------------------------------------------------------------------------------
}	// end class
