package spinworks;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;

/**
 * Class for drawing rotated (vertical) text on screen or printer
 * @author marat,
 * University of Manitoba
 */
public class RotateText 
{
//-------------------------------------------------------------------------------------------------
	/**
	 * Creates rotated text as an image
	 * @param text the text
	 * @param font 
	 * @param foreground color
	 * @param background color
	 * @param style SWT.UP or SWT.DOWN
	 * @return the image with the rotated text
	 */
	public static Image createRotatedText(String text, Font font, 
			Color foreground, Color background, int style) 
	{
		// Get the current display
		Display display = Display.getCurrent();
		if (display == null) SWT.error(SWT.ERROR_THREAD_INVALID_ACCESS);
		
		// Create a GC to calculate font's dimensions
		GC gc = new GC(display);
		gc.setFont(font);
		
		// Determine string's dimensions
		Point pt = gc.textExtent(text);
		
		// Dispose that gc
		gc.dispose();
		
		// Create an image the same size as the string
		Image stringImage = new Image(display, pt.x, pt.y);
	
		// Create a gc for the image
		gc = new GC(stringImage);
		gc.setFont(font);
		gc.setForeground(foreground);
		gc.setBackground(background);
		
		// Draw the text onto the image
		gc.drawText(text, 0, 0);
		
		// Draw the image vertically onto the  GC
		Image image = createRotatedImage(stringImage, style);
		
		// Dispose the new GC
		gc.dispose();
		
		// Dispose the horizontal image
		stringImage.dispose();
		
		// Return the rotated image
		return image;
	}
	
//-------------------------------------------------------------------------------------------------
	/**
	 * Rotates an image 90 degrees
	 * @param image the input image
	 * @param style SWT.UP or SWT.DOWN
	 * @return the rotated image
	 */
	 public static Image createRotatedImage(Image image, int style) 
	 {
		// Get the current display
		Display display = Display.getCurrent();
		if (display == null) SWT.error(SWT.ERROR_THREAD_INVALID_ACCESS);
		
		// Use the image's data to create a rotated image's data
		ImageData sd = image.getImageData();
		ImageData dd = new ImageData(sd.height, sd.width, sd.depth, sd.palette);
		
		// Determine which way to rotate, depending on up or down
		boolean up = (style & SWT.UP) == SWT.UP;
		
		// Run through the horizontal pixels
		for (int sx = 0; sx < sd.width; sx++) 
		{
			// Run through the vertical pixels
			for (int sy = 0; sy < sd.height; sy++) 
			{
				// Determine where to move pixel to in destination image data
				int dx = up ? sy : sd.height - sy - 1;
				int dy = up ? sd.width - sx - 1 : sx;
				
				// Swap the x, y source data to y, x in the destination
				dd.setPixel(dx, dy, sd.getPixel(sx, sy));
			}
		}
		
		// Return the vertical image
		return new Image(display, dd);
	 }
//-------------------------------------------------------------------------------------------------
}
