/**
 * 
 */
package spinworks;

import java.awt.Desktop;
import java.io.File;
import java.io.PrintWriter;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.concurrent.TimeUnit;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MenuAdapter;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.MessageBox;

import decon.DeconUtil;
import dialogs.DeconDialog;
import dialogs.DeconPeaksDialog;
import simulation.SS_util;
import simulation.SimOptionsDialog;
import simulation.SimPar;
import simulation.Simpson;

/**
 * @author marat
 *
 */
public class SimulationMenuListeners extends SW_main
{
	public static void addSimulationMenuListeners()
	{
				
		// simulation Menu, update menu items when dropped		
	
		simulationMenu.addMenuListener  (new MenuAdapter() 
		{
			public void menuShown(MenuEvent e) 
			{
				if(sp.n_shifts == 0)
				{
					runSimulationItem.setEnabled(false);
					editSimulationOptionsItem.setEnabled(false);
				}
				else
				{
					runSimulationItem.setEnabled(true);
					editSimulationOptionsItem.setEnabled(true);
				}
				
				if(gp.calc_array == null || gp.data_array == null)
				{
					autoAssignTransitionsItem.setEnabled(false);
					deleteAssignmentsItem.setEnabled(false);
					readAssignedTransitionsItem.setEnabled(false);
					saveAssignedTransitionsItem.setEnabled(false);
					loadOptimizedParametersItem.setEnabled(false);
				}
				else
				{
					autoAssignTransitionsItem.setEnabled(true);
					deleteAssignmentsItem.setEnabled(true);
					readAssignedTransitionsItem.setEnabled(true);
					saveAssignedTransitionsItem.setEnabled(true);
					loadOptimizedParametersItem.setEnabled(true);
				}
				
				if(sp.calc_mode == SimPar.HIRES)
				{
					modeNUMMRIT_Item.setSelection(true);
					modeDECON_Item.setSelection(false);
				}
				if(sp.calc_mode == SimPar.DECON)
				{
					modeDECON_Item.setSelection(true);
					modeNUMMRIT_Item.setSelection(false);
				}
				
			}				
		});
		
		modeNUMMRIT_Item.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				sp.calc_mode = SimPar.HIRES;
			}
		});
		
		modeDECON_Item.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				sp.calc_mode = SimPar.DECON;
			}
		});
		
		runSimulationItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{				
				int i, j;
				int nvar = 0;
				
				//TODO need to work on external module identification a bit
				
				String module_folder;	// local module folder
				
				if(gp.module_folder.length() < 2) 
					module_folder = SW_main.parentFolder;
				else
					module_folder = gp.module_folder;
				
				if (gp.log_on) Util.log_error("external module folder: " + module_folder);
							
				if(!Files.exists(Paths.get(gp.scratch_folder)))	// scratch folder doesn't exist
				{
					Util.show_and_log("Scratch folder " + gp.scratch_folder + " doesn't exist");
					// TODO Do you want to create it ????
	                return;
				}
								
				if(!Files.isWritable(Paths.get(gp.scratch_folder))) // scratch folder isn't writable
				{
					Util.show_and_log("Scratch folder " + gp.scratch_folder + " is read only");
	                return;
				}
				
				/*boolean switched = false;

				if (sp.calc_mode == sim_par.DNMR)			// since all DNMR par sets are valid for
				{											// NUMMRIT, we can do a temporary to HIRES
					sp.calc_mode = sim_par.HIRES;			// mode while in DNMR mode
					switched = true;
				}*/

				if (sp.n_shifts == 0)
				{
	                Util.show_and_log(
	                    "The number of shifts seems to be 0?  Have you defined a spin system?");
					return;
				}

				for (i = 0; i < sp.n_shifts; i++)
					if (sp.shifts[i].iterate) nvar++;

				for (i = 0; i < sp.n_shifts; i++)
					for (j = 0; j < sp.n_shifts; j++)
						if (sp.j_coupling.iterate[i][j]) nvar++;

				if (nvar == 0 && sp.optmiz == 1)
				{
	                Util.show_and_log("You have asked for an optimization",
	                    " but there are no parameters selected to optimize!");
					return;
				}

//		NOTE: data_folder string already has trailing backslash (or slash for Linux)
				String sim_out, sim_in, trans_out, asn_trans, cmd_line;
				
				if (gp.data_array != null && q.data_folder.length() > 2)
				{
					sim_out = q.data_folder + "sim_out.txt";
					sim_in = q.data_folder + "spin_system";
					trans_out = q.data_folder + "TRANS01";
					asn_trans = q.data_folder + "asn_trans";
				}
				else
				{                
	                sim_out = gp.scratch_folder + "/sim_out.txt";
					sim_in = gp.scratch_folder + "/spin_system";
					trans_out = gp.scratch_folder + "/TRANS01";
					asn_trans = gp.scratch_folder + "/asn_trans";
				}

	            if (gp.log_on) Util.log_error("path to sim file is :" + gp.scratch_folder + "/sim_file");
	            
	            PrintWriter pw;
	    		String fileName = gp.scratch_folder + "/sim_file";

	    		//	open file
	    		try { pw = new PrintWriter(fileName); }
	    		catch (Exception ex)
	    		{
	    			Util.show_and_log("run_simulation: " + fileName, ex.getMessage());
	    			return;
	    		}

	            pw.println(sim_in);
	            pw.println(sim_out);
	            pw.println(trans_out);
	            pw.println(asn_trans);

	            if(gp.log_on) 
	            {
	                Util.log_error("parts of simulation file:");
	                Util.log_error("       input: " + sim_in);
	                Util.log_error("       output: " + sim_out);
	                Util.log_error("       transitions: " + trans_out);
	                Util.log_error("       assigned transitions: " + asn_trans);
	            }
	            pw.close();            
	           
	            if(!Files.exists(Paths.get(module_folder)))	// external module folder doesn't exist
				{
					Util.show_and_log("module folder " + module_folder + " doesn't exist");
	                return;
				}

				if (sp.n_trans > 0) SS_util.putTrans(asn_trans, gp, sp);	// write any assigned transitions
				
				String args;
				if(currentOS == osTypes.WINDOWS)
				{
					cmd_line = String.format("\"%s/sim_main.exe\"", module_folder);
					args = "/c " + cmd_line + " " + gp.scratch_folder + "/sim_file";
				}
				else if(currentOS == osTypes.MACOS)
				{
					cmd_line = String.format("%s/sim_main.exe", module_folder);
					args = cmd_line + " " + gp.scratch_folder + "/sim_file";
				}
				else	// default is UNIX like
				{
					cmd_line = String.format("%s/sim_main.exe", module_folder);
					args = cmd_line + " " + gp.scratch_folder + "/sim_file";
				}
					            
	            if (gp.log_on) Util.log_error("sim command: " + args);    
	            

//		Clear out old simulation  
	            try
	            { 
	            	if(Files.exists(Paths.get(trans_out)))	
	            		Files.delete(Paths.get(trans_out));	
	            }	            	
	            catch(Exception ex)
            	{
            		Util.log_error(ex.getMessage());
            		ex.printStackTrace();
            	}

	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));			            
	            try
	            {
	            	Process proc;		// is this safe?
	            	if(currentOS == osTypes.WINDOWS)
	            		proc = Runtime.getRuntime().exec("cmd.exe " + args);
	            	else if(currentOS == osTypes.MACOS)
	            		proc = Runtime.getRuntime().exec(args);
	            	else	// default is UNIX like
	            		proc = Runtime.getRuntime().exec(args);
	            	
	            	if(!proc.waitFor(sp.time_out, TimeUnit.SECONDS))
	            	{
	            		Util.show_and_log("Simulation has timed-out");
	            		update_spect = true;
	    				canvas.redraw();
	    				statusStripLabel2.setText("Simulation time-out");
	    				statusStripLabel1.setText("Ready");
	            		return;
	            	}
	            }
	            catch(Exception ex)
	            {
	            	ex.printStackTrace();
	            	Util.show_and_log("running cmd.exe " + args, ex.getMessage());
	            }

				if (SS_util.getTrans(trans_out, gp, sp, p, q) > 0)	// get some transitions, if available
				{
					Util.alloc_calc(gp, p, q);
					SS_util.genspec(gp, sp, q);
					gp.calc_max = Math.abs(Util.dscan(gp.calc_array, q.SI, 0, q.SI));
					if (gp.calc_max < 100.0) gp.calc_max = 100.0;	// prevents possible overflow in next step
					gp.calc_scale = (double)gp.dysize * 0.80 / gp.calc_max;
				}

				for (i = 0; i < sp.n_trans; i++) 
				    gp.calc_peaks[i].obs_freq = 0.0;

				SS_util.getAssignedTransitionFile(asn_trans, gp, sp );		

//		update display
				update_spect = true;
				canvas.redraw();
				statusStripLabel2.setText("NUMMRIT Simulation Complete");
	            if (gp.log_on) Util.log_error("NUMMRIT Simulation Complete");
				statusStripLabel1.setText("Ready");
				//if (switched) sp.calc_mode = sim_par.DNMR;
				shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));		
				gp.log_on = false;
			}
		});

//-------------------------------------------------------------------------------------------------
		listSimulationOutputItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				File outputFile = null;
				if (Desktop.isDesktopSupported()) // new way!  portable??
				{
				    try 
				    {
				    	
				    	if (gp.data_array != null && q.data_folder.length() > 2)
				    		outputFile = new File(q.data_folder + "sim_out.txt");
				    	else
				    		outputFile = new File(gp.scratch_folder + "/sim_out.txt");				    						        
				    } 
				    catch (NullPointerException ex) 
				    {
				        Util.show_and_log("NullPointerException: no application for this file type?? " +
				             q.data_folder + "sim_out.txt", ex.getMessage());
				    }
				    
				    try {	Desktop.getDesktop().open(outputFile);} 
				    catch (Exception ex) 
				    {
				    	 Util.show_and_log("no application for this file type?? " +
					             outputFile, ex.getMessage());
					}
				}
				else
					Util.show_and_log("Desktop not supported");
			}
		});

//-------------------------------------------------------------------------------------------------
		editSimulationOptionsItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				// Save old values to check for changes that might require updates
				// to internal parameters or display
				double old_linewidth = sp.linewidth;
				double old_sw = q.SW * q.SFO1;
				double old_offset = q.offset;
				double old_sf = q.SFO1;
				int old_lineshape = sp.lineshape;
				int old_size = q.SI;
				boolean redraw = false;
					
				// run the dialog
				SimOptionsDialog dlg = new SimOptionsDialog(shell, 
  						SWT.APPLICATION_MODAL | SWT.TITLE | SWT.BORDER);
				dlg.open();
				
				if(sp.lineshape != old_lineshape 
					   	   || Math.abs(old_linewidth - sp.linewidth) > 0.000001)
								redraw = true;
				
				if(gp.data_array == null)
				{
					// TODO can this stuff be simplified??
					if (old_size != q.SI)
					{
						p.SI = q.SI;
						q.hzperpt = (q.SW * q.SFO1) / (double)(q.SI);
						Util.alloc_calc(gp, p, q);
						redraw = true;
					}

					if (Math.abs(q.SW * q.SFO1 - old_sw) > 0.00001)
					{
						p.SW = q.SW;
						q.hzperpt = (q.SW * q.SFO1) / (double)(q.SI);
						redraw = true;
					}

					if (Math.abs(q.offset - old_offset) > 0.00001)
					{
						//TODO is q.OFFSET ever used??
						q.OFFSET = (double)(q.offset / q.SFO1); p.OFFSET = q.OFFSET;
						redraw = true;
					}

					if (Math.abs(old_sf - q.SFO1) > 0.000001)
					{
						p.SFO1 = q.SFO1;
						q.SF = q.SFO1 + ((q.SFO1 * q.SW) / 2.0 - q.offset) / 1000000.0;
						p.SF = q.SF;
						q.hzperpt = (q.SW * q.SFO1) / (double)(q.SI);
						redraw = true;
					}
														
					if (redraw)									// parameters changed
					{											// so re-generate spectrum
						if (sp.calc_mode == SimPar.HIRES)
							SS_util.genspec(gp, sp, q);
						gp.freq1 = q.offset;
						gp.freq2 = q.offset - (q.SW * q.SFO1);
						gp.cursor_flag = 0;
						update_spect = true;
						canvas.redraw();
						// and update display
					}
				}
			}
		});
		
		readAssignedTransitionsItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				FileDialog fd = new FileDialog(shell, SWT.OPEN);
	            fd.setText("Read Assigned Transitions from a Named File"); 
	            
	            if (gp.data_array != null)
					fd.setFilterPath(q.data_folder);
	            else
	            	fd.setFilterPath(gp.default_folder);

	            String selected = fd.open();

	            if(selected != null)
	            {
	            	int n = SS_util.getAssignedTransitionFile(selected, gp, sp);
	            	statusStripLabel2.setText(String.format("%d Assigned Transitions Read", n));
	            }
				statusStripLabel1.setText("Ready");
				
				update_spect = true;
				canvas.redraw();
			}
		});
		
		saveAssignedTransitionsItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{								
				FileDialog fd = new FileDialog(shell, SWT.SAVE);
	            fd.setText("Save Assigned Transitions to a Named File"); 
	            
	            if (gp.data_array != null)
					fd.setFilterPath(q.data_folder);
	            else
	            	fd.setFilterPath(gp.default_folder);

	            String selected = fd.open();

	            if(selected != null)
	            {
	            	int n = SS_util.putTrans(selected, gp, sp);
	            	statusStripLabel2.setText(String.format("%d Assigned Transitions Saved", n));
	            }
				statusStripLabel1.setText("Ready");
			}
		});
		
		autoAssignTransitionsItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				int calcx, calc_index, i;

				for (i = 0; i < sp.n_trans; i++)
				{
					calcx = (int)((gp.freq1 - gp.calc_peaks[i].calc_freq) / gp.hzperpix);
					calc_index = (int)((q.offset - gp.calc_peaks[i].calc_freq) / q.hzperpt);

					if (calc_index >= 0 && calc_index < q.SI)
					{
						if (calcx > 0 && calcx < gp.dxsize)
						{
							gp.calc_peaks[i].obs_freq
							= PeakPick.autoPick(gp.calc_peaks[i].calc_freq, gp, q, sp);
						}
					}
				}
	            if (gp.log_on) Util.log_error("region autoassigned");
	            statusStripLabel1.setText("Ready");
	            statusStripLabel2.setText("region auto assigned");
	            update_spect = true;
				canvas.redraw();
			}
		});
		
		deleteAssignmentsItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				int calcx, calc_index, i;

				for (i = 0; i < sp.n_trans; i++)
				{
					calcx = (int)((gp.freq1 - gp.calc_peaks[i].calc_freq) / gp.hzperpix);
					calc_index = (int)((q.offset - gp.calc_peaks[i].calc_freq) / q.hzperpt);

					if (calc_index >= 0 && calc_index < q.SI)
					{
						if (calcx > 0 && calcx < gp.dxsize)
						{
							gp.calc_peaks[i].obs_freq = 0.0;
						}
					}
				}
	            if (gp.log_on) Util.log_error("assigned peaks in displayed region deleted");
	            statusStripLabel1.setText("Ready");
	            statusStripLabel2.setText("assigned peaks in displayed region deleted");
	            update_spect = true;
				canvas.redraw();
			}
		});
		
		loadOptimizedParametersItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				String trans_out;
				if (gp.data_array != null)		
					trans_out = q.data_folder + "TRANS01";				    
				else
					trans_out = gp.scratch_folder + "/TRANS01";

	            if (gp.log_on) Util.log_error("optimized values loaded from: " + trans_out);

				SS_util.getOpt(trans_out, q, sp);

	            // update "current" spin system
	            String target;
	            if (gp.data_array == null || q.data_folder.length() < 2)
	            	target = gp.scratch_folder + "/spin_system";
	            else
	            	target = q.data_folder + "spin_system";
	            
	            SS_util.write_spins(target, gp, sp, q);

	            if (gp.log_on) Util.log_error("updated spin system saved to: " + target);

	            update_spect = true;
	            canvas.redraw();
			}
		});
		
		openSimpsonFIDItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{			
				FileDialog fd = new FileDialog(shell, SWT.OPEN);
	            fd.setText("Open a Simpson FID file");
	             	             
	            
	            fd.setFilterPath(gp.default_folder);
	             
	            String selected = fd.open();
	            if(selected == null)
	            	return;
				
				q.data_path = selected;
				File fi = null;
				try
				{
					fi = new File(q.data_path );
					q.data_folder = fi.getParent() + '/';
				}
				catch (Exception ex)
				{
		            Util.show_and_log("Open Simpson FID(): " + q.data_path , ex.getMessage());
		            ex.printStackTrace();
					return;
				}
	            q.simpson_referenced = false;

	            if(Simpson.readSimpFID(q.data_path, gp, p, q) < 0)
	                return; 

	            gp.freq1 = q.offset;
	            gp.freq2 = q.offset - q.SW * q.SFO1;
	            gp.expt_scale = (double)gp.dysize * 0.80 / gp.data_max;

	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
	            gp.cursor_flag = 0;						      // erase any spurious cursor lines
	            gp.display_mode = SW_main.modes.MODE_FID;
	            update_spect = true;
	            canvas.redraw();		// force screen re-draw
			}
		});
		
		openSimpsonSpectrumItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				FileDialog fd = new FileDialog(shell, SWT.OPEN);
	            fd.setText("Open a Simpson Spectrum file");
	             	             
	            
	            fd.setFilterPath(gp.default_folder);
	             
	            String selected = fd.open();
				
	            if(selected == null)
	            	return;
	            
				q.data_path = selected;
				File fi = null;
				try
				{
					fi = new File(q.data_path );
					q.data_folder = fi.getParent() + '/';
				}
				catch (Exception ex)
				{
		            Util.show_and_log("Open Simpson Spectrum(): " + q.data_path , ex.getMessage());
		            ex.printStackTrace();
					return;
				}
	            q.simpson_referenced = false;

	            if(Simpson.readSimpSpect(q.data_path, gp, p, q) < 0)
	                return; 

	            gp.freq1 = q.offset;
	            gp.freq2 = q.offset - q.SW * q.SFO1;           
	            gp.calc_scale = (double)gp.dysize * 0.80 / (double)gp.calc_max;

	            shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
	            gp.cursor_flag = 0;						      // erase any spurious cursor lines
	            gp.display_mode = SW_main.modes.MODE_1D;
	            update_spect = true;
	            canvas.redraw();		// force screen re-draw
			}
		});
			// external module folder
		
		editDeconParametersItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				DeconPeaksDialog dlg = new DeconPeaksDialog(shell, SWT.NONE | SWT.TITLE | SWT.BORDER);
				dlg.open();
			}
		});
				
		copyDeconPeaksItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				sp.calc_mode = SimPar.DECON;
				
				for (int i = 0; i < gp.n_peaks; i++)
	            {
	                gp.decon_peaks[i] = new PeakInfo(gp.peaks[i].freq, gp.peaks[i].intensity, 
	                    1.0, 0.0, gp.peaks[i].label);
	            }
	            gp.n_decon_peaks = gp.n_peaks;

	            if (gp.log_on)
	                Util.log_error(String.format("peaks copied to decon list: %d", gp.n_decon_peaks));

	            SW_main.statusStripLabel2.setText(
	                String.format("peaks copied to decon list: %d", gp.n_decon_peaks));
	            
	            // also start decon dialog
	            Util.alloc_calc(gp, p, q);
	            DeconUtil.genspec(gp, q);
	            gp.calc_max = Math.abs(Util.dscan(gp.calc_array, q.SI, 0, q.SI));
	            gp.calc_scale = gp.expt_scale;
	            
				DeconDialog dlg = new DeconDialog(shell, SWT.NONE | SWT.TITLE | SWT.BORDER);
				dlg.open();
				
				update_spect = true;
	            canvas.redraw();		// force screen re-draw
			}
		});
		
		interactiveDeconPanelItem.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				if(gp.n_decon_peaks == 0) return;
	            Util.alloc_calc(gp, p, q);
	            DeconUtil.genspec(gp, q);
	            gp.calc_max = Math.abs(Util.dscan(gp.calc_array, q.SI, 0, q.SI));
	            gp.calc_scale = gp.expt_scale;
	            
				DeconDialog dlg = new DeconDialog(shell, SWT.NONE | SWT.TITLE | SWT.BORDER);
				dlg.open();
				
				update_spect = true;
	            canvas.redraw();		// force screen re-draw
			}
		});
		
		/*calculateRMS_Item.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
			}
		});*/
		
	}	// end addSpinSystemMenuListeners()
//-------------------------------------------------------------------------------------------------
}//	end of class
