package spinworks;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;

public class ToolbarListeners extends SW_main
{

	public static void addToolbarListeners()
	{	
		toolbarOpen.addSelectionListener  (new SelectionAdapter() 
		{
			@Override
			public void widgetSelected(SelectionEvent e) 
			{
				FileDialog fd = new FileDialog(shell, SWT.OPEN);
				fd.setText("Open an NMR FID file");


				try	// Just so File Open doesn't fail if something is strange
				{
					if(q.data_type == data_types.BRUKER && q.data_folder != null)
						fd.setFilterPath(q.data_folder);
					else if(gp.default_folder != null)				
						fd.setFilterPath(gp.default_folder);
				}
				catch(Exception ex) { ex.printStackTrace();}

				String selected = fd.open();
				if(selected != null)
				{
					q.data_mode = modes.MODE_1D;
					FileOpen.openSelectedFile(selected);
					update_spect = true;
					canvas.redraw();
				}
			}
		});
		
		 toolbarPrev.addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 int exno = FileUtilBruker.get_ex_no(q.data_folder);
                 if (exno <= 1) return;
                 double lastFreq1 = gp.freq1;
                 double lastFreq2 = gp.freq2;

                 for (int i = exno - 1; i > 0; i--)
                 {
                     if(FileUtilBruker.load_new_exno(i))
                         break;  
                 }
                 // restore current dispay limits, 1D only
                 // nicer for stepping through a series of related spectra
                 if(gp.display_mode == modes.MODE_1D)
                 {
                	 gp.freq1 = lastFreq1;
                	 gp.freq2 = lastFreq2;
                 }
        	 }
         });
		 
		 toolbarNext.addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 int exno = FileUtilBruker.get_ex_no(q.data_folder);
                 if (exno <= 0) return;
                 double lastFreq1 = gp.freq1;
                 double lastFreq2 = gp.freq2;

                 for (int i = exno + 1; i <= 999; i++)
                 {
                     if(FileUtilBruker.load_new_exno(i))
                         break;
                 }   
                 // restore current dispay limits, 1D only
                 // nicer for stepping through a series of related spectra
                 if(gp.display_mode == modes.MODE_1D)
                 {
                	 gp.freq1 = lastFreq1;
                	 gp.freq2 = lastFreq2;
                 }
        	 }
         });
		 
		 toolbarSave.addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));

 	 			if (q.data_mode == modes.MODE_1D)
 	 			{
 	                DX_Util.write_jdx(q.data_folder + "spectrum.dx", gp, q); // JCAMP-DX
 	                FileUtil.saveProc1D(gp, q);	// SpinWorks native
 	 			}
 	 			else if (q.data_mode == modes.MODE_2D)	// no JCAMP-DX for 2D
 	 				FileUtil.saveProc2D(false, gp, q, q_f1);
 
 	 			statusStripLabel1.setText("Ready");
 	 			statusStripLabel2.setText("Processed data saved");
 	 			shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_ARROW));
        	 }
         });
		 
		 toolbarPrint.addSelectionListener  (new SelectionAdapter() 	// print spectrum
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 PrintHandler.doPrint();
        	 }
         });
		 
		 toolbar3DMinus.addSelectionListener  (new SelectionAdapter() 	// display previous 3D plane
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 if (q.data_type == data_types.NMRPIPE)
                 {
                     int slice = NMRpipe.getSliceNumber(q.data_path);

                     if (slice <= 1) return;
                     slice--;

                     NMRpipe.load_new_slice(slice);
                     return;	// don't need to go further
                 }

                 if (gp.current_plane == 0) return;

                 gp.current_plane--;

                 if (gp.dimensions_processed == 2)
                 {
                     if(gp.mode_3D == modes_3D.XY)
                         FileUtil3D.loadPlane_XYt(gp.current_plane, gp, q, q_f2);
                     else if(gp.mode_3D == modes_3D.XZ)
                         FileUtil3D.loadPlane_XZt(gp.current_plane, gp, q, q_f1);

                     statusStripLabel2.setText(String.format("Plane %d", gp.current_plane + 1));
                 }
                 else if (gp.dimensions_processed == 3)
                 {
                     double plane_shift = 0.0;
                     if (gp.mode_3D == modes_3D.XY)
                     {
                         Util_3D.load_XY_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, 
                             q.SI, q_f2.SI);
                         plane_shift = (q_f1.offset - gp.current_plane * q_f1.hzperpt)
                             / q_f1.SF;               
                     }
                     else if (gp.mode_3D == modes_3D.XZ)
                     {
                         Util_3D.load_XZ_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
                         plane_shift = (q_f2.offset - gp.current_plane * q_f2.hzperpt)
                             / q_f2.SF;
                     }

                     statusStripLabel2.setText(String.format("Plane %d: %.2f ppm", gp.current_plane + 1,
                         plane_shift));
                 }

                 gp.new_2D_view = true; 
                 update_spect = true;
                 canvas.redraw();
        	 }
         });
		 
		 toolbar3DPlus.addSelectionListener  (new SelectionAdapter() // display next 3D plane
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 if (q.data_type == data_types.NMRPIPE)
                 {
                     int slice = NMRpipe.getSliceNumber(q.data_path);

                     if (slice <= 0) return;
                     slice++;

                     NMRpipe.load_new_slice(slice);
                     return;
                 }

                 if ((gp.mode_3D == modes_3D.XY && gp.current_plane >= q_f1.SI - 1)
                     || (gp.mode_3D == modes_3D.XZ && gp.current_plane >= q_f2.SI - 1)
                     || (gp.mode_3D == modes_3D.YZ && gp.current_plane >= q.SI - 1)) return;

                 gp.current_plane++;    		 
        		 
        		 if (gp.dimensions_processed == 2)
                 {
                     if(gp.mode_3D == modes_3D.XY)
                         FileUtil3D.loadPlane_XYt(gp.current_plane, gp, q, q_f2);
                     else if(gp.mode_3D == modes_3D.XZ)
                         FileUtil3D.loadPlane_XZt(gp.current_plane, gp, q, q_f1);

                     statusStripLabel2.setText(String.format("Plane %d", gp.current_plane + 1));
                 }
                 else if (gp.dimensions_processed == 3)
                 {
                     double plane_shift = 0.0;
                     if (gp.mode_3D == modes_3D.XY)
                     {
                         Util_3D.load_XY_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, 
                             q.SI, q_f2.SI);
                         plane_shift = (q_f1.offset - gp.current_plane * q_f1.hzperpt)
                             / q_f1.SF;               
                     }
                     else if (gp.mode_3D == modes_3D.XZ)
                     {
                         Util_3D.load_XZ_plane(gp.current_plane, Proc_3D.XYZ, gp.rows_real, q.SI, q_f1.SI);
                         plane_shift = (q_f2.offset - gp.current_plane * q_f2.hzperpt)
                             / q_f2.SF;
                     }

                     statusStripLabel2.setText(String.format("Plane %d: %.2f ppm", gp.current_plane + 1,
                         plane_shift));
                 }
        		 gp.new_2D_view = true; 
        		 update_spect = true;
                 canvas.redraw();
        	 }       	 
         });
		 
		 // Tidy these up
		 toolBarWorkspace[0].addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 for(int i = 0; i < 7; i++)
        			 if(i != 0) toolBarWorkspace[i].setSelection(false);
        		 
        		 Workspace.selectedIndexChanged(0);
        	 }
         });
		 
		 toolBarWorkspace[1].addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 for(int i = 0; i < 7; i++)
        			 if(i != 1) toolBarWorkspace[i].setSelection(false);
        		 Workspace.selectedIndexChanged(1);
        	 }
         });
		 
		 toolBarWorkspace[2].addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 for(int i = 0; i < 7; i++)
        			 if(i != 2) toolBarWorkspace[i].setSelection(false);
        		 
        		 Workspace.selectedIndexChanged(2);
        	 }
         });
		 
		 toolBarWorkspace[3].addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 for(int i = 0; i < 7; i++)
        			 if(i != 3) toolBarWorkspace[i].setSelection(false);
        		 
        		 Workspace.selectedIndexChanged(3);
        	 }
         });
		 
		 toolBarWorkspace[4].addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 for(int i = 0; i < 7; i++)
        			 if(i != 4) toolBarWorkspace[i].setSelection(false);
        		 
        		 Workspace.selectedIndexChanged(4);
        	 }
         });
		 
		 toolBarWorkspace[5].addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 for(int i = 0; i < 7; i++)
        			 if(i != 5) toolBarWorkspace[i].setSelection(false);
        		 
        		 Workspace.selectedIndexChanged(5);
        	 }
         });
		 
		 toolBarWorkspace[6].addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 for(int i = 0; i < 7; i++)
        			 if(i != 6) toolBarWorkspace[i].setSelection(false);
        		 
        		 Workspace.selectedIndexChanged(6);
        	 }
         });
		 
		 toolbarF2scan.addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 if(toolbarF2scan.getSelection())
        		 {
        			 gp.f2_trace_on = true;
        			 gp.f2_from_scan = true;
        			 gp.f2_trace = new double[q.SI];
        			 gp.offset_f2_trace = q.offset;
        			 gp.hzperpt_f2_trace = q.hzperpt;
        			 gp.f2_trace_scale = ((double)gp.xsize_2D * 0.80) / gp.data_max_2D;
        			 gp.size_f2_trace = q.SI;
        		 }
        		 else
        		 {
        			 gp.f2_from_scan = false;
        			 gp.f2_trace_on = false;
        		 }
        		 gp.new_2D_view = true;
        		 update_spect = true;
    			 canvas.redraw();
        	 }
         });
		 
		 toolbarF1scan.addSelectionListener  (new SelectionAdapter() 
         {
        	 @Override
        	 public void widgetSelected(SelectionEvent e) 
        	 {
        		 if(toolbarF1scan.getSelection())
        		 {
        			 gp.f1_trace_on = true;
        			 gp.f1_from_scan = true;
        			 gp.f1_trace = new double[q_f1.SI];
        			 gp.offset_f1_trace = q_f1.offset;
        			 gp.hzperpt_f1_trace = q_f1.hzperpt;
        			 gp.f1_trace_scale = ((double)gp.ysize_2D * 0.80) / gp.data_max_2D;
        			 gp.size_f1_trace = q_f1.SI;
        		 }
        		 else
        		 {
        			 gp.f1_from_scan = false;
        			 gp.f1_trace_on = false;
        		 }
        		 gp.new_2D_view = true;
        		 update_spect = true;
    			 canvas.redraw();
        	 }
         });	
	}	// end addToolbarListeners()
}
