package spinworks;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.widgets.Display;

/**
 * A class describing a trace (used for stacked trace display and plots).
 * @author marat, 
 * University of Manitoba
 */
public class Trace implements Cloneable
{
	/**
	 * constructor function copies values
	 * in current data parameters and allocates
	 * an array for the trace
	 * @param gp a SpinWorks global parameter object
	 * @param q the status acquisition parameters for the current data set
	 */
	public Trace(global_par gp, data_par q)		
	{											
		size = q.SI;
		offset = q.offset;
		hzperpt = q.hzperpt;
		sf = q.SF;
		sfo1 = q.SFO1;
		yscale = gp.expt_scale;
		data = new double[size];
        idata = new double[size];
		folder = q.data_folder;
		title = gp.plot_title;
		Device device = Display.getCurrent();
		colour = new Color(device, 100, 100, 255);
		print_colour = new Color(device, 0, 0, 0);
	}

//-------------------------------------------------------------------------------------------------
	protected Object clone() throws CloneNotSupportedException 
	{
		return super.clone();
	}

//-------------------------------------------------------------------------------------------------
// data members
	public int size;							// number of data points
	public double offset;						// frequency of first point in Hz.
	public double hzperpt;						// point resolution for trace
	public double sf;							// frequency of 0 PPM
	public double sfo1;							// transmitter frequency;
	public double yscale;						// vertical scaling
	public double[] data;					    // actual data array
    public double[] idata;                      // imaginary data
	public String folder;						// original location of the data
	public String title;						// title of original data
	public Color colour;						// colour for display
	public Color print_colour;					// colour for printer
}

