
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.MessageBox;

public class Util extends SW_main
{
/**
 * Show and log an error message
 * @param message the message
 */
	public static void show_and_log(String message)
	{
		addToLog(message);
		System.out.println(message);
		MessageBox dlg = new MessageBox(shell, SWT.ICON_ERROR);
		dlg.setText("SpinWorks J Error");
		dlg.setMessage(message);
		dlg.open();
	}
	
/**
 * Show and log an error message, in two lines (for longer messages)
 * @param message1 First part of the message
 * @param message2 Second part of the message,
 * typically ex.getMessage() from an exception
 */
	public static void show_and_log(String message1, String message2)
	{
		addToLog(message1);
		addToLog("   >>" + message2);
		MessageBox dlg = new MessageBox(shell, SWT.ICON_ERROR);
		dlg.setText("SpinWorks J Error");
		dlg.setMessage(message1 + lineSep + message2);
		dlg.open();
	}
	
/**
 * Log an error condition (no dialog)
 * @param message The message
 */
	public static void log_error(String message)
	{
		addToLog(message);
		System.out.println(message);
	}
	
/**
 * Log an error condition (two parts)
 * @param message1 First part of message
 * @param message2 Second line of message
 */
	public static void log_error(String message1, String message2)
	{
		addToLog(message1);
		addToLog("   >>" + message2);
		System.out.println(message1);
		System.out.println("   >>" + message2);
	}
	
/**
 * Adds a line to the error log
 * @param message the message string
 */
	static void addToLog(String message)
	{	
		FileWriter fw;
		String fileName = SW_main.gp.scratch_folder + "/SWJ_Error.log";
		
		File fi;
		try
		{	fi = new File(gp.scratch_folder);	}
		catch(Exception ex)
		{
			System.out.println(ex.getMessage());
			ex.printStackTrace();
			return;
		}
		
		if(! fi.exists())	// check if scratch folder exists, if not make it
		{
			MessageBox messageBox = new MessageBox(shell, SWT.ICON_QUESTION
					| SWT.YES | SWT.NO);
			messageBox.setMessage("Scratch folder " + gp.scratch_folder + " doesn't exist." 
					+ lineSep 
					+ " Do you want to create it");
			messageBox.setText("SpinWorks scratch folder location");
			int response = messageBox.open();
			if (response == SWT.YES)
			{
				try{	fi.mkdir();	}
				catch(Exception ex)
				{
					System.out.println(ex.getMessage());
					ex.printStackTrace();
				}
			}
		}
		
//		open file
			try { fw = new FileWriter(fileName, true); }
			catch (IOException ex)
			{
				ex.printStackTrace();
				System.out.println("addToLog(): IOException:" + ex.getMessage());
				return;
			}
			
			try{	fw.write(message + lineSep);	}
			catch(IOException ex)
			{
				ex.printStackTrace();
				System.out.println("addToLog(): IOException:" + ex.getMessage());
			}
					
			try{	fw.close();	}
			catch(IOException ex)
			{
				ex.printStackTrace();
				System.out.println("addToLog(): IOException:" + ex.getMessage());
			}
	}
	
//-------------------------------------------------------------------------------------------------	
/**
 * Given a screen location, return the frequency
 * @param x The screen x location in pixels
 * @param gp A SpinWorks global parameter object
 * @return The frequency in Hz.
 */
    public static double pt_to_freq(int x, global_par gp)
    { return gp.freq1 - (double)x * gp.hzperpix; }
 
/**
 * Given a screen location, return the frequency, used for direct and indirect dimensions
 * @param x The screen x (direct) oy y (indirect) location in pixels
 * @param freq1 freq of the left-most point in the display region
 * @param hzperpix Hertx per pixel in the display region
 * @return the frequency of the point in Hz
 */
    public static double pt_to_freq(int x, double freq1, double hzperpix)
    { return freq1 - (double)x * hzperpix; }

//-------------------------------------------------------------------------------------------------
/**
 * converts a frequency in Hz to screen location
 * @param freq
 * @param gp
 * @return
 */
    public static int freq_to_pt(double freq, global_par gp)
    {
        int x;
        try {x = (int) Math.round((gp.freq1 - freq) / gp.hzperpix); }
        catch(Exception ex)
        {
            Util.log_error(ex.getMessage());
            return 0;
        }
        if (x < 0) return 0;
        else if (x > gp.dxsize) return gp.dxsize;
        else return x;
    }
    
/**
 * converts a frequency in Hz to screen location, direct or indirect
 * @param freq the frequency in Hz
 * @param freq1 left most point in the display dimension
 * @param hzperpix Hertz per pixel in the display dimension
 * @param size number of points in the screen dimension
 * @return the screen location
 */
    public static int freq_to_pt(double freq, double freq1, double hzperpix, int size)
    {
        int x;
        try {x = (int) Math.round((freq1 - freq) / hzperpix); }
        catch(Exception ex)
        {
            Util.log_error(ex.getMessage());
            return 0;
        }
        if (x < 0) return 0;
        else if (x > size) return size;
        else return x;
    }

//-------------------------------------------------------------------------------------------------
/**
 * Converts a freq in Hz to an index in the spectrum file
 * @param freq
 * @param q
 * @return
 */
    public static int freq_to_index(double freq, data_par q)
    {
        int x;
        try { x = (int) Math.round((q.offset - freq) / q.hzperpt); }
        catch(Exception ex)
        {
            Util.log_error("util.freq_to_index():  " + ex.getMessage());
            return 0;
        }
        if (x < 0) return 0;
        else if (x >= q.SI) return q.SI - 1;
        else return x;
    }
    
//------------------------------------------------------------------------------------------------
/**
 * Allocate space for processed data
 * @param gp a SpinWorks "global parameter" object
 * @param p "set" data parameters
 * @param q "status" data parameters
 */
    public static void alloc_proc(global_par gp, data_par p, data_par q)
    {
    	try
    	{
    		gp.i_data_array = new double[p.SI];
    		gp.data_array = new double[p.SI];
    	}
    	catch (Exception ex)
    	{
    		Util.show_and_log("alloc_proc(): ", ex.getMessage() );
    		ex.printStackTrace();
    		return;
    	}

    	q.SI = p.SI;					// Since space is allocated, update status parameter

    	zero_array(gp.data_array);
    	zero_array(gp.i_data_array);
    }
   
/**
 * Allocate space for processed data
 * @param gp a SpinWorks "global parameter" object
 * @param size number of data points
 */
    public static void alloc_proc(global_par gp, int size)
    {
        try
        {
            gp.i_data_array = new double[size];
            gp.data_array = new double[size];
        }
        catch (Exception ex)
        {
            Util.show_and_log("alloc_proc(): ", ex.getMessage());
            return;
        }

         zero_array(gp.data_array);
         zero_array(gp.i_data_array);
    }

  //-------------------------------------------------------------------------------------------------
/**
 * Allocate space (real and imaginary)for processed data
 * @param gp a SpinWorks "global parameter" object
 * @param p "set" data parameters
 * @param q "status" data parameters
 */
    public static void alloc_calc(global_par gp, data_par p, data_par q)
    {
    	try
    	{
    		gp.calc_array = new double[p.SI];
    		gp.i_calc_array = new double[p.SI];
    	}
    	catch (Exception ex)
    	{
    		Util.show_and_log("alloc_calc(): ", ex.getMessage() );
    		ex.printStackTrace();
    		return;
    	}

    	q.SI = p.SI;					// Since space is allocated, update status parameter

    	zero_array(gp.calc_array);
    	zero_array(gp.i_calc_array);
    }
   
/**
 * Allocate space (real and imaginary) for processed data
 * @param gp a SpinWorks "global parameter" object
 * @param size number of data points
 */
    public static void alloc_calc(global_par gp, int size)
    {
        try
        {
            gp.calc_array = new double[size];
            gp.i_calc_array = new double[size];
        }
        catch (Exception ex)
        {
            Util.show_and_log("alloc_proc(): ", ex.getMessage());
            return;
        }

         zero_array(gp.calc_array);
         zero_array(gp.i_calc_array);
    }
  //-------------------------------------------------------------------------------------------------	
/**
 * Allocate space for processing buffer and zero it
 * @param size number of points in the processing buffer
 * @param gp a SpinWorks global parameter object
 */
    public static void alloc_proc_buffer(int size, global_par gp)
    {
    	try{gp.proc_buffer = new double[size];}
    	catch (Exception ex)
    	{
    		Util.show_and_log("alloc_proc_buffer(): ", ex.getMessage());
    		gp.proc_buffer = null;
    		ex.printStackTrace();
    		return;
    	}  
    }

 //-------------------------------------------------------------------------------------------------
  /**
   * "free" processing buffer. Memory management is actually now handled by the framework 
   * garbage collection mechanism, so this method simply flags that the arrays are no longer valid
   * @param gp a SpinWorks global parameter object
   */
      public static void free_proc_buffer(global_par gp)  
      {
    	  gp.proc_buffer = null;      
      }
      
 /**
  * "free" processed data arrays. Memory management is actually now handled by the framework 
  * garbage collection mechanism, so this method simply flags that the arrays are no longer valid
  * @param gp a SpinWorks global parameter object
  */
      public static void free_proc(global_par gp)
      {
    	 gp.data_array = null;
    	 gp.i_data_array = null;
      }
      
//-------------------------------------------------------------------------------------------------
 /**
  * zero any array (double)
  * @param array the array to be set to 0.0
  */
      public static void zero_array(double[] array)
      {
    	  if (array == null) return;
 
    		  for(int i = 0; i < array.length; i++)
    			  array[i] = 0.0;
      }

//-------------------------------------------------------------------------------------------------
 /**
  * zero any array (integer)
  * @param array the array to be set to 0
  */
      public static void zero_array(int[] array)
      {
    	  if (array == null) return;

    		  for (int i = 0; i < array.length; i++)
    			  array[i] = 0;
      }

//-------------------------------------------------------------------------------------------------
 /**
  * Check that frequencies are in the correct order (F1 > F2)
  * @param gp A SpinWorks global par object
  */
      public static void check_order(global_par gp)
      {
    	  double temp;

    	  if (gp.freq1 < gp.freq2)				       // reorder direct if necessary
    	  {
    		  temp = gp.freq1;
    		  gp.freq1 = gp.freq2;
    		  gp.freq2 = temp;
    	  }

    	  if (gp.display_mode == SW_main.modes.MODE_2D)	// same for indirect
    	  {
    		  if (gp.freq1_indirect < gp.freq2_indirect)
    		  {
    			  temp = gp.freq1_indirect;
    			  gp.freq1_indirect = gp.freq2_indirect;
    			  gp.freq2_indirect = temp;
    		  }
    	  }
      }

//-------------------------------------------------------------------------------------------------
/**
 * Calculate horizontal scaling factors
 * @param gp A SpinWorks global parameter object
 */
    public static void hscale(global_par gp)
    {
    	if (gp.display_mode == SW_main.modes.MODE_2D)
    	{
    		gp.hzperpix = (gp.freq1 - gp.freq2) / (double)(gp.xsize_2D - 1);
    		gp.hzperpix_f1 = (gp.freq1_indirect - gp.freq2_indirect) / (double)(gp.ysize_2D - 1);
    	}
    	else
    		gp.hzperpix = (gp.freq1 - gp.freq2) / (double)(gp.dxsize - 1);
    }

//-------------------------------------------------------------------------------------------------
/**
 * Calculate display scaling factors, etc.
 * @param gp A SpinWorks global parameter object
 * @param q The status data parameters
 */
    public static void rescale(global_par gp, data_par q)
    {
    	gp.data_max = Math.abs((double)dscan(gp.data_array, q.SI, 0, q.SI));
    	gp.data_noise = noise(gp.data_array, q.SI, q.aq_mode);
    	if (gp.data_max < 1.0) gp.data_max = 1000000.0;
    }
    
//-------------------------------------------------------------------------------------------------
/**
 * This method calculates the rms noise in the array "array" from two
 * 40 point blocks at either end of the spectrum.
 * @param array The data array
 * @param n The size of the array
 * @param mode Detection mode. If DSP, avoids "smilies" at ends of spectrum
 * @return The RMS noise in the spectrum
 */
    public static double noise(double[] array, int n, SW_main.det_modes mode)
    {
    	if (array == null)
    		return -1;

    	int start = 0, end = n - 1;

    	if (mode == SW_main.det_modes.DSP)	// avoid "smilies" at ends of
    	{									// DSP processed data
    		start = n / 32;
    		end = n - (n / 32);
    	}

    	if (end <= start) return -2;

    	double sum, avgl, avgr, rmsl, rmsr;
    	int i;

    	sum = 0.0;
    	for (i = start; i < start + 32; i++)
    		sum = sum + array[i];
    	avgr = sum / 32.0;

    	sum = 0.0;
    	for (i = end - 32; i < end; i++)
    		sum = sum + array[i];
    	avgl = sum / 32.0;

    	sum = 0.0;
    	for (i = start; i < start + 32; i++)
    		sum = sum + (avgr - array[i]) * (avgr - array[i]);
    	rmsr = Math.sqrt(sum) / 32.0;

    	sum = 0.0;
    	for (i = end - 32; i < end; i++)
    		sum = sum + (avgl - array[i]) * (avgl - array[i]);
    	rmsl = Math.sqrt(sum) / 32.0;

    	return ((rmsl + rmsr) / 2.0);
    }

//-------------------------------------------------------------------------------------------------
/**
 * This method calculates the rms noise in the array "array" from two
 * 40 point blocks at either end of the spectrum.
 * @param array The data array
 * @param n The size of the array
 * @param mode Detection mode. If DSP, avoids "smilies" at ends of spectrum
 * @return The absolute maximum noise excursion
 */
    public static double peakNoise(double[] array, int n, SW_main.det_modes mode)
    {
    	if (array == null)
    		return -1;

    	int start = 0, end = n - 1;

    	if (mode == SW_main.det_modes.DSP)	// avoid "smilies" at ends of
    	{									// DSP processed data
    		start = n / 32;
    		end = n - (n / 32);
    	}

    	if (end <= start) return -2;

    	double max = 0.0;
    	int i;

    	for (i = start; i < start + 32; i++)
    		if(Math.abs(array[i]) > max) max = Math.abs(array[i]);

    	for (i = end - 32; i < end; i++)
    		if(Math.abs(array[i]) > max) max = Math.abs(array[i]);

    	return max;
    }
//-------------------------------------------------------------------------------------------------
/** This function scans the calculated or experimental spectra 
* and returns the absolute maximum value between point1 and point2. 
* The name is a hold-over from the earliest days of Xsim, and should be changed
* @param array the data array to scan
* @param n the number of points in the array
* @param point1 the starting index
* @param point2 the ending index
* @return the maximum between point1 and point2
*/
      public static double dscan(double[] array, int n, int point1, int point2)
      {
          int i = 0;

          if (array == null || n == 0) return 1000000.0;

          if (point1 < 0) point1 = 0;
          if (point2 >= n) point2 = n - 1;

          double max = 0.0;

          try
          {
              for (i = point1; i < point2; i++)
              {
            	  if (Math.abs(array[i]) > max)
                      max = Math.abs(array[i]);
              }
          }
          catch (Exception ex)
          {           
              Util.show_and_log(String.format("index error in dscan(): i = %d ", i), ex.getMessage());
              ex.printStackTrace();
          }

          return max;
      }
    
//-------------------------------------------------------------------------------------------------
/**
 * Calculate the chi squared deviation between the calculated and experimental
 * spectra in the region between f1 and f2.  Normalizes spectra to same intensity.
 * @param f1 The starting frequency (Hz.)
 * @param f2 The ending frequency (Hz.)
 * @param gp A SpinWorks global parameter object
 * @param q The status data parameters
 * @return The chi squared deviation between the experimental and calculated sspectra
 */
      public static double chi_squared(double f1, double f2, global_par gp, data_par q)
      {
    	  if (gp.calc_array == null || gp.data_array == null)
    	  {
    		  Util.show_and_log(
    				  "You must have both experimental and simulated data\n available for this function");
    		  return 0.0;
    	  }

    	  double y_ex, y_sim;						// normalized intensities

    	  int point1 = (int)((q.offset - f1) / q.hzperpt);
    	  int point2 = (int)((q.offset - f2) / q.hzperpt);

    	  if ((point1 < 0) || (point1 >= q.SI))
    		  point1 = 0;

    	  if ((point2 < 0) || (point2 >= q.SI))
    		  point2 = q.SI - 1;

    	  double max_ex = dscan(gp.data_array, q.SI, point1, point2);
    	  double max_sim = dscan(gp.calc_array, q.SI, point1, point2);

    	  double chi_sq = 0.0;

    	  for (int i = point1; i <= point2; i++)
    	  {
    		  if (gp.calc_array[i] != 0)
    		  {
    			  y_ex = gp.data_array[i] / max_ex;
    			  y_sim = gp.calc_array[i] / max_sim;
    			  chi_sq = chi_sq + ((y_ex - y_sim) * (y_ex - y_sim) / y_sim);
    		  }
    	  }
    	  return chi_sq;
      }

//-------------------------------------------------------------------------------------------------
/**
 * Calculate rms deviation between expt and calculated, normalized version
 * @param f1 Left most frequency in Hz.
 * @param f2 Right most frequency in Hz.
 * @param gp A SpinWorks global par object
 * @param q A SpinWorks data par object
 * @param normalize If true, each spectrum will be scaled individually (e.g. for DNMR)
 * @return RMS deviation
 */
	public static double rms_dev(double f1, double f2, global_par gp, data_par q,
			boolean normalize)
	{
		if (gp.calc_array == null || gp.data_array == null)
		{
			Util.show_and_log(
					"You must have both experimental and simulated data\n available for this function");
			return 0.0;
		}

		double y_ex, y_sim;						// normalized intensities

		int point1 = (int)((q.offset - f1) / q.hzperpt);
		int point2 = (int)((q.offset - f2) / q.hzperpt);

		if ((point1 < 0) || (point1 >= q.SI))
			point1 = 0;

		if ((point2 < 0) || (point2 >= q.SI))
			point2 = q.SI - 1;

		double max_ex = dscan(gp.data_array, q.SI, point1, point2);
		double max_sim = dscan(gp.calc_array, q.SI, point1, point2);

		double sum_sq = 0.0;

		for (int i = point1; i <= point2; i++)
		{

			if (normalize)  // scale each spectrum to its max (for DNMR)
			{
				y_ex = gp.data_array[i] / max_ex;
				y_sim = gp.calc_array[i] / max_sim;
			}
			else            // scale both spectra to max in experimental (for deconvolution)
			{
				y_ex = gp.data_array[i] / max_ex;
				y_sim = gp.calc_array[i] / max_ex;
			}

			sum_sq = sum_sq + (y_ex - y_sim) * (y_ex - y_sim);
		}
		return Math.sqrt(sum_sq / (double)(point2 - point1));
	}
	
	public static void waitCursor()
	{	shell.setCursor(new Cursor(Display.getDefault(), SWT.CURSOR_WAIT));	}

//-------------------------------------------------------------------------------------------------
/**
 * return the assigned 2D peak index closest to the given frequency (in Hz)
 * @param gp a SpinWorks workspace parameter object
 * @param f2 the F2 (acquisition dimension) frequency in Hz
 * @param f1 the F1 (indirect dimension) frequency in Hz
 * @param tol the search tolerance in pixels
 * @return the index of the assigned peak, -1 if not found.
 */
	public static int closestPeak(global_par gp, double freq, int tol)
	{
		double tolFreq = gp.hzperpix * (double)tol;

		for (int i = 0; i < gp.n_peaks; i++)
		{
			if (Math.abs(gp.peaks[i].freq - freq) < tolFreq)
				return i;
		}
		return -1;
	}
	
//-------------------------------------------------------------------------------------------------
	public static int selectTrace(int x, int y, int tol)
	{
		if(gp.n_traces == 0) return 0;		
		
		int dxsize = gp.stack_x_size;

    	if (gp.stack_x_size_auto)
    		dxsize = gp.dxsize - gp.stack_x_offset * gp.n_traces;
    	else
    		dxsize = gp.stack_x_size;
    	
    	double hzperpix = (gp.freq1 - gp.freq2)/(double)(dxsize-1);
    	  	   					
		for(int i = 0; i < gp.n_traces; i++)
		{
			// get the index into the spectrum from the cursor x value
	    	int index = (int) Math.round((q.offset
	    		- (gp.freq1 - (double)x * hzperpix)) / gp.stack_trace[i].hzperpt);
	    	
	    	if(index < 0) index = 0;
			else if(index >= q.SI) index = q.SI -1;
	    	
			int j;
			
			if (gp.show_spectrum == false || gp.stack_y_offset == 0)
    			j = i;
    		else
    			j = i + 1;

    		int off;
    		if (gp.stack_y_offset == 0)
    			off = 50;
    		else
    			off = gp.stack_y_offset * (j);
    		
			int datay = gp.dysize
					- (int)(gp.stack_trace[i].data[index] * gp.expt_scale)
					- gp.expt_y_offset
					- off;
			
			System.out.println(String.format("index = %d  y = %d datay = %d", index, y, datay));
		}
		
		return -1;
	}
		
//-------------------------------------------------------------------------------------------------
}	// end class

