
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.widgets.Display;

import simulation.SimPar;

/**
 * Workspace, holds copies of global and data parameters
 * @author marat, 
 * University of Manitoba
 */
public class Workspace
{
	/**
	 * Workspace constructor copies the current global, data and simulation
	 * parameters into a Workspace Object
	 * @param _gp
	 * @param _p
	 * @param _q
	 * @param _p_f1
	 * @param _q_f1
	 */
	public  Workspace(global_par _gp, 
			data_par _p, data_par _q, data_par _p_f1, data_par _q_f1, SimPar _sp)
	{
		try
		{
			gp = (global_par) _gp.clone();
			p = (data_par) _p.clone();
			q = (data_par) _q.clone();
			p_f1 = (data_par) _p_f1.clone();
			q_f1 = (data_par) _q_f1.clone();
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			return;
		}
		
		try{	sp = (SimPar) _sp.clone();	}
		catch(Exception ex){	ex.printStackTrace();	}

		pos_level = new Level[16];
		neg_level = new Level[16];

		peaks = new PeakInfo[PeakPick.MAX_PEAKS];
		integrals = new IntegInfo[SW_main.MAX_INTEGRALS];
		stack_trace = new Trace[SW_main.MAX_TRACES];
		printInsetBox = new PrintInsetBox[SW_main.MAX_BOXES];

		for (int i = 0; i < SW_main.MAX_LEVELS; i++)			// create contouring levels
		{
			try
			{
				pos_level[i] = (Level) _gp.pos_level[i].clone();
				neg_level[i] = (Level) _gp.neg_level[i].clone();
			}
			catch(Exception ex)
			{
				ex.printStackTrace();
				return;
			}
		}

		try
		{
			for (int i = 0; i < gp.n_peaks; i++)
				peaks[i] = (PeakInfo) _gp.peaks[i].clone();
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			return;
		}

		try
		{
			for (int i = 0; i < gp.n_integrals; i++)
				integrals[i] = (IntegInfo)_gp.integrals[i].clone();
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
			return;
		}

		for (int i = 0; i < gp.n_traces; i++)
		{
			try {	stack_trace[i] = (Trace)_gp.stack_trace[i].clone(); }
			catch(Exception ex){	ex.printStackTrace();	}
		}

		for (int i = 0; i < gp.n_boxes; i++)
		{
			try{	printInsetBox[i] = (PrintInsetBox)_gp.printInsetBox[i].clone();	}
			catch(Exception ex) {	ex.printStackTrace(); }
		}
	}

//-------------------------------------------------------------------------------------------------
	/**
	 * Set the selected workspace
	 * @param index of the workspace (0 to MAX_WS)
	 */
	public static void selectedIndexChanged(int index)
	{

		if (index == SW_main.aws) return;    // didn't change WS, go
                                                   // no further
        if (SW_main.gp.phase_mode)  // can't change WS in integration or phasing modes
        {
            Util.show_and_log("Exit phase mode before changing workspace");
            return;
        }

        if (SW_main.gp.integ_mode)
        {
        	Util.show_and_log("Exit phase mode before changing workspace");
            return;
        }
        
        SW_main.ws[SW_main.aws] = new Workspace(
        		SW_main.gp, SW_main.p, SW_main.q, 
        		SW_main.p_f1, SW_main.q_f1, sp);	// load current data into WS
        
        SW_main.ws[SW_main.aws].has_data = true;	// flag has data

		SW_main.aws = index;  

		if (SW_main.ws[SW_main.aws].has_data)			// if the selected WS has data
		{												    // then load it
			try
			{
				SW_main.gp = (global_par)SW_main.ws[SW_main.aws].gp.clone();
				SW_main.p = (data_par)SW_main.ws[SW_main.aws].p.clone();
				SW_main.q = (data_par)SW_main.ws[SW_main.aws].q.clone();
				SW_main.p_f1 = (data_par)SW_main.ws[SW_main.aws].p_f1.clone();
				SW_main.q_f1 = (data_par)SW_main.ws[SW_main.aws].q_f1.clone();
				//SW_main.sp = (sim_par)ws[active_ws].sp.Clone();

				for (int i = 0; i < SW_main.MAX_LEVELS; i++)	// create contouring levels
				{
					SW_main.gp.pos_level[i] = 
							(Level)SW_main.ws[SW_main.aws].pos_level[i].clone(); // seems to work
					SW_main.gp.neg_level[i] = 
							(Level)SW_main.ws[SW_main.aws].neg_level[i].clone();
				}
				for (int i = 0; i < SW_main.gp.n_peaks; i++)
					SW_main.gp.peaks[i] = (PeakInfo)SW_main.ws[SW_main.aws].peaks[i].clone();

				for (int i = 0; i < SW_main.gp.n_integrals; i++)
					SW_main.gp.integrals[i] = 
					(IntegInfo)SW_main.ws[SW_main.aws].integrals[i].clone();

				for (int i = 0; i < SW_main.gp.n_traces; i++)
					SW_main.gp.stack_trace[i] 
							= (Trace)SW_main.ws[SW_main.aws].stack_trace[i].clone();

				for (int i = 0; i < SW_main.gp.n_boxes; i++)
					SW_main.gp.printInsetBox[i] 
							= (PrintInsetBox)SW_main.ws[SW_main.aws].printInsetBox[i].clone();			
			}
			catch(Exception ex)
			{
				ex.printStackTrace();	// TODO better exception handling
			}
		}

		else											    // no data in WS, create new WS
		{               
			SW_main.gp.new_ws();
            if (SW_main.gp.a4_paper)                        // override default paper size
            {                                               // if A4 paper option  (Is there a
            	SW_main.gp.plot_xsize = 280.0;              // neater way to do this??)
            	SW_main.gp.plot_ysize = 150.0;
            }
            //registry_util.get_colours(gp);                // TODO set the stored colours

            SW_main.q.new_ws();
            SW_main.p.new_ws();
		}

//Update application title bar
		SW_main.shell.setText(SW_main.version + SW_main.q.data_folder);

		SW_main.shell.setCursor(new Cursor(Display.getDefault(), 
				SWT.CURSOR_ARROW)); 								// re-set cursor
		SW_main.statusStripLabel1.setText("Ready");					// and update embelishments
		SW_main.statusStripLabel2.setText( 
            String.format("%s  %d scans",  SW_main.q.pulse_sequence, SW_main.q.NS));

        if (SW_main.gp.log_on) Util.log_error(String.format(
            "workspace changed to : %d", SW_main.aws + 1));

//Set button panel and toolbar to current 1D/2D status
		if(SW_main.q.data_mode == SW_main.modes.MODE_1D)
			InterfaceUtil.set1D();
		else if(SW_main.q.data_mode == SW_main.modes.MODE_2D)
		{
			InterfaceUtil.set2D();
			SW_main.gp.new_2D_view = true;	// must re-contour

			if (SW_main.gp.pos_only)								
				Util_2D.calc_pos_only_colors(SW_main.gp);
			else		
				Util_2D.calc_default_colours(SW_main.gp);
		}
      
		SW_main.shell.redraw();		//SW_main_Resize();	
		InterfaceUtil.updateToolbar();
				
		SW_main.update_spect = true;
		SW_main.canvas.redraw();

        if (SW_main.gp.log_on)
            Util.log_error(String.format("Workspace changed to %d", SW_main.aws));      
	}

//-------------------------------------------------------------------------------------------------
// data members
	public boolean has_data;
	public global_par gp;
	public data_par p;
	public data_par q;
	public data_par p_f1;
	public data_par q_f1;
	public static SimPar sp;

	//  It should be possible to actually use the objects embedded in the ws.gp object TO DO !
	//  rather than duplicate them here
	public Level[] pos_level;						// positive contour levels
	public Level[] neg_level;						// negative contour levels
	public PeakInfo[] peaks;
	public IntegInfo[] integrals;
	public Trace[] stack_trace;
	public PrintInsetBox[] printInsetBox;
}
