
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

import spinworks.SW_main.FnMODEs;
import spinworks.SW_main.LP_modes;
import spinworks.SW_main.det_modes_IND;
import spinworks.SW_main.det_modes;
import spinworks.SW_main.phasing;
import spinworks.SW_main.windows;

/**
 * An object containing all of the parameters (acquisition and processing)for a data set.
 * Separate objects are used for the acquisition dimension and each indirect dimension.
 * @author marat,
 * University of Manitoba
 */
public class data_par implements Cloneable
{
	/**
	 *  constructor, set default parameters
	 */
	public data_par() 
    {
        SF = 499.9975;	
        SFO1 = 500.0;
        SI = 16384;
        TD = 1024;
        td_eff = 1024;
        aq_mode = det_modes.SIM;
        SW = 10.0;
        SW_Hz = 5000.0;
        dwell = 1.0 / (SFO1 * SW);
        PH_mod = SW_main.phasing.NONE;
        WDW = SW_main.windows.EM;
        mq_spin = 1.5;
        mq_quantum = 3;
        mq_shear_angle = 0.0;
        hzperpt = 5000.0 / (double)SI;
        offset = 5000.0;
        data_type = SW_main.data_types.BRUKER;
        data_mode = SW_main.modes.MODE_1D;
        domain_type = 0;        // add enumeration
        varian_array = 0;
        nblocks = 0;
        fcor = 0.5;
        fid_bias = false;
        lp = SW_main.LP_modes.LP_OFF;
        lp_coef = 8;
        lp_cutoff = 0.01;
        lp_pred = 0; 
        lp_input = 64;
        lp_polish = false;
        left_shift = 0;
        cls = 0;
        filter_type = SW_main.filters.FILT_NONE;
        filter_points = 19;
        baseline_filter = 2;
        baseline_thresh = 1.5;
        baseline_lambda = 10000000.0;
        DSPFVS = 0;
        GRPDLY = 0.0;
        DECIM = 0;
        MC2 = SW_main.det_modes_IND.QF;
        FnMODE = SW_main.FnMODEs.UNDEFINED;
        AQORDER = 0;
        DTYPA = 0;
        REVERSE = false;
        field_converted = false;
        simpson_referenced = false;
        data_path = "";
        data_folder = null;
        par_path = "";
        pulse_sequence = "undefined";
        NUC1 = "undefined";
        INSTRUM = "undefined";
        jcamp_dx_comp = DX_Util.comp_modes.FIX;
    }

//-------------------------------------------------------------------------------------------------
	/**
	 * re-sets some data parameter object values to defaults for a new workspace
	 */
	 public void new_ws() 
     {
         SF = 499.9975;
         SFO1 = 500.0;
         SI = 16384;
         SW = 10.0;
         dwell = 1.0 / (SFO1 * SW);

         hzperpt = 5000.0 / (double)SI;
         offset = 5000.0;
         
         data_path = "";
         data_folder = null;

         simpson_referenced = false;

         pulse_sequence = "undefined";
         NUC1 = "undefined";
         INSTRUM = "undefined";
     }
	 
//-------------------------------------------------------------------------------------------------
	 @Override
	 public Object clone() throws CloneNotSupportedException 
	 {
		 return (data_par)super.clone();
	 }
		
//-------------------------------------------------------------------------------------------------
//  Since the ancestors of SpinWorks (Assign and Xsim) only handled Bruker data, many parameter 
//  names are identical to those used by Bruker Software.  These are in upper case.  Recently added 
//  parameters (to handle other data formats, etc.  Are in lower case.

    public int AQ_mod;				// acquisition mode (sequential, simultaneous, etc.)
    public int AQORDER;             // acquisition order for 3D (0 = 321, 1 = 312)
    public SW_main.byte_orders BYTORDA;	            // acquisition byte order (BIG = Sun, etc.  LITTLE = INTEL, NSC)
    public SW_main.byte_orders BYTORDP;	            // processing byte order (BIG = Sun, etc.  LITTLE = INTEL, NSC)
    public int DTYPA;               // fixed or floating point FID
    public double DECIM;            // decimation factor (Bruker Avance only)
    public int DIGMOD;              // Digital acquisition mode
    public int DSPFIRM;             // DSP firmware version (Bruker Avance only)
    public int DSPFVS;              // DSP version
    public double GRPDLY;           // DSP group delay
    public double GB;               // Gaussian factor (Bruker style)
    public double LB;               // exponential line broadening
    public det_modes_IND MC2;       // F1 detection mode (TPPI, States, etc.)
    public FnMODEs FnMODE;          // F1 detection mode (TPPI, States, etc.  NEW!!)
    public int NS;                  // number of scans (transients)
    public double OFFSET;           // shift (ppm) of first data point
    public double PHC0;             // zero order phase correction
    public double PHC1;             // first order phase correction
    public phasing PH_mod;          // phasing mode
    public boolean REVERSE;         // whether this dimension should be reversed
    public int SI;                  // size of data in complex points
    public double SF;               // absolute frequency in MHz of 0 ppm
    public double SFO1;             // transmitter frequency
    public double SFO2;             // decoupler frequency
    public double SFO3;             // second decoupler frequency
    public int SSB;                 // sine bell shift (Bruker style)
    public double SW;               // spectral width (ppm)  PRIMARY!
    public double SW_Hz;            // spectral width in Hz.  (used for some routines, not primary)
    public int TD;                  // size of FID in points (TD/2 complex pairs)
    public int td_eff;              // effective TD (after LP or whatever)
    public windows WDW;             // window type (EM, GM, TRAF, etc.

    public double mq_spin;          // spin value for NQ shearing transform
    public int mq_quantum;          // quantum selection for mq experiment
    public double mq_shear_angle;   // shearing phase angle for mqmas shearing transform

    public SW_main.data_types data_type;           // type of data (BRUKER, VNMR, etc)
    public SW_main.modes data_mode; // 1D or 2D
    public int domain_type;         // 0 = time, 1 = freq
    public SW_main.det_modes aq_mode;	// acquisition mode (defined differently than Bruker!)
    public int varian_array;        // varian "array" parameter, for 3D file ordering
    public int nblocks;             // number of blocks in an arrayed Varian experiment

    public double offset;           // frequency in Hz of the first data point (1D and 2D)
    public double hzperpt;          // digital resolution in both dimensions
    public double dwell;            // dwell time in sec per complex pair.

    public double ssb;              // sine bell shift in degrees;
    public double fcor;             // first point correction
    public boolean fid_bias;        // whether FID bias correction should be applied;
    public int left_shift;          // left shift in points
    public int cls;                 // circular left shift value in points

    public LP_modes lp;             // LP mode
    public int lp_coef;             // LP coefficients
    public int lp_pred;             // number of predicted points
    public int lp_input;            // number on input points
    public double lp_cutoff;        // LP cutoff value
    public boolean lp_polish;       // whether LP coefficients should be "polished"

    public SW_main.filters filter_type;         // solvent filter type
    public int filter_points;       // solvent filter width, time domain points

    public int baseline_filter;     // baseline filter width
    public double baseline_thresh;  // baseline threshold
    public double baseline_lambda;  // Whittaker lambda value for baseline

    public int ex_no;               // experiment number (Bruker)
    public int pro_no;              // processed data number (Bruker)

    public boolean field_converted; // whether this data has been field converted for 2D ref.
    public boolean simpson_referenced; // true if a simpson data set has been referenced
                                    // prevents default (SF = SFO1) referencing on re-processing

    public String data_path;        // full path to the data
    public String par_path;         // full path to parameters
    public String data_folder;      // folder that the data 
    public String titlepath;        // full path to the title of text file
    public String pulse_sequence;   // name of pulse sequence
    public String NUC1;             // name of observed nucleus
    public String INSTRUM;          // name (or type) of instrument
    public DX_Util.comp_modes jcamp_dx_comp;       // JCAMP DX compression type.
}

