/// <copyright>
///	Copyright (c) 2006-2017

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;

import spinworks.SW_main.byte_orders;

public class fileUtilMagritek
{
	/**
	 * 
	 * @param fileName
	 * @param gp
	 * @param p
	 * @param q
	 * @param p_f1
	 * @param q_f1
	 * @return
	 */
	public static boolean get_par(String fileName,  global_par gp, data_par p, data_par q,
			data_par p_f1, data_par q_f1)
	{
		String instring;
		String delim = "[ =,\t]+";
		String[] token;
		double dwell = 0.0;         // dwell time in usec.
		double swidth = 2000.0;     // spectral width in direct dimension
		double swidth_f1 = 2000.0;  // spectral width in indirect dimension
		double low_freq = 0.0;      // right most frequency
		double low_freq_f1 = 0.0;   // right most frequency
		int nrSteps = 0;            // number of Steps for 2D
		q_f1.SFO1 = 0.0;

		if (gp.log_on)
			Util.log_error("Opening parameter file:  " + fileName);

		// try to open the parameter file 
        BufferedReader br;
        try { br = new BufferedReader(new FileReader(fileName)); }
        catch (Exception ex)
        {
            Util.show_and_log("get_pars(), cannot open: " + fileName, ex.getMessage());
            return false;
        }

        for(;;)
        {
        	try {instring = br.readLine();} 
        	catch (IOException ex) 
        	{
        		ex.printStackTrace();
        		Util.show_and_log("get_par), read error: " + fileName, ex.getMessage());
        		try { br.close(); } 
        		catch (IOException e) { e.printStackTrace(); }
        		return false;
        	}
        	if(instring == null) break;

        	if (instring.length() > 3)
        	{
        		token = instring.split(delim);

        		try
        		{
        			if (token[0].equalsIgnoreCase("nrPnts"))             // acquired points (r or i)
        			{
        				if (gp.log_on)
        					Util.log_error("get_par()  " + "nrPnts = " + token[1]);
        				q.TD = Integer.parseInt(token[1]) * 2;
        			}
        		}
        		catch (Exception ex)
        		{ Util.show_and_log("get_par(): " + instring, ex.getMessage()); }

        		try
        		{
        			if (token[0].equalsIgnoreCase("nrSteps"))             // acquired points (r or i)
        			{
        				if (gp.log_on)
        					Util.log_error("get_par()  " + "nrSteps = " + token[1]);
        				nrSteps = Integer.parseInt(token[1]);
        			}
        		}
        		catch (Exception ex)
        		{ Util.show_and_log("get_par(): " + instring, ex.getMessage()); }

        		try
        		{
        			if (token[0].equalsIgnoreCase("b1Freq"))             // transmitter frequency
        			{
        				if (gp.log_on)
        					Util.log_error("getpar()  " + "b1Freq  MHz = " + token[1]);
        				q.SFO1 = Double.parseDouble(token[1]);
        			}
        		}
        		catch (Exception ex)
        		{ Util.show_and_log("get_par(): " + instring, ex.getMessage()); }

        		try
        		{
        			if (token[0].equalsIgnoreCase("b1Freq2"))            // transmitter frequency F1
        			{
        				if (gp.log_on)
        					Util.log_error("getpar()  " + "b1Freq MHz = ", token[1]);
        				q_f1.SFO1 = Double.parseDouble(token[1]);
        			}
        		}
        		catch (Exception ex)
        		{ Util.show_and_log("get_par(): " + instring, ex.getMessage()); }

        		try
        		{
        			if (token[0].equalsIgnoreCase("dwellTime"))           // dwell time)
        			{
        				if (gp.log_on)
        					Util.log_error("get_par()  " + "DwellTime  usec = " + token[1]);
        				dwell = Double.parseDouble(token[1]) * 1.0e-6;
        			}
        		}
        		catch (Exception ex)
        		{ Util.show_and_log("get_par(): " + instring, ex.getMessage()); }

        		try
        		{
        			if (token[0].equalsIgnoreCase("bandwidth"))           // dwell time)
        			{
        				if (gp.log_on)
        					Util.log_error("get_par()  " + "bandwidth kHz = " + token[1]);
        				swidth = Double.parseDouble(token[1]) * 1000.0;
        			}
        		}
        		catch (Exception ex)
        		{ Util.show_and_log("get_par(): " + instring, ex.getMessage()); }

        		try
        		{
        			if (token[0].equalsIgnoreCase("bandwidth2"))           // dwell time)
        			{
        				if (gp.log_on)
        					Util.log_error("get_par()  " + "bandwidth2 kHz = " + token[1]);
        				swidth_f1 = Double.parseDouble(token[1]) * 1000.0;
        			}
        		}                 
        		catch (Exception ex)
        		{ Util.show_and_log("get_par(): " + instring, ex.getMessage()); }                  

        		try
        		{
        			if (token[0].equalsIgnoreCase("lowestFrequency"))      // right-most freq.
        			{
        				if (gp.log_on)
        					Util.log_error("getpar()  " + "lowestFrequency Hz = " + token[1]);
        				low_freq = Double.parseDouble(token[1]);
        			}
        		}
        		catch (Exception ex)
        		{ Util.show_and_log("get_par(): " + instring, ex.getMessage()); }

        		try
        		{
        			if (token[0].equalsIgnoreCase("lowestFrequency2"))      // right-most freq.
        			{
        				if (gp.log_on)
        					Util.log_error("getpar()  " + "lowestFrequency2 Hz = ", token[1]);
        				low_freq_f1 = Double.parseDouble(token[1]);
        			}
        		}
        		catch (Exception ex)
        		{ Util.show_and_log("get_par(): " + instring, ex.getMessage()); }

        		try
        		{
        			if (token[0].equalsIgnoreCase("nrScans"))             // nunber of scans
        			{
        				if (gp.log_on)
        					Util.log_error("get_par()  " + "nrScans = " + token[1]);
        				q.NS = Integer.parseInt(token[1]);
        			}
        		}
        		catch (Exception ex)
        		{ Util.show_and_log("get_par(): " + instring, ex.getMessage()); }

        		try
        		{
        			if (token[0].equalsIgnoreCase("graphTitle"))          // title
        			{
        				if (gp.log_on)
        					Util.log_error("get_par()  " + "graphTitle = " + token[1]);
        				gp.plot_title = token[1];

        				// Strip leading and trailing " characters
        				gp.plot_title = gp.plot_title.replace('\"', ' ');
        				gp.plot_title = gp.plot_title.trim();
        			}
        		}
        		catch (Exception ex)
        		{ Util.show_and_log("get_par(): " + instring, ex.getMessage()); }

        		try
        		{
        			if (token[0].equalsIgnoreCase("experiment"))          // title
        			{
        				if (gp.log_on)
        					Util.log_error("get_par()  " + "experiment = " + token[1]);
        				q.pulse_sequence = token[1];

        				// Strip leading and trailing " characters
        				q.pulse_sequence = q.pulse_sequence.replace('\"', ' ');
        				q.pulse_sequence = q.pulse_sequence.trim();
        			}
        		}
        		catch (Exception ex)
        		{ Util.show_and_log("get_par(): " + instring, ex.getMessage()); }			
        	}
        }
	        			
        if (nrSteps == 0)
        	q.data_mode = SW_main.modes.MODE_1D;
        else
        {
        	q.data_mode = SW_main.modes.MODE_2D;
        	q_f1.TD = nrSteps;
        }

        if (q.data_mode == SW_main.modes.MODE_2D   // if freq in indirect dimension not
        		&& q_f1.SFO1 == 0.0)                   // defined assume homonuclear
        {
        	q_f1.SFO1 = q.SFO1;
        }

        p.LB = 1.0;	// set reasonable default

        // set some reasonable default processing parameters

        if (q.pulse_sequence.equalsIgnoreCase("COSY"))
        {
        	p.WDW = SW_main.windows.SINE;
        	p_f1.WDW = SW_main.windows.SINE;
        	p.ssb = 0.0;
        	p_f1.ssb = 0.0;
        }
        if (q.pulse_sequence.equalsIgnoreCase("HMBC"))
        {
        	p.WDW = SW_main.windows.QSINE;
        	p_f1.WDW = SW_main.windows.QSINE;
        	p.ssb = 75.0;
        	p_f1.ssb = 75.0;
        }

        if (q.pulse_sequence.equalsIgnoreCase("HETCOR")
        		|| q.pulse_sequence.equalsIgnoreCase("HSQC")
        		|| q.pulse_sequence.equalsIgnoreCase("HMQC"))
        {
        	p.WDW = SW_main.windows.QSINE;
        	p_f1.WDW = SW_main.windows.QSINE;
        	p.ssb = 90.0;
        	p_f1.ssb = 90.0;
        }

        if (q.pulse_sequence.equalsIgnoreCase( "JRES"))
        {
        	p.WDW = SW_main.windows.EM;
        	p_f1.WDW = SW_main.windows.SINE;
        	p.LB = 1.0;
        }

        // some default processing parameters
        p_f1.MC2 = SW_main.det_modes_IND.QF;
        p_f1.PH_mod = SW_main.phasing.MAGNITUDE;
        p_f1.REVERSE = true;

        //  generate SpinWorks specific parameters
        if(dwell != 0.0)
        	swidth = 1.0 / dwell;   // use dwell in preference to bandwidth?    

        q.SW = swidth / q.SFO1;
        q.offset = low_freq + swidth;
        q.SF = q.SFO1 - (q.offset - (swidth / 2.0)) / 1.0e6;
        q.SI = q.TD;

        if (q.data_mode == SW_main.modes.MODE_2D)
        {	
        	q_f1.SW = swidth_f1 / q_f1.SFO1;
        	q_f1.offset = low_freq_f1 + swidth_f1;
        	q_f1.SF = q_f1.SFO1 - (q_f1.offset - (swidth_f1 / 2.0)) / 1.0e6;
        	q_f1.TD = nrSteps;
        	p_f1.SI = q_f1.TD * 2;		// set reasonable size parameters
        	q_f1.SI = p_f1.SI;
        	p.SI = q.TD * 2;
        }

		return true;
	}
	
//-------------------------------------------------------------------------------------------------
	/**
	 * 
	 * @param fileName
	 * @param q
	 * @param gp
	 * @return
	 */
	public static int get_fid(String fileName, data_par q, global_par gp)
	{
		int bytes_read = 0;

		FileInputStream fs;

		File file = new File(fileName);

		try { fs = new FileInputStream(file); }
		catch (IOException ex)
		{
			Util.show_and_log("get_fid(): cannot open: " + fileName, ex.getMessage());
			return -1;
		}

		int fileSize = (int)file.length();
		q.TD = (fileSize - 32) / 6;  // for bad nPoits??
		
		try { gp.fid = new double[q.TD]; }
		catch (Exception ex)
		{
			try { fs.close(); } 
			catch (IOException e) { e.printStackTrace(); }
			Util.show_and_log("get_fid(): cannot allocate space for fid "
					+ fileName, ex.getMessage());
			return -2;
		}
	      
		byte[] b = new byte[(int)file.length()];

		try { fs.read(b); } 
		catch (IOException ex) 
		{
			ex.printStackTrace();
			try { fs.close(); } 
			catch (IOException e) { e.printStackTrace(); }
			return -3;
		}

		ByteBuffer bb = ByteBuffer.wrap(b);		
		bb.order(ByteOrder.LITTLE_ENDIAN);
	      	
		for (int i = 0; i < q.TD / 2 + 8; i++)  	// position to start of data
			bb.getInt();                 			// skipping x values

		try 	// get the actual FID
		{
			for(int i = 0; i < q.TD; i++)
			{
				gp.fid[i] = (double)bb.getFloat();
				bytes_read += 4;
			}
		}
	    catch(Exception ex) 
		{ 
	    	ex.printStackTrace();
	    	try { fs.close(); } 
		    catch (IOException ex2) { ex2.printStackTrace(); }
	    	return -4;
	    }
		
		for (int i = 1; i < q.TD; i = i + 2)    	// change signs of imaginaries 
			gp.fid[i] = -gp.fid[i];            		// to reverse spectrum

		try { fs.close(); } 
	    catch (IOException ex) { ex.printStackTrace(); }
		return bytes_read;
	}

//-------------------------------------------------------------------------------------------------	
	/**
	 * 
	 * @param fileName
	 * @param q
	 * @param q_f1
	 * @param gp
	 * @return
	 */
	public static int get_ser(String fileName, data_par q, data_par q_f1, global_par gp)
	{
		int bytes_read = 0;

		FileInputStream fs;

		File file = new File(fileName);

		try { fs = new FileInputStream(file); }
		catch (IOException ex)
		{
			Util.show_and_log("get_fid(): cannot open: " + fileName, ex.getMessage());
			return -1;
		}
		
		 try { gp.ser = new double[q_f1.TD][q.TD]; }
         catch (Exception ex)
         {
        	 try { fs.close(); } 
     	     catch (IOException ex2) { ex2.printStackTrace(); }
             Util.show_and_log("get_ser(): cannot allocate space for ser "
                 + fileName, ex.getMessage());
             return -2;
         }			
	      
		byte[] b = new byte[(int)file.length()];

		try { fs.read(b); } 
		catch (IOException ex) 
		{
			ex.printStackTrace();
			try { fs.close(); } 
			catch (IOException e) { e.printStackTrace(); }
			return -3;
		}

		ByteBuffer bb = ByteBuffer.wrap(b);		
		bb.order(ByteOrder.LITTLE_ENDIAN);

		for (int i = 0; i < 8; i++)  	// position to start of data
			bb.getInt();      			// header only, no x values in 2D

		try
		{
			for (int i = 0; i < q_f1.TD; i++)
			{
				for (int j = 0; j < q.TD - 1; j = j + 2)
				{
					gp.ser[i][j] = (double)bb.getFloat();
					gp.ser[i][j + 1] = -(double)bb.getFloat();
					bytes_read = bytes_read + 8;                  
				}
			}
		}
		catch(Exception ex)
		{
			try { fs.close(); } 
		    catch (IOException ex2) { ex2.printStackTrace(); }
			return -4;
		}

		try { fs.close(); } 
	    catch (IOException ex) { ex.printStackTrace(); }
		return bytes_read;
	}
//-------------------------------------------------------------------------------------------------
}	//end of class
