
/// <copyright>
///	Copyright (c) 2006-2016

///	Kirk Marat
///	The University of Manitoba
///	Winnipeg, Manitoba, Canada
///	R3T 2N2

///	All rights reserved.

///	This source may be distributed, provided that the author and the University are 
/// appropriately acknowledged.  However, incorporation of any part into any commercial product 
/// is prohibited without prior written authorization. 
///
/// see:  http://www.gnu.org/licenses/gpl.html
///
/// Note:
/// Distribution of this source or any of its components into the People's Republic of China 
/// is strictly prohibited.
/// 
///	The author disclaims all warranties with regard to this software, including all implied 
///	warranties of merchantability and fitness. In no event, shall the author or the University of 
///	Manitoba be liable for any special, indirect or consequential damages, or any damages whatsoever 
///	resulting from loss of use, data or profits, whether in an action of contract, negligence or 
///	other tortuous action, arising out of or in connection with the use or performance of this 
///	software.

///	All trademarks mentioned (e.g. XwinNMR, VNMR(J), Windows, etc.) are property of their 
///	respective owners.
/// </copyright>

package spinworks;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.widgets.Display;

import decon.DeconUtil;
import simulation.SimPar;
import simulation.Transition;
import spinworks.SW_main.osTypes;

/**
 * A global (more accurately workspace) parameter block.
 * Also used by inset box dialogs
 * @author marat
 *
 */
public class global_par implements Cloneable
{
	/**
	 * constructor
	 */
	public global_par()  
	{
// default start-up settings
		cursor_flag = 0;
		freq1 = 5000.0;		        	freq2 = 0.0;
		t1 = 0.0;                  	 	t2 = 3.2768;
		freq1_indirect = 5000.0;	    freq2_indirect = 0.0;
		last_freq1 = 5000.0;        	last_freq2 = 0.0;
		last_freq1_indirect = 5000.0;   last_freq2_indirect = 0.0;

		display_mode = SW_main.modes.MODE_1D;
		display_2D = SW_main.display_modes.IMAGE;
		mode_3D = SW_main.modes_3D.XZ;

		if(SW_main.currentOS == osTypes.WINDOWS)
			scratch_folder = "C:\\Temp";
		else if(SW_main.currentOS == osTypes.MACOS)
			scratch_folder = SW_main.userHome;
		else
			scratch_folder = "";
		
		left_2D = 70;
		right_2D = 100;
		top_2D = 50;
		bottom_2D = 50;
		scale_2D = 1.0;
		vblock = 1;
		block_2D = 0;
		n_traces = 0;
		n_boxes = 0;
		stack_y_offset = 30;
		stack_x_offset = 0;
		stack_x_size = 50;
		stack_x_size_auto = true;
		stack_whitewash = false;
		stack_label_type = 0;		// numeric
		f2_trace_on = false; f1_trace_on = false;
		axis_offset = 50;
		axis_unit = SW_main.units.PPM;
		axis_unit_indirect = SW_main.units.PPM;
		peak_unit = SW_main.units.PPM;
		expt_y_offset = 100; calc_y_offset = 200;
		v_rescale = true;
		phase_mode = false;
		integ_mode = false;
		bpt_mode = false;
		stack_dlg_on = false;
		f2_from_scan = false;
        f1_from_scan = false;
		integ_scale = 0.0000002;
		raw_max = 100000.0;
		interp_on = true;					
		pp_sens = 0.02;		
		linewidth = 1.0;
		pp_thresh = 0.1;
		bin_start = 10.0;
		bin_end = 0.0;
		bin_width = 0.05;
		bin_method = 0;
		pp_sign = 0;
		pp_digits = 3;
		int_digits = 3;
		adjust_mi = false;
		pivot = 0; pivot_ind = 0;
		expt_read = false;
		ea_sort_done = false;
		dimensions_processed = 0;
		current_plane = 0;
		proc_buffer = null;							// processing buffer
		fid = null;									// fid array
		ser = null;									// series of fids (2D) array
		ser_buffer = null;                          // buffer for 3D/4D
		data_array = null;							// real data array
		i_data_array = null;						// imaginary data array
		calc_array = null;							// simulated array
		i_calc_array = null;						// simulated array
		res_array = null;
		i_res_array = null;
		rows_real = null;
		rows_imag = null;
		irows_real = null;
		irows_imag = null;
		f1_trace = null;
		f2_trace = null;
		f2_trace_scale_active = false;
		f1_trace_scale_active = false;
		n_levels = SW_main.MAX_LEVELS;
		pos_only = false;
		colour_contour = false;
		contour_spacing = 2;
		level_spacing_increment = SW_main.level_spacing.LOG;
		level_spacing_factor = 1.2;
		auto_load = false;
		auto_save = false;
		a4_paper = false;
		log_on = false;
		show_simulated = true;
		show_spectrum = true;
		show_integrals = true;
		show_zero_line = true;
		show_yaxis = true;
		show_title = true;
		match_on = true;
		n_bl_pts = 0;
		baseline_type = 0;
		abs_degree = 5;

		baseline_x = new double[200];
		baseline_y = new double[200];
		baseline_sig = new double[200];
		n_peaks = 0;
		n_decon_peaks = 0;
		active_peak = 0;
		n_integrals = 0;
		integ_cal = 0.000002;
		integ_calibrated = false;
		int_cal_value = 1.0;
		current_integral = -1;
		current_bias = 0.0;
		current_slope = 0.0;
		retain_integral_regions = false;
		//width = 1.0;

		plot_xsize = 250.0;					// graphics units are mm.
		plot_ysize = 150.0;
		plot_xoffset = 7.0;
		plot_yoffset = 15.0;

		plot_axis = true;
		plot_frame = true;
		plot_grid = false;
		plot_integrals = true;
		plot_integral_values = true;	
		plot_integrals_inset = true;
		plot_parameters = true;
		plot_peak_labels = true;
		plot_simulated = true;
		plot_spectrum = true;
		plot_title_flag = true;

		parameters_right = false;
		axis_font_height = (float)3.5;		// (float) casts are just to stop
		peak_font_height = (float)2.7;		// annoying compiler warnings in VS
		integ_font_height = (float)2.7;		// the font constructor expects floats
		title_font_height = (float)4.0;
		par_font_height = (float)2.7;
		spect_pen_thickness = (float)0.1;
		trace_offset = 5.0;                // 5 mm from 2D to trace

		Device device = Display.getCurrent();  

		back_colour = new Color(device, 0, 0, 0);     
		spect_colour = new Color(device, 255, 255, 0);
		sim_colour = new Color(device, 80, 80, 255);
		integ_colour = new Color(device, 255, 255, 255);
		active_integ_colour = new Color(device, 255, 0, 0);
		axis_colour = new Color(device, 0, 255, 255);
		cursor_colour = new Color(device, 0, 255, 0);
		colour_red = new Color(device, 255, 0, 0);
		colour_green = new Color(device, 0, 255, 0);
		colour_blue = new Color(device, 0, 0, 255);
		colour_dark_blue = new Color(device, 0, 0, 150);
		colour_magenta = new Color(device, 255, 0, 100);
		
		cosy_colour = new Color (device, 0, 150, 0);
		tocsy_colour = new Color (device, 0, 0, 200);
		noesy_colour = new Color (device, 150, 0, 0);
		hmbc_colour = new Color (device, 150, 150, 0);
		
		print_spect_colour = new Color(device, 0, 0, 0);	// These colours are for print 
        print_sim_colour = new Color(device, 0, 0, 0);	    // output, default all black
        print_integ_colour = new Color(device, 0, 0, 0);
        print_axis_colour = new Color(device, 0, 0, 0);
		
		displayFontSize = -1;	// -1 is "set by display size"

		// TODO turn ROIs into an object!

		roi_1_f2_left = 5000.0;				// Internal frequency units are Hz.
		roi_1_f2_right = 0.0;
		roi_1_f1_left = 5000.0;
		roi_1_f1_right = 0.0;

		roi_2_f2_left = 5000.0;
		roi_2_f2_right = 0.0;
		roi_2_f1_left = 5000.0;
		roi_2_f1_right = 0.0;

		roi_3_f2_left = 5000.0;
		roi_3_f2_right = 0.0;
		roi_3_f1_left = 5000.0;
		roi_3_f1_right = 0.0;

		roi_4_f2_left = 5000.0;
		roi_4_f2_right = 0.0;
		roi_4_f1_left = 5000.0;
		roi_4_f1_right = 0.0;

		interp_on = true;
		rotate_2D = false;
		overlay_2D = false;
		row_col = SW_main.trace_modes.ROWS;
		nslices = 0;
		tracking_cursor = true;
		slice_no = new int[32];
		row_scale = new double[32];
		pos_level = new Level[16];
		neg_level = new Level[16];
		peaks = new PeakInfo[PeakPick.MAX_PEAKS];
		decon_peaks = new PeakInfo[PeakPick.MAX_PEAKS];
		calc_peaks = new Transition[SimPar.MAX_TRANS];
		n_segments = 0;
		n_segments_neg = 0;
		showCOSYcorr = true;
		showTOCSYcorr = true;
		showNOESYcorr = true;
		showHMBCcorr = true;

		plot_title = "no title";

		frange = 20.0;				// the mask range in Hz 
		fmask = 0.0;				// the mask frequency in Hz 
		syn_mask_width = 1.0;		// synthetic mask peak width 
		rec_width = 0.1;			// reconstruction linewidth in Hz. 
		loop_gain = 0.03;			// the loop gain 
		threshold = 0.01;			// threshold as a fraction of the tallest peak 
		synmask = 0;				// synthetic or extracted mask? 
		hw_shape = 0;				// lineshape used for reconstruction
		hw_mask_shape = 0;			// mask lineshape

		//	special parameters for 2D/3D F1 HOGWASH
		loop_gain_f1 = 0.1;				// separate loop gain for 2D
		mask_points_f1 = 256;			// the mask range in points (used in 2D)
		rec_width_f1 = 10.0;			// reconstruction linewidth in Hz (2D)
		threshold_f1 = 0.05;			// threshold as a fraction of the tallest peak 

		//	hw_shape_f1 =
		//		(int)SW_main.shapes.Gauss;	// lineshape used for reconstruction

		//	special parameters for 2D F2 HOGWASH
		loop_gain_f2 = 0.1;				// separate loop gain for 2D
		mask_points_f2 = 64;			// the mask range in points (used in 2D)
		rec_width_f2 = 20.0;			// reconstruction linewidth in Hz (2D)
		threshold_f2 = 0.05;			// threshold as a fraction of the tallest peak 

		//    hw_shape_f2 =
		//        (int)SW_main.shapes.Gauss;	// lineshape used for reconstruction

		integrals = new IntegInfo[SW_main.MAX_INTEGRALS];
		stack_trace = new Trace[SW_main.MAX_TRACES];
		printInsetBox = new PrintInsetBox[SW_main.MAX_BOXES];

		//create 2D pixels, contour line segments, etc. one time, for run-time efficiency
		for (int i = 0; i < SW_main.MAX_LEVELS; i++)			// create contouring levels
		{
			pos_level[i] = new Level();
			neg_level[i] = new Level();
		}

		seg_pos = new Segment[Contour.MAX_SEGMENTS];
		seg_neg = new Segment[Contour.MAX_SEGMENTS];
		
		if(SW_main.currentOS == osTypes.WINDOWS)
			module_folder = "C:\\Program Files (X86)\\SpinWorks Software\\SpinWorks Java";
		else if(SW_main.currentOS == osTypes.MACOS)
			module_folder = "/Applications/SpinWorksJ.app/Contents/MacOS";
		else
			module_folder = "";
	}
	
//-------------------------------------------------------------------------------------------------
/**
 * re-set some parameters on new data-set.  This call is appropriate when reading a new 
 * data set into an existing WS.  Parameters such as fonts, colours, etc. are retained.
 */
    public void new_data()                 
    {
        plot_title = "no title";				// clear plot title 
        cursor_flag = 0;
        nslices = 0;
        n_boxes = 0;
        block_2D = 0;
		data_array = null;
		i_data_array = null;
        res_array = null;
        i_res_array = null;
        
        f2_from_scan = false;
        f1_from_scan = false;

        n_peaks = 0;
        active_peak = 0;

        f1_trace_on = false;
        f2_trace_on = false;
        n_segments = 0;
        n_segments_neg = 0;
        new_2D_view = true;
        dimensions_processed  = 0;
        ea_sort_done = false;
        raw_max = 100000.0;
    }

//-------------------------------------------------------------------------------------------------
/**
 * This sets default "gp" parameters appropriate for switching to a new unoccupied workspace 
 *  Most parameters including colours, font sizes, etc. are re-set to defaults
 */
	public void new_ws()                 // re-set some parameters for new WS
	{
        cursor_flag = 0;			     // default start-up settings
        freq1 = 5000.0; freq2 = 0.0;
        t1 = 0.0;       t2 = 3.2768;
        freq1_indirect = 5000.0; freq2_indirect = 0.0;
        display_mode = SW_main.modes.MODE_1D;
        display_2D = SW_main.display_modes.IMAGE;
        mode_3D = SW_main.modes_3D.XZ;
        left_2D = 70;
        right_2D = 100;
        top_2D = 50;
        bottom_2D = 50;
        scale_2D = 1.0;
        vblock = 1;
        block_2D = 0;
        n_traces = 0;
        n_boxes = 0;
        stack_y_offset = 30;
        stack_x_offset = 0;
        stack_x_size = 50;
        stack_x_size_auto = true;
        stack_whitewash = false;
        stack_label_type = 0;		// numeric
        f2_trace_on = false; f1_trace_on = false;
        axis_offset = 50;
        axis_unit = SW_main.units.PPM;
        axis_unit_indirect = SW_main.units.PPM;
        peak_unit = SW_main.units.PPM;
        expt_y_offset = 100; calc_y_offset = 200;
        v_rescale = true;
        phase_mode = false;
        integ_mode = false;
        bpt_mode = false;
        stack_dlg_on = false;
        f2_from_scan = false;
        f1_from_scan = false;
        integ_scale = 0.0000002;
        raw_max = 100000.0;
        interp_on = true;
        pp_sens = 0.02;
        linewidth = 1.0;
        pp_thresh = 0.1;
        bin_start = 10.0;
        bin_end = 0.0;
        bin_width = 0.05;
        bin_method = 0;
        pp_sign = 0;
        pp_digits = 3;
        int_digits = 3;
        adjust_mi = false;
        pivot = 0; pivot_ind = 0;

        expt_read = false;
        ea_sort_done = false;
        dimensions_processed = 0;

        proc_buffer = null;							// processing buffer
        fid = null;									// fid array
        ser = null;									// series of fids (2D) array
        ser_buffer = null;                          // buffer for 3D/4D
        ser_buffer = null;                          // ser file buffer for 3D
        data_array = null;							// real data array
        i_data_array = null;						// imaginary data array
        calc_array = null;							// simulated array
        i_calc_array = null;						// simulated array
        res_array = null;
        i_res_array = null;
        rows_real = null;
        rows_imag = null;
        irows_real = null;
        irows_imag = null;
        f1_trace = null;
        f2_trace = null;

        f2_trace_scale_active = false;
        f1_trace_scale_active = false;
        f2_trace_path = "";
        f1_trace_path = "";

        n_levels = SW_main.MAX_LEVELS;
        pos_only = false;
        colour_contour = false;
        contour_spacing = 2;
        level_spacing_increment = SW_main.level_spacing.LOG;
        level_spacing_factor = 1.2;
        show_simulated = true;
        show_spectrum = true;
        show_integrals = true;
        show_zero_line = true;
        show_yaxis = true;
        show_title = true;
        match_on = true;
        f2_from_scan = false;
        f1_from_scan = false;
        n_bl_pts = 0;
        baseline_type = 0;
        abs_degree = 5;

        n_peaks = 0;
		n_decon_peaks = 0;
        active_peak = 0;
        n_integrals = 0;
        integ_cal = 0.000002;
        integ_calibrated = false;
        int_cal_value = 1.0;

        plot_xsize = 250.0;					    // graphics units are mm.
        plot_ysize = 150.0;
        plot_xoffset = 7.0;
        plot_yoffset = 15.0;

        plot_axis = true;
        plot_frame = true;
        plot_grid = false;
        plot_integrals = true;
        plot_integral_values = true;
        plot_integrals_inset = true;
        plot_parameters = true;
        plot_peak_labels = true;
        plot_simulated = true;
        plot_spectrum = true;
        plot_title_flag = true;

        parameters_right = false;
        
        axis_font_height = (float)3.5;		// (float) casts are just to stop
        peak_font_height = (float)2.7;		// annoying compiler warnings
        integ_font_height = (float)2.7;		// the font constructor expects floats
        title_font_height = (float)4.0;
        par_font_height = (float)2.7;
        spect_pen_thickness = (float)0.1;
        trace_offset = 5.0;                 // 5 mm from 2D to trace
       
        roi_1_f2_left = 5000.0;				// Internal frequency units are Hz.
        roi_1_f2_right = 0.0;
        roi_1_f1_left = 5000.0;
        roi_1_f1_right = 0.0;

        roi_2_f2_left = 5000.0;
        roi_2_f2_right = 0.0;
        roi_2_f1_left = 5000.0;
        roi_2_f1_right = 0.0;

        roi_3_f2_left = 5000.0;
        roi_3_f2_right = 0.0;
        roi_3_f1_left = 5000.0;
        roi_3_f1_right = 0.0;

        roi_4_f2_left = 5000.0;
        roi_4_f2_right = 0.0;
        roi_4_f1_left = 5000.0;
        roi_4_f1_right = 0.0;

        interp_on = true;
        rotate_2D = false;
        overlay_2D = false;
        row_col = SW_main.trace_modes.ROWS;
        nslices = 0;
        tracking_cursor = true;
        n_segments = 0;
        n_segments_neg = 0;
        plot_title = "New Workspace";

        frange = 20.0;				// the mask range in Hz 
        fmask = 0.0;				// the mask frequency in Hz 
        syn_mask_width = 1.0;		// synthetic mask peak width 
        rec_width = 0.1;			// reconstruction linewidth in Hz. 
        loop_gain = 0.03;			// the loop gain 
        threshold = 0.01;			// threshold as a fraction of the tallest peak 
        synmask = 0;				// synthetic or extracted mask? 
        hw_shape = 0;				// lineshape used for reconstruction
        hw_mask_shape = 0;			// mask lineshape

        //	special parameters for 2D F1 HOGWASH
        loop_gain_f1 = 0.1;				// separate loop gain for 2D
        mask_points_f1 = 256;			// the mask range in points (used in 2D)
        rec_width_f1 = 10.0;			// reconstruction linewidth in Hz (2D)
        threshold_f1 = 0.05;			// threshold as a fraction of the tallest peak 
//        hw_shape_f1 =
//            (int)SW_main.shapes.Gauss;	// lineshape used for reconstruction
	}
		
//-------------------------------------------------------------------------------------------------
	@Override
	public Object clone() throws CloneNotSupportedException 
	{
		return (global_par)super.clone();
	}
	
//-------------------------------------------------------------------------------------------------
//  data members

    public  int dxsize, dysize;				        // total size of drawing area
    public  int xsize_2D, ysize_2D;			        // size of 2D spectral area

    public  int left_2D, right_2D,					// 2D display margins, pixels
				top_2D, bottom_2D;

    public  int axis_offset;						// axis offset in 1D
    public  int expt_y_offset, calc_y_offset;		// y offset of spectra

    public  SW_main.units axis_unit;				// Hz, ppm or sec
    public  SW_main.units axis_unit_indirect;		// ditto, for F1
	public	SW_main.units peak_unit;				// units for peaks on plot

    public  int cursor_flag;						// number of cursors currently disp.
    public  double freq1, freq2;					// current left and right edges in Hz.
    public  double t1, t2;                          // current left and right edges of FID in s
    public  double new_freq1, new_freq2;
    public  double new_t1, new_t2;
    public  double last_freq1, last_freq2;
    public  double freq1_indirect, freq2_indirect;	// same for indirect dimension
    public  double new_freq1_indirect, new_freq2_indirect;
    public  double last_freq1_indirect, last_freq2_indirect;

    public  double hzperpix, hzperpix_f1;			// Hz per pixel, both dimensions
    
    public SW_main.modes display_mode;              // current display mode, 1D, 2D,or FID
    public SW_main.display_modes display_2D;		// 2D display mode, IMAGE or CONTOUR
    public SW_main.modes_3D mode_3D;                // XZ, XY, or YZ

    public  double calc_scale, expt_scale,			// vertical scaling factors
				    spect_scale, integ_scale;
	public	double integ_scale_2D;
    public  double fid_scale;
    
    public  boolean v_rescale;						// whether vertical re-scale after proc.

    public  double data_max;						// maximum values in data
    public  double data_max_2D;						// maximum value in data (2D)
    public  double data_max_3D;						// maximum value in data (3D)
	public	double raw_max;							// maximum value in the raw data (fid or ser file)
	public	double calc_max;						// maximum value in simulated spectrum
    public  double data_noise;						// rms noise in data								

    public  double scale_2D;						// 2D scaling factor

    public  int vblock;								// Varian block number
    public  int block_2D;							// block of the 2D file
	public	int n_traces;							// number of stacked traces
	public	int n_boxes;							// number of inset boxes

	public int stack_y_offset;						// vertical spacing (pixels) of stacked plot
	public int stack_x_offset;						// horizontal spacing (pixels) of stacked plot
    public int stack_x_size;                        // explicit x size for stack trace
    public boolean stack_x_size_auto;               // automatic x sizing on stack display?
    public boolean stack_whitewash;					// if a stacked plot should be "whitewashed"
	public int stack_label_type;					// type of labelling for stacked traces
	
    public  boolean f2_trace_on, f1_trace_on;		// traces, display or not
    public  int size_f2_trace, size_f1_trace;		// sizes of the traces
    public  boolean f2_from_scan, f1_from_scan;		// whether f1 and f2 are from matrix
    public  double offset_f2_trace, offset_f1_trace;// offsets of traces
    public  double hzperpt_f2_trace, hzperpt_f1_trace;	// Hz. per point for the traces
	public double f1_trace_scale, f2_trace_scale;
	public boolean f1_trace_scale_active,			// if button scaling of traces on
				f2_trace_scale_active;
    public String f1_trace_path,                    // paths to the external trace files
                  f2_trace_path;

    public  boolean new_2D_view;					// flag re-draw of 2D area

	public boolean show_simulated;					// whether the spectra are displayed or not
	public boolean show_spectrum;
    public boolean show_integrals;
    public boolean show_zero_line;
    public boolean show_yaxis;
    public boolean show_title;
	public boolean match_on;		

    public  boolean phase_mode;						// whether in phase mode
    public  boolean integ_mode;						// whether in integration mode
	public  boolean bpt_mode;						// whether in baseline point mode
	public 	boolean stack_dlg_on;					// a stack trace dialog is running
    public  int pivot, pivot_ind;					// phase pivot point (both dim.)
    public  double ph0, ph1;						// interactive phase constants

    public  int td_eff;								// effective TD (e.g. after LP, etc.)
    public  boolean expt_read;						// whether a data set has been read
			
    public boolean ea_sort_done;					// whether data has had ea sort
	public int dimensions_processed;				// how many of the 2D/3D dimensions have been processed
    public int current_plane;                       // current 2D plane

    public  int n_levels;							// number of 2D levels
    public  boolean pos_only;						// true if only positive levels displayed
    public  int contour_spacing;					// contour spacing
    public SW_main.level_spacing 
         level_spacing_increment;             		// contour level: linear or log 
    public double level_spacing_factor;             // factor for log spacing
	public boolean colour_contour;					// whether contours should be in colour

    public  Level[] pos_level;						// positive contour levels
    public  Level[] neg_level;						// negative contour levels

    public Segment[] seg_pos;
    public Segment[] seg_neg;

    public boolean auto_load, auto_save;			// whether processed data
												    // should be loaded or saved
    public boolean log_on;                          // whether logging is on
    public boolean a4_paper;                        // for the rest of the world

    public  int n_bl_pts;							// number of baseline points
    public int baseline_type;                       // polynomial or spline
    public  int abs_degree;							// degree of polynomial used for 
												    // baseline correction
	public double[] baseline_x;
	public double[] baseline_y;
	public double[] baseline_sig;
    
    public int n_integrals;						    // number of integrals
    public double integ_cal;						// integral calibration factor
    public boolean integ_calibrated;				// whether an integral has been cal.
    public double int_cal_value;
    public int current_integral;                    // current integral 
    public double current_bias;
    public double current_slope;
    public boolean retain_integral_regions;         // if integral regions should be retained on new data

    public  boolean interp_on;						// whether peak interpolation is on
    public  double pp_sens;							// peak picking sensitivity
    public  double pp_thresh;						// peak picking threshold
    public  double bin_start;                       // start shift for data binning
    public double bin_end;                          // end shift for data binning 
    public double bin_width;                        // width of each data bin
    public int bin_method;
	public int pp_sign;								// sign of picked peaks (-1, 0, 1)
    public int pp_digits;                           // number of digits to right of decimal (peaks)
    public int int_digits;                          // number of digits to right of decimal (integrals)
    public double linewidth;						// search width for peak picking  originally in sim_pars
	public boolean adjust_mi;						// true if in MI adjust mode
    public int n_peaks;								// number of picked peaks.  
    public int n_decon_peaks;
    public int active_peak;                         // the currently active peak for 2D peak assignment

    public  double plot_xsize, plot_ysize;			// plot size in mm.
    public  double plot_xoffset, plot_yoffset;		// plot offsets in mm

	public boolean plot_axis;						// whether the 1D axis should be plotted
    public boolean plot_frame;						// whether spectrum should be "framed"   
	public boolean plot_integrals;					// whether the integrals should be printed
    public boolean plot_integral_values;			// whether the integrals should be printed
    public boolean plot_integrals_inset;			// whether to plot integrals on insets
    public boolean plot_parameters;					// whether the parameters should be printed
    public boolean plot_peak_labels;				// whether the peak labels should be printed
	public boolean plot_simulated;					// whether any simulated spectrum should be printed
	public boolean plot_spectrum;					// whether the spectrum should be plotted
	public boolean plot_title_flag;					// whether the plot title should be drawn
    public boolean plot_grid;                       // whether to plot a grid on the spectrum
   
    public boolean  parameters_right;				// whether 2D parameters should be on right
    public float axis_font_height;					// font height (in mm) for plot objects
    public float peak_font_height;					// they are float to avoid casts in creating
    public float integ_font_height;				    // fonts.
    public float title_font_height;
    public float par_font_height;
	public float spect_pen_thickness;
    public double trace_offset;                     // distance from 2D box to traces

    // colours for display
    public Color back_colour;			// background colour
    public Color spect_colour;			// spectrum colour
    public Color sim_colour;			// simulated colour
    public Color integ_colour;			// integral colour
    public Color active_integ_colour; 	// colour of the active integral
    public Color axis_colour;			// axis colour
    public Color cursor_colour;			// cursor colour
    
    public Color cosy_colour;			// colour for COSY correlations
    public Color tocsy_colour;			// colour for TOCSY correlations
    public Color noesy_colour;			// colour for NOESY correlations
    public Color hmbc_colour;			// colour for HMBC correlations
    
    public Color colour_red;     		// generic red
    public Color colour_green;			// generic green
    public Color colour_blue;			// generic blue
    public Color colour_dark_blue;		// dark blue
    public Color colour_magenta;		// magenta
    public int displayFontSize;			// optional display font sizes


	// colours for print (hard-copy)
	public Color print_spect_colour;	// spectrum colour
	public Color print_sim_colour;		// simulated colour
	public Color print_integ_colour; 	// integral colour
	public Color print_axis_colour;		// integral colour
	
    // TO DO turn ROIs into an object!
	public double roi_1_f2_left;					// regions of interest
	public double roi_1_f2_right;
	public double roi_1_f1_left;
	public double roi_1_f1_right;

	public double roi_2_f2_left;					// regions of interest
	public double roi_2_f2_right;
	public double roi_2_f1_left;
	public double roi_2_f1_right;

	public double roi_3_f2_left;					// regions of interest
	public double roi_3_f2_right;
	public double roi_3_f1_left;
	public double roi_3_f1_right;

    public double roi_4_f2_left;					// regions of interest
    public double roi_4_f2_right;
    public double roi_4_f1_left;
    public double roi_4_f1_right;

    public  boolean rotate_2D;						// if 2D should be rotated
    public  boolean overlay_2D;                     // if in 2D overlay mode
    public  boolean tracking_cursor;				// whether to display tracking cursor
    public  SW_main.trace_modes row_col;			// whether in row or column mode
    public  int nslices;							// number of 2D slices
    public  int active_slice;
    public  int[]		slice_no;				    // array of slice numbers
    public  double[]	row_scale;				    // scaling for each slice
    public  int n_segments;							// number of contour segments
    public  int n_segments_neg;						// number of negative contour segments
    public  boolean showCOSYcorr;					// whether COSY correlations are shown
    public  boolean showTOCSYcorr;					// whether TOCSY correlations are shown
    public  boolean showNOESYcorr;					// whether NOESY correlations are shown
    public  boolean showHMBCcorr;					// whether HMBC correlations are shown
    
    //  data arrays
    public double[] proc_buffer;                    // processing buffer
    public double[] fid;                            // fid array
    public double[][] ser;                          // series of fids (2D) array
    public byte[] ser_buffer;                       // ser file buffer for 3D/4D
    public double[] data_array;                     // real data array
    public double[] i_data_array;                   // imaginary data array
    public double[] calc_array;						// array for simulated spectra (real)
    public double[] i_calc_array;					// array for simulated spectra (imag)
    public double[] res_array;                      // resultant array for calculations (real)
    public double[] i_res_array;                    // resultant array for calculations (imag)

    public float[][] rows_real;						// 2D matrices, 4 quadrants
    public float[][] rows_imag;
    public float[][] irows_real;
    public float[][] irows_imag;						

	public double[] f1_trace;						// 2D 1D reference spectra
	public double[] f2_trace;

	public PeakInfo[] peaks;						// picked peaks
	public PeakInfo[] decon_peaks;					// peaks for deconvolution
	public Transition[] calc_peaks;				// calculated peaks from sim routines

	public int trans_index;							// index into the transition file

	public String   plot_title;                     // title for the plot
	public String	default_folder;				    // base data folder
	public String	module_folder;					// external (simulation) module folder
	public String	scratch_folder;					// writable scratch folder

//HOGWASH parameters
	public double frange;							// the mask range in Hz 
	public double fmask;							// the mask frequency in Hz 
	public double syn_mask_width;					// synthetic mask peak width 
	public double rec_width;						// reconstruction linewidth in Hz. 
	public double loop_gain;						// the loop gain 
	public double threshold;						// threshold as a fraction of the tallest peak 
	public int synmask;								// synthetic or extracted mask? 
	public int hw_shape;							// lineshape used for reconstruction
	public int hw_mask_shape;						// mask lineshape

//special parameters for 2D/3D F1 HOGWASH
	public double loop_gain_f1;						// separate loop gain for 2D
	public int mask_points_f1;						// the mask range in points (used in 2D)
	public double rec_width_f1;						// reconstruction linewidth in Hz (2D)
	public double threshold_f1;						// threshold as a fraction of the tallest peak 
	public int hw_shape_f1;							// lineshape used for reconstruction

//special parameters for 3D F2 HOGWASH
    public double loop_gain_f2;						// separate loop gain for 2D
    public int mask_points_f2;						// the mask range in points (used in 2D)
    public double rec_width_f2;						// reconstruction linewidth in Hz (2D)
    public double threshold_f2;						// threshold as a fraction of the tallest peak 
    public int hw_shape_f2;							// lineshape used for reconstruction

    public  IntegInfo[] integrals;					// integral region definitions
	public Trace[] stack_trace;                     // stacked trace objects
	public  PrintInsetBox[] printInsetBox;		    // print inset box definitions
}


